<?

/******************************************************************************
 *                                                                            *
 *      Klasa obsługująca wszystkie wywołania (steruje akcjami)               *
 *                                                                            *
 ******************************************************************************/

require_once('View.class.php');
require_once('PDOS.class.php');

if(!defined('AVAILABLE_LANG'))
    define('AVAILABLE_LANG', 'PL');

if(!defined('DEFAULT_LANG'))
    define('DEFAULT_LANG', 'PL');

abstract class Controller
{
    /**
     *  Akcja jaką ma wykonać kontroler
     *  @var string
     */
    protected $action;
    
    /**
     *  Rodzaj akcji 
     *  @var string
     */
    protected $action_type;
    
    /**
     *  Pod akcja, dla templatow statycznych 
     *  @var string
     */
    protected $template;
    
    /**
     *  Nazwa kontrolera 
     *  @var string
     */
    protected $name;
    
    /**
     *  Językna jaki ma być przetworzona strona
     *  @var string
     */
    protected $lang;
    
    /**
     *  Obiekt obsługi warstwy widoku 
     *  @var object View
     */
    public $view;
    
    /**
     *  Przechowuje obiekt autoryzacji
     *  @var object Auth
     */
    public $auth;
    
    /**
     * Wersja klasy autoryzacji
     * @var integer
     * @tutorial 1 - stara klasa auth, 2 - nowa klasa DB, 3 - nowa klasa SOAP
     */
    protected $authorization_version = 1;
    
    /**
     * Kod autoryzacji strony w SOAP
     * @var string
     */
    protected $authorization_soapcode = null;
    
    /**
     *  Tablica dostępu do kolejnych akcji 
     *  @var array
     */
    protected $access = null;
    
    /**
     *  Identyfikator danej strony 
     *  @var string
     */
    protected $site_id = null;
    
    /**
     *  Uchwyt do bazy danych
     *  @deprecated
     */
    protected $db;
    
    /**
     *  Zmienna debugowania 
     *  @var boolean
     */
    protected $debug = false;
    
    /**
     *  Tablica domyślnego dostępu do akcji kontrolera, kopiowana do $access
     *  @var array
     */
    protected $default_access = array(
        'auth_require' => false, //Akcja wymaga autoryzacji
        'auth_secure'  => false, //Akcja wymaga dodatkowych zabezpieczeń
        'allow_groups' => null,  //Dostęp tylko dla grup użytkowników array, null - dla wszystkich
        'allow_uts'	   => null,  //Dostęp tylko dla typów uzytkownikow 
        'allow_bots'   => true,  //Akcja dostępna dla botów
        'save_log'     => true,  //Zapisuje wywołania do logu
        'save_bot_log' => false, //Zapisuje wywołania botów do logu
        );
    
    /**
     *	Czy zapisać akcje kontrolera, ustawiać na true tylko gdy kontroler 
     *	ma już pełen zestaw akcji i wyłączać po dodaniu do bazy, przy 
     *	przeniesieniu na inną bazę (np. z testu na produkcje) ponownie 
     *	ustawić na true aż do dodania do nowej bazy.
     *	@var boolean
     */
    protected $save_site = false;
    
    /**
     *  Konstruktor uruchamia akcję
     *  @param string $lang - Język który ma być ładowany np. 'PL' - DO WYWALENIA
     *  @param string $action - Nazwa akcji
     *  @param array $assign - Tablica zmiennych wysyłanych do wszystkich szablonów
     **/
    public function __construct($lang = null, $action = null, $assign = null, $check_only = false)
    {
        if($check_only)
            return;
        
        $this->name = str_replace('Controller', '', get_class($this));
        if($this->checkActionType($action)){
            $this->language();
            //Takie coś możliwe jest i poprawnie działa tylko w PHPie ;-)
            define('CONTR_NAME', $this->name);
            define('CONTR_ACTION', $this->action);
            define('CONTR_LANG', $this->lang);
            if(!defined('SITE_ID'))
                define('SITE_ID', $this->site_id);
            
            //Uruchamianie obiektu szablonów
            $this->view = new View($this->lang, $this->name, $this->action);
            
            //Wysyłanie domyślnej tablicy zmiennych do szablonów
            $this->view->assign('selected_language', $this->lang);
            $this->view->assign('available_languages', explode(',', AVAILABLE_LANG));
            if(is_array($assign))
                foreach($assign as $var => $val)
                    if(!is_numeric($var{0}))
                        $this->view->assign($var, $val);
            
            //Autoryzacja użytkownika
            $this->authorize();
            
            //Zapis do bazy 
            $this->saveSite();
            
            //Odpalanie akcji lub statycznego szablonu
            $this->executeAction();
        }else{
            header('Status: 404 Not Found');
            header('HTTP/1.0 404 Not Found');
        }
    }
    
    /**
     *  Sprawdza czy akcja istnieje lub wysyła w maliny
     **/
    protected function checkActionType($action)
    {
        if(isset($_POST['xjxfun']) && isset($_POST['xjxr'])){//AJAX
            $this->action = $action;
            $this->action_type = 'ajax';
            return true;
        }elseif(method_exists($this, 'action_'.$action)){//Zwykła akcja
            $this->action = $action;
            $this->action_type = 'dynamic';
            return true;
        }elseif(file_exists(SMARTY_CONFIG_DIR.'/'.$this->name.'/template/static/'.$action.'.tpl')){//Strona statyczna
            $this->action = $action;
            $this->action_type = 'static';
            $this->template = null;
            return true;
        }elseif(is_dir(SMARTY_CONFIG_DIR.'/'.$this->name.'/template/static/'.$action.'/')){//Zestaw stron statycznych
            $this->action = $action;
            $this->action_type = 'static';
            $template = isset($_GET['s'])?$_GET['s']:'index';
            if(!file_exists(SMARTY_CONFIG_DIR.'/'.$this->name.'/template/static/'.$action.'/'.$template.'.tpl')){
                $this->action_type = 'err404';
                return false;
            }
            $this->template = $template;
            return true;
        }else{
            $this->action_type = 'err404';
            return false;
        }
    }
    
    /**
     *  Odpala odpowiednią funkcję
     **/
    public function executeAction()
    {
        $this->basic_controler_action();
        if($this->action_type == 'dynamic'){
            $function = 'action_'.$this->action;
            $this->$function();
        }elseif($this->action_type == 'ajax'){
            $function = 'action_'.$this->action;
            $this->$function();
        }else{
            $this->static_action($this->action, $this->template);
        }
    }
    
    /**
     *  Obsługa wersji językowych
     **/
    protected function language()
    {
        //Ustawianie języka
        if(isset($_POST['languageident'])){
            $lang = $_POST['languageident'];
            setcookie('LANG', $lang, time()+99999, '/', $_SERVER['HTTP_HOST']);
            $this->redirect();
        }elseif(isset($_GET['languageident'])){
            $lang = $_GET['languageident'];
            setcookie('LANG', $lang, time()+99999, '/', $_SERVER['HTTP_HOST']);
            $this->redirect();
        }elseif(isset($_COOKIE['LANG'])){
            $lang = $_COOKIE['LANG'];
        }else{
            $lang = DEFAULT_LANG;
        }
        
        //Sprawdzanie zgodności języka z ustawieniami
        if(in_array($lang, explode(',', AVAILABLE_LANG)))
            $this->lang = $lang;
        else
            $this->lang = DEFAULT_LANG;
        
        //setcookie('LANG', $this->lang, time()+99999, '/', $_SERVER['HTTP_HOST']);
    }
    
    /**
     *  Autoryzacja użytkownika
     **/
    protected function authorize()
    {
        //Czy to okno modalne
        $modal = false;
        
        //Ustawianie parametrów dostępu
        $this->access[$this->action] = $this->default_access;
        $this->setAccess();
        
        //Uruchamianie obiektu autoryzacji
        switch($this->authorization_version){
            case 1:
                require_once 'authorize.class.php';
                $this->auth = new Authorize($this->access[$this->action]);
                break;
            case 2:
                require_once 'authorizeDB.class.php';
                $this->auth = new AuthorizeDB($this->access[$this->action]);
                break;
            case 3:
                require_once 'authorizeSOAP.class.php';
                $this->auth = new AuthorizeSOAP($this->access[$this->action], $this->authorization_soapcode);
                break;
            default:
                trigger_error('Nieprawidlowa wersja klasy autoryzacji!', E_USER_ERROR);
                break;
        }
        //Wysłanie żądania wyświetlenia okna modalnego
        if(isset($_POST['authmodal'])){
            $this->view->assign('authmodal', 1);
            $modal = true;
        }
        $this->view->assign('reminder_url', '/user/remember/');
        $this->view->assign('register_url', '/user/register_step1/');
        $this->view->assign('register_url2', '/user/register_step1/');
        
        //Próba zalogowania
        if(isset($_POST['authlogin']) && isset($_POST['authpasswd'])){
            $login = $_POST['authlogin'];
            $passwd = $_POST['authpasswd'];
            $secure = isset($_POST['authsecure'])?true:false;
            $this->auth->start_session($login, $passwd, $secure);
            if(count($this->auth->errors)==0)
                $this->redirect('self');
        }
        
        //odbior kodu autoryzacji HTTP
        if(isset($_POST['user_auth_code']) && $this->authorization_version!=1){
            $this->auth->obtainSession($_POST['user_auth_code']);
        }
        
        //Sprawdzenie sesji usera, zgodnie z Authorization_engine
        if(count($this->auth->errors)==0){
            try{
                $this->auth->refresh_session();
            }catch(Exception $e){
                $this->redirect('self');
            }
        }
        
        //wyslanie do redirecta podczas wymagania logowania
        if(isset($this->auth->errors['not_logged']) && $this->authorization_version==3 && !$this->auth->isSoapAuthenticate()){
            $this->auth->redirectSession();
        }
        
        if(isset($_POST['redirect_session']) && $this->authorization_version==3){
            $this->auth->redirectSession();
        }
        
        //Próba wylogowania
        if((isset($_GET['logout']) && $_GET['logout']==1) || (isset($_POST['logout']) && $_POST['logout']==1)){
            $this->auth->stop_session();
	//	$this->auth->errors['not_logged']=true ;//DAREK
            $this->redirect('self');
        }
        
        //Przekazywanie nazwy użytkownika
        $this->view->assign('username', $this->auth->username);
        define('CONTR_USER', $this->auth->username);
        
        //Uruchomienie domyślnych akcji kontrolera
        $this->setDefault();
        
        if(count($this->auth->errors)>0){
            if($this->action_type == 'ajax'){
                //Gdy żądanie wychodzi z ajaxa w akcji która wymaga autoryzacji
                //robimy reload strony aby pokazać okno logowania
                echo "<?xml version=\"1.0\" encoding=\"utf-8\" ?><xjx><cmd cmd=\"as\" id=\"ttdiv_1\" prop=\"innerHTML\">S</cmd><cmd cmd=\"js\">Swindow.location.reload()</cmd></xjx>";
                exit;
            }elseif(!$modal || $this->access[$this->action]['auth_require']){
                //Strona logowania zamiast strony wynikowej
                $this->view->assign('autherrors', $this->auth->errors);
                $this->loginForm();
            }else{
                //Strona zakryta oknem modalnym
                $this->view->assign('autherrors', $this->auth->errors);
                $this->view->assign('authmodal', 1);
                return;
            }
        }
    }
    
    /**
     * Wyświetlenie formularza logowania
     */
    protected function loginForm()
    {
        if($this->access[$this->action]['auth_secure'])
            $this->view->assign('need_secure', true);
        $this->view->setBody('login_form.tpl');
        $this->view->display();
        exit;
    }
    
    /**
     *  Tą funkcje należy przedefiniowywać w celu ustawienia parametrów dostępu
     *  na przykład:
     *  $this->access['index']['auth_require'] = true;
     *  $this->access['index']['auth_secure'] = true;
     *  $this->access['index']['allow_groups'] = array('EDITORS','USERADMINS');
     **/
    protected function setAccess(){}
    
    /**
     *  Funkcja przygotowujaca dane i wykonująca wszystkie akcje [przeniesione do konstruktora]
     *  funkcja do usunięcia!!!
     *  @deprecated
     **/
    public function execute(){}
    
    /**
     *  Tutaj można wstawić to co wykonywać ma kontroler tuż przed uruchomieniem
     *  każdej z akcji, tu jest już za późno na ustawianie dostępu, należy ją przedefiniowywać
     *  na przykład:
     *  $this->view->setCss('dla_calego_kontrolera.css');
     **/
    protected function basic_controler_action(){}
    protected function setDefault(){}
    
    /**
     *  Wykonywane gdy nie ma podanej akcji a jest szablon statyczny
     *  @param string $static_template - nazwa statycznego szablonu dla kontrolera
     **/
    public function static_action($static_template, $subtemplate = null)
    {
        if($subtemplate == null){
            $this->view->setTemplate("static/{$static_template}.tpl");
            /*$html = $this->view->display(false);
            //Podmiana linku typu "strona.html" na nasze linki "strona/"
            $matches = null;
            preg_match_all('(href=(\'|")([^/\\\'"]+)(\'|"))', $html, $matches);
            $from = $matches[2];
            $to = array();
            foreach($from as $file){
                list($action,) = explode('.', $file);
                $to[] = $action.'/';
            }
            $html = str_replace($from, $to, $html);
            $this->view->setContent($html);*/
            $this->view->display();
        }else{
            $this->view->setTemplate("static/{$static_template}/{$subtemplate}.tpl");
            $this->view->display();
        }
    }
    
    /**
     *  Przykładowa definicja domyślnej akcji 'index',
     *  wyświetla domyślny szablon
     **/
    public function action_index()
    {
        $this->view->setTemplate('index.tpl');
        $this->view->display();
    }
    
    /**
     *  Funkcja wspierająca przekierowania
     *  @param string $url - cel przekierowania, gdy puste powrót do poprzedniej strony, gdy 'self' przekierowanie na tą samą stronę
     **/
    public function redirect($url = null)
    {
        if($url==null) $url = $_SERVER['HTTP_REFERER'];
        elseif($url=='self') $url = str_replace(array('?logout=1', 'logout=1', ',logout,1', 'logout,1,', 'logout,1/', 'logout,1'), '', $_SERVER['REQUEST_URI']);
        elseif(defined('DOMAIN') && strpos($url, 'http')===false) $url = DOMAIN . $url;
        header('Location: ' . $url);
        exit($url);
    }
    
    /**
     *  Aktualizacja w bazie informacji o konrolerze, w bazie testowej
     **/
    protected function saveSite()
    {
        if(!$this->save_site || $this->site_id==null)
            return;
        
        $sql = "SELECT COUNT(1) FROM main_site WHERE site_id='{$this->site_id}'";
        if(PDOS::_()->getOne($sql)==0){
            $sql = "INSERT INTO main_site (site_id) VALUES ('{$this->site_id}')";
            PDOS::_()->exec($sql);
        }elseif(rand(0,100)<0){
            return;//Omijaj dalsze sprawdzenia i aktualizacje
        }
        
        $devel = false;
        if(strpos($_SERVER['SERVER_NAME'], 'dev3.')!==false || strpos($_SERVER['SERVER_NAME'], 'tst.')!==false)
            $devel = true;
        
        if(defined('DOMAIN'))
            $host = DOMAIN;
        elseif(isset($_SERVER['HTTP_HOST']))
            $host = $_SERVER['HTTP_HOST'];
        else
            $host = null;
        
        if(defined('DOC_ROOT'))
            $path = DOC_ROOT;
        elseif(isset($_SERVER['SCRIPT_FILENAME']))
            $path = $_SERVER['SCRIPT_FILENAME'];
        else
            $path = null;
        
        if($devel){
            $sql = "UPDATE main_site SET dev_host='{$host}', dev_path='{$path}', last_upd=NOW() WHERE site_id='{$this->site_id}'";
        }else{
            $sql = "UPDATE main_site SET host='{$host}', path='{$path}', last_upd=NOW() WHERE site_id='{$this->site_id}'";
        }
        PDOS::_()->exec($sql);
        return;
    }
    
    /**
     * 
     */
    public function returnActionType()
    {
        return $this->action_type;
    }
    
    /**
     * 
     */
    public function returnSiteId()
    {
        return $this->site_id;
    }
}

?>
