<?php

require_once('Controller.class.php');

/**
 * 
 * Rozszerzenie kontrolera specjalnie na potrzeby CMSa, dodano uprawnienia.
 * @author Slawek
 *
 */
abstract class ControllerCMS extends Controller
{
    protected $authorization_version = 1;
    
    /**
     *  Autoryzacja użytkownika
     */
    protected function authorize()
    {
		
        //Ustawianie parametrów dostępu
        $this->access[$this->action] = $this->default_access;
        $this->setAccess();
        
        //Uruchamianie obiektu autoryzacji
        require_once 'authorize.class.php';
        if(defined('DBNAME'))
            $this->auth = new Authorize($this->access[$this->action], PDOS::getInstance(DBNAME));
        else
            $this->auth = new Authorize($this->access[$this->action]);
        
        //Próba zalogowania
        if(isset($_POST['authlogin']) && isset($_POST['authpasswd'])){
            $login = $_POST['authlogin'];
            $passwd = $_POST['authpasswd'];
            $secure = isset($_POST['authsecure'])?true:false;
            if($this->auth->start_session($login, $passwd, $secure))
                $this->redirect('self');
        }
        
        //Sprawdzenie sesji usera, zgodnie z Authorization_engine
        $this->auth->refresh_session();
        
        //Próba wylogowania
        if((isset($_GET['logout']) && $_GET['logout']==1) || (isset($_POST['logout']) && $_POST['logout']==1)){
            $this->auth->stop_session();
            $this->redirect('self');
        }
        
        //Sprawdzenie dostępu do akcji kontrolera CMS
        if(defined('NEWCMS') && constant('NEWCMS')==true){//Nowy CMS
            $this->auth->checkCmsAccess($this->name, $this->action);
        }else{//Stary CMS
            $this->auth->check_access_action($this->name, $this->action);
            //Lista dostępnych dla zalogowanego akcji
            $this->view->assign('access', $this->auth->get_access());
        }
        
        //Przekazywanie nazwy użytkownika
        $this->view->assign('username', $this->auth->username);
        define('CONTR_USER', $this->auth->username);
        
        if(count($this->auth->errors)>0){
			
            $this->loginForm();
			
        }
    }
    
    /**
     * Wyświetlenie formularza logowania
     */
    protected function loginForm()
    {
		
        $this->view->assign('errors', $this->auth->errors);
        $this->view->setBody('login_form.tpl');
        $this->view->display();
        exit;
    }
    
    /**
     *  Aktualizacja w bazie informacji o konrolerze, w bazie testowej
     */
    protected function saveSite()
    {
        if(!$this->save_site)
            return;
        
        $system = null;
        if($this->site_id!=null)
            $system = $this->site_id;
        $controller = $this->name;
        
        require_once('DB_Easy.class.php');
        $db = DB_connect('burak_editor');
        if($system!=null){
            $sql = "SELECT COUNT(1) FROM cms.system WHERE system_name = '{$system}'";
            if(!$db->getOne($sql))
                $db->exec("INSERT INTO cms.system (system_name) VALUES ('{$system}')");
        }
        
        $actions = get_class_methods($controller.'Controller');
        foreach($actions as $action){
            if(substr($action, 0, 7) != 'action_')
                continue;
            $action = str_replace('action_', '', $action);
            
            $in1 = false;
            $sql = "SELECT COUNT(1) FROM cms.actions WHERE controller='{$controller}' AND action='{$action}'";
            if($db->getOne($sql)){
                $sql = "UPDATE cms.actions SET last_update=NOW() WHERE controller='{$controller}' AND action='{$action}'";
                $db->exec($sql);
            }else{
                $id1 = $db->getOne('SELECT MAX(action_id) FROM cms.actions');
                $action_id = $id1+1;
                $sql = "INSERT INTO cms.actions (action_id, controller, action, system, last_update) VALUES ('{$action_id}', '{$controller}', '{$action}', '{$system}', NOW())";
                $db->exec($sql);
            }
        }
    }
}

?>