<?php

namespace Model;

use System\Database\AbstractTable;
use System\Admin\AdminModelInterface;

class AdminDictLGroups extends AbstractTable implements AdminModelInterface {

    protected $table = 'cms.dict_lgroups';
    protected $fields = array('group_id', 'group_name', 'group_descr', 'is_active');

    public function getRecords(array $queryConfig = array()) {
        try {
            $query = "SELECT * FROM {$this->table}";

            $query = $this->prepareWhereQuery($query, $queryConfig);
            $query = $this->prepareQuery($query, $queryConfig);
            $resource = $this->get($query);
            return array($resource, $this->getRecordsCount($queryConfig));
        } catch (\Exception $Exception) {
            return array(array(), 0);
        }
    }

    public function getRecordsCount(array $queryConfig = array()) {
        try {
            $query = "SELECT COUNT(*) as amount FROM {$this->table}";

            $query = $this->prepareWhereQuery($query, $queryConfig);
            $resource = $this->get($query)->current();
            return (int) $resource['amount'];
        } catch (\Exception $Exception) {
            return 0;
        }
    }

    public function getRecord($ID) {
        try {
            return $resource = $this->get("SELECT * FROM {$this->table} WHERE group_id = {$ID}")->current();
        } catch (\Exception $Exception) {
            return array();
        }
    }
    
    public function getRecordByName($groupName) {
        try {
            return $resource = $this->get("SELECT * FROM {$this->table} WHERE group_name = '{$groupName}'")->current();
        } catch (\Exception $Exception) {
            return array();
        }
    }

    public function insertRecord(array $recordData) {
        try {
            $fields = array();
            $values = array();
            foreach ($recordData as $field => $value) {
                if (($value && $value != "''") || $field == 'is_active') {
                    $fields[] = $field;
                    if ($field == 'is_active') {
                        $values[] = $value ? 'true' : 'false';
                    } else {
                        $values[] = $value;
                    }
                }
            }

            $fields = '(' . implode(',', $fields) . ')';
            $values = '(' . implode(',', $values) . ')';

            $ID = (int) $this->adapter->createStatement("INSERT INTO {$this->table}
                {$fields} VALUES {$values}
                RETURNING group_id")->execute()->current()['group_id'];

            return ($ID > 0) ? $ID : null;
        } catch (\Exception $Exception) {
            return null;
        }
    }

    public function updateRecord($ID, array $recordData) {
        try {
            $fieldsAndValues = array();
            foreach ($recordData as $field => $value) {
                if (($value && $value != "''") || $field == 'is_active') {
                    if ($field == 'is_active') {
                        $fieldsAndValues[] = ' ' . $field . '=' . ($value ? 'true' : 'false') . ' ';
                    } else {
                        $fieldsAndValues[] = ' ' . $field . '=' . $value . ' ';
                    }
                } else {
                    $fieldsAndValues[] = ' ' . $field . '= null ';
                }
            }
            $fieldsAndValues = implode(',', $fieldsAndValues);

            $this->adapter->createStatement("UPDATE {$this->table} SET {$fieldsAndValues} WHERE group_id = {$ID}")->execute();

            return true;
        } catch (\Exception $Exception) {
            return false;
        }
    }

    public function deleteRecord($ID) {
        try {
            $this->adapter->createStatement("DELETE FROM {$this->table} WHERE group_id = {$ID}")->execute();
            return true;
        } catch (\Exception $Exception) {
            return false;
        }
    }

    protected function prepareWhereQuery($query, array $queryConfig = array()) {
        if (isset($queryConfig['where']) && count($queryConfig['where']) > 0) {
            $whereQuery = array();
            foreach ($queryConfig['where'] as $where) {
                reset($where);
                $field = trim(key($where));
                $value = trim($where[$field]);
                if (in_array($field, $this->fields)) {
                    switch ($field) {
                        case 'group_id':
                            $whereQuery[] = " {$field} = {$value} ";
                            break;
                        case 'group_name':
                        case 'group_descr':
                            $whereQuery[] = " {$field} ILIKE '%{$value}%' ";
                            break;
                        case 'is_active':
                            $whereQueryBool = array();
                            foreach (explode(',', $value) as $subValue) {
                                $whereQueryBool[] = " {$field} = '{$subValue}' ";
                            }
                            $whereQuery[] = ' (' . implode(' OR ', $whereQueryBool) . ') ';
                            break;
                    }
                }
            }
            if (!empty($whereQuery)) {
                $query .= ' WHERE ' . implode(' AND ', $whereQuery);
            }
        }

        return $query;
    }

}

?>