<?php

namespace Model;

use System\Database\AbstractTable;
use System\Admin\AdminModelInterface;
use Zend\Db\Sql\Sql as Sql;
use Zend\Db\Sql\TableIdentifier as TableIdentifier;
use Zend\Db\ResultSet\ResultSet as ResultSet;

class AdminMainActions extends AbstractTable implements AdminModelInterface {

    protected $table = 'cms.main_actions';
    protected $fields = array('action_id', 'actions.action_id',
        'system', 'actions.system',
        'controller', 'actions.controller',
        'action', 'actions.action',
        'last_update', 'actions.last_update',
        'exists', 'actions.exists',
        'is_new', 'actions.is_new');

    public function getRecords(array $queryConfig = array()) {
        try {
            $query = "SELECT * FROM {$this->table} AS actions ";

            reset($queryConfig['order']);
            $orderField = key($queryConfig['order']);
            $orderDirection = $queryConfig['order'][$orderField];
            unset($queryConfig['order'][$orderField]);
            $queryConfig['order']['actions.' . $orderField] = $orderDirection;

            $query = $this->prepareWhereQuery($query, $queryConfig);
            $query = $this->prepareQuery($query, $queryConfig);
            $resource = $this->get($query);

            return array($resource, $this->getRecordsCount($queryConfig));
        } catch (\Exception $Exception) {
            return array(array(), 0);
        }
    }

    public function getRecordsCount(array $queryConfig = array()) {
        try {
            $query = "SELECT COUNT(*) as amount FROM {$this->table} AS actions ";

            $query = $this->prepareWhereQuery($query, $queryConfig);
            $resource = $this->get($query)->current();
            return (int) $resource['amount'];
        } catch (\Exception $Exception) {
            return 0;
        }
    }

    public function getRecord($ID) {
        try {
            return $resource = $this->get("SELECT * FROM {$this->table} WHERE action_id = {$ID}")->current();
        } catch (\Exception $Exception) {
            return array();
        }
    }

    public function insertRecord(array $recordData) {
        try {
            $exists = $recordData['exists'] ? 'true' : 'false';
            $isNew = $recordData['is_new'] ? 'true' : 'false';

            $ID = (int) $this->adapter->createStatement("INSERT INTO {$this->table} 
            (system, controller, action, last_update, exists, is_new) VALUES 
            ('{$recordData['system']}', '{$recordData['controller']}', '{$recordData['action']}', NOW(), {$exists}, {$isNew})
                RETURNING action_id")->execute()->current()['action_id'];

            return ($ID > 0) ? $ID : null;
        } catch (\Exception $Exception) {
            return null;
        }
    }

    public function getUniqueRecord($system, $controller, $action) {
        try {
            return $resource = $this->get("SELECT * FROM {$this->table} WHERE system = '{$system}' AND controller = '{$controller}' AND action = '{$action}'")->current();
        } catch (\Exception $Exception) {
            return array();
        }
    }

    public function updateRecord($ID, array $recordData) {
        try {
            $exists = $recordData['exists'] ? 'true' : 'false';
            $isNew = $recordData['is_new'] ? 'true' : 'false';

            $this->adapter->createStatement("UPDATE {$this->table} SET
            system = '{$recordData['system']}', controller = '{$recordData['controller']}', action = '{$recordData['action']}', last_update = NOW(), exists = {$exists}, is_new = {$isNew}
            WHERE action_id = {$ID}
            RETURNING action_id")->execute();

            return true;
        } catch (\Exception $Exception) {
            return false;
        }
    }

    public function deleteRecord($ID) {
        try {
            $this->adapter->createStatement("DELETE FROM {$this->table} WHERE action_id = {$ID}")->execute();
            return true;
        } catch (\Exception $Exception) {
            return false;
        }
    }

    protected function prepareWhereQuery($query, array $queryConfig = array()) {
        if (isset($queryConfig['where']) && count($queryConfig['where']) > 0) {
            $whereQuery = array();
            foreach ($queryConfig['where'] as $where) {
                reset($where);
                $field = trim(key($where));
                $value = trim($where[$field]);
                if (in_array($field, $this->fields) || $field == 'last_update_min' || $field == 'last_update_max' || $field == 'group_id') {
                    switch ($field) {
                        case 'action_id':
                            $whereQuery[] = " actions.{$field} = {$value} ";
                            break;
                        case 'system':
                        case 'controller':
                        case 'action':
                            $whereQuery[] = " actions.{$field} ILIKE '%{$value}%' ";
                            break;
                        case 'last_update_min':
                            $value = explode('/', $value);
                            $whereQuery[] = " actions.last_update >= '" . $value[2] . '-' . $value[1] . '-' . $value[0] . " 00:00:00' ";
                            break;
                        case 'last_update_max':
                            $value = explode('/', $value);
                            $whereQuery[] = " actions.last_update <= '" . $value[2] . '-' . $value[1] . '-' . $value[0] . " 23:59:59' ";
                            break;
                        case 'exists':
                        case 'is_new':
                            $whereQueryBool = array();
                            foreach (explode(',', $value) as $subValue) {
                                $whereQueryBool[] = " actions.{$field} = '{$subValue}' ";
                            }
                            $whereQuery[] = ' (' . implode(' OR ', $whereQueryBool) . ') ';
                            break;
                        case 'group_id':
                            $whereQuery[] = " rel.{$field} = {$value} ";
                            break;
                    }

                    if ($field == 'group_id') {
                        $query .= ' INNER JOIN cms.rel_group_action AS rel ON (actions.action_id = rel.action_id) ';
                    }
                }
            }
            if (!empty($whereQuery)) {
                $query .= ' WHERE ' . implode(' AND ', $whereQuery);
            }
        }
        
        return $query;
    }

    public function getSystems() {
        $SQL = new Sql($this->adapter);
        $select = $SQL->select(new TableIdentifier(explode('.', $this->table)[1], explode('.', $this->table)[0]));

        $select = $select->columns(array('system'))->quantifier(\Zend\Db\Sql\Select::QUANTIFIER_DISTINCT);
        $select = $SQL->prepareStatementForSqlObject($select);
        $resultSet = new ResultSet;
        $resultSet->initialize($select->execute());

        $systems = array();
        foreach ($resultSet as $result) {
            $systems[] = $result->getArrayCopy()['system'];
        }

        return $systems;
    }

    public function getControllers() {
        $SQL = new Sql($this->adapter);
        $select = $SQL->select(new TableIdentifier(explode('.', $this->table)[1], explode('.', $this->table)[0]));

        $select = $select->columns(array('controller'))->quantifier(\Zend\Db\Sql\Select::QUANTIFIER_DISTINCT);
        $select = $SQL->prepareStatementForSqlObject($select);
        $resultSet = new ResultSet;
        $resultSet->initialize($select->execute());

        $controllers = array();
        foreach ($resultSet as $result) {
            $controllers[] = $result->getArrayCopy()['controller'];
        }

        return $controllers;
    }

    public function getActionsToAdd($groupID) {
        $SQL = new Sql($this->adapter);
        $actionTable = new TableIdentifier(explode('.', $this->table)[1], explode('.', $this->table)[0]);
        $actionGroupTable = new TableIdentifier('rel_group_action', 'cms');

        $groupID = (int) $groupID;

        $where = new \Zend\Db\Sql\Where();
        $where->equalTo('action_group.group_id', $groupID);
        $subSelect = $SQL->select(array('actions' => $actionTable))
                ->columns(array('action_id'))
                ->join(array('action_group' => $actionGroupTable), 'actions.action_id = action_group.action_id', array(), \Zend\Db\Sql\Select::JOIN_LEFT)
                ->where($where)
                ->group(array('actions.action_id'))
                ->order('actions.action_id ASC');

        $where = new \Zend\Db\Sql\Where();
        $where->addPredicate(new \Zend\Db\Sql\Predicate\NotIn('action_group.action_id', $subSelect))->or->isNull('action_group.group_id');

        $select = $SQL->select(array('actions' => $actionTable))
                ->columns(array('action_id', 'system', 'controller', 'action'))
                ->join(array('action_group' => $actionGroupTable), 'actions.action_id = action_group.action_id', array(), \Zend\Db\Sql\Select::JOIN_LEFT)
                ->where($where)
                ->group(array('actions.action_id', 'actions.system', 'actions.controller', 'actions.action'))
                ->order('actions.action_id ASC');
        $select = $SQL->prepareStatementForSqlObject($select);
        $resultSet = new \Zend\Db\ResultSet\ResultSet();
        $resultSet->initialize($select->execute());

        $actions = array();
        foreach ($resultSet as $result) {
            $actions[] = $result->getArrayCopy();
        }

        return $actions;
    }

    public function getActionsToDelete($groupID) {
        $SQL = new Sql($this->adapter);
        $actionTable = new TableIdentifier(explode('.', $this->table)[1], explode('.', $this->table)[0]);
        $actionGroupTable = new TableIdentifier('rel_group_action', 'cms');

        $groupID = (int) $groupID;

        $where = new \Zend\Db\Sql\Where();
        $where->EqualTo('action_group.group_id', $groupID);

        $select = $SQL->select(array('actions' => $actionTable))
                ->columns(array('action_id', 'system', 'controller', 'action'))
                ->join(array('action_group' => $actionGroupTable), 'actions.action_id = action_group.action_id', array(), \Zend\Db\Sql\Select::JOIN_LEFT)
                ->where($where)
                ->group(array('actions.action_id', 'actions.system', 'actions.controller', 'actions.action'))
                ->order('actions.action_id ASC');
        $select = $SQL->prepareStatementForSqlObject($select);
        $resultSet = new \Zend\Db\ResultSet\ResultSet();
        $resultSet->initialize($select->execute());

        $actions = array();
        foreach ($resultSet as $result) {
            $actions[] = $result->getArrayCopy();
        }

        return $actions;
    }

}

?>
