<?php

namespace Model;

use System\Database\AbstractTable;
use System\Admin\AdminModelInterface;
use Zend\Db\Sql\Sql as Sql;
use Zend\Db\Sql\TableIdentifier as TableIdentifier;
use Zend\Db\ResultSet\ResultSet as ResultSet;

class AdminMainUsers extends AbstractTable implements AdminModelInterface {

    protected $table = 'cms.main_users';
    protected $fields = array(
        'user_id', 'users.user_id',
        'user_login', 'users.user_login',
        'name', 'users.name',
        'lastname', 'users.lastname',
        'esc_id', 'users.esc_id');

    public function getRecords(array $queryConfig = array()) {
        try {
            $query = "SELECT * FROM {$this->table} AS users ";

            $query = $this->prepareWhereQuery($query, $queryConfig);

            reset($queryConfig['order']);
            $orderField = key($queryConfig['order']);
            $orderDirection = $queryConfig['order'][$orderField];
            unset($queryConfig['order'][$orderField]);
            $queryConfig['order']['users.' . $orderField] = $orderDirection;

            $query = $this->prepareQuery($query, $queryConfig);
            $resource = $this->get($query);
            return array($resource, $this->getRecordsCount($queryConfig));
        } catch (\Exception $Exception) {
            return array(array(), 0);
        }
    }

    public function getRecordsCount(array $queryConfig = array()) {
        try {
            $query = "SELECT COUNT(*) as amount FROM {$this->table} AS users ";

            $query = $this->prepareWhereQuery($query, $queryConfig);
            $resource = $this->get($query)->current();
            return (int) $resource['amount'];
        } catch (\Exception $Exception) {
            return 0;
        }
    }

    public function getRecord($ID) {
        try {
            return $resource = $this->get("SELECT * FROM {$this->table} WHERE user_id = {$ID}")->current();
        } catch (\Exception $Exception) {
            return array();
        }
    }

    public function getRecordByEsculapID($EsculapID) {
        try {
            return $resource = $this->get("SELECT * FROM {$this->table} WHERE esc_id = {$EsculapID}")->current();
        } catch (\Exception $Exception) {
            return array();
        }
    }

    public function insertRecord(array $recordData) {
        try {
            $fields = array();
            $values = array();
            foreach ($recordData as $field => $value) {
                if ($value && $value != "''") {
                    $fields[] = $field;
                    $values[] = $value;
                }
            }

            $fields = '(' . implode(',', $fields) . ')';
            $values = '(' . implode(',', $values) . ')';

            $ID = (int) $this->adapter->createStatement("INSERT INTO {$this->table}
                {$fields} VALUES {$values}
                RETURNING user_id")->execute()->current()['user_id'];

            return ($ID > 0) ? $ID : null;
        } catch (\Exception $Exception) {
            return null;
        }
    }

    public function updateRecord($ID, array $recordData) {
        try {
            $fieldsAndValues = array();
            foreach ($recordData as $field => $value) {
                if ($value && $value != "''") {
                    $fieldsAndValues[] = ' ' . $field . '=' . $value . ' ';
                } else {
                    $fieldsAndValues[] = ' ' . $field . '= null ';
                }
            }
            $fieldsAndValues = implode(',', $fieldsAndValues);

            $this->adapter->createStatement("UPDATE {$this->table} SET {$fieldsAndValues} WHERE user_id = {$ID}")->execute();

            return true;
        } catch (\Exception $Exception) {
            return false;
        }
    }

    public function deleteRecord($ID) {
        try {
            $this->adapter->createStatement("DELETE FROM {$this->table} WHERE user_id = {$ID}")->execute();
            return true;
        } catch (\Exception $Exception) {
            return false;
        }
    }

    protected function prepareWhereQuery($query, array $queryConfig = array()) {
        if (isset($queryConfig['where']) && count($queryConfig['where']) > 0) {
            $whereQuery = array();
            foreach ($queryConfig['where'] as $where) {
                reset($where);
                $field = trim(key($where));
                $value = trim($where[$field]);
                if (in_array($field, $this->fields) || $field == 'group_id' || $field == 'lgroup_id') {
                    switch ($field) {
                        case 'user_id':
                        case 'esc_id':
                            $whereQuery[] = " users.{$field} = {$value} ";
                            break;
                        case 'user_login':
                        case 'name':
                        case 'lastname':
                            $whereQuery[] = " users.{$field} ILIKE '%{$value}%' ";
                            break;
                        case 'group_id':
                            $whereQuery[] = " rel.{$field} = {$value} ";
                            break;
                        case 'lgroup_id':
                            $whereQuery[] = " rel.group_id = {$value} ";
                            break;
                    }
                }

                if ($field == 'group_id') {
                    $query .= ' INNER JOIN cms.rel_group_user AS rel ON (users.user_id = rel.user_id) ';
                }
                if ($field == 'lgroup_id') {
                    $query .= ' INNER JOIN cms.rel_lgroup_user AS rel ON (users.user_id = rel.user_id) ';
                }
            }
            if (!empty($whereQuery)) {
                $query .= ' WHERE ' . implode(' AND ', $whereQuery);
            }
        }
        return $query;
    }

    public function getUsersToAdd($groupID) {
        $SQL = new Sql($this->adapter);
        $userTable = new TableIdentifier(explode('.', $this->table)[1], explode('.', $this->table)[0]);
        $userGroupTable = new TableIdentifier('rel_group_user', 'cms');

        $groupID = (int) $groupID;

        $where = new \Zend\Db\Sql\Where();
        $where->equalTo('user_group.group_id', $groupID);
        $subSelect = $SQL->select(array('users' => $userTable))
                ->columns(array('user_id'))
                ->join(array('user_group' => $userGroupTable), 'users.user_id = user_group.user_id', array(), \Zend\Db\Sql\Select::JOIN_LEFT)
                ->where($where)
                ->group(array('users.user_id'))
                ->order('users.user_id ASC');

        $where = new \Zend\Db\Sql\Where();
        $where->addPredicate(new \Zend\Db\Sql\Predicate\NotIn('user_group.user_id', $subSelect))->or->isNull('user_group.group_id');

        $select = $SQL->select(array('users' => $userTable))
                ->columns(array('user_id', 'user_login'))
                ->join(array('user_group' => $userGroupTable), 'users.user_id = user_group.user_id', array(), \Zend\Db\Sql\Select::JOIN_LEFT)
                ->where($where)
                ->group(array('users.user_id', 'users.user_login'))
                ->order('users.user_id ASC');
        $select = $SQL->prepareStatementForSqlObject($select);

        $resultSet = new \Zend\Db\ResultSet\ResultSet();
        $resultSet->initialize($select->execute());

        $users = array();
        foreach ($resultSet as $result) {
            $users[] = $result->getArrayCopy();
        }

        return $users;
    }

    public function getUsersToDelete($groupID) {
        $SQL = new Sql($this->adapter);
        $userTable = new TableIdentifier(explode('.', $this->table)[1], explode('.', $this->table)[0]);
        $userGroupTable = new TableIdentifier('rel_group_user', 'cms');

        $groupID = (int) $groupID;

        $where = new \Zend\Db\Sql\Where();
        $where->EqualTo('user_group.group_id', $groupID);

        $select = $SQL->select(array('users' => $userTable))
                ->columns(array('user_id', 'user_login'))
                ->join(array('user_group' => $userGroupTable), 'users.user_id = user_group.user_id', array(), \Zend\Db\Sql\Select::JOIN_LEFT)
                ->where($where)
                ->group(array('users.user_id', 'users.user_login'))
                ->order('users.user_id ASC');
        $select = $SQL->prepareStatementForSqlObject($select);
        $resultSet = new \Zend\Db\ResultSet\ResultSet();
        $resultSet->initialize($select->execute());

        $users = array();
        foreach ($resultSet as $result) {
            $users[] = $result->getArrayCopy();
        }

        return $users;
    }

    public function getUsersToAddToLGroup($groupID) {
        $SQL = new Sql($this->adapter);
        $userTable = new TableIdentifier(explode('.', $this->table)[1], explode('.', $this->table)[0]);
        $userGroupTable = new TableIdentifier('rel_lgroup_user', 'cms');

        $groupID = (int) $groupID;

        $where = new \Zend\Db\Sql\Where();
        $where->equalTo('user_group.group_id', $groupID);
        $subSelect = $SQL->select(array('users' => $userTable))
                ->columns(array('user_id'))
                ->join(array('user_group' => $userGroupTable), 'users.user_id = user_group.user_id', array(), \Zend\Db\Sql\Select::JOIN_LEFT)
                ->where($where)
                ->group(array('users.user_id'))
                ->order('users.user_id ASC');

        $where = new \Zend\Db\Sql\Where();
        $where->addPredicate(new \Zend\Db\Sql\Predicate\NotIn('user_group.user_id', $subSelect))->or->isNull('user_group.group_id');

        $select = $SQL->select(array('users' => $userTable))
                ->columns(array('user_id', 'user_login'))
                ->join(array('user_group' => $userGroupTable), 'users.user_id = user_group.user_id', array(), \Zend\Db\Sql\Select::JOIN_LEFT)
                ->where($where)
                ->group(array('users.user_id', 'users.user_login'))
                ->order('users.user_id ASC');
        $select = $SQL->prepareStatementForSqlObject($select);

        $resultSet = new \Zend\Db\ResultSet\ResultSet();
        $resultSet->initialize($select->execute());

        $users = array();
        foreach ($resultSet as $result) {
            $users[] = $result->getArrayCopy();
        }

        return $users;
    }

    public function getUsersToDeleteFromLGroup($groupID) {
        $SQL = new Sql($this->adapter);
        $userTable = new TableIdentifier(explode('.', $this->table)[1], explode('.', $this->table)[0]);
        $userGroupTable = new TableIdentifier('rel_lgroup_user', 'cms');

        $groupID = (int) $groupID;

        $where = new \Zend\Db\Sql\Where();
        $where->EqualTo('user_group.group_id', $groupID);

        $select = $SQL->select(array('users' => $userTable))
                ->columns(array('user_id', 'user_login'))
                ->join(array('user_group' => $userGroupTable), 'users.user_id = user_group.user_id', array(), \Zend\Db\Sql\Select::JOIN_LEFT)
                ->where($where)
                ->group(array('users.user_id', 'users.user_login'))
                ->order('users.user_id ASC');
        $select = $SQL->prepareStatementForSqlObject($select);
        $resultSet = new \Zend\Db\ResultSet\ResultSet();
        $resultSet->initialize($select->execute());

        $users = array();
        foreach ($resultSet as $result) {
            $users[] = $result->getArrayCopy();
        }

        return $users;
    }

}

?>