<?php
namespace System\Content;

use System\Content\Config as ContentConfig;
use System\Content\CollectionConfig;
use Model\ContentMainContentCollection as MainCollection;
use System\Content\Element;

class ContentFactory {

    public function __construct() {
        
    }

    public static function collectionFactory(CollectionConfig $config) {
        
    }

    public static function factory($config, &$count = null) {

        if (is_array($config)) {
            $config = new ContentConfig($config);
        }

        if (null == $config->CONTENT_TYPE) {

            $className = '\Model\MainContentSearchHelper';
        } else {

            $className = '\Model\MainContent' . ucfirst(strtolower($config->CONTENT_TYPE));
        }



        $dbObject = new $className;

        $specs = is_array($config->SPEC_TYPES) ? implode(',', $config->SPEC_TYPES) : $config->SPEC_TYPES;


        // nazwa metody, która wyszuka kontent. ex dla esculapa zawiera profilowanie
        $methodName = '';
        if ($config->isCms()) {

            $methodName = 'getElements';
        } else {

            $methodName = 'getElementsEx';
        }

        $elements = $dbObject->$methodName(
                preg_replace('/([a-zA-Z0-9\.]+)/', '\'$1\'', $specs), $config->ROWS_NUMBER, $config->ROWS_START, $config->getOptions()
        );

        if (count($elements) == 2 && $dbObject instanceof \Model\MainContentQuiz) {
            //var_dump($elements);
        }

        if (count($elements) > 0 && $dbObject instanceof \Model\MainContentPublication) {
            $idList = array();
            $idPublList = array();

            foreach ($elements as $element) {
                if (null !== $element['journal_id']) {
                    $idList[$element['journal_id']] = $element['journal_id'];
                }
                $idPublList[$element['content_id']] = $element['content_id'];
            }

            if (count($idList)) {
                $DictJournals = new \Model\DictJournals;
                $journals = $DictJournals->getJournals($idList);
                
                $MainEditions = new \Model\MainEditions;
                $editions = $MainEditions->getEditions($idPublList);
                
                foreach ($elements as $key => $element) {
                    if (!array_key_exists($element['journal_id'], $journals)) {
                        $element['journalName'] = '';
                    } else {
                        $elements[$key]['journalName'] = $journals[$element['journal_id']];
                    }
                    
                    if (!array_key_exists($element['content_id'], $editions)) {
                        $element['edition'] = '';
                    } else {
                        $elements[$key]['edition'] = $editions[$element['content_id']];
                    }
                }
                
            }
        } elseif (count($elements) > 0 && $dbObject instanceof \Model\MainContentQuiz) {

            $idList = array();

            foreach ($elements as $element) {

                if (1 > (int)$element['content_id']) continue;

                $idList[] = $element['content_id'];
            }



            $ExtContentQiuz = new \Model\ExtContentQuiz;
            $answers = $ExtContentQiuz->getAnswers($idList, $element['lang_id']);

            $answersSet = array();

            foreach ($answers as $k => $answer) {

                $answersSet[$answer['content_id']][] = $answer;
            }

            foreach ($elements as $key => $element) {

                $element['answers'] = isset($answersSet[$element['content_id']]) ? $answersSet[$element['content_id']] : array();
                $elements[$key] = $element;
            }
        }

        if ($config->GET_COMMENTS == 1 && count($elements)) {
            $contentIdList = array();


            foreach ($elements as $element) {

                $contentIdList[] = $element['content_id'];
            }

            if (count($contentIdList)) {
                $MainContentComment = new \Model\MainContentComment();
                $comments = $MainContentComment->getComments($contentIdList);

                // grupowanie komentarzy
                $comments_tmp = array();
                foreach ($comments as $comment) {
                    $comments_tmp[$comment['content_id']][] = $comment;
                }
                $comments = $comments_tmp;

                foreach ($elements as $key => $element) {

                    if (!isset($comments[$element['content_id']])) {
                        continue;
                    }

                    $element['comments'] = $comments[$element['content_id']];
                    $elements[$key] = $element;
                }
            }
        }

        if ($config->GET_MAIN_SITE == true && count($elements)) {
            $mainSiteIdList = array();

            foreach ($elements as $element) {
                if (!in_array($element['main_site_id'], $mainSiteIdList)) {
                    $mainSiteIdList[] = $element['main_site_id'];
                }
            }

            if (count($mainSiteIdList)) {
                $mainSiteDict = new \Model\TableGateway\MainSite();
                $mainSites = $mainSiteDict->getByID($mainSiteIdList);

                $mainSitesTmp = array();
                foreach ($mainSites as $mainSite) {
                    if (!isset($mainSitesTmp[$mainSite->getId()])) {
                        $mainSitesTmp[$mainSite->getId()] = $mainSite;
                    }
                }

                foreach ($elements as $key => $element) {
                    $element['main_site'] = $mainSitesTmp[$element['main_site_id']];
                    $elements[$key] = $element;
                }
            }
        }

        if ($dbObject instanceof \Model\MainContentVideo) {
            foreach ($elements as &$element) {
                $VideoStream = new \Model\MainContentVideoStream();
                $MainStream = $VideoStream->getStream($element['main_stream_id']);
                //$ThumbStream = $VideoStream->getStream($element['thumb_stream_id']);
                $element['main_stream'] = $MainStream;
                //$element['thumb_stream'] = $ThumbStream;
            }
        }

        if (true == $config->LOAD_COLLECTIONS) {

            $contentIdList = array();
            foreach ($elements as $element) {

                $contentIdList[] = $element['content_id'];
            }

            $RelContentCollection = new \Model\RelContentCollection;
            $collectins = $RelContentCollection->getCollections($contentIdList);

            foreach ($elements as $key => $element) {

                $element['collections'] = implode(',', $collectins[$element['content_id']]);
                $elements[$key] = $element;
            }
        } else {

            foreach ($elements as &$element) {
                $element['collections'] = null;
            }
        }

        return self::prepareElements($elements);
    }

    public static function count(ContentConfig $config) {

        if (null == $config->CONTENT_TYPE) {

            $className = '\Model\MainContentSearchHelper';
        } else {

            $className = '\Model\MainContent' . ucfirst(strtolower($config->CONTENT_TYPE));
        }

        $dbObject = new $className;

        if ($config->isCms()) {

            $method = 'count';
        } else {

            $method = 'countEx';
        }

        $res = $dbObject->$method(
                preg_replace('/([0-9a-zA-Z\.]+)/', '\'$1\'', implode(',', $config->SPEC_TYPES)), $config->getOptions()
        );

        return (int)$res;
    }

    protected static function prepareElements($elementsList) {
        $result = array();

        $t = 0;
        if (count($elementsList))
                foreach ($elementsList as $element) {
                switch ($element['type']) {
                    case 'news':
                        $result[] = new Element\News($element);

                        break;
                    case 'publ':

                        $result[] = new Element\Publication($element);

                        break;
                    case 'movie':
                    case 'video':

                        $result[] = new Element\Video($element);

                        break;
                    case 'event':

                        $result[] = new Element\Event($element);

                        break;
                    case 'quiz':
                        $t = 1;
                        $result[] = new Element\Quiz($element);

                        break;
                }
            }

        return $result;
    }

}

?>