<?php

namespace Model;

use System\Database\AbstractTable,
	Zend\Db\Sql\Select,
	Zend\Db\Sql\Expression;

class ExtContentQuiz extends AbstractTable {
	protected $table = 'ext_content_quiz';
	
	public function getAnswers( $contentId , $langId ) {
		
		try {
			
			if( is_array( $contentId ) ) {
				foreach( $contentId as $k => $v )
					$contentId[ $k ] = (int)$v;
				
				$contentId = implode( ',' , $contentId );
				
				
			} else {
			
				$contentId = (int)$contentId;
			}
			
			
			$select = new Select;
			$select->from( $this->table )
				   ->where( "content_id in( {$contentId} )" )
				   ->where( array( 'lang_id' => $langId ) )
				   ->order( array( 'queue' => 'asc' ) );
			
			$res = $this->selectWith( $select );
			
			return $res->toArray();
			
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	
	public function isAnswerExists( $contentId , $langId , $answerNo ) {
		
		try {
			
			$res = $this->select( array( 'content_id' => (int)$contentId , 'lang_id' => $langId , 'answer_no' => (int)$answerNo ) );
			if( $res->count() )
				return true;
			return false;
			
		} catch( \Exception $e ) { 
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
		
	public function getGrouppedAnswers( $contentId , $langId ) {
		
		try {
			
			$langId = strtoupper( $langId );
			
			$select = new Select;
			$select->from( $this->table )
				   ->columns( array( '*' ,
									 'count' => new Expression( "(
									 SELECT count( 1 ) from rel_content_quiz_answer
									 WHERE rel_content_quiz_answer.content_id = {$contentId} AND 
										   lang_id = '{$langId}' AND 
										   rel_content_quiz_answer.answer_no = {$this->table }.answer_no )" ) ) )
				   ->where( array( 'ext_content_quiz.content_id' => $contentId , 'ext_content_quiz.lang_id' => $langId  ) );
			
			return $this->selectWith( $select )->toArray();
			
		} catch( \Exceptoin $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	
	public function addAnswer( $contentId , $langId , $answerText , $answerQueue , $type = 'radio' ) {
		
		try {
			
			$this->insert(array(
				'content_id' => $contentId,
				'lang_id' => strtoupper( $langId ),
				'description' => $answerText,
				'queue' => $answerQueue,
				'type' => $type
			));
			
			return $this->lastInsertValue;
			
			
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	
	public function remove( $id ) {
		
		try {
			
			if( is_array( $id ) ) {
				foreach( $id as $k => $v )
					$id[ $k ] = (int)$v;
				
			} else {
			
				$id = array( (int)$id );
			}
			
			foreach( $id as $v )
				$this->delete( array( 'answer_no' => $id ) );
		
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	
	public function updateAnswer( $answerNo , $answerText , $answerQueue ) {
		
		try {
			
			$this->update(array(
				'description' => $answerText,
				'queue' => $answerQueue
			) , "answer_no = {$answerNo}" );
			
		} catch( \Exceptoin $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	public function countAnswers( $contentId , $langId ) {
		
		try {
		
		
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
}


?>