<?php
namespace Model\HydratingResultSet;

use Zend\Db\ResultSet\HydratingResultSet;
use Zend\Stdlib\Hydrator\ClassMethods;
use Model\Entity\BannerPlace as BannerPlaceEntity;
use Model\Entity\MainSite as MainSiteEntity;
use Model\TableGateway\BannerPlace as BannerPlaceTableGateway;
use Model\TableGateway\MainSite as MainSiteTableGateway;

class BannerPlace extends HydratingResultSet {

    protected $bannerPlaceHydrator = null;
    protected $mainSiteHydrator = null;
    protected $bannerPlaceObjectPrototype = null;
    protected $mainSiteObjectPrototype = null;

    public function __construct() {
        $this->bannerPlaceHydrator = new ClassMethods();
        $this->mainSiteHydrator = new ClassMethods();
        $this->bannerPlaceObjectPrototype = new BannerPlaceEntity();
        $this->mainSiteObjectPrototype = new MainSiteEntity();
    }

    public function getBannerPlaceHydrator() {
        return $this->bannerPlaceHydrator;
    }

    public function getMainSiteHydrator() {
        return $this->mainSiteHydrator;
    }

    public function current() {
        if ($this->buffer === null) {
            $this->buffer = -2;
        } elseif (is_array($this->buffer) && isset($this->buffer[$this->position])) {
            return $this->buffer[$this->position];
        }

        $data = $this->dataSource->current();
        
        $bannerPlaceData = array();
        $mainSiteData = array();
        foreach ($data as $dataColumn => $dataValue) {
            if (substr($dataColumn, 0, strlen(MainSiteTableGateway::getAlias()) + 1) == MainSiteTableGateway::getAlias() . '.') {
                unset($data[$dataColumn]);
                $mainSiteData[str_replace( MainSiteTableGateway::getAlias() . '.', '', $dataColumn)] = $dataValue;
            }
        }
        $bannerPlaceData = $data;

        $bannerPlaceObject = is_array($data) ? $this->bannerPlaceHydrator->hydrate($bannerPlaceData, clone $this->bannerPlaceObjectPrototype) : false;
        $mainSiteObject = is_array($data) ? $this->mainSiteHydrator->hydrate($mainSiteData, clone $this->mainSiteObjectPrototype) : false;

        $bannerPlaceObject->setMainSite($mainSiteObject);

        if (is_array($this->buffer)) {
            $this->buffer[$this->position] = $bannerPlaceObject;
        }

        return $bannerPlaceObject;
    }

    public function toArray() {
        $return = array();
        foreach ($this as $row) {
            $return[] = array_merge($this->getBannerPlaceHydrator()->extract($row), $this->getMainSiteHydrator()->extract($row));
        }
        return $return;
    }

}
