<?php
namespace Model\HydratingResultSet;

use Zend\Db\ResultSet\HydratingResultSet;
use Zend\Stdlib\Hydrator\ClassMethods;
use Model\Entity\MainSiteContentPage as MainSiteContentPageEntity;
use Model\Entity\MainSite as MainSiteEntity;
use Model\Entity\DictMainSiteContentTypeView as DictMainSiteContentTypeViewEntity;
use Model\TableGateway\MainSiteContentPage as MainSiteContentPageTableGateway;
use Model\TableGateway\MainSite as MainSiteTableGateway;
use Model\TableGateway\DictMainSiteContentTypeView as DictMainSiteContentTypeViewTableGateway;

class MainSiteContentPage extends HydratingResultSet {

    protected $mainSiteContentPageHydrator = null;
    protected $mainSiteHydrator = null;
    protected $dictMainSiteContentViewTypeHydrator = null;
    protected $mainSiteContentPagePrototype = null;
    protected $mainSitePrototype = null;
    protected $dictMainSiteContentTypeViewPrototype = null;

    public function __construct() {
        $this->mainSiteContentPageHydrator = new ClassMethods();
        $this->mainSiteHydrator = new ClassMethods();
        $this->dictMainSiteContentTypeViewHydrator = new ClassMethods();

        $this->mainSiteContentPagePrototype = new MainSiteContentPageEntity();
        $this->mainSitePrototype = new MainSiteEntity();
        $this->dictMainSiteContentTypeViewPrototype = new DictMainSiteContentTypeViewEntity();
    }

    public function getMainSiteContentPageHydrator() {
        return $this->mainSiteContentPageHydrator;
    }

    public function getMainSiteHydrator() {
        return $this->mainSiteHydrator;
    }

    public function getDictMainSiteContentTypeViewHydrator() {
        return $this->dictMainSiteContentTypeViewHydrator;
    }

    public function current() {
        if ($this->buffer === null) {
            $this->buffer = -2;
        } elseif (is_array($this->buffer) && isset($this->buffer[$this->position])) {
            return $this->buffer[$this->position];
        }

        $data = $this->dataSource->current();

        $mainSiteContentPageData = array();
        $mainSiteData = array();
        $dictMainSiteContentTypeViewData = array();
        
        foreach ($data as $dataColumn => $dataValue) {
            if (substr($dataColumn, 0, strlen(MainSiteTableGateway::getAlias()) + 1) == MainSiteTableGateway::getAlias() . '.') {
                unset($data[$dataColumn]);
                $mainSiteData[str_replace(MainSiteTableGateway::getAlias() . '.', '', $dataColumn)] = $dataValue;
            } elseif (substr($dataColumn, 0, strlen(DictMainSiteContentTypeViewTableGateway::getAlias()) + 1) == DictMainSiteContentTypeViewTableGateway::getAlias() . '.') {
                unset($data[$dataColumn]);
                $dictMainSiteContentTypeViewData[str_replace(DictMainSiteContentTypeViewTableGateway::getAlias() . '.', '', $dataColumn)] = $dataValue;
            }
        }
        $mainSiteContentPageData = $data;
        
        if (!empty($mainSiteContentPageData)) {
            $mainSiteContentTypeEntity = $this->getMainSiteContentPageHydrator()->hydrate($mainSiteContentPageData, clone $this->mainSiteContentPagePrototype);
        } else {
            $mainSiteContentTypeEntity = null;
        }
        if (!empty($mainSiteData)) {
            $mainSiteEntity = $this->getMainSiteHydrator()->hydrate($mainSiteData, clone $this->mainSitePrototype);
        } else {
            $mainSiteEntity = null;
        }
        if (!empty($dictMainSiteContentTypeViewData)) {
            $dictMainSiteContentTypeViewEntity = $this->getDictMainSiteContentTypeViewHydrator()->hydrate($dictMainSiteContentTypeViewData, clone $this->dictMainSiteContentTypeViewPrototype);
        } else {
            $dictMainSiteContentTypeViewEntity = null;
        }

        if ($mainSiteContentTypeEntity != null) {
            if ($mainSiteEntity != null) {
                $mainSiteContentTypeEntity->setMainSite($mainSiteEntity);
            }
            if ($dictMainSiteContentTypeViewEntity != null) {
                $mainSiteContentTypeEntity->setDictMainSiteContentTypeView($dictMainSiteContentTypeViewEntity);
            }
        }
        
        if (is_array($this->buffer)) {
            $this->buffer[$this->position] = $mainSiteContentTypeEntity;
        }

        return $mainSiteContentTypeEntity;
    }

    public function toArray() {
        $return = array();
        foreach ($this as $row) {
            $return[] = array_merge(
                    $this->getMainSiteContentPageHydrator()->extract($row), $this->getMainSiteHydrator()->extract($row), $this->getDictMainSiteContentTypeViewHydrator()->extract($row)
            );
        }
        return $return;
    }

}
