<?php
namespace Model\HydratingResultSet;

use Zend\Db\ResultSet\HydratingResultSet;
use Zend\Stdlib\Hydrator\ClassMethods;

use Model\Entity\MainSiteUserAccess as MainSiteUserAccessEntity;
use Model\Entity\MainSite as MainSiteEntity;
use Model\Entity\MainUser as MainSiteUserEntity;

use Model\TableGateway\MainSite as MainSiteTableGateway;
use Model\TableGateway\MainUser as MainUserTableGateway;

class MainSiteUserAccess extends HydratingResultSet {

    protected $mainSiteUserAccessHydrator = null;
    protected $mainSiteHydrator = null;
    protected $mainUserHydrator = null;

    protected $mainSiteUserAccessPrototype = null;
    protected $mainSitePrototype = null;
    protected $mainUserPrototype = null;

    public function __construct() {
        $this->mainSiteUserAccessHydrator = new ClassMethods();
        $this->mainSiteHydrator = new ClassMethods();
        $this->mainUserHydrator = new ClassMethods();

        $this->mainSiteUserAccessPrototype = new MainSiteUserAccessEntity();
        $this->mainSitePrototype = new MainSiteEntity();
        $this->mainUserPrototype = new MainSiteUserEntity();
    }

    public function getMainSiteUserAccessHydrator() {
        return $this->mainSiteUserAccessHydrator;
    }

    public function getMainSiteHydrator() {
        return $this->mainSiteHydrator;
    }

    public function getMainUserHydrator() {
        return $this->mainUserHydrator;
    }

    public function current() {
        if ($this->buffer === null) {
            $this->buffer = -2;
        } elseif (is_array($this->buffer) && isset($this->buffer[$this->position])) {
            return $this->buffer[$this->position];
        }

        $data = $this->dataSource->current();

        $mainSiteData = array();
        $mainUserData = array();

        foreach ($data as $dataColumn => $dataValue) {
            if (substr($dataColumn, 0, strlen(MainSiteTableGateway::getAlias()) + 1) == MainSiteTableGateway::getAlias() . '.') {
                unset($data[$dataColumn]);
                $mainSiteData[str_replace(MainSiteTableGateway::getAlias() . '.', '', $dataColumn)] = $dataValue;
            }
            if (substr($dataColumn, 0, strlen(MainUserTableGateway::getAlias()) + 1) == MainUserTableGateway::getAlias() . '.') {
                unset($data[$dataColumn]);
                $mainUserData[str_replace(MainUserTableGateway::getAlias() . '.', '', $dataColumn)] = $dataValue;
            }
        }
        $mainSiteUserAccessData = $data;

        if (!empty($mainSiteUserAccessData)) {
            $mainSiteUserAccessEntity = $this->getMainSiteUserAccessHydrator()->hydrate($mainSiteUserAccessData, clone $this->mainSiteUserAccessPrototype);
        } else {
            $mainSiteUserAccessEntity = null;
        }
        if (!empty($mainSiteData)) {
            $mainSiteEntity = $this->getMainSiteHydrator()->hydrate($mainSiteData, clone $this->mainSitePrototype);
        } else {
            $mainSiteEntity = null;
        }
        if (!empty($mainUserData)) {
            $mainUserEntity = $this->getMainUserHydrator()->hydrate($mainUserData, clone $this->mainUserPrototype);
        } else {
            $mainUserEntity = null;
        }

        if ($mainSiteUserAccessEntity != null) {
            if ($mainSiteEntity != null) {
                $mainSiteUserAccessEntity->setMainSite($mainSiteEntity);
            }
            if ($mainUserEntity != null) {
                $mainSiteUserAccessEntity->setMainUser($mainUserEntity);
            }
        }

        if (is_array($this->buffer)) {
            $this->buffer[$this->position] = $mainSiteUserAccessEntity;
        }

        return $mainSiteUserAccessEntity;
    }

    public function toArray() {
        $return = array();
        foreach ($this as $row) {
            $return[] = array_merge(
                    $this->getMainSiteUserAccessHydrator()->extract($row), $this->getMainSiteHydrator()->extract($row), $this->getMainUserHydrator()->extract($row)
            );
        }
        return $return;
    }

}
