<?php

namespace Model;

use System\Database\AbstractTable;

class MainBc extends AbstractTable
{
    protected $table = 'main_bc';


    CONST FULL_CREATOR = 'full_creator';


    protected $rules = array(
        'dict_client_id'        => 'client',
        'main_users_id_created' => 'creator'
    );


    public function addCampaign($campaignName, $companyId, $userId)
    {

        try {

            $companyId = (int)$companyId;
            $userId = (int)$userId;

            $res = $this->adapter->createStatement(
                "
				INSERT INTO {$this->table()} ( name , dict_client_id , main_users_id_created ) VALUES
											 ( '{$campaignName}' , {$companyId} , {$userId} )
			"
            )->execute();

            if (1 == $res->getAffectedRows()) {
                ;
            }

            return true;

            return false;

        } catch (\Exception $e) {

            throw new \Exception(__METHOD__ . '. ' . $e->getMessage());
        }
    }


    /**
     * @param null  $campaignId
     * @param array $config
     * @param bool  $count
     * @throws \Exception
     * @return array
     */
    public function getCampaign($campaignId = null, $config = array(), &$count = false)
    {
        if (is_array($campaignId)) {

            $campaignId = implode(',', $campaignId);
            if (preg_match('/[0-9]+((,[0-9])+)?/', $campaignId)) {
                $id = $campaignId;
            } else {
                $id = null;
            }

        } elseif (is_numeric($campaignId)) {
            $id = (int)$campaignId;
        } else {
            $id = null;
        }

        try {
            $additSelect = '';
            $additWhere = '';
            $additFrom = '';
            $order = 'ORDER BY bc.id DESC';
            $limit = '';
            $page = 1;

            $limitValue = 25;

            if (is_array($config)) {
                if (in_array(self::FULL_CREATOR, $config)) {

                    $additSelect .= ' , cu.name as creator_name , cu.user_login as creator_login , cu.lastname as creator_lastname ';
                    $additFrom .= ' , cms.main_users cu ';
                    $additWhere .= ' AND bc.main_users_id_created = cu.user_id ';
                }
            }
            if (array_key_exists('where', $config) &&
                is_array($config['where'])
            ) {

                foreach ($config['where'] as $whereConstraint) {

                    foreach ($whereConstraint as $key => $value) {
                        switch ($key) {
                            case 'content_id':

                                $value = (int)$value;

                                $additWhere .= " AND bc.id = {$value}";

                                break;
                            case 'title':

                                $additWhere .= " AND lower( bc.name ) like '%' || lower( '{$value}' ) || '%'";

                                break;
                            case 'dict_client_id':

                                $value = explode(',', $value);
                                foreach ($value as $k => $v) {

                                    $value[$k] = (int)$v;
                                }

                                $value = implode(',', $value);
                                $additWhere .= " AND bc.dict_client_id in ( {$value} )";

                                break;
                            case 'date_from':

                                $value = strtotime('midnight', (int)$value);
                                $additWhere .= " AND bc.created >= to_timestamp( {$value} ) ";

                                break;
                            case 'date_to':

                                $value = strtotime('23:59:59', (int)$value);
                                $additWhere .= " AND bc.created <= to_timestamp( {$value} ) ";

                                break;
                            case 'creator' :

                                $value = (int)$value;
                                $additWhere .= " AND bc.main_users_id_created = {$value}";

                                break;
                            case 'creator_val' :

                                $value = (int)$value;
                                $additWhere .= " AND bc.main_users_id_created = {$value}";

                                break;
                        }
                    }
                }
            }
            if (array_key_exists('order', $config) &&
                is_array($config['order'])
            ) {

                $field = key($config['order']);
                $type = current($config['order']);

                switch ($field) {
                    case 'client':

                        $order = "ORDER BY c.name {$type}";

                        break;
                    case 'title':

                        $order = "ORDER BY bc.name {$type} ";

                        break;
                    case 'content_id':

                        $order = "ORDER BY bc.id {$type}";

                        break;
                    case 'add_date':

                        $order = "ORDER BY bc.created {$type}";

                        break;
                    case 'creator':

                        if (!in_array(self::FULL_CREATOR, $config)) {

                            $additFrom .= ' , cms.main_users cu ';
                            $additWhere .= ' AND bc.main_users_id_created = cu.user_id ';
                        }

                        $order = "ORDER BY concat( cu.name , ' ' , cu.lastname ) {$type}";

                        break;
                }

            }
            if (array_key_exists('limit', $config)) {

                $limitValue = (int)$config['limit'];

                $limit = "LIMIT {$limitValue}";

            }
            if (array_key_exists('page', $config)) {

                $page = (int)$config['page'];

                $offset = $limitValue * $page - $limitValue;

                $limit .= " OFFSET {$offset}";
            }

            if ($count) {

                $res = $this->get(
                    "
					SELECT bc.* , c.name as client_name {$additSelect} 
					FROM {$this->table()} bc , {$this->table('dict_client')} c {$additFrom}
					WHERE bc.dict_client_id = c.id {$additWhere} " . (null !== $id ? " AND bc.id in( {$id} )" : '') . "
				"
                );

                $count = $res->count();
            }


            $res = $this->get(
                "
				SELECT bc.* , c.name as client_name {$additSelect} 
				FROM {$this->table()} bc , {$this->table('dict_client')} c {$additFrom}
				WHERE bc.dict_client_id = c.id {$additWhere} " . (null !== $id ? " AND bc.id in( {$id} )" : '') . "
				{$order}
				{$limit};
			"
            );


            return $this->translate($res);

        } catch (\Exception $e) {

            throw new \Exception(__METHOD__ . '. ' . $e->getMessage());
        }
    }
}
