<?php

namespace Model;

use System\Database\AbstractTable;

class MainBcElement extends AbstractTable {
	protected $table = 'main_bc_element';
	
	protected $rules = array(
		'dict_client_id' => 'client',
		'main_users_id_created' => 'creator',
		'main_bc_id' => 'bcId'
	);
	
	public function getElementsForCampaign( $bcId , $additionalRestrictions = array() ) {
		
		$additionalColumns = '';
		
		try {
			
			if( !is_array( $bcId ) ) {
				$bc = array( $bcId );
			}
			
			$bcId = implode( ',' , $bcId );
			
			if( !preg_match( '/^[0-9]+((,[0-9]+)+)?$/' , $bcId ) ) {
				
				throw new \Exception( "niepoprawne dane wejściowe" );
			}
			
			$additionalQuery = '';
			
			if( isset( $additionalRestrictions[ 'from_range' ] ) ) {
				$range = $additionalRestrictions[ 'from_range' ];
				if( isset( $range[ 'from' ] ) &&
					isset( $range[ 'to' ] ) ) {
					
					$additionalQuery .= "
						AND ( 
							( 
								e.starts <= timestamp 'epoch' + INTERVAL '1 second' * {$range[ 'from' ]} AND 
								e.ends >= timestamp 'epoch' + INTERVAL '1 second' * {$range[ 'from' ]}
							) OR (
								e.starts >= timestamp 'epoch' + INTERVAL '1 second' * {$range[ 'from' ]} AND
								e.ends <= timestamp 'epoch' + INTERVAL '1 second' * {$range[ 'to' ]}
							) OR (
								e.starts <= timestamp 'epoch' + INTERVAL '1 second' * {$range[ 'to' ]} AND
								e.ends >= timestamp 'epoch' + INTERVAL '1 second' * {$range[ 'to' ]}
							)
						)
					";
				}
			} if( in_array( 'not_disabled' , $additionalRestrictions ) ) {
				
				$additionalQuery .= ' AND e.status < 4';
				
			} if( in_array( 'load_elements_statistics' , $additionalRestrictions ) ) {
				
				$additionalColumns = "( select count( distinct( coalesce( mev.main_user_id , 0 ) ) ) from {$this->table( 'main_bc_element_view' )} mev WHERE mev.main_bc_element_id = e.id ) as unique_views, 
									  ( select count( distinct( coalesce( meh.main_user_id , 0 ) ) ) from {$this->table( 'main_bc_element_hit' )} meh WHERE meh.main_bc_element_id = e.id ) as unique_clicks, ";
			}
			
			
			
			$res = $this->get("
				SELECT {$additionalColumns} e.* , e.id as elementid , starts , ends , e.dict_bc_element_type_id as id , et.name as type , ep.id as place , e.main_users_id_created as creator , mu.name as creator_name , mu.lastname as creator_lastname, ms.active as main_site_active
				FROM {$this->table()} e , {$this->table( 'dict_bc_element_type' )} et , {$this->table( 'dict_bc_element_place' )} ep , cms.main_users mu , pl_content.main_site ms 
				WHERE e.dict_bc_element_type_id = et.id AND e.dict_bc_element_place_id = ep.id AND e.main_users_id_created = mu.user_id AND ep.main_site_id = ms.id AND 
					  e.main_bc_id in ( {$bcId} )
					  {$additionalQuery}
					  ORDER BY status DESC , starts DESC
			");
			
			
			if( $res->count() < 1 ) return array();
			
			
			$result = array();
			$elementsIdList = array();
			
			
			foreach( $res as $key => $row ) {
				
				$elementsIdList[ $row[ 'elementid' ] ] = $row[ 'elementid' ];
				
				$row[ 'starts' ] = strtotime( substr( $row[ 'starts' ] , 0 , 16 ) );
				$row[ 'ends' ] = strtotime( substr( $row[ 'ends' ] , 0 , 16 ) );
				
				$result[] = $row;
			}
			
			
			$RelBcElemenTarget = new RelBcElementTarget;
			$targetBase = $RelBcElemenTarget->getTarget( $elementsIdList );
			$target = array();
			
			
			foreach( $targetBase as $t ) {
				
				if( !isset( $target[ $t[ 'elementid' ] ] ) ) {
					$target[ $t[ 'elementid' ] ] = array();
				}
				
				$code = explode( ',' , $t[ 'code' ] );
				
				foreach( $code as $k => $v ) {
					
					$code[ $k ] = trim( $v );
				}
				
				
				$target[ $t[ 'elementid' ] ][ $t[ 'target_type' ] ] = $code;
			}
			
			foreach( $result as $k => $row ) {
				
				if( !isset( $target[ $row[ 'elementid' ] ] ) ) {
					
					$result[ $k ][ 'target_data' ] = null;
					continue;
				}
				
				$result[ $k ][ 'target_data' ] = $target[ $row[ 'elementid' ] ];
			}
			
			
			return $this->translate( $result );
			
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	
	/**
	 * Wyszukanie elementów z podanego zakresu
	 */
	public function getElementsFromRange( $starts , $ends , $searchTypes = array() , $activeOny = true ) {
		
		
		
		
		try {
			
			foreach( $searchTypes as $key => $val ) {
				
				$searchTypes[ $key ] = (int)$val;
			}
			
			
			$res = $this->get("
				SELECT e.* , e.id as elementid , starts , ends , e.dict_bc_element_type_id as id , et.name as type , ep.id as place , e.main_users_id_created as creator , mu.name as creator_name , mu.lastname as creator_lastname, ms.active as main_site_active
				FROM {$this->table()} e , {$this->table( 'dict_bc_element_type' )} et , {$this->table( 'dict_bc_element_place' )} ep , cms.main_users mu , pl_content.main_site ms
				WHERE e.dict_bc_element_type_id = et.id AND e.dict_bc_element_place_id = ep.id AND e.main_users_id_created = mu.user_id AND ep.main_site_id = ms.id AND 
					  e.starts <= ( SELECT timestamp 'epoch' + INTERVAL '{$starts} second' ) AND 
					  e.ends >= ( SELECT timestamp 'epoch' + INTERVAL '{$ends} second' ) 
					  " . ( true == $activeOny ? ' AND e.status = 1 ' : '' ). "
					  " . ( 0 < count( $searchTypes ) ? " AND e.dict_bc_element_place_id in (" . implode( ',' , $searchTypes ) . ")" : '' ) . "
					  
			");
			
			
			if( $res->count() < 1 ) return array();
			
			
			$res = $this->getTarget( $res );
			
			return $this->translate( $res );
			
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	
	protected function getTarget( $rows ) {
		
		$result = array();
		$elementsIdList = array();
		
		foreach( $rows as $key => $row ) {
			
			$elementsIdList[ $row[ 'elementid' ] ] = $row[ 'elementid' ];
			
			$row[ 'starts' ] = strtotime( substr( $row[ 'starts' ] , 0 , 16 ) );
			$row[ 'ends' ] = strtotime( substr( $row[ 'ends' ] , 0 , 16 ) );
			
			$result[] = $row;
		}
		
		
		$RelBcElemenTarget = new RelBcElementTarget;
		$targetBase = $RelBcElemenTarget->getTarget( $elementsIdList );
		$target = array();
		
		
		foreach( $targetBase as $t ) {
			
			if( !isset( $target[ $t[ 'elementid' ] ] ) ) {
				$target[ $t[ 'elementid' ] ] = array();
			}
			
			$code = explode( ',' , $t[ 'code' ] );
			
			foreach( $code as $k => $v ) {
			
				$code[ $k ] = trim( $v );
			}
			
				
			$target[ $t[ 'elementid' ] ][ $t[ 'target_type' ] ] = $code;
		}
		
		foreach( $result as $k => $row ) {
			
			if( !isset( $target[ $row[ 'elementid' ] ] ) ) {
				
				$result[ $k ][ 'target_data' ] = null;
				continue;
			}
				
			$result[ $k ][ 'target_data' ] = $target[ $row[ 'elementid' ] ];
		}
		
		return $result;
	}
	
	public function addElement( $bcId , $elementTypeId , $elementPlaceId , $creatorId , $starts , $ends , $state , $additionalStats , $data ) {
		
		try {
			
			$state = (int)$state;
			if( $state < 0 || $state > 3) {
				$state = 3; // dezaktywowany
			}
			
			$res = $this->adapter->createStatement("
				INSERT INTO {$this->table()} ( main_bc_id , main_users_id_created , starts , ends , dict_bc_element_type_id , dict_bc_element_place_id , status , additional_stats , data )
									  VALUES ( {$bcId} , {$creatorId} , to_timestamp({$starts}) , to_timestamp({$ends}) , {$elementTypeId} , {$elementPlaceId} , {$state} , '{$additionalStats}' , '{$data}' )
 			")->execute();
			
			
			if( $res->getAffectedRows() !== 1 )
				return false;
			
			$res = $this->get("select lastval() as id");
			$res = $res->current();
			
			return $res[ 'id' ];
			
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. Nieudało siż zappisać nowego elementu kampanii ' . var_export( func_get_args() ) . '. ' . $e->getMessage() );
		}
	}
	
	
	public function getElement( $mainBcElementId , $cfg ) {
		
		try {
			
			if( is_array( $mainBcElementId ) ) {
				
				$mainBcElementId = implode( ',' , $mainBcElementId );
			} else {
				
				$mainBcElementId = (int)$mainBcElementId;
			}
			
			$res = $this->get("
				SELECT e.* , e.id as elementid , starts , ends , e.dict_bc_element_type_id as id , e.id as elementId , et.name as type , ep.id as place , e.main_users_id_created as creator , mu.name as creator_name , mu.lastname as creator_lastname, ms.active as main_site_active
				FROM {$this->table()} e , {$this->table( 'dict_bc_element_type' )} et , {$this->table( 'dict_bc_element_place' )} ep , cms.main_users mu , pl_content.main_site ms
				WHERE e.dict_bc_element_type_id = et.id AND e.dict_bc_element_place_id = ep.id AND e.main_users_id_created = mu.user_id AND ep.main_site_id = ms.id AND 
					  e.id in( {$mainBcElementId} )
			");
			
			if( $res->count() < 1 ) return false;
			
			$result = array();
			$elementsIdList = array();
			
			
			foreach( $res as $key => $row ) {
				
				$elementsIdList[ $row[ 'elementid' ] ] = $row[ 'elementid' ];
				
				$row[ 'starts' ] = strtotime( substr( $row[ 'starts' ] , 0 , 16 ) );
				$row[ 'ends' ] = strtotime( substr( $row[ 'ends' ] , 0 , 16 ) );
				
				$result[] = $row;
			}
			
			
			if( empty( $cfg[ 'NO_TARGET' ] ) ) {
				
				$RelBcElemenTarget = new RelBcElementTarget;
				$targetBase = $RelBcElemenTarget->getTarget( $elementsIdList );
				$target = array();
				
				
				foreach( $targetBase as $t ) {
					
					if( !isset( $target[ $t[ 'elementid' ] ] ) ) {
						$target[ $t[ 'elementid' ] ] = array();
					}
					
					$code = explode( ',' , $t[ 'code' ] );
					
					foreach( $code as $k => $v ) {
						
						$code[ $k ] = trim( $v );
					}
					
					
					$target[ $t[ 'elementid' ] ][ $t[ 'target_type' ] ] = $code;
				}
				
				foreach( $result as $k => $row ) {
					
					if( !isset( $target[ $row[ 'elementid' ] ] ) ) {
						
						$result[ $k ][ 'target_data' ] = null;
						continue;
					}
					
					$result[ $k ][ 'target_data' ] = $target[ $row[ 'elementid' ] ];
				}
			}
			
			if( !is_int( $mainBcElementId ) ) {
				
				return $this->translate( $result );
			}
			
			return $this->translate( current( $result ) );
			
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	
	public function incrementHits( $bcElementId ) {
		
		try {
				
			$res = $this->adapter->createStatement("
				UPDATE {$this->table()} SET clicks = clicks + 1 WHERE id = {$bcElementId}
			")->execute();
			
			return 1 == $res->getAffectedRows() ? true : false;
			
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	public function incrementView( $mainBcElementId ) {
		
		try {
			
			
			if( is_array( $mainBcElementId ) ) {
				
				$rows = count( $mainBcElementId );
				$mainBcElementId = implode( ',' , $mainBcElementId );
			
			} else {
			
				$rows = 1;
				$mainBcElementId = (int)$mainBcElementId;
			}
			
			
			
			$res = $this->adapter->createStatement("
				UPDATE {$this->table()} SET views = views + 1 WHERE id in( {$mainBcElementId} )
			")->execute();
			
			if( $rows == $res->getAffectedRows() )
				return true;
			return false;
			
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	
	public function updateElement( $bcElementId , $dictElementTypeId , $dictElementPLaceId , $starts , $ends , $status , $additionalStatistics , $data ) {
		
		try {
			
			$dictElementTypeId = (int)$dictElementTypeId;
			$dictElementPLaceId = (int)$dictElementPLaceId;
			$starts = (int)$starts;
			$ends = (int)$ends;
			$status = (int)$status;
			
			$res = $this->adapter->createStatement("
				UPDATE {$this->table()} SET dict_bc_element_place_id = {$dictElementPLaceId} , dict_bc_element_type_id = {$dictElementTypeId} , starts = to_timestamp( {$starts} ),
					   ends = to_timestamp( {$ends} ) , status = {$status} , data = '{$data}' , additional_stats = '{$additionalStatistics}'
				WHERE id = {$bcElementId}
			")->execute();
			
			if( 1 == $res->getAffectedRows() )
				return true;
			return false;
		
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
        
        public function getElementsForPlace($placeID) {
            $res = $this->adapter->createStatement("SELECT id FROM {$this->table()} WHERE dict_bc_element_place_id = {$placeID}")->execute();
            return $this->translate($res);
        }
}

?>