<?php
namespace Model;

use Model\MainContentHelper as MainContentHelperVideo;

class MainContentVideo extends MainContentHelperVideo
{

    protected $table = 'main_content_video';

    protected function query($specTypes, $additionalRestrictions = array())
    {

        if (!is_string($specTypes) || !preg_match('/^([,]?\'[A-Za-z0-9\.]+\')+$/', $specTypes)) {
            $specTypes = '';
        }

        if (isset($additionalRestrictions['ID_LIST'])) {
            $additionalRestrictions['ID_LIST'] = implode(',', $additionalRestrictions['ID_LIST']);
        }

        if (isset($additionalRestrictions['CONTENT_ID'])) {

            return (int)$additionalRestrictions['CONTENT_ID'];
        } else {

            if (isset($additionalRestrictions['COLLECTIONS'])) {
                $collections = array();

                foreach ($additionalRestrictions['COLLECTIONS'] as $collectionId) {

                    if (0 < (int)$collectionId) {

                        $collections[] = (int)$collectionId;
                    }
                }

                if (!count($collections)) {

                    unset($collections);
                } else {

                    $collections = implode(',', $collections);
                }
            }

            $query = "
				SELECT distinct( mcv.content_id )
				FROM {$this->table()} mcv
				LEFT JOIN {$this->table('main_content')} AS mc ON mc.content_id = mcv.content_id and mc.lang_id = mcv.lang_id
				LEFT JOIN {$this->table('rel_content_spec')} AS rcs ON mcv.content_id = rcs.content_id and mcv.lang_id = rcs.lang_id
				WHERE mcv.content_id > 0
				" . (!isset($additionalRestrictions['ALL']) ? ' AND mc.visible_in_esculap = true ' : '') . "
				" . (!isset($additionalRestrictions['ID_LIST']) && strlen($specTypes) ? "AND rcs.spec_code in({$specTypes})" : '') . "
				" . (isset($additionalRestrictions['ID_LIST']) ? "and mcv.content_id in ({$additionalRestrictions['ID_LIST']})" : '') . "
			";
        }

        return $query;
    }

    public function getElements($specTypes, $start, $offset, $additionalRestrictions = array())
    {
        $query = $this->query($specTypes, $additionalRestrictions);
        $res = $this->queryHelper($start, $offset, $query, $additionalRestrictions);

        return $this->fetchAll($res);
    }

    public function getElementsEx($specTypes, $limit, $offset, $additionalRestrictions = array(), $count = false)
    {
        $order = '';

        if (isset($additionalRestrictions['ORDER_SPECS']) &&
            !isset($additionalRestrictions['COLLECTIONS'])
        ) {

            $specsStr = '';
            $order = 'match_level ASC, ';
            $other = false;
            $clearCase = true; // szyści case
            $columns = "mc.content_id , min( CASE";


            ksort($additionalRestrictions['ORDER_SPECS']);


            foreach ($additionalRestrictions['ORDER_SPECS'] as $priority => $specs) {

                foreach ($specs as $spec) {
                    if (false !== $other) {
                        $other = true;
                    }

                    $clearCase = false;

                    $columns .= " WHEN rcs.spec_code = '{$spec}' THEN {$priority}";

                    if (strlen($specsStr)) {
                        $specsStr .= ',';
                    }

                    $specsStr .= "'{$spec}'";
                }
            }

            if ($clearCase) {

                $order = '';
                unset($specsStr);
                $columns = "mc.content_id";
            } else {

                if ($other) {
                    $columns .= " ELSE 10";
                }

                $columns .= " END ) as match_level";
            }
        } else {

            $columns = 'mc.content_id';
        }


        if (isset($additionalRestrictions['COLLECTIONS'])) {
            $collections = array();

            foreach ($additionalRestrictions['COLLECTIONS'] as $collectionId) {

                if (0 < (int)$collectionId) {

                    $collections[] = (int)$collectionId;
                }
            }

            if (!count($collections)) {

                unset($collections);
            } else {

                $collections = implode(',', $collections);
            }
        }

        try {
            if (isset($additionalRestrictions['ACTIVE']) && $additionalRestrictions['ACTIVE'] === false) {
                $additionalRestrictions['ALL'] = true;
            }

            $albumId = 0;
            $ordinalNumber = 0;

            $contentList = [];
            if (isset($additionalRestrictions['SUGGESTED_VIDEOS']) && $additionalRestrictions['SUGGESTED_VIDEOS'] != 0) {
                $select = "SELECT dict_album_id, ordinal_number FROM pl_content.rel_content_album WHERE main_content_id = {$additionalRestrictions['SUGGESTED_VIDEOS']} LIMIT 1";
                $res = $this->adapter->createStatement($select)->execute();

                foreach ($res as $row) {
                    $albumId = (integer)$row['dict_album_id'];
                    $ordinalNumber = (integer)$row['ordinal_number'];
                }
            }

            $prevLimit = round($limit / 2);
            $nextLimit = round($limit / 2);
            $amount = 0;
            if ($albumId != 0 && $ordinalNumber != 0) {
                $selectPrev = "SELECT {$columns} FROM {$this->table()} mcv, {$this->table('main_content')} mc LEFT JOIN {$this->table('rel_content_spec')} AS rcs ON mc.content_id = rcs.content_id AND mc.lang_id = rcs.lang_id
            " . (isset($collections) ? "LEFT JOIN {$this->table('rel_content_collection')} AS rcc ON rcc.main_content_id = mc.content_id AND rcc.lang_id = mc.lang_id" : '') .
                    (($albumId != 0) ? "LEFT JOIN {$this->table('rel_content_album')} AS rca ON rca.main_content_id = mc.content_id AND rca.lang_id = mc.lang_id" : '') . "
                WHERE
                mc.content_id = mcv.content_id AND mc.lang_id = mcv.lang_id
                " . (!isset($additionalRestrictions['ALL']) ? ' AND mc.visible_in_esculap = true ' : '') . "
                " . ($albumId != 0 ? ' AND rca.dict_album_id = ' . $albumId : '') . "
                " . ($ordinalNumber != 0 ? ' AND rca.ordinal_number < ' . $ordinalNumber . ' AND rca.ordinal_number > 0 ' : '') . "
                " . (isset($additionalRestrictions['CONTENT_ID']) ? "AND mc.content_id = {$additionalRestrictions['CONTENT_ID']}" : '') . "
                " . (isset($additionalRestrictions['NOT_CONTENT_ID']) ? "AND mc.content_id not in ( " . $additionalRestrictions['NOT_CONTENT_ID'] . " ) " : '') . "
                " . (isset($additionalRestrictions['MAIN_SITE_ID']) && $additionalRestrictions['MAIN_SITE_ID'] ? "AND mc.main_site_id = {$additionalRestrictions['MAIN_SITE_ID']}" : '') . "
                " . (isset($collections) ? "AND rcc.main_collection_id in ( {$collections} ) " : '') . "
            GROUP BY rca.ordinal_number, mc.content_id
            ORDER BY rca.ordinal_number DESC
            " . (false === $count ? " LIMIT {$prevLimit} OFFSET {$offset}" : '');
                $prevRes = $this->adapter->createStatement($selectPrev)->execute();

                if ($prevRes->count() < $prevLimit) {
                    $nextLimit = $nextLimit + ($prevLimit - $prevRes->count());
                }

                $selectNext = "SELECT {$columns} FROM {$this->table()} mcv, {$this->table('main_content')} mc LEFT JOIN {$this->table('rel_content_spec')} AS rcs ON mc.content_id = rcs.content_id AND mc.lang_id = rcs.lang_id
            " . (isset($collections) ? "LEFT JOIN {$this->table('rel_content_collection')} AS rcc ON rcc.main_content_id = mc.content_id AND rcc.lang_id = mc.lang_id" : '') .
                    (($albumId != 0) ? "LEFT JOIN {$this->table('rel_content_album')} AS rca ON rca.main_content_id = mc.content_id AND rca.lang_id = mc.lang_id" : '') . "
                WHERE
                mc.content_id = mcv.content_id AND mc.lang_id = mcv.lang_id
                " . (!isset($additionalRestrictions['ALL']) ? ' AND mc.visible_in_esculap = true ' : '') . "
                " . ($albumId != 0 ? ' AND rca.dict_album_id = ' . $albumId : '') . "
                " . ($ordinalNumber != 0 ? ' AND rca.ordinal_number > ' . $ordinalNumber : '') . "
                " . (isset($additionalRestrictions['CONTENT_ID']) ? "AND mc.content_id = {$additionalRestrictions['CONTENT_ID']}" : '') . "
                " . (isset($additionalRestrictions['NOT_CONTENT_ID']) ? "AND mc.content_id not in ( " . $additionalRestrictions['NOT_CONTENT_ID'] . " ) " : '') . "
                " . (isset($additionalRestrictions['MAIN_SITE_ID']) && $additionalRestrictions['MAIN_SITE_ID'] ? "AND mc.main_site_id = {$additionalRestrictions['MAIN_SITE_ID']}" : '') . "
                " . (isset($collections) ? "AND rcc.main_collection_id in ( {$collections} ) " : '') . "
            GROUP BY rca.ordinal_number, mc.content_id
            ORDER BY rca.ordinal_number ASC
            " . (false === $count ? " LIMIT {$nextLimit} OFFSET {$offset}" : '');

                $nextRes = $this->adapter->createStatement($selectNext)->execute();

                if (($prevRes->count() + $nextRes->count()) < $limit) {
                    $prevLimit = $prevLimit + ($nextLimit - $nextRes->count());
                    $selectPrev = "SELECT {$columns} FROM {$this->table()} mcv, {$this->table('main_content')} mc LEFT JOIN {$this->table('rel_content_spec')} AS rcs ON mc.content_id = rcs.content_id AND mc.lang_id = rcs.lang_id
            " . (isset($collections) ? "LEFT JOIN {$this->table('rel_content_collection')} AS rcc ON rcc.main_content_id = mc.content_id AND rcc.lang_id = mc.lang_id" : '') .
                        (($albumId != 0) ? "LEFT JOIN {$this->table('rel_content_album')} AS rca ON rca.main_content_id = mc.content_id AND rca.lang_id = mc.lang_id" : '') . "
                WHERE
                mc.content_id = mcv.content_id AND mc.lang_id = mcv.lang_id
                " . (!isset($additionalRestrictions['ALL']) ? ' AND mc.visible_in_esculap = true ' : '') . "
                " . ($albumId != 0 ? ' AND rca.dict_album_id = ' . $albumId : '') . "
                " . ($ordinalNumber != 0 ? ' AND rca.ordinal_number < ' . $ordinalNumber . ' AND rca.ordinal_number > 0 ' : '') . "
                " . (isset($additionalRestrictions['CONTENT_ID']) ? "AND mc.content_id = {$additionalRestrictions['CONTENT_ID']}" : '') . "
                " . (isset($additionalRestrictions['NOT_CONTENT_ID']) ? "AND mc.content_id not in ( " . $additionalRestrictions['NOT_CONTENT_ID'] . " ) " : '') . "
                " . (isset($additionalRestrictions['MAIN_SITE_ID']) && $additionalRestrictions['MAIN_SITE_ID'] ? "AND mc.main_site_id = {$additionalRestrictions['MAIN_SITE_ID']}" : '') . "
                " . (isset($collections) ? "AND rcc.main_collection_id in ( {$collections} ) " : '') . "
            GROUP BY rca.ordinal_number, mc.content_id
            ORDER BY rca.ordinal_number DESC
            " . (false === $count ? " LIMIT {$prevLimit} OFFSET {$offset}" : '');
                    $prevRes = $this->adapter->createStatement($selectPrev)->execute();
                }

                $amount = $prevRes->count() + $nextRes->count();


                $prevContent = [];
                foreach ($prevRes as $row) {
                    $prevContent[] = $row['content_id'];
                }
                $prevContent = array_reverse($prevContent);
                foreach ($prevContent as $row) {
                    $contentList[$row] = $row;
                }

                foreach ($nextRes as $row) {
                    $contentList[$row['content_id']] = $row['content_id'];
                }

                foreach ($contentList as $c) {
                    $additionalRestrictions['NOT_CONTENT_ID'] .= ','.$c;
                }
            }

            $limit = (integer) $limit;
            if ($amount < $limit || $limit == 0) {
                $select = "
			SELECT {$columns}
			FROM {$this->table()} mcv, {$this->table('main_content')} mc
			LEFT JOIN {$this->table('rel_content_spec')} AS rcs ON mc.content_id = rcs.content_id AND mc.lang_id = rcs.lang_id
			" . (
                    isset($collections) ? "LEFT JOIN {$this->table('rel_content_collection')} AS rcc ON rcc.main_content_id = mc.content_id AND rcc.lang_id = mc.lang_id" : ''
                    ) . (
                    ($albumId != 0) ? "LEFT JOIN {$this->table('rel_content_album')} AS rca ON rca.main_content_id = mc.content_id AND rca.lang_id = mc.lang_id" : ''
                    ) . "
			WHERE
				mc.content_id = mcv.content_id AND mc.lang_id = mcv.lang_id
				" . (!isset($additionalRestrictions['ALL']) ? ' AND mc.visible_in_esculap = true ' : '') . "
				" . ($albumId != 0 ? ' AND rca.dict_album_id = ' . $albumId : '') . "
				" . (isset($additionalRestrictions['CONTENT_ID']) ? "AND mc.content_id = {$additionalRestrictions['CONTENT_ID']}" : '') . "
				" . (isset($additionalRestrictions['NOT_CONTENT_ID']) ? "AND mc.content_id not in ( " . $additionalRestrictions['NOT_CONTENT_ID'] . " ) " : '') . "
				" . (isset($additionalRestrictions['MAIN_SITE_ID']) && $additionalRestrictions['MAIN_SITE_ID'] ? "AND mc.main_site_id = {$additionalRestrictions['MAIN_SITE_ID']}" : '') . "
				" . (isset($collections) ? "AND rcc.main_collection_id in ( {$collections} ) " : '') . "
			GROUP BY( mc.content_id )
			ORDER BY {$order} mc.content_id DESC
			" . (false === $count ?
                        " LIMIT {$limit}
					  OFFSET {$offset}" : '');

                $res = $this->adapter->createStatement($select)->execute();
                foreach ($res as $row) {
                    $contentList[$row['content_id']] = $row['content_id'];
                }
                foreach ($contentList as $c) {
                    $additionalRestrictions['NOT_CONTENT_ID'] .= ','.$c;
                }

                if ($albumId != 0 && $res->count() < $limit) {
                    $limit2 = $limit - $res->count();
                    $select = "SELECT {$columns} FROM {$this->table()} mcv, {$this->table('main_content')} mc
                           LEFT JOIN {$this->table('rel_content_spec')} AS rcs ON mc.content_id = rcs.content_id AND mc.lang_id = rcs.lang_id
                           " . (
                        isset($collections) ? "LEFT JOIN {$this->table('rel_content_collection')} AS rcc ON rcc.main_content_id = mc.content_id AND rcc.lang_id = mc.lang_id" : ''
                        ) . " WHERE
				mc.content_id = mcv.content_id AND mc.lang_id = mcv.lang_id
				" . (!isset($additionalRestrictions['ALL']) ? ' AND mc.visible_in_esculap = true ' : '') . "
				" . (isset($additionalRestrictions['CONTENT_ID']) ? "AND mc.content_id = {$additionalRestrictions['CONTENT_ID']}" : '') . "
				" . (isset($additionalRestrictions['NOT_CONTENT_ID']) ? "AND mc.content_id not in ( " . $additionalRestrictions['NOT_CONTENT_ID'] . " ) " : '') . "
				" . (isset($additionalRestrictions['MAIN_SITE_ID']) && $additionalRestrictions['MAIN_SITE_ID'] ? "AND mc.main_site_id = {$additionalRestrictions['MAIN_SITE_ID']}" : '') . "
				" . (isset($collections) ? "AND rcc.main_collection_id in ( {$collections} ) " : '') . "
			GROUP BY( mc.content_id )
			ORDER BY {$order} mc.content_id DESC
			" . (false === $count ?
                            " LIMIT {$limit2}
					  OFFSET {$offset}" : '');

                    $res2 = $this->adapter->createStatement($select)->execute();

                    $amount = $amount + $res->count() + $res2->count();
                } else {
                    $amount = $amount + $res->count();
                }
            }

            if ($count) {
                return $amount;
            }

            // jeśli nic nie znalazłem nie póbuje pobierać danych
            if (!$amount) {
                return array();
            }

            if (isset($res2)) {
                foreach ($res2 as $row) {
                    $contentList[$row['content_id']] = $row['content_id'];
                }
            }

            $idListSearch = implode(',', $contentList);
            $select = "
				SELECT 
					* ,
					coalesce( ( SELECT ROUND( AVG( evaluation ) , 1 ) FROM {$this->table('main_content_evaluation')} mce where mce.content_id = mc.content_id AND mce.lang_id = mc.lang_id ) , 0 ) as evaluation ,
					coalesce( ( SELECT count( 1 ) FROM {$this->table('main_content_evaluation')} mce WHERE mce.content_id = mc.content_id AND mce.lang_id = mc.lang_id ) , 0 ) as evaluated
				FROM {$this->table()} mcn, {$this->table('main_content')} mc
				WHERE mc.content_id = mcn.content_id AND mc.lang_id = mcn.lang_id AND mc.content_id in ( {$idListSearch} )
				ORDER BY array_index(array[{$idListSearch}], mc.content_id) --array_index to procedura
			";

            $res = $this->adapter->createStatement($select)->execute();

            foreach ($res as $row) {
                $contentList[$row['content_id']] = $row;
            }

            return $contentList;
        } catch (\Exception $e) {
            exit($e->getMessage());

            return array();
        }
    }

    public function countEx($specTypes, $additionalRestrictions = array())
    {
        return $this->getElementsEx($specTypes, null, null, $additionalRestrictions, true);
    }

    public function count($specTypes, $additionalRestrictions = array())
    {
        $query = $this->query($specTypes, $additionalRestrictions);
        $res = $this->queryHelper(0, 1, $query, $additionalRestrictions, true);
        $res = $res->current();

        return $res['total'];
    }

    public function addVideo($contentId, $langId, $title, $head, $body, $author, $copyrights, $date, $mainStreamID)
    {
        try {
            $res = $this->adapter->createStatement(
                "
                INSERT INTO {$this->table()} ( content_id , lang_id , title , head, body,  author , copyrights , date , main_stream_id ) VALUES
                ( {$contentId} , '{$langId}' , '{$title}' , '{$head}', '{$body}', '{$author}' , '{$copyrights}' , '{$date}', {$mainStreamID} )
            "
            )->execute();

            if ($res->getAffectedRows() == 1) {
                return true;
            }

            return false;
        } catch (\Exception $e) {
            throw new \Exception(__METHOD__ . 'Nie udało sie dodać video. ' . $e->getMessage());
        }
    }

    public function updateVideo($contentId, $langId, $title, $head, $body, $author, $copyrights, $date)
    {
        try {
            $res = $this->adapter->createStatement(
                "
                UPDATE {$this->table()} SET title = '{$title}' , author = '{$author}' , 
                copyrights = '{$copyrights}' , date = '{$date}' , body = '{$body}', head = '{$head}'
                WHERE content_id = {$contentId} AND lang_id = '{$langId}'
            "
            )->execute();

            if ($res->getAffectedRows() == 1) {
                return true;
            }

            return false;
        } catch (\Exception $e) {

            throw new \Exception(__METHOD__ . 'Nie udało sie nadpisać video. ' . $e->getMessage());
        }
    }

}

?>