<?php

namespace Model;
use System\Database\AbstractTable;

class MainPhrase extends AbstractTable {
	protected $table = 'main_phrase';
	
	public function getAll( $language , $system = null ) {
		if(is_string($language)) {
			$language = strtoupper($language);
		}
		
		if(null !== $system && !is_array( $system) && strlen($system) !== 1) {
			$system = null;
		} elseif( null !== $system )  {
			if( is_array( $system ) ) {
				$system = implode( ',' , $system );
			}
			
			$system = preg_replace( '/([a-z])/' , '\'$1\'' , $system );
		}
		
		$res = $this->adapter->createStatement("SELECT mp.text_id,mpv.value
												FROM main_phrase mp, main_phrase_value mpv
												WHERE mpv.lang_id = '$language' AND mp.phrase_id = mpv.phrase_id
												".(null !== $system ? "AND mp.system in ({$system}) " : '')."
											   ");
		
		$res = $res->execute();
		
		$result = array();
		
		foreach($res as $row) {
			$result[$row['text_id']] = $row['value'];
		}
		
		return $result;
	}
	
	public function getAllEx($language,$system = null) {
		if(is_string($language)) {
			$language = strtoupper($language);
		}
		
		if( is_array( $system ) ) {
			$system = implode( ',' , $system );
			
		} elseif(null !== $system && strlen($system) !== 1) {
			$system = null;
		}
		
		$system = strtolower($system);
		$system = preg_replace( '/([a-z])/' , '\'$1\'' , $system );
		
		$res = $this->adapter->createStatement("
			SELECT mp.*,mpv.*
			FROM main_phrase mp, main_phrase_value mpv
			WHERE mpv.lang_id = '$language' AND mp.phrase_id = mpv.phrase_id
			".(null !== $system ? "AND mp.system in ({$system})" : '')
		)->execute();
		
		
		$result = array();
		
		foreach($res as $row) {
			$result[] = $row;
		}
		
		return $result;
	}
	
	public function getRowsForQuery($where = array(),$order = array(),$limit = 1,$page = 0) {
		
		try {
			
			$offset = $limit * $page;
			if($offset > 0) $offset--; 
			
			$d = $this->parseQuery($where,$order);
			$_where = $d['where'];
			$_order = $d['order'];
			
			$res = $this->adapter->createStatement("
				SELECT mp.*,mpv.*
				FROM main_phrase mp, main_phrase_value mpv
				WHERE ( mp.phrase_id = mpv.phrase_id OR mpv.phrase_id is null ) AND 
				lower ( mpv.lang_id ) in ( 'ru' , 'en' , 'pl' )
				$_where
				".(null !== $_order ? 'ORDER BY ' . $_order : '' )."
				LIMIT $limit OFFSET $offset
			")->execute();
			
			$result = array();
			
			foreach($res as $row) {
				
				$result[] = $row;
			}
			
			return $result;
			
		} catch(\Exception $e) {
			throw new \Exception(__METHOD__.'. Błąd sql.'.pg_last_error().'. '.$e->getMessage());
		}
	}
	
	public function count($where = array(),$order = array()) {
		
		try {
			
			$d = $this->parseQuery($where);
			$_where = $d['where'];
			
			$res = $this->adapter->createStatement("
				SELECT count(1) as total
				FROM main_phrase mp, main_phrase_value mpv
				WHERE ( mp.phrase_id = mpv.phrase_id OR mpv.phrase_id is null )
				$_where
			")->execute();
			
			$result = 0;
			
			$res = $res->current();
			$result = $res['total'];
			
			return $result;
			
		} catch(\Exception $e) {
			throw new \Exception(__METHOD__.'. Błąd sql.'.pg_last_error().'. '.$e->getMessage());
		}
	}
	
	protected function parseQuery($where = array(),$order = array()) {
		
			$_where = '';
			$_order = null;
			
			foreach($where as $col => $cond) {
				
				if($col == 'lang') {
					if(is_array($cond)) $cond = implode(',',$cond);
					$cond = preg_replace('/([a-zA-Z]+)/',"'$1'",$cond);
					
					$_where .= " AND lower(mpv.lang_id) in ($cond)";
				} elseif($col == 'phrase') {
					$_where .= " AND mp.text_id like '%$cond%'";
				} elseif($col == 'namespace') {
					$_where .= " AND lower(mp.namespace) like '%$cond%'";
				} elseif($col == 'value') {
					$_where .= " AND lower(mpv.value) like '%'||lower('{$cond}')||'%'";
				} elseif($col == 'lang_not_in') {
					$cond = strtolower(implode(',',$cond));
					$cond = preg_replace('/([a-zA-Z]+)/',"'$1'",$cond);
					
					$_where .= "AND lower(mpv.lang_id) not in({$cond})";
				} elseif($col == 'system_in') {
					$cond = strtolower(implode(',',$cond));
					$cond = preg_replace('/([a-zA-Z]+)/',"'$1'",$cond);
					
					$_where .= " AND lower(mp.system) in({$cond})";
				} elseif($col == 'not_lock') {
					$_where .= " AND mp.locked = false";
				} elseif($col == 'system') {
					
					$cond = strtolower($cond);
					$cond = preg_replace('/([a-zA-Z]+)/',"'$1'",$cond);
					
					$_where .= " AND mp.system in($cond)";
					
				}
			}
			
			foreach($order as $col => $cond) {
				if(strlen($_order)) $_order .= ',';
				
				if($col == 'lang') {
					$_order .= "mpv.lang_id $cond";
				} elseif($col == 'add_date') {
					$_order .= "mp.add_date $cond";
				} elseif($col == 'phrase') {
					$_order .= "mp.text_id $cond";
				} elseif($col == 'value') {
					$_order .= "mpv.value $cond";
				} elseif($col == 'namespace') {
					$_order .= "mp.namespace $cond";
				} elseif($col == 'system') {
					$_order .= "mp.system $cond";
				}
			}
		
		
		return array(
			'where' => $_where,
			'order' => $_order
		);
	}
	
	public function getRow($phraseId) {
		
		try { 
			
			$res = $this->adapter->createStatement("
				SELECT * FROM {$this->table}, main_phrase_value
				WHERE {$this->table}.phrase_id = main_phrase_value.phrase_id AND {$this->table}.phrase_id = ".((int)$phraseId)."
			")->execute();
			
			$result = array();
			
			foreach($res as $row) {
				$result[] = $row;
			}
			
			return $result;
			
		} catch(\Exception $e) {
			throw new \Exception(__METHOD__.'. Błąd odczytu frazy "'.$phraseId.'"');
		}
	}
	
	public function updatePhrase($phraseId,$system = null,$namespace = null) {
		
		try {
			if(null !== $system) {
				$res = $this->adapter->createStatement("
					UPDATE {$this->table} SET 
						system = '{$system}'
					WHERE phrase_id = {$phraseId}
				")->execute();
			}
			
			if(null !== $namespace) {
				$res = $this->adapter->createStatement("
					UPDATE {$this->table} SET 
						namespace = '{$namespace}'
					WHERE phrase_id = {$phraseId}
				")->execute();
			}
			
			return true;
			
		} catch(\Exception $e) {
			throw new \Exception(__METHOD__.'. Błąd przy updacie');
		}
	}
	
	public function insertPhrase($textId,$system,$namespace = null) {
		
		try {
			
			$res = $this->adapter->createStatement("
				INSERT INTO {$this->table} (text_id,system,namespace) VALUES 
				('{$textId}',".(null == $system || !strlen($system) ? "NULL" : "'{$system}'").",".(null == $namespace || !strlen($namespace) ? "NULL" : "'{$namespace}'").")
			")->execute();
			
			$res = $this->adapter->createStatement("SELECT LASTVAL()")->execute();
			$res = $res->current();
			
			if((int)$res['lastval'] > 0)
				return $res['lastval'];
			return null;
			
		} catch(\Exception $e) {
			throw new \Exception(__METHOD__.'. Błąd przy updacie');
		}
	}
}

?>

