<?php

namespace Model;


class MainUsersSync extends \System\Database\AbstractTable {

	protected $table = 'main_users_sync';
	
	protected $_columnsToSerch = array('');
	
	public function getNewRows($where = array(),$order = array(),$limit = 20, $page = 0) {
		
		$offset = $limit * $page;
		if($offset > 0) $offset--; 
		
		$restrictions = $this->prepearQuery($where,$order);
		$_order = $restrictions['order'];
		$_where = $restrictions['where'];
		
		$res = $this->adapter->createStatement(
			"SELECT mu.*,mus.* FROM main_users_sync mus ".
			"INNER JOIN main_users mu ON mus.user_login = mu.user_login ".
			// użytkownik musi być zweryfikowany
			"WHERE mu.real_status = 'ACTIVE' ".
			// użytkownik nie może być wyłączony z synchronizacji
			"AND mu.sync=true ". 
				// biorę pod uwagę tylko "nowych" użytkowników
				//"AND mus.record_status = 'N' ".
			// nowe kryterium mapowania
			" AND mus.trust_level in(1,2) ".
			// dodatkowy where 
			(!is_null($_where) ? "AND $_where " : "").
			// order
			(!is_null($_order) ? "ORDER BY $_order " : "").
			// limit
			"LIMIT $limit OFFSET $offset"
		);
		
		$res = $res->execute();
		$result = array();
		
		foreach($res as $row) {
			$result[] = $row;
		}
			
		return $result;
	}
	
	public function countNewRows($where = array()) {
		
		$restrictions = $this->prepearQuery($where);
		$_where = $restrictions['where'];
		
		$res = $this->adapter->createStatement(
			"SELECT count(*) FROM main_users_sync mus ".
			"INNER JOIN main_users mu ON mus.user_login = mu.user_login ".
			// użytkownik musi być zweryfikowany
			"WHERE mu.real_status = 'ACTIVE' ".
			// użytkownik nie może być wyłączony z synchronizacji
			"AND mu.sync=true ". 
			// nowe kryterium mapowania
			"AND mus.trust_level in(1,2) ".
				// biorę pod uwagę tylko "nowych" użytkowników
				//"AND mus.record_status = 'N' ".
			// dodatkowy where z filtra
			(!is_null($_where) ? "AND $_where " : "")
		);
		
		$res = $res->execute();
		$res = $res->current();
		return $res['count'];
	}
	
	/*
	"ON mu.user_login = mus.user_login WHERE (mu.real_status = 'ACTIVE' OR (mu.real_status <> 'NEW' AND mus.record_status = 'S')) ".
    "AND mu.sync=true AND ((mus.mod_date > '$last_mod_date') OR (mus.record_status = 'N'))";
	*/
	public function getChangedRows($where = array(),$order = array(),$limit = 20, $page = 0) {
		
		$offset = $limit * $page;
		if($offset > 0) $offset--; 
		
		$restrictions = $this->prepearQuery($where,$order);
		$_order = $restrictions['order'];
		$_where = $restrictions['where'];
		
		$res = $this->adapter->createStatement(
			"SELECT mu.*,mus.* FROM main_users_sync mus ".
			"INNER JOIN main_users mu ON mus.user_login = mu.user_login ".
			// użytkownik musi być zweryfikowany
			"WHERE (mu.real_status = 'ACTIVE' ".
				// lub ma status inny niż aktywny i nowy ale był już synchronizowany
				"OR (mu.status <> 'NEW' AND mus.record_status = 'S'))". 
			// użytkownik nie może być wyłączony z synchronizacji
			"AND mu.sync = true ".
			// nowe kryterium mapowania
			" AND mus.trust_level in(4,5,6)".
			// dodatkowy where
			(!is_null($_where) ? "AND $_where " : '').
			// order 
			(!is_null($_order) ? "ORDER BY $_order " : '').
			// limit
			"LIMIT $limit OFFSET $offset"
		);
		
		$res = $res->execute();
		$result = array();
		
		foreach($res as $row) {
			$result[] = $row;
		}
			
		return $result;
	}
	
	public function countChangedRows($where = array()) {
		$restrictions = $this->prepearQuery($where);
		$_where = $restrictions['where'];
		
		$res = $this->adapter->createStatement(
			"SELECT count(*) as \"count\" FROM main_users_sync mus ".
			"INNER JOIN main_users mu ON mus.user_login = mu.user_login ".
			// użytkownik musi być zweryfikowany
			"WHERE (mu.real_status = 'ACTIVE' ".
				// lub ma status inny niż aktywny i nowy ale był już synchronizowany
				"OR (mu.status <> 'NEW' AND mus.record_status = 'S'))". 
			// użytkownik nie może być wyłączony z synchronizacji
			"AND mu.sync = true ". 
			// nie biorę pod uwagę tylko "nowych" użytkowników
			// poprzednia wersja "AND mus.record_status != 'N' ".
			// nowe kryterium mapowania
			" AND mus.trust_level in(4,5,6)".
			// dodatkowy where
			(!is_null($_where) ? " AND $_where " : '')
		);
		
		$res = $res->execute();
		$res = $res->current();
		return $res['count'];
	}
	
	public function getAllRows($where = array(),$order = array(),$limit = 20, $page = 0) {
		$offset = $limit * $page;
		if($offset > 0) $offset--; 
		
		$restrictions = $this->prepearQuery($where,$order);
		$_order = $restrictions['order'];
		$_where = $restrictions['where'];
		
		$res = $this->adapter->createStatement(
		    "SELECT mu.*,mus.* FROM main_users_sync mus ".
			"INNER JOIN main_users mu ON mus.user_login = mu.user_login ".
			(!is_null($_where) ? " WHERE $_where " : '').
			(!is_null($_order) ? " ORDER BY $_order " : '').
			"LIMIT $limit OFFSET $offset"
		);
		
		$res = $res->execute();
		
		$result = array();
		
		foreach($res as $row) {
			$result[] = $row;
		}
		
		return $result;
	}
	
	public function countAllRows($where = array()) {
		
		$restrictions = $this->prepearQuery($where);
		$_where = $restrictions['where'];
		
		$res = $this->adapter->createStatement(
		    "SELECT count(*) as \"count\" FROM main_users_sync mus ".
			"INNER JOIN main_users mu ON mus.user_login = mu.user_login ".
			(!is_null($_where) ? "WHERE $_where " : '')
		);
		
		$res = $res->execute();
		$res = $res->current();
		return (int)@$res['count'];
	}
	
	protected function prepearQuery($where = array(),$order = array()) {
		$_order = null;
		
		if(count($order)) {
			foreach($order as $column => $type) {
			
				//$type = addslashes($value);
				
				if(count($_order)) $_order .= ', ';
				
				$_order .= "$column $type";
			}
		}
		
		$_where = null;
		if(count($where)) {
			foreach($where as $column => $value) {
				if(strlen($_where)) $_where .= ' AND ';
				
				//$value = addslashes($value);
				
				if($column == 'start_date_from') {
					$_where .= " start_date >= (SELECT TIMESTAMP 'epoch' + $value * INTERVAL '1 second')";
				} elseif($column == 'start_date_to') {
					$value = strtotime('23:59',$value);
					$_where .= " start_date <= (SELECT TIMESTAMP 'epoch' + $value * INTERVAL '1 second')";
				} elseif($column == 'user_type') {
					$value = preg_replace('/([A-Za-z0-9_]+)/',"'$1'",$value);
					$_where .= " user_type in($value)";
				} elseif($column == 'modification_date_from') {
					$_where .= " modification_date >= (SELECT TIMESTAMP 'epoch' + $value * INTERVAL '1 second')";
				} elseif($column == 'modification_date_to') {
					$value = strtotime('23:59',$value);
					$_where .= " modification_date <= (SELECT TIMESTAMP 'epoch' + $value * INTERVAL '1 second')";
				} elseif($column == 'cegedim_id') {
					$value = explode(',',$value);
					
					$tmp_where = '';
					
					foreach($value as $_v) {
						if(strlen($tmp_where)) $tmp_where.= ' or ' ;
						if((int)$_v == 0) $tmp_where .= 'mu.cegedim_id is null';
						if((int)$_v == 1) $tmp_where .= 'mu.cegedim_id is not null';
					}
					
					if(strlen($tmp_where)) {
						$_where .= "( $tmp_where )";
					}
				} elseif($column == 'first_name') {
				
					$_where .= " lower(mu.first_name) like('%'||lower('$value')||'%' )";
				} elseif($column == 'last_name') {
					
					$_where .= " lower(mu.last_name) like('%'||lower('$value')||'%' )";
				} elseif($column == 'phys_licence') {
					
					$_where .= " mu.phys_licence = '".$value."'";
				} elseif($column == 'email_ext') {
					
					$_where .= " lower(mu.email_ext) like('%'||lower('$value')||'%' )";
				} elseif($column == 'full_address') {
					
					//$_where .= "( zip like('$value') )";
					$_where .= " ( 
						lower(email_ext)  like('%'||lower('$value')||'%') OR lower(email_esc)  like('%'||lower('$value')||'%') OR lower(address)  like('%'||lower('$value')||'%') 
						OR lower(addr_house) like('%'||lower('$value')||'%') OR lower(addr_apartm) like('%'||lower('$value')||'%')
						OR lower(cityname) like('%'||lower('$value')||'%') OR zip like ('$value') 
						OR lower(work_addr_str) like('%'||lower('$value')||'%') OR lower(work_addr_house) like('%'||lower('$value')||'%')
						OR lower(work_addr_apartm) like('%'||lower('$value')||'%')
						OR lower(workcity) like('%'||lower('$value')||'%') OR work_zip like ('$value') 
						OR lower(work_place_name) like('%'||lower('$value')||'%') OR lower(work_ward_name) like('%'||lower('$value')||'%')
					 )";
				} elseif($column == 'trust_level') {
					
					$_where .= " trust_level in($value)";
				} elseif($column == 'ok_id') {
				
					$_where .= " lower(ok_id) like '%'||lower('$value')||'%'";
				}
			}
		}
		
		return array(
			'where' => $_where,
			'order' => $_order
		);
	}
	
	public function getLastUpdatedFieldsList($userLogin) {
		$map = array(1 => 'first_name',	2 => 'last_name', 3 => 'address', 		4 => 'cityname', 5 => 'zip', 6 => 'tel',
					 7 => 'email_esc', 	8 => 'email_ext', 9 => 'phys_licence', 	10 => '' , 11 => '', 12 => 'status',
					 13 => '', 14 => 'addr_prefix',15 => 'addr_house',16 => 'addr_apartm',17 => 'tel2',18 => 'work_place_name',
					 19 => 'work_ward_name', 20 => 'work_addr_prefix',21 => 'work_addr_str',22 => 'work_addr_house', 23 => 'work_addr_apartm', 24 => 'workcity',
					 25 => 'work_zip', 26 => 'work_tel',27 => '',28 => 'user_type', 29 => '',30 => '',
					 31 => 'work_country', 32 => '',33 => '', 34 => '', 35 => '', 36 => 'real_status',
					 37 => '', 38 => '',39 => 'retired', 40 => 'education_place', 41 => 'education_finish'	 
		);
		
		$res = $this->adapter->createStatement("SELECT change_map FROM {$this->table} WHERE user_login = '$userLogin'");
		$res = $res->execute();
		$res = $res->current();
		$res = $res['change_map'];
		
		$decodedStr = $res;
		$decodedArray = array();
		$position = 1;
		
		while(strlen($decodedStr)) {
			$tmp = substr($decodedStr,0,1);
			
			if($tmp == 1) $decodedArray[$position] = $map[$position];
			
			$decodedStr = substr($decodedStr,1);
			$position++;
		}
		
		$res = $res->execute();
		$result = array();	
		
		foreach($res as $row) {
			$result[] = $row;
		}
		
		return $result;
	}
		
	public function changeTrustLevel($userLogin,$trustLevel) {
		
		$MainUsersTrustLevel = new MainUsersTrustLevels;
		$trustLevelInfo = $MainUsersTrustLevel->getTrustLevelData($trustLevel);
		$changeDateNeed = (int)$trustLevelInfo['change_update_time'];
		
		$res = $this->adapter->createStatement("SELECT trust_level FROM {$this->table} where user_login = '$userLogin'")->execute();
		$res = $res->current();
		$res = $res['trust_level'];
		
		if($res !== $trustLevel) {
			
			$res = $this->adapter->createStatement("
				UPDATE main_users_sync set trust_level = {$trustLevel} ".($changeDateNeed == 1 ? ', trust_level_change = now()' : '')." WHERE user_login = '$userLogin'")->execute();
			
			return (bool)$res;
		}
		
		return true;
	}
}

?>