<?php

namespace Model;

use System\Database\AbstractTable,
	Zend\Db\Sql\Select;


class MainViewCode extends AbstractTable {
	protected $table = 'main_view_code';
	
	public function getCode( $specs , $types , $mainUserId, $terminate = null) {
		$code = $this->generateCode();
		try {
            $data = [
                'main_user_id' => (int)$mainUserId,
                'types' => $types,
                'specs' => $specs,
                'code' => $code
            ];

            if ($terminate != null) {
                $data['terminate'] = $terminate;
            }

			$this->insert($data);
		
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
		
		return $code;
	}
	
	protected function generateCode() {
		
		$charset = array(
			0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 ,
			'A' , 'B' , 'C' , 'D' , 'E' ,  'F' ,  'G' ,  'H' ,  'I' ,  'J' ,  'K' ,  'L' ,  'M' ,  'N' ,  'O' ,  'P' , 'R' ,  'S' ,  'T' ,  'U' ,  'W' ,  'X' ,  'Y' ,  'Z' 
		);
		
		$max = count( $charset ) - 1;
		
		
		do {
			
			$code = '';
			
			for( $i = 0; $i < 12; $i++ ) {
				
				$code .= $charset[ rand( 0 , $max ) ];
			}
			
		} while( !$this->isUnique( $code ) );
		
		return $code;
	}	
	
	
	public function isUnique( $code ) {
		
		try {
			
			$res = $this->select( array( 'code' => $code ) );
			if( $res->count() > 0 )
				return false;
			return true;
			
		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
	
	
	public function loadData( $code ) {
		
		try {
			
			$select = new Select;
			$select->from( $this->table )
				   ->where( array( 'code' => $code ) )
				   ->where( "terminate > now()" );
			
			
			$res = $this->selectWith( $select );
			
			
			if( !$res->count() )
				return false;
			return $res->current()->getArrayCopy();
			
 		} catch( \Exception $e ) {
			
			throw new \Exception( __METHOD__ . '. ' . $e->getMessage() );
		}
	}
}


?>