<?php
namespace Model\TableGateway;

use System\Database\AbstractTable;
use Zend\Db\Sql\TableIdentifier;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Zend\Db\ResultSet\HydratingResultSet;
use Zend\Db\Sql\Where as Where;
use Zend\Db\Sql\Expression as Expression;
use Zend\Db\ResultSet\ResultSet as ResultSet;
use Model\Entity\BannerPlace as Entity;
use Model\TableGateway\MainSite;
use Model\HydratingResultSet\BannerPlace as BannerPlaceHydratingResultSet;

class BannerPlace extends AbstractTable {

    protected static $tableName = 'dict_bc_element_place';
    protected static $schema = 'pl_content';
    protected static $alias = 'banner_place';
    protected static $tableColumns = array('id', 'name', 'describe', 'image', 'region', 'max_collisions', 'available_for_elements', 'main_site_id');

    public function getAll(array $queryConfig = array()) {
        $select = $this->sql->select()
                ->columns(self::$tableColumns)
                ->join(
                array(MainSite::getAlias() => new TableIdentifier(MainSite::getTableName(), MainSite::getSchema())), $this->table . '.main_site_id = ' . MainSite::getAlias() . '.id', MainSite::getNamespacedTableColumns()
        );
        $select = $this->prepareWhereQuery($select, $queryConfig);

        //var_dump($select->getSqlString());die();

        $this->resultSetPrototype = new BannerPlaceHydratingResultSet();
        $results = $this->selectWith($select);

        return $results;
    }

    public function countAll(array $queryConfig = array()) {
        $select = $this->sql->select();
        unset($queryConfig['order']);
        unset($queryConfig['limit']);
        unset($queryConfig['page']);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $select->columns(array('count' => new Expression("COUNT(*)")));

        $this->resultSetPrototype = new ResultSet();
        $result = $this->selectWith($select)->current();
        $result = (int)$result['count'];

        return $result;
    }

    public function isImageNameUnique($imageName) {
        $select = $this->sql->select();

        $where = new Where();
        $where->equalTo('image', $imageName);
        $select->where($where);
        $select->limit(1);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        $result = $this->selectWith($select);

        if ($result->count() == 1) {
            return false;
        } else {
            return true;
        }
    }

    public function getByID($ID) {
        $select = $this->sql->select();

        $where = new Where();
        if (!is_array($ID)) {
            $where->equalTo('id', $ID);
        } else {
            $where->in('id', $ID);
        }
        $select->where($where);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        if (!is_array($ID)) {
            $result = $this->selectWith($select)->current();
        } else {
            $result = $this->selectWith($select);
        }

        return $result;
    }

    public function saveEntity(Entity $Entity) {
        $insert = $this->sql->insert();
        $insert->values(array(
            'name' => $Entity->getName(),
            'describe' => $Entity->getDescribe(),
            'image' => $Entity->getImage(),
            'region' => $Entity->getRegion(),
            'max_collisions' => $Entity->getMaxCollisions(),
            'available_for_elements' => '{' . implode(',', $Entity->getAvailableForElements()) . '}',
            'main_site_id' => $Entity->getMainSiteId(),
        ));

        $insertString = $this->sql->getSqlStringForSqlObject($insert);
        $insertString .= ' RETURNING id';

        $ID = (int)$this->adapter->createStatement($insertString)->execute()->current()['id'];

        if ($ID > 0) {
            return $ID;
        } else {
            return false;
        }
    }

    public function updateEntity(Entity $Entity) {
        $update = $this->sql->update();

        $where = new Where();
        $where->equalTo('id', $Entity->getId());

        $update->set(array(
            'name' => $Entity->getName(),
            'describe' => $Entity->getDescribe(),
            'image' => $Entity->getImage(),
            'region' => $Entity->getRegion(),
            'max_collisions' => $Entity->getMaxCollisions(),
            'available_for_elements' => '{' . implode(',', $Entity->getAvailableForElements()) . '}',
            'main_site_id' => $Entity->getMainSiteId(),
        ));
        $update->where($where);

        return $this->updateWith($update);
    }

    public function deleteEntity(Entity $Entity) {
        $delete = $this->sql->delete();

        $where = new Where();
        $where->equalTo('id', $Entity->getId());

        $delete->where($where);

        return $this->deleteWith($delete);
    }

    private function prepareWhereQuery(&$select, $queryConfig) {
        if (isset($queryConfig['where']) && count($queryConfig['where']) > 0) {
            $where = new Where();

            foreach ($queryConfig['where'] as $whereConfig) {
                reset($whereConfig);
                $field = trim(key($whereConfig));
                $value = trim($whereConfig[$field]);

                switch ($field) {
                    case 'id':
                        $value = (int)$value;
                        $where->equalTo(self::$schema . '.' . self::$tableName . '.' . $field, $value);
                        break;
                    case 'name':
                        $where->expression('LOWER(' . self::$schema . '.' . self::$tableName . '.' . $field . ') LIKE ?', '%' . mb_strtolower($value, 'UTF-8') . '%');
                        break;
                    case 'describe':
                        $where->expression('LOWER(' . self::$schema . '.' . self::$tableName . '.' . $field . ') LIKE ?', '%' . mb_strtolower($value, 'UTF-8') . '%');
                        break;
                    case 'max_collisions_min':
                        $where->greaterThanOrEqualTo(self::$schema . '.' . self::$tableName . '.' . 'max_collisions', $value);
                        break;
                    case 'max_collisions_max':
                        $where->lessThanOrEqualTo(self::$schema . '.' . self::$tableName . '.' . 'max_collisions', $value);
                        break;
                    case 'main_site_id':
                        $value = explode(',', $value);
                        foreach ($value as $key => $val) {
                            if ($val == 'null') {
                                unset($value[$key]);
                            }
                        }

                        if (count($value) > 1) {
                            foreach ($value as $key => $val) {
                                if (!$val) {
                                    unset($value[$key]);
                                } else {
                                    $value[$key] = (int)$val;
                                }
                            }
                            $where->in($field, $value);
                        } elseif (count($value) == 1) {
                            $where->equalTo($field, $value[0]);
                        }
                        

                        break;
                }
            }
            $select->where($where);
        }

        if (isset($queryConfig['order'])) {
            reset($queryConfig['order']);
            $field = trim(key($queryConfig['order']));
            $direction = trim($queryConfig['order'][$field]);
            if (strpos($field, '.') === false) {
                $field = self::$tableName . '.' . $field;
            }
            $select->order($field . ' ' . $direction);
        }

        if (isset($queryConfig['limit'])) {
            $select->limit((int)$queryConfig['limit']);
        }

        if (isset($queryConfig['page']) && isset($queryConfig['limit'])) {
            $queryConfig['page'] < 1 ? $page = 0 : $page = (int)$queryConfig['page'] - 1;
            $offset = $page * $queryConfig['limit'];
            $select->offset($offset);
        }

        return $select;
    }

}