<?php
namespace Model\TableGateway;

use System\Database\AbstractTable;
use Zend\Db\Sql\TableIdentifier;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Zend\Db\ResultSet\HydratingResultSet;
use Model\Entity\DictMainSiteContentType as Entity;
use Zend\Db\Sql\Where as Where;
use Zend\Db\Sql\Expression as Expression;
use Zend\Db\ResultSet\ResultSet as ResultSet;
use Model\TableGateway\MainSiteContentPage as MainSiteContentPageTableGateway;
use Zend\Db\Sql\Predicate\NotIn;

class DictMainSiteContentType extends AbstractTable {

    protected static $tableName = 'dict_main_site_content_type';
    protected static $schema = 'pl_content';
    protected static $alias = 'dict_main_site_content_type';
    protected static $tableColumns = array('id', 'name');

    public function getAll(array $queryConfig = array()) {
        $select = $this->sql->select()->columns(self::$tableColumns);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        $results = $this->selectWith($select);

        return $results;
    }

    public function countAll(array $queryConfig = array()) {
        $select = $this->sql->select();
        unset($queryConfig['order']);
        unset($queryConfig['limit']);
        unset($queryConfig['page']);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $select->columns(array('count' => new Expression("COUNT(*)")));

        $this->resultSetPrototype = new ResultSet();
        $result = $this->selectWith($select)->current();
        $result = (int)$result['count'];

        return $result;
    }

    public function getByID($ID) {
        $select = $this->sql->select();

        $where = new Where();
        if (!is_array($ID)) {
            $where->equalTo('id', $ID);
        } else {
            $where->in('id', $ID);
        }
        $select->where($where);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        if (!is_array($ID)) {
            $result = $this->selectWith($select)->current();
        } else {
            $result = $this->selectWith($select);
        }

        return $result;
    }

    public function getUrelatedWithMainSiteID($MainSiteID) {
        $subselect = $this->sql->select()
                ->columns(array('id'))
                ->join(array(
            MainSiteContentPageTableGateway::getAlias() => new TableIdentifier(MainSiteContentPageTableGateway::getTableName(), MainSiteContentPageTableGateway::getSchema())), $this->table . '.id = ' . MainSiteContentPageTableGateway::getAlias() . '.dict_main_site_content_type_id', array()
        );
        $where = new Where();
        $where->equalTo(MainSiteContentPageTableGateway::getTableName() . '.main_site_id', $MainSiteID);
        $subselect->where($where);

        $select = $this->sql->select();
        $select->where(new NotIn('id', $subselect));

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        $result = $this->selectWith($select);

        return $result;
    }

    public function saveEntity(Entity $Entity) {
        $insert = $this->sql->insert();
        $insert->values(array(
            'name' => $Entity->getName(),
        ));

        return $this->insertWith($insert);
    }

    public function updateEntity(Entity $Entity) {
        $update = $this->sql->update();

        $where = new Where();
        $where->equalTo('id', $Entity->getId());

        $update->set(array(
            'name' => $Entity->getName(),
        ));
        $update->where($where);

        return $this->updateWith($update);
    }

    public function deleteEntity(Entity $Entity) {
        $delete = $this->sql->delete();

        $where = new Where();
        $where->equalTo('id', $Entity->getId());

        $delete->where($where);

        return $this->deleteWith($delete);
    }

    private function prepareWhereQuery(&$select, $queryConfig) {
        if (isset($queryConfig['where']) && count($queryConfig['where']) > 0) {
            $where = new Where();

            foreach ($queryConfig['where'] as $whereConfig) {
                reset($whereConfig);
                $field = trim(key($whereConfig));
                $value = trim($whereConfig[$field]);

                switch ($field) {
                    case 'id':
                        $where->equalTo(self::$schema . '.' . self::$tableName . '.' . $field, $value);
                        break;
                    case 'name':
                        $where->expression('LOWER(' . self::$schema . '.' . self::$tableName . '.' . $field . ') LIKE ?', '%' . mb_strtolower($value, 'UTF-8') . '%');
                        break;
                }
            }
            $select->where($where);
        }

        if (isset($queryConfig['order'])) {
            reset($queryConfig['order']);
            $field = trim(key($queryConfig['order']));
            $direction = trim($queryConfig['order'][$field]);
            if (strpos($field, '.') === false) {
                $field = self::$tableName . '.' . $field;
            }
            $select->order($field . ' ' . $direction);
        }

        if (isset($queryConfig['limit'])) {
            $select->limit((int)$queryConfig['limit']);
        }

        if (isset($queryConfig['page']) && isset($queryConfig['limit'])) {
            $queryConfig['page'] < 1 ? $page = 0 : $page = (int)$queryConfig['page'] - 1;
            $offset = $page * $queryConfig['limit'];
            $select->offset($offset);
        }

        return $select;
    }

}
