<?php
namespace Model\TableGateway;

use System\Database\AbstractTable;
use Zend\Db\Sql\TableIdentifier;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Zend\Db\ResultSet\HydratingResultSet;
use Model\Entity\MainEdition as Entity;
use Zend\Db\Sql\Where as Where;
use Zend\Db\Sql\Expression as Expression;
use Zend\Db\ResultSet\ResultSet as ResultSet;

class MainEditions extends AbstractTable
{

    protected static $tableName = 'main_editions';
    protected static $schema = 'pl_content';
    protected static $alias = 'editions';
    protected static $tableColumns = array(
        'id_edition',
        'edition_year',
        'edition_nr',
        'edition_book_nr',
        'book_nr',
        'book_month',
        'book_year',
        'subject',
        'spec_code',
        'chief_nr',
        'image_name',
        'status'
    );

    public function getAll(array $queryConfig = array())
    {
        $select = $this->sql->select()->columns(self::$tableColumns);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        $results = $this->selectWith($select);

        return $results;
    }

    public function getSubjects()
    {
        $select = $this->sql->select()->columns(self::$tableColumns);

        $where = new Where();
        $where->equalTo('status', 'VIS');
        $select->where($where);
        $select->quantifier(\Zend\Db\Sql\Select::QUANTIFIER_DISTINCT);

        $select->order(new Expression('subject ASC NULLS LAST'));

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        $results = $this->selectWith($select);

        $data = [];
        foreach ($results as $result) {
            if (strpos($result->getSubject(), '-')) {
                $data[] = trim(explode('-', $result->getSubject())[0]);
            } elseif (strpos($result->getSubject(), ',')) {
                $data[] = trim(explode(',', $result->getSubject())[0]);
            } else if ($result->getSubject()) {
                $data[] = trim($result->getSubject());
            }
        }

        $data = array_unique($data);

        return $data;
    }

    public
    function countAll(
        array $queryConfig = array()
    ) {
        $select = $this->sql->select();
        unset($queryConfig['order']);
        unset($queryConfig['limit']);
        unset($queryConfig['page']);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $select->columns(array('count' => new Expression("COUNT(*)")));

        $this->resultSetPrototype = new ResultSet();
        $result = $this->selectWith($select)->current();
        $result = (int)$result['count'];

        return $result;
    }

    public
    function getByID(
        $ID
    ) {
        $select = $this->sql->select();

        $where = new Where();
        if (!is_array($ID)) {
            $where->equalTo('id_edition', $ID);
        } else {
            $where->in('id_edition', $ID);
        }
        $select->where($where);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        if (!is_array($ID)) {
            $result = $this->selectWith($select)->current();
        } else {
            $result = $this->selectWith($select);
        }

        return $result;
    }

    public
    function saveEntity(
        Entity $Entity
    ) {
        $insert = $this->sql->insert();
        $insert->values(
            array(
                'edition_year'    => $Entity->getEditionYear(),
                'edition_nr'      => $Entity->getEditionNr(),
                'edition_book_nr' => $Entity->getEditionBookNr(),
                'book_nr'         => $Entity->getBookNr(),
                'book_month'      => $Entity->getBookMonth(),
                'book_year'       => $Entity->getBookYear(),
                'subject'         => $Entity->getSubject(),
                'spec_code'       => $Entity->getSpecCode(),
                'chief_nr'        => $Entity->getChiefNr(),
                'image_name'      => $Entity->getImageName(),
                'status'          => $Entity->getStatus(),
            )
        );

        return $this->insertWith($insert);
    }

    public
    function deleteEntity(
        Entity $Entity
    ) {
        $delete = $this->sql->delete();

        $where = new Where();
        $where->equalTo('id_edition', $Entity->getIdEdition());

        $delete->where($where);

        return $this->deleteWith($delete);
    }

    public
    function updateEntity(
        Entity $Entity
    ) {
        $update = $this->sql->update();

        $where = new Where();
        $where->equalTo('id_edition', $Entity->getIdEdition());

        $update->set(
            array(
                'edition_year'    => $Entity->getEditionYear(),
                'edition_nr'      => $Entity->getEditionNr(),
                'edition_book_nr' => $Entity->getEditionBookNr(),
                'book_nr'         => $Entity->getBookNr(),
                'book_month'      => $Entity->getBookMonth(),
                'book_year'       => $Entity->getBookYear(),
                'subject'         => $Entity->getSubject(),
                'spec_code'       => $Entity->getSpecCode(),
                'chief_nr'        => $Entity->getChiefNr(),
                'image_name'      => $Entity->getImageName(),
                'status'          => $Entity->getStatus(),
            )
        );
        $update->where($where);

        return $this->updateWith($update);
    }

    private
    function prepareWhereQuery(
        &$select, $queryConfig
    ) {
        if (isset($queryConfig['where']) && count($queryConfig['where']) > 0) {
            $where = new Where();

            foreach ($queryConfig['where'] as $whereConfig) {
                reset($whereConfig);
                $field = trim(key($whereConfig));
                $value = trim($whereConfig[$field]);

                switch ($field) {
                    case 'id_edition':
                    case 'book_year':
                        $where->equalTo($field, $value);
                        break;
                    case 'edition_year':
                    case 'edition_nr':
                    case 'edition_book_nr':
                    case 'book_nr':
                    case 'book_month':
                    case 'subject':
                    case 'spec_code':
                    case 'chief_nr':
                        $where->expression('LOWER(' . $field . ') LIKE ?', '%' . mb_strtolower($value, 'UTF-8') . '%');
                        break;
                    case 'status':
                        if ($value === true || $value === 'true') {
                            $where->equalTo($field, 'VIS');
                        } else {
                            $where->equalTo($field, 'HID');
                        }
                        break;
                }
            }
            $select->where($where);
        }

        if (isset($queryConfig['order'])) {
            reset($queryConfig['order']);
            $field = trim(key($queryConfig['order']));
            $direction = trim($queryConfig['order'][$field]);
            if (strpos($field, '.') === false) {
                $field = self::$tableName . '.' . $field;
            }

            $fieldExploded = explode('.', $field);

            if (end($fieldExploded) != 'id_edition') {
                if (end($fieldExploded) == 'edition_nr' || end($fieldExploded) == 'edition_book_nr' || end($fieldExploded) == 'book_nr') {
                    $select->order(
                        new Expression("
                        CASE WHEN position('-' in {$field}) = 0 THEN 
                            {$field}::INTEGER
                        ELSE
                            substring({$field} from 1 for position('-' in {$field})-1)::INTEGER
                        END" . ' ' . $direction . ' NULLS LAST, id_edition DESC NULLS LAST')
                    );
                } elseif (end($fieldExploded) == 'book_month') {
                    $translator = \System\Translator\TranslatorEx::get();
                    $monthOrder = explode(',', $translator->translate('main_months_names'));
                    $lastKey = 0;
                    foreach ($monthOrder as $key => $month) {
                        $monthOrder[$key] = ' when \'' . mb_strtoupper($month, 'UTF-8') . '\' then ' . ($key + 1);
                        $lastKey = ($key + 1);
                    }
                    if ($direction == 'desc') {
                        $monthOrder[($lastKey + 1)] = ' else ' . 0 . ' end ';
                    } else {
                        $monthOrder[($lastKey + 1)] = ' else ' . ($lastKey + 1) . ' end ';
                    }

                    $select->order(
                        new Expression(' case ' .
                            " CASE WHEN position('-' in {$field}) = 0 THEN " . $field . " ELSE substring({$field} from 1 for position('-' in {$field})-1) END "
                            . ' ' . implode($monthOrder) . ' ' . $direction . ', id_edition DESC NULLS LAST')
                    );
                } else {
                    $select->order(new Expression($field . ' ' . $direction . ' NULLS LAST, id_edition DESC NULLS LAST'));
                }
            } else {
                $select->order(new Expression($field . ' ' . $direction . ' NULLS LAST'));
            }
        }

        if (isset($queryConfig['limit'])) {
            $select->limit((int)$queryConfig['limit']);
        }

        if (isset($queryConfig['page']) && isset($queryConfig['limit'])) {
            $queryConfig['page'] < 1 ? $page = 0 : $page = (int)$queryConfig['page'] - 1;
            $offset = $page * $queryConfig['limit'];
            $select->offset($offset);
        }

        return $select;
    }

}