<?php
namespace Model\TableGateway;

use System\Database\AbstractTable;
use Zend\Db\Sql\TableIdentifier;
use Model\TableGateway\MainSite;
use Model\TableGateway\DictMainSiteContentType;
use Model\TableGateway\DictMainSiteContentTypeView;
use Model\HydratingResultSet\MainSiteContentPage as MainSiteContentPageResultSet;
use Model\Entity\MainSiteContentPage as Entity;
use Zend\Db\Sql\Where as Where;
use Zend\Db\Sql\Expression as Expression;
use Zend\Db\ResultSet\ResultSet as ResultSet;

class MainSiteContentPage extends AbstractTable {

    protected static $tableName = 'main_site_content_page';
    protected static $schema = 'pl_content';
    protected static $alias = 'main_site_content_page';
    protected static $tableColumns = array('id', 'main_site_id', 'dict_main_site_content_type_view_id', 'view');

    public function getAll(array $queryConfig = array()) {
        $select = $this->sql->select()
                        ->columns(self::$tableColumns)
                        ->join(array(
                            MainSite::getAlias() => new TableIdentifier(MainSite::getTableName(), MainSite::getSchema())), $this->table . '.main_site_id = ' . MainSite::getAlias() . '.id', MainSite::getNamespacedTableColumns())
                        ->join(array(
                            DictMainSiteContentTypeView::getAlias() => new TableIdentifier(DictMainSiteContentTypeView::getTableName(), DictMainSiteContentTypeView::getSchema())), $this->table . '.dict_main_site_content_type_view_id = ' . DictMainSiteContentTypeView::getAlias() . '.id', DictMainSiteContentTypeView::getNamespacedTableColumns()
                        )->join(array(
            DictMainSiteContentType::getAlias() => new TableIdentifier(DictMainSiteContentType::getTableName(), DictMainSiteContentType::getSchema())), DictMainSiteContentTypeView::getAlias() . '.dict_main_site_content_type_id = ' . DictMainSiteContentType::getAlias() . '.id', DictMainSiteContentType::getNamespacedTableColumns()
        );
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $this->resultSetPrototype = new MainSiteContentPageResultSet();
        $results = $this->selectWith($select);

        return $results;
    }

    public function countAll(array $queryConfig = array()) {
        $select = $this->sql->select()
                        ->columns(array())
                        ->join(array(
                            MainSite::getAlias() => new TableIdentifier(MainSite::getTableName(), MainSite::getSchema())), $this->table . '.main_site_id = ' . MainSite::getAlias() . '.id', array())
                        ->join(array(
                            DictMainSiteContentTypeView::getAlias() => new TableIdentifier(DictMainSiteContentTypeView::getTableName(), DictMainSiteContentTypeView::getSchema())), $this->table . '.dict_main_site_content_type_view_id = ' . DictMainSiteContentTypeView::getAlias() . '.id', array()
                        )->join(array(
            DictMainSiteContentType::getAlias() => new TableIdentifier(DictMainSiteContentType::getTableName(), DictMainSiteContentType::getSchema())), DictMainSiteContentTypeView::getAlias() . '.dict_main_site_content_type_id = ' . DictMainSiteContentType::getAlias() . '.id', array()
        );
        unset($queryConfig['order']);
        unset($queryConfig['limit']);
        unset($queryConfig['page']);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $select->columns(array('count' => new Expression("COUNT(*)")));

        $this->resultSetPrototype = new ResultSet();
        $result = $this->selectWith($select)->current();
        $result = (int)$result['count'];

        return $result;
    }

    public function getByID($ID) {
        $select = $this->sql->select()
                ->columns(self::$tableColumns)
                ->join(array(
                    MainSite::getAlias() => new TableIdentifier(MainSite::getTableName(), MainSite::getSchema())), $this->table . '.main_site_id = ' . MainSite::getAlias() . '.id', MainSite::getNamespacedTableColumns())
                ->join(array(
            DictMainSiteContentTypeView::getAlias() => new TableIdentifier(DictMainSiteContentTypeView::getTableName(), DictMainSiteContentTypeView::getSchema())), $this->table . '.dict_main_site_content_type_view_id = ' . DictMainSiteContentTypeView::getAlias() . '.id', DictMainSiteContentTypeView::getNamespacedTableColumns()
        );

        $where = new Where();
        if (!is_array($ID)) {
            $where->equalTo(self::$schema . '.' . self::$tableName . '.id', $ID);
        } else {
            $where->in(self::$schema . '.' . self::$tableName . '.id', $ID);
        }
        $select->where($where);

        $this->resultSetPrototype = new MainSiteContentPageResultSet();
        if (!is_array($ID)) {
            $result = $this->selectWith($select)->current();
        } else {
            $result = $this->selectWith($select);
        }

        return $result;
    }

    public function getByMainSiteID($MainSiteID) {
        $select = $this->sql->select()
                ->columns(self::$tableColumns)
                ->join(array(
                    MainSite::getAlias() => new TableIdentifier(MainSite::getTableName(), MainSite::getSchema())), $this->table . '.main_site_id = ' . MainSite::getAlias() . '.id', MainSite::getNamespacedTableColumns())
                ->join(array(
            DictMainSiteContentTypeView::getAlias() => new TableIdentifier(DictMainSiteContentTypeView::getTableName(), DictMainSiteContentTypeView::getSchema())), $this->table . '.dict_main_site_content_type_view_id = ' . DictMainSiteContentTypeView::getAlias() . '.id', DictMainSiteContentTypeView::getNamespacedTableColumns()
        );

        $where = new Where();
        if (!is_array($MainSiteID)) {
            $where->equalTo(self::$schema . '.' . self::$tableName . '.main_site_id', $MainSiteID);
        } else {
            $where->in(self::$schema . '.' . self::$tableName . '.main_site_id', $MainSiteID);
        }
        $select->where($where);

        $this->resultSetPrototype = new MainSiteContentPageResultSet();
        $result = $this->selectWith($select);

        return $result;
    }

    public function saveEntity(Entity $Entity) {
        $insert = $this->sql->insert();
        $insert->values(array(
            'main_site_id' => $Entity->getMainSiteId(),
            'dict_main_site_content_type_view_id' => $Entity->getDictMainSiteContentTypeViewId(),
            'view' => $Entity->getView(),
        ));

        return $this->insertWith($insert);
    }

    public function updateEntity(Entity $Entity) {
        $update = $this->sql->update();

        $where = new Where();
        $where->equalTo('id', $Entity->getId());

        $update->set(array(
            'main_site_id' => $Entity->getMainSiteId(),
            'dict_main_site_content_type_view_id' => $Entity->getDictMainSiteContentTypeViewId(),
            'view' => $Entity->getView(),
        ));
        $update->where($where);

        return $this->updateWith($update);
    }

    public function deleteEntity(Entity $Entity) {
        $delete = $this->sql->delete();

        $where = new Where();
        $where->equalTo('id', $Entity->getId());

        $delete->where($where);

        return $this->deleteWith($delete);
    }

    private function prepareWhereQuery(&$select, $queryConfig) {
        if (isset($queryConfig['where']) && count($queryConfig['where']) > 0) {
            $where = new Where();

            foreach ($queryConfig['where'] as $whereConfig) {
                reset($whereConfig);
                $field = trim(key($whereConfig));
                $value = trim($whereConfig[$field]);

                switch ($field) {
                    case 'id':
                        $value = (int)$value;
                        $where->equalTo(self::$schema . '.' . self::$tableName . '.' . $field, $value);
                        break;
                    case 'main_site_id':
                        $value = explode(',', $value);
                        foreach ($value as $key => $valueElement) {
                            $value[$key] = (int)$value[$key];
                            if ($value[$key] == 0) {
                                unset($value[$key]);
                            }
                        }
                        $value = array_values($value);

                        if (count($value) > 1) {
                            $where->in(self::$schema . '.' . self::$tableName . '.' . $field, $value);
                        } elseif (count($value) == 1) {
                            $where->equalTo(self::$schema . '.' . self::$tableName . '.' . $field, $value[0]);
                        }
                        break;
                    case 'dict_main_site_content_type_id':
                        $value = explode(',', $value);
                        foreach ($value as $key => $valueElement) {
                            $value[$key] = (int)$value[$key];
                            if ($value[$key] == 0) {
                                unset($value[$key]);
                            }
                        }
                        $value = array_values($value);

                        if (count($value) > 1) {
                            $where->in(DictMainSiteContentType::getTableName() . '.id', $value);
                        } elseif (count($value) == 1) {
                            $where->equalTo(DictMainSiteContentType::getTableName() . '.id', $value[0]);
                        }
                        break;
                }
            }
            $select->where($where);
        }

        if (isset($queryConfig['order'])) {
            reset($queryConfig['order']);
            $field = trim(key($queryConfig['order']));
            $direction = trim($queryConfig['order'][$field]);
            if (strpos($field, '.') === false) {
                $field = self::$tableName . '.' . $field;
            }
            $select->order($field . ' ' . $direction);
        }

        if (isset($queryConfig['limit'])) {
            $select->limit((int)$queryConfig['limit']);
        }

        if (isset($queryConfig['page']) && isset($queryConfig['limit'])) {
            $queryConfig['page'] < 1 ? $page = 0 : $page = (int)$queryConfig['page'] - 1;
            $offset = $page * $queryConfig['limit'];
            $select->offset($offset);
        }

        return $select;
    }

}
