<?php
namespace Model\TableGateway;

use System\Database\AbstractTable;
use Zend\Db\Sql\TableIdentifier;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Zend\Db\ResultSet\HydratingResultSet;
use Model\Entity\MainSiteInsomniaQuestion as Entity;
use Zend\Db\Sql\Where as Where;
use Zend\Db\Sql\Expression as Expression;
use Zend\Db\ResultSet\ResultSet as ResultSet;

class MainSiteInsomniaQuestion extends AbstractTable {

    protected static $tableName = 'main_site_insomnia_question';
    protected static $schema = 'esculap_int';
    protected static $alias = 'main_site_insomnia_question';
    protected static $tableColumns = array('id', 'email', 'title', 'question', 'submit_date');

    public function getAll(array $queryConfig = array()) {
        $select = $this->sql->select()->columns(self::$tableColumns);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        $results = $this->selectWith($select);

        return $results;
    }

    public function countAll(array $queryConfig = array()) {
        $select = $this->sql->select();
        unset($queryConfig['order']);
        unset($queryConfig['limit']);
        unset($queryConfig['page']);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $select->columns(array('count' => new Expression("COUNT(*)")));

        $this->resultSetPrototype = new ResultSet();
        $result = $this->selectWith($select)->current();
        $result = (int)$result['count'];

        return $result;
    }

    public function getByID($ID) {
        $select = $this->sql->select();

        $where = new Where();
        if (!is_array($ID)) {
            $where->equalTo('id', $ID);
        } else {
            $where->in('id', $ID);
        }
        $select->where($where);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        if (!is_array($ID)) {
            $result = $this->selectWith($select)->current();
        } else {
            $result = $this->selectWith($select);
        }

        return $result;
    }

    public function deleteEntity(Entity $Entity) {
        $delete = $this->sql->delete();

        $where = new Where();
        $where->equalTo('id', $Entity->getId());

        $delete->where($where);

        return $this->deleteWith($delete);
    }

    private function prepareWhereQuery(&$select, $queryConfig) {
        if (isset($queryConfig['where']) && count($queryConfig['where']) > 0) {
            $where = new Where();

            foreach ($queryConfig['where'] as $whereConfig) {
                reset($whereConfig);
                $field = trim(key($whereConfig));
                $value = trim($whereConfig[$field]);

                switch ($field) {
                    case 'id':
                        $where->equalTo($field, $value);
                        break;
                    case 'email':
                        $where->expression('LOWER(' . $field . ') LIKE ?', '%' . mb_strtolower($value, 'UTF-8') . '%');
                        break;
                    case 'submit_date_min':
                        $value = str_replace('/', '-', $value);
                        $value = explode('-', $value);
                        $value = $value[2] . '-' . $value[1] . '-' . $value[0] . ' 00:00:00';
                        $where->expression('submit_date >= ?', $value);
                        break;
                    case 'submit_date_max':
                        $value = str_replace('/', '-', $value);
                        $value = explode('-', $value);
                        $value = $value[2] . '-' . $value[1] . '-' . $value[0] . ' 23:59:59';
                        $where->expression('submit_date <= ?', str_replace('/', '-', $value));
                        break;
                }
            }
            $select->where($where);
        }

        if (isset($queryConfig['order'])) {
            reset($queryConfig['order']);
            $field = trim(key($queryConfig['order']));
            $direction = trim($queryConfig['order'][$field]);
            if (strpos($field, '.') === false) {
                $field = self::$tableName . '.' . $field;
            }
            $select->order($field . ' ' . $direction);
        }

        if (isset($queryConfig['limit'])) {
            $select->limit((int)$queryConfig['limit']);
        }

        if (isset($queryConfig['page']) && isset($queryConfig['limit'])) {
            $queryConfig['page'] < 1 ? $page = 0 : $page = (int)$queryConfig['page'] - 1;
            $offset = $page * $queryConfig['limit'];
            $select->offset($offset);
        }

        return $select;
    }

}