<?php
namespace Model\TableGateway;

use System\Database\AbstractTable;
use Zend\Db\Sql\TableIdentifier;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Zend\Db\ResultSet\HydratingResultSet;
use Zend\Db\Sql\Where as Where;
use Zend\Db\Sql\Expression as Expression;
use Zend\Db\ResultSet\ResultSet as ResultSet;
use Model\Entity\MainUser as Entity;

class MainUser extends AbstractTable {

    protected static $tableName = 'main_users';
    protected static $schema = 'esculap_int';
    protected static $alias = 'main_users';
    protected static $tableColumns = array('id', 'login', 'password',
        'email', 'lang', 'work_country',
        'onekey_id', 'user_type', 'trust_level',
        'start_date', 'modification_date', 'retired',
        'regulations', 'email_esc', 'sex', 'quick_registration');

    public function getAll(array $queryConfig = array()) {
        $select = $this->sql->select()->columns(self::$tableColumns);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        $results = $this->selectWith($select);

        return $results;
    }

    public function countAll(array $queryConfig = array()) {
        $select = $this->sql->select();
        unset($queryConfig['order']);
        unset($queryConfig['limit']);
        unset($queryConfig['page']);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $select->columns(array('count' => new Expression("COUNT(*)")));

        $this->resultSetPrototype = new ResultSet();
        $result = $this->selectWith($select)->current();
        $result = (int)$result['count'];

        return $result;
    }

    public function getByID($ID) {
        $select = $this->sql->select();

        $where = new Where();
        if (!is_array($ID)) {
            $where->equalTo('id', $ID);
        } else {
            $where->in('id', $ID);
        }
        $select->where($where);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        if (!is_array($ID)) {
            $result = $this->selectWith($select)->current();
        } else {
            $result = $this->selectWith($select);
        }

        return $result;
    }

    public function saveEntity(Entity $Entity) {
        $insert = $this->sql->insert();
        $insert->values(array(
            'login' => $Entity->getLogin(),
            'password' => $Entity->getPassword(),
            'email' => $Entity->getEmail(),
            'lang' => $Entity->getLang(),
            'work_country' => $Entity->getWorkCountry(),
            'onekey_id' => $Entity->getOnekeyId(),
            'user_type' => $Entity->getUserType(),
            'trust_level' => $Entity->getTrustLevel(),
            'start_date' => $Entity->getStartDate(),
            'modification_date' => $Entity->getModificationDate(),
            'retired' => $Entity->getRetired(),
            'regulations' => $Entity->getRegulations(),
            'email_esc' => $Entity->getEmailEsc(),
            'sex' => $Entity->getSex(),
            'quick_registration' => $Entity->getQuickRegistration()
        ));

        return $this->insertWith($insert);
    }

    public function deleteEntity(Entity $Entity) {
        $delete = $this->sql->delete();

        $where = new Where();
        $where->equalTo('id', $Entity->getId());

        $delete->where($where);

        return $this->deleteWith($delete);
    }

    public function updateEntity(Entity $Entity) {
        $update = $this->sql->update();

        $where = new Where();
        $where->equalTo('id', $Entity->getId());

        $update->set(array(
                'login' => $Entity->getLogin(),
                'password' => $Entity->getPassword(),
                'email' => $Entity->getEmail(),
                'lang' => $Entity->getLang(),
                'work_country' => $Entity->getWorkCountry(),
                'onekey_id' => $Entity->getOnekeyId(),
                'user_type' => $Entity->getUserType(),
                'trust_level' => $Entity->getTrustLevel(),
                'start_date' => $Entity->getStartDate(),
                'modification_date' => $Entity->getModificationDate(),
                'retired' => $Entity->getRetired(),
                'regulations' => $Entity->getRegulations(),
                'email_esc' => $Entity->getEmailEsc(),
                'sex' => $Entity->getSex(),
                'quick_registration' => $Entity->getQuickRegistration()
        ));
        $update->where($where);

        return $this->updateWith($update);
    }

    private function prepareWhereQuery(&$select, $queryConfig) {
        if (isset($queryConfig['where']) && count($queryConfig['where']) > 0) {
            $where = new Where();

            foreach ($queryConfig['where'] as $whereConfig) {
                reset($whereConfig);
                $field = trim(key($whereConfig));
                $value = trim($whereConfig[$field]);

                switch ($field) {
                    case 'id':
                        $where->equalTo($field, $value);
                        break;
                }
            }
            $select->where($where);
        }

        if (isset($queryConfig['order'])) {
            reset($queryConfig['order']);
            $field = trim(key($queryConfig['order']));
            $direction = trim($queryConfig['order'][$field]);
            if (strpos($field, '.') === false) {
                $field = self::$tableName . '.' . $field;
            }
            $select->order($field . ' ' . $direction);
        }

        if (isset($queryConfig['limit'])) {
            $select->limit((int)$queryConfig['limit']);
        }

        if (isset($queryConfig['page']) && isset($queryConfig['limit'])) {
            $queryConfig['page'] < 1 ? $page = 0 : $page = (int)$queryConfig['page'] - 1;
            $offset = $page * $queryConfig['limit'];
            $select->offset($offset);
        }

        return $select;
    }

}