<?php
namespace Model\TableGateway;

use System\Database\AbstractTable;
use Zend\Db\Sql\TableIdentifier;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Zend\Db\ResultSet\HydratingResultSet;
use Model\Entity\RelPublEdition as Entity;
use Zend\Db\Sql\Where as Where;
use Zend\Db\Sql\Expression as Expression;
use Zend\Db\ResultSet\ResultSet as ResultSet;

class RelPublEdition extends AbstractTable {

    protected static $tableName = 'rel_publ_edition';
    protected static $schema = 'pl_content';
    protected static $alias = 'rel_p_e';
    protected static $tableColumns = array('id_edition', 'id_publ');

    public function getAll(array $queryConfig = array()) {
        $select = $this->sql->select()->columns(self::$tableColumns);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $this->resultSetPrototype = new HydratingResultSet(new ClassMethodsHydrator(), new Entity());
        $results = $this->selectWith($select);

        return $results;
    }

    public function countAll(array $queryConfig = array()) {
        $select = $this->sql->select();
        unset($queryConfig['order']);
        unset($queryConfig['limit']);
        unset($queryConfig['page']);
        $select = $this->prepareWhereQuery($select, $queryConfig);

        $select->columns(array('count' => new Expression("COUNT(*)")));

        $this->resultSetPrototype = new ResultSet();
        $result = $this->selectWith($select)->current();
        $result = (int)$result['count'];

        return $result;
    }
    
    public function saveEntity(Entity $Entity) {
        $insert = $this->sql->insert();
        $insert->values(array('id_edition' => $Entity->getIdEdition(), 'id_publ' => $Entity->getIdPubl()));

        return $this->insertWith($insert);
    }

    public function deleteEntity(Entity $Entity) {
        $delete = $this->sql->delete();

        $where = new Where();
        $where->equalTo('id_publ', $Entity->getIdPubl());

        $delete->where($where);
        
        return $this->deleteWith($delete);
    }

    public function updateEntity(Entity $Entity) {
        $update = $this->sql->update();

        $where = new Where();
        $where->equalTo('id_publ', $Entity->getIdPubl());

        $update->set(array('id_edition' => $Entity->getIdEdition(), 'id_publ' => $Entity->getIdPubl()));
        $update->where($where);

        return $this->updateWith($update);
    }

    private function prepareWhereQuery(&$select, $queryConfig) {
        if (isset($queryConfig['where']) && count($queryConfig['where']) > 0) {
            $where = new Where();

            foreach ($queryConfig['where'] as $whereConfig) {
                reset($whereConfig);
                $field = trim(key($whereConfig));
                $value = trim($whereConfig[$field]);

                switch ($field) {
                    case 'id_edition':
                    case 'id_publ':
                        $where->equalTo($field, $value);
                        break;
                }
            }
            $select->where($where);
        }

        if (isset($queryConfig['order'])) {
            reset($queryConfig['order']);
            $field = trim(key($queryConfig['order']));
            $direction = trim($queryConfig['order'][$field]);
            if (strpos($field, '.') === false) {
                $field = self::$tableName . '.' . $field;
            }
            $select->order($field . ' ' . $direction);
        }

        if (isset($queryConfig['limit'])) {
            $select->limit((int)$queryConfig['limit']);
        }

        if (isset($queryConfig['page']) && isset($queryConfig['limit'])) {
            $queryConfig['page'] < 1 ? $page = 0 : $page = (int)$queryConfig['page'] - 1;
            $offset = $page * $queryConfig['limit'];
            $select->offset($offset);
        }

        return $select;
    }

}