<?php

namespace System\Admin;

use Model\AdminMainActions;
use Model\AdminMainUsers;
use Model\AdminDictGroups;
use Model\AdminDictLGroups;
use Model\AdminMainAdminGroups;
use System\Admin\Product\AdminActionProduct;
use System\Admin\Product\AdminUserProduct;
use System\Admin\Product\AdminGroupProduct;
use System\Admin\Product\AdminLGroupProduct;
use System\Admin\Product\AdminAGroupProduct;

class AdminFactory {

    const PRODUCT_ACTION = 'product_action';
    const PRODUCT_USER = 'product_user';
    const PRODUCT_GROUP = 'product_group';
    const PRODUCT_LGROUP = 'product_lgroup';
    const PRODUCT_AGROUP = 'product_agroup';

    protected static $EMPTY_PRODUCTSET = array(array(), 0);
    protected static $productModelMap = array(
        self::PRODUCT_ACTION => 'Model\AdminMainActions',
        self::PRODUCT_USER => 'Model\AdminMainUsers',
        self::PRODUCT_GROUP => 'Model\AdminDictGroups',
        self::PRODUCT_LGROUP => 'Model\AdminDictLGroups',
        self::PRODUCT_AGROUP => 'Model\AdminMainAdminGroups',
    );
    protected static $productProductMap = array(
        self::PRODUCT_ACTION => 'System\Admin\Product\AdminActionProduct',
        self::PRODUCT_USER => 'System\Admin\Product\AdminUserProduct',
        self::PRODUCT_GROUP => 'System\Admin\Product\AdminGroupProduct',
        self::PRODUCT_LGROUP => 'System\Admin\Product\AdminLGroupProduct',
        self::PRODUCT_AGROUP => 'System\Admin\Product\AdminAGroupProduct',
    );
    protected static $productIdColumnMap = array(
        self::PRODUCT_ACTION => 'action_id',
        self::PRODUCT_USER => 'user_id',
        self::PRODUCT_GROUP => 'group_id',
        self::PRODUCT_LGROUP => 'group_id',
        self::PRODUCT_AGROUP => 'group_code',
    );

    public static function produce($productType, array $queryConfig = array()) {
        try {
            if (!class_exists(self::$productModelMap[$productType])) {
                return self::$EMPTY_PRODUCTSET;
            }

            $productModel = new self::$productModelMap[$productType]();
            list($productsData, $count) = $productModel->getRecords($queryConfig);

            if ($count < 1) {
                return self::$EMPTY_PRODUCTSET;
            }

            $products = array();
            foreach ($productsData as $productData) {
                $products[$productData[self::$productIdColumnMap[$productType]]] = new self::$productProductMap[$productType]($productData);
            }

            return array($products, $count);
        } catch (\Exception $Exception) {
            throw new \Exception(__METHOD__ . '. ' . $Exception->getMessage());
        }
    }

}

?>
