<?php

namespace System\Authentication;
use Zend\Authentication\Adapter\AdapterInterface;
use System\Soap\Client;
use Zend\Authentication\Result;

class AuthenticationAdapter implements AdapterInterface {
	
	protected $_client;
	protected $_siteCode;
	protected $_userCode;
	protected $_authRequire;
	protected $_logRequest;
	
	
	public function __construct($siteCode,$userCode,$authRequire = true,$logRequest = true,$lang = null , $wsdl = null , $eloginUrl = null ) {
		
		
		$this->_userCode = $userCode;
		$this->_siteCode = $siteCode;
		$this->_authRequire = $authRequire;
		$this->_logRequest = $logRequest;
		
		
		$this->_client = new Client(
			$this->_siteCode,
			$this->_userCode, 
			$lang, 
			$wsdl,
			$eloginUrl
		);
		
	}
	
	
	public function getReturnUri() {
	
		return $this->requestUri;
	}
	
	
	public function logRequire( $logRequire ) {
	
		$this->_logRequest = (bool)$logRequire;
	}
	
	public function authenticate() {
		
		$result = 0;
		$getIdentity = array('isLogged' => 0);
		
		$client = $this->_client;
		
		$client->authRequire($this->_authRequire);
		
		// sprawdzam czy uytkownik jest zalogowany
		
		if( $client->isLogged() ) {
			
			$result = 1;
			$identity = $client->getUserData();
			$identity['user_code'] = $client->getUserCode();
			$identity['isLogged'] = 1;
			
			// ustalanie specializacji 
			
			
			if( !is_array( $identity[ 'specs' ] ) ) 
				$identity[ 'specs' ] = array();
			
			
			if( !count( $identity[ 'specs' ] ) )
					$identity[ 'specs' ] = array( 'MAIN' => array( '01' ) );
					
			
			$orderSpec = array();
			
			foreach( $identity[ 'specs' ] as $specType => $specs ) {
				if( !is_array( $specs ) ) 
					continue;
					
				$specType = preg_replace(
					array( '/MAIN/' , '/CURR/' , '/ADDIT/' , '/INTE/' ),
					array( 2 , 3 , 4 , 5 ),
					$specType
				);
				
				foreach( $specs as $spec ) {
					
					$orderSpec[ $specType ][] = $spec;
				}
				
			}
			
			
			$identity[ 'full_spec_code' ] = $orderSpec;
			
			
			if(isset($identity['specs']['MAIN']) && is_array($identity['specs']['MAIN']) && count($identity['specs']['MAIN']) > 0) {
			
				$identity['spec_code'] = $identity['specs']['MAIN'];
				
				if(isset($identity['specs']['ADDIT']) && is_array($identity['specs']['ADDIT']) && count($identity['specs']['ADDIT']) >= 1) {
					$identity['spec_code'] = array_merge( $identity['spec_code'] , $identity[ 'specs' ][ 'ADDIT' ] );
				}
				
			} elseif(isset($identity['specs']) && isset($identity['specs']['INTE']) && count($identity['specs']['INTE'])) {
				
				$identity['spec_code'] = $identity['specs']['INTE'];
				
			}
		}
		
		return new Result($result,$identity);
	}
	
	public function hasIdentity($authRequire = false , $backUrl = null) {
		$client = $this->_client;
		
		// zalogowanie odwiedzin klienta
		if($this->_logRequest) $client->logVisit();
		
		
		$client->authRequire( $authRequire );
		
		
		if( (bool)$client->isLogged( $backUrl ) )
			return $client->getUserCode();
		return false;
	}
	
	public function logout() {
		
		if($this->_userCode && $this->_siteCode) {
			
			$this->_client->authRequire(false);
			
			if($this->_client->isLogged()) {
				return $this->_client->logout();
			}
		}
		
		return false;
	}
}	

?>