<?php

namespace System\Authentication;

use Model\CmsMainUsers,
	System\Observable;

class CmsAuthentication extends Observable {
	
	const CODE_REFRESH = 100;
	
	protected $userlogin;
	protected $userId;
	protected $authenticated = false;
	protected $sessionExpiration;
	protected $dbHandle = null;
	protected $data = array();
	protected $logged = false;
	protected $sessionObj;
	
	
	public function __construct() {
		
		$sess = new \Zend\Session\Container( 'auth_code' );
		$this->sessionObj = $sess;
	}
	
	
	public function dbHandle() {
		
		if( null == $this->dbHandle ) {
			$this->dbHandle = new \Model\cmsMainUsersSession;
		}
		
		return $this->dbHandle;
	}
	
	
	/**
	 * Sprawdzam czy user jest zautentykowany i ewentualnie doliczam mu czas ważności sesji
	 **/
	public function isAuthenticated( $authLifetime = 1800 ) {
		$this->sessionExpiration = (int)$authLifetime;
		
		$code = $this->getCode();
		
		if( false !== ( $data = $this->dbHandle()->getSession( $code , $this->sessionExpiration ) ) ) {
			$this->logger = true;
			
			$refresh = $data[ 'refresh' ];
			//if( $refresh > 0 && $refresh % self::CODE_REFRESH == 0 ) {
			
				// drastyczne rozwiązanie w razie niudanej regeneracji kodu - logout
				//if( !$this->regenerateCode() ) {
					///$this->logout();
				//}
			//}
			
			$this->data = unserialize( $data[ 'data' ] );
			
			return true;
		}
		
		return false;
	}
	
	
	public function login( $login , $password ) {
	
		$authResult;
		
		try {
			
			if( false == ( $authResult = $this->dbHandle()->startSession( $login , $password , $this->sessionExpiration ) ) ) {
			
				$this->notify( 'login_fila' , array( 'login' => $login , 'password' => $password ) );
				
				return false;
			}
			
			
			$this->setCode( $authResult[ 'code' ] );
			
			return true;
			
		} catch( \Exception $e ) {
		
			exit( __METHOD__ . ' ' . $e->getMessage() );
			
			return false;
		}
	}
	
	
	public function logout() {
		
		unset( $this->sessionObj->code );
		//setcookie( 'a_code' ,  null , 1 , '/' , '.esculap.com' );
		
		return $this->dbHandle()->endSession( $this->getCode() );
	}
	
	
	public function getCode() {
		
		if( isset( $this->sessionObj->code ) )
			return $this->sessionObj->code;
		return  null;
	}
	
	
	public function setCode( $code ) {
		
		$this->sessionObj->code = $code;
	}
	
	
	public function getData() {
		
		return $this->data;
	}
	
	
	public function destroySession() {
		
		unset( $_SESSION['auth'] );
	}
	
	
	protected function regenerateCode() {
		
		if( $this->logger ) {
			
			if( false == ( $code = $this->dbHandle()->regenerateCode( $this->getCode() ) ) ) return false;
			
			$this->setCode( $code );
			
			return true;
		}
		
		return false;
	}
}

?>