<?php

namespace System\BannerCampaign;

class BannerCampaign
{
    protected $change = false;
    protected $id = null;
    protected $name = null;
    protected $client = null;
    protected $creator = null;
    protected $created = null;
    protected $elements = array();

    public function __construct($id = null, $name = null, \System\Client\Client $client = null, Supervisor\Creator $creator = null, $created = null)
    {
        $this->id = (int)$id;
        $this->name = $name;
        $this->client = $client;
        $this->creator = $creator;
        $this->created = $created;
    }

    /**
     * Zwraca id kampanii
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Zwraca nazwę kampanii
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Ustaiwam nazwę
     */
    public function setName($name)
    {
        $this->setChange();
        $this->name = $name;
    }

    /**
     * Zwraca id clienta
     */
    public function getClient()
    {
        return $this->client;
    }

    /**
     * Ustaiwa Id klienta
     */
    public function setClient(\System\Client\Client $client)
    {
        $this->client = (int)$client;
    }

    /**
     * Zwraca id osoby dodającej kampanię z tabeli cms.main_users
     */
    public function getCreator()
    {
        return $this->creator;
    }

    /**
     * Ustawia osobę dodającą jeśli niema jeszcze id
     */
    public function setCreator(Supervisor\Creator $creator)
    {
        $this->creator = (int)$creator;
    }

    /**
     * Dodanie elementu do kampanii
     */
    public function addElement(Element\ElementAbstract $element)
    {
        $element->setParent($this);
        $this->elements[] = $element;
    }

    /**
     * Dodanie listy elementów. Adapter dla addElement
     */
    public function addElements(array $elements)
    {
        foreach ($elements as $element) {
            $this->addElement($element);
        }
    }

    /**
     * Pobranie daty utworzenia
     */
    public function getCreatedDate()
    {
        return date('d-m-Y', strtotime($this->created));
    }

    public function clearElements()
    {
        $this->elements = array();
    }

    public function isCollision($bannerCampaign, &$collisionsList)
    {
        if (is_array($bannerCampaign)) {
            $res = false;
            foreach ($bannerCampaign as $campaign) {
                if ($this->isCollision($campaign, $collisionsList)) {
                    $res = true;
                }
            }

            return $res;
        } else {
            $res = false;
            /** @var \System\BannerCampaign\BannerCampaign $bannerCampaign */
            foreach ($this->getElements() as $element) {
                /** @var Element\ElementAbstract $element */
                foreach ($bannerCampaign->getElements() as $testElement) {
                    /** @var Element\ElementAbstract $testElement */
                    if ($element->isCollision($testElement, $collisionsList)) {
                        $res = true;
                    }
                }
            }

            return $res;
        }
    }

    /**
     * Zapis/zmiana elementów
     */
    public function saveElements()
    {
        foreach ($this->elements as $element) {
            $element->save();
        }
    }

    /**
     * Zwraca listę elementów
     */
    public function getElements()
    {

        return $this->elements;
    }


}

?>