<?php

namespace System\BannerCampaign;

use Model\MainBc;
use System\Client\Client;

class CampaignFactory
{
    CONST FULL_CREATOR = 'full_creator';
    CONST LOAD_ELEMENTS = 'load_elements';
    CONST LOAD_ELEMENTS_TARGET = 'load_elements_target';
    CONST LOAD_ELEMENTS_STATISTICS = 'load_elements_statistics';

    /**
     * @param integer $campaignId
     * @param array   $configArr
     * @throws \Exception
     * @return BannerCampaign
     */
    public static function one($campaignId, array $configArr = array())
    {
        try {
            $MainBc = new MainBc();
            $bc = $MainBc->getCampaign($campaignId, $configArr);

            if (!isset($bc[0])) {
                throw new \Exception('Brak kampanii id="' . $campaignId . '"');
            } else {
                $bc = $bc[0];
            }

            $Bc = new BannerCampaign(
                $bc['id'],
                $bc['name'],
                new Client(
                    $bc['client'],
                    $bc['client_name']
                ),
                new Supervisor\Creator(
                    $bc['creator'],
                    @$bc['creator_name'],
                    @$bc['creator_lastname'],
                    @$bc['creator_login']
                ),
                $bc['created']
            );

            if (in_array(self::LOAD_ELEMENTS, $configArr)) {
                $elements = self::getElements($Bc->getId(), $configArr);
                foreach ($elements as $elementList) {
                    $Bc->addElements($elementList);
                }
            }

            return $Bc;
        } catch (\Exception $exception) {
            throw new \Exception(__METHOD__ . '. ' . $exception->getMessage());
        }
    }

    /**
     * @param array $configArr
     * @param null  $count
     * @throws \Exception
     * @return array
     */
    public static function all(array $configArr = array(), &$count = null)
    {

        try {

            $MainBc = new MainBc;

            if (!in_array(self::FULL_CREATOR, $configArr)) {
                $configArr[] = self::FULL_CREATOR;
            }

            $bc = $MainBc->getCampaign(null, $configArr, $count);

            if (count($bc) < 1) {

                return array();
                //throw new \Exception( 'Brak kampanii id="' . $campaignId . '"' );
            }

            $campaigns = array();
            $idList = array();
            foreach ($bc as $campaignData) {

                $idList[] = $campaignData['id'];

                $campaigns[$campaignData['id']] = new BannerCampaign(
                    $campaignData['id'],
                    $campaignData['name'],
                    new Client(
                        $campaignData['client'],
                        $campaignData['client_name']
                    ),
                    new Supervisor\Creator(
                        $campaignData['creator'],
                        $campaignData['creator_name'],
                        $campaignData['creator_lastname'],
                        $campaignData['creator_login']
                    ),
                    $campaignData['created']
                );
            }


            if (in_array(self::LOAD_ELEMENTS, $configArr)) {
                $elements = self::getElements($idList, $configArr);

                foreach ($elements as $campaignId => $campaignElements) {

                    $campaigns[$campaignId]->addElements($campaignElements);
                }
            }

            return $campaigns;

        } catch (\Exception $e) {

            throw new \Exception(__METHOD__ . '. ' . $e->getMessage());
        }
    }


    protected static function getElements($campaignId, $cfgArr)
    {

        if (!is_array($campaignId)) {
            $campaignId = array((int)$campaignId);
        }


        $elements = Element\ElementFactory::all($campaignId, $cfgArr);


        $result = array_flip($campaignId);

        foreach ($result as $k => $v) {

            $result[$k] = array();

        }
        foreach ($elements as $element) {

            $result[$element->getBcId()][] = $element;

        }

        return $result;
    }

}

?>