<?php

namespace System\BannerCampaign;
use System\Content\ContentFactory;
use System\Std\ObserverInterface;

class Collector {
	
	protected static $instance;
	protected static $observers = array();
	protected static $sUserLogin = null;
	
	
	protected function __construct() {}
	
	
	protected static function attach(ObserverInterface $obj) {
		self::$observers[] = $obj;
	}
	
	
	protected static function detach($className) {}
	
	
	protected static function notify($name,$data) {
		foreach(self::$observers as $observer) {
			$observer->notify($name,$data);
		}
	}
	
	
	/**
	* Szukam możliwych kampanii dla podanych specializacji dla podanych miejsc
	*/ 
	public static function init($specs,$places,$sUserLogin) {
		self::attach(new BannerObserver);
		self::$sUserLogin = $sUserLogin;
		
		$matches = array();
		
		$MainBannerCampaign = new \Model\MainBannerCampaign;
		
		$campaigns = $MainBannerCampaign->getCampaigns(
			$specs,
			$places,
			time()
		);
		
		foreach($campaigns as $campaignType => $campaignData) {
			
			$matches[$campaignType] = self::process($campaignType,$campaignData);
		}
		
		return $matches;
	}
	
	public static function process($type,$data) {
		switch($type) {
			case 'mainBanner';
				// baner główny może być tylko jeden więc jeśli jeest więcej niż jeden sprawdzam, który ma najmniej odsłon i go wybieram
				
				if(count($data) > 1) {
					$min = null;
					$index = null;
					
					foreach($data as $key => $row) {
						if(null === $min) { 
							$min = $row['views']; 
							$index = $key; 
							continue;
						}
						
						if($row['views'] < $min) {
							$min = $row['views'];
							$index = $key;
						}
					}
					
					$data = $data[$index];
				} else {
					$data = current($data);
				}
				
				self::notify('banner_view',array('id' => $data['mbtc_id'], 'ulogin' => self::$sUserLogin));
				
				return array(
					'bgUrl' => $data['background_url'],
					'bannerUrl' => $data['billboard_url'],
					'dst' => $data['billboard_dst'],
					'cfgId' => $data['mbtc_id']
				);
				
				break;
			case 'sponsored':
				$max = 5;
				$res = array();
				
				if(count($data) > $max) {
					// sprawdzam, które z contentów mają najmniej wyświetleń i wybieram je
					
					$aTmpData = array();
					for($i = 0;$i < $max; $i++) {
						$aTmpData[] = $data[$i];
					}
					
					$data = $aTmpData;
				}
				
				$idList = array();
				$configsList = array();
				
				$connector = array(); // tablica umożliwiająca połączenie i dodanie danych do przetworzonych obiektów publication
				
				foreach($data as $row) {	
					self::notify('banner_view',array('id' => $row['mbtc_id'], 'ulogin' => self::$sUserLogin));
					$idList[] = $row['content_id'];
					$connector[$row['content_id']] = $row['mbtc_id'];
				}
				
				$publ = ContentFactory::factory(new \System\Content\Config(array(
					'CONTENT_TYPE' => 'publication',
					'SPEC_TYPES' => array(),
					'ROWS_NUMBER' => $max,
					'ROWS_START' => 0,
					'ID_LIST' => $idList,
                                        'GET_MAIN_SITE' => true,
				)));
				
				// przypisanie mbtc_id do obiektów publication
				foreach($publ as $key => $obj) {
					$obj->cfgId = $connector[$obj->id];
					
					$publ[$key] = $obj;
				}
				
				return $publ;
				
				break;
			case 'slider':
				$max = 4;
				$res = array();
				
				$connector = array(); // tablica umożliwiająca połączenie i dodanie danych do przetworzonych obiektów publication
				$idList = array();
				
				
				if(count($data) > $max) {
					// sprawdzam, które z contentów mają najmniej wyświetleń i wybieram je
					
					$aTmpData = array();
					for($i = 0;$i < $max; $i++) {
						$aTmpData[] = $data[$i];
					}
					
					$data = $aTmpData;
				}
				
				
				foreach($data as $row) {
					self::notify('banner_view',array('id' => $row['mbtc_id'], 'ulogin' => self::$sUserLogin));
					$idList[] = $row['content_id'];
					$connector[$row['content_id']] = $row['mbtc_id'];
				}
				
				$news = ContentFactory::factory(new \System\Content\Config(array(
					'CONTENT_TYPE' => 'news',
					'SPEC_TYPES' => array(),
					'ROWS_NUMBER' => $max,
					'ROWS_START' => 0,
					'ID_LIST' => $idList,
                                        'GET_MAIN_SITE' => true,
				)));
				
				// przypisanie mbtc_id do obiektów publication
				foreach($news as $key => $obj) {
					$obj->cfgId = $connector[$obj->id];
					
					$news[$key] = $obj;
				}
				
				return $news;
				
				break;
			case 'rightSide':
				$max = 1;
				
				if(count($data) > 1) {
					$min = null;
					$index = null;
					
					foreach($data as $key => $row) {
						if(null === $min) { 
							$min = $row['views']; 
							$index = $key; 
							continue;
						}
						
						if($row['views'] < $min) {
							$min = $row['views'];
							$index = $key;
						}
					}
					
					$data = $data[$index];
					
				} else {
					$data = current($data);
				}
				
				self::notify('banner_view',array('id' => $data['mbtc_id'], 'ulogin' => self::$sUserLogin));
				
				return array(
					'bannerUrl' => $data['banner_url'],
					'dst' => $data['banner_dst'],
					'cfgId' => $data['mbtc_id']
				);
				
				
				return $data;
				
				break;
		}	
	}
}

?>