<?php

namespace System\BannerCampaign;

use System\BannerCampaign\Element\ElementAbstract;

class Collision
{
    protected $collisions = [];
    protected $dayRangeCollisions = [];
    protected $dayRangePrepared = false;

    public function appendCollision(ElementAbstract $element, $from, $to, $conflict)
    {
        if (empty($this->collisions[$element->getElementId()])) {
            $this->collisions[$element->getElementId()] = [];
        }

        if (isset($conflict['type'])) {
            $conflict = array($conflict);
        }

        $this->collisions[$element->getElementId()][] = [
            'element'  => $element,
            'from'     => $from,
            'to'       => $to,
            'conflict' => $conflict
        ];
    }

    public function getCollisions()
    {
        return $this->collisions;
    }

    public function getCollision($day = null)
    {
        if (null == $day) {
            return $this->collisions;
        }

        if (!$this->dayRangePrepared) {
            $this->prepareDayRangeCollisions();
        }

        if (isset($this->dayRangeCollisions[$day])) {
            return $this->dayRangeCollisions[$day];
        }

        return [];
    }

    public function prepareDayRangeCollisions()
    {
        foreach ($this->collisions as $elementCollisions) {
            foreach ($elementCollisions as $collision) {
                $from = strtotime('midnight', $collision['from']);
                $to = strtotime('midnight', $collision['to']);

                $fuse = 0;
                while ($from <= $to && $fuse < 1000) {
                    if (empty($this->dayRangeCollisions[$from])) {
                        $this->dayRangeCollisions[$from] = array();
                    }

                    $this->dayRangeCollisions[$from][] = array(
                        'element' => $collision['element'],
                        'from'    => $collision['from'],
                        'to'      => $collision['to']
                    );

                    $from = strtotime('+1 day midnight', $from);
                    $fuse++;
                }
            }
        }

        $this->dayRangePrepared = true;
    }

    public function getDayRangePrepared()
    {
        return $this->dayRangeCollisions;
    }

    public function getCollisionsAmountByDay()
    {
        $tempArray = [];

        foreach ($this->collisions as $elementCollisions) {
            foreach ($elementCollisions as $collision) {
                $from = strtotime('midnight', $collision['from']);
                $to = strtotime('midnight', $collision['to']);

                while ($from <= $to) {
                    if (empty($tempArray[$from])) {
                        $tempArray[$from]['specType'] = 0;
                        $tempArray[$from]['unlogged'] = 0;
                    }

                    $onlyOneIncrement = true;
                    foreach ($collision['conflict'] as $conflict) {
                        if ($conflict['type'] == 3) {
                            $tempArray[$from]['unlogged']++;
                        } elseif ($conflict['type'] != 3) {
                            if ($onlyOneIncrement) {
                                $tempArray[$from]['specType']++;
                                $onlyOneIncrement = false;
                            }
                        }
                    }

                    $from = strtotime('+1 day midnight', $from);
                }
            }
        }

        $result = [];
        foreach ($tempArray as $from => $collisionInDay) {
            $collisionAmount = 0;
            if ($collisionInDay['unlogged'] > $collisionInDay['specType']) {
                $collisionAmount = $collisionInDay['unlogged'];
            } else {
                $collisionAmount = $collisionInDay['specType'];
            }

            $result[$from] = $collisionAmount;
        }

        return $result;
    }
}