<?php
/**
 * W podklasach należy zdefiniować metodę toArray zwracającą listę pól i ich wartości oraz wywołującą metodę toArray rodzica i połączyś tablice.
 *
 */
namespace System\BannerCampaign\Element;

abstract class ElementAbstract
{

    abstract public function setTypeData($jsonData);

    abstract public function getTypeData();

    protected $functionsToCompare = array();
    protected $id;
    protected $bcId;
    protected $elementId;
    protected $changed = false;
    protected $elementTypeId;
    protected $elementTypeName;
    protected $starts;
    protected $ends;
    protected $target;
    protected $place;
    protected $view;
    protected $hits;
    protected $uniqueViews;
    protected $uniqueHits;
    protected $parent;
    protected $creator;
    protected $status;
    protected $mainSiteActive = false;
    protected $additionalStats;
    // true jeśli element jest fejkowy :p 
    protected $fake = false;

    /**
     * Zawiera nazwy pol specyficznych dla konkretnego typu lementu, które jeśli są wypełnione odpowiednia funkcja sygnalizuje gotowość to publikacji elementu (przy updacie elementu o statusie 2 - rezerwacja, sprawdzam czywypełniono wszystkie elementy, jeśli tak to zmieniam status na 1 - aktywny);
     */
    protected $fullFilled = array();

    const STATUS_ACTIVE = 1;
    const STATUS_RESERVED = 2;
    const STATUS_LOCKED = 3;
    const STATUS_DISABLED = 4;

    protected $statuses = array(
        self::STATUS_ACTIVE   => 'active',
        self::STATUS_RESERVED => 'reserved',
        self::STATUS_LOCKED   => 'locked',
        self::STATUS_DISABLED => 'disabled'
    );
    // czy został wyłączony przez użytkownika
    protected $deactivated;
    // czy jest aktualnie aktywny
    protected $active;
    protected $name = '';

    public function __construct($data = array())
    {

        $this->parseData($data);
    }

    public function setFake($state)
    {

        $this->fake = (bool)$state;
    }

    public function isFake()
    {

        return (bool)$this->fake;
    }

    public function parseData($data)
    {
        // tu niewolno przypisywać wartości domyślnych jeśli jakieś pole jest niezdefiniowane ponieważ funkcja może być wywoływana dla zainicializowanych obiektów w celu uzupełnienia ich danych nieznanym zestawem.


        if (isset($data['deactivated'])) {
            $this->deactivated = (bool)$data['deactivated'];
        }
        if (isset($data['views'])) {
            $this->views = $data['views'];
        }
        if (isset($data['clicks'])) {
            $this->hits = $data['clicks'];
        }
        if (isset($data['unique_views'])) {
            $this->uniqueViews = $data['unique_views'];
        }
        if (isset($data['unique_clicks'])) {
            $this->uniqueHits = $data['unique_clicks'];
        }
        if (isset($data['id'])) {
            $this->id = $data['id'];
        }
        if (isset($data['elementid'])) {
            $this->elementId = $data['elementid'];
        }
        if (isset($data['bcId'])) {
            $this->bcId = $data['bcId'];
        }
        if (isset($data['starts'])) {
            $this->setStarts($data['starts']);
        }
        if (isset($data['ends'])) {
            $this->setEnds($data['ends']);
        }
        if (isset($data['additional_stats'])) {
            $this->additionalStats = $data['additional_stats'];
        }
        if (isset($data['status'])) {

            $this->status = (int)$data['status'];
        }
        if (isset($data['targetData'])) {

            $this->setTarget($data['targetData']);
        }
        if (isset($data['creator'])) {

            $this->creator = new \System\BannerCampaign\Supervisor\Creator(
                $data['creator'], isset($data['creator_name']) ? $data['creator_name'] : null, isset($data['creator_lastname']) ? $data['creator_lastname'] : null
            );
        }
        if (isset($data['place'])) {

            $this->setPlace($data['place']);
        }
        if (isset($data['data'])) {

            $this->setTypeData($data['data']);
        }
        if (isset($data['fake'])) {

            $this->fake = (bool)$data['fake'];
        }

        if (isset($data['main_site_active'])) {
            $this->mainSiteActive = (bool)$data['main_site_active'];
        }
    }

    final public function isDataFilled()
    {

        // proste porównanie długości zawartości pól

        if (!isset($this->fullFilled)) {
            throw new \Exception(__METHOD__ . '. No fullFilled list!');
        }

        foreach ($this->fullFilled as $field) {

            if (!isset($this->$field) ||
                !strlen(trim($this->$field))
            ) {

                return false;
            }
        }

        return true;
    }

    public function isReservation()
    {

        if (isset($this->statuses[$this->status]) &&
            'reserved' == $this->statuses[$this->status]
        ) {
            return true;
        }

        return false;
    }

    public function isLocked()
    {

        if (isset($this->statuses[$this->status]) &&
            'locked' == $this->statuses[$this->status]
        ) {
            return true;
        }

        return false;
    }

    public function isStarted()
    {

        if (isset($this->statuses[$this->status]) &&
            'active' == $this->statuses[$this->status]
        ) {
            return true;
        }

        return false;
    }

    public function isDisabled()
    {

        if (isset($this->statuses[$this->status]) &&
            'disabled' == $this->statuses[$this->status]
        ) {
            return true;
        }

        return false;
    }

    public function isRunning()
    {

        if (time() > $this->getStarts() && time() < $this->getEnds()) {
            return true;
        }

        return false;
    }

    public function isMainSiteActive()
    {
        return (bool)$this->mainSiteActive;
    }

    public function compare(ElementAbstract $element)
    {

        $differences = array();

        $functions = array(
            'starts', 'ends', 'status', 'elementid'
        );

        $functions = array_merge($functions, $this->functionsToCompare);

        foreach ($functions as $function) {
            $field = $function;
            $function = 'get' . ucfirst($function);

            if (method_exists($this, $function)) {

                if ($this->$function() !== $element->$function()) {

                    $differences[$field] = array(
                        'old' => $element->$function(),
                        'new' => $this->$function()
                    );
                }
            }
        }

        return $differences;
    }

    public function setParent(\System\BannerCampaign\BannerCampaign &$campaign)
    {

        $this->parent = $campaign;
    }

    public function getStatus($translate = false)
    {

        if (false === $translate) {
            return $this->status;
        }

        if (array_key_exists($this->status, $this->statuses)) {

            return $this->statuses[$this->status];
        }

        return 'undefined';
    }

    public function getName()
    {
        return $this->name;
    }

    public function setStatus($status)
    {
        $this->status = (int)$status;
    }

    public function getParent()
    {
        return $this->parent;
    }

    public function getDeactivated()
    {
        return (bool)$this->deactivated;
    }

    public function setDeactivated($value)
    {
        $this->deactivated = (bool)$deactivated;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getBcId()
    {
        return $this->bcId;
    }

    public function markChange()
    {
        $this->changed = true;
    }

    public function getViews()
    {
        return (int)$this->views;
    }

    public function getHits()
    {
        return (int)$this->hits;
    }

    public function getStarts()
    {
        return $this->starts;
    }

    public function setStarts($starts)
    {

        if (-1 < strpos($starts, '-') || -1 < strpos($starts, '-')) {

            $this->starts = strtotime($starts);

            return;
        }

        $this->starts = (int)$starts;
    }

    public function getUniqueViews()
    {

        return $this->uniqueViews;
    }

    public function getUniqueHits()
    {

        return $this->uniqueHits;
    }

    public function getEnds()
    {
        return $this->ends;
    }

    public function setEnds($ends)
    {

        if (-1 < strpos($ends, '-') || -1 < strpos($ends, '-')) {

            $this->ends = strtotime($ends);

            return;
        }

        $this->ends = (int)$ends;
    }

    public function getCreator()
    {

        if (null == $this->creator) {

            throw new \Exception(__METHOD__ . '. creator nie został zainicializowany');
        }

        return $this->creator;
    }

    public function toArray()
    {

        $res = array(
            'starts'          => $this->getStarts(),
            'ends'            => $this->getEnds(),
            'id'              => $this->getId(),
            'elementId'       => $this->getElementId(),
            'status'          => $this->getStatus(),
            'campaign'        => $this->getParent(),
            'place'           => $this->getPlace()->getId(),
            'type'            => $this->getId(),
            'additionalStats' => $this->getAdditionalStats()
        );

        $res = array_merge(
            $res, $this->getTarget()->toArray()
        );

        return $res;
    }

    public function setAdditionalStats($additionalStats)
    {

        $this->additionalStats = $additionalStats;
    }

    public function getAdditionalStats()
    {

        return $this->additionalStats;
    }

    public function setCreator($creaotr)
    {

        if (is_subclass_of($creator, '\System\BannerCampaign\Supervisor\Creator')) {

            $this->creator = $creator;
        } elseif (is_numeric($creator)) {

            $this->creator = new \System\BannerCampaign\Supervisor\Creator($creator);
        }
    }

    public function getTarget()
    {

        if (null === $this->target) {

            throw new \Exception(__METHOD__ . '. target nie jest jeszcze zainicializowany.');
        }

        return $this->target;
    }

    public function setTarget($data)
    {

        if (!is_subclass_of($data, 'System\BannerCampaign\Element\Target\TargetAbstract')) {
            try {

                $data = \System\BannerCampaign\Element\Target\TargetFactory::createElement($data);
            } catch (\Exception $e) {

                throw new \Exception(__METHOD__ . '. ' . $e->getMessage());
            }
        }


        if (null !== $this->target) {

            $this->target = $this->target->setTarget($data);
        } else {

            $this->target = $data;
        }
    }

    public function getPlace()
    {

        if (null == $this->place) {

            throw new \Exception(__METHOD__ . '. Element miejsca baneru nie został jeszcze zainicializowany');
        }

        return $this->place;
    }

    public function setPlace($id)
    {

        if (is_subclass_of($id, '\System\BannerCampaign\Element\Place\PlaceAbstract')) {

            $this->place = $id;
        } elseif (is_numeric($id)) {

            $this->place = \System\BannerCampaign\Element\Place\PlaceFactory::createPlace($id);
        }
    }

    public function getElementId()
    {
        return $this->elementId;
    }

    public function fill($data)
    {
        $this->setStarts($data['starts']);
        $this->setEnds($data['ends']);
        $this->setTarget($data['targteId'], $data['target']);
    }

    /**
     * Sprawdza czy występuje kolizja
     * @param ElementAbstract                            $element           testowany element
     * @param \System\BannerCampaign\Collision|Collision $conflictingRanges zakres kolizii
     * @throws \Exception
     * @return bool
     */
    public function isCollision(ElementAbstract $element, \System\BannerCampaign\Collision &$conflictingRanges = null)
    {
        if ($element->isDisabled()) {
            return false;
        }

        if ($element->getPlace()->getId() == $this->getPlace()->getId()) { // sprawdzam czy miejsce jest takie samo
            $match = false;
            $thisTarget = $this->getTarget();
            $elementTarget = $element->getTarget();

            /* Poprawka 20.03.2014 */
            /* Rozbicie targetu na typ/spec i unlogged */
            $thisTargetArray = array();
            $elementTargetArray = array();

            if (is_a($thisTarget, '\System\BannerCampaign\Element\Target\CompositeTarget')) {
                $specTypeCompositeTarget = new \System\BannerCampaign\Element\Target\CompositeTarget();
                $unloggedCompositeTarget = new \System\BannerCampaign\Element\Target\CompositeTarget();

                $specTypeOccurred = false;
                $unloggedOccurred = false;

                foreach ($thisTarget->getTarget() as $target) {
                    if (is_a($target, '\System\BannerCampaign\Element\Target\BySpecialization') || is_a($target, '\System\BannerCampaign\Element\Target\ByUserType')) {
                        $specTypeCompositeTarget->setTarget($target);
                        $specTypeOccurred = true;
                    }
                    if (is_a($target, '\System\BannerCampaign\Element\Target\Unlogged')) {
                        $unloggedCompositeTarget->setTarget($target);
                        $unloggedOccurred = true;
                    }
                }

                if ($specTypeOccurred) {
                    if (count($specTypeCompositeTarget->getTarget()) == 1) {
                        $thisTargetArray[] = $specTypeCompositeTarget->getTarget()[0];
                    } else {
                        $thisTargetArray[] = $specTypeCompositeTarget;
                    }
                }

                if ($unloggedOccurred) {
                    $thisTargetArray[] = $unloggedCompositeTarget;
                }
            } else {
                $thisTargetArray[] = $thisTarget;
            }

            if (is_a($elementTarget, '\System\BannerCampaign\Element\Target\CompositeTarget')) {
                $specTypeCompositeTarget = new \System\BannerCampaign\Element\Target\CompositeTarget();
                $unloggedCompositeTarget = new \System\BannerCampaign\Element\Target\CompositeTarget();

                $specTypeOccurred = false;
                $unloggedOccurred = false;

                foreach ($elementTarget->getTarget() as $target) {
                    if (is_a($target, '\System\BannerCampaign\Element\Target\BySpecialization') || is_a($target, '\System\BannerCampaign\Element\Target\ByUserType')) {
                        $specTypeCompositeTarget->setTarget($target);
                        $specTypeOccurred = true;
                    }
                    if (is_a($target, '\System\BannerCampaign\Element\Target\Unlogged')) {
                        $unloggedCompositeTarget->setTarget($target);
                        $unloggedOccurred = true;
                    }
                }

                if ($specTypeOccurred) {
                    if (count($specTypeCompositeTarget->getTarget()) == 1) {
                        $elementTargetArray[] = $specTypeCompositeTarget->getTarget()[0];
                    } else {
                        $elementTargetArray[] = $specTypeCompositeTarget;
                    }
                }

                if ($unloggedOccurred) {
                    $elementTargetArray[] = $unloggedCompositeTarget;
                }
            } else {
                $elementTargetArray[] = $elementTarget;
            }

            foreach ($thisTargetArray as $thisTarget) {
                foreach ($elementTargetArray as $elementTarget) {
                    /* Poprawka 18.03.2014 */
                    /* Dopełnienie targetów */
                    if (!is_a($thisTarget, '\System\BannerCampaign\Element\Target\CompositeTarget')) {
                        if (is_a($thisTarget, '\System\BannerCampaign\Element\Target\BySpecialization')) {
                            $userTypeTarget = new \System\BannerCampaign\Element\Target\ByUserType(2, array('*'));
                            $compositeTarget = new \System\BannerCampaign\Element\Target\CompositeTarget();
                            $compositeTarget->setTarget($thisTarget);
                            $compositeTarget->setTarget($userTypeTarget);
                        }

                        if (is_a($thisTarget, '\System\BannerCampaign\Element\Target\ByUserType')) {
                            $specTarget = new \System\BannerCampaign\Element\Target\BySpecialization(1, array('*'));
                            $compositeTarget = new \System\BannerCampaign\Element\Target\CompositeTarget();
                            $compositeTarget->setTarget($specTarget);
                            $compositeTarget->setTarget($thisTarget);
                        }

                        if (is_a($thisTarget, '\System\BannerCampaign\Element\Target\Unlogged')) {
                            $compositeTarget = new \System\BannerCampaign\Element\Target\CompositeTarget();
                            $compositeTarget->setTarget($thisTarget);
                        }

                        $thisTarget = $compositeTarget;
                    }

                    if (!is_a($elementTarget, '\System\BannerCampaign\Element\Target\CompositeTarget')) {
                        if (is_a($elementTarget, '\System\BannerCampaign\Element\Target\BySpecialization')) {
                            $userTypeTarget = new \System\BannerCampaign\Element\Target\ByUserType(2, array('*'));
                            $compositeTarget = new \System\BannerCampaign\Element\Target\CompositeTarget();
                            $compositeTarget->setTarget($elementTarget);
                            $compositeTarget->setTarget($userTypeTarget);
                        }

                        if (is_a($elementTarget, '\System\BannerCampaign\Element\Target\ByUserType')) {
                            $specTarget = new \System\BannerCampaign\Element\Target\BySpecialization(1, array('*'));
                            $compositeTarget = new \System\BannerCampaign\Element\Target\CompositeTarget();
                            $compositeTarget->setTarget($specTarget);
                            $compositeTarget->setTarget($elementTarget);
                        }

                        if (is_a($elementTarget, '\System\BannerCampaign\Element\Target\Unlogged')) {
                            $compositeTarget = new \System\BannerCampaign\Element\Target\CompositeTarget();
                            $compositeTarget->setTarget($elementTarget);
                        }

                        $elementTarget = $compositeTarget;
                    }

                    $match = false;
                    $conflicts = $thisTarget->isCollisionEx($elementTarget, $match);

                    if (($conflicts !== false && $match == true)) {
                        /* Pierwszy przypadek - element koliduje z lewej strony z this */
                        if ($element->getStarts() <= $this->getStarts() && $element->getEnds() >= $this->getStarts() && $element->getEnds() <= $this->getEnds()) {
                            $conflictingRanges->appendCollision($element, $this->getStarts(), $element->getEnds(), $conflicts);

                            return true;
                        } else /* Drugi przypadek - element zawiera w sobie this */
                            if ($element->getStarts() <= $this->getStarts() && $element->getEnds() >= $this->getEnds()) {
                                $conflictingRanges->appendCollision($element, $this->getStarts(), $this->getEnds(), $conflicts);

                                return true;
                            } else /* Trzeci przypadek - element zawiera się w this */
                                if ($element->getStarts() >= $this->getStarts() && $element->getStarts() <= $this->getEnds() && $element->getEnds() <= $this->getEnds()) {
                                    $conflictingRanges->appendCollision($element, $element->getStarts(), $element->getEnds(), $conflicts);

                                    return true;
                                } else /* Czwarty przypadek - element koliduje z prawej strony z this */
                                    if ($element->getStarts() >= $this->getStarts() && $element->getStarts() <= $this->getEnds() && $element->getEnds() >= $this->getEnds()) {
                                        $conflictingRanges->appendCollision($element, $element->getStarts(), $this->getEnds(), $conflicts);

                                        return true;
                                    }

                        return false;
                    }
                }
            }

            return false;
        }

        return false;
    }

    public function isTargetMatch($dataToCompare, &$match = null)
    {
        if (is_array($dataToCompare)) {

            $bySpecializationMatch = false;
            $byUserTypeMatch = false;
            $unloggedMatch = false;

            $bySpecializationUsed = false;
            $byUserTypeUsed = false;
            $unloggedUsed = false;

            if (is_a($this->getTarget(), '\System\BannerCampaign\Element\Target\CompositeTarget')) {
                foreach ($this->getTarget()->getTarget() as $target) {
                    if (is_a($target, 'System\BannerCampaign\Element\Target\BySpecialization')) {
                        $bySpecializationUsed = true;
                    }
                    if (is_a($target, 'System\BannerCampaign\Element\Target\ByUserType')) {
                        $byUserTypeUsed = true;
                    }
                    if (is_a($target, 'System\BannerCampaign\Element\Target\Unlogged')) {
                        $unloggedUsed = true;
                    }
                }
            } else {
                if (is_a($this->getTarget(), 'System\BannerCampaign\Element\Target\BySpecialization')) {
                    $bySpecializationUsed = true;
                }
                if (is_a($this->getTarget(), 'System\BannerCampaign\Element\Target\ByUserType')) {
                    $byUserTypeUsed = true;
                }
                if (is_a($this->getTarget(), 'System\BannerCampaign\Element\Target\Unlogged')) {
                    $unloggedUsed = true;
                }
            }

            foreach ($dataToCompare as $data) {
                if ($this->isTargetMatch($data, $match)) {
                    if (is_a($data, 'System\BannerCampaign\Element\Target\BySpecialization')) {
                        $bySpecializationMatch = true;
                    }
                    if (is_a($data, 'System\BannerCampaign\Element\Target\ByUserType')) {
                        $byUserTypeMatch = true;
                    }
                    if (is_a($data, 'System\BannerCampaign\Element\Target\Unlogged')) {
                        $unloggedMatch = true;
                    }
                }
            }

            if ($unloggedUsed && $unloggedMatch) {
                return true;
            }

            if ($bySpecializationUsed && $byUserTypeUsed) {
                if ($byUserTypeMatch && $bySpecializationMatch) {
                    return true;
                }
            } elseif (($bySpecializationUsed && $bySpecializationMatch) || ($byUserTypeUsed && $byUserTypeMatch)) {
                return true;
            }

            return false;
        } else {
            if (!is_a($dataToCompare, 'System\BannerCampaign\Element\Target\TargetAbstract')) {
                throw new \Exception(__METHOD__ . '. Podano podano nieprawidłowy argument. Argument powinien być typu System\BannerCampaign\Element\Target\TargetAbstract lub być tablicę obieków tej klasy');
            }

            if (is_array($this->getTarget()->isCollision($dataToCompare, $match))) {
                return true;
            } else {
                return false;
            }
        }
    }

    public function hasStats()
    {

        if ((int)$this->getViews() > 0) {
            return true;
        }

        return false;
    }

    public function __toString()
    {

        return $this->name;
    }

}

?>