<?php

namespace System\BannerCampaign\Element;

use System\Database\StandardAdapter;
use System\Observable;

class ElementDecorator extends Observable
{
    protected $element;

    public function __construct(ElementAbstract $element)
    {
        $this->attach(new ElementObserver);
        $this->element = $element;
    }

    /**
     * @param integer $mainUsersId
     * @param null    $bcId
     * @param boolean $convertReservation
     * @throws \Exception
     * @return boolean
     */
    public function save($mainUsersId, $bcId = null, $convertReservation = false)
    {
        $oldState = null;

        if (is_subclass_of($bcId, 'System\BannerCampaign\Element\ElementAbstract')) {
            $oldState = $bcId;
            $bcId = null;
        }


        $MainBcElement = new \Model\MainBcElement;

        StandardAdapter::beginTransaction();

        try {

            if (is_a($this->element, 'System\BannerCampaign\Element\Billboard')) {

                $MainDimages = new \Model\MainDimages();

                if (strlen($this->element->getBillboard())) {

                    if (false !== ($imageData = $MainDimages->getFile($this->element->getBillboard()))) {

                        $this->element->setBillboardMime($imageData['data_type']);
                    }

                }
                if (strlen($this->element->getBanner())) {

                    if (false !== ($imageData = $MainDimages->getFile($this->element->getBanner()))) {

                        $this->element->setBannerMime($imageData['data_type']);
                    }
                }

            } elseif (is_a($this->element, 'System\BannerCampaign\Element\Banner')) {

                $MainDimages = new \Model\MainDimages();

                if (strlen($this->element->getBanner())) {

                    if (false !== ($imageData = $MainDimages->getFile($this->element->getBanner()))) {

                        $this->element->setBannerMime($imageData['data_type']);
                    }
                }
            }

            $notify = null;

            if (null !== $this->element->getElementId()) {

                if (!$this->element->isDataFilled() && $convertReservation) {
                    throw new \Exception('error', 50);
                }

                // zmaiana statusu na aktywny jeśli status jest = 'reserved'
                if ($this->element->isReservation() && $this->element->isDataFilled() && $convertReservation) {
                    $notify['statusChange'] = array('element' => $this->element, 'old' => $this->element->getStatus(), 'new' => 1);
                    $this->element->setStatus(1);
                }

                if (true == $MainBcElement->updateElement(
                        $this->element->getElementId(),
                        $this->element->getId(),
                        $this->element->getPlace()->getId(),
                        $this->element->getStarts(),
                        $this->element->getEnds(),
                        $this->element->getStatus(),
                        $this->element->getAdditionalStats(),
                        json_encode($this->element->getTypeData())
                    )
                ) {

                    $elementTargetDecorator = new \System\BannerCampaign\Element\Target\TargetDecorator($this->element->getTarget());
                    $elementTargetDecorator->save($this->element->getElementId());

                    if (null !== $oldState && count(($differences = $this->element->compare($oldState)))) {

                        $this->notify(
                            'update',
                            array(
                                'user'    => $mainUsersId,
                                'data'    => $differences,
                                'element' => $this->element
                            )
                        );
                    }


                } else {

                    throw new \Exception(__METHOD__ . '. Nieudało się zapisać zmian w elemencie.');
                }

            } else {

                $elementId = $MainBcElement->addElement(
                    null !== $bcId ? $bcId : $this->element->getBcId(),
                    $this->element->getId(),
                    $this->element->getPlace()->getId(),
                    $this->element->getCreator()->getId(),
                    $this->element->getStarts(),
                    $this->element->getEnds(),
                    $this->element->getStatus(),
                    $this->element->getAdditionalStats(),
                    json_encode($this->element->getTypeData())
                );


                if (false == $elementId) {
                    throw new \Exception(__METHOD__ . '. Niedodano elementu!');
                }


                $elementTargetDecorator = new \System\BannerCampaign\Element\Target\TargetDecorator($this->element->getTarget());
                $elementTargetDecorator->save($elementId);

            }


            if (null !== $notify) {

                foreach ($notify as $action => $data) {

                    $this->notify($action, $data);
                }
            }

            StandardAdapter::commit();

            return true;

        } catch (\Exception $e) {

            StandardAdapter::rollback();
            throw new \Exception(__METHOD__ . '. ' . $e->getMessage(), 1, $e);
        }
    }


    public function stopElement($mainUsersId)
    {

        if (!$this->element->isLocked()) {

            $className = get_class($this->element);

            $oldStatus = $this->element->getStatus();

            $this->element->setStatus($className::STATUS_LOCKED);
            $this->save($mainUsersId);

            $this->notify(
                'statucchange', array(
                    'from'    => $oldStatus,
                    'to'      => $this->element->getStatus(),
                    'element' => $this->element,
                    'user'    => $mainUsersId
                )
            );
        }
    }

    public function startElement($mainUsersId)
    {

        if (!$this->element->isStarted()) {

            $className = get_class($this->element);

            $oldStatus = $this->element->getStatus();

            $this->element->setStatus($className::STATUS_ACTIVE);
            $this->save($mainUsersId);

            $this->notify(
                'statucchange', array(
                    'from'    => $oldStatus,
                    'to'      => $this->element->getStatus(),
                    'element' => $this->element,
                    'user'    => $mainUsersId
                )
            );
        }
    }


    public function disableElement($mainUsersId)
    {

        if (!$this->element->isDisabled()) {

            $className = get_class($this->element);

            $oldStatus = $this->element->getStatus();

            $this->element->setStatus($className::STATUS_DISABLED);
            $this->save($mainUsersId);

            $this->notify(
                'statucchange', array(
                    'from'    => $oldStatus,
                    'to'      => $this->element->getStatus(),
                    'element' => $this->element,
                    'user'    => $mainUsersId
                )
            );
        }
    }


    public static function notifyHit($mainBcElementId, $userId, $usercode, $sitecode, $url)
    {

        try {

            StandardAdapter::beginTransaction();

            $MainBcElement = new \Model\MainBcElement;
            $MainBcElementHit = new \Model\MainBcElementHit;

            $MainBcElement->incrementHits((int)$mainBcElementId);
            $MainBcElementHit->register(
                $mainBcElementId,
                $userId,
                $usercode,
                $sitecode,
                $url
            );

            StandardAdapter::commit();

        } catch (\Exception $e) {

            StandardAdapter::rollback();
            throw new \Exception(__METHOD__ . '. ' . $e->getMessage());
        }
    }


    public static function notifyView($elementIdList, $userId, $usercode, $sitecode, $url = null)
    {

        try {

            if (is_array($elementIdList) && 1 > count($elementIdList) ||
                !is_array($elementIdList) && 1 > (int)$elementIdList
            ) {

                return;
            }

            StandardAdapter::beginTransaction();

            $MainBcElementView = new \Model\MainBcElementView;
            $MainBcElementView->registerView(
                $elementIdList,
                $userId,
                $sitecode,
                $usercode,
                $url
            );

            $MainBcElement = new \Model\MainBcElement;
            $MainBcElement->incrementView($elementIdList);

            StandardAdapter::commit();

        } catch (\Exception $e) {

            StandardAdapter::rollback();
            throw new \Exception(__METHOD__ . '. ' . $e->getMessage());
        }
    }

    public static function decorateElement($element, $language)
    {

        if (is_array($element)) {

            foreach ($element as $_element) {

                if (!is_subclass_of($_element, 'System\BannerCampaign\Element\ElementAbstract')) {

                    throw new \Exception("Argument pass to decorateElement must be en instance of System\BannerCampaign\Element\ElementAbstract or a array of this elements.");
                }
            }

        } elseif (!is_subclass_of($element, 'System\BannerCampaign\Element\ElementAbstract')) {

            throw new \Exception("Argument pass to decorateElement must be en instance of System\BannerCampaign\Element\ElementAbstract or a array of this elements.");
        } else {

            $element = array($element);
        }

        foreach ($element as $key => $_element) {

            switch ($_element->getName()) {
                case 'content':

                    // Jeśli mam do czynienia z contentem to muszę pobrać jego treść.

                    static $mainContent = null;
                    if (null == $mainContent) {
                        $mainContent = new \Model\MainContent;
                    }

                    $contentId = $_element->getContentId();

                    try {


                        $baseContentData = $mainContent->getContent($contentId, $language);
                        $ContentTypeObj = null;

                        if (false !== $baseContentData) {

                            if ('news' == $baseContentData['type']) {

                                $ContentTypeObj = new \Model\MainContentNews;

                            } elseif ('publ' == $baseContentData['type']) {

                                $ContentTypeObj = new \Model\MainContentPublication;
                            } elseif ('event' == $baseContentData['type']) {

                                $ContentTypeObj = new \Model\MainContentEvent;
                            }

                            if (null !== $ContentTypeObj) {

                                $contentData = $ContentTypeObj->getElement($contentId, $language);

                                $_element->parseData(
                                    array(
                                        'description' => $contentData['body'],
                                        'title'       => $contentData['title'],
                                        'image'       => $contentData['avatar_id'],
                                        'date'        => $contentData['add_date']
                                    )
                                );

                                $element[$key] = $_element;
                            }
                        }
                    } catch (\Exception $e) {
                    }
                    break;
            }
        }

        return $element;
    }
}
