<?php

namespace System\BannerCampaign\Element;

use Model\MainBcElement,
    System\BannerCampaign\Supervisor\Creator,
    System\BannerCampaign\Element\Target\TargetFactory;

class ElementFactory
{


    CONST NOT_DISABLED = 'not_disabled';


    protected function __construct()
    {
    }


    public static function all($bcId, $config = array())
    {

        try {

            $MainBcElement = new MainBcElement;

            $result = array();

            $foundedElements = $MainBcElement->getElementsForCampaign($bcId, $config);

            foreach ($foundedElements as $element) {

                $result[] = self::createElement($element);
            }

            return $result;

        } catch (\Exception $e) {

            exit($e->getMessage());
            throw new \Exception(__METHOD__ . '. ' . $e->getMessage());
        }
    }


    /**
     * Pobiera pasujące elementy kampanii
     * @param array $userInformatins zawiera klucze specs,type
     * @return array lista pasujących kampanii
     */
    public static function matching($userInformations, $places = array())
    {

        try {

            $MainBcElement = new MainBcElement;

            $foundedElements = $MainBcElement->getElementsFromRange(
                time(),
                time(),
                $places,
                true
            );


            $result = array();

            $elementsIdList = array();

            foreach ($foundedElements as $k => $element) {

                $elementsIdList[] = $element['elementid'];
                $foundedElements[$k] = self::createElement($element);
            }

            // trigger_error( json_encode( $elementsIdList ) );

            // porównanie wybranych elementów z $userInformations

            $targetToCompare = array();
            foreach ($userInformations as $type => $data) {

                $target = TargetFactory::createElement(
                    array(
                        $type => $data
                    )
                );

                if (null !== $target) {

                    $targetToCompare[] = $target;
                }
            }


            $matchingElements = array();
            $a = array();

            foreach ($foundedElements as $element) {

                if ($element->isTargetMatch($targetToCompare)) {


                    $a[] = $element->getElementId();

                    $matchingElements[] = $element;
                }

            }

            return $matchingElements;

        } catch (\Exception $e) {

            trigger_error($e->getMessage());
        }

        return array();
    }


    public static function get($bcElementId, $config = array())
    {

        try {

            $MainBcElement = new MainBcElement;

            $elements = $MainBcElement->getElement($bcElementId, $config);

            if (null == $elements) {

                if (is_array($bcElementId)) {
                    $bcElementId = implode(',', $bcElementId);
                }

                throw new \Exception(__METHOD__ . '. Element width id = ' . $bcElementId . ' doesnt exista');
            }

            foreach ($elements as $key => $data) {

                $result[$key] = self::createElement($data);
            }

            return $result;

        } catch (\Exception $e) {

            throw new \Exception(__METHOD__ . '. ' . $e->getMessage());
        }
    }


    /**
     * @param integer $bcElementId
     * @param array $config
     * @throws \Exception
     * @return ElementAbstract
     */
    public static function one($bcElementId, $config = array())
    {
        try {
            $MainBcElement = new MainBcElement;
            $result = null;
            $element = $MainBcElement->getElement($bcElementId, $config);

            if (null == $element) {
                if (is_array($bcElementId)) {
                    $bcElementId = implode(',', $bcElementId);
                }

                throw new \Exception(__METHOD__ . '. Element width id = ' . $bcElementId . ' doesnt exista');
            }
            $result = self::createElement($element);
            return $result;
        } catch (\Exception $e) {
            throw new \Exception(__METHOD__ . '. ' . $e->getMessage());
        }
    }


    public static function createElement($elementData)
    {

        if (!isset($elementData['targetData']) && isset($elementData['target_data'])) {

            $elementData['targetData'] = $elementData['target_data'];
        }


        switch (strtolower(trim($elementData['type']))) {
            case 1:
            case 'content':

                return new Content($elementData);
            case 2:
            case 'banner':

                return new Banner($elementData);
            case 3:
            case 'billboard':

                return new Billboard($elementData);
            case 4:
            case 'content_data':

                return new ContentData($elementData);
            default:

                return new Undefinded($elementData);
        }
    }


    /**
     * Zwraca obiek odpowiedniej istancji
     */
    public static function makeObject($elementTypeId)
    {

    }
}

?>