<?php
namespace System\BannerCampaign\Element\Target;

abstract class TargetAbstract {

    protected $target = array();
    protected $id;

    public function __construct($id, $data = array()) {

        $this->id = $id;
        $this->setTarget($data);
    }

    /**
     * Musi zwracać w kluczu target id typu targetu oraz wartości wybranego targetu
     */
    abstract public function toArray();

    public function setTarget($targetData) {

        if (is_a($targetData, 'System\BannerCampaign\Element\Target\TargetAbstract')) {

            $target = new CompositeTarget;
            $target->setTarget($this);
            $target->setTarget($targetData);

            return $target;
        } else {

            if (!is_array($targetData)) {

                $targetData = explode(',', $targetData);
            }

            foreach ((array)$targetData as $targetEntity) {

                if (is_string($targetEntity) || is_int($targetEntity)) {
                    $targetEntity = trim($targetEntity);
                    $this->target[] = $targetEntity;
                }
            }

            return $this;
        }
    }

    public function getId() {
        return $this->id;
    }

    public function getTarget() {
        return $this->target;
    }

    /**
     * $matched szawiera true jeśłi porównanie odbyło się, porównanie może się nieodbyć jeśli dargety są różnych typów
     */
    public function isCollision(TargetAbstract $targetToCheck, &$match = null) {
        if (is_a($targetToCheck, '\System\BannerCampaign\Element\Target\CompositeTarget')) {
            return $targetToCheck->isCollision($this, $match);
        }

        $testElementTarget = $targetToCheck->getTarget();

        $conflicts = array();
        if ($this->getId() == $targetToCheck->getId()) {
            $match = true;

            foreach ((array)$this->target as $targetElement) {
                if (in_array($targetElement, $testElementTarget)) {
                    $conflicts[] = $targetElement;
                }
            }
        }

        return count($conflicts) > 0 ? array(
            'type' => $this->getId(),
            'elements' => $conflicts) : false;
    }

    public function isCollisionEx(TargetAbstract $targetToCheck, &$match = null) {
        if (is_a($targetToCheck, '\System\BannerCampaign\Element\Target\CompositeTarget')) {
            return false;
        }

        $testElementTarget = $targetToCheck->getTarget();
        $conflicts = array();

        if (($this->getId() == $targetToCheck->getId())) {
            $match = true;
            
            foreach ((array)$this->target as $targetElement) {
                if (in_array($targetElement, $testElementTarget) || in_array('*', $testElementTarget) || $targetElement == '*') {
                    if (in_array('*', $testElementTarget)) {
                        $conflicts[] = $targetElement;
                    } elseif ($targetElement == '*') {
                        foreach ($testElementTarget as $testElementTargetElement) {
                            $conflicts[] = $testElementTargetElement;
                        }
                    } else {
                        $conflicts[] = $targetElement;
                    }
                }
            }
        }

        return count($conflicts) > 0 ? array(
            'type' => $this->getId(),
            'elements' => $conflicts) : false;
    }

    public function __toString() {
        $result = '<ul>';

        foreach ($this->target as $target) {
            $result .= "<li>{$target}</li>";
        }

        $result .= '</ul>';
    }

}
