<?php

namespace System\Content\Element;
use System\Content\ElementComment,
	System\Translator\TranslatorEx;

abstract class AbstractContentElement {

	/**
	* Zawiera list potrzebnych kluczy. Wykoystywany do parsowania tablicy w konstruktorach klas potomnych
	*/
	protected $_needed = array();
	
	protected $comments = array();
	
	public function setComments( $comments ) {
		$this->$comments = $comments;
	}
	
	public function __construct($data) {
		$missingKeys = array();
		
		if(!$this->parseArray($data,$missingKeys)) {
			//throw new \Exception(__METHOD__.". Brak nastpujcych kluczy w tablicy przekazanej do konstruktora. Brak kluczy o nazwach ".implode(',',$missingKeys));
		}
		
		$this->processInputData($data);
	}
	
	protected function parseArray( $inputArray , &$missing = null ) {
		$_missing = array();
		
		
		foreach($this->_needed as $key => $value) {
			
			if( !array_key_exists( $key , $inputArray ) ) {
				$_missing[] = $key;
			}
		}
		
		if( null !== $missing ) {
			
			$missing = $_missing;
		}
		
		return count($_missing) ? false : true;
	}
	
	public function __set( $name , $value ) {
		
		$this->$name = $value;
	}
	
	public function __get($name) {
		if(isset($this->$name))
			return $this->$name;
		return null;
	}
	
	
	final public function processInputData( $data ) {
		
		foreach($this->_needed as $propertyName => $field) {
			
			if( $propertyName == 'news_type' && is_int( $data[ $propertyName ] ) ) {
				$this->$field = $data[ $propertyName ];
				
				$DictNewsType = new \Model\DictNewsType;
				$type = $DictNewsType->getTypeName( $data[ $propertyName ] );
				
				$TranslatorEx = TranslatorEx::get();
				$this->newsTypeName = $TranslatorEx->translate( 'main_' . $type );
				
			} elseif( isset( $data[ $propertyName ] ) ) {
			
				$this->$field = $data[ $propertyName ];
			}
			
			unset( $data[ $propertyName ] );
		}
		
		foreach( $data as $name => $value ) {
			
			if( !isset( $this->$name ) ||
				null == $this->$name ) {
				
				if( $name == 'evaluation' ) {
					
					if( (int)$value > 0 && strpos( (string)$value , '.' ) >= 0 ) {
						
						$round = (string)$value;
						$round = explode( '.' , $round );
						
						if( $round[ 1 ] > 0 && $round[ 1 ] < 3 ) {
							
							$round[ 1 ] = 0;
							
						} elseif( $round[ 1 ] >=3 && $round[ 1 ] < 7 ) {
							
							$round[ 1 ] = 5;
						} elseif( $round[ 1 ] >= 7 && $round[ 1 ] < 9 ) {
							
							if( $round[ 0 ] < 5 ) {
								$round[ 0 ]++;
							}
							$round[ 1 ] = 0;
						}
						
						$round = $round[ 0 ] . '.' . $round[ 1 ];
						
						$this->evaluationRounded = $round;
					} else {
						$this->evaluationRounded = '0';
					}
				}
				
				$this->$name = $value;
			}
		}
		
		if( isset( $data[ 'comments' ] ) ) {
			$this->comments = array();
			
			foreach($data['comments'] as $commentData) {
				
				$this->comments[] = new ElementComment( $commentData );
			}
		}
	}		
}

?>