<?php
namespace System\Controller;

use Zend\Filter\Word\CamelCaseToDash;
use Zend\Mvc\Controller\AbstractActionController;
use Zend\View\Model\ViewModel;
use Zend\Escaper\Escaper;
use Zend\Permissions\Acl\Acl,
    Zend\Permissions\Acl\Role\GenericRole as Role,
    Zend\Permissions\Acl\Resource\GenericResource as Resource,
    Zend\Mvc\MvcEvent,
    System\Registry\Registry,
    System\Authentication\CmsAuthentication,
    System\View\Helper\ScriptHelper,
    System\Form\QueryParser;

/**
 * @method Plugin\ExtendedRedirect extendedRedirect()
 * @method Plugin\ListRequestParser listRequestParser()
 * @method Plugin\ListChildViewCreator listChildViewCreator()
 */
class BaseActions extends AbstractActionController
{

    protected $View, $acl = null;
    protected $Log = null;
    protected $Cache = null;
    protected $translator = null;
    protected $lang = null;
    protected $cfg = null;
    protected $dbAdapter;
    /** @var \System\Authentication\CmsAuthentication */
    protected $auth;

    public function onDispatch(MvcEvent $e)
    {
        if (method_exists($this, 'init')) {
            $this->init();
        }

        return parent::onDispatch($e);
    }

    protected function init()
    {

        $cfg = $this->getServiceLocator()->get('config');
        $this->cfg = $cfg;

        // ustawia warunek 
        \Model\MainContentHelper::setCms();


        $this->auth = new CmsAuthentication();

        require('PDOS.class.php');

        $lang = @$_COOKIE['LANG']; // grupa językowa
        $phraseLang = @$_COOKIE['phrase']; // język fraz


        if ($lang == null) { // jeśli język nie został ustawiony wcześniej pobieram pierwszy język akceptowany w przeglądarce, jeśli ten język jest nieobsługiwany ustawiam domyślny język.
            if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE']) && in_array(substr($_SERVER['HTTP_ACCEPT_LANGUAGE'], 0, 2), array('pl', 'ru', 'ua'))) {
                $lang = substr($_SERVER['HTTP_ACCEPT_LANGUAGE'], 0, 2);
            } else {
                $lang = 'pl';
            }
        }


        if ($phraseLang == null) { // szukam języka fraz
            if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE']) && in_array(substr($_SERVER['HTTP_ACCEPT_LANGUAGE'], 0, 2), array('pl', 'ru', 'ua', 'en'))) {
                $phraseLang = substr($_SERVER['HTTP_ACCEPT_LANGUAGE'], 0, 2);
            } else {
                $phraseLang = 'pl';
            }
        }


        $this->View = new ViewModel();
        $this->View->escaper = new Escaper();
        $this->lang = $lang;
        $this->View->lang = $lang;
        $this->layout()->lang = $lang;


        $this->layout()->esculapDomain = $this->cfg['esculapDomain'];
        $this->View->esculapDomain = $this->cfg['esculapDomain'];


        // obiekt odpowiedzialn za ładowanie skryptów podanych w kontrolerach
        $this->layout()->scriptHelper = new ScriptHelper;


        $this->View->phraseLang = $phraseLang;
        $this->phraseLang = $phraseLang;
        $this->layout()->phraseLang = $phraseLang;


        if (isset($cfg['database']['escape']['apos'])) {
            define("ESCAPE_APOS", $cfg['database']['escape']['apos']);
        } else {
            define("ESCAPE_APOS", "''");
        }


        // Ustawiam adapter
        $adapter = \System\Database\StandardAdapter::getInstance($this->cfg['database']['adapter']['int_esculap']);
        \System\Database\AbstractTable::setSchema($this->lang . '_content');
        //$adapter->createStatement( "SET search_path TO esculap_int , \"\$user\",public,emailnig,postfix,esculap,data_buff,elearning" )->execute(); 
        // licznik czasu
        if ('check_terminate' == $this->getEvent()->getRouteMatch()->getMatchedRouteName()) {
            \System\Database\AbstractTable::setSchema('cms, ' . $this->lang . '_content');

            return;
        }

        if ($this->auth->isAuthenticated($this->cfg['auth_lifetime'])) {

            $this->identity = $this->auth->getData();

            $session = new \Zend\Session\Container('cmsauth');
            $session->setExpirationSeconds($this->cfg['auth_lifetime']);
            $session->userId = $this->identity['user_id'];


            $_SESSION['auth'] = $this->identity;
            $this->identity['user_login'] = $this->identity['username'];


            // przekazanie do layoutu długości trwamnia sesji w sekundach w celu odświerzenia strony po zakończeniu się sesji
            $this->layout()->sessionExpireSeconds = $this->cfg['auth_lifetime'];


            $this->View->identity = $this->identity;
            $this->layout()->identity = $this->identity;


            // pobieram grupy językowe do, których należy użytkownik
            $cmsRelLgroupUser = new \Model\cmsRelLgroupUser;


            $availableLanguages = array();
            $userLgroups = $cmsRelLgroupUser->getGroupForUserWidthStrKey($this->identity['user_id']);


            foreach ($userLgroups as $userLangGroup) {

                $availableLanguages[] = strtolower($userLangGroup['group_name']);
            }


            if (!count($availableLanguages)) {

                $this->logger()->err("Urzytkownik {$this->identity['user_id']} niema przypisanej żadnej grupy językowej. Dostał grupę \"{$this->lang}\"", __NAMESPACE__, __LINE__);
            } else {

                if (!in_array(strtolower($lang), $availableLanguages)) {

                    $lang = strtolower(current($availableLanguages));

                    $this->lang = $lang;
                    $this->View->lang = $lang;
                    $this->layout()->lang = $lang;
                }
            }

            // inicializacja aclki
            $this->initAcl();


            // inicializacja translatora
            Registry::set('lang', $lang);
            Registry::set('phraseLang', $phraseLang);


            $this->translator = $this->translator();


            $this->View->translator = $this->translator();
            $this->layout()->translator = $this->translator();

            \System\View\LayoutCreator::setUrlHelper($this->plugin('url'));
            \System\View\LayoutCreator::prepearMenu($this->acl, $this->identity['username']);
            // ustawianie schematu kontentu
            \System\Database\AbstractTable::setContentSchema($lang . '_content');


            // pobieram informacje do layoutu
            $DictLgroups = new \Model\DictLgroups;
            $allGroups = $DictLgroups->getAll();
            $this->layout()->allLanguages = $allGroups;
            $this->allLangGroups = $allGroups;
        } else {

            if ('login' !== $this->getEvent()->getRouteMatch()->getMatchedRouteName()) {

                if (isset($_SERVER['REQUEST_URI'])) {

                    $backurlSession = new \Zend\Session\Container('backurl');
                    $backurlSession->backurl = $_SERVER['REQUEST_URI'];
                    $backurlSession->setExpirationHops(1);
                }

                exit(header("Location: " . $this->url()->fromRoute('login')));
            }
        }


        Registry::set('lang', $lang);
        Registry::set('phraseLang', $phraseLang);


        $this->View->translator = $this->translator();
        $this->View->listUrl = str_replace('/relational_view/true', '', $this->getRequest()->getRequestUri());
        $this->layout()->translator = $this->translator();

        if ($this->getEvent()->getRouteMatch()->getParam('relational_view') == 'true') {
            $this->layout()->setTemplate('layout/relational');
        }
    }

    public function initAcl()
    {
        $Acl = new Acl();

        $Groups = new \Model\cmsDictGroups();
        $roles = $Groups->getGroups();

        $Actions = new \Model\cmsMainActions();
        $_actions = $Actions->getActions();
        $actions = array();
        foreach ($_actions as $actionRow) {
            $actions[$actionRow['action_id']] = $actionRow;
        }

        $Rules = new \Model\cmsRelGroupAction();
        $rules = $Rules->getRules();

        // role użytkownika
        $UserRoles = new \Model\cmsRelGroupUser();
        $userFunctions = $UserRoles->getRolesForuser($this->identity['user_id']);

        foreach ($actions as $actionRow) {

            if (!$Acl->hasResource(/* $actionRow['system'].'-'. */
                $actionRow['controller']
            )
            ) {
                $Acl->addResource(
                    new Resource(/* $actionRow['system'].'-'. */
                        $actionRow['controller']
                    )
                );
            }

            if (!$Acl->hasResource(/* $actionRow['system'].'-'. */
                $actionRow['controller'] . '-' . $actionRow['action']
            )
            ) {
                $Acl->addResource(
                    new Resource(/* $actionRow['system'].'-'. */
                        $actionRow['controller'] . '-' . $actionRow['action']
                    )
                );
            }
        }

        foreach ($roles as $role) {
            $Acl->addRole(new Role($role));
        }

        foreach ($rules as $rule) {

            $role = $rule['group_id'];
            $action = $actions[$rule['action_id']];

            //echo "{$action['controller']} - {$action['action']}<br><br>";

            if (!$Acl->isAllowed(
                $role, /* $action['system'].'-'. */
                $action['controller']
            )
            ) {
                $Acl->allow(
                    $role, /* $action['system'].'-'. */
                    $action['controller']
                );
            }
            if (!$Acl->isAllowed(
                $role, /* $action['system'].'-'. */
                $action['controller'] . '-' . $action['action']
            )
            ) {
                $Acl->allow(
                    $role, /* $action['system'].'-'. */
                    $action['controller'] . '-' . $action['action']
                );
            }
        }


        // dodanie grup językowych jako składników
        $cmsDictLgroups = new \Model\cmsDictLgroups;
        foreach ($cmsDictLgroups->getAll() as $langGroup) {
            $Acl->addResource(new Resource('language_group_' . strtolower($langGroup['group_name'])));
        }


        // pobieram grupy językowe do, których należy użytkownik
        $cmsRelLgroupUser = new \Model\cmsRelLgroupUser;
        $userLgroups = $cmsRelLgroupUser->getGroupForUserWidthStrKey($this->identity['user_id']);


        // dodaję role użytkownika czyli jego login i nadaję mu jego funkcje
        $Acl->addRole(new Role($this->identity['username']), $userFunctions);


        // dodaję grupy językowe dla użytkowinka. Dopiero tutaj bo wcześniej rola użyutkownika nie istniała
        foreach ($userLgroups as $userLangGroup) {
            $Acl->allow($this->identity['username'], 'language_group_' . strtolower($userLangGroup['group_name']));
        }


        $this->acl = $Acl;
        $this->View->acl = $Acl;
        $this->layout()->acl = $Acl;
        $userId = $this->identity['username'];
        $controller = $this;

        $this->getEventManager()->attach(
            'aclCheck', function ($e) use ($Acl, $userId, $controller) {

                $action = $e->getParams();
                $action = isset($action['aclAction']) ? $action['aclAction'] : '';
                $isAllowed = false;

                try {
                    if ($Acl->isAllowed($userId, $action)) {
                        $isAllowed = true;
                    }
                } catch (\Exception $e) {

                }

                if (!$isAllowed) {

                    $controller->redirect()->toRoute('home');
                }
            }
        );
    }

    public function viewRoot($namespace, $className)
    {
        $filter = new CamelCaseToDash();
        $namespace = explode('\\', $namespace)[0];
        $className = explode('\\', $className);
        $className = str_replace('Controller', '', end($className));
        $viewRoot = $filter->filter($namespace) . '/' . $filter->filter($className) . '/';
        $viewRoot = mb_strtolower($viewRoot, 'UTF-8');

        return $viewRoot;
    }

    /**
     * @return \Zend\View\Model\ViewModel
     */
    public function getView()
    {
        return $this->View;
    }

    /**
     * @return \System\Translator\TranslatorStorageEx
     */
    public function getTranslator()
    {
        return $this->translator;
    }

    /**
     * @return \Zend\Http\Request
     */
    public function getRequest()
    {
        return $this->request;
    }

    /**
     * @return \Zend\Permissions\Acl\Acl
     */
    public function getAcl()
    {
        return $this->acl;
    }

    /**
     * @return array
     */
    public function getIdentity()
    {
        return $this->auth->getData();
    }

    /**
     * @return \System\Log\LoggerAdapter
     */
    protected function logger()
    {

        $logger = $this->getServiceLocator()->get('logger');

        return $logger;
    }

    protected function cache()
    {

        $cache = $this->getServiceLocator()->get('cache');

        return $cache;
    }

    protected function translator()
    {

        $sm = $this->getServiceLocator();
        $t = $sm->get('translatorEx');

        return $t;
    }

    protected function parseRequest($queryCacheName, array $query = array('conditions' => array(), 'order' => array('id' => 'desc'), 'limit' => 25, 'page' => 1))
    {
        $routeName = $this->getEvent()->getRouteMatch()->getMatchedRouteName();
        $queryCache = new \Zend\Session\Container($routeName . '_' . get_class($this) . '_' . $queryCacheName);

        isset($query['conditions']) ?: $query['conditions'] = array();
        isset($query['order']) ?: $query['order'] = array('id' => 'desc');
        isset($query['limit']) ?: $query['limit'] = 25;
        isset($query['page']) ?: $query['page'] = 1;

        if ($this->request->isXmlHttpRequest()) {
            foreach ($query as $queryPart => &$queryValue) {
                if ($this->request->getPost($queryPart)) {
                    $queryValue = $this->request->getPost($queryPart);
                }
            }
            $queryCache->query = $query;
        } elseif (isset($queryCache->query)) {
            $query = $queryCache->query;
        }

        return QueryParser::parseQuery($query['conditions'], $query['order'], $query['limit'], $query['page']);
    }

    protected function generateListView(array $viewData)
    {
        $pages = ceil($viewData['count'] / $viewData['limit']);
        !($pages <= 0) ?: $pages = 1;

        $ListView = new ViewModel();
        $ListView->setTemplate($viewData['listSubview']);
        $ListView->translator = $this->translator();
        $ListView->escaper = new Escaper();
        $ListView->listData = $viewData['listData'];
        $ListView->tableCurrentPage = $viewData['page'];
        $ListView->tablePagesNum = $pages;
        $ListView->count = $viewData['count'];
        $ListView->acl = $this->View->acl;
        $ListView->identity = $this->identity;

        if (isset($viewData['aclName'])) {
            $ListView->allowView = $this->acl->isAllowed($this->identity['username'], $viewData['aclName'] . '-view') ? true : false;
            $ListView->allowEdit = $this->acl->isAllowed($this->identity['username'], $viewData['aclName'] . '-edit') ? true : false;
            $ListView->allowDelete = $this->acl->isAllowed($this->identity['username'], $viewData['aclName'] . '-delete') ? true : false;
        }

        $this->View->tableCurrentPage = $viewData['page'];
        $this->View->tablePagesNum = $pages;
        $this->View->order = $viewData['order'];
        $this->View->limit = $viewData['limit'];

        $whereView = array();
        foreach ($viewData['where'] as $whereCondition) {
            $whereView = array_merge($whereView, $whereCondition);
        }

        $this->View->where = $whereView;
        $this->View->addChild($ListView, 'list');

        $this->layout()->scriptHelper
            ->attach(new ScriptHelper\Javascript('js/jquery/inputmask/jquery.inputmask.js'))
            ->attach(new ScriptHelper\Javascript('js/jquery/inputmask/jquery.inputmask.numeric.extensions.js'))
            ->attach(new ScriptHelper\Javascript('js/jquery/inputmask/jquery.inputmask.date.extensions.js'))
            ->attach(
                new ScriptHelper\Javascript(
                    null, "$(function() {
                    $('.esculap-integer-input').inputmask('integer', { allowMinus: false, allowPlus: false, rightAlignNumerics: false });
                    $('.esculap-date-input').inputmask('date');
                  });"
                )
            );

        if ($this->request->isXmlHttpRequest()) {
            if ((isset($viewData['user_id']) && $viewData['user_id'] > 0) || (isset($viewData['group_id']) && $viewData['group_id'] > 0) || (isset($viewData['lgroup_id']) && $viewData['lgroup_id'] > 0)) {
                $this->View->setTerminal(true);

                return $this->View;
            } else {
                $ListView->setTerminal(true);

                return $ListView;
            }
        } else {
            return $this->View;
        }
    }

}

?>

