<?php

namespace System\Controller;

use Model\DictSpec;
use Model\TableGateway\MainSiteUserAccess;
use Zend\Mvc\Controller\AbstractActionController,
    System\Authentication\AuthenticationAdapter,
    System\Authentication\AuthenticationService,
    Zend\Authentication\Storage\Session as SessionStorage,
    Zend\View\Model\ViewModel,
    System\Authentication\AuthenticationStorage,
    Zend\Mvc\MvcEvent,
    System\User\CookieStorage,
    Zend\EventManager\EventManager,
    System\Registry\Registry,
	System\BannerCampaign\Collector as BannerCampaignsCollector,
	System\Impersonate\User as UserImpersonate,
	System\Observer\GlobalObserver,
	System\Soap\WbServClient;
use System\View\Helper\BannerDisplayer;
use Zend\Session\SessionManager;

abstract class EsculapBaseActions extends AbstractActionController {


	/**
	* Mówi czy właśnie wyświetlany jest podgląd treści. Wykorzystywany do podglądu np. czy treści wyświetlają się poprawnie i jak dziełają banery.
	*/
	protected $contentDebug = false;


	/**
	 * Ignorowanie specializacji do wyszukiwania kontentu...
	 */
	protected $content_ignoreSpecValue = 0;


	/**
	 * Jeszcze niużywane ale zapewne wktótce będzie potrzebna...
	 */
	protected $baner_ignoreSpecValue = 0;


	/**
	* Zawiera id aktualnie oglądanego contentu w celu wyłączenia go z listy idków, które mągą zostać wybrane do layoutu.
	*/


	/**
	 * Wyłącza ładowanie elementów kampanii jeśli ustawione na true
	 */
	protected $noBcInit = false;
	protected $bcViewMode = null;
	protected $bcViewModeElements = array();


	/**
	 * Mówi w jaki sposób wczytać kampanie reklamowe - wczytanie danych osobowych ze tabeli main_view_content
	 */
	protected $contentView = false;
	protected $contentViewCode = null;


	protected $commentsAddPossybility = true;
	protected $contentEvaluationPossibility = false;


	protected $viewModeUrl = '';


	protected $initLayout = false;
	protected $initBannerCampaigns = false;


	protected $currentWatchContentId = null;

	protected $authRequired = true;

    /** @var \Zend\View\Model\ViewModel */
	protected $view;
	protected $_siteCode;
	protected $_userCode = '';
	protected $lang;
	protected $cookieLang;

	protected $wbServClient;
    protected $bannersInited = false;

    /** @var \MainSite\Strategy\MainSiteStrategy */
    protected $mainSiteStrategy = null;
    /** @var \Authentication\Identity */
    protected $identity = null;

	public function onDispatch( MvcEvent $me ) {
        if ($this->request->getQuery( 'contentviewcode', false)) {
            $session = new \Zend\Session\Container('contentviewcode');
            $session->contentviewcode = $this->request->getQuery( 'contentviewcode');
            $session->urlGet = 'contentviewcode';
        } else {
            $sessionManager = new SessionManager();
            unset($sessionManager->getStorage()['contentviewcode']);
        }

		$this->auth();

		if( method_exists( $this , 'init' ) ) {
			call_user_func_array(array($this,'init'),array());
		}

		parent::onDispatch( $me );
	}


	/**
	 * Próba uwierzytelnienia użytkownika
	 */
	abstract protected function auth();


	/**
	 * Inicializacja obiektów dow layoutu
	 */
	abstract protected function initLayout();

    public function getCurrentWatchContentId() {
        return $this->currentWatchContentId;
    }

	/**
	* Inicializacja
	*/
	protected function init() {
        $conf = $this->getServiceLocator()->get('config');
		$this->cfg = $conf;


		$sessionCfg = new \Zend\Session\Config\StandardConfig;
		$sessionCfg->setOptions(array(
			'cookie_domain' => '.test.esculap.com'
		));



		if( 'home' == $this->getEvent()->getRouteMatch()->getMatchedRouteName() ||
			'main' == $this->getEvent()->getRouteMatch()->getMatchedRouteName() )
			$this->layout()->index = 1;



		//echo $sessionCfg->getCookieDomain();


		/*$this->sessionManager = new \Zend\Session\SessionManager( $sessionCfg );
		\Zend\Session\Container::setDefaultManager( $this->sessionManager );


		//session_set_cookie_params( 0 , '/' , '.esculap.com' );


		/**
		 * Proteza jaksiemasz.
		 * - zamienia wartość klucza SPEC_TYPES w \System\Content\Config na array() co powoduje, że niema filtrowania kontentu
		 */
		if( $this->content_ignoreSpecValue == 1 ) {

			Config::ignoreSpecs( true );
		}


		$this->view = new ViewModel();


		$this->getEventManager()->attach( '*' , function( $e ) {

			//GlobalObserver::setLogger( $this->logger );

			GlobalObserver::notify(
				$e->getName(),
				$e->getParams(),
				$e->getTarget()
			);

		});


		//obiekt logowania
		$this->logger = $this->getServiceLocator()->get( 'logger' );

		// inicializacja obiektu obsługującego cookie
		$cookieStorage = new CookieStorage();
		$this->cookieLang = $cookieStorage->getLang();


		// pobranie uchwytu dla bazy danych dla sesji
		$sessionDbHandle = $conf[ 'database' ][ 'session' ];

		$this->layout()->elogin_url = $conf[ 'elogin' ];
		$this->layout()->productin = (bool)@$conf[ 'productin' ];


		if( isset( $conf[ 'pivik' ] ) ) {
			$this->layout()->pivik_target1 = @$conf[ 'pivik' ][ 'target1' ];
			$this->layout()->pivik_target2 = @$conf[ 'pivik' ][ 'target2' ];
		}

		$this->readLanguage();

		// ustawienie domyślnego adaptera. Przeniesione tu żeby możliwe było debugowanie kontentu i banerów.
		$adapter = \System\Database\StandardAdapter::getInstance( $this->cfg['database']['adapter']['int_esculap'] );
		$adapter->createStatement( 'SET search_path TO esculap_int , elearning, public, postfix, data_buff' )->execute();
		$_SESSION['content_added'] = 1;




		$this->view->lang = $this->lang;
		$this->layout()->lang = $this->lang;
		$this->_siteCode = $this->cfg[ 'site' ][ $this->lang ][ 'code' ];



		// ustawianie schematu kontentu
		$this->setSchema();
		$adapter->createStatement( "SET search_path TO {$this->getSchema()}, esculap_int , elearning, public, postfix, data_buff" )->execute();
		//\System\Database\AbstractTable::setContentSchema($lang.'_content');

		$this->identity = $this->identity();

        if ($this->identity) {
            $specCodes = [0 => null, 1 => null];
            foreach ($this->identity->specs as $key => $spec) {
                if ($key == 'MAIN') {
                    if (is_array($spec) && isset($spec[0])) {
                        $specCodes[0] = $spec[0];
                    } elseif (is_string($spec) && $spec) {
                        $specCodes[0] = $spec;
                    }
                } elseif ($key == 'CURR') {
                    if (is_array($spec) && isset($spec[0])) {
                        $specCodes[1] = $spec[0];
                    } elseif (is_string($spec) && $spec) {
                        $specCodes[1] = $spec;
                    }
                }
            }
            $dictSpec = new DictSpec();
            $specNames = $dictSpec->getNamesForSpecs($specCodes);
            if ($specCodes[0] && isset($specNames[$specCodes[0]])) {
                $this->layout()->mainSpecName = $specNames[$specCodes[0]];
            }
            if ($specCodes[1] && isset($specNames[$specCodes[1]])) {
                $this->layout()->currSpecName = $specNames[$specCodes[1]];
            }
        }

		$this->view->identity = $this->identity;
		$this->layout()->identity = $this->identity;

		// inicializacja translatora
		Registry::set('lang',$this->lang);
		Registry::set('system','e');
		$this->view->translator = $this->translator();
		$this->layout()->translator = $this->translator();


		$this->layout()->keywords = $this->translator()->translate( 'main_meta_tag_keywords' );
		$this->layout()->description = $this->translator()->translate( 'main_meta_tag_description ' );
		$this->layout()->title = $this->translator()->translate( 'main_meta_tag_title' );

		# Wpisanie do konfiguracji id z main_site na podstawie hosta
                $host = trim($this->getRequest()->getServer('HTTP_HOST'));

                # Taki hotfix niestety, żeby test działał
                $host = explode('.', $host);
                if ($host[0] == 'pl' || $host[0] == 'ru') {
                    unset($host[0]);
                }
                $host = implode('.', $host);

                $host = '//' . $host . '/';
                if ($host == '//test.esculap.com/' || $host == '//local.esculap.com/') { //Mapowanie na sztywno, żeby działał test.esculap.com i local
                    $host = '//esculap.com/';
                } else {
                    $mainSites = $this->cfg['mainSiteId'];
                    unset($mainSites['esculap']);
                    foreach ($mainSites as $mainSiteName => $mainSiteID) {
                        if ($host == "//local.{$mainSiteName}.esculap.com/") {
                            $host = "//{$mainSiteName}.esculap.com/";
                        }
                    }
                }

                $mainSite = $this->getServiceLocator()->get('MainSiteTableGateway')->getByHost($host);
                if ($mainSite && $mainSite->getId()) {
                    $show = false;
                    if ($mainSite->getPublic()) {
                        $show = true;
                    } else{
                        $mainSiteUserAccess = new MainSiteUserAccess();
                        if ($this->identity) {
                            $access = $mainSiteUserAccess->getByMainSiteAndUser($mainSite->getId(), $this->identity->get('user_id'));
                            if ($access && $access->getMainUserId() == $this->identity->get('user_id')) {
                                $show = true;
                            }
                        }
                    }

                    if ($show) {
                    $this->cfg['main_site_id'] = $mainSite->getId();

                    if ($this->cfg['main_site_id'] != 1) {
                        $this->layout()->keywords = $this->translator()->translate('main_site' . $this->cfg['main_site_id'] . '_meta_tag_keywords');
                        $this->layout()->description = $this->translator()->translate('main_site' . $this->cfg['main_site_id'] . '_meta_tag_description');
                        $this->layout()->title = $this->translator()->translate('main_site' . $this->cfg['main_site_id'] . '_meta_tag_title');
                        $this->cfg['domain'] = trim($host, '/');

                        try {
                            $this->mainSiteStrategy = new \MainSite\Strategy\MainSiteStrategy($this->getMainSiteID());
                        } catch (\MainSite\Strategy\MainSiteStrategyException $exception) {
                            $this->mainSiteStrategy = null;
                        }
                    }
                    } else {
                        $this->redirect()->toUrl('//esculap.com/');
                    }
                } elseif ($mainSite && $mainSite->getId()) {
                    $this->redirect()->toUrl('//esculap.com/');
                } else {
                    $this->cfg['main_site_id'] = null;
                }

                // przekazanie do layoutu długości trwamnia sesji w sekundach w celu odświerzenia strony po zakończeniu się sesji
                $this->layout()->sessionExpireSeconds = 3600;
                $this->layout()->domain = $this->cfg[ 'domain' ];
                $this->view->domain = $this->cfg[ 'domain' ];
                $this->layout()->mainSiteID = $this->cfg[ 'main_site_id' ];
                $this->view->mainSiteID = $this->cfg[ 'main_site_id' ];




		// obiekt przechwytujący niezłapane wyjątki i przekierowujący na inną stronę
		$controller = $this;
		$log = $this->logger;
		$this->getEventManager()->attach( 'error' , function($e) use ( $log , $controller ) {

			$log->crit('Nazwa '.$e->getName().",target ".(get_class($e->getTarget())).'. Dump: '.var_dump($e->getParams()),__METHOD__,__LINE__);
			$controller->redirect()->toUrl('/error');

		});


		//exit( (int)$this->bcViewMode );

		if( !isset( $this->noLayout ) ) {

			if( true === $this->initBannerCampaigns ) {
				$this->isBcViewMode();
				$this->initBannerCampaigns();

			} if( true === $this->initLayout ) {
				$this->initLayout();

			}

		}

            if ($this->params()->fromQuery('logout') == 1 || $this->params()->fromQuery('logout') == '1') {
                $authService = $this->getServiceLocator()->get( 'EsculapAuth' );
		$authService->logout();
            }
	}

	/**
	 * Sprawdzam czy używam podglądu banerów
	 *
	 */
	protected function isBcViewMode( $checkOnly = false ) {

		if( null !== $this->bcViewMode )
			return $this->bcViewMode;


		if( $this->request->getQuery( 'bcvcode' , false ) ) {

			if( $checkOnly )
				return true;

			$this->readLanguage();
			$this->setSchema();

			$bcViewModeCode = $this->request->getQuery( 'bcvcode' );
			$MainBcViewCode = new \Model\MainBcViewCode;

			try {
				if( ( $codeData = $MainBcViewCode->getCodeSettings( $bcViewModeCode , true ) ) ) {
					$this->bcViewModeElements = explode( ',' , $codeData[ 'elements' ] );
					$this->bcViewModeCode = $bcViewModeCode;
					$this->bcViewMode = true;
					$this->viewModeUrl = '?bcvcode=' . $this->bcViewModeCode;
					$this->layout()->addToHrefs = $this->viewModeUrl;

                    if (strlen($this->bcViewModeCode) == 12) {
                        $session = new \Zend\Session\Container('bcvcode');
                        $session->bcvcode = $this->bcViewModeCode;
                        $session->urlGet = 'bcvcode';
                    } else {
                        $sessionManager = new SessionManager();
                        unset($sessionManager->getStorage()['bcvcode']);
                    }

                    $this->getEventManager()->trigger(
						'content_debug',
						$this,
						array(
							'state' => true,
							'namespace' => __NAMESPACE__,
							'line' => __LINE__
						)
					);
					return true;
				}
			} catch( \Exception $e ) {
				$this->logger()->err( $e->getMessage() , __METHOD__ , __LINE__ );
			}
		}

		$this->bcViewMode = false;
		return false;
	}

	protected function readLanguage() {
		$lang = null;

		if( null == $this->lang && null !== $this->cookieLang ) {

			$this->lang = $this->cookieLang;

		} elseif( null !== $this->lang ) {

			// zapisanie wartości lang jeśli jeest przekazana w gecie w celu utrzymania kompatybilności z poniższym sposobem wybierania języka.
			$lang = $this->lang;

		} elseif( preg_match('/^(www\.)?(pl|ru)\.(.*)/',$_SERVER['HTTP_HOST'],$matches) ) {

			$lang = $matches[2];

			$this->lang = $lang;

		} elseif( null == $this->lang ) {
			$lang = @$_COOKIE[ 'lang' ];

			if($lang == null) { // jeśli język nie został ustawiony wcześniej pobieram pierwszy język akceptowany w przeglądarce, jeśli ten język jest nieobsługiwany ustawiam domyślny język.

				if(isset($_SERVER['HTTP_ACCEPT_LANGUAGE']) && in_array(strtolower(substr($_SERVER['HTTP_ACCEPT_LANGUAGE'],0,2)),array('pl','ru','ua'))) {
					$lang = substr($_SERVER['HTTP_ACCEPT_LANGUAGE'],0,2);
				} else {
					$lang = 'pl';
				}
			}

			$this->lang = $lang;
		}

	}















	/**
	* Ustawia debugowanie fraz
	*/
	protected function checkPhraseDebug() {
		$this->translator()->debug( true );
		$this->layout()->phraseDebug = 1;
	}


	public function initTranslator($lang) {
		$translator = new \System\Translator\Translator($lang);
		return $translator;
	}

	public function &translator() {
		$sm = $this->getServiceLocator();
		$t = $sm->get('translatorEx');

		return $t;
	}


	protected function initBannerCampaignEx() {
        // Dla locala
        //return;

        if (!$this->bannersInited && !$this->isMainSite('bezsennosc')) {
		if( !$this->bcViewMode ) {
			$bcElements = $this->getElementFormWbserv();

		} else {
			$bcElements = $this->loadElementsFoCode();
		}


		if( false == $bcElements ) {

			$this->logger()->err( 'The creation of the elements of the campaign was not possible' , __METHOD__ , __LINE__ );
		}

		\System\View\Helper\BannerDisplayer::init( $bcElements , $this->viewModeUrl );
        }
	}



	protected function getElementFormWbserv() {
        // Dla locala
        //return false;

		try {

			$this->wbServClient = new WbServClient(
				$this->_siteCode,
				null == $this->identity() ? null : $this->identity()->user_code,
				$this->lang,
				$this->cfg[ 'wbserv_addr' ]
			);

		} catch( \Exception $e ) {

			$this->logger()->err( $e->getMessage() , __NAMESPACE__ , __LINE__ );
			$this->selectedBcElements = array();

			return false;
		}


		if( true === $this->noBcInit ) {

			return false;
		}


		$setName = $this->getEvent()->getRouteMatch()->getMatchedRouteName() == 'home' ? 'esc.com.main' : 'esc.com.sub';
		if ($this->request->getQuery('contentviewcode')) {
                    $this->contentViewCode = $this->request->getQuery('contentviewcode');
                }

		$bcElements = $this->wbServClient->getElements( $setName , $this->contentViewCode );

		//print_r( $setName );

		foreach( $bcElements as $key => $bcElementData ) {

			$bcElements[ $key ] = \System\BannerCampaign\Element\ElementFactory::createElement( $bcElementData );
		}

                //foreach ($bcElements as $bcElement) {
                //    if ($bcElement->getPlace()->getName() == 'slider') {
                //        var_dump($bcElement);
                //    }
                //}
                //die();

		return $bcElements;
	}


	protected function loadElementsFoCode() {

		try {

			$bcElements = \System\BannerCampaign\Element\ElementFactory::get( $this->bcViewModeElements );
			$bcElements = \System\BannerCampaign\Element\ElementDecorator::decorateElement( $bcElements , $this->lang );

			$isData = false;
			$targetSpecs = array();
			foreach( $bcElements as $bcElement ) {
				$bcElementTarget = $bcElement->getTarget()->toArray();

				if( !isset( $bcElementTarget[ 'specs_value' ] ) )
					continue;

				$specValues = explode( ',' , $bcElementTarget[ 'specs_value' ] );

				foreach( $specValues as $specValue ) {
					$specValue = explode( '_' , $specValue );
					if( 2 !== count( $specValue ) )
						continue;

					$isData = true;

					$specType = preg_replace( array( '/l/' , '/p/' , '/i/' ) , array( 'MAIN' , 'CURR' , 'INTE' ) , $specValue[ 0 ] );

					$targetSpecs[ $specType ][ $specValue[ 1 ] ] = $specValue[ 1 ];
				}
			}


			if( !$isData )
				$targetSpecs = array( 'MAIN' => array( '01' ) );


			$UserImpersonate = new UserImpersonate( new \System\Authentication\AuthenticationIdentity( array() ) );
			$UserImpersonate->setSpect( $targetSpecs );

			return $bcElements;

		} catch( \Exception $e ) {

			$this->logger()->err( $e->getMessage() , __METHOD__ , __LINE__ );
		}

		return false;
	}



	public function initBannerCampaigns() {

		$this->initBannerCampaignEx();
	}


	public function logger() {
			return $this->logger;
	}

	// ustawianie schematu kontentu
	public function setSchema() {

		\System\Database\AbstractTable::setSchema( $this->lang .'_content');
	}

	public function getSchema() {

		return \System\Database\AbstractTable::getContentSchema();
	}

    public function getMainSiteID() {
        return $this->cfg['main_site_id'];
    }

    public function isMainSite($mainSite = null) {
        $mainSites = $this->cfg['mainSiteId'];
        unset($mainSites['esculap']);

        if ($mainSite == null) {
            foreach ($mainSites as $mainSite) {
                if ($this->getMainSiteID() == $mainSite) {
                    return true;
                }
            }
        } else {
            if (isset($this->cfg['mainSiteId'][$mainSite]) && $this->getMainSiteID() == $this->cfg['mainSiteId'][$mainSite]) {
                return true;
            } else {
                return false;
            }
        }

        return false;
    }

    public function getView() {
        return $this->view;
    }

    public function setLayoutContent($content, $type = null) {
        if ($type != null) {
            $contents = $this->layout()->contents;
            $contents[$type] = $content;
            $this->layout()->contents = $contents;

            if (!$this->layout()->getVariable('eventDrivenContent', false)) {
                $this->layout()->setVariable('eventDrivenContent', true);
            }
        }
    }

    public function getSponsoredNews($sponsoredNewsPlace = '') {
        $sponsoredNewsIDs = [];
        $sponsoredNewsArray = BannerDisplayer::getElementsFromPlace($sponsoredNewsPlace);
		is_array($sponsoredNewsArray) ? : $sponsoredNewsArray = [];
        foreach ($sponsoredNewsArray as $key => $sponsoredNews) {
            if ($sponsoredNews->isFake()) {
                unset($sponsoredNewsArray[$key]);
            } else {
                if (method_exists($sponsoredNews, 'getContentId')) {
                    $sponsoredNewsIDs[] = $sponsoredNews->getContentId();
                }
            }
        }
    }

}
