<?php
namespace System\Controller;

use System\Content\ContentFactory,
    System\Content\Config as Config;
use Zend\View\Model\ViewModel;

class EsculapBaseActionsInsecure extends EsculapBaseActions
{

    protected $initLayout = true;
    protected $initBannerCampaigns = true;

    protected function init()
    {
        parent::init();

        if ($this->isMainSite()) {
            $this->layout()->setTemplate('MainSiteLayout' . $this->cfg['main_site_id']);

            $route = $this->params()->fromRoute();
            $module = explode('\\', $route['controller'])[0];
            $controller = explode('\\', $route['controller']);
            $controller = end($controller);

            $module = preg_replace('/\B([A-Z])/', '_$1', $module);
            $controller = preg_replace('/\B([A-Z])/', '_$1', $controller);
            $action = preg_replace('/\B([A-Z])/', '_$1', $route['action']);

            $module = strtolower($module);
            $controller = strtolower($controller);
            $action = strtolower($action);

            //$template = $module . '/' . $controller . '/' . $action . '_mainsite' . $this->cfg['main_site_id'] . '.phtml';
            $template = 'content/' . $module . '/' . $action . '_mainsite' . $this->cfg['main_site_id'] . '.phtml';
            $resolver = $this->getEvent()->getApplication()->getServiceManager()->get('Zend\View\Resolver\TemplatePathStack');

            if ($resolver->resolve($template) !== false) {
                $this->view->setTemplate($template);
            }
        } else {
            $this->layout()->setTemplate('ApplicationLayout');
        }
    }

    protected function auth()
    {

        $this->getEventManager()->trigger('authenticate', $this, array('authRequired' => false));
    }

    protected function initLayout()
    {
        if ((
                $this->getEvent()->getRouteMatch()->getMatchedRouteName() == 'news_content' ||
                $this->getEvent()->getRouteMatch()->getMatchedRouteName() == 'publications_content' ||
                $this->getEvent()->getRouteMatch()->getMatchedRouteName() == 'questionnaire_view' ||
                $this->getEvent()->getRouteMatch()->getMatchedRouteName() == 'event_content'
            ) &&
            !$this->currentWatchContentId
        ) {
            $this->currentWatchContentId = $this->getEvent()->getRouteMatch()->getParam('id');
        }

        if ($this->isMainSite()) {
            $contentsConfig = $this->mainSiteStrategy->generateLayoutConfiguration($this);
            $setMainNews = false;
            $setMainPublication = false;
        } else {
            $newsConfig = [
                'CONTENT_TYPE'   => 'news',
                'ORDER_SPECS'    => $this->identity ? $this->identity->full_spec_code : null,
                'ROWS_NUMBER'    => 10,
                'ROWS_START'     => 0,
                'NOT_CONTENT_ID' => array($this->currentWatchContentId),
                'GET_MAIN_SITE'  => true,
                'MAIN_SITE_ID'   => (bool)null
            ];

            if ($this->getEvent()->getRouteMatch()->getMatchedRouteName() == 'news_index') {
                $newsConfig['CONTENT_OF_THE_DAY'] = true;
            }

            $contentsConfig = [
                'news'        => new Config($newsConfig),
                'publication' => new Config([
                        'CONTENT_TYPE'   => 'publication',
                        'ORDER_SPECS'    => $this->identity ? $this->identity->full_spec_code : null,
                        'ROWS_NUMBER'    => 5,
                        'ROWS_START'     => 0,
                        'NOT_CONTENT_ID' => array($this->currentWatchContentId),
                        'GET_MAIN_SITE'  => true,
                        'MAIN_SITE_ID'   => (bool)null,
                    ]),
                'event'       => new Config([
                        'CONTENT_TYPE'    => 'event',
                        'ORDER_SPECS'     => $this->identity ? $this->identity->full_spec_code : null,
                        'ROWS_NUMBER'     => 2,
                        'ROWS_START'      => 0,
                        'NOT_CONTENT_ID'  => array($this->currentWatchContentId),
                        'GET_MAIN_SITE'   => true,
                        'MAIN_SITE_ID'    => (bool)null,
                        'UPCOMING_EVENTS' => true,
                        'ESCULAP_ORDER'   => 'date_start ASC NULLS LAST',
                    ]),
                'quiz'        => new Config([
                        'CONTENT_TYPE'   => 'quiz',
                        'ROWS_NUMBER'    => 1,
                        'ROWS_START'     => 0,
                        'NOT_CONTENT_ID' => array($this->currentWatchContentId),
                        'GET_COMMENTS'   => 1,
                        'GET_MAIN_SITE'  => true,
                        'MAIN_SITE_ID'   => 1,
                    ]),
            ];
            $setMainNews = true;
            $setMainPublication = true;
        }
                
        $news = ContentFactory::factory($contentsConfig['news']);
        $publications = ContentFactory::factory($contentsConfig['publication']);
        $events = ContentFactory::factory($contentsConfig['event']);
        $quiz = ContentFactory::factory($contentsConfig['quiz']);
        if (isset($contentsConfig['archivalPublication'])) {
            $archivalPublications = ContentFactory::factory($contentsConfig['archivalPublication']);
        } else {
            $archivalPublications = array();
        }

        if (isset($news[0]) && $setMainNews) {
            $mainNews = array($news[0]);
            unset($news[0]);
        } else {
            $mainNews = array();
        }
        if (isset($publications[0]) && $setMainPublication) {
            $mainPublication = array($publications[0]);
            unset($publications[0]);
        } else {
            $mainPublication = array();
        }

        $specialForYou = array();

        $this->layout()->contents = array(
            'news'          => $news,
            'publ'          => $publications,
            'events'        => $events,
            'quiz'          => $quiz,
            'archivalPubl'  => $archivalPublications,
            'mainNews'      => $mainNews,
            'mainPubl'      => $mainPublication,
            'specialForYou' => $specialForYou,
        );

        if ($this->isMainSite()) {
            $this->mainSiteStrategy->generateLayoutContent($this);
        }
    }

    protected function initBannerCampaignEx()
    {
        $bcElements = array(
            new \System\BannerCampaign\Element\Billboard(array(
                'bcId'          => 0,
                'id'            => 3,
                'bannerUrl'     => 'ZCHL8FN6EHEM',
                'bannerMime'    => 'image/png',
                'billboardUrl'  => '22KPJZYIX8CP',
                'billboardMime' => 'image/png',
                'url'           => '//esculap.com?utm_source=esculap&utm_medium=billboard&utm_campaign=defcamp',
                'place'         => 1,
                'fake'          => true
            ))
        );

        if (!$this->isMainSite('bezsennosc')) {
            $bcElements = $this->getElementFormWbserv();
        }

        \System\View\Helper\BannerDisplayer::init($bcElements);
    }

    protected function initWbserv()
    {

        if (null !== $this->wbServClient) {
            return;
        }

        try {


            try {

                $userCode = null;

                $identity = $this->identity();


                if (null !== $identity &&
                    null !== $identity->user_code
                ) {

                    $userCode = $identity->user_code;
                }
            } catch (\Exception $e) {

            }

            $this->wbServClient = new \System\Soap\WbServClient(
                $this->_siteCode, $userCode, $this->lang, $this->cfg['wbserv_addr']
            );
        } catch (\Exception $e) {

            echo $e->getMessage();

            $this->logger()->err($e->getMessage(), __NAMESPACE__, __LINE__);
            $this->selectedBcElements = array();

            return false;
        }
    }

    protected function getElementFormWbserv()
    {
        // Dla locala
        //return;
        if (false == $this->initBannerCampaigns) {
            return;
        }

        $this->initWbserv();
        if (true === $this->noBcInit) {

            return false;
        }

        $setName = 'esc.com.unsec';
        if ($this->request->getQuery('contentviewcode')) {
            $this->contentViewCode = $this->request->getQuery('contentviewcode');
        }
        $bcElements = $this->wbServClient->getElements($setName, $this->contentViewCode);


        foreach ($bcElements as $key => $bcElementData) {

            $bcElements[$key] = \System\BannerCampaign\Element\ElementFactory::createElement($bcElementData);
        }


        return $bcElements;
    }

}

?>