<?php

namespace System\Controller;

use System\Content\ContentFactory,
    System\Content\Config as Config,
    System\Impersonate\User as UserImpersonate,
    Model\MainViewCode;
use Zend\View\Model\ViewModel;
use Zend\Http\Header\SetCookie;

class EsculapBaseActionsSecure extends EsculapBaseActions
{

    protected $initLayout = true;
    protected $initBannerCampaigns = true;

    protected function init()
    {
        parent::init();

        if ($this->isMainSite()) {
            $this->layout()->setTemplate('MainSiteLayout' . $this->cfg['main_site_id']);

            $route = $this->params()->fromRoute();
            $module = explode('\\', $route['controller'])[0];
            $controller = explode('\\', $route['controller']);
            $controller = end($controller);

            $module = preg_replace('/\B([A-Z])/', '_$1', $module);
            $controller = preg_replace('/\B([A-Z])/', '_$1', $controller);
            $action = preg_replace('/\B([A-Z])/', '_$1', $route['action']);

            $module = strtolower($module);
            $controller = strtolower($controller);
            $action = strtolower($action);

            //$template = $module . '/' . $controller . '/' . $action . '_mainsite' . $this->cfg['main_site_id'] . '.phtml';
            $template = 'content/' . $module . '/' . $action . '_mainsite' . $this->cfg['main_site_id'] . '.phtml';
            $resolver = $this->getEvent()->getApplication()->getServiceManager()->get('Zend\View\Resolver\TemplatePathStack');

            if ($resolver->resolve($template) !== false) {
                $this->view->setTemplate($template);
            }
        } else {
            $this->layout()->setTemplate('ApplicationLayout');
        }
    }

    protected function auth()
    {
        // Dla locala
        //$this->identity = new \System\Authentication\AuthenticationIdentity(array());
        //return;
        if (!$this->preCheckContentDebug() && !$this->isBcViewMode(true) && !$this->isContentView(true)) {
            $ar = true;
            if ('home' == $this->getEvent()->getRouteMatch()->getMatchedRouteName()) {
                $ar = false;
            }

            if (!isset($this->cfg['auth_require']) || $this->cfg['auth_require']) {
                $this->getServiceLocator()->get('EsculapEvent')->trigger('authenticate', $this, array('authRequired' => $ar));
            }

            if ($this->identity()) {
                $this->_userCode = $this->identity()->user_code;
                $cookie = new SetCookie(); // Zend\Http\Header\SetCookie instance
                $cookie->setDomain('esculap.com');
                $cookie->setPath('/');
                $cookie->setName('usertype_esculap');
                $cookie->setExpires(time() + 365*24*60*60);
                $cookie->setValue($this->identity()->user_type);
                $response = $this->getResponse()->getHeaders();
                $response->addHeader($cookie);
                $cookie1 = new SetCookie(); // Zend\Http\Header\SetCookie instance
                $cookie1->setDomain('esculap.com');
                $cookie1->setPath('/');
                $cookie1->setName('tendency_esculap');
                $cookie1->setExpires(time() + 365*24*60*60);
                $cookie1->setValue($this->identity()->spec_ok_pl);
                $response = $this->getResponse()->getHeaders();
                $response->addHeader($cookie1);
            } else {
                header("Location:" . $this->url()->fromRoute('main'));
                exit();
            }
        }
    }

    protected function initLayout()
    {
        if ($this->preCheckContentDebug() || $this->isBcViewMode(true) || $this->isContentView(true)) {
            if ($this->preCheckContentDebug()) {
                // przykładowy input wykorzystujący plik contentPhrase.js:
                // <input type="button" value="testowanie podglądu contentu" class="content_debug_test" spec="SP.WRU.X6,SP.WRU.47,SP.WRU.3F,SP.WRU.6V" lang="ru">
                // atrybut spec może być ustaiony na auto - wtedy automatycznie wybrane zostaną specializacje dla tego contentu

                $this->initLayout = true;
                $this->initBannerCampaigns = false;

                $this->layout()->contentDebug = 1;
                $this->contentDebug = 1;

                $this->identity = new \System\Authentication\AuthenticationIdentity(array());
                $this->setContentDebug();

                // ustawiam flagi unieważniające zliczanie odsłon kontentu i banerów
                \System\BannerCampaign\BannerObserver::$COUNT_STATISTICS = false;
            } elseif ($this->isBcViewMode()) {

                $this->identity = new \System\Authentication\AuthenticationIdentity(array());

                $this->initLayout = true;
                $this->initBannerCampaigns = true;

                $this->commentsAddPossybility = false;
                $this->contentEvaluationPossibility = false;
                $this->contentDebug = 1;

            } elseif ($this->isContentView()) {
                $codeData = $this->isContentView();

                $this->initLayout = true;
                $this->initBannerCampaigns = true;

                $specs = array();
                $codeDataSpecs = explode(',', $codeData['specs']);
                foreach ($codeDataSpecs as $specCode) {
                    $specCode = explode('_', $specCode);
                    $specType = preg_replace(array('/l/', '/p/', '/i/'), array('MAIN', 'CURR', 'INTE'), $specCode[0]);
                    $specCode = $specCode[1];

                    $specs[$specType][$specCode] = $specCode;
                }

                //var_dump($specs);
                $this->identity = new \System\Authentication\AuthenticationIdentity(array());
                $UserImpersonate = new UserImpersonate($this->identity);
                $UserImpersonate->setSpect($specs);
            }
        }

        if ((
                $this->getEvent()->getRouteMatch()->getMatchedRouteName() == 'news_content' ||
                $this->getEvent()->getRouteMatch()->getMatchedRouteName() == 'publications_content' ||
                $this->getEvent()->getRouteMatch()->getMatchedRouteName() == 'questionnaire_view' ||
                $this->getEvent()->getRouteMatch()->getMatchedRouteName() == 'event_content'
            ) &&
            !$this->currentWatchContentId
        ) {
            $this->currentWatchContentId = $this->getEvent()->getRouteMatch()->getParam('id');
        }

        if ($this->isMainSite()) {
            $contentsConfig = $this->mainSiteStrategy->generateLayoutConfiguration($this);
            $setMainNews = false;
            $setMainPublication = false;
        } else {
            $contentsConfig = [
                'news'        => new Config([
                        'CONTENT_TYPE'   => 'news',
                        'ORDER_SPECS'    => $this->identity ? $this->identity->full_spec_code : null,
                        'ROWS_NUMBER'    => 5,
                        'ROWS_START'     => 0,
                        'NOT_CONTENT_ID' => array($this->currentWatchContentId),
                        'GET_MAIN_SITE'  => true,
                        'MAIN_SITE_ID'   => (bool)null,
                    ]),
                'publication' => new Config([
                        'CONTENT_TYPE'   => 'publication',
                        'ORDER_SPECS'    => $this->identity ? $this->identity->full_spec_code : null,
                        'ROWS_NUMBER'    => 5,
                        'ROWS_START'     => 0,
                        'NOT_CONTENT_ID' => array($this->currentWatchContentId),
                        'GET_MAIN_SITE'  => true,
                        'MAIN_SITE_ID'   => (bool)null,
                    ]),
                'event'       => new Config([
                        'CONTENT_TYPE'    => 'event',
                        'ORDER_SPECS'     => $this->identity ? $this->identity->full_spec_code : null,
                        'ROWS_NUMBER'     => 2,
                        'ROWS_START'      => 0,
                        'NOT_CONTENT_ID'  => array($this->currentWatchContentId),
                        'GET_MAIN_SITE'   => true,
                        'MAIN_SITE_ID'    => (bool)null,
                        'UPCOMING_EVENTS' => true,
                        'ESCULAP_ORDER'   => 'date_start ASC NULLS LAST',
                    ]),
                'quiz'        => new Config([
                        'CONTENT_TYPE'   => 'quiz',
                        'ROWS_NUMBER'    => 1,
                        'ROWS_START'     => 0,
                        'NOT_CONTENT_ID' => array($this->currentWatchContentId),
                        'GET_COMMENTS'   => 1,
                        'GET_MAIN_SITE'  => true,
                        'MAIN_SITE_ID'   => 1,
                    ]),
            ];
            $setMainNews = true;
            $setMainPublication = true;
        }

        if (class_exists('\Application\Legacy\System\Content\CachedContentFactory')) {
            $factory = new \Application\Legacy\System\Content\CachedContentFactory();
            $news = $factory::factory($contentsConfig['news']);
            $publications = $factory::factory($contentsConfig['publication']);
            $events = $factory::factory($contentsConfig['event']);
            $quiz = $factory::factory($contentsConfig['quiz']);
            if (isset($contentsConfig['archivalPublication'])) {
                $archivalPublications = $factory::factory($contentsConfig['archivalPublication']);
            } else {
                $archivalPublications = array();
            }
        } else {
            $news = ContentFactory::factory($contentsConfig['news']);
            $publications = ContentFactory::factory($contentsConfig['publication']);
            $events = ContentFactory::factory($contentsConfig['event']);
            $quiz = ContentFactory::factory($contentsConfig['quiz']);
            if (isset($contentsConfig['archivalPublication'])) {
                $archivalPublications = ContentFactory::factory($contentsConfig['archivalPublication']);
            } else {
                $archivalPublications = array();
            }
        }

        if (isset($news[0]) && $setMainNews) {
            $mainNews = array($news[0]);
            unset($news[0]);
        } else {
            $mainNews = array();
        }
        if (isset($publications[0]) && $setMainPublication) {
            $mainPublication = array($publications[0]);
            unset($publications[0]);
        } else {
            $mainPublication = array();
        }

        $specialForYou = array();

        $this->layout()->contents = array(
            'news'          => $news,
            'publ'          => $publications,
            'events'        => $events,
            'quiz'          => $quiz,
            'archivalPubl'  => $archivalPublications,
            'mainNews'      => $mainNews,
            'mainPubl'      => $mainPublication,
            'specialForYou' => $specialForYou,
        );

        if ($this->isMainSite()) {
            $this->mainSiteStrategy->generateLayoutContent($this);
        }
    }

    /**
     * Ustawia debugowanie kontentu
     */
    protected function setContentDebug()
    {

        if ($this->preCheckContentDebug()) {


            // ustawianie języka i schematu
            $lang = $this->request->getQuery('lang');
            $this->lang = $lang;


            // pobranie listy specializacji przesłanych getem
            $specs = $this->request->getQuery('spec');

            $this->getEventManager()->trigger(
                'content_debug',
                $this,
                array(
                    'state'     => true,
                    'namespace' => __NAMESPACE__,
                    'line'      => __LINE__
                )
            );

            if ($specs == 'auto') {

                $this->setSchema();
                $id = $this->getEvent()->getRouteMatch()->getParam('id');


                $RelContentSpec = new \Model\RelContentSpec;
                $specs = $RelContentSpec->getSpecsForContent($id);

                $specsValid = false;
                foreach ($specs as $spec) {
                    if (strlen($spec) > 0) {
                        $specsValid = true;
                        break;
                    }
                }

                // zabezpieczenie przed żuceniem wyjądku w przypadku gdy niema żadnej specializacji // język polski umar
                if (!count($specs) || !$specsValid) {
                    $specs = array(1);
                }

            } elseif (strlen($specs)) {

                $specs = explode(',', $this->request->getQuery('spec', array()));

            } else {

                $specs = array(1);
            }


            $UserImpersonate = new UserImpersonate($this->identity);
            $UserImpersonate->setSpect($specs);
        }
    }


    /**
     * Wyłączakonieczność logowania
     */
    protected function preCheckContentDebug()
    {

        if (null !== $this->request->getQuery('content_debug', null) &&
            ('news_content' == $this->getEvent()->getRouteMatch()->getMatchedRouteName() ||
                'news_contentnews_view_unsaved' == $this->getEvent()->getRouteMatch()->getMatchedRouteName() ||
                'event_content' == $this->getEvent()->getRouteMatch()->getMatchedRouteName() ||
                'event_contentnews_view_unsaved' == $this->getEvent()->getRouteMatch()->getMatchedRouteName() ||
                'publications_content' == $this->getEvent()->getRouteMatch()->getMatchedRouteName() ||
                'publications_content_view_unsaved' == $this->getEvent()->getRouteMatch()->getMatchedRouteName() ||
                'videos_content' == $this->getEvent()->getRouteMatch()->getMatchedRouteName() ||
                'videos_content_view_unsaved' == $this->getEvent()->getRouteMatch()->getMatchedRouteName())
        ) {


            return true;
        }

        return false;
    }

    /**
     * Sprawdzam czy mamy do czynienia z podglądem treści "judasz"
     *
     */
    protected function isContentView($checkOnly = false)
    {

        static $codeData = false;

        if (false !== $codeData) {
            return $codeData;
        }

        if (null !== $this->request->getQuery('contentviewcode', null)) {

            if ($checkOnly) {
                return true;
            }

            //$MainViewCode = new MainViewCode(\System\Database\StandardAdapter::getInstance($this->getServiceLocator()->get('config')['database']['adapter']['int_esculap']));
            $MainViewCode = new MainViewCode; //(\System\Database\StandardAdapter::getInstance($this->getServiceLocator()->get('config')['database']['adapter']['int_esculap']));

            try {
                $codeData = $MainViewCode->loadData($this->request->getQuery('contentviewcode'));

                if (false === ($codeData = $MainViewCode->loadData($this->request->getQuery('contentviewcode')))) {
                    return false;
                }

                $this->viewModeUrl = '?contentviewcode=' . $codeData['code'];
                $this->layout()->addToHrefs = $this->viewModeUrl;

                $this->contentView = true;
                $this->contentViewCode = $codeData['code'];

            } catch (\Exception $e) {

                $this->log()->err("Nieudana próba odczytania kodu. " . $e->getMessage(), __METHOD__, __LINE__);

                return false;
            }

            return true;
        }

        return false;
    }

}

?>