<?php
namespace System\Controller\Plugin;

use Zend\Config\Config;
use Zend\Mvc\Controller\Plugin\AbstractPlugin;
use Zend\Session\Container as SessionContainer;
use System\Form\QueryParser as QueryParser;

class ListRequestParser extends AbstractPlugin
{
    public function parseRequest($mvcEvent, array $query = array('conditions' => array(), 'order' => array('id' => 'desc'), 'limit' => 25, 'page' => 1, 'adapt' => false))
    {
        $request = $mvcEvent->getRequest();
        $queryCache = new SessionContainer(trim(str_replace('%2C', '_', str_replace('/', '_', $request->getRequestUri())), '_'));

        isset($query['conditions']) ? : $query['conditions'] = array();
        isset($query['order']) ? : $query['order'] = array('id' => 'desc');
        isset($query['limit']) ? : $query['limit'] = 25;
        isset($query['page']) ? : $query['page'] = 1;

        if ($request->getPost('clear_session_cache') == true || true) {
            $queryCache->getManager()->getStorage()->clear(trim(str_replace('%2C', '_', str_replace('/', '_', $request->getRequestUri())), '_'));
        }

        if ($request->isXmlHttpRequest()) {
            if ($queryCache->query) {
                $query = $queryCache->query;
            }

            for ($i = 1; ; $i++) {
                if ($mvcEvent->getRouteMatch()->getParam('filter' . $i)) {
                    $paramGet = $mvcEvent->getRouteMatch()->getParam('filter' . $i);
                    $value = explode(',', $paramGet, 2)[0] . '.' . explode(',', $paramGet, 2)[1];
                    $query['conditions'][explode(',', $paramGet, 2)[0]] = $value;
                } else {
                    break;
                }
            }

            if ($mvcEvent->getRouteMatch()->getParam('order')) {
                $query['order'] = explode(',', $mvcEvent->getRouteMatch()->getParam('order'), 2);
                $query['order'] = array($query['order'][0] => $query['order'][1]);
            }

            if ($mvcEvent->getRouteMatch()->getParam('page')) {
                $query['page'] = (int)$mvcEvent->getRouteMatch()->getParam('page');
            }

            if ($mvcEvent->getRouteMatch()->getParam('limit')) {
                $query['limit'] = (int)$mvcEvent->getRouteMatch()->getParam('limit');
            }

            foreach ($query as $queryParameter => &$queryValue) {
                if ($request->getPost($queryParameter)) {
                    $queryValue = $request->getPost($queryParameter);
                }
            }

            $queryCache->query = $query;
        } elseif (isset($queryCache->query)) {
            $query = $queryCache->query;
        }

        if (isset($query['adapt']) && $query['adapt']) {
            foreach ($query['conditions'] as $key => $condition) {
                $condition = explode('.', $condition);
                $condition = end($condition);
                $condition = ' ' . $condition;
                $query['conditions'][$key] = $condition;
            }
        }

        return array_merge(QueryParser::parseQuery($query['conditions'], $query['order'], $query['limit'], $query['page']), array('relational_pagination' => $request->getPost('relational_pagination')));
    }

    public
    function getCachedRequest(
        $identifier, array $query = array('conditions' => array(), 'order' => array('id' => 'desc'), 'limit' => 25, 'page' => 1)
    ) {
        $queryCache = new SessionContainer(trim(str_replace('%2C', '_', str_replace('/', '_', $identifier)), '_'));

        isset($query['conditions']) ? : $query['conditions'] = array();
        isset($query['order']) ? : $query['order'] = array('id' => 'desc');
        isset($query['limit']) ? : $query['limit'] = 25;
        isset($query['page']) ? : $query['page'] = 1;

        if (isset($queryCache->query)) {
            $query = $queryCache->query;
        }

        return QueryParser::parseQuery($query['conditions'], $query['order'], $query['limit'], $query['page']);
    }

    public
    function isRelational(
        $controller
    ) {
        if ($controller->getRequest()->isXmlHttpRequest() && (
                $controller->getEvent()->getRouteMatch()->getParam('relational_view') != 'true' ||
                $controller->View->getVariable('request')['relational_pagination']
            )
        ) {
            return true;
        } else {
            return false;
        }
    }

    public
    function calculatePage(
        $request, $countAll
    ) {
        $request['pages'] = (int)ceil((int)$countAll / (int)$request['limit']);
        if ($request['pages'] <= 0) {
            $request['pages'] = 1;
        }

        return $request;
    }

    public
    function adaptForDoctrine(
        array $request
    ) {
        $config = array();

        if (isset($request['where']) && count($request['where']) > 0) {
            foreach ($request['where'] as $where) {
                reset($where);
                $field = trim(key($where));
                if (!is_array($where[$field])) {
                    $value = trim($where[$field]);
                } else {
                    $value = $where[$field];
                }

                $config['where'][] = array('field' => $field, 'value' => $value);
            }
        }

        if (isset($request['order'])) {
            reset($request['order']);
            $field = trim(key($request['order']));
            $direction = trim($request['order'][$field]);

            $config['order'] = array('field' => $field, 'direction' => $direction);
        }

        if (isset($request['limit'])) {
            $config['limit'] = $request['limit'];
        }

        if (isset($request['page']) && isset($request['limit'])) {
            $request['page'] < 1 ? $page = 0 : $page = (int)$request['page'] - 1;
            $offset = $page * $request['limit'];

            $config['page'] = $request['page'];
            $config['offset'] = $offset;
        }

        $config = new Config($config);

        return $config;
    }

}
