<?php
namespace System\Database;

use Zend\Db\TableGateway\AbstractTableGateway;
use Application\Legacy\TableIdentifier;

abstract class AbstractTable extends AbstractTableGateway {



    protected static $tableName = null;
    protected static $schema = null;
    protected static $alias = null;
    protected static $tableColumns = null;
    protected static $_contentSchema = null;
    protected static $_defaultTsearchLanguage = 'pl';
    protected static $_defaultTsearchLanguageTranslations = array(
        'pl' => 'polish',
        'en' => 'english',
        'ru' => 'russian',
        'ua' => 'russian',
    );
    protected static $searchPatch = 'esculap_int , esculap , public , postfix, archiwum';
    // tablica translacji nazw, każdy model definiuje własne reguły
    protected $rules = [];

    final public function __construct($adapter = null) {


        if (null !== $adapter) {
            $this->adapter = $adapter;
        } else {
            $this->adapter = StandardAdapter::getInstance();
        }

        if (static::$schema) {
            $this->table = new ExtendedTableIdentifier(static::$tableName, static::$schema);
        } elseif (static::$tableName) {
            $this->table = new ExtendedTableIdentifier(static::$tableName);
        } else {
            $this->table = new ExtendedTableIdentifier($this->table);
        }

        try {
            $this->initialize();
        } catch (\Exception $e) {

            trigger_error($e->getMessage());
        }
    }

    public function fetchAll($queryResult) {
        $result = array();

        foreach ($queryResult as $k => $v) {
            $result[$k] = $v;
        }

        return $result;
    }

    public function get($data) {
        return $this->adapter->createStatement($data)->execute();
    }

    public function fetchRow($query = null) {

        $result = array();
        $res = $this->select($query);

        foreach ($res as $k => $v) {
            return (array)$v;
        }

        return array();
    }

    public static function getContentSchema() {
        return self::$_contentSchema;
    }

    public function getContnetSchemaLangSc() {

        if (null == self::$_contentSchema) return null;

        $res = explode('_', self::$_contentSchema);

        return $res[0];
    }

    public static function setSchema($contentSchema) {

        StandardAdapter::getInstance()->createStatement('set search_path to ' . $contentSchema . ' , ' . self::$searchPatch)->execute();

        return self::setContentSchema($contentSchema);
    }

    public static function setContentSchema($contentSchema) {
        self::$_contentSchema = $contentSchema;
    }

    public function table($name = null) {
        if (null !== ($schema = self::getContentSchema())) {
            return is_null($name) ? $schema . '.' . $this->table : $schema . '.' . $name;
        }

        return is_null($name) ? $this->table : $name;
    }

    /**
     * Ustawia domyślny język dla wyszukiwarek.
     */
    public function setDefaultsTsearchLanguage($sc) {
        if (array_key_exists($sc, self::$_defaultTsearchLanguageTranslations)) {

            self::$_defaultTsearchLanguage = $sc;
        }
    }

    /**
     * Zwraca ustawiony język dka wyszukiwarek lub domyślny
     */
    public static function getTsearchConfig() {


        if (null == self::$_contentSchema) {
            return self::$_defaultTsearchLanguageTranslations[self::$_defaultTsearchLanguage];
        }

        $sc = preg_replace('/^([a-z]+)_[a-z]+$/', '$1', strtolower(self::$_contentSchema));

        if (!array_key_exists($sc, self::$_defaultTsearchLanguageTranslations)) {

            return self::$_defaultTsearchLanguageTranslations[self::$_defaultTsearchLanguage];
        }
        return self::$_defaultTsearchLanguageTranslations[$sc];
    }

    /**
     * 	Tłumaczenie kluczy tablicy podanej w argumencie zgodnie z regułami we właściwości roles
     */
    public function translate($data) {

        if (!is_array($this->rules)) {
            return $data;
        }


        $result = array();


        foreach ($data as $key => $value) {

            if (is_array($value)) {

                $result[$key] = $this->translate($value);
            } else {

                if (array_key_exists($key, $this->rules)) {

                    $result[$this->rules[$key]] = $value;
                } else {

                    $result[$key] = $value;
                }
            }
        }


        return $result;
    }

    public function getTable() {
        if (strpos($this->table, '.') !== false) {
            return (string)explode('.', $this->table)[1];
        } else {
            return (string)$this->table;
        }
    }

    public function getAdapter() {
        return $this->adapter;
    }

    public static function getTableName() {
        return static::$tableName;
    }

    public static function getSchema() {
        return static::$schema;
    }

    public static function getAlias() {
        return static::$alias;
    }

    public static function getTableColumns() {
        return static::$tableColumns;
    }

    public static function getNamespacedTableColumns() {
        $namespacedTableNames = array();
        foreach (static::$tableColumns as $tableColumn) {
            $namespacedTableNames[static::$alias . '.' . $tableColumn] = $tableColumn;
        }
        return $namespacedTableNames;
    }

    protected function prepareQuery($query, array $queryConfig = array()) {
        if (isset($queryConfig['order'])) {
            if (count($queryConfig['order']) == 1) {
                reset($queryConfig['order']);
                $orderField = key($queryConfig['order']);
                $orderDirection = $queryConfig['order'][$orderField];
                if ((strtolower($orderDirection) == 'asc' || strtolower($orderDirection) == 'desc') && in_array($orderField, $this->fields)) {
                    $query .= " ORDER BY {$orderField} {$orderDirection} ";
                }
            }
        }

        if (isset($queryConfig['limit'])) {
            $queryConfig['limit'] < 1 ? $queryConfig['limit'] = 1 : $queryConfig['limit'] = (int)$queryConfig['limit'];
            $query .= " LIMIT {$queryConfig['limit']} ";
        }

        if (isset($queryConfig['page'])) {
            $queryConfig['page'] < 1 ? $queryConfig['page'] = 1 : $queryConfig['page'] = (int)$queryConfig['page'];
            $queryConfig['page'] = $queryConfig['page'] - 1;
            $offset = $queryConfig['page'] * $queryConfig['limit'];
            $query .= " OFFSET {$offset} ";
        }

        return $query;
    }

}

?>
