<?php
namespace System\Form;

use System\Validator\RegexBase;
use System\Translator;
use Zend\Validator\NotEmpty;
use Zend\Validator\Regex;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Model\Entity\AvatarType as AvatarTypeEntity;
use Model\TableGateway\AvatarType as AvatarTypeTableGateway;

class AvatarType extends AbstractForm implements DrawableInterface {

    protected $name = 'avatar_type';

    public function __construct($formMode, $data = null) {
        $translator = Translator\TranslatorEx::get();
        $tableGateway = new AvatarTypeTableGateway();

        if (defined('parent::' . $formMode)) {
            $this->setFormMode($formMode);
        } else {
            $this->setFormMode(parent::ADD);
        }

        if ($data && $this->getFormMode() == parent::EDIT) {
            $data = $this->adaptData($data);
        }
        
        $fields = array(
            array('name' => 'name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('label' => $translator->translate('name', 'avatarTypeDictionary'))),
            array('name' => 'submit-button', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('class' => 'form-submit', 'value' => $translator->translate('save'))),
        );

        if ($this->formMode == parent::VIEW) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox') {
                    $field['attributes']['disabled'] = 'disabled';
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }

        $this->validation = array(
            'name' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'regex', 'break_chain_on_failure' => true, 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(Regex::NOT_MATCH => $translator->translate('error_name_identifier_invalid')))),
                    array('name' => 'string_length', 'break_chain_on_failure' => true, 'options' => array('max' => 100, 'messages' => array(\Zend\Validator\StringLength::TOO_LONG => $translator->translate('error_value_more_than_100')))),
                    array('name' => 'Zend\Validator\Db\NoRecordExists', 'options' => array('adapter' => $tableGateway->getAdapter(), 'schema' => AvatarTypeTableGateway::getSchema(), 'table' => AvatarTypeTableGateway::getTableName(), 'field' => 'name', 'exclude' => (isset($data['name']) && $data['name']) ? array('field' => 'name', 'value' => $data['name']) : null, 'messages' => array(\Zend\Validator\Db\NoRecordExists::ERROR_RECORD_FOUND => $translator->translate('record_already_exists')))),
                )
            ),
        );

        parent::init($fields);

        $this->_messages = array(
            'name' => array(
                $translator->translate('error_value_is_required'),
                $translator->translate('error_name_identifier_invalid'),
                $translator->translate('error_value_more_than_100'),
                $translator->translate('record_already_exists'),
            )
        );

        if ($data && $this->getFormMode() == parent::EDIT) {
            $this->setData($data);
        }
    }

    public function draw() {
        $queue = array(
            'name',
            'submit-button'
        );

        if ($this->formMode == parent::VIEW) {
            array_pop($queue);
        }

        $this->form->queue = $queue;
        return $this->form;
    }

    public function getData() {
        $data = $this->form->getData();
        $hydrator = new ClassMethodsHydrator();
        return $hydrator->hydrate($data, new AvatarTypeEntity());
    }

    public function adaptData($data) {
        if ($data instanceof AvatarTypeEntity) {
            $hydrator = new ClassMethodsHydrator();
            $data = $hydrator->extract($data);
        }
        return $data;
    }

    public function setData($data) {
        $data = $this->adaptData($data);
        parent::setData($data);
    }

}

?>