<?php
namespace System\Form;

use System\Validator\RegexBase;
use System\Translator;
use Zend\Validator\NotEmpty;
use Zend\Validator\Regex;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Model\Entity\BannerPlace as Entity;
use Model\TableGateway\BannerPlace as BannerPlaceTableGateway;

class BannerPlace extends AbstractForm implements DrawableInterface {

    protected $name = 'dict_bc_element_place';
    protected $filterExceptions = array('available_for_elements');

    public function __construct($formMode, $mainSiteSelect, $bannerElementTypeSelect, $data = null, $imageName = null) {
        $translator = Translator\TranslatorEx::get();
        $tableGateway = new BannerPlaceTableGateway();

        if (defined('parent::' . $formMode)) {
            $this->setFormMode($formMode);
        } else {
            $this->setFormMode(parent::ADD);
        }

        if ($data && $this->getFormMode() == parent::EDIT) {
            $data = $this->adaptData($data);
        }

        $mainSiteSelectOptions = array('' => '');
        $mainSiteSelectValidation = array();
        foreach ($mainSiteSelect as $mainSiteSelectOption) {
            $mainSiteSelectOptions[] = array(
                'value' => (int)$mainSiteSelectOption->getId(),
                'label' => (string)$mainSiteSelectOption->getName(),
            );
            $mainSiteSelectValidation[] = (int)$mainSiteSelectOption->getId();
        }

        $bannerElementTypeSelectOptions = array('' => '');
        $bannerElementTypeSelectValidation = array();
        foreach ($bannerElementTypeSelect as $bannerElementTypeSelectOption) {
            $bannerElementTypeSelectOptions[] = array(
                'value' => (int)$bannerElementTypeSelectOption['id'],
                'label' => (string)$bannerElementTypeSelectOption['name'],
            );
            $bannerElementTypeSelectValidation[] = (int)$bannerElementTypeSelectOption['id'];
        }

        $BannerPlaces = $tableGateway->getAll();
        $BannerPlacePreviews = array('' => '');
        foreach ($BannerPlaces as $BannerPlace) {
            if (!in_array($BannerPlace->getImage(), $BannerPlacePreviews) && trim($BannerPlace->getImage())) {
                $BannerPlacePreviews[$BannerPlace->getImage()] = $BannerPlace->getImage();
            }
        }
        
        $fields = array(
            array('name' => 'name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('label' => $translator->translate('name', 'bannerPlace'))),
            array('name' => 'describe', 'type' => 'Zend\Form\Element\Textarea', 'attributes' => array('class' => 'form-textarea', 'req' => '', 'val' => 'textarea'), 'options' => array('label' => $translator->translate('describe', 'bannerPlace'))),
            array('name' => 'image', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('label' => $translator->translate('image', 'bannerPlace'))),
            array('name' => 'max_collisions', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'num'), 'options' => array('label' => $translator->translate('max_collisions', 'bannerPlace'))),
            array('name' => 'max_occurrences', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'num'), 'options' => array('label' => $translator->translate('max_occurrences', 'bannerPlace'))),
            array('name' => 'available_for_elements', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('id' => 'banner_element_type_select', 'multiple' => 'multiple', 'req' => '', 'val' => 'num'), 'options' => array('value_options' => $bannerElementTypeSelectOptions, 'label' => $translator->translate('available_for_elements', 'bannerPlace'))),
            array('name' => 'main_site_id', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('id' => 'main_site_select', 'req' => '', 'val' => 'num'), 'options' => array('value_options' => $mainSiteSelectOptions, 'label' => $translator->translate('main_site_id', 'bannerPlace'))),
            array('name' => 'submit-button', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('class' => 'form-submit', 'value' => $translator->translate('save')))
        );

        if ($this->getFormMode() == parent::ADD) {
            $fields[] = array('name' => 'preview', 'type' => 'System\Form\Element\BannerPlacePreview', 'attributes' => array('id' => 'preview-file', 'class' => 'inp-form banner_place_preview', 'select' => $BannerPlacePreviews), 'options' => array('label' => $translator->translate('banner_place_preview')));
        } elseif ($this->getFormMode() == parent::EDIT) {
            $fields[] = array('name' => 'preview', 'type' => 'System\Form\Element\BannerPlacePreview', 'attributes' => array('edit' => true, 'id' => 'preview-file', 'class' => 'inp-form banner_place_preview', 'image_name' => $imageName, 'select' => $BannerPlacePreviews), 'options' => array('label' => $translator->translate('banner_place_preview')));
        } elseif ($this->getFormMode() == parent::VIEW) {
            $fields[] = array('name' => 'preview', 'type' => 'System\Form\Element\BannerPlacePreview', 'attributes' => array('view' => true, 'id' => 'preview-file', 'class' => 'inp-form banner_place_preview', 'image_name' => $imageName), 'options' => array('label' => $translator->translate('banner_place_preview')));
        }
        
        if ($this->formMode == parent::VIEW) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox' || $field['type'] == 'Zend\Form\Element\Select') {
                    $field['attributes']['disabled'] = 'disabled';
                } elseif ($field['type'] == 'Zend\Form\Element\File') {
                    $field['attributes']['class'] = $field['attributes']['class'] . ' input-dissabled';
                    $field['attributes']['disabled'] = 'disabled';
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }

        $this->validation = array(
            'name' => array(
                'required' => true,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'regex', 'break_chain_on_failure' => true, 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(Regex::NOT_MATCH => $translator->translate('error_name_identifier_invalid')))),
                    array('name' => 'string_length', 'options' => array('max' => 100, 'messages' => array(\Zend\Validator\StringLength::TOO_LONG => $translator->translate('error_value_more_than_100')))),
                    array('name' => 'Zend\Validator\Db\NoRecordExists', 'options' => array('adapter' => $tableGateway->getAdapter(), 'schema' => BannerPlaceTableGateway::getSchema(), 'table' => BannerPlaceTableGateway::getTableName(), 'field' => 'name', 'exclude' => (isset($data['name']) && $data['name']) ? array('field' => 'name', 'value' => $data['name']) : null, 'messages' => array(\Zend\Validator\Db\NoRecordExists::ERROR_RECORD_FOUND => $translator->translate('record_already_exists')))),
                )
            ),
            'describe' => array(
                'required' => true,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'string_length', 'options' => array('max' => 100, 'messages' => array(\Zend\Validator\StringLength::TOO_LONG => $translator->translate('error_value_more_than_100')))),
                )
            ),
            'image' => array(
                'required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'regex', 'break_chain_on_failure' => true, 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(Regex::NOT_MATCH => $translator->translate('error_name_identifier_invalid')))),
                    array('name' => 'string_length', 'options' => array('max' => 100, 'messages' => array(\Zend\Validator\StringLength::TOO_LONG => $translator->translate('error_value_more_than_100')))),
                )
            ),
            'max_collisions' => array(
                'required' => true,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'regex', 'break_chain_on_failure' => true, 'options' => array('pattern' => '/' . RegexBase::get('num') . '/D', 'messages' => array(Regex::NOT_MATCH => $translator->translate('error_value_must_be_integer')))),
                    array('name' => 'between', 'options' => array('min' => 1, 'max' => 100, 'messages' => array(\Zend\Validator\Between::NOT_BETWEEN => $translator->translate('error_value_from_1_to_100')))),
                )
            ),
            'max_occurrences' => array(
                'required' => true,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'regex', 'break_chain_on_failure' => true, 'options' => array('pattern' => '/' . RegexBase::get('num') . '/D', 'messages' => array(Regex::NOT_MATCH => $translator->translate('error_value_must_be_integer')))),
                    array('name' => 'between', 'options' => array('min' => 1, 'max' => 100, 'messages' => array(\Zend\Validator\Between::NOT_BETWEEN => $translator->translate('error_value_from_1_to_100')))),
                )
            ),
            'available_for_elements' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'System\Validator\Multiselect', 'options' => array('haystack' => $bannerElementTypeSelectValidation), 'messages' => array(\System\Validator\Multiselect::VALUES_NOT_ALLOWED => $translator->translate('error_select_value'))),
                ),
                'filters' => array(
                    array('name' => 'callback', 'options' => array(
                            'callback' => function($elements) {
                                foreach ($elements as $key => $element) {
                                    $this->filter($element);
                                    $elements[$key] = $element;
                                }
                                return $elements;
                            }))
                ),
            ),
            'main_site_id' => array(
                'required' => true,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'in_array', 'options' => array('haystack' => $mainSiteSelectValidation, 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $translator->translate('error_select_value')))),
                )
            ),
            'preview' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'System\Validator\FileAvatar', 'options' => array('allowEmpty' => true, 'allowOverride' => false,
                            'messages' => array(
                                \System\Validator\File::FILE_ERROR => $translator->translate('file_error'),
                            )))
                )
            ),
        );

        parent::init($fields);

        if ($data && $this->getFormMode() == parent::EDIT) {
            $this->setData($data);
        }
    }

    public function draw() {
        $queue = array(
            'name',
            'describe',
            'max_collisions',
            'max_occurrences',
            'available_for_elements',
            'preview',
            'main_site_id',
            'submit-button'
        );

        if ($this->formMode == parent::VIEW) {
            array_pop($queue);
        }

        $this->form->queue = $queue;
        return $this->form;
    }

    public function getData() {
        $data = $this->form->getData();
        $hydrator = new ClassMethodsHydrator();
        return $hydrator->hydrate($data, new Entity());
    }

    public function adaptData($data) {
        if ($data instanceof Entity) {
            $hydrator = new ClassMethodsHydrator();
            $data = $hydrator->extract($data);
        }
        
        return $data;
    }

    public function setData($data) {
        $data = $this->adaptData($data);
        parent::setData($data);
    }
    
    public function addachDefaultFilters() {
        if (null !== $this->filterChain) return;

        $this->filtersForAll[] = new \Zend\Filter\StringTrim;
        $this->filtersForAll[] = new \System\Filter\PregReplaceEx(array('match' => "/\'/", 'replace' => "\""));
    }

}

?>