<?php

namespace System\Form;

use Zend\Filter;

class CmsAddAdminActionGroup extends AbstractForm implements DrawableInterface {

    protected $name = 'add_admin_action_group';
    protected $translations;
    protected $edit = false;

    public static function neededTranslations() {
        return array(
            'save', 'action', 'action_does_not_exist', 'error_value_is_required'
        );
    }

    public function __construct(array $translations, array $actionsToAdd) {
        $this->translations = $translations;

        $actionsToAddList = array();
        $actionsToAddListValidation = array();
        foreach ($actionsToAdd as $action) {
            if ($action['module']) {
                $action['module'] = ' ' . $action['module'];
            }
            
            $actionsToAddList[] = array(
                'value' => (int) $action['action_id'],
                'label' => (string) $action['system'] . $action['module'] . ' ' . $action['controller'] . ' ' . $action['action'],
            );
            $actionsToAddListValidation[] = (int) $action['action_id'];
        }

        $fields = array(
            array('name' => 'action', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('multiple' => 'multiple', 'id' => 'action_autocomplete', 'req' => '', 'val' => 'num'), 'options' => array('value_options' => $actionsToAddList, 'label' => $this->translations['action'])),
            array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['save'], 'class' => 'submitStep form-submit'))
        );

        $this->validation = array(
            'action' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'System\Validator\Multiselect', 'options' => array('haystack' => $actionsToAddListValidation), 'messages' => array(\System\Validator\Multiselect::VALUES_NOT_ALLOWED => $this->translations['action_does_not_exist']))
                )
            ),
        );

        parent::init($fields);

        $this->_messages = array(
            'action' => array(
                $this->translations['error_value_is_required'],
                $this->translations['action_does_not_exist']
            ),
        );
    }

    public function getData() {
        return $this->form->getData();
    }

    public function draw() {
        $queue = array(
            'action',
            'submit'
        );

        $this->form->queue = $queue;
        return $this->form;
    }

    public function addachDefaultFilters() {
        if (null !== $this->filterChain)
            return;

        $this->filtersForAll[] = new Filter\StringTrim;
        $this->filtersForAll[] = new \System\Filter\PregReplaceEx(array('match' => "/'/", 'replace' => "''"));
    }

}

?>