<?php

namespace System\Form;

use System\Validator\RegexBase;

class CmsAddEditAdminAGroup extends AbstractForm implements DrawableInterface {

    protected $name = 'add_edit_admin_agroup';
    protected $translations;
    protected $edit = false;

    public static function neededTranslations() {
        return array(
            'save', 'group_code', 'group_name', 'error_value_is_required', 'error_name_identifier_invalid'
        );
    }

    public function __construct(array $translations, $edit = false, $view = false) {
        $this->translations = $translations;
        $this->edit = (bool) $edit;
        $this->view = (bool) $view;

        $fields = array(
            array('name' => 'group_code', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('label' => $this->translations['group_code'])),
            array('name' => 'group_name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'title'), 'options' => array('label' => $this->translations['group_name'])),
            array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['save'], 'class' => 'submitStep form-submit'))
        );

        if ($this->view) {
            foreach ($fields as &$field) {
                $field['attributes']['readonly'] = 'readonly';
            }
        }

        $this->validation = array(
            'group_code' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                    array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_identifier_invalid'])))
                )
            ),
            'group_name' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                ),
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                    array('name' => '\Zend\Filter\StringTrim'),
                    array('name' => '\System\Filter\PregReplaceEx', 'options' => array('match' => "/\'/", 'replace' => "\"")),
                )
            ),
        );

        parent::init($fields);

        $this->_messages = array(
            'group_code' => array(
                $this->translations['error_value_is_required'],
                $this->translations['error_name_identifier_invalid']
            ),
            'group_name' => array(
                $this->translations['error_value_is_required']
            )
        );
    }

    public function getData() {
        return $this->form->getData();
    }

    public function draw() {
        $queue = array(
            'group_code',
            'group_name',
            'submit'
        );

        if ($this->view) {
            array_pop($queue);
        }

        $this->form->queue = $queue;
        return $this->form;
    }

}

?>