<?php

namespace System\Form;

use System\Validator\RegexBase;

class CmsAddEditAdminAction extends AbstractForm implements DrawableInterface {

    protected $name = 'add_edit_admin_action';
    protected $translations;
    protected $edit = false;

    public static function neededTranslations() {
        return array(
            'save', 'system', 'module', 'controller', 'action', 'exists', 'is_new', 'error_value_is_required', 'error_name_identifier_invalid', 'checkbox_error'
        );
    }

    public function __construct(array $translations, $edit = false, $view = false, $systems = array(), $modules = array(), $controllers = array(), $isRelated = false) {
        $this->translations = $translations;
        $this->edit = (bool) $edit;
        $this->view = (bool) $view;
        $this->isRelated = (bool) $isRelated;

        $systemsList = array();
        $systemsListValidation = array();
        foreach ($systems as $system) {
            $systemsList[] = array(
                'value' => (string) $system,
                'label' => (string) $system
            );
            $systemsListValidation[] = $system;
        }

        $controllersList = array();
        $controllersListValidation = array();
        foreach ($controllers as $controller) {
            $controllersList[] = array(
                'value' => (string) $controller,
                'label' => (string) $controller
            );
            $controllersListValidation[] = $controller;
        }
        
        $modulesList = array();
        $modulesListValidation = array();
        foreach ($modules as $module) {
            $modulesList[] = array(
                'value' => (string) $module,
                'label' => (string) $module
            );
            $modulesListValidation[] = $module;
        }

        if ($this->view) {
            $fields = array(
                array('name' => 'system', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('value_options' => $systemsList, 'label' => $this->translations['system'])),
                array('name' => 'module', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('value_options' => $modulesList, 'label' => $this->translations['module'])),
                array('name' => 'controller', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('value_options' => $controllersList, 'label' => $this->translations['controller'])),
                array('name' => 'action', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier', 'id' => 'admin_action_input'), 'options' => array('label' => $this->translations['action'])),
                array('name' => 'exists', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('val' => '1'), 'options' => array('label' => $this->translations['exists'])),
                array('name' => 'is_new', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('val' => '1'), 'options' => array('label' => $this->translations['is_new'])),
                array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['save'], 'class' => 'submitStep form-submit'))
            );
        } elseif ($this->edit) {
            $fields = array(
                array('name' => 'system', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('class' => 'autocomplete', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('value_options' => $systemsList, 'label' => $this->translations['system'])),
                array('name' => 'module', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('class' => 'autocomplete', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('value_options' => $modulesList, 'label' => $this->translations['module'])),
                array('name' => 'controller', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('class' => 'autocomplete', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('value_options' => $controllersList, 'label' => $this->translations['controller'])),
                array('name' => 'action', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier', 'id' => 'admin_action_input'), 'options' => array('label' => $this->translations['action'])),
                array('name' => 'exists', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('val' => '1'), 'options' => array('label' => $this->translations['exists'])),
                array('name' => 'is_new', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('val' => '1'), 'options' => array('label' => $this->translations['is_new'])),
                array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['save'], 'class' => 'submitStep form-submit'))
            );
            if ($this->isRelated) {
                foreach ($fields as &$field) {
                    if ($field['name'] == 'system' || $field['name'] == 'module' || $field['name'] == 'controller' || $field['name'] == 'action') {
                        $field['attributes']['readonly'] = 'readonly';
                        $field['attributes']['class'] = $field['attributes']['class'] . ' cannot-edit';
                    }
                }
            }
        } else {
            $fields = array(
                array('name' => 'system', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('class' => 'autocomplete', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('value_options' => $systemsList, 'label' => $this->translations['system'])),
                array('name' => 'module', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('class' => 'autocomplete', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('value_options' => $modulesList, 'label' => $this->translations['module'])),
                array('name' => 'controller', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('class' => 'autocomplete', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('value_options' => $controllersList, 'label' => $this->translations['controller'])),
                array('name' => 'action', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form-select2', 'req' => '', 'val' => 'nameIdentifier', 'id' => 'admin_action_input'), 'options' => array('label' => $this->translations['action'])),
                array('name' => 'exists', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('val' => '1'), 'options' => array('label' => $this->translations['exists'])),
                array('name' => 'is_new', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('val' => '1'), 'options' => array('label' => $this->translations['is_new'])),
                array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['save'], 'class' => 'submitStep form-submit'))
            );
        }

        if ($this->view) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox') {
                    $field['attributes']['disabled'] = 'disabled';
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }

        $this->validation = array(
            'system' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                    array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_identifier_invalid'])))
                )
            ),
            'module' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_identifier_invalid'])))
                )
            ),
            'controller' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                    array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_identifier_invalid'])))
                )
            ),
            'action' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                    array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_identifier_invalid'])))
                )
            ),
            'exists' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'in_array', 'options' => array('haystack' => array(0, 1)), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['checkbox_error']))
                )
            ),
            'is_new' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'in_array', 'options' => array('haystack' => array(0, 1)), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['checkbox_error']))
                )
            ),
        );

        parent::init($fields);

        $this->_messages = array(
            'system' => array(
                $this->translations['error_value_is_required'],
                $this->translations['error_name_identifier_invalid']
            ),
            'module' => array(
                $this->translations['error_value_is_required'],
                $this->translations['error_name_identifier_invalid']
            ),
            'controller' => array(
                $this->translations['error_value_is_required'],
                $this->translations['error_name_identifier_invalid']
            ),
            'action' => array(
                $this->translations['error_value_is_required'],
                $this->translations['error_name_identifier_invalid']
            ),
            'exists' => array(
                $this->translations['checkbox_error']
            ),
            'is_new' => array(
                $this->translations['checkbox_error']
            ),
        );
    }

    public function getData() {
        return $this->form->getData();
    }

    public function draw() {
        $queue = array(
            'system',
            'module',
            'controller',
            'action',
            'exists',
            'is_new',
            'submit'
        );

        if ($this->view) {
            array_pop($queue);
        }

        $this->form->queue = $queue;
        return $this->form;
    }

    public function setData($data) {
        foreach ($data as $name => &$value) {
            if ($name == 'exists' || $name == 'is_new') {
                if ($value == 1) {
                    $this->form->get($name)->setAttribute('checked', 'checked');
                } else {
                    $value = 0;
                }
            }
        }

        parent::setData($data);
    }

}

?>