<?php

namespace System\Form;

use System\Validator\RegexBase;

class CmsAddEditAdminLGroup extends AbstractForm implements DrawableInterface {

    protected $name = 'add_edit_admin_lgroup';
    protected $translations;
    protected $edit = false;

    public static function neededTranslations() {
        return array(
            'save', 'group_name', 'group_descr', 'is_active', 'error_value_is_required', 'error_name_identifier_invalid', 'checkbox_error'
        );
    }

    public function __construct(array $translations, $edit = false, $view = false, $isRelated = false) {
        $this->translations = $translations;
        $this->edit = (bool) $edit;
        $this->view = (bool) $view;
        $this->isRelated = (bool) $isRelated;

        $fields = array(
            array('name' => 'group_name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('label' => $this->translations['group_name'])),
            array('name' => 'group_descr', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'val' => 'title'), 'options' => array('label' => $this->translations['group_descr'])),
            array('name' => 'is_active', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('val' => '1'), 'options' => array('label' => $this->translations['is_active'])),
            array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['save'], 'class' => 'submitStep form-submit'))
        );

        if ($this->view) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox') {
                    $field['attributes']['disabled'] = 'disabled';
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }
        
        if ($this->edit && $this->isRelated) {
            foreach ($fields as &$field) {
                if ($field['name'] == 'group_name') {
                    $field['attributes']['readonly'] = 'readonly';
                    $field['attributes']['class'] = $field['attributes']['class'] . ' cannot-edit';
                }
            }
        }

        $this->validation = array(
            'group_name' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                    array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_identifier_invalid'])))
                )
            ),
            'group_descr' => array(
                'required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                    array('name' => '\Zend\Filter\StringTrim'),
                    array('name' => '\System\Filter\PregReplaceEx', 'options' => array('match' => "/\'/", 'replace' => "\"")),
                )
            ),
            'is_active' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'in_array', 'options' => array('haystack' => array(0, 1)), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['checkbox_error']))
                )
            ),
        );

        parent::init($fields);

        $this->_messages = array(
            'group_name' => array(
                $this->translations['error_value_is_required'],
                $this->translations['error_name_identifier_invalid']
            ),
            'is_active' => array(
                $this->translations['checkbox_error']
            ),
        );
    }

    public function getData() {
        return $this->form->getData();
    }

    public function draw() {
        $queue = array(
            'group_name',
            'group_descr',
            'is_active',
            'submit'
        );

        if ($this->view) {
            array_pop($queue);
        }

        $this->form->queue = $queue;
        return $this->form;
    }

    public function setData($data) {
        foreach ($data as $name => $value) {
            if ($name == 'is_active' && $value == 1) {
                $this->form->get($name)->setAttribute('checked', 'checked');
            } else {
                $value = 0;
            }
        }

        parent::setData($data);
    }

}

?>