<?php

namespace System\Form;

use System\Validator\RegexBase;

class CmsAddEditAdminUser extends AbstractForm implements DrawableInterface {

    protected $name = 'add_edit_admin_user';
    protected $translations;
    protected $edit = false;

    public static function neededTranslations() {
        return array(
            'save', 'esculap_user', 'user_login', 'name', 'lastname', 'esc_id', 'admin_user_esc_id_dont_exists', 'error_value_is_required', 'error_name_identifier_invalid', 'error_name_sname_invalid', 'error_name_not_int_invalid'
        );
    }

    public function __construct(array $translations, $esculapUsers, $edit = false, $view = false) {
        $this->translations = $translations;
        $this->edit = (bool) $edit;
        $this->view = (bool) $view;

        $esculapUsersListValidation = array();
        foreach ($esculapUsers as $esculapUser) {
            $esculapUsersListValidation[] = $esculapUser['login'];
        }

        $fields = array(
            array('name' => 'esculap_user', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('req' => '', 'val' => 'nameIdentifier', 'class' => 'autocomplete'), 'options' => array('label' => $this->translations['esculap_user'])),
            //array('name' => 'user_login', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('label' => $this->translations['user_login'])),
            array('name' => 'name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'val' => 'sname'), 'options' => array('label' => $this->translations['name'])),
            array('name' => 'lastname', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'val' => 'sname'), 'options' => array('label' => $this->translations['lastname'])),
            //array('name' => 'esc_id', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'val' => 'num'), 'options' => array('label' => $this->translations['esc_id'])),
            array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['save'], 'class' => 'submitStep form-submit'))
        );

        if ($this->view || $this->edit) {
            $fields = array(
                //array('name' => 'esculap_user', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('req' => '', 'val' => 'num', 'class' => 'chosen_plugin'), 'options' => array('value_options' => $clientsList, 'label' => $this->translations['esculap_user'])),
                array('name' => 'user_login', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'nameIdentifier'), 'options' => array('label' => $this->translations['user_login'])),
                array('name' => 'name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'val' => 'sname'), 'options' => array('label' => $this->translations['name'])),
                array('name' => 'lastname', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'val' => 'sname'), 'options' => array('label' => $this->translations['lastname'])),
                array('name' => 'esc_id', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'val' => 'num'), 'options' => array('label' => $this->translations['esc_id'])),
                array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['save'], 'class' => 'submitStep form-submit'))
            );

            foreach ($fields as &$field) {
                if ($this->view || ($this->edit && ($field['name'] == 'user_login' || $field['name'] == 'esc_id'))) {
                    $field['attributes']['readonly'] = 'readonly';
                    if ($this->edit) {
                        $field['attributes']['class'] = $field['attributes']['class'] . ' cannot-edit';
                    }
                }
            }
        }

        $this->validation = array(
            'esculap_user' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                    array('name' => 'in_array', 'options' => array('haystack' => $esculapUsersListValidation, 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['admin_user_esc_id_dont_exists'])))
                ),
            ),
            'name' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('sname') . '/', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_sname_invalid'])))
                )
            ),
            'lastname' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('sname') . '/', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_sname_invalid'])))
                )
            ),
        );

        if ($this->view || $this->edit) {
            $this->validation = array(
                'user_login' => array(
                    'required' => true,
                    'validators' => array(
                        array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                        array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_identifier_invalid'])))
                    )
                ),
                'name' => array(
                    'required' => false,
                    'validators' => array(
                        array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('sname') . '/', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_sname_invalid'])))
                    )
                ),
                'lastname' => array(
                    'required' => false,
                    'validators' => array(
                        array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('sname') . '/', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_sname_invalid'])))
                    )
                ),
                'esc_id' => array(
                    'required' => false,
                    'validators' => array(
                        array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                        array('name' => 'Zend\I18n\Validator\Int', 'options' => array('messages' => array(\Zend\I18n\Validator\Int::NOT_INT => $this->translations['error_name_not_int_invalid']))),
                    ),
                ),
            );
        }

        parent::init($fields);

        $this->_messages = array(
            'esculap_user' => array(
                $this->translations['error_value_is_required'],
                $this->translations['admin_user_esc_id_dont_exists']
            ),
            'user_login' => array(
                $this->translations['error_value_is_required'],
                $this->translations['error_name_identifier_invalid']
            ),
            'name' => array(
                $this->translations['error_name_sname_invalid'],
            ),
            'lastname' => array(
                $this->translations['error_name_sname_invalid'],
            ),
            'esc_id' => array(
                $this->translations['error_value_is_required'],
                $this->translations['error_name_not_int_invalid']
            ),
        );

        if ($this->view || $this->edit) {
            $this->_messages = array(
                'user_login' => array(
                    $this->translations['error_value_is_required'],
                    $this->translations['error_name_identifier_invalid']
                ),
                'name' => array(
                    $this->translations['error_name_sname_invalid'],
                ),
                'lastname' => array(
                    $this->translations['error_name_sname_invalid'],
                ),
                'esc_id' => array(
                    $this->translations['error_value_is_required'],
                    $this->translations['error_name_not_int_invalid']
                ),
            );
        }
    }

    public function getData() {
        return $this->form->getData();
    }

    public function draw() {
        $queue = array(
            'esculap_user',
            //'user_login',
            'name',
            'lastname',
            //'esc_id',
            'submit'
        );

        if ($this->view || $this->edit) {
            $queue = array(
                //'esculap_user',
                'user_login',
                'name',
                'lastname',
                'esc_id',
                'submit'
            );

            if ($this->view) {
                array_pop($queue);
            }
        }

        $this->form->queue = $queue;
        return $this->form;
    }

}

?>