<?php

namespace System\Form;

use System\Validator\RegexBase;
use Model\DictSpec;
use Model\TableGateway\MainEditions as MainEditionsTableGateway;

class CmsAddEditPublication extends AbstractForm implements DrawableInterface {
	
	protected $translations;
	protected $name = 'add_edit_publication';
	protected $graphics = array();
	
	// lista b��d�w przekazanych domy�lnie do widoku, w celu u�ycia ich do walidacji js-em
	protected $messages = array();
	
	// czy formularz odpalony jest do edycji
	protected $edit = false;
	
	protected $filterExceptions = array( 'content' );
	
	public static function neadedTranslations() {
		return array(
			'main_save' , 'main_next' , 'main_prev' , 'main_title' , 'main_headers' , 'main_content' , 'main_avatar' , 'main_additional_graphics' , 'main_add' , 'main_set' , 'main_content_and_graphics' ,
			'main_additional_informations' , 'main_specialityies_and_collections' , 'main_specializations' , 'main_collections' , 'main_copytight' , 'main_author' , 'main_type' , 'main_active' , 'main_serchable',
			'main_active_for' , 'main_date' , 'main_language' , 'error_value_is_required' , 'error_select_value' , 'error_value_less_than_5' , 'error_value_is_not_numeric' , 'error_date_format_slashes' ,
			'main_select' , 'main_view' , 'main_additional_file' , 'main_keywords' , 'main_journal' , 'main_page' , 'error_value_must_be_numeric' , 'error_page_invalid' , 'main_english_title' ,
			'main_global_collection' , 'main_content_of_the_day' , 'main_content_of_the_day_eror', 'main_site', 'edition_year', 'edition_information', 'edition_nr', 
                        'edition_book_nr', 'book_nr', 'book_year', 'chief_nr', 'main_months_names', 'book_month', 'subject', 'error_value_must_be_integer', 'main_edition', 'record_dont_exists', 
		);
	}
	
	public function __construct( array $translations , $publicationTypes , $journalsList , $languages , $specNames , $mainSiteSelect, $mainEditionsLink, $edit = false ) {
		$this->translations = $translations;
		$tableGateway = new MainEditionsTableGateway();
                
		$this->edit = (bool)$edit;
		
		$_publicationTypes = array();
		$_publicationTypesValidate = array();
		foreach( $publicationTypes as $val => $label ) {
			$_publicationTypesValidate[] = $val;
		
			$_publicationTypes[ $val ] = array(
				'value' => $val,
				'label' => $label
			);
		}
		
		$_languages = array();
		$_languageValidate = array();
		foreach( $languages as $val => $label ) {
			$_languageValidate[] = $val;
			
			$_languages[ $val ] = array(
				'value' => $val,
				'label' => $label
			);
		}
		
		$_journalsValidation = array( 0 );
		foreach( $journalsList as $journalId => $journal ) {
			$_journals[] = array(
				'value' => $journalId,
				'label' => $journal
			);
			
			$_journalsValidation[] = $journalId;
		}
                
                $mainSiteSelectOptions = array('' => '');
                $mainSiteSelectValidation = array();
                foreach ($mainSiteSelect as $mainSiteSelectOption) {
                    $mainSiteSelectOptions[] = array(
                        'value' => (int)$mainSiteSelectOption->getId(),
                        'label' => (string)$mainSiteSelectOption->getName(),
                    );
                    $mainSiteSelectValidation[] = (int)$mainSiteSelectOption->getId();
                }
		
                $monthNames = explode(',', $this->translations['main_months_names']);
                foreach ($monthNames as $key => $monthName) {
                    $monthNames[$key] = mb_strtoupper($monthName, 'UTF-8');
                }
                
                $monthSelectOptions = array(0 => '');
                foreach ($monthNames as $monthSelectOption) {
                    $monthSelectOptions[] = array(
                        'value' => (string)$monthSelectOption,
                        'label' => (string)$monthSelectOption,
                    );
                    $monthSelectValidation[] = (string)$monthSelectOption;
                }
                $monthSelectValidation[] = '0';
                
                $dict = new DictSpec();
                $specs = $dict->getAllEx();
                
                $specSelectOptions = array();
                $specSelectValidation = array();
                foreach ($specs as $specSelectOption) {
                    if ($specSelectOption['mdm_name'] != '.') {
                        $specSelectOptions[] = array(
                            'value' => (string)$specSelectOption['mdm_name'],
                            'label' => (string)$specSelectOption['mdm_name'],
                        );
                    }
                    $specSelectValidation[] = (string)$specSelectOption['mdm_name'];
                }
                
		$fields = array(
			array( 'name' => 'title' , 'type' => 'Zend\Form\Element\Text' , 'attributes' => array( 'class' => 'inp-form-long' , 'req' => '' , 'val' => 'min5chars' ) , 'options' => array( 'label' => $this->translations[ 'main_title' ]) ),
			array( 'name' => 'english_title' , 'type' => 'Zend\Form\Element\Text' , 'attributes' => array( 'class' => 'inp-form-long' , 'val' => 'min5chars' ) , 'options' => array( 'label' => $this->translations[ 'main_english_title' ]) ),
			array( 'name' => 'headers' , 'type' => 'Zend\Form\Element\Text' , 'attributes' => array( 'class' => 'inp-form-long' , 'req' => '' , 'val' => 'min5chars' ) , 'options' => array( 'label' => $this->translations[ 'main_headers' ] ) ),
			array( 'name' => 'content' , 'type' => 'Zend\Form\Element\Textarea' , 'attributes' => array( 'req' => 'document_value:minlength:1' , 'val' => 'textarea' , 'class' => 'form-textarea long middle ckeditor' ) , 'options' => array( 'label' => $this->translations[ 'main_content' ] ) ),
			//array( 'name' => 'content_view' , 'type' => 'Zend\Form\Element\Button' , 'attributes' => array( 'class' => 'main_content_view_news_content form-submit' , 'value' => $this->translations[ 'main_view' ] ) , 'options' => array( 'label' => $this->translations[ 'main_view' ] ) ),
			array( 'name' => 'copyrights' , 'type' => 'Zend\Form\Element\Text' , 'attributes' => array( 'class' => 'inp-form' , 'req' => '' ) , 'options' => array( 'label' => $this->translations[ 'main_copytight' ] ) ) ,
			array( 'name' => 'author' , 'type' => 'Zend\Form\Element\Text' , 'attributes' => array( 'class' => 'inp-form' , 'req' => '' ) , 'options' => array( 'label' => $this->translations[ 'main_author' ] ) ) ,
			array( 'name' => 'type' , 'type' => 'Zend\Form\Element\Select' , 'attributes' => array( 'class' => 'styledselect_form_1 news_type') , 'options' => array( 'req' => '' , 'vla' => 'num' , 'value_options' => $_publicationTypes , 'label' => $this->translations[ 'main_type' ] ) ) ,
			array( 'name' => 'date' , 'type' => 'Zend\Form\Element\Text' , 'attributes' => array( 'maxlength' => '10', 'req' => '' , 'val' => 'dateSlash' , 'class' => 'date-pick inp-form-date date-pick-max-date-set' , 'maxdate' => date( 'Y/m/d' , strtotime( '+1 days' ) ) ) , 'options' => array( 'label' => $this->translations[ 'main_date' ] ) ),
			array( 'name' => 'language' , 'type' => 'Zend\Form\Element\Select' , 'attributes' => array( 'class' => 'styledselect_form_1' ) , 'options' => array( 'label' => $this->translations[ 'main_language' ] , 'value_options' => $_languages ) ),
			array( 'name' => 'active' , 'type' => 'Zend\Form\Element\Checkbox' , 'attributes' => array( 'value' => '1' ) , 'options' => array( 'label' => $this->translations[ 'main_active' ] ) ),
			array( 'name' => 'searchable' , 'type' => 'Zend\Form\Element\Checkbox' , 'attributes' => array( 'value' => '1' ) , 'options' => array( 'label' => $this->translations[ 'main_serchable' ] ) ),
			array( 'name' => 'avatar' , 'type' => 'Zend\Form\Element\Button' , 'attributes' => array( 'class' => 'content_add_edit_avatar form-submit' , 'value' => $this->translations[ 'main_select' ] ) , 'options' => array( 'label' => $this->translations[ 'main_avatar' ] ) ),
			array( 'name' => 'avatar_value' , 'type' => 'Zend\Form\Element\Hidden' , 'attributes' => array( 'id' => 'avatar' , 'req' => '' , 'val' => 'avatar' , 'class' => 'off_site_error no_height content_add_edit_avatar_value') , 'options' => array() ),
			array( 'name' => 'page' , 'type' => 'Zend\Form\Element\Text' , 'attributes' => array( 'val' => 'pageRange' , 'class' => 'inp-form' ) , 'options' => array( 'label' => $this->translations[ 'main_page' ] ) ),
			array( 'name' => 'specs' , 'type' => 'Zend\Form\Element\Button' , 'attributes' => array( 'class' => 'content_news_add_edit_specs form-submit' , 'value' => $this->translations[ 'main_select' ] ) , 'options' => array( 'label' => $this->translations[ 'main_specializations' ] ) ),
			array( 'name' => 'specs_value' , 'type' => 'Zend\Form\Element\Hidden' , 'attributes' => array( 'class' => 'no_height content_news_add_edit_specs_value') , 'options' => array() ),
			//array( 'name' => 'collections' , 'type' => 'Zend\Form\Element\Button' , 'attributes' => array( 'class' => 'content_news_add_edit_collections' , 'value' => $this->translations[ 'main_select' ] ) , 'options' => array( 'label' => $this->translations[ 'main_collections' ] ) ),
			//array( 'name' => 'collections_value' , 'type' => 'Zend\Form\Element\Hidden' , 'attributes' => array( 'class' => 'content_news_add_edit_collections_value') , 'options' => array() ),
			
			
			array( 'name' => 'esculap_collection' , 'type' => 'Zend\Form\Element\Checkbox' , 'attributes' => array( 'value' => '1' ) , 'options' => array( 'label' => $this->translations[ 'main_global_collection' ] ) ),
			array( 'name' => 'content_of_the_day' , 'type' => 'Zend\Form\Element\Checkbox' , 'attributes' => array( 'value' => '1' ) , 'options' => array( 'label' => $this->translations[ 'main_content_of_the_day' ] ) ),
			
			
			// kolekcje
			array( 'name' => 'collection' , 'type' => 'Zend\Form\Element\Hidden' , 'options' => array() , 'attributes' => array( 'class' => 'no_height' ) ),
			array( 'name' => 'collection_button' , 'type' => 'Zend\Form\Element\Button' , 'options' => array( 'label' => $this->translations[ 'main_collections' ] ) , 'attributes' => array( 'value' => $this->translations[ 'main_set' ] , 'class' => 'collection_button publ form-submit' ) ),
			
			
			array( 'name' => 'nextStep', 'type' => 'Zend\Form\Element\Button','attributes' => array( 'title' => $this->translations['main_next'] , 'class' => 'info-tooltip next_button submitStep' ) ),
			array( 'name' => 'prevStep', 'type' => 'Zend\Form\Element\Button','attributes' => array( 'title' => $this->translations['main_prev'] , 'class' => 'info-tooltip prev_button prevStep' ) ),
			array( 'name' => 'save', 'type' => 'Zend\Form\Element\Button','attributes' => array( 'title' => $this->translations['main_save'] ,'class' => 'info-tooltip save_button submitStep' ) ),
			array( 'name' => 'spec_names' , 'type' => 'Zend\Form\Element\Hidden' , 'attributes' => array( 'class' => 'no_height content_spec_names' ) ),
			array( 'name' => 'graphics' , 'type' => 'Zend\Form\Element\File' , 'attributes' => array( 'id' => 'new_image' , 'class' => 'content_add_edit_graphics' , 'value' => $this->translations[ 'main_add' ] ) , 'options' => array( 'label' => $this->translations[ 'main_additional_graphics' ] ) ),
			array( 'name' => 'graphics_value' , 'type' => 'Zend\Form\Element\Hidden' , 'attributes' => array( 'class' => 'no_height content_add_edit_graphic_value') , 'options' => array() ),
			array( 'name' => 'document' , 'type' => 'Zend\Form\Element\Button' , 'attributes' => array( 'class' => 'form-submit main_file_load_list publication ' , 'value' => $this->translations[ 'main_select' ] ) , 'options' => array( 'label' => $this->translations[ 'main_additional_file' ] ) ),
			array( 'name' => 'document_value' , 'type' => 'Zend\Form\Element\Hidden' , 'attributes' => array( 'class' => 'no_height main_file_value' ) , 'options' => array( 'label' => '' ) ),
			array( 'name' => 'keywords' , 'type' => 'Zend\Form\Element\Text' , 'attributes' => array( 'req' => '' , 'val' => 'keywords' , 'class' => 'inp-form-long' ) , 'options' => array( 'label' => $this->translations[ 'main_keywords' ] ) ),
			array( 'name' => 'journal' , 'type' => 'Zend\Form\Element\Select' , 'attributes' => array( 'req' => '' , 'val' => 'numeric' , 'class' => 'styledselect_form_1'  ) , 'options' => array( 'value_options' => $_journals , 'label' => $this->translations[ 'main_journal' ] ) ),
                    
                        array('name' => 'main_site_id', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('id' => 'main_site_select', 'req' => '', 'val' => 'num'), 'options' => array('value_options' => $mainSiteSelectOptions, 'label' => $this->translations['main_site'])),
                        
                        array('name' => 'edition_year', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form'), 'options' => array('label' => $this->translations['edition_year'])),
                        array('name' => 'edition_nr', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form'), 'options' => array('label' => $this->translations['edition_nr'])),
                        array('name' => 'edition_book_nr', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form'), 'options' => array('label' => $this->translations['edition_book_nr'])),
                        array('name' => 'book_nr', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form'), 'options' => array('label' => $this->translations['book_nr'])),
                        array('name' => 'book_year', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'val' => 'numeric'), 'options' => array('label' => $this->translations['book_year'])),
                        array('name' => 'chief_nr', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form'), 'options' => array('label' => $this->translations['chief_nr'])),
                        array('name' => 'book_month', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('class' => 'styledselect_form_1'), 'options' => array('value_options' => $monthSelectOptions , 'label' => $this->translations['book_month'])),
                        array('name' => 'subject', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('id' => 'subject_select'), 'options' => array('value_options' => $specSelectOptions , 'label' => $this->translations['subject'])),
                    
                        array('name' => 'main_editions', 'type' => '\System\Form\Element\SelectRelationalHidden', 'attributes' => array('class' => 'no_height select_relational')),
                        array('name' => 'main_editions_visible', 'type' => '\System\Form\Element\SelectRelationalVisible', 'attributes' => array('class' => 'select_relational'), 'options' => array('link' => $mainEditionsLink, 'label' => $this->translations['main_edition'])),
		);
			
			$this->validation = array(
				'title' => array(
					'required' => true,
					'validators' => array(
						array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) ),
						array( 'name' => 'string_length' , 'options' => array( 'min' => 5 , 'messages' => array( \Zend\Validator\StringLength::TOO_SHORT => $this->translations[ 'error_value_less_than_5' ] ) ) )
					)
				),
				'headers' => array(
					'required' => true,
					'validators' => array(
						array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) ),
						array( 'name' => 'string_length' , 'options' => array( 'min' => 5 , 'messages' => array( \Zend\Validator\StringLength::TOO_SHORT => $this->translations[ 'error_value_less_than_5' ] ) ) )
					)
				),
				'content' => array(
					'required' => true,
					'validators' => array(
						array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) ),
						array( 'name' => 'string_length' , 'options' => array( 'min' => '5' , 'messages' => array( \Zend\Validator\StringLength::TOO_SHORT => $this->translations[ 'error_value_less_than_5' ] ) ) )
					)
				),
				'name' => array(
					'required' => false,
					'validators' => array()
				),
				'copyrights' => array(
					'required' => true,
					'validators' => array(
						array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) )
					)
				),
				'author' => array(
					'required' => true,
					'validators' => array(
						array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) )
					)
				),
				'type' => array(
					'required' => true,
					'validators' => array(
						array( 'name' => 'in_array' , 'options' => array( 'haystack' => $_publicationTypesValidate , 'messages' => array( \Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations[ 'error_select_value' ] ) ) )
					)
				),
				'language' => array(
					'required' => true,
					'validators' => array(
						array( 'name' => 'in_array' , 'options' => array( 'haystack' => $_languageValidate , 'messages' => array( \Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations[ 'error_select_value' ] ) ) ) 
					)
				),
				'date' => array(
					'required' => true,
					'validators' => array(
						array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) ),
						array( 'name' => 'regex' , 'options' => array( 'pattern' => '/[0-9]{2}\/[0-9]{2}\/[0-9]{4}/' , 'messages' => array( \Zend\Validator\Regex::NOT_MATCH => $this->translations[ 'error_date_format_slashes' ] ) ) )
					)
				),
				'avatar' => array( 
					'required' => false,
					'validators' => array(
						array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) )
					)
				),
				'keywords' => array(
					'required' => true,
					'validators' => array( 
						array( 'name' => 'regex' , 'options' => array( 'pattern' => '/' . RegexBase::get( 'keywords' ) . '/' , 'messages' => array( \Zend\Validator\Regex::NOT_MATCH => 'niepasuje' ) ) ),
						array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) )
					)
				),
				'journal' => array(
					'required' => true,
					'validators' => array(
						array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) ),
						array( 'name' => 'in_array' , 'options' => array( 'haystack' => $_journalsValidation , 'messages' => array( \Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations[ 'error_select_value' ] ) ) )
					)
				),
				'page' => array(
					'required' => false,
					'validators' => array(
						array( 'name' => 'regex' , 'options' => array( 'pattern' => '/' . RegexBase::get( 'pageRange' ) . '/' , 'messages' => array( \Zend\Validator\Regex::NOT_MATCH => $this->translations[ 'error_page_invalid' ] ) ) )
					)
				),
				'esculap_collection' => array(
					'required' => false
				),
				'content_of_the_day' => array(
					'required' => false,
					'validators' => array(
						array( 'name' => 'in_array' , 'options' => array( 'haystack' => array( 0 , 1 ) ) , 'messages' => array( \Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations[ 'main_content_of_the_day_eror' ] ) )
					)
				),
				'active' => array( 'required' => false ),
				'searchable' => array( 'required' => false ),
				'specs' => array( 'required' => false ),
				'specs_value' => array( 'required' => false ),
				'avatar_value' => array( 'required' => false ),
				'graphics' => array( 'required' => false ),
				'graphics_value' => array( 'required' => false ),
				'spec_names' => array( 'required' => false ),
				'document' => array( 'required' => false ),
				'document_value' => array( 'required' => false ),
                                
				'edition_year' => array( 'required' => false ),
				'edition_nr' => array( 'required' => false ),
				'edition_book_nr' => array( 'required' => false ),
				'book_nr' => array( 'required' => false ),
				'book_year' => array( 'required' => false,
                                    'validators' => array(
						array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('num') . '/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_value_must_be_integer']))),
					)
                                ),
				'chief_nr' => array( 'required' => false ),
				'subject' => array( 'required' => false ),
                                'book_month' => array(
					'required' => false,
					'validators' => array(
						array( 'name' => 'in_array' , 'options' => array( 'haystack' => $monthSelectValidation , 'messages' => array( \Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations[ 'error_select_value' ] ) ) )
					)
				),
                            
                                'main_site_id' => array(
                                    'required' => true,
                                    'validators' => array(
                                        array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                                        array('name' => 'in_array', 'options' => array('haystack' => $mainSiteSelectValidation, 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['error_select_value']))),
                                    )
                                ),
                            
                                'main_editions' => array( 'required' => false,
                                    'validators' => array(
                                        array('name' => 'Zend\Validator\Db\RecordExists', 'options' => array('adapter' => $tableGateway->getAdapter(), 'schema' => MainEditionsTableGateway::getSchema(), 'table' => MainEditionsTableGateway::getTableName(), 'field' => 'id_edition', 'messages' => array(\Zend\Validator\Db\RecordExists::ERROR_NO_RECORD_FOUND => $this->translations['record_dont_exists']))),
                                    )
                                ),
                                
                                'main_editions_visible' => array( 'required' => false),
			);
			
		
		parent::init( $fields );
		
		$this->_messages = array(
			'title' => array(
				$this->translations[ 'error_value_is_required' ],
				$this->translations[ 'error_value_less_than_5' ]
			),
			'english_title' => array(
				$this->translations[ 'error_value_less_than_5' ]
			),
			'headers' => array(
				$this->translations[ 'error_value_is_required' ],
				$this->translations[ 'error_value_less_than_5' ]
			),
			'author' => array(
				$this->translations[ 'error_value_is_required' ]
			),
			'copyrights' => array(
				$this->translations[ 'error_value_is_required' ]
			),
			'type' => array(
				$this->translations[ 'error_select_value' ]
			),
			'date' => array(
				$this->translations[ 'error_value_is_required' ],
				$this->translations[ 'error_date_format_slashes' ]
			),
			'language' => array(
				$this->translations[ 'error_select_value' ]
			),
			'avatar' => array(
				$this->translations[ 'error_value_is_required' ]
			),
			'keywords' => array(
				'niepasuje',
				$this->translations[ 'error_value_is_required' ]
			),
			'journal' => array(
				$this->translations[ 'error_value_is_required' ],
				$this->translations[ 'error_select_value' ]
			),
			'page' => array(
				$this->translations[ 'error_page_invalid' ]
			),
                        'main_site_id' => array(
				$this->translations[ 'error_value_is_required' ],
				$this->translations[ 'error_select_value' ],
			),
                        'book_month' => array(
                                $this->translations[ 'error_select_value' ],
                        ),
                        'book_year' => array (
                            $this->translations['error_value_must_be_integer'],
                        ),
                        'main_editions' => array (
                            $this->translations['record_dont_exists'],
                        )
		);
		
	}
	
	public function getData() {
		return $this->form->getData();
	}
	
	public function draw() {
		// zrobione w ten spos�b bo zdj�cia dodatkowe oraz awatar dodawane s� dynamicznie i niemo�na okre�li� ich ilo�ci
		$firstStepFields = array(
			'title',
			'english_title',
			'headers',
			'keywords',
			'content',
			//'content_view',
			'avatar',
			'avatar_value',
			'graphics',
			'graphics_value',
			'document',
			'document_value',
			'collection',
			'collection_button'
		);
		
		$queue = array(
			1 => array(
				'name' => $this->translations[ 'main_content_and_graphics' ],
				'lock' => 0,
				'elements' => $firstStepFields,
				'navigation' => array(
					'nextStep'
				)
			),
			2 => array(
				'name' => $this->translations[ 'main_additional_informations' ],
				'lock' => true == $this->edit ? 0 : 1 ,
				'elements' => array(
					'author',
					'copyrights',
					'type',
					'journal',
                                        'main_editions',
                                        'main_editions_visible',
					'page',
					'date',
					'language',
                                        'main_site_id',
					'active',
					'searchable',
					'esculap_collection',
					'content_of_the_day',
					'specs',
					'specs_value',
					'spec_names',
				),
				'navigation' => array(
					'prevStep',
                                        'save',
				)
			),
                        /*3 => array(
                            'name' => $this->translations[ 'edition_information' ],
				'lock' => true == $this->edit ? 0 : 1 ,
				'elements' => array(
                                        'book_year',
                                        'book_month',
                                        'subject',
					'edition_year',
					'edition_nr',
					'edition_book_nr',
					'book_nr',
					'chief_nr',
				),
				'navigation' => array(
					'prevStep',
					'save'
				)
                        )*/
		);
		
		
		$this->form->queue = $queue;
			
		return $this->form;
	}
	
	public function setData( $data ) {
//		/var_dump($data);
		foreach( $data as $name => $value ) {
                    
			if( ( $name == 'active' || $name == 'searchable' || $name == 'esculap_collection' || $name == 'content_of_the_day' ) && $value == 1 ) {
				
				$this->form->get( $name )->setAttribute( 'checked' , 'checked' );
			} else {
				$value = 0;
			}
			
			
			if( $name == 'document_value' && strlen( $value ) > 1 ) {
				
				$this->validation[ 'content' ][ 'required' ] = false;
			}
		}
		
		parent::setData( $data );
                //var_dump($this);
                //die();
	}
}


?>