<?php

namespace System\Form;

use Zend\Filter;

class CmsDeleteAdminUserGroup extends AbstractForm implements DrawableInterface {

    protected $name = 'delete_admin_user_group';
    protected $translations;
    protected $edit = false;

    public static function neededTranslations() {
        return array(
            'delete', 'user', 'user_is_not_related', 'error_value_is_required'
        );
    }

    public function __construct(array $translations, array $usersToDelete) {
        $this->translations = $translations;

        $usersToDeleteList = array();
        $usersToDeleteListValidation = array();
        foreach ($usersToDelete as $user) {
            $usersToDeleteList[] = array(
                'value' => (int) $user['user_id'],
                'label' => (string) $user['user_login'],
            );
            $usersToDeleteListValidation[] = (int) $user['user_id'];
        }

        $fields = array(
            array('name' => 'user', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('multiple' => 'multiple', 'id' => 'user_autocomplete', 'req' => '', 'val' => 'num'), 'options' => array('value_options' => $usersToDeleteList, 'label' => $this->translations['user'])),
            array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['delete'], 'class' => 'submitStep form-submit'))
        );

        $this->validation = array(
            'user' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'System\Validator\Multiselect', 'options' => array('haystack' => $usersToDeleteListValidation), 'messages' => array(\System\Validator\Multiselect::VALUES_NOT_ALLOWED => $this->translations['user_is_not_related']))
                )
            ),
        );

        parent::init($fields);

        $this->_messages = array(
            'user' => array(
                $this->translations['error_value_is_required'],
                $this->translations['user_is_not_related']
            ),
        );
    }

    public function getData() {
        return $this->form->getData();
    }

    public function draw() {
        $queue = array(
            'user',
            'submit'
        );

        $this->form->queue = $queue;
        return $this->form;
    }

    public function addachDefaultFilters() {
        if (null !== $this->filterChain)
            return;

        $this->filtersForAll[] = new Filter\StringTrim;
        $this->filtersForAll[] = new \System\Filter\PregReplaceEx(array('match' => "/'/", 'replace' => "''"));
    }

}

?>