<?php

namespace System\Form;

class CmsQuizAddEdit extends AbstractForm implements DrawableInterface {
	
	protected $edit;
	protected $name = 'quiz_add';
	
	
	protected $filterExceptions = array( 'answers' );
	
	
	public static function neadedTranslations() {
		return array(
			'main_name' , 'main_question' , 'main_active' , 'main_save' , 'main_collections' , 'main_global_collection' , 'main_set' , 'quiz_main_informations' , 'main_global_collection' , 'main_answers' , 'error_value_is_required',
			'error_value_less_than_3', 'main_site', 'error_select_value'
		);
	}
	
	
	public function draw() {
		
		$queue = array(
			1 => array(
				'name' => $this->translations[ 'quiz_main_informations' ],
				'lock' => 0,
				'elements' => array(
					'name',
					'question',
					'answers',
					'main_site_id',
					'active',
					'esculap_collection',
					'collection',
					'collection_button'
				),
				'navigation' => array(
					'save'
				)
			)
		);
		
		
		$this->form->queue = $queue;
			
		return $this->form;
	}
	
	
	public function __construct( $translations , $mainSiteSelect, $edit = false ) {
		$this->translations = $translations;
		
                $mainSiteSelectOptions = array('' => '');
                $mainSiteSelectValidation = array();
                foreach ($mainSiteSelect as $mainSiteSelectOption) {
                    $mainSiteSelectOptions[] = array(
                        'value' => (int)$mainSiteSelectOption->getId(),
                        'label' => (string)$mainSiteSelectOption->getName(),
                    );
                    $mainSiteSelectValidation[] = (int)$mainSiteSelectOption->getId();
                }
                
		$this->edit = (bool)$edit;
		if( $this->edit )
			$this->name = preg_replace( '/add/' , 'edit' , $this->name );
		
		
		$fields = array(
			array( 'name' => 'name' , 'type' => 'text' , 'attributes' => array( 'class' => 'inp-form' ) , 'options' => array( 'label' => $this->translations[ 'main_name' ] ) ),
			array( 'name' => 'question' , 'type' => 'textarea' , 'attributes' => array( 'class' => 'form-textarea' ) , 'options' => array( 'label' => $this->translations[ 'main_question' ] ) ),
			array( 'name' => 'answers' , 'type' => '\System\Form\Element\QuizAnswers' , 'attributes' => array( 'multiple' => 'multiple' ) , 'options' => array( 'label' => $this->translations[ 'main_answers' ] , 'value_options' => array( ) ) ),
                    
                        array('name' => 'main_site_id', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('id' => 'main_site_select'), 'options' => array('value_options' => $mainSiteSelectOptions, 'label' => $this->translations['main_site'])),
                    
			array( 'name' => 'active' , 			'type' => 'Zend\Form\Element\Checkbox' , 'attributes' => array( 'value' => "1" ) , 'options' => array( 'label' => $this->translations[ 'main_active' ] ) ),
			array( 'name' => 'esculap_collection' , 'type' => 'Zend\Form\Element\Checkbox' , 'attributes' => array( 'value' => "1" ) , 'options' => array( 'label' => $this->translations[ 'main_global_collection' ] ) ),
			array( 'name' => 'save', 'type' => 'Zend\Form\Element\Button','attributes' => array( 'title' => $this->translations['main_save'] ,'class' => 'info-tooltip save_button submitStep' ) ),
			array( 'name' => 'collection' , 'type' => 'Zend\Form\Element\Hidden' , 'options' => array() , 'attributes' => array( 'class' => 'no_height' ) ),
			array( 'name' => 'collection_button' , 'type' => 'Zend\Form\Element\Button' , 'options' => array( 'label' => $this->translations[ 'main_collections' ] ) , 'attributes' => array( 'value' => $this->translations[ 'main_set' ] , 'class' => 'collection_button event form-submit' ) ),
		);
		
		$this->validation = array(
			'name' => array(
				'required' => true,
				'validators' => array(
					array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) ),
					array( 'name' => 'string_length' , 'options' => array( 'min' => 3 , 'messages' => array( \Zend\Validator\StringLength::TOO_SHORT => $this->translations[ 'error_value_less_than_3' ] ) ) )
				)
			),
			'question' => array(
				'required' => true,
				'validators' => array(
					array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) )
				)
			),
			'answers' => array(
				'required' => true,
				'validators' => array(
					array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) )
				)
			),
			'active' => array(
				'required' => false,
				'validators' => array(
					array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) ),
					array( 'name' => 'in_array' , 'options' => array( 'haystack' => array( '0' , '1' ) ) )
				)
			),
			'esculap_collection' => array(
				'required' => false,
				'validators' => array(
					array( 'name' => 'not_empty' , 'options' => array( 'messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_value_is_required' ] ) ) ),
					array( 'name' => 'in_array' , 'options' => array( 'haystack' => array( '0' , '1' ) ) )
				)
			),
                    'main_site_id' => array(
                        'required' => true,
                        'validators' => array(
                            array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                            array('name' => 'in_array', 'options' => array('haystack' => $mainSiteSelectValidation, 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['error_select_value']))),
                        )
                    ),
		);
		
		
		
		parent::init( $fields );
	}
	
	
	public function setData( $data ) {
		parent::setData( $data );
		
		if( isset( $data[ 'answers' ] ) ) {
			$answers = array();
			$answersValues = array();
			
			foreach( (array)$data[ 'answers' ] as $k => $answer ) {
				$value = isset( $data[ 'answers_value' ][ $k ] ) ? $data[ 'answers_value' ][ $k ] : $k; 
				
				$answersValues[] = $answer;
				
				$answers[] = array(
					'value' => $value,
					'label' => $answer,
					'selected' => true
				);
			}
			
			$this->form->get( 'answers' )->setValueOptions( $answers );
			$this->form->get( 'answers' )->setValue( $answersValues );
		} if( isset( $data[ 'active' ] ) && 1 == (int)$data[ 'active' ] ) {
			
			$this->form->get( 'active' )->setAttributes( array( 'checked' => true ) );
			
		} if( isset( $data[ 'esculap_collection' ] ) && 1 == (int)$data[ 'esculap_collection' ] ) {
			
			$this->form->get( 'esculap_collection' )->setAttributes( array( 'checked' => true ) );
		}
	}
	
}


?>