<?php

namespace System\Form;

use Zend\Form\Element;
use Zend\Filter;

class ContentFileAddEdit extends AbstractForm implements DrawableInterface {

    protected $translations;
    protected $name = 'fileEditAdd';
    protected $allowOverride = false;

    public static function neededTranslations() {
        return array(
            'file', 'error_is_empty', 'main_save', 'file_exists', 'file_error', 'file_name_error', 'override', 'checkbox_error'
        );
    }

    public function __construct(array $translations, $new = false, $allowOverride = false) {
        $this->translations = $translations;
        $this->parseTranslations(self::neededTranslations());
        $this->setAllowOverride($allowOverride);

        $fields = array(
            array('name' => 'file', 'type' => 'Zend\Form\Element\File', 'options' => array('label' => $this->translations['file']), 'attributes' => array('class' => '' . (false == $new ? 'input-dissabled ' : '') . 'inp-form')),
            array('name' => 'override', 'type' => 'Zend\Form\Element\Checkbox', 'options' => array('label' => $this->translations['override']), 'attributes' => array('class' => '' . (false == $new ? 'input-dissabled' : '') . ' inp-form')),
            array('name' => 'submit_button', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['main_save'], 'class' => 'form-submit'))
        );

        $this->validation = array(
            'file' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
                    array('name' => 'System\Validator\File', 'options' => array('allowOverride' => $this->getAllowOverride(),
                            'messages' => array(
                                \System\Validator\File::FILE_EXISTS => $this->translations['file_exists'],
                                \System\Validator\File::FILE_ERROR => $this->translations['file_error'],
                                \System\Validator\File::FILE_NAME_ERROR => $this->translations['file_name_error'])))
                )
            ),
            'override' => array(
                'required' => false,
                'validators' => array(array('name' => 'in_array', 'options' => array('haystack' => array(0, 1)), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['checkbox_error'])))
            )
        );

        parent::init($fields);
    }

    public function draw() {
        if ($this->allowOverride == true) {
            $this->form->queue = array('file', 'override', 'submit_button');
        } else {
            $this->form->queue = array('file', 'submit_button');
        }

        return $this->form;
    }

    public function addachDefaultFilters() {
        if (null !== $this->filterChain)
            return;

        $this->filtersForAll[] = new Filter\StringTrim;
        $this->filtersForAll[] = new \System\Filter\PregReplaceEx(array('match' => "/\'/", 'replace' => "\""));
    }

    public function getAllowOverride() {
        return $this->allowOverride;
    }

    public function setAllowOverride($allowOverride) {
        $this->allowOverride = $allowOverride;
    }

}

?>