<?php
namespace System\Form;

use Zend\Form\Element;
use Zend\Filter;
use System\Validator\RegexBase;

class ContentFileAvatarAddEdit extends AbstractForm implements DrawableInterface {

    protected $translations;
    protected $name = 'fileAvatarEditAdd';
    protected $allowOverride = false;
    protected $changeGroup = false;

    public static function neededTranslations() {
        return array(
            'file', 'error_is_empty', 'main_save', 'file_exists', 'file_error', 'file_name_error', 'override', 'checkbox_error', 'active', 'group', 'group_does_not_exist', 'error_name_invalid', 'name_avatar', 'error_value_is_required'
        );
    }

    public function __construct(array $translations, $avatarTypes, $new = false, $allowOverride = false, $changeGroup = false) {
        $this->translations = $translations;
        $this->parseTranslations(self::neededTranslations());
        $this->setAllowOverride($allowOverride);
        $this->changeGroup = $changeGroup;

        $avatarTypesList = array('' => '');
        $avatarTypesListValidation = array('');
        foreach ($avatarTypes as $id => $avatarType) {
            $avatarTypesList[] = array(
                'value' => (int)$id,
                'label' => (string)$avatarType,
            );

            $avatarTypesListValidation[] = (int)$id;
        }

        if ($this->getAllowOverride()) {
            $fields = array(
                array('name' => 'file', 'type' => 'Zend\Form\Element\File', 'options' => array('label' => $this->translations['file']), 'attributes' => array('class' => '' . (false == $new ? 'input-dissabled ' : '') . 'inp-form')),
                //array('name' => 'name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'avatar'), 'options' => array('label' => $this->translations['name_avatar'])),
                //array('name' => 'group', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('id' => 'group_autocomplete', 'req' => '', 'val' => 'num'), 'options' => array('value_options' => $avatarTypesList, 'label' => $this->translations['group'])),
                //array('name' => 'active', 'type' => 'Zend\Form\Element\Checkbox', 'options' => array('label' => $this->translations['active'])),
                //array('name' => 'override', 'type' => 'Zend\Form\Element\Checkbox', 'options' => array('label' => $this->translations['override']), 'attributes' => array('class' => '' . (false == $new ? 'input-dissabled' : '') . ' inp-form')),
                array('name' => 'submit_button', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['main_save'], 'class' => 'form-submit'))
            );
        } elseif ($this->changeGroup) {
            $fields = array(
                //array('name' => 'file', 'type' => 'Zend\Form\Element\File', 'options' => array('label' => $this->translations['file']), 'attributes' => array('class' => '' . (false == $new ? 'input-dissabled ' : '') . 'inp-form')),
                //array('name' => 'name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'avatar'), 'options' => array('label' => $this->translations['name_avatar'])),
                array('name' => 'group', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('id' => 'group_autocomplete', 'req' => '', 'val' => 'num'), 'options' => array('value_options' => $avatarTypesList, 'label' => $this->translations['group'])),
                //array('name' => 'active', 'type' => 'Zend\Form\Element\Checkbox', 'options' => array('label' => $this->translations['active'])),
                //array('name' => 'override', 'type' => 'Zend\Form\Element\Checkbox', 'options' => array('label' => $this->translations['override']), 'attributes' => array('class' => '' . (false == $new ? 'input-dissabled' : '') . ' inp-form')),
                array('name' => 'submit_button', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['main_save'], 'class' => 'form-submit'))
            );
        } else {
            $fields = array(
                array('name' => 'file', 'type' => 'Zend\Form\Element\File', 'options' => array('label' => $this->translations['file']), 'attributes' => array('class' => '' . (false == $new ? 'input-dissabled ' : '') . 'inp-form')),
                array('name' => 'name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => 'avatar'), 'options' => array('label' => $this->translations['name_avatar'])),
                array('name' => 'group', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('id' => 'group_autocomplete', 'req' => '', 'val' => 'num'), 'options' => array('value_options' => $avatarTypesList, 'label' => $this->translations['group'])),
                //array('name' => 'active', 'type' => 'Zend\Form\Element\Checkbox', 'options' => array('label' => $this->translations['active'])),
                array('name' => 'override', 'type' => 'Zend\Form\Element\Checkbox', 'options' => array('label' => $this->translations['override']), 'attributes' => array('class' => '' . (false == $new ? 'input-dissabled' : '') . ' inp-form')),
                array('name' => 'submit_button', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $this->translations['main_save'], 'class' => 'form-submit'))
            );
        }

        if ($this->getAllowOverride()) {
            $this->validation = array(
                'file' => array(
                    'required' => true,
                    'validators' => array(
                        array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
                        array('name' => 'System\Validator\FileAvatar', 'options' => array('allowOverride' => $this->getAllowOverride(),
                                'messages' => array(
                                    \System\Validator\File::FILE_ERROR => $this->translations['file_error'],
                                )))
                    )
                ),
            );
        } elseif ($this->changeGroup) {
            $this->validation = array(
                'group' => array(
                    'required' => true,
                    'validators' => array(
                        array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                        array('name' => 'in_array', 'options' => array('haystack' => $avatarTypesListValidation), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['group_does_not_exist']))
                    )
                )
            );
        } else {
            $this->validation = array(
                'file' => array(
                    'required' => true,
                    'validators' => array(
                        array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
                        array('name' => 'System\Validator\FileAvatar', 'options' => array('allowOverride' => $this->getAllowOverride(),
                                'messages' => array(
                                    \System\Validator\File::FILE_ERROR => $this->translations['file_error'],
                                )))
                    )
                ),
                'group' => array(
                    'required' => true,
                    'validators' => array(
                        array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                        array('name' => 'in_array', 'options' => array('haystack' => $avatarTypesListValidation), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['group_does_not_exist']))
                    )
                ),
                /* 'active' => array(
                  'required' => false,
                  'validators' => array(array('name' => 'in_array', 'options' => array('haystack' => array(0, 1)), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['checkbox_error'])))
                  ), */
                'name' => array(
                    'required' => true,
                    'validators' => array(
                        array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_value_is_required']))),
                        array('name' => 'regex', 'options' => array('pattern' => '/' . RegexBase::get('avatar') . '/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_name_invalid'])))
                    )
                ),
                'override' => array(
                    'required' => false,
                    'validators' => array(array('name' => 'in_array', 'options' => array('haystack' => array(0, 1)), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $this->translations['checkbox_error'])))
                )
            );
        }

        parent::init($fields);
    }

    public function draw() {
        if ($this->allowOverride == true) {
            $this->form->queue = array('file', /* 'name', 'group', *//* 'active', 'override', */ 'submit_button');
        } elseif ($this->changeGroup) {
            $this->form->queue = array(/*'file', 'name',*/'group',/* 'active', */ 'submit_button');
        } else {
            $this->form->queue = array('file', 'name', 'group', /* 'active', */ 'submit_button');
        }

        return $this->form;
    }

    public function addachDefaultFilters() {
        if (null !== $this->filterChain) return;

        $this->filtersForAll[] = new Filter\StringTrim;
        $this->filtersForAll[] = new \System\Filter\PregReplaceEx(array('match' => "/\'/", 'replace' => "\""));
    }

    public function getAllowOverride() {
        return $this->allowOverride;
    }

    public function setAllowOverride($allowOverride) {
        $this->allowOverride = $allowOverride;
    }

}

?>