<?php

namespace System\Form;
use Zend\Form\Element;
use Zend\Filter;

class ContentPhraseAddEdit extends AbstractForm implements DrawableInterface {

	protected $translations;
	protected $name = 'phraseEditAdd';
	
	public static function neededTranslations() {
		return array(
			'main_identifier','main_phrase_lang_pl','main_phrase_lang_en','main_phrase_lang_ru','main_phrase_lang_ua','main_phrase_system','namespace','error_is_empty','main_save','main_phrase_locked'
		);
	} 
	
	public function __construct(array $translations,$allowedFields,$systems = null,$namespace = null,$new = false) {
		$this->translations = $translations;
		$this->parseTranslations(self::neededTranslations());
		
		$fields = array(
			array('name' => 'id','options' => array('label' => $this->translations['main_identifier']), 'attributes' => array('class' => ''.(false == $new ? 'input-dissabled' : '').' inp-form')),
			array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit','attributes' => array('value' => $this->translations['main_save'],'class' => 'submitStep form-submit'))
		);
		
		if(isset($allowedFields['lock'])) {
			$fields[] = array('name' => 'locked','type' => 'Zend\Form\Element\Checkbox','options' => array('use_hidden_element' => true,'checked_value' => '1','unchecked_value' => '0','label' => $this->translations['main_phrase_locked']));
		} else {
			$fields[] = array('name' => 'locked','type' => 'Zend\Form\Element\Checkbox','options' => array('use_hidden_element' => true,'checked_value' => '1','unchecked_value' => '0','label' => $this->translations['main_phrase_locked']) , 'attributes' => array( 'disabled' => 'disabled' ) );
		}
		
		if(isset($allowedFields['pl'])) {
			$fields[] = array('name' => 'phrase_pl','type' => 'Zend\Form\Element\Textarea','options' => array('label' => $this->translations['main_phrase_lang_pl']), 'attributes' => array('class' => ($allowedFields['pl'] == 0 ? 'input-dissabled ' : '').'form-textarea'));
		}
		
		if(isset($allowedFields['ru'])) {
			$fields[] = array('name' => 'phrase_ru','type' => 'Zend\Form\Element\Textarea','options' => array('label' => $this->translations['main_phrase_lang_ru']), 'attributes' => array('class' => ($allowedFields['ru'] == 0 ? 'input-dissabled ' : '').'form-textarea'));
		}
		
		if(isset($allowedFields['ua'])) {
			$fields[] = array('name' => 'phrase_ua','type' => 'Zend\Form\Element\Textarea','options' => array('label' => $this->translations['main_phrase_lang_ua']), 'attributes' => array('class' => ($allowedFields['ua'] == 0 ? 'input-dissabled ' : '').'form-textarea'));
		}
		
		if(isset($allowedFields['en'])) {
			$fields[] = array('name' => 'phrase_en','type' => 'Zend\Form\Element\Textarea','options' => array('label' => $this->translations['main_phrase_lang_en']), 'attributes' => array('class' => ($allowedFields['en'] == 0 ? 'input-dissabled ' : '').'form-textarea'));
		}
		
		if(null !== $systems) {
			$fields[] = array('name' => 'system','type' => 'Zend\Form\Element\Select','options' => array('value_options' => $systems,'label' => $this->translations['main_phrase_system']),'attributes' => array('class' => 'styledselect_form_1'));
		} if(null !== $namespace) {
			$fields[] = array('name' => 'namespace', 'options' => array('label' => $this->translations['namespace']),'attributes' => array('class' => 'inp-form'));
		}
		
		$this->validation = array(
			'id' => array(
				'required' => true,
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
				)
			),
			'phrase_pl' => array(
				'required' => false
			),	
			'locked' => array(
				'required' => false,
			),
			'phrase_ru' => array(
				'required' => false
			),
			'phrase_en' => array(
				'required' => false
			),
			'phrase_ua' => array(
				'required' => false
			)
		);
		
		parent::init($fields);
	}
	
	
	public function draw() {
		$queue = array('id');
		
		if(null !== $this->getForm()->get('system')) {
			$queue[] = 'system';
		} if(null !== $this->getForm()->get('namespace')) {
			$queue[] = 'namespace';
		} if(null !== $this->getForm()->get('locked')) {
			$queue[] = 'locked';
		} if(null !== $this->getForm()->get('phrase_pl')) {
			$queue[] = 'phrase_pl';
		} if(null !== $this->getForm()->get('phrase_ru')) {
			$queue[] = 'phrase_ru';
		} if(null !== $this->getForm()->get('phrase_ua')) {
			$queue[] = 'phrase_ua';
		} if(null !== $this->getForm()->get('phrase_en')) {
			$queue[] = 'phrase_en';
		}
		
		
		$queue = array_merge($queue,array('submit'));
		$this->form->queue = $queue;
			
		return $this->form;
	}
	
	public function addachDefaultFilters() {
		if(null !== $this->filterChain) return;
		
		$this->filtersForAll[] = new Filter\StringTrim;
		$this->filtersForAll[] = new Filter\StripTags(array('allowTags' => array( 'a' , 'i' , 'u' , 'b' , 'p' , 'br' ) , 'allowAttribs' => array( 'href' ) ));
		$this->filtersForAll[] = new \System\Filter\PregReplaceEx(array('match' => "/\'/",'replace' => "\""));
	}
}

?>