<?php
namespace System\Form;

use System\Validator\RegexBase;
use System\Translator;
use Zend\Validator\NotEmpty;
use Zend\Validator\Regex;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Model\Entity\MainSite as Entity;

class EntityGenerator extends AbstractForm implements DrawableInterface {

    protected $name = 'entity_generator';

    public function __construct($formMode, $data = null) {
        $translator = Translator\TranslatorEx::get();

        if (defined('parent::' . $formMode)) {
            $this->setFormMode($formMode);
        } else {
            $this->setFormMode(parent::ADD);
        }

        if ($data && $this->getFormMode() == parent::EDIT) {
            $data = $this->adaptData($data);
        }

        $fields = array(
            array('name' => 'name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => ''), 'options' => array('label' => $translator->translate('name', 'generator'))),
            array('name' => 'submit-button', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('class' => 'form-submit', 'value' => $translator->translate('generate'))),
        );

        if ($this->formMode == parent::VIEW) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox') {
                    $field['attributes']['disabled'] = 'disabled';
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }

        $this->validation = array(
            'name' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                ),
                'filters' => array(
                    array(
                        'name' => '\Zend\Filter\StripTags',
                        'name' => '\Zend\Filter\StringTrim',
                        'name' => '\System\Filter\PregReplaceEx', 'options' => array('match' => "/\'/", 'replace' => "\""),
                    ),
                ),
            ),
        );

        parent::init($fields);
        
        $this->_messages = array(
            'name' => array(
                $translator->translate('error_value_is_required'),
            ),
        );

        if ($data && $this->getFormMode() == parent::EDIT) {
            $this->setData($data);
        }
    }

    public function draw() {
        $queue = array(
            'name',
            'submit-button',
        );

        if ($this->formMode == parent::VIEW) {
            array_pop($queue);
        }

        $this->form->queue = $queue;
        return $this->form;
    }

    public function getData() {
        $data = $this->form->getData();
        return $data;
//        $hydrator = new ClassMethodsHydrator();
//        return $hydrator->hydrate($data, new Entity());
    }

    public function adaptData($data) {
        if ($data instanceof Entity) {
            $hydrator = new ClassMethodsHydrator();
            $data = $hydrator->extract($data);
        }
        return $data;
    }

    public function setData($data) {
        $data = $this->adaptData($data);
        parent::setData($data);
    }

    public function addachDefaultFilters() {
        if ($this->filterChain !== null) return;
    }

}

?>