<?php

namespace System\Form;
use Zend\Form\Element as FormElement;

class EventAdd extends AbstractForm implements DrawableInterface {
	
	protected $translations;
	protected $name = 'EventAdd';
	protected $validation;
	
	protected $filterExceptions = array('spec');
	
	public static function neededTranslations() {
		return array(
			'main_event_title','main_city','main_address','main_postal_code','main_website','main_event_start','main_event_end','main_additional_info','main_event_type','main_submit','error_is_empty',
			'error_not_match','error_to_short','main_organizer_name','main_organizer_address','main_tel','main_fax','main_email'
		);
	}
	
	public function __construct(array $translations,$eventTypes,$specTypes) {
		$this->translations = $translations;
		$this->parseTranslations(self::neededTranslations());
		
		$fields = array(
			array('name' => 'type','type' => 'Zend\Form\Element\Select',  'options' => array('value_options' => $eventTypes,'label' => $this->translations['main_event_type'])),
			array('name' => 'title', 'options' => array('label' => $this->translations['main_event_title'])),
			array('name' => 'city', 'options' => array('label' => $this->translations['main_city'])),
			array('name' => 'address', 'options' => array('label' => $this->translations['main_address'])),
			array('name' => 'postal_code', 'attributes' => array('class' => 'number'), 'options' => array('label' => $this->translations['main_postal_code'])),
			array('name' => 'website', 'options' => array('label' => $this->translations['main_website'])),
			array('name' => 'start', 'attributes' => array('class' => 'date_picker'), 'options' => array('label' => $this->translations['main_event_start'])),
			array('name' => 'end', 'attributes' => array('class' => 'date_picker'), 'options' => array( 'label' => $this->translations['main_event_end'])),
			array('name' => 'info', 'type' => 'Zend\Form\Element\Textarea', 'options' => array('label' => $this->translations['main_additional_info'])),
			array('name' => 'spec', 'type' => 'System\Form\Element\Select', 'options' => array('value_options' => $specTypes,'label' => $this->translations['main_additional_info']),'attributes' => array('multiple' => 'multiple')),
			array('name' => 'organizer', 'options' => array('label' => $this->translations['main_organizer_name'])),
			array('name' => 'organizer_address', 'type' => 'Zend\Form\Element\Textarea', 'options' => array('label' => $this->translations['main_organizer_address'])),
			array('name' => 'organizer_tel', 'options' => array('label' => $this->translations['main_tel'])),
			array('name' => 'organizator_fax', 'options' => array('label' => $this->translations['main_fax'])), 
			array('name' => 'organizator_email', 'options' => array('label' => $this->translations['main_email'])),
			array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit','attributes' => array('value' => $this->translations['main_submit'],'class' => 'btn1')),
		);
		
		$this->validation = array(
			'type' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty'])))
				)
			),
			'title' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty'])))
				)
			),
			'city' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty'])))
				)
			),
			'address' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty'])))
				)
			),
			'postal_code' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
					array('name' => 'regex', 'options' => array('pattern' => '/[0-9-]+/','messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_not_match'])))
				)
			),
			'website' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
				)
			),
			'end' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
					array('name' => 'regex', 'options' => array('pattern' => '/[0-9]{4}-[0-9]{2}-[0-9]{2}/', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_not_match']))),
					array('name' => 'System\Validator\ComparisonGreaterThan')
				)
			),
			'start' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
					array('name' => 'regex', 'options' => array('pattern' => '/[0-9]{4}-[0-9]{2}-[0-9]{2}/', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_not_match']))),
					array('name' => 'System\Validator\ComparisonGreaterThan','options' => array('messages' => array(\System\Validator\ComparisonGreaterThan::INVALID_VALUE => $this->translations['error_not_match'])))
				)
			), 
			'info' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
					array('name' => 'string_length','options' => array('min' => 10, 'messages' => array(\Zend\Validator\StringLength::TOO_SHORT => $this->translations['error_to_short'])))
				),
				'filters' => array(
					array('name' => 'strip_tags')
				)
			),
			'spec' => array(
				'validators' => array(
					array( 'name' => 'not_empty', 'options' => array('messages' => array( \Zend\Validator\NotEmpty::IS_EMPTY => $this->translations[ 'error_is_empty' ] ) ) ),
					array( 'name' => 'System\Validator\Spec' )
				)
			),
			'organizer' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
				)
			),
			'organizer_address' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
					array('name' => 'string_length','options' => array('min' => 10, 'messages' => array(\Zend\Validator\StringLength::TOO_SHORT => $this->translations['error_to_short'])))
				)
			),
			'organizer_tel' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
					array('name' => 'regex', 'options' => array('pattern' => '/[0-9 -\(\)]{6}/', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_not_match'])))
				)
			),
			'organizator_fax' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
					array('name' => 'regex', 'options' => array('pattern' => '/[0-9 -]{6}/', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_not_match'])))
				)
			),
			'organizator_email' => array(
				'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
					//array('name' => 'email_address','options' => array('messages' => array(\Zend\Validator\EmailAddress::INVALID_FORMAT => $this->translations['error_not_match'])))
				)
			)
		);
		
		parent::init($fields);
	}
	
	/**
	* Przeładowna funkcja ustawiająca dane.
	*/
	public function setData($data) {
		//if(is_array($data)) {
			//$data = parent::setData($data);
		//}
		
		
		// należy przetworzyć klucz spec aby działał z esulapowym polem wybierania specializacji
		if(isset($data['spec'])) {
		
			$options = array();
			$selected = array();
			foreach($this->getForm()->get('spec')->getValueOptions() as $option) {
				
				if(isset($option['value']) && in_array($option['value'],$data['spec'])) {
					
					$selected[] = $this->filter($option['value']);
					$option['selected'] = 'selected';
				}
				
				$options[] = $option;
			}
			
			
			//print_r(get_class_methods($this->getForm()->get('spec')));
			
			$data['spec'] = $selected;
			
			$this->getForm()->get('spec')->setValue($selected);
			$this->getForm()->get('spec')->setValueOptions($options);
		}
		
		parent::setData($data);
		//$this->getForm()->setData($data);		
	}
	
	 public function draw() {
		
		$this->form->queue = array(
			'type','title','city','address','postal_code','website','start','end','info','spec','organizer','organizer_address','organizer_tel','organizator_fax','organizator_email','submit'
		);
		return $this->form;
	}
}

?>