<?php

namespace System\Form;
use Zend\Form\Element as FormElement;

class EventSerch extends AbstractForm implements DrawableInterface {
	
	protected $translations;
	protected $name = 'EventAdd';
	protected $validation;
	
	public static function neededTranslations() {
		return array(
			'main_specs','main_submit','main_country','main_organizer_name','main_year','main_month','error_is_empty','error_not_match', 'main_event_type', 'main_search','main_months'
		);
	}
	
	public function __construct(array $translations,$eventTypes,$specsList ) {
		$this->translations = $translations;
		$this->parseTranslations(self::neededTranslations());
		
		
		
		$temp_months = explode( ',' , $this->translations[ 'main_months' ] );
		
		$months = array();
		$currentMonth = (int)date('m');
		
		foreach($temp_months as $key => $month) {
			
			$temp = array(
				'value' => (int)$key + 1,
				'label' => $month
			);
			
			$months[] = $temp;
		}
		
		$fields = array(
			array('name' => 'spec','type' => 'Zend\Form\Element\Select', 'options' => array('value_options' => $specsList,'label' => $this->translations['main_specs']),'attributes' => array('class' => 'big')),
			array('name' => 'type','type' => 'Zend\Form\Element\Select', 'options' => array('value_options' => $eventTypes,'label' => $this->translations['main_event_type'])),
			array('name' => 'organization', 'options' => array('label' => $this->translations['main_organizer_name']),'attributes' => array('class' => 'normal')),
			array('name' => 'year','type' => 'Zend\Form\Element\Select','options' => array('value_options' => array(array('value' => date('Y'),'label' => date('Y')),array('value' => date('Y') + 1,'label' => date('Y') + 1)),'label' => $this->translations['main_year']),'attributes' => array('class' => 'small')),
			array('name' => 'month', 'type' => 'Zend\Form\Element\Select', 'options' => array('value_options' => $months,'label' => $this->translations['main_month']),'attributes' => array('class' => 'small' , 'value' => $currentMonth )),
			array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit','attributes' => array('value' => $this->translations['main_search'],'class' => 'btn1')),
		);
		
		$this->validation = array(
			'spec' => array(
				'required' => false,
				/*'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty']))),
					array('name' => 'System\Validator\Spec')
				)*/
			),
			'type' => array(
				'required' => false,
				/*'validators' => array(
					array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $this->translations['error_is_empty'])))
				)*/
			),
			'organization' => array(
				'required' => false
			),
			'year' => array(
				'validators' => array(
					array('name' => 'regex', 'options' => array('pattern' => '/^[0-9]{4}$/','messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_not_match'])))
				)
			),
			'month' => array(
				'validators' => array(
					array('name' => 'regex', 'options' => array('pattern' => '/^[0-9]{1,2}$/','messages' => array(\Zend\Validator\Regex::NOT_MATCH => $this->translations['error_not_match']))),
					array('name' => 'greater_than', 'options' => array('min' => 0, 'messages' => array(\Zend\Validator\GreaterThan::NOT_GREATER => $this->translations['error_not_match']))),
					array('name' => 'less_than','options' => array('max' => 13, 'messages' => array(\Zend\Validator\LessThan::NOT_LESS => $this->translations['error_not_match']))),
				)
			)
		);
		
		parent::init($fields);
	}
	
	 public function draw() {
		
		$this->form->queue = array(
			'spec','type','organization','year','month','submit'
		);
		return $this->form;
	}
}

?>