<?php
namespace System\Form;

use System\Validator\RegexBase;
use System\Translator;
use Zend\Validator\NotEmpty;
use Zend\Validator\Regex;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Model\Entity\MainEdition as Entity;
use Model\TableGateway\MainEditions as MainEditionsTableGateway;

class MainEditions extends AbstractForm implements DrawableInterface {

    protected $name = 'main_editions';

    public function __construct($formMode, $data = null) {
        $translator = Translator\TranslatorEx::get();
        $tableGateway = new MainEditionsTableGateway();

        if (defined('parent::' . $formMode)) {
            $this->setFormMode($formMode);
        } else {
            $this->setFormMode(parent::ADD);
        }

        if ($data instanceof Entity) {
            $imageName = $data->getImageName();
        }

        if ($data && $this->getFormMode() == parent::EDIT) {
            $data = $this->adaptData($data);
        }

        $monthNames = explode(',', $translator->translate('main_months_names'));
        foreach ($monthNames as $key => $monthName) {
            $monthNames[$key] = mb_strtoupper($monthName, 'UTF-8');
        }

        $monthSelectOptions = array(0 => '');
        foreach ($monthNames as $key => $monthSelectOption) {
            $monthSelectOptions[] = array(
                'value' => (string)$monthSelectOption,
                'label' => (string)$monthSelectOption,
            );
            if ($key < 11) {
                $monthSelectOptions[] = array(
                    'value' => (string)$monthSelectOption . '-' . $monthNames[$key + 1],
                    'label' => (string)$monthSelectOption . '-' . $monthNames[$key + 1],
                );
            }
            $monthSelectValidation[] = (string)$monthSelectOption;
            if ($key < 11) {
                $monthSelectValidation[] = (string)$monthSelectOption . '-' . $monthNames[$key + 1];
            }
        }
        $monthSelectValidation[] = '0';

        $tableGateway = new MainEditionsTableGateway();
        $listData = $tableGateway->getSubjects();

        $specSelectOptions = array();
        $specSelectValidation = array();
        foreach ($listData as $specSelectOption) {
            if ($specSelectOption) {
                $specSelectOptions[] = array(
                    'value' => (string)mb_convert_case($specSelectOption, MB_CASE_UPPER, "UTF-8"),
                    'label' => (string)mb_convert_case($specSelectOption, MB_CASE_UPPER, "UTF-8"),
                );
            }
            $specSelectValidation[] = (string)mb_convert_case($specSelectOption, MB_CASE_UPPER, "UTF-8");
        }

        $fields = array(
            array('name' => 'edition_year', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form'), 'options' => array('label' => $translator->translate('edition_year'))),
            array('name' => 'edition_nr', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form'), 'options' => array('label' => $translator->translate('edition_nr'))),
            array('name' => 'edition_book_nr', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form'), 'options' => array('label' => $translator->translate('edition_book_nr'))),
            array('name' => 'book_nr', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form'), 'options' => array('label' => $translator->translate('book_nr'))),
            array('name' => 'book_year', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'val' => 'numeric'), 'options' => array('label' => $translator->translate('book_year'))),
            array('name' => 'chief_nr', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form-long'), 'options' => array('label' => $translator->translate('chief_nr'))),
            array('name' => 'book_month', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('class' => 'styledselect_form_1'), 'options' => array('value_options' => $monthSelectOptions, 'label' => $translator->translate('book_month'))),
            array('name' => 'subject', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('id' => 'subject_select'), 'options' => array('value_options' => $specSelectOptions, 'label' => $translator->translate('subject'))),
            array('name' => 'status', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('value' => '0'), 'options' => array('label' => $translator->translate('visibility'))),
            array('name' => 'submit-button', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('class' => 'form-submit', 'value' => $translator->translate('save')))
        );

        if ($this->getFormMode() == parent::ADD) {
            $fields[] = array('name' => 'cover', 'type' => 'Zend\Form\Element\File', 'attributes' => array('id' => 'cover-file', 'class' => 'inp-form'), 'options' => array('label' => $translator->translate('cover')));
        } elseif ($this->getFormMode() == parent::EDIT) {
            $fields[] = array('name' => 'cover', 'type' => 'System\Form\Element\ImageChange', 'attributes' => array('id' => 'cover-file', 'class' => 'inp-form', 'image_name' => $imageName), 'options' => array('label' => $translator->translate('cover')));
        } elseif ($this->getFormMode() == parent::VIEW) {
            $fields[] = array('name' => 'cover', 'type' => 'System\Form\Element\ImageView', 'attributes' => array('id' => 'cover-file', 'class' => 'inp-form', 'image_name' => $imageName), 'options' => array('label' => $translator->translate('cover')));
        }

        if ($this->formMode == parent::VIEW) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox' || $field['type'] == 'Zend\Form\Element\Select') {
                    $field['attributes']['disabled'] = 'disabled';
                } elseif ($field['type'] == 'Zend\Form\Element\File') {
                    $field['attributes']['class'] = $field['attributes']['class'] . ' input-dissabled';
                    $field['attributes']['disabled'] = 'disabled';
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }

        $this->validation = array(
            'chief_nr' => array('required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                )
            ),
            'subject' => array('required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    //array('name' => 'in_array', 'options' => array('haystack' => $specSelectValidation, 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $translator->translate('error_select_value'))))
                )
            ),
            //----------------------------------------------
            'book_nr' => array('required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'regex', 'options' => array('pattern' => '/^(|[0-9]*|[0-9]+[\-]{1}[0-9]+)$/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $translator->translate('error_value_must_be_integer_range')))),
                )
            ),
            'edition_nr' => array('required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'regex', 'options' => array('pattern' => '/^(|[0-9]*|[0-9]+[\-]{1}[0-9]+)$/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $translator->translate('error_value_must_be_integer_range')))),
                )
            ),
            //----------------------------------------------
            'edition_year' => array('required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'regex', 'options' => array('pattern' => '/^M{0,4}(CM|CD|D?C{0,3})(XC|XL|L?X{0,3})(IX|IV|V?I{0,3})$/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $translator->translate('error_value_must_roman_integer')))),
                )
            ),
            'edition_book_nr' => array('required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'regex', 'options' => array('pattern' => '/^[0-9]*$/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $translator->translate('error_value_must_be_integer')))),
                )
            ),
            'book_year' => array('required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'regex', 'options' => array('pattern' => '/^[0-9]*$/D', 'messages' => array(\Zend\Validator\Regex::NOT_MATCH => $translator->translate('error_value_must_be_integer')))),
                )
            ),
            'book_month' => array(
                'required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'in_array', 'options' => array('haystack' => $monthSelectValidation, 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $translator->translate('error_select_value'))))
                )
            ),
            'status' => array(
                'required' => false,
                'filters' => array(
                    array('name' => '\Zend\Filter\StripTags'),
                ),
                'validators' => array(
                    array('name' => 'in_array', 'options' => array('haystack' => array(0, 1)), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $translator->translate('checkbox_error')))
                )
            ),
            'cover' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'System\Validator\FileAvatar', 'options' => array('allowEmpty' => true, 'allowOverride' => false,
                            'messages' => array(
                                \System\Validator\File::FILE_ERROR => $translator->translate('file_error'),
                            )))
                )
            ),
        );

        parent::init($fields);

        if ($data && $this->getFormMode() == parent::EDIT) {
            $this->setData($data);
        }
    }

    public function draw() {
        $queue = array(
            'subject',
            'chief_nr',
            'book_month',
            'edition_book_nr',
            'book_year',
            'edition_year',
            'edition_nr',
            'book_nr',
            'cover',
            'status',
            'submit-button'
        );

        if ($this->formMode == parent::VIEW) {
            array_pop($queue);
        }

        $this->form->queue = $queue;
        return $this->form;
    }

    public function getData() {
        $data = $this->form->getData();
        $hydrator = new ClassMethodsHydrator();
        $entity = $hydrator->hydrate($data, new Entity());

        if ($entity->getSubject() == '') {
            $entity->setSubject(null);
        }

        if ($entity->getChiefNr() == '') {
            $entity->setChiefNr(null);
        }

        if ($entity->getBookMonth() == '' || $entity->getBookMonth() == '0') {
            $entity->setBookMonth(null);
        }

        if ($entity->getEditionBookNr() == '') {
            $entity->setEditionBookNr(null);
        }

        if ($entity->getBookYear() == 0) {
            $entity->setBookYear(null);
        }

        if ($entity->getEditionYear() == '') {
            $entity->setEditionYear(null);
        }

        if ($entity->getEditionNr() == '') {
            $entity->setEditionNr(null);
        }

        if ($entity->getBookNr() == '') {
            $entity->setBookNr(null);
        }

        if ($entity->getSpecCode() == '') {
            $entity->setSpecCode(null);
        }

        if ($entity->getStatus() == 1) {
            $entity->setStatus('VIS');
        } else {
            $entity->setStatus('HID');
        }

        return $entity;
    }

    public function adaptData($data) {
        if ($data instanceof Entity) {
            $hydrator = new ClassMethodsHydrator();
            $entity = $data;

            $data = $hydrator->extract($data);
            if ($entity->getStatus() == 'VIS') {
                $data['status'] = 1;
            } else {
                $data['status'] = 0;
            }
        }
        return $data;
    }

    public function setData($data) {
        $data = $this->adaptData($data);
        parent::setData($data);
    }

    public function addachDefaultFilters() {
        if (null !== $this->filterChain) return;

        $this->filtersForAll[] = new \Zend\Filter\StringTrim;
        $this->filtersForAll[] = new \System\Filter\PregReplaceEx(array('match' => "/\'/", 'replace' => "\""));
    }

}
?>