<?php
namespace System\Form;

use System\Validator\RegexBase;
use System\Translator;
use Zend\Validator\NotEmpty;
use Zend\Validator\Regex;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Model\Entity\MainSite as Entity;
use Model\TableGateway\MainSite as MainSiteTableGateway;

class MainSite extends AbstractForm implements DrawableInterface {

    protected $name = 'main_site';

    public function __construct($formMode, $data = null) {
        $translator = Translator\TranslatorEx::get();
        $tableGateway = new MainSiteTableGateway();

        if (defined('parent::' . $formMode)) {
            $this->setFormMode($formMode);
        } else {
            $this->setFormMode(parent::ADD);
        }

        if ($data && $this->getFormMode() == parent::EDIT) {
            $data = $this->adaptData($data);
        }

        $fields = array(
            array('name' => 'name', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => ''), 'options' => array('label' => $translator->translate('name', 'mainSite'))),
            array('name' => 'url', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => '', 'val' => ''), 'options' => array('label' => $translator->translate('url', 'mainSite'))),
            array('name' => 'active', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('val' => 0), 'options' => array('label' => $translator->translate('active'))),
            array('name' => 'public', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('val' => 0), 'options' => array('label' => $translator->translate('public'))),
            array('name' => 'layout', 'type' => 'System\Form\Element\Code', 'attributes' => array('class' => 'form-textarea-big', 'id' => 'ace_editor_layout'), 'options' => array('label' => $translator->translate('layout_view', 'mainSite'))),
            //array('name' => 'content', 'type' => 'System\Form\Element\ContentViews', 'attributes' => array('class' => 'form-textarea-big'), 'options' => array('selectOptions' => $selectOptions, 'label' => $translator->translate('content', 'mainSite'))),
            array('name' => 'submit-button', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('class' => 'form-submit', 'value' => $translator->translate('save'))),
            array('name' => 'nextStep', 'type' => 'Zend\Form\Element\Button', 'attributes' => array('title' => $translator->translate('main_next'), 'class' => 'info-tooltip next_button submitStep')),
            array('name' => 'prevStep', 'type' => 'Zend\Form\Element\Button', 'attributes' => array('title' => $translator->translate('main_prev'), 'class' => 'info-tooltip prev_button prevStep')),
            array('name' => 'save', 'type' => 'Zend\Form\Element\Button', 'attributes' => array('title' => $translator->translate('main_save'), 'class' => 'info-tooltip save_button submitStep')),
                //array('name' => 'layout', 'type' => 'Sytsem\Form\Element\Code', 'attributes' => array('class' => 'form-textarea-big', 'id' => 'ace_editor_layout'), 'options' => array('label' => $translator->translate('layout_view', 'mainSite'))),
                //array('name' => 'subpage', 'type' => 'System\Form\Element\Subpage', 'attributes' => array('class' => 'form-textarea-big'), 'options' => array('label' => $translator->translate('subpages', 'mainSite'))),
        );

        if ($this->formMode == parent::VIEW) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox') {
                    $field['attributes']['disabled'] = 'disabled';
                } elseif ($field['type'] == 'System\Form\Element\Code') {
                    $field['options']['readonly'] = true;
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }

        $this->validation = array(
            'name' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'regex', 'break_chain_on_failure' => true, 'options' => array('pattern' => RegexBase::get('nameVisible'), 'messages' => array(Regex::NOT_MATCH => $translator->translate('error_name_identifier_invalid')))),
                    array('name' => 'string_length', 'break_chain_on_failure' => true, 'options' => array('max' => 100, 'messages' => array(\Zend\Validator\StringLength::TOO_LONG => $translator->translate('error_value_more_than_100')))),
                    array('name' => 'Zend\Validator\Db\NoRecordExists', 'options' => array('adapter' => $tableGateway->getAdapter(), 'schema' => MainSiteTableGateway::getSchema(), 'table' => MainSiteTableGateway::getTableName(), 'field' => 'name', 'exclude' => (isset($data['name']) && $data['name']) ? array('field' => 'name', 'value' => $data['name']) : null, 'messages' => array(\Zend\Validator\Db\NoRecordExists::ERROR_RECORD_FOUND => $translator->translate('record_already_exists')))),
                ),
                'filters' => array(
                    array(
                        'name' => '\Zend\Filter\StripTags',
                        'name' => '\Zend\Filter\StringTrim',
                        'name' => '\System\Filter\PregReplaceEx', 'options' => array('match' => "/\'/", 'replace' => "\""),
                    ),
                ),
            ),
            'url' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'regex', 'break_chain_on_failure' => true, 'options' => array('pattern' => '/' . RegexBase::get('url') . '/D', 'messages' => array(Regex::NOT_MATCH => $translator->translate('error_url_incorrect')))),
                    array('name' => 'string_length', 'break_chain_on_failure' => true, 'options' => array('max' => 100, 'messages' => array(\Zend\Validator\StringLength::TOO_LONG => $translator->translate('error_value_more_than_100')))),
                    array('name' => 'Zend\Validator\Db\NoRecordExists', 'options' => array('adapter' => $tableGateway->getAdapter(), 'schema' => MainSiteTableGateway::getSchema(), 'table' => MainSiteTableGateway::getTableName(), 'field' => 'url', 'exclude' => (isset($data['url']) && $data['url']) ? array('field' => 'url', 'value' => $data['url']) : null, 'messages' => array(\Zend\Validator\Db\NoRecordExists::ERROR_RECORD_FOUND => $translator->translate('record_already_exists')))),
                ),
                'filters' => array(
                    array(
                        'name' => '\Zend\Filter\StripTags',
                        'name' => '\Zend\Filter\StringTrim',
                        'name' => '\System\Filter\PregReplaceEx', 'options' => array('match' => "/\'/", 'replace' => "\""),
                    ),
                ),
            ),
            'layout' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                ),
                /*'filters' => array(
                    array(
                        'name' => '\Zend\Filter\HtmlEntities', 'options' => array('doublequote' => false, ENT_QUOTES),
                    ),
                ),*/
            ),
            'content' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                ),
            ),
            'subpages' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                ),
            ),
            'active' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'in_array', 'options' => array('haystack' => array(0, 1)), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $translator->translate('checkbox_error')))
                )
            ),
            'public' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'in_array', 'options' => array('haystack' => array(0, 1)), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $translator->translate('checkbox_error')))
                )
            ),
        );

        parent::init($fields);

        $this->_messages = array(
            'name' => array(
                $translator->translate('error_value_is_required'),
            //$translator->translate('error_name_identifier_invalid'),
            ),
            'url' => array(
                $translator->translate('error_value_is_required'),
            //$translator->translate('error_url_incorrect'),
            ),
            'layout' => array(
                $translator->translate('error_value_is_required'),
            ),
            'content' => array(
                $translator->translate('error_value_is_required'),
            ),
            'subpages' => array(
                $translator->translate('error_value_is_required'),
            ),
            'active' => array(
                $translator->translate('checkbox_error'),
            ),
            'public' => array(
                $translator->translate('checkbox_error'),
            )
        );

        if ($data && $this->getFormMode() == parent::EDIT) {
            $this->setData($data);
        }
    }

    public function draw() {
        /* $queue = array(
          'name',
          'url',
          'active',
          'layout',
          'submit-button'
          ); */
        $translator = Translator\TranslatorEx::get();

        if ($this->formMode == parent::VIEW) {
            $save = array(
                'prevStep',
            );
        } else {
            $save = array(
                'prevStep',
                'save',
            );
        }

        $queue = array(
            1 => array(
                'name' => $translator->translate('basic_data', 'main_site'),
                'lock' => 0,
                'elements' => array(
                    'name',
                    'url',
                    'active',
                    'public',
                ),
                'navigation' => array(
                    'nextStep'
                )
            ),
            2 => array(
                'name' => $translator->translate('layout', 'main_site'),
                'lock' => ($this->formMode == parent::VIEW) ? 0 : 1,
                'elements' => array(
                    'layout',
                ),
                'navigation' => $save,
            ),
        );

        /* $queue = array(
          1 => array(
          'name' => $translator->translate('basic_data', 'main_site'),
          'lock' => 0,
          'elements' => array(
          'name',
          'url',
          'active',
          ),
          'navigation' => array(
          'nextStep'
          )
          ),
          2 => array(
          'name' => $translator->translate('layout', 'main_site'),
          'lock' => ($this->formMode == parent::VIEW) ? 0 : 1,
          'elements' => array(
          'layout',
          ),
          'navigation' => array(
          'prevStep',
          'nextStep'
          )
          ),
          3 => array(
          'name' => $translator->translate('content_types', 'main_site'),
          'lock' => ($this->formMode == parent::VIEW) ? 0 : 1,
          'elements' => array(
          'content',
          ),
          'navigation' => array(
          'prevStep',
          'nextStep'
          )
          ),
          4 => array(
          'name' => $translator->translate('subpages', 'main_site'),
          'lock' => ($this->formMode == parent::VIEW) ? 0 : 1,
          'elements' => array(
          'subpages',
          ),
          'navigation' => $save,
          ),
          ); */

        //foreach ($this->contentTypes as $contentFieldsName) {
        //    $queue[3]['elements'][] = $contentFieldsName;
        //}

        /* if ($this->formMode == parent::VIEW) {
          array_pop($queue);
          } */

        $this->form->queue = $queue;
        return $this->form;
    }

    public function getData() {
        $data = $this->form->getData();
        $hydrator = new ClassMethodsHydrator();
        return $hydrator->hydrate($data, new Entity());
    }

    public function adaptData($data) {
        if ($data instanceof Entity) {
            $hydrator = new ClassMethodsHydrator();
            $data = $hydrator->extract($data);
        }
        return $data;
    }

    public function setData($data) {
        $data = $this->adaptData($data);
        parent::setData($data);
    }

    public function addachDefaultFilters() {
        if (null !== $this->filterChain) return;

        //$this->filtersForAll[] = new \Zend\Filter\StringTrim;
        //$this->filtersForAll[] = new \System\Filter\PregReplaceEx(array('match' => "/\'/", 'replace' => "\""));
    }

}

?>