<?php
namespace System\Form;

use System\Validator\RegexBase;
use System\Translator;
use Zend\Validator\NotEmpty;
use Zend\Validator\Regex;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Model\Entity\MainSiteContentPage as Entity;
use Model\TableGateway\MainSite as MainSiteTableGateway;
use Model\TableGateway\DictMainSiteContentType as DictMainSiteContentTypeTableGateway;
use Model\TableGateway\DictMainSiteContentTypeView as DictMainSiteContentTypeViewTableGateway;

class MainSiteContentPage extends AbstractForm implements DrawableInterface {

    protected $name = 'main_site_content_page';

    public function __construct($formMode, $data = null) {
        $translator = Translator\TranslatorEx::get();
        $mainSiteTableGateway = new MainSiteTableGateway();
        $dictMainSiteContentTypeTableGateway = new DictMainSiteContentTypeTableGateway();
        $dictMainSiteContentViewTypeTableGateway = new DictMainSiteContentTypeViewTableGateway();

        if (defined('parent::' . $formMode)) {
            $this->setFormMode($formMode);
        } else {
            $this->setFormMode(parent::ADD);
        }

        if ($data && ($this->getFormMode() == parent::EDIT || $this->getFormMode() == parent::ADD)) {
            $data = $this->adaptData($data);
        }
        
        /*$dictMainSiteContentTypeSelectOptionsData = $dictMainSiteContentTypeTableGateway->getAll();//getUrelatedWithMainSiteID($data['main_site_id']);
        if ($this->getFormMode() == parent::EDIT) {
            $dictMainSiteContentTypeSelectOptions = array($data['dict_main_site_content_type']->getId() => $translator->translate($data['dict_main_site_content_type']->getName(), 'main_site_view_type'));
        } else {
            $dictMainSiteContentTypeSelectOptions = array('' => '');
        }
        $controlArray = array();
        foreach ($dictMainSiteContentTypeSelectOptionsData as $dictMainSiteContentTypeSelectOption) {
            if (!in_array($dictMainSiteContentTypeSelectOption->getId(), $controlArray)) {
                $dictMainSiteContentTypeSelectOptions[] = array(
                    'value' => (int)$dictMainSiteContentTypeSelectOption->getId(),
                    'label' => (string)$translator->translate($dictMainSiteContentTypeSelectOption->getName(), 'main_site_view_type'),
                );
                $controlArray[] = (int)$dictMainSiteContentTypeSelectOption->getId();
            }
        }*/
        
        $dictMainSiteContentViewTypeSelectOptionsData = $dictMainSiteContentViewTypeTableGateway->getUrelatedWithMainSiteID($data['main_site_id']);
        if ($this->getFormMode() == parent::EDIT) {
            $dictMainSiteContentViewTypeSelectOptions = array($data['dict_main_site_content_type_view']->getId() => $translator->translate($data['dict_main_site_content_type_view']->getDictMainSiteContentType()->getName(), 'main_site_view_type') . ' - ' . $data['dict_main_site_content_type_view']->getName());
        } else {
            $dictMainSiteContentViewTypeSelectOptions = array('' => '');
        }
        $controlArray = array();
        foreach ($dictMainSiteContentViewTypeSelectOptionsData as $dictMainSiteContentViewTypeSelectOption) {
            if (!in_array($dictMainSiteContentViewTypeSelectOption->getId(), $controlArray)) {
                $dictMainSiteContentViewTypeSelectOptions[] = array(
                    'value' => (int)$dictMainSiteContentViewTypeSelectOption->getId(),
                    'label' => (string)$translator->translate($dictMainSiteContentViewTypeSelectOption->getDictMainSiteContentType()->getName(), 'main_site_view_type') . ' - ' . $dictMainSiteContentViewTypeSelectOption->getName(),
                );
                $controlArray[] = (int)$dictMainSiteContentViewTypeSelectOption->getId();
            }
        }
        
        $fields = array(
            array('name' => 'main_site_id', 'type' => 'Zend\Form\Element\Hidden', 'attributes' => array('class' => 'no_height'), 'options' => array()),
            //array('name' => 'dict_main_site_content_type_id', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('class' => 'select2_general', 'req' => ''), 'options' => array('value_options' => $dictMainSiteContentTypeSelectOptions, 'label' => $translator->translate('content_type_form_label', 'mainSiteContentPage'))),
            array('name' => 'dict_main_site_content_type_view_id', 'type' => 'Zend\Form\Element\Select', 'attributes' => array('class' => 'select2_general', 'req' => ''), 'options' => array('value_options' => $dictMainSiteContentViewTypeSelectOptions, 'label' => $translator->translate('content_type_view_form_label', 'mainSiteContentPage'))),
            array('name' => 'view', 'type' => 'System\Form\Element\Code', 'attributes' => array('class' => 'form-textarea-big', 'id' => 'ace_editor_layout'), 'options' => array('label' => $translator->translate('view', 'mainSiteContentPage'))),
            array('name' => 'nextStep', 'type' => 'Zend\Form\Element\Button', 'attributes' => array('title' => $translator->translate('main_next'), 'class' => 'info-tooltip next_button submitStep')),
            array('name' => 'prevStep', 'type' => 'Zend\Form\Element\Button', 'attributes' => array('title' => $translator->translate('main_prev'), 'class' => 'info-tooltip prev_button prevStep')),
            array('name' => 'save', 'type' => 'Zend\Form\Element\Button', 'attributes' => array('title' => $translator->translate('main_save'), 'class' => 'info-tooltip save_button submitStep')),
        );

        if ($this->formMode == parent::VIEW) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox' || $field['type'] == 'Zend\Form\Element\Select') {
                    $field['attributes']['disabled'] = 'disabled';
                } elseif ($field['type'] == 'System\Form\Element\Code') {
                    $field['options']['readonly'] = true;
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }

        $this->validation = array(
            'main_site_id' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'Zend\Validator\Db\RecordExists', 'options' => array('adapter' => $mainSiteTableGateway->getAdapter(), 'schema' => MainSiteTableGateway::getSchema(), 'table' => MainSiteTableGateway::getTableName(), 'field' => 'id', 'messages' => array(\Zend\Validator\Db\RecordExists::ERROR_NO_RECORD_FOUND => $translator->translate('record_dont_exists')))),
                ),
                'filters' => array(
                    array(
                        'name' => '\Zend\Filter\StripTags',
                        'name' => '\Zend\Filter\StringTrim',
                        'name' => '\System\Filter\PregReplaceEx', 'options' => array('match' => "/\'/", 'replace' => "\""),
                    ),
                ),
            ),
            'dict_main_site_content_type_view_id' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'Zend\Validator\Db\RecordExists', 'options' => array('adapter' => $dictMainSiteContentViewTypeTableGateway->getAdapter(), 'schema' => DictMainSiteContentTypeViewTableGateway::getSchema(), 'table' => DictMainSiteContentTypeViewTableGateway::getTableName(), 'field' => 'id', 'messages' => array(\Zend\Validator\Db\RecordExists::ERROR_NO_RECORD_FOUND => $translator->translate('record_dont_exists')))),
                ),
                'filters' => array(
                    array(
                        'name' => '\Zend\Filter\StripTags',
                        'name' => '\Zend\Filter\StringTrim',
                        'name' => '\System\Filter\PregReplaceEx', 'options' => array('match' => "/\'/", 'replace' => "\""),
                    ),
                ),
            ),
            'view' => array('required' => false),
        );

        parent::init($fields);

        $this->_messages = array(
            'dict_main_site_content_type_id' => array(
                $translator->translate('error_value_is_required'),
                $translator->translate('record_dont_exists'),
            ),
        );

        if ($data && ($this->getFormMode() == parent::EDIT || $this->getFormMode() == parent::ADD)) {
            $this->setData($data);
        }
    }

    public function draw() {
        $translator = Translator\TranslatorEx::get();

        if ($this->formMode == parent::VIEW) {
            $save = array(
                'prevStep',
            );
        } else {
            $save = array(
                'prevStep',
                'save',
            );
        }

        $queue = array(
            1 => array(
                'name' => $translator->translate('content_type', 'mainSiteContentPage'),
                'lock' => 0,
                'elements' => array(
                    'main_site_id',
                    //'dict_main_site_content_type_id',
                    'dict_main_site_content_type_view_id',
                ),
                'navigation' => array(
                    'nextStep'
                )
            ),
            2 => array(
                'name' => $translator->translate('view_code', 'mainSiteContentPage'),
                'lock' => ($this->formMode == parent::VIEW) ? 0 : 1,
                'elements' => array(
                    'view',
                ),
                'navigation' => $save,
            ),
        );

        $this->form->queue = $queue;
        return $this->form;
    }

    public function getData() {
        $data = $this->form->getData();
        $hydrator = new ClassMethodsHydrator();
        return $hydrator->hydrate($data, new Entity());
    }

    public function adaptData($data) {
        if ($data instanceof Entity) {
            $hydrator = new ClassMethodsHydrator();
            $data = $hydrator->extract($data);
        }
        return $data;
    }

    public function setData($data) {
        $data = $this->adaptData($data);
        parent::setData($data);
    }

    public function addachDefaultFilters() {
        if (null !== $this->filterChain) return;
    }

}

?>