<?php
namespace System\Form;

use System\Validator\RegexBase;
use System\Translator;
use Zend\Validator\NotEmpty;
use Zend\Validator\Regex;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Model\Entity\MainSiteCustomPage as Entity;
use Model\TableGateway\MainSite as MainSiteTableGateway;
use Model\TableGateway\MainSiteContentPage as MainSiteCustomPageTableGateway;

class MainSiteCustomPage extends AbstractForm implements DrawableInterface {

    protected $name = 'main_site_custom_page';

    public function __construct($formMode, $data = null) {
        $translator = Translator\TranslatorEx::get();
        $mainSiteTableGateway = new MainSiteTableGateway();
        $mainSiteCustomPageTableGateway = new MainSiteCustomPageTableGateway();

        if (defined('parent::' . $formMode)) {
            $this->setFormMode($formMode);
        } else {
            $this->setFormMode(parent::ADD);
        }

        if ($data && ($this->getFormMode() == parent::EDIT || $this->getFormMode() == parent::ADD)) {
            $data = $this->adaptData($data);
        }

        $fields = array(
            array('name' => 'main_site_id', 'type' => 'Zend\Form\Element\Hidden', 'attributes' => array('class' => 'no_height'), 'options' => array()),
            array('name' => 'url', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => ''), 'options' => array('label' => $translator->translate('url', 'mainSiteCustomPage'))),
            array('name' => 'login_required', 'type' => 'Zend\Form\Element\Checkbox', 'attributes' => array('val' => 0), 'options' => array('label' => $translator->translate('login_required', 'mainSiteCustomPage'))),
            array('name' => 'view', 'type' => 'System\Form\Element\Code', 'attributes' => array('class' => 'form-textarea-big', 'id' => 'ace_editor_layout'), 'options' => array('label' => $translator->translate('view', 'mainSiteCustomPage'))),
            array('name' => 'nextStep', 'type' => 'Zend\Form\Element\Button', 'attributes' => array('title' => $translator->translate('main_next'), 'class' => 'info-tooltip next_button submitStep')),
            array('name' => 'prevStep', 'type' => 'Zend\Form\Element\Button', 'attributes' => array('title' => $translator->translate('main_prev'), 'class' => 'info-tooltip prev_button prevStep')),
            array('name' => 'save', 'type' => 'Zend\Form\Element\Button', 'attributes' => array('title' => $translator->translate('main_save'), 'class' => 'info-tooltip save_button submitStep')),
        );

        if ($this->formMode == parent::VIEW) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox' || $field['type'] == 'Zend\Form\Element\Select') {
                    $field['attributes']['disabled'] = 'disabled';
                } elseif ($field['type'] == 'System\Form\Element\Code') {
                    $field['options']['readonly'] = true;
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }

        $this->validation = array(
            'main_site_id' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'Zend\Validator\Db\RecordExists', 'options' => array('adapter' => $mainSiteTableGateway->getAdapter(), 'schema' => MainSiteTableGateway::getSchema(), 'table' => MainSiteTableGateway::getTableName(), 'field' => 'id', 'messages' => array(\Zend\Validator\Db\RecordExists::ERROR_NO_RECORD_FOUND => $translator->translate('record_dont_exists')))),
                ),
                'filters' => array(
                    array(
                        'name' => '\Zend\Filter\StripTags',
                        'name' => '\Zend\Filter\StringTrim',
                        'name' => '\System\Filter\PregReplaceEx', 'options' => array('match' => "/\'/", 'replace' => "\""),
                    ),
                ),
            ),
            'url' => array(
                'required' => true,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'regex', 'break_chain_on_failure' => true, 'options' => array('pattern' => '/' . RegexBase::get('nameIdentifier') . '/D', 'messages' => array(Regex::NOT_MATCH => $translator->translate('error_name_identifier_invalid')))),
                    //array('name' => 'string_length', 'break_chain_on_failure' => true, 'options' => array('max' => 100, 'messages' => array(\Zend\Validator\StringLength::TOO_LONG => $translator->translate('error_value_more_than_100')))),
                    //array('name' => 'Zend\Validator\Db\NoRecordExists', 'options' => array('adapter' => $mainSiteCustomPageTableGateway->getAdapter(), 'schema' => MainSiteCustomPageTableGateway::getSchema(), 'table' => MainSiteCustomPageTableGateway::getTableName(), 'field' => 'url', 'exclude' => (isset($data['url']) && $data['url']) ? array('field' => 'url', 'value' => $data['url']) : null, 'messages' => array(\Zend\Validator\Db\NoRecordExists::ERROR_RECORD_FOUND => $translator->translate('record_already_exists')))),
                ),
                'filters' => array(
                    array(
                        'name' => '\Zend\Filter\StripTags',
                        'name' => '\Zend\Filter\StringTrim',
                        'name' => '\System\Filter\PregReplaceEx', 'options' => array('match' => "/\'/", 'replace' => "\""),
                    ),
                ),
            ),
            'view' => array('required' => false),
            'login_required' => array(
                'required' => false,
                'validators' => array(
                    array('name' => 'in_array', 'options' => array('haystack' => array(0, 1)), 'messages' => array(\Zend\Validator\InArray::NOT_IN_ARRAY => $translator->translate('checkbox_error')))
                )
            ),
        );

        parent::init($fields);

        $this->_messages = array(
            'url' => array(
                $translator->translate('error_value_is_required'),
                $translator->translate('error_name_identifier_invalid'),
                $translator->translate('error_value_more_than_100'),
                $translator->translate('record_already_exists'),
            ),
            'login_required' => array(
                $translator->translate('checkbox_error'),
            ),
        );

        if ($data && ($this->getFormMode() == parent::EDIT || $this->getFormMode() == parent::ADD)) {
            $this->setData($data);
        }
    }

    public function draw() {
        $translator = Translator\TranslatorEx::get();

        if ($this->formMode == parent::VIEW) {
            $save = array(
                'prevStep',
            );
        } else {
            $save = array(
                'prevStep',
                'save',
            );
        }

        $queue = array(
            1 => array(
                'name' => $translator->translate('page_url', 'mainSiteCustomPage'),
                'lock' => 0,
                'elements' => array(
                    'main_site_id',
                    'url',
                    'login_required',
                ),
                'navigation' => array(
                    'nextStep'
                )
            ),
            2 => array(
                'name' => $translator->translate('view_code', 'mainSiteCustomPage'),
                'lock' => ($this->formMode == parent::VIEW) ? 0 : 1,
                'elements' => array(
                    'view',
                ),
                'navigation' => $save,
            ),
        );

        $this->form->queue = $queue;
        return $this->form;
    }

    public function getData() {
        $data = $this->form->getData();
        $hydrator = new ClassMethodsHydrator();
        return $hydrator->hydrate($data, new Entity());
    }

    public function adaptData($data) {
        if ($data instanceof Entity) {
            $hydrator = new ClassMethodsHydrator();
            $data = $hydrator->extract($data);
        }
        return $data;
    }

    public function setData($data) {
        $data = $this->adaptData($data);
        parent::setData($data);
    }

    public function addachDefaultFilters() {
        if (null !== $this->filterChain) return;
    }

}

?>