<?php
namespace System\Form;

use System\Translator;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;
use Model\Entity\MainSiteInsomniaQuestion as Entity;

class MainSiteInsomniaQuestion extends AbstractForm implements DrawableInterface
{
    protected $name = 'main_site_insomnia_question';

    public function __construct($formMode, $data = null)
    {
        if (defined('parent::' . $formMode)) {
            $this->setFormMode($formMode);
        } else {
            $this->setFormMode(parent::ADD);
        }

        if ($data && ($this->getFormMode() == parent::EDIT || $this->getFormMode() == parent::ADD)) {
            $data = $this->adaptData($data);
        }

        $fields = [
            ['name' => 'email', 'type' => 'Zend\Form\Element\Text', 'attributes' => ['class' => 'inp-form', 'req' => ''], 'options' => ['label' => 'Email']],
            ['name' => 'title', 'type' => 'Zend\Form\Element\Text', 'attributes' => ['class' => 'inp-form', 'req' => ''], 'options' => ['label' => 'Tytuł']],
            ['name' => 'question', 'type' => 'Zend\Form\Element\Textarea', 'attributes' => ['class' => 'form-textarea', 'req' => ''], 'options' => ['label' => 'Pytanie']],
            ['name' => 'submit_date', 'type' => 'Zend\Form\Element\Text', 'attributes' => ['class' => 'inp-form', 'req' => ''], 'options' => ['label' => 'Data']]
        ];

        if ($this->formMode == parent::VIEW) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox' || $field['type'] == 'Zend\Form\Element\Select') {
                    $field['attributes']['disabled'] = 'disabled';
                } elseif ($field['type'] == 'System\Form\Element\Code') {
                    $field['options']['readonly'] = true;
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }

        $this->validation = [];
        $this->_messages = [];

        parent::init($fields);

        if ($data && ($this->getFormMode() == parent::EDIT || $this->getFormMode() == parent::ADD)) {
            $this->setData($data);
        }
    }

    public function draw()
    {
        $queue = [
            'email',
            'title',
            'question',
            'submit_date'
        ];
        $this->form->queue = $queue;

        return $this->form;
    }

    public function getData()
    {
        $data = $this->form->getData();
        $hydrator = new ClassMethodsHydrator();

        return $hydrator->hydrate($data, new Entity());
    }

    public function adaptData($data)
    {
        if ($data instanceof Entity) {
            $hydrator = new ClassMethodsHydrator();
            $data = $hydrator->extract($data);
        }

        return $data;
    }

    public function setData($data)
    {
        $data = $this->adaptData($data);
        parent::setData($data);
    }

    public function addachDefaultFilters()
    {
        if (null !== $this->filterChain) {
            return;
        }
    }

}
