<?php
namespace System\Form;

use System\Translator;
use Zend\Stdlib\Hydrator\ClassMethods as ClassMethodsHydrator;

use Model\Entity\MainSiteUserAccess as Entity;
use Model\TableGateway\MainSite as MainSiteTableGateway;
use Model\TableGateway\MainUser as MainUserTableGateway;

class MainSiteUserAccess extends AbstractForm implements DrawableInterface
{

    protected $name = 'main_site_user_access';

    public function __construct($formMode, $data = null)
    {
        $translator = Translator\TranslatorEx::get();
        $mainSiteTableGateway = new MainSiteTableGateway;
        $mainUserTableGateway = new MainUserTableGateway;

        if (defined('parent::' . $formMode)) {
            $this->setFormMode($formMode);
        } else {
            $this->setFormMode(parent::ADD);
        }

        if ($data && ($this->getFormMode() == parent::EDIT || $this->getFormMode() == parent::ADD)) {
            $data = $this->adaptData($data);
        }

        $fields = array(
            array('name' => 'main_site_id', 'type' => 'Zend\Form\Element\Hidden', 'attributes' => array('class' => 'no_height'), 'options' => array()),
            array('name' => 'submit', 'type' => 'Zend\Form\Element\Submit', 'attributes' => array('value' => $translator->translate('save'), 'class' => 'submitStep form-submit'))
        );

        if ($this->getFormMode() == parent::ADD) {
            $fields[] = array('name' => 'main_user_id', 'type' => 'Zend\Form\Element\Hidden', 'attributes' => array('class' => 'no_height'), 'options' => array());
            $fields[] = array('name' => 'esculap_user', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('req' => '', 'val' => 'nameIdentifier', 'class' => 'autocomplete'), 'options' => array('label' => 'Użytkownik'));
        } elseif ($this->formMode == parent::VIEW) {
            $fields[] = array('name' => 'main_user_id', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => ''), 'options' => array('label' => 'ID użytkownika'));
            $fields[] = array('name' => 'esculap_user', 'type' => 'Zend\Form\Element\Text', 'attributes' => array('class' => 'inp-form', 'req' => ''), 'options' => array('label' => 'Użytkownik'));
        }

        if ($this->formMode == parent::VIEW) {
            foreach ($fields as &$field) {
                if ($field['type'] == 'Zend\Form\Element\Checkbox' || $field['type'] == 'Zend\Form\Element\Select') {
                    $field['attributes']['disabled'] = 'disabled';
                } elseif ($field['type'] == 'System\Form\Element\Code') {
                    $field['options']['readonly'] = true;
                } else {
                    $field['attributes']['readonly'] = 'readonly';
                }
            }
        }

        $this->validation = array(
            'main_site_id' => array(
                'required'   => true,
                'validators' => array(
                    array('name' => 'not_empty', 'break_chain_on_failure' => true, 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'Zend\Validator\Db\RecordExists', 'options' => array('adapter' => $mainSiteTableGateway->getAdapter(), 'schema' => MainSiteTableGateway::getSchema(), 'table' => MainSiteTableGateway::getTableName(), 'field' => 'id', 'messages' => array(\Zend\Validator\Db\RecordExists::ERROR_NO_RECORD_FOUND => $translator->translate('record_dont_exists')))),
                ),
                'filters'    => array(
                    array(
                        'name' => '\Zend\Filter\StripTags',
                        'name' => '\Zend\Filter\StringTrim',
                        'name' => '\System\Filter\PregReplaceEx', 'options' => ['match' => '/\'/', 'replace' => "\""],
                    ),
                ),
            ),
            'main_user_id' => array(
                'required'   => false,
                'validators' => array(
                    array('name' => 'not_empty', 'options' => array('messages' => array(\Zend\Validator\NotEmpty::IS_EMPTY => $translator->translate('error_value_is_required')))),
                    array('name' => 'Zend\Validator\Db\RecordExists', 'options' => array('adapter' => $mainUserTableGateway->getAdapter(), 'schema' => MainUserTableGateway::getSchema(), 'table' => MainUserTableGateway::getTableName(), 'field' => 'id', 'messages' => array(\Zend\Validator\Db\RecordExists::ERROR_NO_RECORD_FOUND => $translator->translate('record_dont_exists')))),
                ),
            )
        );

        parent::init($fields);

        $this->_messages = array(
            'dict_main_site_content_type_id' => array(
                $translator->translate('error_value_is_required'),
                $translator->translate('record_dont_exists'),
            ),
        );

        if ($data && ($this->getFormMode() == parent::EDIT || $this->getFormMode() == parent::ADD)) {
            $this->setData($data);
        }
    }

    public function draw()
    {
        if ($this->getFormMode() == parent::ADD) {
            $queue = array(
                'main_site_id',
                'main_user_id',
                'esculap_user',
                'submit'
            );
        } elseif ($this->getFormMode() == parent::VIEW) {
            $queue = array(
                'main_site_id',
                'main_user_id',
                'esculap_user'
            );
        }
        $this->form->queue = $queue;

        return $this->form;
    }

    public function getData()
    {
        $data = $this->form->getData();
        $hydrator = new ClassMethodsHydrator();

        return $hydrator->hydrate($data, new Entity());
    }

    public function adaptData($data)
    {
        if ($data instanceof Entity) {
            $hydrator = new ClassMethodsHydrator();
            $data = $hydrator->extract($data);
        }

        return $data;
    }

    public function setData($data)
    {
        $data = $this->adaptData($data);
        if ($this->getFormMode() == parent::VIEW) {
            $mainUserTableGateway = new MainUserTableGateway;
            $user = $mainUserTableGateway->getByID($data['main_user_id']);
            $data['esculap_user'] = $user->getLogin();
        }
        parent::setData($data);
    }

    public function addachDefaultFilters()
    {
        if (null !== $this->filterChain) {
            return;
        }
    }

}
