<?php
namespace System\Form;

class QueryParser {

    public function parse($where, $order, $limit = 1, $page = 25) {

        $result = array(
            'where' => null,
            'order' => null,
            'board' => null,
            'page' => (int)$page,
            'limit' => (int)$limit
        );


        $allowedOrder = array(
            'id' => 'content_id', 'title' => 'title', 'add_date' => 'add_date', 'creator' => 'creator', 'client' => 'client', 'main_site_name' => 'main_site.name'
        );

        $allowedWhere = array(
            'title_val' => 'title', 'id_val' => 'content_id', 'add_date' => 'add_date', 'group_name' => 'lang_id', 'type' => 'news_type', 'specs' => 'spec_cod', 'creator_val' => 'creator_val', 'creator' => 'creator_name',
            'add_date_from' => 'date_from', 'add_date_to' => 'date_to', 'status' => 'active', 'event_type' => 'event_type', 'publ_type' => 'publ_type', 'journal' => 'journal_id',
            'client' => 'dict_client_id', 'main_site_name' => 'main_site_name', 'add_by_user' => 'add_by_user'
        );

        if (is_array($where)) {
            foreach ($where as $key => $value) {

                $firstDot = strpos($value, '.', 0);

                if ($firstDot == -1) {
                    continue;
                }


                $value = array(
                    substr($value, 0, $firstDot),
                    substr($value, $firstDot + 1)
                );


                if (!isset($value[1])) continue;
                $value = $value[1];
                $oValue = $value;

                if (array_key_exists($key, $allowedWhere) && null !== $value) {

                    switch ($allowedWhere[$key]) {
                        case 'creator_val':

                            $value = (int)$value;

                            break;
                        case 'creator_data':

                            break;
                        case 'content_id':

                            if (0 == (int)$value) $value = null;

                            break;
                        case 'add_date':
                        case 'date_from':
                        case 'date_to':

                            if (preg_match('/^[0-9]{2}\/[0-9]{2}\/[0-9]{4}$/', $value)) {
                                $value = explode('/', $value);
                                $value = mktime(0, 0, 0, $value[1], $value[0], $value[2]);
                            } else {

                                $value = null;
                            }

                            if ((int)$value < 1) {

                                $value = null;
                            }


                            break;
                        case 'active':
                        case 'status':
                        case 'add_by_user':

                            $value = explode(',', $value);
                            $valid = true;
                            $val = '';
                            foreach ($value as $row) {

                                if (1 == $row || 2 == $row) {

                                    if (strlen($val)) {
                                        $val .= ',';
                                    }

                                    if (2 == $row) {
                                        $val .= "true";
                                    } if (1 == $row) {
                                        $val .= "false";
                                    }
                                }
                            }

                            if (!strlen($val)) {
                                $value = "true,false";
                            } else {
                                $value = $val;
                            }

                            break;
                        case 'lang_id':
                        case 'news_type':
                        case 'event_type':

                            $value = explode(',', $value);
                            $valid = false;
                            foreach ($value as $val) {
                                if (0 !== $val) {
                                    $valid = 1;
                                }
                            }
                            if (!$valid) {

                                $value = null;
                            } else {

                                $value = implode(',', $value);
                            }
                            break;
                        case 'spec_code':

                            $validator = new \System\Validator\Spec;

                            if (!is_array($value)) {
                                $value = explode(',', $value);
                            }

                            if (!$validator->isValid($value)) {

                                $value = null;
                            } else {


                                $this->specs = $value;
                                $value = implode(',', $value);
                            }

                            break;
                    }

                    if (null !== $value) {

                        $result['board'][$key] = $oValue;
                        $result['where'][] = array(
                            $allowedWhere[$key] => $value
                        );
                    }
                }
            }
        }
        
        if ($order) {

            $value = explode(',', $order);

            if (2 !== count($value)) return;

            $field = $value[0];
            $type = $value[1];

            if (array_key_exists($field, $allowedOrder)) {

                $result['order'] = array($allowedOrder[$field] => $type);
            }
        }

        return $result;
    }

    public static function parseQuery($where, $order, $limit, $page) {
        $result = array('where' => array(), 'order' => null, 'limit', 'page');

        if (is_array($where)) {
            foreach ($where as $field => $value) {
                if (($separator = strpos($value, '.', 0)) == -1) {
                    continue;
                }
                $value = substr($value, $separator + 1);

                if ($value) {
                    $result['where'][] = array($field => $value);
                }
            }
        }

        $whereView = array();
        foreach ($result['where'] as $whereCondition) {
            $whereView = array_merge($whereView, $whereCondition);
        }
        $result['whereView'] = $whereView;

        if (!is_array($order) && (count(explode(',', $order)) == 2)) {
            $orderExploded = explode(',', $order);
            $order = array($orderExploded[0] => $orderExploded[1]);
        }
        if (count($order) === 1) {
            reset($order);
            $orderField = strtolower(trim(key($order)));
            $orderDirection = strtolower(trim($order[$orderField]));
            if ($orderDirection != 'desc' && $orderDirection != 'asc') {
                $result['order'] = null;
            } else {
                $result['order'] = array($orderField => $orderDirection);
            }
        }

        $limit = (int)$limit;
        if ($limit < 1) {
            $limit = 1;
        }
        $result['limit'] = $limit;

        $page = (int)$page;
        if ($page < 1) {
            $page = 1;
        }
        $result['page'] = $page;

        return $result;
    }

}

?>