<?php

namespace System\Misc\FilterType;

use System\Misc\FilterType\AbstractFilterType;

class Column extends AbstractFilterType {

    protected $field = null;
    protected $operation = null;

    const OP_EQ = 'eq';
    const OP_LT = 'lt';
    const OP_LTE = '<=';
    const OP_GT = 'gt';
    const OP_GTE = '>=';
    const OP_LIKE = 'like';
    const OP_IN = 'in';
    const OP_NOTIN = 'not in';

    public function __construct($field, $constraint, $operation) {
        $this->field = $field;
        $this->operation = $operation;
        parent::__construct($constraint);
    }

    public function getQuery() {
        $query = '';

        switch ($this->operation) {
            case self::OP_EQ :
                if (is_numeric($this->constraint) && $this->field != 'user_pl.pwz') {
                    $query = "{$this->field} = {$this->constraint}";
                } else {
                    $query = "{$this->field} = '{$this->constraint}'";
                }

                return $query;
            case self::OP_IN :
                $constraint = $this->constraint;

                if (is_array($constraint)) {
                    $constraint = implode(',', $constraint);
                }

                $constraint = explode(',', $constraint);

                $tmp = '';

                foreach ($constraint as $c) {
                    if (strlen($tmp)) {
                        $tmp .= ',';
                    }

                    $tmp .= "'$c'";
                }

                $condition = $tmp;

                $query = "{$this->field} in ({$condition})";

                return $query;
            case self::OP_LIKE :
                $query = "{$this->field} like '%{$this->constraint}%'";

                return $query;
            case self::OP_GT :
                if ($this->field == 'misc_form.time') {
                    $query = "{$this->field} > '" . date("Y-m-d 00:00:00", $this->constraint) . "'";
                } else {
                    $query = "{$this->field} > {$this->constraint}";
                }

                return $query;
            case self::OP_GTE :
                if ($this->field == 'misc_form.time') {
                    $query = "{$this->field} >= '" . date("Y-m-d 00:00:00", $this->constraint) . "'";
                } else {
                    $query = "{$this->field} >= {$this->constraint}";
                }

                return $query;
            case self::OP_LT :
                if ($this->field == 'misc_form.time') {
                    $query = "{$this->field} < '" . date("Y-m-d 23:59:59", $this->constraint) . "'";
                } else {
                    $query = "{$this->field} < {$this->constraint}";
                }

                return $query;
            case self::OP_LTE :
                if ($this->field == 'misc_form.time') {
                    $query = "{$this->field} <= '" . date("Y-m-d 23:59:59", $this->constraint) . "'";
                } else {
                    $query = "{$this->field} <= {$this->constraint}";
                }

                return $query;
            case self::OP_NOTIN :

                $constraint = $this->constraint;

                if (is_array($constraint)) {
                    $constraint = implode(',', $constraint);
                }

                $constraint = explode(',', $constraint);

                $tmp = '';

                foreach ($constraint as $c) {
                    if (strlen($tmp)) {
                        $tmp .= ',';
                    }
                    $tmp .= "'$c'";
                }

                $condition = $tmp;
                $query = "{$this->field} not in ({$condition})";
                return $query;
        }

        return null;
    }

}

?>