<?php

namespace System\Misc;

use Model\MainMiscForms;

class Presentation {
	
	public static $miscFilesDir = '/';
	
	protected $id,
			  $from,
			  $to,
			  $name,
			  $indexName,
			  $refiuseIndexName,
			  $dirName,
			  $multiFill,
			  $views,
			  $submits,
			  $formName;
	
	public function __construct( $data ) {
		
		if( !is_array( $data ) ) {
			
			throw new Presentation\Exception\PresentationInvalidInitializationDataException( "Initialization data must be an array" );
		}
		
		foreach( $data as $name => $value ) {
			
			if( property_exists( $this , $name ) )
				$this->$name = $value;
		}
	}
	
	
	protected function checkFilte( $name , $goToIndexOnFailure ) {
		
		if( null == $name && true == $goToIndexOnFailure ) {
			
			$name = $this->indexName;
		}
		
		$filePath = self::$miscFilesDir . $this->name . '/' . $name;
	 	
		if( !file_exists( $filePath ) )
			throw new Presentation\Exception\PresentationFileNotExists( "File {$name} in {$this->name} presentatin not exists" );
		
		return $filePath;
	}
	
	
	public function getFile( $name = null , $goToIndexOnFailure = true ) {
		$filePatch = $this->checkFilte( $name , $goToIndexOnFailure );
		
		ob_start();
		
		include $filePatch;
		
		return ob_get_clean();
	}	
	
	
	public function validate( $userId = null ) {
		
		// sprawdzanie czy prezentacja ma się wyświetlić
		if( time() > $this->to ) 
			throw new Presentation\Exception\PresentationExpiredException;
			
		if( time() < $this->from )
			throw new Presentation\Exception\PresentationNotStartedException;
			
			
		// sprawdzanie czy user wypełnił formularz i zgłaszanie błędu jeśli formularz można wypełnić tylko raz
		if( null !== $userId &&
			false == $this->multiFill ) {
			
			$MainMiscForms = new MainMiscForms;
			
			try {
			
				$isUnique = $MainMiscForms->isUserFillForm( $this->formName , $userId );
				
			} catch( \Exception $e ) {
				
				throw new Presentation\Exception\PresentationDatabaseExcetion( $e->getMessage() );
			}
			
			
			if( false === $isUnique )
				return true;
			throw new Presentation\Exception\PresentationCompletedByUserException;
		}
	}
	
	public function __get( $name ) {
		
		if( property_exists( $this , $name ) )
			return $this->$name;
		return null;
	}	
}

?>