<?php

namespace System\Observer;

use System\Observer,
	System\Log\LoggerAdapter;

/**
* Główny obserwator 
*/
class GlobalObserver implements Observer {

	protected static $logger;
	
	/**
	* Flaga mówiąca o tym czy aktywne jest debugowanie. Jeśli debugowanie jest aktywne zdarzenia 'content_view' nie będą obsługiwane
	*/
	protected static $contentDebug = false;
	
	
	const CONTENT_DEBUG_ON = "Wizyta nie została zalogowana, aktywne jest debugowanie kontentu";
	
	
	/**
	* Ustawia obiekt logowania w celu zapisywania ewentualnych błędów. Błędy zapisywane są ze stopniem ważności WARN. Konieczne jest podanie w liście argumentów kluczy namespace oraz line odpowiednie
	* __METHOD__ oraz __LINE__ aby obiekt logowania poprawnie zalogował wpis w przypadku kożyustania z standardowego obiektu logowania System\Log\LoggerAdapter
	* @param System\Log\LoggerAdapter $logger obiekt lohujący
	* @returns void
	*/
	public static function setLogger( LoggerAdapter $logger ) {
		
		self::$logger = $logger;
	}
	
	
	/**
	* Alias do statycznej metody notify zapewniający integralność z interfejsem \System\Observer
	* @param string $name nazwa identyfikująca wydarzenie
	* @param array $data lista argumentów
	* @returns mixed
	*/
	public function update( $name , $data ) {
		return self::notify( $name , $data );
	}
	
	/**
	* Powiadomienie o wystąpieniu zdarzenia
	* @param string $name nazwa identyfikująca wydarzenie
	* @param array $data lista argumentów
	* @returns mixed
	*/
	public static function notify( $name , $data ) {
		
		switch( $name ) {
			case 'content_debug' :
				
				if( isset( $data[ 'state' ] ) && true == (bool)$data[ 'state' ] ) {
					self::$contentDebug = true;
				} else {
					self::$contentDebug = false;
				}
				
				return true;
				
				break;
			case 'content_view': 
				
				if( self::$contentDebug ) return self::CONTENT_DEBUG_ON;
				
				
				try {
					
					$MainContentRate = new \Model\MainContentRate;
					if( !$MainContentRate->incrementContentCounter(
						$data[ 'contentId' ],
						$data[ 'langId' ],
						$data[ 'userId' ]
					)) {
						
						$this->log( "Nieudało się zalogować wizyty użytkownika na stronie z kontentem." , $data[ 'namespace' ] , $data[ 'line' ] );
					
					}
					
					return true;
					
				} catch( \Exception $e ) {
					
					self::log( 
						$e->getMessage(),
						$data[ 'namespace' ],
						$data[ 'line' ]
					);
					
					return false;
				}
				
				break;
			default :
				return false;
		}
	}
	
	/**
	* Punkt logowania błędów
	*/
	protected static function log( $content ) {
		
		if( null !== self::$logger ) {
			
			self::$logger->WARN( $content , __METHOD__ , __LINE__ );
		}
	}
}

?>