<?php

namespace System\Soap;

/**
* Adapter dla wbudowanego w PHP klienta SOAP. Zapewnia pałną funkcjonalność wymaganą do łączenia się z bazą danych serwisu Escula.pl
* @author Cegedim Group Poland Sp. z o.o
* @version 1.0
*/
class Client {
	
	private static $ELOGIN_URL = 'http://esculaplogin.com';
	
	/**
	* Adres servera SOAP
	* @var string
	*/
	
    protected $_wsdl = 'http://sauth.demo.esculap.com/soap';
	/**
	* Obiekt zawierający odpowiedź na ostatnie żądanie
	* @var Response
	*/
    protected $_response = null;
	
	/**
	* Zawiera informację o powodzeniu ostatniego żądania
	* @var boolean
	*/
    protected $_result = null; 
    
	/**
	* Kod użytkownika
	* @var string
	*/
	protected $_userCode;
	
	/**
	* Login użytkownika potrzebny do zapisania ciastka gwarantującego rozpoznanie danego użytkownika gdy nie jest zalogowany
	* @var $_userLogin
	*/
	protected $_userLogin;
	
	/**
	* Czy autentykacja jest możliwa domyślnie - tak
	* @var boolean
	*/
	protected $_authRequire = true;
	
	/**
	* Obiekt klienta soap
	* @var SoapClient
	*/
	protected $_client = null; 
	
	/**
	* Lista funkcji dostępnych na serverze soap
	* @var array
	*/
	protected $_functions = null;
	
	/**
	* Dane pracownika zapisane po pierwszym wywołaniu funkcji getUserData
	* @var array
	*/
	protected $_userData = null;
	
	/**
	* Obiekt odpowiedzi, wypełniany po konkretnym żądaniu
	* @var Response
	*/
	protected $_respons = array();
	
	/**
	* Czy użytkownik jest zalogowany
	* @var boolean
	*/
	protected $_isLogged = false;
	
	/**
	* Czy wizyta została zalogowana w bazie
	* @var boolean
	*/
	protected $_isVisitLogged = false;
	
	/**
	* Lista odpowiedzi otrzymanych od servera w bieżącej sesji
	* @var array
	*/
	protected $_responseMessages = array();
	
	
	/**
	* Preferowana wartość języka
	*/
	protected $_lang = null;
	
	
	/**
	 * Zawiera url na jaki przekierować 
	 */
	protected $requestUri = null;
	
	
	
	/**
	* Konstruktor
	* @param string $userCode kod pracownika - jest to parametr opcionalny
	*
	* @return void
	*/
    public function __construct($siteCode,$userCode = null,$lang = null,$wsdl = null , $eloginUrl = null ) {
		$this->_siteCode = $siteCode;
		$this->_userCode = $userCode;
		$this->_lang = $lang;
		
		if( null !== $wsdl ) {
			
			$this->_wsdl = $wsdl;
		} if( null !== $eloginUrl ) {
			
			self::$ELOGIN_URL = $eloginUrl;
		}
		
		if( isset( $_POST['user_auth_code'] ) ) {
			
			$this->requestUri = @$_SERVER[ 'REQUEST_URI' ];
			
			//print_r( $_SERVER );
			
			//exit();
			
			
			//exit( $_POST['user_auth_code'] );
			$this->_userCode = $_POST['user_auth_code'];
		}
		
		try {
			
			$this->_client = new \SoapClient(null,array('location' => $this->_wsdl ,'uri' => $this->_wsdl,'soap_version' => SOAP_1_2,/*'cache_wsdl' => WSDL_CACHE_NONE*/));
		
		} catch( \Exception $e ) {
			
			exit( $e->getMessage() );
		}
		
	}
	
	public function getReturnUri() {
	
		return $this->requestUri;
	}
	
	/**
	* Zwraca kod pracownika "userCode"
	*
	* @return string
	*/
	public function getUserCode() {
		return $this->_userCode;
	}
	
	/**
	* Sprawdza czy użytkownik jest zautowyzowany a następnie zgodnie z ustawieniami przenosi na stronę logowania lub nie.
	* Główną wartością zwracaną przez funkcję jest wartość bool-owska mówiąca o tym czy użytkownik jest zautoryzowany.
	*
	* @param string $backUrl link link do którego zostanie przekierowana strona po pomyślnym zalogowaniu 
	* @return boolean
	*/ 
	public function isLogged($backUrl = null) {
		
		if(!null === $this->_isLogged) return $this->_isLogged;
		
		try{ 
			$response = $this->_client->isLoggedEx(
				$this->_siteCode,
				$this->_userCode
			);
		} catch( \Exception $e ) {
			
			exit( $e->getMessage() );
		}
		
		
		$this->_response = $this->translateResponse($response);
		
		$this->_result = (bool)$this->_response['authorized'];
		$this->_isLogged = $this->_result;
		
		if($this->_response['message'] == 'Redirect' && true === $this->_authRequire) {
			
			
			self::_redirect($this->_siteCode,$backUrl,$this->lang);
		}
		
		// nadaje usercode
		$r = explode('::',$this->_response['content']);
		
		$this->_userCode = @$r[0];
		$this->_userLogin = @$r[1];
		
		// zapisanie ciastka
		if($this->_result) {
			$this->_client->getData($this->_userCode,$this->_siteCode);
			$this->_generateLongLiveCookie($this->_userLogin);
		}
		
		return $this->_result;
	}
	
	public static function _redirect($siteCode,$url = null,$lang = null) {
		
		if( null === $url && isset( $_SERVER['HTTP_HOST'] ) ) {
			
			list( $protocol , $ver ) = explode( '/' , $_SERVER['SERVER_PROTOCOL'] );
			
			$protocol = $protocol ? strtolower($protocol) : 'http';
			$uri = $protocol.'://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];
			
		} else {
			
			$uri = $url;
		}
		
		$uri = base64_encode( $uri );
		
		exit(header("Location: " . self::$ELOGIN_URL . "/user/login/sc/{$siteCode}/url/{$uri}?lang={$lang}"));
	}
	
	/**
	* Dodaje wpis świadczący o wizycie użytkownika.
	* W przypadku gdy użytkownik nie jest zalogowany a zalogował się kiedyś na tym kmputerze do bazy zalogowany zostanie jego login z flagą"nie zalogowany"
	*
	* @param string $userCode
	* @param forceLog jeśli false(domyślnie) to przy próbie dwuktotnego zalogowania w jednej odsłonie rzucony zostanie wyjątek jeśli true wyjątek nie zostanie rzucony a logowanie do historii wykona się
	* @return void
	*/
	public function logVisit($forceLog = false) {
		
		if($this->_isVisitLogged && !$forceLog) throw new \Exception('This visit is already logged!'); 
		
		$response = $this->_client->logVisit(
			1, // zawsze 1 - sytuacja i tak zostanie zweryfikowana po stronie servera!
			$this->_getLongLiveCookie(), // jeśli użytkowanik nie jest zalogowany a kiedyś się zalogował na tym komputerze zostanie dodany jego login
			$this->_userCode,
			$this->_siteCode,
			$_SERVER['HTTP_HOST'],
			isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : null,
			isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : null
		);
		
		$this->_response = $this->translateResponse($response);
		
		// jeśli logowanie wykonało się poprawnie ustawiam zmienną w klasie
		if((bool)$this->_response['message'] == 'Query ok') {
		
			$this->_isVisitLogged = true;
			return true;
		}
		
		return false;
	}
	
	/**
	* Wylogowanie użytkownika
	* 
	* @return boolean
	*/
	public function logout() {
		if(!is_null($this->_userCode)) $_userCode = $this->_userCode;
		if(is_null($_userCode) && !is_null($userCode)) $_userCode = $userCode;
		
		if(is_null($_userCode)) return false; //throw new \Exception('User code not set!');
		
		$response = $this->_client->logout(
			$this->_siteCode,
			$_userCode
		);
		
		$this->_response = $this->translateResponse($response,$this);
		
		$this->_result = (bool)!$this->_response['authorized'];
		
		return $this->_result;
	}
	
	/**
	* Zwraca tablicę zawierającą kody odpowiedzi oraz zawartość ostatniego żądania
	*/
 	public function getResponse() {
		return $this->_response;
	}
	
	/*
	* Zwraca listę dostępnych funkcji servera soap
	* @return array
	*/
	public function getFunctions() {
		if(is_null($this->_functions)) {
			$functions = @$this->_client->__getFunctions();
			$this->_functions = $functions;
		}
		
		return $this->_functions;
	}
	
	/**
	* Ustawia wartość mówiącą czy wymagana jest autoryzacja. Jeśli ustawiona jest na true(domyślnie) to w przypadku użytkownika, który nie jest zalogowany przekierowany zostaie na stronę esculaplogin.com
	* @param boolean $isRequire
	* @return void
	*/
	public function authRequire( $isRequire ) {
		$this->_authRequire = (bool)$isRequire;
	}
	
	/**
	* Zwraca informację czy autoryzacja jest wymagana
	* @return boolean
	*/ 
	public function isAuthRequire() {
		return (bool)$this->_authRequire;
	}
		
	/**
	* Pobiera dane o użytkowniku
	*
	* @param string $userCode
	* @return mixed
	*/
	public function getUserData() {
		
		$this->_response = $this->translateResponse($this->_client->getData($this->_userCode,$this->_siteCode),$this);
		
		if(!is_null($this->_userData)) return $this->_userData;
		
		$this->_response = $this->translateResponse($this->_client->getData($this->_userCode,$this->_siteCode),$this);
		
		if('Query ok' !== $this->_response['message']) return array();
		
		$this->_userData = $this->_decode((string)$this->_response['content']);
		
		return $this->_userData;
	}
	
	/**
	* Dekoduje informacje o użytkowniku
	* @param string $input
	* @level integer poziom iteracji przy wywołaniu nienależy podawać innego poziomu niż 0!
	* @return array
	*/
	protected function _decode($input,$level = 0) {
		$result = array();
		
		$input = explode(":$level:",$input);
		
		foreach($input as $data) {
			$data = explode("=$level>",$data);
			
			if(2 !== count($data)) continue;
			
			if(!preg_match('/='.($level + 1).'>/',$data[1])) {
				
				$result[$data[0]] = $data[1];
			} else {
				
				$result[$data[0]] = $this->_decode($data[1],$level + 1);
			}
		}
		return $result;
	}
	
	/**
	* Ustalenie ciastka z loginem logującej się osoby
	*
	* @return boolean
	*/
    protected function _generateLongLiveCookie( $userLogin )
    {
		
        if($this->_userCode && !isset($_COOKIE['ESCULAP_USER'])) {
			
			
		$host = $_SERVER['HTTP_HOST'];
		
		if(preg_match('/^www\./',$host)) {
		
			$host = preg_replace('/^www/','',$host);
		}
		
		if(preg_match('/[a-zA-Z]+\.(.*)/',$host)) {
			
			$host = preg_replace('/^([a-z]{2}\.)/','',$host);
		}
		
		
            setcookie('ESCULAP_USER', md5('Long'.$userLogin.'lifE').'+'.$userLogin, time()+31000000, '/' , '.esculap.com' );
			return true;
		}
        return false;
    }
	
    /**
	* Pobranie ciastka
	*
	* @return string/null
	*/ 
    protected function _getLongLiveCookie()
    {
        if(isset($_COOKIE['ESCULAP_USER'])){
            $temp = explode('+', $_COOKIE['ESCULAP_USER']);
            $hash = $temp[0];
            $user = $temp[1];
            if(md5('Long'.$user.'lifE')==$hash)
				return $user;
        }
		
        return null;
    }
	
	protected function translateResponse($responseData) {
		$responseData = @explode('#',(string)$responseData);
        
		if(!is_array($responseData)) throw new \Exception('Bad response push in to constructor');
		
		$response['message'] = (!array_key_exists(0, $responseData)) ? 0 : trim($responseData[0]);
		$response['content'] = (!array_key_exists(1, $responseData)) ? '' : trim($responseData[1]);
        $response['authorized'] = (!array_key_exists(2, $responseData)) ? 0 : trim($responseData[2]);
		
		return $response;
	}
}

?>
