<?php
namespace System\Soap;

use Model\MainAuthCode,
    Model\RelUsersSpec,
    System\BannerCampaign\Element\ElementFactory as BannerCampaignElementFactory,
    System\BannerCampaign\Element\DecisionSystem as BannerCampaignDecisionSystem,
    System\Database\AbstractTable,
    Model\MainUsers,
    System\BannerCampaign\Element\Place\PlaceFactory as ElementPlaceFactory,
    Model\MainViewCode;

class Wbserv {

    const STATUS_OK = 'ok';
    const STATUS_FALI = 'fail';
    const STATUS_ACCESS_DENIED = 'access denied';
    const STATUS_USER_NO_LOGGED = 'user is not logged in';
    const STATUS_ELEMENT_SET_INVALID = 'invalid element set';
    const STATUS_USER_SETTINGS_CODE_NOT_EXISTS = 'user settings code is invalid or not exists or expire';

    protected $functions = array();
    protected $bcElementsSets = array();
    protected $domain = '';
    protected $lang;

    /**
     * Zwraca listę i opisy funkcji dla podanego kodu strony jeśli strona zostanie zweryfikowana
     * @param string $sitecode
     * @return integer
     */
    protected function getFunctions($sitecode) {
        if (!$this->isAvail($sitecode, __FUNCTION__)) return 0;

        // do zrobienia

        return 1;
    }

    /**
     * Zwraca listę elementów kampanii do wyświetlenia
     * @param string $sitecode
     * @param string $usercode
     * @param string $language
     * @param string $elementsSetCode
     * @param string|null $cookieData
     * @param string|null $userSettingsCode 
     * @return string
     */
    public function getCampaignElements($sitecode, $usercode, $language, $elementsSetCode, $cookieData = array(), $userSettingsCode = null) {

        // dorobione aby można btło wybierać elementy do wybranych specek
        if (null !== $userSettingsCode)
                return $this->getCampaignElementsForCodeSettings($sitecode, $language, $elementsSetCode, $userSettingsCode, $cookieData);


        if (!$this->isAvail($sitecode, __FUNCTION__, $language))
                return '<response><error>' . self::STATUS_ACCESS_DENIED . '</error></response>';


        if (!in_array($elementsSetCode, $this->bcElementsSets))
                return '<response><error>' . self::STATUS_ELEMENT_SET_INVALID . '</error></response>';


        // 10.09.13 - dodane po to aby można było wybierać rekalmy dla niezalogowanych
        // if( !( $userId = $this->getUserId( $usercode ) ) )  return '<response><error>' . self::STATUS_USER_NO_LOGGED . '</error></response>';
        if (!( $userId = $this->getUserId($usercode) )) {
            // wybieramy target niezalogowany

            $dataToMatch = array(
                'unlogged' => array(1)
            );

            $userData = $dataToMatch;
            $userId = 'NULL';
            $userCode = 'NULL';
        } else {

            $userData = $this->getUserData($userId);

            $dataToMatch = $userData;

            $specs = $dataToMatch['specialization'];
            $dataToMatch['specialization'] = array();

            foreach ($specs as $specType => $specCodes) {

                $specType = preg_replace(array('/^INTE/', '/^MAIN/', '/^CURR/'), array('i', 'l', 'p'), $specType);

                foreach ($specCodes as $specCode) {

                    $dataToMatch['specialization'][] = $specType . '_' . $specCode;
                }
            }
        }


        $Xml = new \DomDocument;
        if ($cookieData && false !== $Xml->loadXml($cookieData)) {

            $cookieData = array();

            foreach ($Xml->getElementsByTagName('entity') as $element) {

                $cookieData[$element->getAttribute('id')] = $element->nodeValue;
            }
        } else {
            $cookieData = array();
        }

        try {

            $elementsPlace = ElementPlaceFactory::getPlacesForCode($elementsSetCode);
        } catch (\Exception $e) {

            $this->log($e, __METHOD__);

            $elementsPlace = array(0);
        }




        //if( !count( $elementsPlace ) )
        //$elementsPlace = array( 0 );

        //trigger_error(json_encode(BannerCampaignElementFactory::matching($dataToMatch, $elementsPlace)));

        $BcElements = BannerCampaignDecisionSystem::decide($elementsSetCode, BannerCampaignElementFactory::matching($dataToMatch, $elementsPlace), $userData, $cookieData);


        // zarejestrowanie wyświetlenia banerów dla użrzytkownika dla elementów niefikcyjnych
        $elementIdList = array();


        foreach ($BcElements as $element) {

            if (true === $element->isFake()) {

                continue;
            }

            $elementIdList[] = $element->getElementId();
        }


        try {

            \System\BannerCampaign\Element\ElementDecorator::notifyView(
                    $elementIdList, $userId, $usercode, $sitecode
            );
        } catch (\Exception $e) {

            $this->log($e->getMessage() . ". Nieudało się zarejestrować wyświetleń dla wybranej listy banerów. userId = {$userId} , usercpde = {$usercode} , elementIdList = " . implode(',', $elementIdList), __METHOD__);
        }

        return $this->toXml($BcElements, $cookieData, $language);
    }

    /**
     * Zwraca listę elementów kampanii do wyświetlenia dla podanego kodu z ustawionymi spckami i typami userów
     * @param string $sitecode
     * @param string $language
     * @param string $elementsSetCode
     * @param string|null $cookieData
     * @param string|null $userSettingsCode 
     * @return string
     */
    protected function getCampaignElementsForCodeSettings($sitecode, $language, $elementsSetCode, $userSettingsCode, $cookieData = array()) {

        if (!$this->isAvail($sitecode, __FUNCTION__, $language))
                return '<response><error>' . self::STATUS_ACCESS_DENIED . '</error></response>';

        if (!in_array($elementsSetCode, $this->bcElementsSets))
                return '<response><error>' . self::STATUS_ELEMENT_SET_INVALID . '</error></response>';

        \System\Database\AbstractTable::setSchema($language . '_content');
        //return '<response><error>' . self::STATUS_USER_SETTINGS_CODE_NOT_EXISTS . '</error></response>';
        $MainViewCode = new MainViewCode;
        try {

            if (false === ( $codeData = $MainViewCode->loadData($userSettingsCode) )) {

                return '<response><error>' . self::STATUS_USER_SETTINGS_CODE_NOT_EXISTS . '</error></response>';
            }
        } catch (\Exception $e) {

            return '<response><error>' . self::STATUS_USER_SETTINGS_CODE_NOT_EXISTS . '</error></response>';
        }


        $dataToMatch = array();
        $dataToMatchWithSutedSpecs = array();


        $types = explode(',', $codeData['types']);
        foreach ($types as $type) {

            $dataToMatch['user_type'][] = $type;
        }




        $dataToMatchWithSutedSpecs = $dataToMatch;


        $specs = explode(',', $codeData['specs']);
        foreach ($specs as $spec) {

            $dataToMatch['specialization'][] = $spec;

            $spec = explode('_', $spec);
            if (2 !== count($spec)) continue;

            $dataToMatchWithSutedSpecs['specialization'][$spec[0]][] = $spec[1];
        }

        $Xml = new \DomDocument;
        if (false !== $Xml->loadXml($cookieData)) {

            $cookieData = array();

            foreach ($Xml->getElementsByTagName('entity') as $element) {

                $cookieData[$element->getAttribute('id')] = $element->nodeValue;
            }
        } else {
            $cookieData = array();
        }

        try {

            $elementsPlace = ElementPlaceFactory::getPlacesForCode($elementsSetCode);
        } catch (\Exception $e) {


            $this->log($e->getMessag(), __METHOD__);
            $elementsPlace = array(0);
        }

//                $matching = BannerCampaignElementFactory::matching( $dataToMatch , $elementsPlace );
//                
//                $matchingClasses = array();
//                foreach ($matching as $match) {
//                    $matchingClasses[] = get_class($match);
//                }
//                
//                ob_start();
//                var_dump($matchingClasses);
//                $matchingClassesDump = ob_get_clean();
//                
//                return '<response><error>' . "matching yo: {$matchingClassesDump}" . '</error></response>';

        $BcElements = BannerCampaignDecisionSystem::decide($elementsSetCode, BannerCampaignElementFactory::matching($dataToMatch, $elementsPlace), $dataToMatchWithSutedSpecs, $cookieData);



        return $this->toXml($BcElements, $cookieData, $language);
    }

    /**
     * @param string $sitecode
     * @param string $usercode
     * @param string $language
     * @param int $mainBcElementId
     * @param string $url
     * @return boolean
     */
    public function notifyClick($sitecode, $usercode, $language, $mainBcElementId, $url) {

        if (!$this->isAvail($sitecode, __FUNCTION__, $language))
                return '<response><error>' . self::STATUS_ACCESS_DENIED . '</error></response>';

        // umożliwiamy zliczanie klików niewiadomego pochodzenia....
        //if( !( $userId = $this->getUserId( $usercode ) ) ) return '<response><error>' . self::STATUS_USER_NO_LOGGED . '</error></response>';


        trigger_error($usercode);


        if (!( $userId = $this->getUserId($usercode) )) {


            trigger_error('sad');


            $userId = 'NULL';
            $usercode = 'NULL';
        }


        try {

            if (\System\BannerCampaign\Element\ElementDecorator::notifyHit(
                            $mainBcElementId, $userId, $usercode, $sitecode, $url
                    )) return true;
        } catch (\Exception $e) {

            //trigger_error( $e->getMessage() );
            $this->log($e->getMessage(), __METHOD__);
        }

        return false;
    }

    /**
     * @param string $sitecode
     * @param string $usercode
     * @param string $language
     * @param int $idContent
     * @param string $nick
     * @param string $content
     * @param int|null $parentContentCommmentId
     * @return bool
     */
    public function addComment($sitecode, $usercode, $language, $idContent, $nick, $content, $parentContentCommmentId = null) {
        if (!$this->isAvail($sitecode, __FUNCTION__, $language))
                return '<response><error>' . self::STATUS_ACCESS_DENIED . '</error></response>';
        if (!( $userId = $this->getUserId($usercode) ))
                return '<response><error>' . self::STATUS_USER_NO_LOGGED . '</error></response>';

        $filter = new \Zend\Filter\FilterChain;
        $filter->attach(new \Zend\Filter\PregReplace(array('pattern' => "/'/", 'replacement' => "''")));
        $filter->attach(new \Zend\Filter\StripTags(array(
            'allowTags' => array('b,i,u,br')
        )));

        $content = $filter->filter($content);

        $MainContentComment = new \Model\MainContentComment;

        try {

            if ($MainContentComment->add($idContent, $language, $content, $nick, $userId, $parentContentCommmentId)) {

                return true;
            }

            return false;
        } catch (\Exception $e) {

            $this->log($e->getMessage(), __METHOD__);

            return false;
        }
    }

    /**
     * @param string $sitecode
     * @param string $usercode
     * @param string $language
     * @param int $contentId
     * @return string
     */
    public function getContent($sitecode, $usercode, $language, $contentId) {
        if (!$this->isAvail($sitecode, __FUNCTION__, $language))
                return '<response><error>' . self::STATUS_ACCESS_DENIED . '</error></response>';
        if (!( $userId = $this->getUserId($usercode) ))
                return '<response><error>' . self::STATUS_USER_NO_LOGGED . '</error></response>';

        /*
          try {

          $content = \System\Content\ContentFactory::factory( new \System\Content\Config( array(
          'CONTENT_TYPE' => null,
          'SPEC_TYPES' => array(),
          'CONTENT_ID' => 823,
          'GET_COMMENTS' => 1
          ) , true ));

          } catch( \Exception $e ) {

          return $e->getMessage();

          echo $e->getMessage();
          }

          echo 'asdsad';
         */
        //na chwilę obecną możliwe jest zwrócenie wyłącznie typów contentu takich jak publication i news
    }

    /**
     * Zwraca listę różnych typów contentu
     * 
     * @param string $sitecode  
     * @param string $usercode
     * @param integer $end
     * @param integer|null $start
     * @param array|null $types
     * @param bool|null $comments
     * @return string
     */
    protected function getContentList($sitecode, $usercode, $end, $start = 0, $types = array(), $comments = false) {
        if (!$this->isAvail($sitecode, __FUNCTION__)) return '';
    }

    /**
     * Sprawdza czy dla podanego sitecoda dostępna jest podana funkcja
     *
     * @param string $sitecode
     * @param string $functionName
     * @return boole
     */
    protected function isAvail($sitecode, $functionName, $lang) {
        // pobranie listy funkcji dostępnych dla danego sitecode i porównanie z nazwą wywoływanej funkcji

        $MainAuthSite = new \Model\MainAuthSite;


        if (false === ( $siteData = $MainAuthSite->get($sitecode) ))
                return false;
        if (!isset($_SERVER['REMOTE_ADDR']) || trim($_SERVER['REMOTE_ADDR']) !== trim($siteData['server_addr']))
                return false;
        if (0 > strpos($siteData['webserv_functions'], $functionName))
                return false;

        if (!in_array(strtolower(trim($lang)), explode(',', strtolower(trim($siteData['wbserv_lang'])))))
                return false;

        $this->bcElementsSets = explode(',', strtolower(trim($siteData['bc_set_available'])));
        $this->functions = explode(',', $siteData['webserv_functions']);
        $this->domain = $siteData['location'];
        $this->lang = strtolower(trim($lang));

        AbstractTable::setContentSchema($this->lang . '_content');

        return true;
    }

    /**
     * Pobiera id użytkownika na podstawie authcoda
     *
     * @param string $usercode
     * @return int|bools
     */
    protected function getUserId($usercode) {

        $MainAuthCode = new MainAuthCode;

        try {

            if (false !== ( $userId = $MainAuthCode->getUserId($usercode) ))
                    return $userId;
            return false;
        } catch (\Exception $e) {

            $this->log($e, __METHOD__);
            return false;
        }
    }

    /**
     * 
     * 
     */
    protected function getUserData($userId) {
        $userId = (int)$userId;

        $result = array(
            'specialization' => array(
                'i' => array(),
                'l' => array(),
                'p' => array()
            ),
            'user_type' => ''
        );

        $RelUsersSpec = new RelUsersSpec;

        $userSpecs = $RelUsersSpec->getSpecsForUserById($userId);


        foreach ($userSpecs as $userSpec) {

            switch ($userSpec['spec_type']) {
                case 'INTE':

                    $result['specialization']['i'][] = $userSpec['spec_code'];

                    break;
                case 'MAIN':

                    $result['specialization']['l'][] = $userSpec['spec_code'];

                    break;
                case 'CURR':

                    $result['specialization']['p'][] = $userSpec['spec_code'];

                    break;
            }
        }


        $MainUsers = new MainUsers;
        $userBasicData = $MainUsers->getUser($userId);

        $result['user_type'] = $userBasicData['user_type'];

        return $result;
    }

    /**
     * Koduje listę elementów do xml-a
     * @param array $data
     * @return string
     */
    protected function toXml($bcElementsList, $cookieData, $language) {


        $Xml = new \DomDocument('1.0', 'UTF-8');
        $root = $Xml->createElement('response');
        $mainElementsGroup = $Xml->createElement('elements');

        $root->appendChild($mainElementsGroup);
        $Xml->appendChild($root);


        foreach ((array)$bcElementsList as $element) {

            $elementRoot = $Xml->createElement('element');
            $elementRoot->setAttribute('bcid', $element->getBcId());
            $elementRoot->setAttribute('id', $element->getElementId());

            if ($element->isFake()) {

                $elementRoot->setAttribute('fake', true);
            }

            $newElement = $Xml->createElement('type');
            $newElementText = $Xml->createTextNode($element->getId());
            $newElement->appendChild($newElementText);
            $elementRoot->appendChild($newElement);

            $newElement = $Xml->createElement('place');
            $newElementText = $Xml->createTextNode($element->getPlace()->getId());
            $newElement->appendChild($newElementText);
            $elementRoot->appendChild($newElement);

            $newElement = $Xml->createElement('additional_stats');
            $newElementText = $Xml->createTextNode($element->getAdditionalStats());
            $newElement->appendChild($newElementText);
            $elementRoot->appendChild($newElement);

            foreach ($element->getTypeData() as $name => $value) {
                $newElement = $Xml->createElement($name);
                //trigger_error($value);
                $newElementText = $Xml->createTextNode($value);
                $newElement->appendChild($newElementText);
                $elementRoot->appendChild($newElement);
            }

            $comments = $Xml->createElement('comments');
            //$elementRoot->appendChild( $comments );
            // sprawdzam czy do rozpatrywanego typu należy pobrać jakąś treść i komentarze
            // trochę dziadostwo ale cóż począć...
            if (1 == $element->getId() //||
            //4 == $element->getId() 
            ) {


                if (1 == $element->getId()) {

                    // Jeśli mam do czynienia z contentem to muszę pobrać jego treść.

                    static $mainContent = null;
                    if (null == $mainContent)
                            $mainContent = new \Model\MainContent;

                    $contentId = $element->getContentId();


                    try {


                        $baseContentData = $mainContent->getContent($contentId, $language);
                        $ContentTypeObj = null;

                        if (false !== $baseContentData) {

                            if ('news' == $baseContentData['type']) {

                                $ContentTypeObj = new \Model\MainContentNews;
                            } elseif ('publ' == $baseContentData['type']) {

                                $ContentTypeObj = new \Model\MainContentPublication;
                            } elseif ('event' == $baseContentData['type']) {

                                $ContentTypeObj = new \Model\MainContentEvent;
                            }

                            if (null !== $ContentTypeObj) {

                                $contentData = $ContentTypeObj->getElement($contentId, $language);

                                $newElement = $Xml->createElement('image');
                                $newElementText = $Xml->createTextNode($contentData['avatar_id']);
                                $newElement->appendChild($newElementText);
                                $elementRoot->appendChild($newElement);

                                $newElement = $Xml->createElement('title');
                                $newElementText = $Xml->createTextNode($contentData['title']);
                                $newElement->appendChild($newElementText);
                                $elementRoot->appendChild($newElement);

                                $date = strtotime(substr($contentData['add_date'], 0, 10));

                                $newElement = $Xml->createElement('date');
                                $newElementText = $Xml->createTextNode($date);
                                $newElement->appendChild($newElementText);
                                $elementRoot->appendChild($newElement);

                                $description = $Xml->createElement('description');
                                $description->appendChild($description->ownerDocument->createCDATASection($contentData['body']));

                                $elementRoot->appendChild($description);
                            }
                        }
                    } catch (\Exception $e) {

                        $this->log($e->getMessage(), __MESSAGE__);
                    }
                }

                // jeśli mamy do czynienia z reklamą będącą contentem, pobieram komentarze

                $contentId = $element->getContentId();

                try {

                    static $mainContentComment = null;
                    if (null == $mainContentComment)
                            $mainContentComment = new \Model\MainContentComment;

                    $allComments = $mainContentComment->getComments($contentId);

                    foreach ($allComments as $commentEntity) {

                        $comment = $Xml->createElement('comment');
                        $comment->setAttribute('id', $commentEntity['comment_id']);
                        if (null !== $commentEntity['parent_comment_id'])
                                $comment->setAttribute('parentid', $commentEntity['parent_comment_id']);

                        $newElement = $Xml->createElement('content');
                        $newElementText = $Xml->createTextNode($commentEntity['body']);
                        $newElement->appendChild($newElementText);
                        $elementRoot->appendChild($newElement);

                        $newElement = $Xml->createElement('nick');
                        $newElementText = $Xml->createTextNode($commentEntity['nick']);
                        $newElement->appendChild($newElementText);
                        $elementRoot->appendChild($newElement);

                        $date = strtotime(substr($commentEntity['add_date'], 0, 10));

                        $newElement = $Xml->createElement('date');
                        $newElementText = $Xml->createTextNode($date);
                        $newElement->appendChild($newElementText);
                        $elementRoot->appendChild($newElement);
                        $comments->appendChild($comment);
                    }
                } catch (\Exception $e) {

                    $this->log($e->getMessage(), __MESSAGE__);
                }
            }


            $mainElementsGroup->appendChild($elementRoot);
        }


        $cookie = $Xml->createElement('history');
        $root->appendChild($cookie);

        foreach ((array)$cookieData as $elementId => $count) {

            $elementData = $Xml->createElement('entity', $count);
            $elementData->setAttribute('id', $elementId);

            $cookie->appendChild($elementData);
        }

        return $Xml->saveXML();
    }

    /**
     * @param string $msg 
     * @param string $methodName
     * @tryutn void
     */
    protected function log($msg, $methodName) {

        trigger_error(
                $msg . "\n" . $methodName
        );
    }

}

?>