<?php

/**
 * Klasa obsługująca SOAP
 */

namespace System\Soap;



class Server {
	private static $_eloginUri = 'http://esculaplogin.com/user/login/';
	protected $_visitIsLogged = false; // pomocnicza zmienna pozwalająca łatwiej skomunikować metedę isLogged w sytuacji gdy loduje żądanie
	protected $_isLoginCheck = false; // podobnie jak wyżej zmienna pomocnicza mówiąca czy funkcja logVisit wywoływana jest z wnętrza isLogged
	protected static $_SESS_MAX_TIME = 60; // czas jaki dodany zostanie do długości ważności sesji
	
    const QUERY_OK = 'Query ok';
    const QUERY_FAIL = 'Query fail';
    const LOGIN_FAILURE = 'Login failure';
    const USER_LOGGED_IN = 'User are logged in';
    const USER_NOT_LOGGED_ID ='User are not logged in';
    const USER_LOGGED_OUT = 'User logged out';
    const LOGIN_FAIL = 'Login failure';
    const UNDEFINED_ACTION = 'Undefined action';
    const ACCESS_ALLOWED = 'Access is allow';
    const ACCESS_DENIED = 'Access is deny';
	const REDIRECT = 'Redirect';
    const LOGOUT_DONE = 'User logged out';
	const LOGOUT_FAIL = 'Logout failed';
	const BAD_USER_CODE = 'Bad user code';
	const BAD_SITE_CODE = 'Bad site code';
	const QUERY_LOGGED = 'Query logged';
	const QUERY_NOT_LOGGED = 'Query are not logged';
	
    /**
     * Zawiera dostępne typy restrykcji oraz możliwość ich zastosowania np: nie zastosujemy operatora > z nazwą profesji
     * @var array $_availableRestrictions
     */
    
    protected $_availabelRestrictions = array(
        'profession' => 'eq,like', // 
        'city' => 'eq,like'//
    );
    
    /**
     * Zawiera dostępne typy restrykcji
     * @var array $_availableRestrictionsMod
     */
    
    protected $_availableRestrictionsMod = array(
        'lt', // <
        'gt', // >
        'eq', // ==
        'like' // '%$%'
    
    );
	
	
	CONST ELOGIN_GATE = 'http://esculaplogin.com';
	
	
    /**
     * Zawiera obiekt kontaktujący się z bazą uzytkowników
     * @var object $_dbObject
     */
    
    protected $_dbObject = null;
    
    /**
     * Zawera przetłumaczone kody odpowiedzi
     * @var array $_responseCodes
     */
    protected $_responseCodes = array(
        
    );
	
    /**
     * logowanie odwiedzin
	 * @param int $logged jeśli true parametr $usercode oznacza user_login a nie auth_code
	 * @param string $lastLoggedUser ostation zalogowany użytkownik
	 * @param string $usercode kod pracownika
	 * @param string $sitecode
	 * @param string $host $_SERVER['HTTP_HOST']
	 * @param string $uri $_SERVER['REQUEST_URI']
	 * @param string $referer $_SERVER['HTTP_REFERER']
     * @return string
     */
    
    public function logVisit( $logged , $lastLoggedUser , $usercode , $sitecode , $host , $uri , $referer ) {
        $response = array(
			'code' => 0,
			'content' => '',
			'authorized' => 0
		);
		
		$MainAuthSite = new \Model\MainAuthSite();
		
		
		
		if( !$MainAuthSite->isSiteCodeExists( $sitecode ) ) { // jeśli kod strony nie jestnieje
			
			$response['code'] = self::QUERY_FAIL;
			return implode('#',$response);
		}
		
		$MainAuthCode = new \Model\MainAuthCode();
		$MainUsers = new \Model\MainUsers();
		
		// zgodność z poprzednimi wersjami
		if($MainAuthCode->isAuthCodeExists($usercode)) {
			
			$userId = $MainAuthCode->getUserId( $usercode );
			$userId = @$userId[ 'id' ];
			$isLoggedIn = true;
			
		} elseif( !$logged || !$MainAuthCode->isAuthCodeExists($usercode) && $MainUsers->isLoginExists( $lastLoggedUser ) ) {
			
			$userId = $MainUsers->getUserData( $lastLoggedUser );
			$userId = @$userId[ 'id' ];
			$isLoggedIn = false;
			
		} else {
			
			$userId = $MainUsers->getUserData( $lastLoggedUser );
			$userId = $userId[ 'id' ];
			$isLoggedIn = false;
			
		}
		
		
		//trigger_error( '>|' );
		
		$MainUsersLog = new \Model\MainUsersLog( $usercode );
		
		
		//trigger_error( '|<' );
		
		if( 0 < (int)$userId && $MainUsersLog->appendRow( $userId , $host , $uri , $referer , $isLoggedIn , $usercode ) ) {
			
			// jeśli użytkownik jest zalogowany a funkcja isLoginCheck nie jest wywoływana z wnętrza funkcji isLogged przedłużam ważność sesji użytkownika
			
			$MainAuthCode->refresh($usercode,self::$_SESS_MAX_TIME);
			
			$response['code'] = self::QUERY_OK;
			
		} else {
			
			$response['code'] = self::QUERY_FAIL;
		}
		
		return implode('#',$response);
    }
    
	/**
     * alias do logVisit w celu zapewnienia kompatybilości
	 * @param string $sitecode kod strony
	 * @param string $usercode kod pracownika
	 * @param string $host $_SERVER['HTTP_HOST']
	 * @param string $uri $_SERVER['REQUEST_URI']
	 * @param string $referer $_SERVER['HTTP_REFERER']
     * @return string
     */
	public function refresh($sitecode, $usercode, $host, $uri, $referer) {
		try {
			
			
			$this->logVisit( 1 , 0 , $usercode , $sitecode , $host , $uri , $referer );
			
		} catch( \Exception $e ) {
			
			trigger_error( $e->getMessage() );
		}
		return 'success->ok';
	}
	
	/**
     * alias do refresh w sytuacji gdy użytkownik nie jest zalogowany
	 * @param string $sitecode kod strony
	 * @param string $usercode kod pracownika
	 * @param string $host $_SERVER['HTTP_HOST']
	 * @param string $uri $_SERVER['REQUEST_URI']
	 * @param string $referer $_SERVER['HTTP_REFERER']
     * @return string
     */
	public function refreshUnlogged($sitecode, $usercode, $host, $uri, $referer) {
		try {
		
			$this->logVisit( 1 , $usercode , 0 , $sitecode , $host , $uri , $referer );
			
		} catch( \Exception $e ) {
			
			trigger_error( $e->getMessage() );
		}
		
		return 'success->ok'.$usercode;
	}
	
	/**
	* Sprawdza czy logowanie przez soap jest możliwe
	* @param string $siteCode 
	* @return string
	*/
	public function isSoapAllowed($siteCode) {
		
		$MainAuthSite = new \Model\MainAuthSite();
		
		if(1 !== $MainAuthSite->isSiteCodeExists($siteCode)) return 'error->bad_sitecode'.$MainAuthSite->isSiteCodeExists($siteCode);
			
		if(1 == $MainAuthSite->isSoapAllowed($siteCode)) {
			return 'success->on';
		} else {
			return 'success->off';
		}
		
        return 'error->uncatch_error';
	}
	
	/**
	* Funkcja napisana aby zapewnić kompatybilność ze starą wersją
	* @param string $sitecode
	* @param string $usercode
	* @return string
	*/
	public function isLogged($sitecode,$usercode) {
		
		$MainAuthSite = new \Model\MainAuthSite();
        if(!$MainAuthSite->isSiteCodeExists($sitecode)) return 'error->bad_sitecode';
		
		$MainAuthCode = new \Model\MainAuthCode();
        if(!$MainAuthCode->isAuthCodeExists($usercode)) return 'error->bad_usercode';
		
        return 'success->ok';
	}

	
    /**
     * sprawdza czy uzytkownik jest zalogowany
     * @param string $siteCode
     * @param string $userCode kod uzytkownika
	 * @return string
     */
    public function isLoggedEx( $siteCode , $authCode ) {
		$response = array(
			'code' => 0,
			'content' => null,
			'authorized' => 0
		);
		
		//trigger_error( $authCode );
		
        $MainAuthSite = new \Model\MainAuthSite;
		
		// sprawdzam czy istnieje i pobietam kod strony 
        if(!$MainAuthSite->isSiteCodeExists($siteCode)) {
		
			$response['code'] = self::QUERY_FAIL;
			return implode('#',$response);
		}
		
		$MainAuthCode = new \Model\MainAuthCode();
		
		if(!$MainAuthCode->isAuthCodeExists( $authCode ) ) {
		
			if(!$MainAuthSite->isSoapLoginAvailable( $siteCode ) ) {
				
				$response['code'] = self::REDIRECT;
				$response[ 'content' ] = self::ELOGIN_GATE;
			} else {
			
				$response['code'] = self::ACCESS_DENIED;
			}	
			
			return implode('#',$response);
		}
		
		$userId = $MainAuthCode->getUserId( $authCode );
			
		$response['code'] = self::QUERY_OK;
		$response['authorized'] = 1;
		$response['content'] = $authCode.'::'.$userId;
		
		return implode('#',$response);
    }
	
	/**
	 * alias do logout
	 * @param string $sitecode
	 * @param string $usercode
	 * @returns string
	 */
	public function endSession( $sitecode , $usercode ) {
	
		$this->logout( $sitecode , $usercode );
		
		return "success->ok";
	}
	
	
    /**
     * Wylogowanie użytkownika
     * 
     * @param string $sitecode kod strony
     * @param string $userCode kod pracownika
     * @return string
     */
    public function logout( $sitecode , $userCode ) {
        $response = array(
            'code' => 0,
            'content' => "",
			'authorized' => 0
        );
		
        $MainAuthSite = new \Model\MainAuthSite();
		
		if($MainAuthSite->isSiteCodeExists($sitecode)) {
			$MainUsers = new \Model\MainUsers();
			$MainAuthCode = new \Model\MainAuthCode();
				
			if($MainAuthCode->isAuthCodeExists($userCode)) {
				
				
				if($MainAuthCode->removeCode($userCode)) {
					$this->gc();
					
					$MainUsersLogin = new \Model\MainUsersLogin();
		
					$MainUsersLogin->setEndDate($userCode);
					
					$response['code'] = self::QUERY_OK;
					$response['authorized'] = 0;
					
					return implode('#',$response);
				} else {
					
					$response['code'] = self::ACCESS_DENIED;
					$response['authorized'] = 1;
				}
			}
		}
		
		$response['code'] = self::QUERY_FAIL;
		
        return implode('#',$response);
    }
	
	
	/**
	* Rozpoczynanie sesji jeśli możliwe jest logowanie za pomosą SOAP
	* @param string $sitecode
	* @param string $login
	* @param string $pass
	* @param string $us 
	* @param string $up
	* @param string $ui
	* @param string $lang
	* return string
	*/
	public function createSession( $sitecode , $login , $password , $us , $up , $uri , $lang = 'pl' ) {
		
		$response = array(
            'code' => 0,
            'content' => '',
			'authorized' => 0
        );
		
		
		// sprawdzam sitecode
		$MainAuthSite = new \Model\MainAuthSite;
		
		try {
			
			
			$isSitecodeExists = $MainAuthSite->isSiteCodeExists( $sitecode );
			
			if( !$isSitecodeExists ) {
				$response[ 'code' ] = self::BAD_SITE_CODE;
				
				return implode( '###' , $response );
			}
			
		} catch( \Exception $e ) {
			
			$response[ 'code' ] = self::BAD_SITE_CODE;
				
			return implode( '###' , $response );
		}
		
		
		// sprawdzam czy logowanie przez soap jest możliwe
		try {
			if( !$MainAuthSite->isSoapLoginAvailable( $sitecode ) ) {
				$response[ 'code' ] = self::ACCESS_DENIED;
				
				return implode( '###' , $response );
			}
			
		} catch( \Exception $e ) {
			
			$response[ 'code' ] = self::ACCESS_DENIED;
				
			return implode( '###' , $response );
		}
		
		// sprawdzanie użytkownika
		$MainUsers = new \Model\MainUsers;
		try {
			
			$userData = $MainUsers->getUserByLogin( $login );
			
			if( !count( $userData ) || 
				!isset( $userData[ 'password' ] ) ||
				$password !== $userData[ 'password' ] ||
				!isset( $userData[ 'trust_level' ] ) ) {
				
				$response[ 'code' ] = self::QUERY_FAIL;
				return implode( '###' , $response );	
			}
			
			if( 10 > $userData[ 'trust_level' ] ) {
				
				if( 2 == (int)$userData[ 'trust_level' ] ) {
					
					$response[ 'content' ] = 'locked';
					
				} elseif( 5 == (int)$userData[ 'trust_level' ] ) {
					
					$response[ 'content' ] = 'need_activation';
				}
				
				$response[ 'code' ] = self::QUERY_FAIL;
				
				return implode( '###' , $response );
			}
			
		} catch( \Exception $e ) {
			
			$response[ 'code' ] = self::QUERY_FAIL;
				
			return implode( '###' , $response );
		}
		
		
		// tworzenie i zapisywanie nowego authcoda
		$MainAuthCode = new \Model\MainAuthCode;
		$userId = $userData[ 'id' ];
		
		try {
			if( !( $authCode = $MainAuthCode->isCodeExistsForUserEx( $userId ) ) ) {
			
				$authCode = $MainAuthCode->getNewAuthCodeEx( $userId , $us , $up , $ui );
			
			} else {
				
				$MainAuthCode->refreshEx( $authCode , self::$_SESS_MAX_TIME );
			}
			
		} catch( \Exception $e) {
				
			$response[ 'code' ] = self::QUERY_FAIL . '. ' . $e->getMessage();
					
			return implode( '###' , $response );
		}
		
		
		$response[ 'code' ] = self::QUERY_OK;
		$response[ 'authorized' ] = 1;
		
		$MainUsersLogin = new \Model\MainUsersLogin;
		$MainUsersLogin->logLogin(
			$userId,
			$authCode,
			$us,
			$up,
			$ui
		);
		
		return implode( '###' , $response );
	}
	
	
	
	/**
	* Rozpoczynanie sesji
	* @param string $sitecode
	* @param string $login
	* @param string $password
	* @param string $us 
	* @param string $up
	* @param string $ui
	* return boolean
	*/
	public function beginSession( $sitecode , $login , $password , $us , $up , $ui ) {
		
		
		// sprawdzam sitecode
		$MainAuthSite = new \Model\MainAuthSite;
		
		try {
			
			
			$isSitecodeExists = $MainAuthSite->isSiteCodeExists( $sitecode );
			
			if( !$isSitecodeExists ) {
				return 'error->bad_sitecode';
			}
			
		} catch( \Exception $e ) {
			
			return 'error->bad_sitecode';
		}
		
		
		// sprawdzam czy logowanie przez soap jest możliwe
		try {
			if( !$MainAuthSite->isSoapLoginAvailable( $sitecode ) ) {
				return 'error->soap_deny';
			}
			
		} catch( \Exception $e ) {
			return 'error->soap_deny';
		}
		
		// sprawdzanie użytkownika
		$MainUsers = new \Model\MainUsers;
		try {
			
			$userData = $MainUsers->getUserByLogin( $login );
			
			if( !count( $userData ) || 
				!isset( $userData[ 'password' ] ) ||
				$password !== $userData[ 'password' ] ||
				!isset( $userData[ 'trust_level' ] ) ) {
				
				return 'error->bad_password';
			}
			
			if( 10 > $userData[ 'trust_level' ] ) {
				
				if( 2 == (int)$userData[ 'trust_level' ] ) {
					
					return 'error->locked';
					
				} elseif( 5 == (int)$userData[ 'trust_level' ] ) {
					
					return 'error->need_activation';
				}
				
				$response[ 'code' ] = self::QUERY_FAIL;
				
				return implode( '###' , $response );
			}
			
		} catch( \Exception $e ) {
			
			return 'error->bad_password';
		}
		
		
		// tworzenie i zapisywanie nowego authcoda
		$MainAuthCode = new \Model\MainAuthCode;
		$userId = $userData[ 'id' ];
		
		try {
			if( !( $authCode = $MainAuthCode->isCodeExistsForUserEx( $userId ) ) ) {
			
				$authCode = $MainAuthCode->getNewAuthCodeEx( $userId , $us , $up , $ui );
			
			} else {
				
				$MainAuthCode->refreshEx( $authCode , self::$_SESS_MAX_TIME );
			}
			
		} catch( \Exception $e) {
				
			$response[ 'code' ] = self::QUERY_FAIL . '. ' . $e->getMessage();
					
			return implode( '###' , $response );
		}
		
		
		return "success->".$authCode;
	}
	
	
	
	/**
	* Zwraca informacje o użytkowniku. Pod indexem 1 'content' znajdują się informacje w postacie: nazwa_pola=>wartość,nazwa_pola=>wartość
	* @param string $userCode
	* @param string $siteCode
	* @return string
	*/
	public function getData($userCode,$siteCode) {
		$response = array(
			'code' => 0,
			'content' => '',
			'authorized' => 1
		);
		
		$MainUsers = new \Model\MainUsers();
		$MainAuthCode = new \Model\MainAuthCode();
		
		$userId = $MainAuthCode->getUserId( $userCode );
		
		$userData = $MainUsers->getUser( $userId , true );
		$MainAuthSite = new \Model\MainAuthSite();
		
		if( $userId ) {
			// sprawdzam poprawność wykonania funkcji
			if(null !== $userData && count($userData)) {
				
				
				$allowedFields = $MainAuthSite->getAllowedFieldsList($siteCode);
				
				
				if(is_array($allowedFields)) {
					
					$response['code'] = self::QUERY_OK;
			
					$_tmp = \Model\MainUsers::filterReturnData( $userData , $allowedFields , $userId );
					
					if( !isset( $_tmp[ 'user_id' ] ) && isset( $_tmp[ 'id' ] ) ) {
						
						$_tmp[ 'user_id' ] = $_tmp[ 'id' ];
					}
					
					$response['content'] = $this->_encode($_tmp);
					
					return implode('#',$response);
				}
			}
		}
		$response['authorized'] = 0;
		$response['code'] = self::QUERY_FAIL;
		return implode('#',$response);
	}
	
	/**
	* pobranie danych o użytkowniku (wersja dla starego klienta)
	* @param string $sitecode 
	* @param string $usercode
	* @return string
	*/
	public function userData( $sitecode , $usercode ) {
		
		$MainAuthSite = new \Model\MainAuthSite;
		
		if(!$MainAuthSite->isSiteCodeExists( $sitecode ) ) return 'error->bad_sitecode';
		
		$MainAuthCode = new \Model\MainAuthCode();
		
		if( !$MainAuthCode->isAuthCodeExists( $usercode ) ) return 'error->bad_usercode';
		
		$MainUsers = new \Model\MainUsers();
		
		// pobieram listę dostępnych pól
		
		$allowedFields = $MainAuthSite->getAllowedFieldsList($sitecode);
		
		
		$userId = $MainAuthCode->getUserId( $usercode );
		
		$userData = $MainUsers->getUser( $userId , true );
		
		$_userData = \Model\MainUsers::filterReturnData( $userData , $allowedFields , $userId );
		
		if( isset( $_userData[ 'login' ] ) ) {
			
			$_userData[ 'user_login' ] = $_userData[ 'login' ];
		} if( !isset( $_userData[ 'user_id' ] ) && isset( $_userData[ 'id' ] ) ) {
						
			$_userData[ 'user_id' ] = $_userData[ 'id' ];
		}
		
		if(count($_userData)) {
			
			return 'success->'.serialize($_userData); 
		}
		
		return 'error->no_data';
	}
	
	/**
	* Koduje odpowiedź
	* @param array $input tablica wejściowa
	* @level integer posiom przy wywołaniu funkcji nienależy podawać poziomu jnnego niż 0!
	*/
	protected function _encode($input,$level = 0) {
		$result = '';
		
		foreach($input as $key => $val) {
			if(strlen($result)) $result .= ":$level:";
			
			if(!is_array($val)) {
				
				$result .= "$key=$level>$val";
				
			} else {
				
				$result .= "$key=$level>{$this->_encode($val,$level + 1)}";
			}
		}
		
		return $result;
	}
	
	/**
	* Czyści przestarzałem wpisy w baize
	*/
	
	public function gc() {
		$MainAuthCode = new \Model\MainAuthCode();
		$MainAuthCode->gc();
	}
}

?>