<?php

namespace System\Translator;
use \Model\MainPhrase;

class TranslatorEx {
	
	protected static $isInit = false;
	protected static $translatorInstance = null;
	
	public static function init( $lang , $system = 'c' ) {
		if(self::$isInit) {
			return true;
		}
	
		
			
		$MainPhrase = new MainPhrase;
		
		$data = $MainPhrase->getAllEx( $lang , $system );
		
		// wartości
		$values = array();
		// przestrzeń nazw pod indeksem 0 to domyślna przestrzeń nazw....
		$namespaces = array();
		
		
		$key = 0;
		foreach($data as $dataRow) {
			
			$values[$key] = trim($dataRow['value']);
			
			if(null == $dataRow['namespace']) {
				$namespace = 0;
			} else {
				$namespace = trim($dataRow['namespace']);
			}
			
			
			if(!isset($namespaces[$namespace])) {
				$namespaces[$namespace] = array();
			}
			
			
			// mapa kluczy, która przyśpieszy wyszukiwanie
			$namespaces[$namespace][$key] = trim($dataRow['text_id']);
			
			
			$key++;
		}
		
		$storage = new TranslatorStorageEx( $values , $namespaces , $lang , $system );
		self::$translatorInstance = $storage;
		self::$isInit = true;
	}
	
	public static function get() {
		if(false === self::$isInit) {
			throw new \Exception(__METHOD__.'. Translator nie został jeszcze zainicializowany');
		}
		
		return self::$translatorInstance;
	}
	
	public static function setInstance(TranslatorStorageEx $instance) {
		
		self::$isInit = true;
		self::$translatorInstance =  $instance;
	}

	protected function __construct() {}
}

class TranslatorStorageEx extends \System\Observable {
	protected $locked = false;
	
	protected $data;
	protected $namespaces;
	protected $namespace;
	protected $system;
	protected $lang;
	protected $debug = false;
	
	public function __construct( $values , $namespaces , $lang , $system ) {
		
		
	
		$this->data = $values;
		$this->lang = $lang;
		$this->system = $system;
		$this->namespaces = $namespaces;
		$this->lock();
	}
	
	/**
	* Zabezpiecza przed zapisem
	*/
	public function lock() {
		$this->locked = true;
	}
	
	public function __set($name,$value) {
		if(false === $this->locked) {
			$this->$name = $value;
			return;
		}
		
		throw new Exception(__CLASS__." jest tylko do odczytu");
	}
	
	/**
	* Tłumaczy frazę
	* @param $name string nazwa frazy
	* @param $namespace string przestrzeń nazw. Jeśli null sprawdzam w zmiennej ustawionej przez setNamespace()
	* @see System\Translator\TranslatorStorageEx::setNamespace()
	*/
	public function translate( $name , $namespace = null ) {
		$name = trim($name);
		
		// sprawdzam czy przestrzeń nazw istnieje
		$nSpace = null === $namespace ? $this->namespace : $this->checkNamespace($namespace);
		
		// najpierw szukam w ustalonej przestrzeni nazw chyba, że przetrzeń nazw jest null-em
		
		if( null == ( $res = $this->seek( $name , $nSpace ) ) ) {
			
			$res = $this->seek( $name );
			
			if( null == $res ) {
				
				$system = $this->system;
				
				if( is_array( $system ) ) {
					$system = implode( ',' , $system );
				}
				
				$this->notify(
					'phrase_not_found',
					array(
						'message' => "Brak frazy \"{$name}\". System \"{$system}\". Język \"{$this->lang}\" Przestrzeń nazw: \"{$nSpace}\"",
						'namespace' => '',
						'line' => ''
					)
				);
				
				$res = $name;
			}
		}
		
		return $res;
	}
	
	/**
	* Wyszukuje nazwy w przestrzeni nazw
	* @param $var string szukana fraza (text_id)
	* @param $namespace string/0 przestrzeń nazw jeśli 0 równoznaczne z szykaniem w domyślnej przestrzeni nazw
	* @return string/null
	*/
	protected function seek( $var , $namespace = 0) {
		$namespace = trim($namespace);
		$translatedPhrase = null;
		
		if( !array_key_exists( $namespace , $this->namespaces ) ) return null;
		
		$n = $this->namespaces[ $namespace ];
		
		foreach( $n as $valueKey => $textId) {
			if($textId == $var) {
				
				$translatedPhrase = $this->data[$valueKey];
			}
		}
		
		return $translatedPhrase;
	}
	
	
	/**
	 *
	 *
	 */
	public function translateArray( $arrayToTranslate , $namespace = 0 ) {
		
		if( !is_array( $arrayToTranslate ) ) {
			return array();
		}
		
		$result = array();
		foreach( $arrayToTranslate as $phrase) {
			
			$result[ $phrase ] = $this->translate( $phrase , $namespace );
		}
		
		return $result;
	}
	
	
	/**
	* Uruchamaia i wyłącza tryb debugowania w zależności od wartości przekazanego parametru
	* @param $state bool Parametr ustawiający true - wyłączający false tryb debugowania. Można niepodawaćtego parametru, wtedy obecna wartość zamieniona zostanie na przeciwną
	*/
	public function debug( $state = null ) {
		
		if( null !== $state ) {
			
			$this->debug = (bool) $state;
		} else {	
			
			$this->debug = !$this->debug;
		}
		
		return $this->debug;
	}
	
	
	/**
	* Alias do funkcji translate
	* @param $name string nazwa frazy
	* @return string/null
	*/
	public function __get($name) {
		return $this->translate($name);
	}
	
	/**
	* Ustawia przestrzeń nazw w, której będzie wyszukiwana fraza w razie nie znalezienia jej w ogolnym zbiorze...
	* @param $namespace string przestrzeń nazw
	*/
	public function setNamespace($namespace) {
		$this->namespace = $this->checkNamespace($namespace);
	}
	
	public function clearNamespace() {
		$this->namespace = 0;
	}
	
	/**
	* Zwraca przestrzeń nazw
	*/
	public function getNamespace() {
		return $this->namespace;
	}
	
	protected function checkNamespace($namespace) {
		if(isset($this->namespaces[$namespace]))
			return $namespace;
		return 0;
	}
}

?>