<?php

namespace System\User\FilterType;
use System\User\FilterType\AbstractFilterType;

class Column extends AbstractFilterType {
	
	protected $field = null;
	protected $operation = null;
	
	const OP_EQ = 'eq';
	const OP_LT = 'lt';
	const OP_GT = 'gt';
	const OP_LIKE = 'like';
	const OP_IN = 'in';
	const OP_NOTIN = 'not in';
	
	public function __construct( $field, $constraint, $operation ) {
	
		$this->field = $field;
		$this->operation = $operation;
		
		parent::__construct( $constraint );
	}
	
	public function getQuery() {
		$q = '';

		switch( $this->operation ) {
			case self::OP_EQ :
				
				if(is_numeric( $this->constraint )) {
					
					$q = "{$this->field} = {$this->constraint}";
					
				} else {
					
					$q = "{$this->field} ilike '{$this->constraint}'";
				}
				
				return $q;
			case self::OP_IN :
				$constraint = $this->constraint;
				
				if(is_array( $constraint )) {
					
					$constraint = implode( ',' , $constraint );
				}
				
				$constraint = explode( ',' , $constraint );
				
				$tmp = '';
				
				foreach( $constraint as $c ) {
					if(strlen($tmp)) {
						$tmp .= ',';
					}
					
					$tmp .= "'$c'";
				}
				
				$condition = $tmp;
				
				$q = "{$this->field} in ({$condition})";
								
				return $q;
			case self::OP_LIKE :
				
				$q = "{$this->field} ilike '%{$this->constraint}%'";
			
				return $q;
			case self::OP_GT :
				
				if( $this->field == 'start_date' ) {
					
					$q = "{$this->field} > (SELECT TIMESTAMP 'epoch' + {$this->constraint} * INTERVAL '1 second')";
					
				} else {
					
					$q = "{$this->field} > {$this->constraint}";
				}
				
				return $q;
			case self::OP_LT :
				
				if( $this->field == 'start_date' ) {
					
					$q = "{$this->field} < (SELECT TIMESTAMP 'epoch' + {$this->constraint} * INTERVAL '1 second')";
					
				} else {
					
					$q = "{$this->field} < {$this->constraint}";
				}
				
				return $q;
			case self::OP_NOTIN :
				
				$constraint = $this->constraint;
				
				if(is_array( $constraint )) {
					
					$constraint = implode( ',' , $constraint );
				}
				
				$constraint = explode( ',' , $constraint );
				
				$tmp = '';
				
				foreach( $constraint as $c ) {
					if(strlen($tmp)) {
						$tmp .= ',';
					}
					
					$tmp .= "'$c'";
				}
				
				$condition = $tmp;
				
				$q = "{$this->field} not in ({$condition})";
							
				return $q;
		}
		
		return null;
	}
}

?>