<?php

namespace System\Validator;

use \Zend\Validator\AbstractValidator;
use \Model\MainFile;

class File extends AbstractValidator {

    const FILE_EXISTS = 'fileExists';
    const FILE_ERROR = 'fileError';
    const FILE_NAME_ERROR = 'file_name_error';

    protected $messageTemplates = array(
        self::FILE_EXISTS => 'File with this name already exists',
        self::FILE_ERROR => 'Error while uploading file',
        self::FILE_NAME_ERROR => 'Incorrect file name - only alphanumeric characters and underscore are allowed',
    );

    public function isValid($value) {
        try {
            if ($value['error'] != 0) {
                $this->error(self::FILE_ERROR);
                return false;
            }

            if (!preg_match('/^[0-9a-z\_\.]+$/', $value['name'])) {
                $this->error(self::FILE_NAME_ERROR);
                return false;
            }

            if (!((bool) $this->getOption('allowOverride'))) {
                $MainFile = new MainFile();
                $file = $MainFile->getRowByFileName($value['name']);
                if (!empty($file)) {
                    $this->error(self::FILE_EXISTS);
                    return false;
                }
            }
        } catch (\Exception $e) {
            $this->error(self::FILE_ERROR);
            return false;
        }

        return true;
    }

}

?>