<?php
namespace System\View\Helper\DataGrid;

use Zend\View\Helper\AbstractHelper;

class DataGrid extends AbstractHelper {

    protected $view;
    protected $dataGrid = '';
    protected $dataGridFilter = '';
    protected $dataGridPaginator = '';
    protected $dataGridColumns = array();

    const COLUMN_DATA = 'COLUMN_DATA';
    const COLUMN_VIRTUAL = 'COLUMN_VIRTUAL';
    const COLUMN_MENU = 'COLUMN_MENU';

    public function __invoke() {
        $this->view = $this->getView();
        return $this;
    }

    public function setFilter(\System\View\Helper\DataGrid\DataGridFilter $dataGridFilter) {
        $this->dataGridFilter = $dataGridFilter;
        return $this;
    }

    public function getFilter() {
        return $this->dataGridFilter;
    }

    public function addColumn(array $column) {
        $dataGridColumn = '';

        switch ($column['type']) {
            case self::COLUMN_DATA:
                if (isset($this->view->request['order'][$column['name']])) {
                    $dataGridColumnOrder = ' order="' . (strtolower($this->view->request['order'][$column['name']]) == 'desc' ? 'd' : 'u') . '"';
                } else {
                    $dataGridColumnOrder = '';
                }

                $dataGridColumn .= '<th class="table-header-repeat no_mark line-left">';
                $dataGridColumn .= '<a class="esculapTableOrder" name="' . $column['name'] . '"' . $dataGridColumnOrder . '>';
                $dataGridColumn .= $column['label'] . '</a></th>';
                break;
            case self::COLUMN_VIRTUAL:
                $dataGridColumn .= '<th class="table-header-repeat no_mark line-left"><span>';
                $dataGridColumn .= $column['label'] . '</span></th>';
                break;
            case self::COLUMN_MENU:
                $dataGridColumn .= '<th width="120" class="table-header-repeat no_mark line-left">';
                $dataGridColumn .= '<a class="no_arrow esculapTableFilterReset resetFiltersUnavailable">';
                $dataGridColumn .= $this->view->translator->translate('ok_map_reset_filter') . '</a></th>';
                break;
        }

        $this->dataGridColumns[] = $dataGridColumn;

        return $this;
    }

    public function generate() {
        $this->dataGrid .= '<div class="esculapTable" data-url="' . $this->view->listUrl . '">';
        $this->dataGrid .= $this->dataGridFilter->generate();
        $this->dataGrid .= '<div class="table-content">';

        $this->dataGrid .= '<table width="100%" cellpadding="0" cellspacing="0" id="product-table"'
                . ($this->view->relationalSelectFieldName ? 'data-relational-select-fieldname="' . $this->view->relationalSelectFieldName . '"' : '')
                . ($this->view->relationalSelect ? 'class="relational_select_table"' : '')
                . '>';
        $this->dataGrid .= '<tr class="esculapTableHeader">' . implode(null, $this->dataGridColumns) . '</tr>';
        $this->dataGrid .= $this->view->list;
        $this->dataGrid .= '</table>';

        $this->dataGrid .= $this->view->dataGridPaginator()->generate();
        $this->dataGrid .= '</div></div>';

        return $this->dataGrid;
    }

}
