<?php
namespace System\View\Helper\DataGrid;

use Zend\View\Helper\AbstractHelper;

class DataGridFilter extends AbstractHelper {

    const FIELD_BOOLEAN = 'FIELD_BOOLEAN';
    const FIELD_INTEGER = 'FIELD_INTEGER';
    const FIELD_NUMERIC = 'FIELD_NUMERIC';
    const FIELD_TEXT = 'FIELD_TEXT';
    const FIELD_DATE = 'FIELD_DATE';
    const FIELD_INTEGER_RANGE = 'FIELD_INTEGER_RANGE';
    const FIELD_NUMERIC_RANGE = 'FIELD_FLOAT_RANGE';
    const FIELD_DATE_RANGE = 'FIELD_DATE_RANGE';
    const FIELD_SELECT = 'FIELD_SELECT';
    const FIELD_MULTISELECT = 'FIELD_MULTISELECT';

    protected $view;
    protected $dataGridFilter = '';
    protected $dataGridFilterFields = array();

    public function __invoke() {
        $this->view = $this->getView();
        return $this;
    }

    public function addField(array $field) {
        $dataGridFilterField = '';
        $dataGridFilterFieldValue = isset($this->view->request['whereView'][$field['name']]) ? $this->view->request['whereView'][$field['name']] : '';

        $dataGridFilterFieldValueMin = isset($this->view->request['whereView'][$field['name'] . '_min']) ? $this->view->request['whereView'][$field['name'] . '_min'] : '';
        $dataGridFilterFieldValueMax = isset($this->view->request['whereView'][$field['name'] . '_max']) ? $this->view->request['whereView'][$field['name'] . '_max'] : '';

        switch ($field['type']) {
            case self::FIELD_BOOLEAN:
                if (!isset($field['label_true'])) {
                    $field['label_true'] = $this->view->translator->translate('yes');
                }
                if (!isset($field['label_false'])) {
                    $field['label_false'] = $this->view->translator->translate('no');
                }
                $dataGridFilterField .= '<div class="esculapTableSerchBrick esculapTableSerchBrick-medium-width esculap-filter-brick">';
                $dataGridFilterField .= '<div class="header">' . $field['label'] . '</div>';
                $dataGridFilterField .= '<select name="' . $field['name'] . '" class="esculap-boolean-filter">';
                $dataGridFilterField .= '<option value=""></option>';
                $dataGridFilterField .= '<option value="true"' . ($dataGridFilterFieldValue == 'true' ? ' selected="selected"' : '') . '>' . $field['label_true'] . '</option>';
                $dataGridFilterField .= '<option value="false"' . ($dataGridFilterFieldValue == 'false' ? ' selected="selected"' : '') . '>' . $field['label_false'] . '</option>';
                $dataGridFilterField .= '</select>';
                $dataGridFilterField .= '</div>';
                break;
            case self::FIELD_INTEGER:
                $dataGridFilterField .= '<div class="esculapTableSerchBrick esculapTableSerchBrick-medium-width esculap-filter-brick">';
                $dataGridFilterField .= '<div class="header">' . $field['label'] . '</div>';
                $dataGridFilterField .= '<input name="' . $field['name'] . '" class="inp-form esculap-integer-filter" type="text" value="' . $dataGridFilterFieldValue . '">';
                $dataGridFilterField .= '</div>';
                break;
            case self::FIELD_NUMERIC:
                $dataGridFilterField .= '<div class="esculapTableSerchBrick esculapTableSerchBrick-medium-width esculap-filter-brick">';
                $dataGridFilterField .= '<div class="header">' . $field['label'] . '</div>';
                $dataGridFilterField .= '<input name="' . $field['name'] . '" class="inp-form esculap-numeric-filter" type="text" value="' . $dataGridFilterFieldValue . '">';
                $dataGridFilterField .= '</div>';
                break;
            case self::FIELD_TEXT:
                $dataGridFilterField .= '<div class="esculapTableSerchBrick esculapTableSerchBrick-medium-width esculap-filter-brick">';
                $dataGridFilterField .= '<div class="header">' . $field['label'] . '</div>';
                $dataGridFilterField .= '<input name="' . $field['name'] . '" class="inp-form" type="text" value="' . $dataGridFilterFieldValue . '">';
                $dataGridFilterField .= '</div>';
                break;
            case self::FIELD_SELECT:
                $dataGridFilterField .= '<div class="esculapTableSerchBrick esculapTableSerchBrick-medium-width esculap-filter-brick">';
                $dataGridFilterField .= '<div class="header">' . $field['label'] . '</div>';
                $dataGridFilterField .= '<select name="' . $field['name'] . '" class="esculap-boolean-filter">';
                $dataGridFilterField .= '<option value=""></option>';
                foreach ($field['options'] as $option) {
                    $dataGridFilterField .= "<option value=\"{$option['value']}\"" . ($dataGridFilterFieldValue == $option['value'] ? ' selected="selected"' : '') . '>' . $option['label'] . '</option>';
                }
                $dataGridFilterField .= '</select>';
                $dataGridFilterField .= '</div>';
                break;
            case self::FIELD_MULTISELECT:
                $dataGridFilterFieldValues = explode(',', $dataGridFilterFieldValue);
                $dataGridFilterField .= '<div class="esculapTableSerchBrick esculapTableSerchBrick-medium-width esculap-filter-brick">';
                $dataGridFilterField .= '<div class="header">' . $field['label'] . '</div>';
                $dataGridFilterField .= '<select multiple name="' . $field['name'] . '" class="esculap-boolean-filter">';
                $dataGridFilterField .= '<option value=""></option>';
                foreach ($field['options'] as $option) {
                    $selected = false;
                    foreach ($dataGridFilterFieldValues as $key => $dataGridFilterFieldValue) {
                        if ($dataGridFilterFieldValue == $option['value']) {
                            $selected = true;
                            unset($dataGridFilterFieldValues[$key]);
                        }
                    }
                    
                    $dataGridFilterField .= "<option value=\"{$option['value']}\"" . ($selected ? ' selected="selected"' : '') . '>' . $option['label'] . '</option>';
                }
                $dataGridFilterField .= '</select>';
                $dataGridFilterField .= '</div>';
                break;
            case self::FIELD_DATE: //TODO
                $dataGridFilterField .= '';
                break;
            case self::FIELD_INTEGER_RANGE:
                $dataGridFilterField .= '<div class="esculapTableSerchBrick esculapTableSerchBrick-medium-width esculap-filter-brick">';
                $dataGridFilterField .= '<div class="header">' . $field['label'] . ' ' . $this->view->translator->translate('od', 'DataGridFilter') . '</div>';
                $dataGridFilterField .= '<input name="' . $field['name'] . '_min" class="inp-form esculap-integer-filter" type="text" value="' . $dataGridFilterFieldValueMin . '">';
                $dataGridFilterField .= '</div>';
                $dataGridFilterField .= '<div class="esculapTableSerchBrick esculapTableSerchBrick-medium-width esculap-filter-brick">';
                $dataGridFilterField .= '<div class="header">' . $field['label'] . ' ' . $this->view->translator->translate('do', 'DataGridFilter') . '</div>';
                $dataGridFilterField .= '<input name="' . $field['name'] . '_max" class="inp-form esculap-integer-filter" type="text" value="' . $dataGridFilterFieldValueMax . '">';
                $dataGridFilterField .= '</div>';
                break;
            case self::FIELD_NUMERIC_RANGE: //TODO
                $dataGridFilterField .= '';
                break;
            case self::FIELD_DATE_RANGE:
                $dataGridFilterField .= '<div class="esculapTableSerchBrick esculapTableSerchBrick-medium-width esculap-filter-brick">';
                $dataGridFilterField .= '<div class="header">' . $field['label'] . ' ' . $this->view->translator->translate('od', 'DataGridFilter') . '</div>';
                $dataGridFilterField .= '<input readonly name="' . $field['name'] . '_min" class="inp-form esculap-start-date-filter date-input" type="text" value="' . $dataGridFilterFieldValueMin . '">';
                $dataGridFilterField .= '</div>';
                $dataGridFilterField .= '<div class="esculapTableSerchBrick esculapTableSerchBrick-medium-width esculap-filter-brick">';
                $dataGridFilterField .= '<div class="header">' . $field['label'] . ' ' . $this->view->translator->translate('do', 'DataGridFilter') . '</div>';
                $dataGridFilterField .= '<input readonly name="' . $field['name'] . '_max" class="inp-form esculap-end-date-filter date-input" type="text" value="' . $dataGridFilterFieldValueMax . '">';
                $dataGridFilterField .= '</div>';
                break;
        }

        $this->dataGridFilterFields[] = $dataGridFilterField;

        return $this;
    }

    public function generate() {
        if (!empty($this->dataGridFilterFields)) {
            $this->dataGridFilter .= '<form class="esculapSerchTable" method="post"><div class="esculapTableSerchBricksPlace">';
            $this->dataGridFilter .= implode(null, $this->dataGridFilterFields);
            $this->dataGridFilter .= '<div class="esculapTableSerchBrick esculapTableSerchBrickSubmit">';
            $this->dataGridFilter .= '<input type="submit" class="form-submit submitStep" value="' . $this->view->translator->translate('main_search') . '">';
            $this->dataGridFilter .= '</div></div></form>';
        }

        return $this->dataGridFilter;
    }

}
