<?php
namespace System\View\Helper\DataGrid;

use Zend\View\Helper\AbstractHelper;

class DataGridList extends AbstractHelper
{

    const OPTION_VIEW = 'OPTION_VIEW';
    const OPTION_EDIT = 'OPTION_EDIT';
    const OPTION_DELETE = 'OPTION_DELETE';
    const OPTION_CUSTOM = 'OPTION_CUSTOM';

    protected $view;
    protected $dataGridList = '';
    protected $dataGridListOptions = array();
    protected $dataGridListData;

    public function __invoke()
    {
        $this->view = $this->getView();

        return $this;
    }

    public function setDataGridListData($dataGridListData)
    {
        $this->dataGridListData = $dataGridListData;

        return $this;
    }

    public function getDataGridListData()
    {
        return $this->dataGridListData;
    }

    public function addOption(array $option)
    {
        $dataGridListOption = '';

        if ((isset($option['acl']) && $this->view->acl->isAllowed($this->view->identity['username'], $option['acl'])) || !isset($option['acl'])) {
            $dataGridListOptionPermission = true;
        } else {
            $dataGridListOptionPermission = false;
        }

        $relationalLink = '';
        if (isset($option['relational']) && $option['relational']) {
            $relationalLink = ' relational-view';
        }

        $selfRelationalLink = '';
        if (isset($option['self-relational']) && $option['self-relational']) {
            $selfRelationalLink = ' self-relational-view';
        }

        $forceConfirm = '';
        if (isset($option['force-confirm']) && $option['force-confirm']) {
            $forceConfirm = ' force-confirm';
        }

        switch ($option['type']) {
            case self::OPTION_VIEW:
                if ($dataGridListOptionPermission) {
                    if (!isset($option['label']) || !$option['label']) {
                        $option['label'] = $this->view->translator->translate('view');
                    }
                    $dataGridListOption .= '<li><a class="view_button action-edit' . $relationalLink . $selfRelationalLink . $forceConfirm . '" href="' . $option['url'] . '/id/[:id]" data-id="[:id]">' . $option['label'] . '</a></li>';
                }
                break;
            case self::OPTION_EDIT:
                if ($dataGridListOptionPermission) {
                    if (!isset($option['label']) || !$option['label']) {
                        $option['label'] = $this->view->translator->translate('edit');
                    }
                    $dataGridListOption .= '<li><a class="view_button action-edit' . $relationalLink . $selfRelationalLink . $forceConfirm . '" href="' . $option['url'] . '/id/[:id]" data-id="[:id]">' . $option['label'] . '</a></li>';
                }
                break;
            case self::OPTION_DELETE:
                if ($dataGridListOptionPermission) {
                    if (!isset($option['label']) || !$option['label']) {
                        $option['label'] = $this->view->translator->translate('delete');
                    }
                    $dataGridListOption .= '<li><a class="view_button action-delete' . $relationalLink . $selfRelationalLink . $forceConfirm . '" href="' . $option['url'] . '/id/[:id]" data-id="[:id]">' . $option['label'] . '</a></li>';
                }
                break;
            case self::OPTION_CUSTOM:
                if ($dataGridListOptionPermission) {
                    $dataGridListOption .= '<li><a class="view_button action-edit' . $relationalLink . $selfRelationalLink . $forceConfirm . '" href="' . $option['url'] . '/id/[:id]" data-id="[:id]">' . $option['label'] . '</a></li>';
                }
                break;
        }

        if ($dataGridListOption) {
            $this->dataGridColumnCount++;
        }

        $this->dataGridListOptions[] = $dataGridListOption;

        return $this;
    }

    public function generate()
    {
        $this->dataGridList .= '<tr class="dataRow esculapTableNavigationOptions esculapTablePagination" options="page:' . $this->view->request['page'] . ',pages:' . $this->view->request['pages'] . '"></tr>';
        if (count($this->dataGridListData) <= 0) {
            $this->dataGridList .= '<tr class="dataRow" class="no_results"><td colspan="100" class="no_results">' . $this->view->translator->translate('main_no_results') . '</td></tr>';
        } else {
            foreach ($this->dataGridListData as $dataGridListData) {
                $this->dataGridList .= '<tr class="dataRow">';
                $this->dataGridList .= $dataGridListData[0];

                if (count($this->dataGridListOptions) > 0) {
                    $this->dataGridList .= '<td class="options_column">';
                    $this->dataGridList .= '<p class="options_menu_button">' . $this->view->translator->translate('options') . '</p><ul class="options_menu">';

                    array_shift($dataGridListData);
                    foreach ($dataGridListData as $field => $data) {
                        $this->dataGridList .= str_replace($field, $data, implode(null, $this->dataGridListOptions));
                    }

                    $this->dataGridList .= '</ul>';
                    $this->dataGridList .= '</td>';
                }

                $this->dataGridList .= '</tr>';
            }
        }

        return $this->dataGridList;
    }

}
