<?php

namespace System\View\Helper;
use Zend\View\Helper\AbstractHelper;
use System\Form\DrawableInterface;

class DrawForm extends AbstractHelper {
	
	protected $view;
	
	public function __invoke(DrawableInterface $form, $view, $style = 'EsculapStandart') {
		$this->view = $view;
		
		switch($style) {
			case 'EsculapStandart':
			default :
				
				return $this->drawEsculapStandart($form);
			break;
			case 'CmsStandard':
			
				return $this->drawCmsStandard($form);
			break;
			case 'CmsPartial':
				
				return $this->drawCmsPartial( $form );
			break;
		}
	}
	
	/**
	* Rysuje formularz w formie standardowej dla esculapa
	* @param System\Form\DrawableInterface $formWrap obiekt klasy System\Form\DrawableInterface
	* @returns string
	*/
	protected function drawEsculapStandart($formWrap) {
		$form = $formWrap->draw();
		$form->prepare();
		
		$queue = $form->queue;
		
		// otwieram znacznik <form>
		$formTemplate = $this->view->form()->openTag($form).'<fieldset class="form4">';
		
		// licznik kontrolek zastosowany aby określić ostatnią 
		$current = 0;
		foreach($queue as $element) {
			$current++;
			
			$wrap = true;
			$element = $form->get($element);
			$elementAsHtml = $this->displayElement($element,$element->getLabel(),$wrap);
			$errors = $this->view->formElementErrors($element);
			
			if($wrap) {
				
				$errors = $this->view->formElementErrors($element);
				
				$formTemplate .= "<div class=\"".(count($queue) == $current ? 'form4-sbt' : '')."".(strlen($errors) ? 'error' : '')."\">
						
						".(strlen($errors) ? "<span style=\"color:red;font-weight:bold;\">{$this->view->formElementErrors($element)}</span>" : '' )."
						
						<label>{$element->getLabel()}</label>
						{$elementAsHtml}
				
				</div>";
			} else {
				
				$formTemplate .= $elementAsHtml;
			}
		}
		
		// zamykam <form>
		$formTemplate .= '</fieldset>'.$this->view->form()->closeTag();
		
		return $formTemplate;
	}
	
	protected function displayElement($element,$label = null,&$wrap = null) {
		
		$key = get_class($element);
		
		$wrap = true;
		
		switch($key) {
			case 'Zend\Form\Element\Text':
			case 'Zend\Form\Element':
				
				return $this->view->formTextEx($element);
					
			case 'Zend\Form\Element\Textarea' :
			
				return $this->view->formTextareaEx($element);
			
			case 'System\Form\Element\Select' :
				
				$wrap = false;
				$r = $this->view->formSystemSelect($element,$label,$this->view->formElementErrors($element));
				
				return $r;
				
				
			case 'Zend\Form\Element\Select':
				
				return $this->view->FormSelectEx($element);
                        case 'System\Form\Element\SelectRelationalVisible':
                                return $this->view->FormSelectRelationalVisible($element);
                            break;
                        case 'System\Form\Element\SelectRelationalHidden':
                                return $this->view->FormSelectRelationalHidden($element);
                            break;
                        case 'System\Form\Element\ImageView':
                            return $this->view->FormImageView($element);
                        break;
                        case 'System\Form\Element\ImageChange':
                            return $this->view->FormImageChange($element);
                        break;
                        case 'System\Form\Element\Code':
                            return $this->view->FormCode($element);
                        break;
                        case 'System\Form\Element\ContentViews':
                            return $this->view->FormContentViews($element);
                        break;
                        case 'System\Form\Element\BannerPlacePreview':
                            return $this->view->FormBannerPlacePreview($element);
                        break;
			case 'Zend\Form\Element\Submit':
				
				return $this->view->formSubmit($element);
				
			case 'Zend\Form\Element\Checkbox':
			
				return $this->view->FormCheckboxEx($element);
			case 'Zend\Form\Element\Button':
			case 'Zend\Form\Element\File':
                            if (isset($element->getValue()['name'])) {
                                $element->setValue($element->getValue()['name']);
                            }
                            return $this->view->FormInputEx($element);
			case 'Zend\Form\Element\Image':
			
				return $this->view->FormInputEx($element);
			case 'Zend\Form\Element\Hidden':
				
				return $this->view->FormHiddenEx($element);
			case 'System\Form\Element\QuizAnswers':
				
				$html = "<div class=\"guiz_answers_control icon_toolbar\"><div><button class=\"answer_add\"></button></div>
				<div class=\"quiz_question_hidden\">
					
					<span class=\"value\">asd</span>
						
					<span class=\"buttons\">
						<button class=\"move_up\"></button>
						<button class=\"move_down\"></button>
						<button class=\"delete\"></button>
					</span>	
				</div>
				";
					
				$options = $element->getValueOptions();
				
				foreach( $options as $option ) {
					$link = '';
                                        foreach ($this->view->links as $links) {
                                            if ($links['value'] == $option[ 'label' ]) {
                                                $link = $links['link'];
                                            }
                                        }
                                        
					$html .= "<div class=\"quiz_question\">
						
						<span class=\"value\">{$option[ 'label' ]}</span>
                                                    <span class=\"buttons\">
							<button class=\"move_up\"></button>
							<button class=\"move_down\"></button>
							<button class=\"delete\"></button>
						</span>
                                                <span style=\"display: block;\">{$link}</span>
						<input type=\"hidden\" name=\"{$element->getName()}[]\" value=\"{$option[ 'label' ]}\">
						<input type=\"hidden\" name=\"{$element->getName()}_value[]\" value=\"{$option[ 'value' ]}\">
						
					</div>";
				}
					
				$html .= "</div>";
				return $html;
			}
	}
	
	protected function drawCmsStandard( $form ) {
		$form = $form->draw();
		$form->prepare();
		
		$queue = $form->queue;
		
		// otwieram znacznik <form>
		$formTemplate = $this->view->form()->openTag($form).'<div class="esculapForm"><div class="stepContent stepContentVisible"><table id="id-form">';
		
		foreach($queue as $element) {
			$element = $form->get($element);
			$errors = $this->view->formElementErrors($element);
			
			if(strlen($errors) > 0) {
				$element->setAttribute('class',$element->getAttribute('class').'-error');
			}

            $class = preg_match( '/no_height/' , $element->getAttribute( 'class' ) ) ? 'no_height' : '';

                if ($class) {
                    $formTemplate .= "<tr class=\"{$class}\" style=\"padding: 0;\"><th class=\"{$class}\" style=\"padding: 0;\">";
                } else {
                    $formTemplate .= "<tr><th>";
                }
            $formTemplate .="{$element->getLabel()}</th>";
            if ($class) {
                $formTemplate .= "<td class=\"{$class}\" style=\"padding: 0;\">";
            } else {
                $formTemplate .= "<td>";
            }
            $formTemplate .= "
						{$this->displayElement($element)}
					</td>";
            if ($class) {
                $formTemplate .= "<td class=\"{$class}\" style=\"padding: 0;\">";
            } else {
                $formTemplate .= "<td>";
            }
            $formTemplate .= "
						<div class=\"".(count($errors) ? 'error_visible tooltip-error' : '')." error\">{$errors}</div>
					</td>
				</td>
			";
		}
		
		$formTemplate .= '</table></div></div>'.$this->view->form()->closeTag();
		
		return $formTemplate;
	}
	
	
	protected function drawCmsPartial( $form ) {
		$FormWrapperObj = $form;
		$form = $form->draw();
		$form->prepare();
		
		$queue = $form->queue;
		
		$formTemplate = $this->view->form()->openTag( $form );
		$formTemplate .= "
			<div class=\"esculapForm\">
				<div class=\"esculapFormProggressBar\">  
		";
		
		/**
		 * Pobieram listę wszystkich kroków
		 */
		foreach( $form->queue as $num => $step ) {
			if( isset( $step[ 'lock' ] ) && 0 == $step[ 'lock' ] ) {
				$stepUnlocked = 1;
			} else {
				$stepUnlocked = 0;
			}
			
			$name = isset( $step[ 'name' ] ) ? $step[ 'name' ] : 'unnamed';
			
			$formTemplate .= "
				
					<div id=\"{$num}\" class=\"step " . ( $stepUnlocked ? 'stepUnlocked' : '' ) . "\">
						<div class=\"stepNo\">{$num}</div>
						<div class=\"stepBgLeft\"></div>
						<div class=\"stepName\">{$name}</div>
						<div class=\"stepBgRight\"></div>
					</div>
			";
		}
		$formTemplate .= "</div>";
		
		/**
		 * Wyciągam pola formularza dla poszczególnych kroków 
		 */
		$v = 0;
		foreach( $form->queue as $num => $step ) {
			
			$formTemplate .= "<div class=\"stepContent content{$num} " . ( $v == 0 ? 'stepContentVisible' : '' ) . "\"><table width=\"100%\">";
			
			foreach( $step[ 'elements' ] as $elementName ) {
				$element = $form->get( $elementName );
                                
                                /*if ($element->getName() == 'pointed') {
                                    var_dump($element); die();
                                }*/
				
				$class = preg_match( '/no_height/' , $element->getAttribute( 'class' ) ) ? 'no_height' : '';
				
				$rowspan = '';
				if( $element->hasAttribute( 'rowspan' ) ) {
					$rowspan = 'rowspan="' . (int)$element->getAttribute( 'rowspan' ) . '"';
				}
				
				$formTemplate .= "
					<tr>
						<th {$rowspan} width=\"150\">
							{$element->getLabel()}
						</th>
						<td {$rowspan} class=\"{$class}\">
							{$this->displayElement( $element )}
						</td>
						<td {$rowspan} class=\"{$class}\">
							<div ". ( count( $element->getMessages() ) ? 'style="display:block;"' : '' ) ." class=\"error tooltip-error\">";
							
							if( count( $element->getMessages() ) > 0 )  {
								
								$errors = $element->getMessages();
								
							} else {
								
								$errors = $FormWrapperObj->getElementMessages( $element->getName() );
							}

							$formTemplate .= '<div class="short"><div class="errors_border" title="';
							
							$i = count( $errors );
							foreach( $errors as $message ) {
								
								$formTemplate .= "- {$message}";
								
								if( --$i > 0 ) {
									$formTemplate .= ';';
								}
							}
							
							$formTemplate .= "\"></div></div></div>
							
						</td>
					</tr>
				";
				
			}
			
			if( isset( $step[ 'navigation' ] ) ) {
				$navigation = $step[ 'navigation' ];
				$formTemplate .= "<tr><td class=\"form_navigarion_bar\" colspan=\"20\" align=\"center\">";
				
				if( is_array( $navigation ) ) {
					
					foreach( $navigation as $name ) {
						if( !is_string( $name ) ) continue; 
						
						$formTemplate .= $this->displayElement( $form->get( $name ) );
					}
					
				} elseif( is_string( $navigation ) )  {
					if( is_string( $navigation ) )
						$formTemplate .= $this->displayElement( $form->get( $navigation ) );
					
				}
				
				$formTemplate .= "</td></tr>";
			}
			
			
			$v++;
			$formTemplate .= "</table></div>";
		}
		
		$formTemplate .= "
			</div>
		";
		$formTemplate .= $this->view->form()->closeTag( $form );
		return $formTemplate;
	}
	
}

?>