<?php
namespace System\View\Helper\Form;

use System\Translator;
use Zend\Form\View\Helper\FormFile;
use Zend\Form\ElementInterface;

class FormBannerPlacePreview extends FormFile {

    protected $validSelectAttributes = array(
        'name' => true,
        'autofocus' => true,
        'disabled' => true,
        'form' => true,
        'required' => true,
        'size' => true,
        'req' => true,
        'val' => true,
    );

    public function render(ElementInterface $element) {
        $name = $element->getName();
        if (empty($name) && $name !== 0) {
            throw new Exception\DomainException(sprintf(
                    '%s requires that the element has an assigned name; none discovered', __METHOD__
            ));
        }

        $translator = Translator\TranslatorEx::get();
        $changeImageText = $translator->translate('upload_new_preview_image');
        $hideImageText = $translator->translate('dont_upload_new_preview_image');
        $noImageText = $translator->translate('no_image_uploaded');

        $attributes = $element->getAttributes();
        $value = $element->getValue();

        $field = '';
        $selectedChoiceClass = '';
        if (isset($value[0])) {
            $value = trim($value[0]);
        } elseif (is_array($value)) {
            $selectedChoiceClass = ' new_selected_switch';
        } elseif (isset($attributes['image_name']) && $attributes['image_name']) {
            $value = $attributes['image_name'];
        }
        
        if (isset($attributes['view']) && $attributes['view']) {
            if (isset($attributes['image_name']) && $attributes['image_name']) {
                $field = '<img style="border: 1px solid #ececec; margin: 0; border-radius: 4px;" src="images/banner/' . $attributes['image_name'] . '.png" alt="' . $name . '">';
            } else {
                $field = '<input type="text" disabled="disabled" class="inp-form" value="' . $noImageText . '">';
            }
        } else {
            $attributes['name'] = $name;
            $field .= '<select name="' . $name . '" class="banner_place_preview_select">';
            foreach ($attributes['select'] as $option) {
                $option = trim($option);
                if ($option == $value) {
                    $field .= '<option selected="selected" value="' . $option . '">' . $option . '</option>';
                } else {
                    $field .= '<option value="' . $option . '">' . $option . '</option>';
                }
            }

            $field .= '</select>
                  <a href="#" class="change_image_switch banner_preview' . $selectedChoiceClass . '" data-file-input="' . $name . '" 
                      data-show-label="' . $changeImageText . '"
                      data-hide-label="' . $hideImageText . '">' . $changeImageText . '</a>';

            $attributes = $element->getAttributes();
            $attributes['type'] = $this->getType($element);
            $attributes['name'] = $name;
            if (array_key_exists('multiple', $attributes) && $attributes['multiple']) {
                $attributes['name'] .= '[]';
            }

            $value = $element->getValue();
            if (is_array($value) && isset($value['name']) && !is_array($value['name'])) {
                $attributes['value'] = $value['name'];
            } elseif (is_string($value)) {
                $attributes['value'] = $value;
            }

            $field .= sprintf(
                    '<input style="display: none;" %s%s', $this->createAttributesString($attributes), $this->getInlineClosingBracket()
            );
        }

        return $field;
    }

}

?>