<?php
namespace System\View\Helper\Form;

use System\Translator;
use Zend\Form\View\Helper\FormFile;
use Zend\Form\ElementInterface;

class FormImageChange extends FormFile {

    protected $validSelectAttributes = array(
        'name' => true,
        'autofocus' => true,
        'disabled' => true,
        'form' => true,
        'required' => true,
        'size' => true,
        'req' => true,
        'val' => true,
    );

    public function render(ElementInterface $element) {
        $name = $element->getName();
        if (empty($name) && $name !== 0) {
            throw new Exception\DomainException(sprintf(
                    '%s requires that the element has an assigned name; none discovered', __METHOD__
            ));
        }

        $translator = Translator\TranslatorEx::get();
        $changeImageText = $translator->translate('change_image');
        $hideImageText = $translator->translate('change_hide_image');

        $attributes = $element->getAttributes();
        $attributes['name'] = $name;
        $field = '<img style="border: 1px solid #ececec; margin: 0; border-radius: 4px;" src="http://esculap.com/image/' . $attributes['image_name'] . '" alt="' . $name . '">
                  <a href="#" class="change_image_switch" data-file-input="' . $name . '" 
                      data-show-label="' . $changeImageText . '"
                      data-hide-label="' . $hideImageText . '">' . $changeImageText . '</a>';

        $attributes = $element->getAttributes();
        $attributes['type'] = $this->getType($element);
        $attributes['name'] = $name;
        if (array_key_exists('multiple', $attributes) && $attributes['multiple']) {
            $attributes['name'] .= '[]';
        }

        $value = $element->getValue();
        if (is_array($value) && isset($value['name']) && !is_array($value['name'])) {
            $attributes['value'] = $value['name'];
        } elseif (is_string($value)) {
            $attributes['value'] = $value;
        }

        if (isset($attributes['image_name']) && $attributes['image_name']) {
            $field .= sprintf(
                    '<input style="display: none;" %s%s', $this->createAttributesString($attributes), $this->getInlineClosingBracket()
            );
        } else {
            $field = sprintf(
                    '<input %s%s', $this->createAttributesString($attributes), $this->getInlineClosingBracket()
            );
        }

        return $field;
    }

}

?>