<?php
namespace System\View\Helper;

use Zend\View\Helper\AbstractHelper;
use Zend\Mvc\Controller\Plugin\FlashMessenger;

class Msg extends AbstractHelper {

    protected $flashMessageNamespaces = array(
        FlashMessenger::NAMESPACE_DEFAULT,
        FlashMessenger::NAMESPACE_INFO,
        FlashMessenger::NAMESPACE_SUCCESS,
        FlashMessenger::NAMESPACE_ERROR,
    );

    public function __invoke($msg, $type = 'notice') {
        if (is_object($msg)) {
            $messages = array();

            foreach ($this->flashMessageNamespaces as $flashMessageNamespace) {
                $msg->setNamespace($flashMessageNamespace);

                if ($msg->hasMessages() || $msg->hasCurrentMessages()) {
                    foreach (array_merge($msg->getMessages(), $msg->getCurrentMessages()) as $message) {
                        $messages[] = $this->generateMessage($message, $flashMessageNamespace);
                    }
                    $msg->clearMessages();
                    $msg->clearCurrentMessages();
                }
            }

            return $messages;
        } else {
            return $this->generateMessage($msg, $type);
        }
    }

    public function generateMessage($msg, $type = 'notice') {
        $base = "<div style=\"font-weight:bold;padding:3px;__style__\" class=\"flash-message\">
				<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\">
					<tbody><tr><td>___msg___<button type=\"button\" class=\"flash-message-close\"></button></td></tr></tbody>
				</table>
			</div>";

        switch ($type) {
            case 'notice':
            case FlashMessenger::NAMESPACE_DEFAULT:
                $c = 'background-color:#FFF4CC;border solid 1px #FAEAC1;color:#E79300;';
                break;
            case 'message':
            case FlashMessenger::NAMESPACE_INFO:
                $c = 'background-color:#E4EDF5;border solid 1px #DCE5ED;color:#3974B2;';
                break;
            case 'ok':
            case FlashMessenger::NAMESPACE_SUCCESS:
                $c = 'background-color:#E0F2CB;border solid 1px #CEECAF;color:#6DA85E;';
                break;
            case 'error':
            case FlashMessenger::NAMESPACE_ERROR:
                $c = 'background-color:#FBD3B1;border solid 1px #FAE2CC;color:#CE271D;';
                break;
            default:
                $c = 'background-color:#FFF4CC;border solid 1px #FAEAC1;color:#E79300;';
        }

        $base = preg_replace('/__style__/', $c, $base);
        $base = preg_replace('/___msg___/', $msg, $base);

        return $base;
    }

}

?>