<?php
// session.inc.php
//
// klasa do obslugi sesji z przechowywaniem w bazie
// by Cegedim Group Poland (mz) XII.2007
//
// aktualizacja 30.10.2008 by mz
// aktualizacja 20.11.2008 by mz - dodane quote w metodzie write, poprawienie session_data
//


//require_once("library.inc.php");

interface http_session {
	public function open($save_path, $session_name);
	public function close();
	public function read($id);
	public function write($id, $data);
	public function destroy($id);
	public function gc($maxlifetime);
}

class Session implements http_session
{
	public $db_config;
	public $session_table = "session_data";
	public $session_idle_time;
	public $session_dead;
	public $session_lifetime;	// w minutach
	public $session_id;
	protected $crc = false;
	private $session_name = null;
	private $client_ip;
	private $server_ip;
	private $host;
    private $dbh;
    
	public function __construct($dbh, $lifetime = 3600, $session_name = null)
	{
		$this->dbh = $dbh;
		ini_set('session.save_handler', 'user');
		
		$this->client_ip = $_SERVER['REMOTE_ADDR'];
		$this->server_ip = $_SERVER['SERVER_ADDR'];
		$host = $_SERVER['HTTP_HOST'];
		if(strpos($host, 'rem-cms.esculap.pl')!==false) $host = 'rem-cms.esculap.pl';
		elseif(strpos($host, 'cms.esculap.pl')!==false) $host = 'cms.esculap.pl';
		elseif(strpos($host, 'tst.esculap.pl')!==false) $host = 'tst.esculap.pl';
		elseif(strpos($host, 'dev3.esculap.pl')!==false) $host = 'dev3.esculap.pl';
		elseif(strpos($host, 'www.esculap.pl')!==false) $host = 'esculap.pl';
		elseif(strpos($host, 'demo.esculap.com')!==false) $host = '.demo.esculap.com';
		elseif(strpos($host, 'tst.esculap.com')!==false) $host = '.tst.esculap.com';
		$this->host = $host;
		$this->session_lifetime = $lifetime;
		session_set_cookie_params(0, '/', $this->host);
		session_set_save_handler(array(&$this, 'open'),
								array(&$this, 'close'),
								array(&$this, 'read'),
								array(&$this, 'write'),
								array(&$this, 'destroy'),
								array(&$this, 'gc'));
		register_shutdown_function('session_write_close');
		if($session_name != null)
			session_name($session_name);
		elseif($host == 'tst.esculap.pl')
			session_name('TSTSESSID');
		elseif($host == 'dev3.esculap.pl')
			session_name('DEVSESSID');
		$this->session_name = session_name();
		if(isset($_POST["PHPSESSID"])) session_id($_POST["PHPSESSID"]);
		session_start();
		if(!$this->is_session(session_id())){
			$this->session_restart();
		}
	}

	public function read($session_id)
	{
		$session_table = $this->session_table;
		$query = "SELECT * FROM $session_table WHERE session_id='$session_id' AND session_name='$this->session_name' ".
				 "AND (server_ip='$this->server_ip' OR server_ip='0.0.0.0') ".
				 "ORDER BY server_ip DESC LIMIT 1";
		$sth = $this->dbh->query($query);
		$result = $sth->fetch();
		return $result['session_data'];
	}
    
	public function open($save_path, $session_name)
	{
		$session_id = $this->session_id = session_id();
		return ($this->dbh)?true:false;
	}

	public function close()
	{
	  	$this->gc($this->session_lifetime);
	  	return true;
	}
	
    public function write($session_id,$session_data)
	{
        $session_table = $this->session_table;
		$session_data = $this->dbh->quote($session_data);
		//$session_id = $this->dbh->quote($session_id);
		//$this->client_ip = $this->dbh->quote($this->client_ip);
		
		$query = "SELECT session_id FROM $session_table WHERE session_id='$session_id' AND session_name='$this->session_name' ".
				 "AND (server_ip='$this->server_ip' OR server_ip='0.0.0.0') ".
				 "ORDER BY server_ip DESC LIMIT 1";
		$result = $this->dbh->query($query);
		$result_record = $result->fetch();
			        
		if ($result && strlen($result_record['session_id']))
		{
			if (strlen($session_data)) 
				$session_data_statement = ", session_data=$session_data";
			else 
				$session_data_statement = '';
			$query = "UPDATE $session_table SET session_time = NOW() $session_data_statement WHERE session_id = '$session_id' ".
                     "AND session_name='$this->session_name' AND server_ip='$this->server_ip'";
			$result = $this->dbh->exec($query);
			if(!$result){
				$query = "UPDATE $session_table SET session_time = NOW() $session_data_statement WHERE session_id = '$session_id' ".
						 "AND session_name='$this->session_name' AND server_ip='0.0.0.0'";
				$result = $this->dbh->exec($query);
			}
			return $result?true:false;
		} else {
			$session_table = $this->session_table;
			$query = "INSERT INTO $session_table (session_id, session_start, session_time, session_data, client_ip, server_ip, session_name) ".
					 "VALUES ( '$session_id' ,'now()','now()', $session_data, '$this->client_ip', '$this->server_ip', '$this->session_name')";
			$result = $this->dbh->exec($query);
			//show($this->dbh->errorInfo(),"dbh error");
			return true;
		}
    }

	public function destroy($session_id)
	{
		$session_table = $this->session_table;
		$query = "DELETE FROM $session_table WHERE session_id ='$session_id' ".
				 "AND (server_ip='$this->server_ip' OR server_ip='0.0.0.0')";
		$result = $this->dbh->exec($query);
		return $result?true:false;
    }

	// garbage collector
  	public function gc($session_lifetime)
	{
		$session_table = $this->session_table;
		$result = $this->dbh->exec("DELETE FROM $session_table WHERE session_time < now() - ($session_lifetime * interval '1 second')");
		return $result?true:false;
	}

	// zniszczenie sesji z usunieciem ciasteczka 
	public function session_restart()
	{
		if (session_name()=='') {
			if($this->session_name != null)
				session_name($this->session_name);
			session_start();
		} else {
			session_destroy();
			session_set_save_handler(array(&$this, 'open'),
									array(&$this, 'close'),
									array(&$this, 'read'),
									array(&$this, 'write'),
									array(&$this, 'destroy'),
									array(&$this, 'gc'));
			if($this->session_name != null)
				session_name($this->session_name);
			session_start();
			session_regenerate_id();
			
			setcookie(session_name(), session_id(), 0, '/', $this->host);
		}
	}

	public function is_session($session_id)
	{
		$session_table = $this->session_table;
		$query = "SELECT session_id FROM $session_table WHERE session_id='$session_id' AND session_name='$this->session_name' ".
				 "AND (server_ip='$this->server_ip' OR server_ip='0.0.0.0') ".
                 "ORDER BY server_ip DESC LIMIT 1";
		$result = $this->dbh->query($query);
		$result_record = $result->fetch();
		if ($result && strlen($result_record['session_id']))
			return true;
		return false;
	}

}//class Session

?>
