<?php

/**
 * Abstrakcyjna klasa obslugi poszczegolnych tabel
 * W klasach potomnych (jedna klasa na jedna tabele z bazy danych)
 * nalezy zdefiniowac:
 * @tutorial Nazwe tabeli wraz z schematem 'schemat.tabela',
 * nazwy obiektow nadrzednych (rodzicow) - beda to pojedyncze eleementy (relacja jeden/wiele do jednego) 'parent',
 * nazwy obiektow podrzednych (dzieci) - beda to listy elementow (relacja jeden do wielu) 'child',
 * pamiętaj aby dodac odpowiednie klasy przy pomocy include_once lub require_once.
 * @author sroszczyk
 * @abstract
 * @version 1.1
 */
abstract class abstractTable
{
    //Elementy do zdefiniowania e klasach potomnych
    protected $table = null;//Nazwa tabeli 'schemat.tabela'
    protected $fields_extend = null;//Pola o jakie moze byc rozszerzony element w abstractTable::extendItem()
    protected $serials = null;//Lista pol serializowanych, automatyczna obsługa takich pól (tylko pola text)
    protected $related = null;//Tablica obiektow zkorelowanych lista 
        //'nazwaKlasy'=>array('type'=>'parent|child', 'from'=>'poleTejklasy', 'to'=>'poleNazwaKlasy')
        //molzliwy jeszcze natural gdy from=to, gdy nie ma to brany jest pkey, gdy wiecej wartosci nalezy podac w array liste
    //protected $parents = null;//Obiekty rodzice 'nazwaRodzica'=>'pole->poleRodzica'
    //protected $childs = null;//Obiekty dzieci 'nazwaDziecka'=>'pole->poleDziecka&pole2->poleDziecka2'
    
    //Elementy pomocnicze ogolne
    //protected $name = null;//Nazwa klasy
    protected $primary = null;//Tablica z nazwami kluczy
    protected $fields = null;//Tablica pol tabeli
    protected $f_type = null;//Typy pol, jak nie zdefiniowano to są to pola 'string', 'numeric', 'boolean', 'date' (data w stringu '2011-01-01 10:59:59)
    protected $debug = false;//Czy zapisywac akcje
    protected $errors = null;//Tablica bledow
    protected $actions = null;//Lista akcji na obiekcie
    protected $save_model = false;//Czy zapisać model
    protected $override = false;//Nadpisuje model
    
    //Elementy pomocnicze jednostki
    protected $exists = false;//Czy element istnieje w bazie
    protected $data = null;//Dane elementu
    protected $sync_data = null;//Dane elementu synchroniczne z tym co jest w bazie
    protected $use_returning = true;//Czy uzywac klauzuli RETURNING
    
    //Elementy pomocnicze listy
    protected $list = null;//Lista elementow
    protected $rows = 99999;//Ilosc elementow na strone
    protected $amount = 0;//Calkowita ilosc elementow
    protected $quick = false;//Szybkie pobieranie pelnych rekordow, bez obiektow (szybkosc kosztem pelnosci danych)
    protected $page = 0;
    protected $alias = null;//Aliasy
    protected $join = null;//Klauzula join
    protected $where = null;//Klauzula where
    protected $order = null;//Klauzula order by
    
    /*****************************************************************************************************
     * 				FUNKCJE OGOLNE
     *****************************************************************************************************/
    private function MIXED_FUNC(){}
    
    /**
     * Konstruktor, aby wczytac element nalezy podac wartosci kluczy glownych zgodnie z wpisem w $this->primary 
     */
    public function __construct()
    {
        $this->useDataModel();
        $num = func_num_args();
	    $args = func_get_args();
	    
	    if($num>0 && $num==count($this->primary)){
	        $i=0;
	        foreach($this->primary as $pri){
	            $this->data[$pri] = $args[$i];
	            $i++;
	        }
	        $this->loadItem();
	        if(!$this->exists)
	            foreach($this->primary as $pri)
	                unset($this->data[$pri]);
	    }
    }
    
	/**
     * Wczytanie informacji o schemacie danych i tworzy plik schematu tabeli
     */
    protected function useDataModel()
    {
        if($this->fields && !$this->override) return;
        //$this->name = get_class($this);
        $table = $schema = null;
        list($schema, $table) = explode('.', $this->table);
        $sql = "SELECT column_name, data_type FROM information_schema.columns ".
        	   "WHERE table_schema = '$schema' AND table_name = '$table'";
        $data = PDOS::_()->getAll($sql);
        foreach($data as $row){
            $this->fields[] = $row['column_name'];
            $this->f_type[$row['column_name']] = $row['data_type'];
        }
        $sql = "SELECT column_name FROM information_schema.key_column_usage ".
               "WHERE table_schema = '$schema' AND table_name = '$table' AND ".
        		"position_in_unique_constraint IS NULL AND constraint_name LIKE '%_pkey'";
        $data = PDOS::_()->getAll($sql);
        foreach($data as $row){
            $this->primary[] = $row['column_name'];
        }
        if(!$this->save_model) return;
        $data = "<?php\n\n";
        $data .= "/**\n";
        $data .= " *  Plik wygenerowany automatycznie, nie edytuj go tylko rozszerz ta klase!\n";
        $data .= " *  @version time:".date('Y-m-d G:i:s')."\n";
        $data .= " *  @author PHP abstractTable.class.php\n";
        $data .= " */\n\n";
        $data .= "class {$table}Table_{$schema}Schema extends abstractTable\n{\n";
        $data .= "\tprotected \$table = '{$this->table}';\n";
        $data .= "\t//Odkomentuj ponizsza linie aby zaktualizowac ten plik!!!\n";
        $data .= "\t//protected \$override = true;\n";
        $data .= "\tprotected \$primary = array(\n";
        foreach($this->primary as $v) $data .= "\t\t'{$v}',\n";
        $data .= "\t);\n";
        $data .= "\tprotected \$fields = array(\n";
        foreach($this->fields as $v) $data .= "\t\t'{$v}',\n";
        $data .= "\t);\n";
        $data .= "\tprotected \$f_type = array(\n";
        foreach($this->f_type as $k => $v) $data .= "\t\t'{$k}' => '{$v}',\n";
        $data .= "\t);\n";
        $data .= "}\n\n";
        $data .= "?>";
        $file = $table.'Table_'.$schema.'Schema.php';
        $path = dirname(__FILE__).'/model/table/'.$file;
        file_put_contents($path, $data);
    }
    
    /**
     * Zwraca elementy modelu tablicy
     */
    public function getTable(){ return $this->table; }
    public function getColumns(){ return $this->fields; }
    public function getDataTypes(){ return $this->f_type; }
    public function getPrimaryColumns(){ return $this->primary; }
    public function getRelated(){ return $this->related; }
    
	/**
     * Pobiera tablice bledow
     */
    public function getErrors(){ return $this->errors; }
    
    /**
     * Czy debugowac
     */
    public function debug($d = null){ if($d!==null) $this->debug = $d?true:false; else $this->debug = !$this->debug; }
    
    /**
     * Zapis do akcji
     */
    protected function saveAction($msg){ if($this->debug) $this->actions[] = $msg; }
    
    /**
     * Zwraca historie akcji
     */
    public function getActions(){ return $this->actions; }
    
    /**
     * Czysci obiekt
     */
    public function clear()
    {
        $this->clearItem();
        $this->clearList();
    }
    
    
    
    /*****************************************************************************************************
     * 				FUNKCJE ZWIAZANE Z ELEMENTEM
     *****************************************************************************************************/
    private function ITEM_FUNC(){}
    
    /**
     * Wczytywanie elementu
     */
    public function loadItem()
    {
        $where = null;
        foreach($this->primary as $pri) 
            if(!isset($this->data[$pri]) || empty($this->data[$pri])) 
                return false;
            else
                $where[] = "{$pri}='{$this->data[$pri]}'";
        
        $select = '"'.implode('", "', $this->fields).'"';
        if($where)
            $where = 'WHERE '.implode(' AND ', $where);
        $sql = "SELECT $select FROM {$this->table} $where";
        $row = PDOS::_()->getRow($sql);
        $this->saveAction($sql);
        if($row){
            foreach($row as $k=>$v){
                if($this->f_type[$k]=='boolean')
                    $this->data[$k] = $v?true:false;
                elseif($this->f_type[$k]=='ARRAY')
                    $this->data[$k] = strlen($v)?explode(',', str_replace(array('{','}'), '', $v)):null;
                elseif($this->f_type[$k]=='text' && is_array($this->serials) && in_array($k, $this->serials))
                    $this->data[$k] = unserialize($v);
                else
                    $this->data[$k] = stripslashes($v);
            }
            $this->sync_data = $this->data;
            $this->exists = true;
            $this->extendItem();
            return true;
        }
        return false;
    }
    
    /**
     * Rozszerzanie rekordu o pola z abstractTable::fields_extend
     */
    protected function extendItem(){}
    
    /**
     * Zapis elementu
     */
    public function saveItem($skip_validation = false)
    {
        if(!$skip_validation && !$this->validateItem()) return false;
        
        $data = null;
        $load_after_save = false;
        foreach($this->data as $k=>$v){
            if(!in_array($k, $this->fields))
                continue;
            if(isset($this->sync_data[$k]) && $this->sync_data[$k]==$v) 
                continue;
            if(is_string($v) && strtoupper(substr($v,0,4))=='SQL:'){//to troche dziwne ale sie przydaje ;)
                $load_after_save = true;
                $data[$k] = str_ireplace('SQL:', '', $v);
            }
            elseif(in_array($k, $this->primary)) 
				$data[$k] = "'".$this->getEscapedString($v)."'";
            elseif($this->f_type[$k]=='boolean')
                $data[$k] = $v?'TRUE':'FALSE';
            elseif($this->f_type[$k]=='ARRAY' && is_array($v) && count($v))
                $data[$k] = 'ARRAY[\''.implode('\',\'', $v).'\']';
            elseif($this->f_type[$k]=='text' && is_array($this->serials) && in_array($k, $this->serials) && is_array($v) && count($v))
                $data[$k] = "'".$this->getEscapedString(serialize($v))."'";
            elseif(empty($v))
                $data[$k] = 'NULL';
            elseif(strpos($this->f_type[$k], 'timestamp')!==false && strtolower($v)=='now')
                $data[$k] = 'NOW()';
            else
				$data[$k] = "'".$this->getEscapedString($v)."'";
        }
        
        if($this->exists){
            if(!$data) return true;//Nic nie trzeba aktualizowac ;)
            $where = null;
            foreach($this->primary as $pri) 
                $where[] = "\"{$pri}\"='{$this->data[$pri]}'";
            $sql = "UPDATE {$this->table} SET ";
            $update = null;
            foreach($data as $k=>$v)
                $update[] = "\"{$k}\"={$v}";
            $sql .= implode(', ', $update);
            if($where)
                $sql .= ' WHERE '.implode(' AND ', $where);
            $res = PDOS::_()->exec($sql);
            $this->saveAction($sql);
            if($res!==false){
                if($load_after_save){
                    $this->loadItem();
                }else{
                    $this->sync_data = $this->data;
                }
                return true;
            }
            return false;
        }else{
            $sql = "INSERT INTO {$this->table} (";
            $sql .= '"'.implode('", "', array_keys($data)).'"';
            $sql .= ") VALUES (";
            $sql .= implode(', ', array_values($data));
            $sql .= ")";
            if($this->use_returning){
                $sql .= " RETURNING ";
                $sql .= implode(', ', $this->primary);
            }else{
                $load_after_save = true;
            }
            $row = PDOS::_()->getRow($sql);
            $this->saveAction($sql);
            if($row!==false){
                foreach($this->primary as $pri)
                    if(isset($row[$pri]))
                        $this->data[$pri] = $row[$pri];
                    else return false;
                if($load_after_save){
                    $this->loadItem();
                }else{
                    $this->sync_data = $this->data;
                    $this->exists = true;
                }
                return true;
            }
            return false;
        }
    }
    
    /**
     * 
     */
    public function deleteItem()
    {
        if($this->exists){
            if($this->related)
            foreach($this->related as $className => $dd){//Usuwamy wszystkie elementy potomne przed usunieciem tego elementu
                if($dd['type']=='child'){
                    foreach($this->getChilds($className)->getListObject() as $obj){
                        $obj->deleteItem();
                    }
                }
            }
            
            $where = null;
            foreach($this->primary as $pri) 
                $where[] = "\"{$pri}\"='{$this->data[$pri]}'";
            $sql = "DELETE FROM {$this->table} WHERE ".implode(' AND ', $where);
            $res = PDOS::_()->exec($sql);
            if($res!==false){
                $this->clearItem();
                return true;
            }
        }else{
            $this->clearItem();
            return true;
        }
        return false;
    }
    
    /**
     * Czysci Item
     */
    public function clearItem()
    {
        $this->data = $this->sync_data = $this->errors = null;
        $this->exists = false;
    }
    
    /**
     * Pobiera wybrane pole
     * @param string $key Nazwa pola do pobrania
     * @param boolean $sync Pobierac dane synchroniczne z baza
     */
    public function getField($key = null, $sync = false)
    {
        $f = 'data';
        if($sync) $f = 'sync_data';
        $data = $this->$f;
        if($key==null){
            $this->saveAction("Pobrano caly rekord z $f");
            return $data;
        }
        $this->saveAction("Pobrano pole $key z $f");
        if(isset($data[$key]))
            return $data[$key];
        return false;
    }
    
    /**
     *	Pobranie wszystkich pol 
     */
    public function getFields($sync = false)
    {
        return $this->getField(null, $sync);
    }
    
    /**
     * Ustawia pole, nie mozna ustawiac primary key'a
     * @param string $key
     * @param mixed $value
     */
    public function setField($key, $value)
    {
        $msg = null;
        if(!in_array($key, $this->fields) && !in_array($key, is_array($this->fields_extend)?$this->fields_extend:array()))
            return false;
        
        //Jesli ustawiamy primary key kasujemy obiekt oprucz primary keyow
        if(in_array($key, $this->primary)){
            //$prid = null;
            //foreach($this->primary as $pri)
            //    $prid[$pri] = $this->data[$pri];
            //$this->clearItem();
            //$this->data = $prid;
            $msg = ' glowne';
        }
        
        $this->saveAction("Ustawiono pole{$msg} $key [".substr(print_r($value, true), 0, 100)."]");
        $this->data[$key] = $value;
        return true;
    }
    
    /**
     * Pobranie kluczy glownych 
     * @return array
     */
    public function getPrimary()
    {
        $prid = null;
        foreach($this->primary as $pri)
            $prid[$pri] = $this->data[$pri];
        return $prid;
    }
    
    /**
     * Brutalne ustawienie wszystkie klucze glowne
     * @param mixed $??? lista kluczy glownych lub tablica z kluczami
     * $return boolean
     */
    public function setPrimary()
    {
	    $this->clearItem();
        $num = func_num_args();
	    $args = func_get_args();
	    if($num==1 && is_array($args[0])){
	        $num = count($args[0]);
	        $args = $args[0];
	    }
	    
	    if($num>0 && $num==count($this->primary)){
	        $i=0;
	        foreach($this->primary as $pri){
	            $this->data[$pri] = isset($args[$pri])?$args[$pri]:$args[$i];
	            $i++;
	        }
	        $this->saveAction("Ustawiono wszystkie pola glowne [".implode(', ', $args)."]");
	        return true;
	    }
	    return false;
    }
    
    /**
     * Walidacja elementu, wyplnia tablice errors
     * @abstract
     */
    public function validateItem(){ return true; }
    
    /**
     * Pobranie obiektu nadrzednego
     * @return object
     * @todo testowac
     */
    public function getParent($className, $fromList = false)
    {
        if(!isset($this->related[$className]) || $this->related[$className]['type']!='parent') return false;
        
        $data = $this->related[$className];
        //$className = $className.'Table';
        $obj = new $className;
        if($this->debug) $obj->debug(true);
        if($this->exists && !$fromList){
            //Relacje po primary z from lub natural
            if(isset($data['from']) && isset($data['to'])){//Relacja from->to
                if(is_array($data['from']) && is_array($data['to'])){
                    foreach($data['from'] as $k => $v){
                        $field = $data['to'][$k];
                        $value = $this->data[$v];
                        $obj->setField($field, $value);
                    }
                }else{
                    $field = $data['to'];
                    $value = $this->data[$data['from']];
                    $obj->setField($field, $value);
                }
            }elseif(isset($data['natural'])){//Relacja naturalna
                if(is_array($data['natural'])){
                    foreach($data['natural'] as $k => $v){
                        $field = $v;
                        $value = $this->data[$v];
                        $obj->setField($field, $value);
                    }
                }else{
                    $field = $data['natural'];
                    $value = $this->data[$data['natural']];
                    $obj->setField($field, $value);
                }
            }
            $obj->loadItem();
            if(is_array($tmp = $obj->getActions())) foreach($tmp as $act) $this->saveAction('PARENT: '.$act);
            return $obj;
        }else{
            $list = $this->getListArray();
            $aor = 0;
            if(is_array($list)){
                foreach($list as $item){
                    if(isset($data['from']) && isset($data['to'])){//Relacja from->to
                        if(is_array($data['from']) && is_array($data['to'])){
                            $aand = 0;
                            foreach($data['from'] as $k => $v){
                                $field = $data['to'][$k];
                                $value = $item[$v];
                                $obj->addCondition($field, '=', $value);
                                $aand++;
                            }
                            if($aand > 1)
                                $obj->mergeConditions($aand, false);
                        }else{
                            $field = $data['to'];
                            $value = $item[$data['from']];
                            $obj->addCondition($field, '=', $value);
                        }
                    }elseif(isset($data['natural'])){//Relacja naturalna
                        if(is_array($data['natural'])){
                            $aand = 0;
                            foreach($data['natural'] as $k => $v){
                                $field = $v;
                                $value = $item[$v];
                                $obj->addCondition($field, '=', $value);
                                $aand++;
                            }
                            if($aand > 1)
                                $obj->mergeConditions($aand, false);
                        }else{
                            $field = $data['natural'];
                            $value = $item[$data['natural']];
                            $obj->addCondition($field, '=', $value);
                        }
                    }/*else{//Relacja naturalna po primarykey'u
                        $aand = 0;
                        foreach($this->primary as $field){
                            $value = $item[$field];
                            $obj->addCondition($field, '=', $value);
                            $aand++;
                        }
                        if($aand > 1)
                            $obj->mergeConditions($aand, false);
                    }*/
                    $aor++;
                }
                if($aor > 1)
                    $obj->mergeConditions($aor);
                $obj->search();
                if(is_array($tmp = $obj->getActions())) foreach($tmp as $act) $this->saveAction('CHILD: '.$act);
            }
            return $obj;
        }
    }
    
    /**
     * Pobranie obiektu podrzednego (lista)
     * @return object
     * @todo testowac
     */
    public function getChilds($className, $fromList = false)
    {
        if(!isset($this->related[$className]) || $this->related[$className]['type']!='child') return null;
        
        $data = $this->related[$className];
        //$className = $className.'Table';
        $obj = new $className;
        if($this->debug) $obj->debug(true);
        if($this->exists && !$fromList){//Wyciaganie elementow potomnych z pojedynczego elementu
            if(isset($data['from']) && isset($data['to'])){//Relacja from->to
                if(is_array($data['from']) && is_array($data['to'])){
                    foreach($data['from'] as $k => $v){
                        $field = $data['to'][$k];
                        $value = $this->data[$v];
                        $obj->addCondition($field, '=', $value);
                    }
                }else{
                    $field = $data['to'];
                    $value = $this->data[$data['from']];
                    $obj->addCondition($field, '=', $value);
                }
            }elseif(isset($data['natural'])){//Relacja naturalna
                if(is_array($data['natural'])){
                    foreach($data['natural'] as $k => $v){
                        $field = $v;
                        $value = $this->data[$v];
                        $obj->addCondition($field, '=', $value);
                    }
                }else{
                    $field = $data['natural'];
                    $value = $this->data[$data['natural']];
                    $obj->addCondition($field, '=', $value);
                }
            }else{//Relacja naturalna po primarykey'u
                foreach($this->primary as $field){
                    $value = $this->data[$field];
                    $obj->addCondition($field, '=', $value);
                }
            }
            $obj->search();
            if(is_array($tmp = $obj->getActions())) foreach($tmp as $act) $this->saveAction('CHILD: '.$act);
            return $obj;
        }else{//Wyciaganie elementow potomnych z listy
            if(!$this->list) return $obj;//Zwracamy pusty obiekt
            
            $list = $this->getListArray();
            $aor = 0;
            foreach($list as $item){
                if(isset($data['from']) && isset($data['to'])){//Relacja from->to
                    if(is_array($data['from']) && is_array($data['to'])){
                        $aand = 0;
                        foreach($data['from'] as $k => $v){
                            $field = $data['to'][$k];
                            $value = $item[$v];
                            $obj->addCondition($field, '=', $value);
                            $aand++;
                        }
                        if($aand > 1)
                            $obj->mergeConditions($aand, false);
                    }else{
                        $field = $data['to'];
                        $value = $item[$data['from']];
                        $obj->addCondition($field, '=', $value);
                    }
                }elseif(isset($data['natural'])){//Relacja naturalna
                    if(is_array($data['natural'])){
                        $aand = 0;
                        foreach($data['natural'] as $k => $v){
                            $field = $v;
                            $value = $item[$v];
                            $obj->addCondition($field, '=', $value);
                            $aand++;
                        }
                        if($aand > 1)
                            $obj->mergeConditions($aand, false);
                    }else{
                        $field = $data['natural'];
                        $value = $item[$data['natural']];
                        $obj->addCondition($field, '=', $value);
                    }
                }else{//Relacja naturalna po primarykey'u
                    $aand = 0;
                    foreach($this->primary as $field){
                        $value = $item[$field];
                        $obj->addCondition($field, '=', $value);
                        $aand++;
                    }
                    if($aand > 1)
                        $obj->mergeConditions($aand, false);
                }
                $aor++;
            }
            if($aor > 1)
                $obj->mergeConditions($aor);
            $obj->search();
            if(is_array($tmp = $obj->getActions())) foreach($tmp as $act) $this->saveAction('CHILD: '.$act);
            return $obj;
        }
    }
    
    /**
     * Czy element istnieje
     */
    public function exists()
    {
        return $this->exists;
    }
    
    /*****************************************************************************************************
     * 				FUNKCJE ZWIAZANE Z LISTA
     *****************************************************************************************************/
    private function LIST_FUNC(){}
    
    /**
     * Dodawanie tabeli (innej klasy) INNER JOIN
     */
    public function addJoin($className)
    {
        /*if(!isset($this->related[$className])) return false;
        $aliasy = array('b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'r', 's', 
        				't', 'u', 'w', 'x', 'y', 'z');
        $alias = $aliasy[count($this->join)];
        $key = $className;//.' '.$alias;
        //$className = $className.'Table';
        $obj = new $className;
        $table = $obj->getTable();
        $join = null;
        for($i = 1; isset($data['from'.$i]); $i++){
            $alias2 = $this->fieldAlias($data['to'.$i]);if(!$alias2) continue;
            $join[] .= "{$alias}.{$data['from'.$i]}={$alias2}.{$data['to'.$i]}";
        }
        if($join){
            $join = "INNER JOIN $table $alias ON ".implode(' AND ', $join);
            $this->join[$key] = $join;
            return true;
        }
        return false;*/
    }
    
    /**
     * Dodanie warunku WHERE
     * @param string $field pole ktore ma byc porownywane
     * @param string $op
     * @param mixed $value wartosc do jakiej ma byc porownane pole
     * @param string $func nazwa funkcji ktora ma objac pole i wartosc jesli wartosc jest czystym stringiem (bez nawiasow)
     * @example $obj->addCondition('pole', '>', 5);
     * @example $obj->addCondition('nazwaKlasy.pole', '>', 5); //nazwaKlasy bez koncowki Table
     * @example $obj->addCondition('pole', '=', 'LOWER(dupa)', 'LOWER');
     * @example $obj->addCondition('pole', 'IS', null);
     * @example $obj->addCondition('pole', 'IN', '(\'text_a\', \'text_b\', \'text_c\')');
     * @example $obj->addCondition('pole', 'LIKE', '%text%');
     * @return boolean
     */
    public function addCondition($field, $op, $value, $func = null)
    {
        $this->saveAction("addCondition: $field; $op; $value; $func");
        if(!($alias = $this->fieldAlias($field))) return false;
        if($value===true) $value = 'TRUE';
        elseif($value===false) $value = 'FALSE';
        elseif($value===null) $value = 'NULL';
        elseif(strpos($value,'(')!==false) $value = $value;
        elseif($func) $value = "$func('$value')";
        else $value = "'$value'";
        $field = "$alias.$field";
        if($func) $field = "$func($field)";
        $this->where[] = "$field $op $value";
        return true;
    }
    
    public function addWhere($whr)
    {
        $this->where[] = $whr;
    }
    
    /**
     * Zamienie AND na OR dla x ostatnich warunków
     */
    public function mergeConditions($x = 2, $asOr = true)
    {
        if($x<2 || $x>count($this->where)) return false;
        
        $imp = $asOr?' OR ':' AND ';
        $where = null;
        for($i=0;$i<$x;$i++)
            $where[] = array_pop($this->where);
        $this->where[] = '('.implode($imp, $where).')';
    }
    
    /**
     * Dodanie sortowania ORDER BY
     */
    public function addOrder($field, $desc = false)
    {
        $this->saveAction("addOrder: $field ".($desc?'desc':'asc'));
        if(!($alias = $this->fieldAlias($field))) return false;
        $field = "$alias.$field";
        $this->order[] = $field.($desc?' DESC':'');
        return true;
    }
    
    /**
     * Sprawdza jaki alias nalezy dac polu
     * @param unknown_type $field pole lub nazwaKlasy.pole
     * @todo przetestowac to
     */
    protected function fieldAlias(&$field)
    {
        if(strpos($field, '.')===false){
            
            if(in_array($field, $this->fields))
                return 'a';
            return false;
            
        }else{
            
            list($className, $field) = explode('.', $field);
            if($this->makeJoin($className)){
                return $this->alias[$className];
            }
            return false;
            
        }
    }
    
    /**
     * Tworzy klauzule join
     * przydatne przy wyszukiwaniu po polach innych skorelowanych obiektow
     * @todo ostre testy ;)
     */
    public function makeJoin($className, $classParent = null)
    {
        $al = array('b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 
        			'o', 'q', 'p', 'r', 's', 't', 'u', 'w', 'x', 'v', 'y', 'z');
        
        if(isset($this->alias[$className]))
            return true;
        
        $join = $palias = $alias = $related = null;
        $make_join = false;
        
        if(isset($this->related[$className])){
            //Obiekty sa bezposrednio polaczone
            
            $palias = 'a';
            $alias = $al[count($this->alias)];
            $related = $this->related[$className];
            $make_join = true;
            
        }elseif($classParent){
            //Obiekt jest podpiety posrednio przez $classParent
            
            $tobj = new $classParent;
            $related = $tobj->getRelated();
            if(isset($this->alias[$classParent]) && isset($related[$className])){
                $related = $related[$className];
                $palias = $this->alias[$classParent];
                $alias = $al[count($this->alias)];
                $make_join = true;
            }
            
        }else{
            //Obiekt jest polaczony posrednio przez inne obiekty
            
            foreach($this->alias as $class => $a){
                $tobj = new $class;
                $related = $tobj->getRelated();
                if(isset($related[$className])){
                    $related = $related[$className];
                    $palias = $this->alias[$class];
                    $alias = $al[count($this->alias)];
                    $make_join = true;
                    break;
                }
            }
            
        }
        
        if($make_join){
            if(isset($related['from'])){
                if(is_array($related['from'])){
                    $tmp = null;
                    foreach($related['from'] as $k=>$from)
                        $tmp[] = "{$palias}.{$from}={$alias}.{$related['to'][$k]}";
                    $join = implode(' AND ', $tmp);
                }else{
                    $join = "{$palias}.{$related['from']}={$alias}.{$related['to']}";
                }
            }elseif(isset($related['natural'])){
                if(is_array($related['natural'])){
                    $tmp = null;
                    foreach($related['natural'] as $k=>$natural)
                        $tmp[] = "{$palias}.{$natural}={$alias}.{$natural}";
                    $join = implode(' AND ', $tmp);
                }else{
                    $join = "{$palias}.{$related['natural']}={$alias}.{$related['natural']}";
                }
            }else{
                $tmp = null;
                $primary = $this->primary;
                if($classParent){
                    $tobj = new $classParent;
                    $primary = $tobj->getPrimary();
                }
                foreach($primary as $k=>$natural)
                    $tmp[] = "{$palias}.{$natural}={$alias}.{$natural}";
                $join = implode(' AND ', $tmp);
            }
            
            $tobj = new $className;
            $table = $tobj->getTable();
            $join = "LEFT OUTER JOIN {$table} {$alias} ON {$join}";
            
            $this->alias[$className] = $alias;
            $this->join[$className] = $join;
            return true;
        }
        
        return false;
    }
    
    /**
     * Elementy zwiazane ze stronicowaniem
     */
    public function setLimit($limit){ $this->rows = $limit; }
    public function setPage($page){ $this->page = $page; } 
    public function getLimit(){ return $this->rows; }
    public function getPage(){ return $this->page; } 
    public function getAmount(){ return $this->amount; }
    
    /**
     * Zwraca tablice paginacji
     * @param integer $range limit przeskoku
     */
    public function getPages($range = 0)
    {
        $pages = ceil($this->amount/$this->rows);
        $page = $this->page;
        $arr = null;
        for($i=0;$i<$pages;$i++)
            if($range==0 || (($page-$range)>$i && ($page+$ranse)<$i) || $i==0 || $i==$page-1)
                $arr[] = $i;
        return $arr;
    }
    
    /**
     * Szybkie pobieranie listy
     * @param boolean $quick szybkie pobieranie calych rekordow, poza obiektem
     */
    public function quick($q = true)
    {
        $this->quick = $q;
    }
    
    /**
     * Uruchamia zapytanie zbierajace liste
     */
    public function search()
    {
        $sql = "SELECT * FROM {$this->table} a ";
        if($this->join){
            $sql .= implode(' ', $this->join).' ';
        }
        //$sql.= implode(' ', $this->join).' ';
        if($this->where)
            $sql.= 'WHERE '.implode(' AND ', $this->where).' ';
        $sqla = str_replace('*', 'COUNT(DISTINCT(a."'.implode('", a."', $this->primary).'"))', $sql);
        $this->amount = PDOS::_()->getOne($sqla);
        $sql .= 'GROUP BY a."'.implode('", a."', $this->primary).'"';
        if($this->order){
            foreach($this->order as $oo){
                $oo = explode(' ', $oo);
                $sql .= ', '.$oo[0];
            }
            $sql.= ' ORDER BY '.implode(', ', $this->order).' ';
        }
        if($this->rows)
            $sql.= "LIMIT {$this->rows} ";
        if($this->page)
            $sql.= 'OFFSET '.($this->rows*$this->page);
        if(!$this->quick)
            $sql = str_replace('*', 'a."'.implode('", a."', $this->primary).'"', $sql);
        $this->list = PDOS::_()->getAll($sql);
        $this->saveAction($sql);
        return $this->list;
    }
    
    /**
     * czysci Liste
     */
    public function clearList()
    {
        $this->amount = 0;
        $this->list = $this->join = $this->where = $this->order = null;
    }
    
    /**
     * Zwraca liste jako klucze glowne
     */
    public function getList()
    {
        return $this->list;
    }
    
    /**
     * Zwraca liste jako tablice obiektow
     */
    public function getListObject()
    {
        if($this->quick) return false;
        if(!is_array($this->list)) return null;
        $list = null;
        foreach($this->list as $primary){
            $tmp = get_class($this);
            $tmp = new $tmp;
            $tmp->setPrimary($primary);
            $tmp->loadItem();
            $list[] = $tmp;
        }
        return $list;
    }
    
    /**
     * Zwraca liste jako tablice rekordow
     */
    public function getListArray()
    {
        if($this->quick) return $this->list;
        $arr = null;
        if(is_array($list = $this->getListObject()))
            foreach($list as $obj)
                $arr[] = $obj->getField();
        return $arr;
    }
    
    /**
     * Przekształca element listy na obiekt
     */
    public function list2item($key = 0)
    {
        $this->clearItem();
        if(isset($this->list[$key])){
            $this->setPrimary($this->list[$key]);
            $this->loadItem();
            if($this->exists)
                return true;
        }
        return false;
    }

	/**
	 * Obejscie escapeowania =o
	 */
	protected function getEscapedString($string, $serializedString = false){
		/*$search = array("'", '"');
		$replace = array("''", """");*/
		$searchArray = array(
			"'", "\\"
		);
		$replaceArray = array(
			"''", "\\\\"
		);

		if($serializedString){
			return str_replace($searchArray, $replaceArray, serialize($string));
		}else{
			return str_replace($searchArray, $replaceArray, $string);
		}
	}
}

?>