<?php

if(!defined('SESSION_EXPIRE'))
    define('SESSION_EXPIRE', 3600);
if(!defined('SECURED_EXPIRE'))
    define('SECURED_EXPIRE', 900);

require_once('system.class.php');
require_once('PDOS.class.php');

/**
 * 
 * Enter description here ...
 * @author sroszczyk
 *
 */
class Authorize
{
    private $db;
    public $username = null;
    private $tuser = null;
    public $cookie_user = null;
    public $funct = array();
    public $team = array();
    public $user_id = null;
    public $auth = null;
    public $client_id = null;
    private $useSoap = false;
    
    /**
     *  Dane dotyczące sesji użytkownika
     **/
    public $sesja;
    
    /**
     *
     **/
    private $secured = false;
        
    /**
     *      Ograniczenia dostępu
     *      'auth_require'  - gdy true akcja wymaga autoryzacji do systemu
     *      'auth_secure'   - gdy true akcja wymaga zabezpieczonej autoryzacji
     *      'keep_secure'   - utrzymuje sesję bezpieczną gdy nie jest wymagana
     *      'ticket_ignore' - gdy true akcja moze ziostac wykonana pomimo aktywnych ticketów
     *      'allow_groups'  - lista grup które mają dostęp, null oznacza że wszyscy mają dostęp
     *      'check_action'  - gdy true dokonywane jest głębsze sprawdzenie dostępu (cms)
     *      'allow_bots'    - gdy true wpuszczane będą boty wyszukiwarek
     *      'save_log'      - gdy true zapisuje wywołanie do logu
     *      'save_bot_log'  - gdy true zapisuje wywołanie bota do logu
     **/
    private $access = array(
                        'auth_require'  => false,
                        'auth_secure'   => false,
                        'keep_secure'   => false,
                        'ticket_ignore' => false,
                        'allow_groups'  => null,
                        'allow_uts'		=> null,
                        'check_action'  => false,
                        'allow_bots'    => true,
                        'save_log'      => true,
                        'save_bot_log'  => false,
                        );
    
    /**
     *  Dane konfiguracyjne autoryzacji
     */
    private $config = array(
                        'users_table' => 'main_users',
                        'group_table' => 'main_admin_groups',
                        'rel_user_group' => 'rel_user_admin_group',
                        'access_table' => 'cms.access_action',
                        'function_table' => 'cms.rel_user_function',
                        'team_table' => 'cms.rel_user_team'
                        );
    
    /**
     *  Tablica ostrzeżeń dotyczących autoryzacji, autoryzacja jest dokonywana
     */
    public $warnings = null;
    
    /**
     *  Tablica błędów autoryzacji, autoryzacja nie jest dokonywana
     */
    public $errors = null;
    
    /**
     *  Przeglądarka usera
     **/
    private $user_agent;
    
    /**
     *
     **/
    private $client_ip;
    
    /**
     *  KONSTRUKTOR
     **/
    public function __construct($access = array(), $db = null)
    {
        $this->user_agent = $_SERVER['HTTP_USER_AGENT'];
        $this->client_ip = $_SERVER['REMOTE_ADDR'];
        //Nadpisywanie ograniczeń dostępu
        if(isset($access['allow_groups']) && $access['allow_groups']!=null){
            if(!is_array($access['allow_groups'])){
                if(strpos($access['allow_groups'], ','))
                    $access['allow_groups'] = explode(',', $access['allow_groups']);
                elseif(strpos($access['allow_groups'], ';'))
                    $access['allow_groups'] = explode(';', $access['allow_groups']);
                else
                    $access['allow_groups'] = explode(' ', $access['allow_groups']);
            }
            for($i=0;$i<count($access['allow_groups']);$i++){
                $access['allow_groups'][$i] = trim($access['allow_groups'][$i]);
            }
        }
        foreach($access as $key => $value){
            if(key_exists($key,$this->access))
                $this->access[$key] = $access[$key];
        }
        if($this->access['auth_secure']){
            $this->access['auth_require'] = $this->access['auth_secure'];
        }
        if($db){
            $this->db = $db;
        }elseif(defined('PDOS_DBNAME')){
            $this->db = PDOS::getInstance();
        }else{
            global $db;
            $this->db = $db;
        }
    }
    
    /**
     *  Tworzy długowieczne ciastko
     **/
    private function generateLongLiveCookie()
    {
        if($this->username && !isset($_COOKIE['ESCULAP_USER']))
            setcookie('ESCULAP_USER', md5('Long'.$this->username.'lifE').'+'.$this->username, time()+31000000, '/');
        return;
    }
    
    /**
     *  Sprawdza długowieczne ciastko
     **/
    private function checkLongLiveCookie()
    {
        if(isset($_COOKIE['ESCULAP_USER'])){
            $temp = explode('+', $_COOKIE['ESCULAP_USER']);
            $hash = $temp[0];
            $user = $temp[1];
            if(md5('Long'.$user.'lifE')==$hash)
                $this->tuser = $user;
        }
        return;
    }
    
    /**
     *  Rozpoczęcie sesji użytkownika
     */
    public function start_session($login, $pass = true, $secured = false)
    {
        $this->errors = null;
        $this->warnings = null;
        $sql = "SELECT user_login, password, status, real_status, need_attention FROM {$this->config['users_table']} WHERE user_login = '{$login}'";
        $row = $this->db->getRow($sql);
        if(!$row){
            $this->errors['bad_login'] = true;
            return false;
        }
        if($row['password']!=$pass && $pass!==true){
            $this->errors['bad_password'] = true;
            return false;
        }
        if($row['status']=='NONE' || $row['status']=='NOACCESS' || $row['real_status']=='DELETED'){
            $this->errors['bad_status'] = true;
            return false;
        }
        $this->username = $row['user_login'];
        $this->check_access = true;
        $this->db->exec("UPDATE main_users SET date_session=NOW() WHERE user_login='$this->username'");
        $_SESSION['auth']['username'] = $this->username;
        $_SESSION['username'] = $this->username;//Dla zgodności z Authorization_engine
        $_SESSION['auth']['session_time'] = time() + SESSION_EXPIRE;
        $_SESSION['auth']['client_ip'] = $this->client_ip;
        
        //Ustawianie długowiecznego ciastka
        $this->generateLongLiveCookie();
        
        //Sprawdzenie długowiecznego znacznika użytkownika
        $this->checkLongLiveCookie();
        
        $sql = "SELECT first_name, last_name, user_type ".
                "FROM main_users WHERE user_login='{$this->username}'";
        $data = $this->db->getRow($sql);
        foreach($data as $key => $value){
            $_SESSION['auth'][$key] = $value;
            $_SESSION['sesja'][$key] = $value;//To będzie można usunąć//
        }
        $this->getCmsUserId();
        
        /*** Uruchamiamy sesje PERL ***/
        if(session_name()=='PHPSESSID'){
            $session_id = $_COOKIE['PHPSESSID'];
            $this->db->beginTransaction();
            $sql = "UPDATE main_users SET id_session='{$session_id}', date_session=NOW() WHERE user_login='{$this->username}'";
            $this->db->exec($sql);
            $sql = "UPDATE sessions SET status='CLOSED', STOP=NOW(), session_id=NULL WHERE user_login='{$this->username}' AND (status='OPENED' OR session_id IS NOT NULL)";
            $this->db->exec($sql);
            $sql = "INSERT INTO sessions (session_id, user_login, start, status, client_ip) ".
                   "VALUES ('{$session_id}', '{$this->username}', NOW(), 'OPENED', '{$this->client_ip}')";
            $this->db->exec($sql);
            $this->db->commit();
        }
        /******************************/
        
        if($secured){
            $_SESSION['auth']['secured_time'] = time() + SECURED_EXPIRE;
        }
        
        //Wywołanie zdarzeń LOGIN w istniejących ticketach usera
        $system = new system($this->username);
        if($row['real_status']=='NEW' && !$row['need_attention']){
            $system->use_process('ACOUNT_ACTIVATION', 'BEGIN');
        }
        $tickets = $system->get_user_tickets($this->username);
        if(is_array($tickets))
            foreach($tickets as $ticket)
                $system->use_process($ticket['operation_name'], 'LOGIN');
        
        $this->startSE4Sess();
        
        return true;
    }
    
    /**
     *  Zakończenie sesji użytkownika
     */
    public function stop_session()
    {
        $this->username = null;
        $this->secured = false;
        $this->user_id = null;
        unset($_SESSION['auth']);
        unset($_SESSION['username']);//Dla zgodności z poprzednią wersją
        unset($_SESSION['sesja']);
        
        /*** Zatrzymujemy sesje PERL ***/
        if(session_name()=='PHPSESSID'){
            if(isset($_COOKIE['PHPSESSID'])){
                $session_id = $_COOKIE['PHPSESSID'];
                $this->db->beginTransaction();
                $sql = "UPDATE main_users SET id_session='deleted' WHERE id_session='{$session_id}'";
                $this->db->exec($sql);
                $sql = "UPDATE sessions SET status='CLOSED', stop=NOW(), session_id=NULL WHERE session_id='{$session_id}'";
                $this->db->exec($sql); 
                $this->db->commit();
            }
        }
        /*******************************/
        
        $this->stopSE4Sess();
        
        return true;
    }
    
    /**
     *  Sprawdzenie czy użytkownik jest zalogowany, w przypadku gdy sesja wygasa
     *  a jest wymagana ustawia odpowiednia flagę
     */
    public function refresh_session()
    {
        /*** Kopiujemy sesję z PERL ***/
        $perl_sess = false;
        $no_perl = true;
        /*if(session_name()=='PHPSESSID'){
            $no_perl = false;
            if(isset($_COOKIE['PHPSESSID'])){
                $session_id = $_COOKIE['PHPSESSID'];
                $sql = "SELECT user_login FROM sessions WHERE session_id='{$session_id}' AND status='OPENED'";
                $username = $this->db->getOne($sql);
                if(isset($username) && $username){
                    $_SESSION['auth']['username'] = $username;
                    $_SESSION['auth']['session_time'] = time() + SESSION_EXPIRE;
                    $perl_sess = true;
                    $this->username = $username;
                }
            }
        }*///To już nie jest potrzebne
        /******************************/
        
        //Sprawdzenie czy jest to bezpieczna sesja
        if($this->access['auth_secure'] && isset($_SESSION['auth']['username']) && isset($_SESSION['auth']['secured_time'])){
            if( $_SESSION['auth']['secured_time'] > time() || $_SESSION['auth']['secured_time'] > time()){
                $_SESSION['auth']['session_time'] = time() + SESSION_EXPIRE;
                $this->username = $_SESSION['auth']['username'];
                if(isset($_SESSION['auth']['user_id']))
                    $this->user_id = $_SESSION['auth']['user_id'];
                $this->secured = true;
            }elseif($_SESSION['auth']['client_ip']!=$this->client_ip && $this->access['auth_secure']){
                $this->errors['ip_change'] = true;
                $this->errors['secured_expire'] = true;
                return false;
            }elseif($this->access['auth_secure']){
                $this->errors['secured_expire'] = true;
                return false;
            }
        }
        //Sprawdzenie czy jest to standardowa sesja
        elseif(isset($_SESSION['auth']['username']) && isset($_SESSION['auth']['session_time'])){
            if($_SESSION['auth']['session_time'] > time()){
                $_SESSION['auth']['session_time'] = time() + SESSION_EXPIRE;
                if(!$this->access['keep_secure'])
                    unset($_SESSION['auth']['secured_time']);
                $this->username = $_SESSION['auth']['username'];
                if(isset($_SESSION['auth']['user_id']))
                    $this->user_id = $_SESSION['auth']['user_id'];
                $this->secured = false;
            }elseif($this->access['auth_require']){
                $this->errors['session_expire'] = true;
                $this->stop_session();
                return false;
            }
        }
        
        //Uzupełnienie tablic sesyjnych
        if($this->username!=null && !isset($_SESSION['auth'])){
            $sql = "SELECT first_name, last_name, phys_license, user_type ".
                "FROM main_users WHERE user_login='{$this->username}'";
            $data = $this->db->getRow($sql);
            foreach($data as $key => $value){
                $this->auth[$key] = $value;
                $_SESSION['auth'][$key] = $value;
                $_SESSION['sesja'][$key] = $value;
            }
            $this->getCmsUserId();
        }elseif(isset($_SESSION['auth'])){
            $this->auth = $_SESSION['auth'];
        }
        
        //Sprawdzenie czy użytkownik nie posiada aktywnej flagi need_attention
        $system = new system($this->username);
        if($system->check_attention() && !$this->access['ticket_ignore']){
            //W przyszłości wywalać błąd
            //$this->errors['access_restrict'] = true;
            //A narazie redirect :/
            if(strpos($_SERVER['HTTP_HOST'], 'tst.esculap.pl')!==false || strpos($_SERVER['HTTP_HOST'], 'dev3.esculap.pl')!==false)
                header('Location: http://tst.esculap.pl/system/');
            else
                header('Location: http://www.esculap.pl/system/');
            exit();
        }
        
        //Ustawianie/przedłużanie długowiecznego ciastka
        $this->generateLongLiveCookie();
        
        //Sprawdzenie długowiecznego znacznika użytkownika
        $this->checkLongLiveCookie();
        
        return $this->check_access();
    }
    
    /**
     *  Sprawdzenie dostępu do strony, w przypadku błędu ustawia odpowiednią flagę błędu
     */
    public function check_access()
    {
        if($this->access['allow_bots'] && $this->is_allowed_bot()){
            if($this->access['save_bot_log']){ $this->write_log(); }//Zapis do logu
            return true;
        }elseif(!$this->access['auth_require']){
            if($this->access['save_log']){ $this->write_log(); }//Zapis do logu
            return true;
        }elseif(($this->username!=null && !$this->access['auth_secure']) || ($this->username!=null && $this->secured)){
            if(is_array($this->access['allow_uts']) && !in_array($this->auth['user_type'], $this->access['allow_uts'])){
                $this->errors['access_deny'] = true;
                return false;
            }
            if($this->access['allow_groups']==null){
                if($this->access['save_log']){ $this->write_log(); }//Zapis do logu
                return true;
            }
            $sql = "SELECT 1 FROM {$this->config['rel_user_group']} WHERE user_login = '{$this->username}' ".
                   "AND group_code IN ('".implode('\', \'', $this->access['allow_groups'])."') AND valid_until > NOW()";
            $result = $this->db->getOne($sql);
            if($result){
                if($this->access['save_log']){ $this->write_log(); }//Zapis do logu
                return true;
            }else{
                $this->errors['access_deny'] = true;
                return false;
            }
        }else{
            if($this->access['auth_secure']){
                $this->errors['not_secured'] = true;
            }else{
                $this->errors['not_logged'] = true;
            }
            return false;
        }
    }
    
    /**
     * Sprawdzenie czy użytkownik ma dostęp do podanej akcji
     * @param string $cont Nazwa wywoływanego kontrolera
     * @param string $action Nazwa wywoływanej akcji
     */
    public function checkCmsAccess($contr, $action)
    {
        if(!$this->access['check_action'])
            return true;
        if($this->username==null)
            return false;
        $sql = <<< ESKUEL
SELECT COUNT(1) 
FROM 
cms.main_users a 
INNER JOIN 
cms.rel_group_user b ON a.user_id=b.user_id 
INNER JOIN 
cms.rel_group_action c ON b.group_id=c.group_id 
INNER JOIN 
cms.main_actions d ON c.action_id=d.action_id 
WHERE a.user_login='{$this->username}' 
AND d.controller='$contr' 
AND d.action='$action' 
ESKUEL;
        if($this->db->getOne($sql))
            return true;
        $this->errors['action_access_deny'] = true;
        return false;
    }
    
    /**
     * 
     */
    public function getCmsUserId()
    {
        $sql = "SELECT user_id FROM cms.main_users WHERE user_login='{$this->username}'";
        $user_id = $this->db->getOne($sql);
        if($user_id)
            $_SESSION['auth']['user_id'] = $this->user_id = $user_id;
        return $user_id;
    }
    
    /**
     * 
     */
    public function getCmsUserGroups(){
        if(!$this->user_id) return null;
        $sql = <<< ESKUEL
SELECT b.group_id, b.group_name 
FROM cms.rel_group_user a 
INNER JOIN cms.dict_groups b 
ON a.group_id=b.group_id
WHERE a.user_id = '{$this->user_id}'
ESKUEL;
	    $access = null;
        foreach($this->db->getAll($sql) as $row){
            $access[$row['group_id']] = $row['group_name'];
        }
        return $access;
    }
    
    /**
     * 
     */
    public function getCmsUserLgroups(){
        if(!$this->user_id) return null;
        $sql = <<< ESKUEL
SELECT b.group_id, b.group_name 
FROM cms.rel_lgroup_user a 
INNER JOIN cms.dict_lgroups b 
ON a.group_id=b.group_id
WHERE a.user_id = '{$this->user_id}'
ESKUEL;
	    $access = null;
        foreach($this->db->getAll($sql) as $row){
            $access[$row['group_id']] = $row['group_name'];
        }
        return $access;
    }
    
    /**
     * Lista akcji zalogowanego użytkownika
     */
    public function cmsAccessList()
    {
        if($this->username==null)
            return null;
        $sql = <<< ESKUEL
SELECT DISTINCT d.controller, d.action 
FROM 
cms.main_users a 
INNER JOIN 
cms.rel_group_user b ON a.user_id=b.user_id 
INNER JOIN 
cms.rel_group_action c ON b.group_id=c.group_id 
INNER JOIN 
cms.main_actions d ON c.action_id=d.action_id 
WHERE a.user_login='{$this->username}' 
ESKUEL;
        $access = null;
        foreach($this->db->getAll($sql) as $row){
            $access[$row['controller']][$row['action']] = true;
        }
        return $access;
    }
    
    /**
     *  Sprawdza czy user ma dostęp do podanej akcji
     *  funkcja mocno powiązana z Controller.class.php
     *  @deprecated
     **/
    public function check_access_action($controller, $action)
    {
        $this->fill_funct();
        $this->fill_team();
        if(!$this->access['check_action'])
            return true;
        if($this->username==null)
            return false;
        $codes = implode("', '", $this->funct);
        $sql = "SELECT COUNT(1) as count, min(aa.resource_check::int) as check ".
               "FROM cms.actions_access aa JOIN cms.actions a ".
               "ON a.action_id=aa.action_id WHERE a.controller='{$controller}' ".
               "AND a.action='{$action}' AND aa.function_code IN ('{$codes}')";
        $res = $this->db->getRow($sql);
        if($res['count']>0){
            if($res['check']==0)
                $this->check_access = false;
            return true;
        }
        $this->errors['action_access_deny'] = true;
        return false;
    }
    
    /**
     *  Pobiera listę dostępnych akcji dla usera
     *  funkcja mocno powiązana z Controller.class.php
     *  @deprecated
     **/
    public function get_access()
    {
        $access = null;
        if(is_array($this->funct)){
            $codes = implode("', '", $this->funct);
            $sql = "SELECT a.controller, a.action FROM cms.actions_access aa ".
                   "JOIN cms.actions a ON a.action_id=aa.action_id WHERE ".
                   "aa.function_code IN ('{$codes}') GROUP BY a.controller, a.action";
            $data = $this->db->getAll($sql);
            foreach($data as $d){
                $access[$d['controller'].'__'.$d['action']] = true;
            }
        }
        return $access;
    }
    
    /**
     *  Pobiera kody funkcji pełnionych przez danego użytkownika
     *  funkcja mocno powiązana z Controller.class.php
     *  @deprecated
     **/
    private function fill_funct()
    {
        $sql = "SELECT function_code FROM {$this->config['function_table']} WHERE ".
               "user_login='{$this->username}'";
        $func_codes = $this->db->getAll($sql);
        $codes = array();
        if(is_array($func_codes)){
            foreach($func_codes as $fc){
                $codes[] = $fc['function_code'];
            }
        }
        $this->funct = $codes;
    }
    
    /**
     *  Pobiera zespoły do których należy użytkownik
     *  funkcja mocno powiązana z Controller.class.php
     *  @deprecated
     **/
    private function fill_team()
    {
        $sql = "SELECT team_code FROM {$this->config['team_table']} WHERE ".
               "user_login='{$this->username}'";
        $func_codes = $this->db->getAll($sql);
        $codes = array();
        if(is_array($func_codes)){
            foreach($func_codes as $fc){
                $codes[] = $fc['team_code'];
            }
        }
        $this->team = $codes;
    }
    
    /**
     *  Zwraca nazwę użytkownika
     */
    public function getUsername()
    {
        return $this->username;
    }
    
    /**
     *  Zwraca tablicę błędów
     */
    public function getErrors()
    {
        return $this->errors;
    }
    
    /**
     *  Sprawdzenie czy strone wywołuje bot wyszukiwarki
     */
    private function is_allowed_bot()
    {
        if(strpos($this->user_agent, "Googlebot") !== false)
            return true;
        if(strpos($this->user_agent, "msnbot") !== false)
            return true;
        if(strpos($this->user_agent, "Yahoo") !== false)
            return true;
        if(strpos($this->user_agent, "W3C_Validator") !== false)
            return true;
        return false;
    }
    
    private function write_log($co = '')
    {return;
        $server_name = $_SERVER['SERVER_NAME'];
        if($server_name=='esculap.pl')
            $server_name = 'www.esculap.pl';
	$logf = fopen('/data/web/logs/' . $server_name . '/' . date("Y.n.j"), "a");
	if (!strlen($co))
            $co = implode("\t", array(
                                date("G:i:s"),
				$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'],
				(isset($_SESSION['auth']['username'])?$_SESSION['auth']['username']:""),
				array_key_exists('HTTP_REFERER', $_SERVER)?$_SERVER['HTTP_REFERER']:'',
				$_SERVER['HTTP_USER_AGENT'],
				$_SERVER['REMOTE_ADDR'],
                                $this->tuser,
                                session_id(),
            ));
	fwrite($logf, $co . "\n");
	fclose($logf);
    }
    
    /**
     *  Uruchamia sesję socialEngin4 tylko jeśli użytkownik ma konto w SE
     **/
    public function startSE4Sess()
    {
        if(!$this->username)
            return;
        $s4db = PDOS::getInstance('social4', true);
        $sql = "SELECT COUNT(1) FROM engine4_users WHERE username LIKE '{$this->username}'";
        if(!$s4db->getOne($sql))
            return;
        $sql = "SELECT user_id FROM engine4_users WHERE username LIKE '{$this->username}'";
        $email = $s4db->getOne($sql);
        
        //Wpisy autoryzacji SE4 do sesji
        $_SESSION['Zend_Auth']['storage'] = $email;
        $_SESSION['auth']['SE4'] = true;
        
        //Statystyki logowań
        $czas = date('Y-m-d');
        $sql = "SELECT COUNT(1) FROM engine4_core_statistics WHERE type='user.logins' AND date>'{$czas} 00:00:00' AND date<'{$czas} 23:59:59'";
        $res = $s4db->getOne($sql);
        if($res)
            $sql = "UPDATE engine4_core_statistics SET value=value+1 WHERE type='user.logins' AND date>'{$czas} 00:00:00' AND date<'{$czas} 23:59:59'";
        else
            $sql = "INSERT INTO engine4_core_statistics (type, date, value) VALUES ('user.logins', '{$czas} 22:00:00', 1)";
        $s4db->exec($sql);
    }
    
    /**
     *  Wyłączanie sesji socialEngine4
     **/
    public function stopSE4Sess()
    {
        if(isset($_SESSION['Zend_Auth']))
            unset($_SESSION['Zend_Auth']);
        if(isset($_SESSION['auth']['SE4']))
            unset($_SESSION['auth']['SE4']);
    }
    
    /**
     *  Sprawdzanie czy sesja socialEngine4 istnieje
     **/
    public function SE4SessExists()
    {
        if(!$this->username)
            return false;
        $s4db = PDOS::getInstance('social4', true);
        $sql = "SELECT COUNT(1) FROM engine4_users WHERE username LIKE '{$this->username}'";
        if($s4db->getOne($sql) && isset($_SESSION['Zend_Auth']))
            return true;
        $this->stopSE4Sess();
        return false;
    }
}



?>