<?php

if(!defined('SESSION_EXPIRE'))
    define('SESSION_EXPIRE', 3600);
if(!defined('SECURED_EXPIRE'))
    define('SECURED_EXPIRE', 900);

require_once('PDOS.class.php');
require_once('model/user.tabs.php');

/**
 * 
 * Autoryzacja bazująca na bazie danych (nowy model) i działająca tak jak stara za razem
 * @author sroszczyk
 *
 */
class AuthorizeDB
{
    public $username = null;
    private $tuser = null;
    public $cookie_user = null;
    public $funct = array();
    public $team = array();
    public $user_id = null;
    public $data = null;
    public $client_id = null;
    
    /**
     * Instancja standardowego klienta SOAP w PHP 
     * @var object
     * @deprecated
     */
    private $soapClient = null;
    
    /**
     * Tymczasowy kod uzytkownika w autoryzacji zdalnej i statystykach
     * @var unknown_type
     */
    private $user_code = null;
    
    /**
     *  Dane dotyczące sesji użytkownika
     *  @deprecated
     **/
    public $sesja;
    
    /**
     * Czy sesja ma byc bezpieczna
     * @var boolean
     */
    public $secured = false;
    
    /**
     *      Ograniczenia dostępu
     *      'auth_require'  - gdy true akcja wymaga autoryzacji do systemu
     *      'auth_secure'   - gdy true akcja wymaga zabezpieczonej autoryzacji
     *      'keep_secure'   - utrzymuje sesję bezpieczną gdy nie jest wymagana
     *      'ticket_ignore' - gdy true akcja moze ziostac wykonana pomimo aktywnych ticketów
     *      'allow_groups'  - lista grup które mają dostęp, null oznacza że wszyscy mają dostęp
     *      'check_action'  - gdy true dokonywane jest głębsze sprawdzenie dostępu (cms)
     *      'allow_bots'    - gdy true wpuszczane będą boty wyszukiwarek
     *      'save_log'      - gdy true zapisuje wywołanie do logu
     *      'save_bot_log'  - gdy true zapisuje wywołanie bota do logu
     **/
    private $access = array(
                        'auth_require'  => false,
                        'auth_secure'   => false,
                        'keep_secure'   => false,
                        'ticket_ignore' => false,
                        'allow_groups'  => null,
                        'allow_uts'		=> null,
                        'check_action'  => false,
                        'allow_bots'    => true,
                        'save_log'      => true,
                        'save_bot_log'  => false,
                        );
    
    /**
     *  Dane konfiguracyjne autoryzacji
     */
    private $config = array(
                        'users_table' => 'main_users',
                        'group_table' => 'main_admin_groups',
                        'rel_user_group' => 'rel_user_admin_group',
                        'access_table' => 'cms.access_action',
                        'function_table' => 'cms.rel_user_function',
                        'team_table' => 'cms.rel_user_team'
                        );
    
    /**
     *  Tablica ostrzeżeń dotyczących autoryzacji, autoryzacja jest dokonywana
     */
    public $warnings = null;
    
    /**
     *  Tablica błędów autoryzacji, autoryzacja nie jest dokonywana
     */
    public $errors = null;
    
    /**
     *  Przeglądarka usera
     **/
    private $user_agent;
    
    /**
     *
     **/
    private $client_ip;
    
    /**
     *  KONSTRUKTOR
     **/
    public function __construct($access = array())
    {
        $this->user_agent = $_SERVER['HTTP_USER_AGENT'];
        $this->client_ip = $_SERVER['REMOTE_ADDR'];
        //Nadpisywanie ograniczeń dostępu
        if(isset($access['allow_groups']) && $access['allow_groups']!=null){
            if(!is_array($access['allow_groups'])){
                if(strpos($access['allow_groups'], ','))
                    $access['allow_groups'] = explode(',', $access['allow_groups']);
                elseif(strpos($access['allow_groups'], ';'))
                    $access['allow_groups'] = explode(';', $access['allow_groups']);
                else
                    $access['allow_groups'] = explode(' ', $access['allow_groups']);
            }
            for($i=0;$i<count($access['allow_groups']);$i++){
                $access['allow_groups'][$i] = trim($access['allow_groups'][$i]);
            }
        }
        foreach($access as $key => $value){
            if(key_exists($key,$this->access))
                $this->access[$key] = $access[$key];
        }
        if($this->access['auth_secure']){
            $this->access['auth_require'] = $this->access['auth_secure'];
        }
    }
    
    /**
     *  Tworzy długowieczne ciastko
     **/
    private function generateLongLiveCookie()
    {
        if($this->username && !isset($_COOKIE['ESCULAP_USER']))
            setcookie('ESCULAP_USER', md5('Long'.$this->username.'lifE').'+'.$this->username, time()+31000000, '/');
        return;
    }
    
    /**
     *  Sprawdza długowieczne ciastko
     **/
    private function checkLongLiveCookie()
    {
        if(isset($_COOKIE['ESCULAP_USER'])){
            $temp = explode('+', $_COOKIE['ESCULAP_USER']);
            $hash = $temp[0];
            $user = $temp[1];
            if(md5('Long'.$user.'lifE')==$hash)
                $this->tuser = $user;
        }
        return;
    }
    
    /**
     * Pobiera kod sesji uzytkownika
     * @param string $user_code
     */
    public function getUserCode(){
        return $this->user_code;
    }
    
    /**
     * Pobieranie sesji za pomocą kodu sesji
     * @param string $user_code
     */
    public function obtainSession($user_code){
        $this->errors = null;
        $this->warnings = null;
        
        try{
            $this->user->setUserCode($user_code);
            $this->data = $this->user->getUserData();
            $this->username = $this->data['user_login'];
        }catch(SoapFault $e){
            $this->errors['soap_fault'] = true;
            return false;
        }catch(SoapUserAuthorizeException $e){
            list($f, $msg) = explode('->', $e->getMessage());
            $this->errors[$msg] = true;
            return false;
        }catch(Exception $e){
            return false;
        }
        
        $MAC = new main_auth_code();
        $MAC->addCondition('auth_code', '=', $user_code);
        $MAC->addOrder('terminate', true);
        $MAC->search();
        if($MAC->getAmount()){
            $MAC->list2item();
        }

        //uzupelnienie tablicy sesji
        $this->data['username'] = $this->username;
        $this->data['session_time'] = time() + SESSION_EXPIRE;
        $this->data['client_ip'] = $this->user_ip;
        $this->data['user_code'] = $user_code;
        if($secured) $this->data['secured_time'] = time() + SECURED_EXPIRE;
        $_SESSION['auth'] = $this->data;
        
        //Ustawianie długowiecznego ciastka
        $this->generateLongLiveCookie();
        
        //Sprawdzenie długowiecznego znacznika użytkownika
        $this->checkLongLiveCookie();
        
        //Pobieranie danych dlka CMSa
        $this->getCmsUserId();
        
        //Startujemy sesję sociala
        $this->startSE4Sess();
        
        return true;
    }
    
    /**
     *  Rozpoczęcie sesji użytkownika
     */
    public function start_session($login, $pass = true, $secured = false)
    {
        $this->errors = null;
        $this->warnings = null;
        
        $user_system = isset($_SERVER['HTTP_USER_AGENT'])?$_SERVER['HTTP_USER_AGENT']:null;
        $user_proxy = isset($_SERVER['HTTP_VIA'])?$_SERVER['HTTP_VIA']:null;
        $user_ip = isset($_SERVER['REMOTE_ADDR'])?$_SERVER['REMOTE_ADDR']:null;
        
        //LOGOWANIE DRIRECT DB
        //Nalezy utrzymac zgodnosc z authorizeServer.class.php
        $user = new main_users($login);
        if(!$user->exists()){
            $this->errors['bad_login'] = true;
            return false;
        }
        if($pass!==true && $user->getField('password')!=$pass){
            $this->errors['bad_password'] = true;
            return false;
        }
        if($user->getField('real_status')=='NEW'){
            $this->errors['need_activation'] = true;
            return false;
        }
        if($user->getField('real_status')!='ACTIVE'){
            $this->errors['locked'] = true;
            return false;
        }
        $forbidden = array('password', 'desc');
        foreach($user->getFields() as $k=>$v){
            if(!in_array($k, $forbidden)) 
                $this->data[$k] = $v;
        }
        //zapis username
        $this->username = $this->data['username'] = $this->data['user_login'];
        //Zapis statystyk i kodow
        //$this->generateUserCode();
        $MAC = new main_auth_code();
        $MAC->addCondition('user_login', '=', $this->username);
        $MAC->addOrder('terminate', true);
        $MAC->search();
        if($MAC->getAmount()){
            $MAC->list2item();
        }else{
            $MAC->setField('user_login', $this->username);
            $MAC->setField('user_system', $user_system);
            $MAC->setField('user_proxy', $user_proxy);
            $MAC->setField('user_ip', $user_ip);
        }
        $max_time = intval(SESSION_EXPIRE*1.2/60);
        $MAC->setField('terminate', "SQL:NOW() + INTERVAL '{$max_time} MINUTES'");
        $MAC->saveItem();
        $this->user_code = $MAC->getField('auth_code');
        
        //uzupelnienie tablicy sesji
        $this->data['session_time'] = time() + SESSION_EXPIRE;
        $this->data['client_ip'] = $this->client_ip;
        $this->data['user_code'] = $this->user_code;
        if($secured) $this->data['secured_time'] = time() + SECURED_EXPIRE;
        $_SESSION['auth'] = $this->data;
        
        //Ustawianie długowiecznego ciastka
        $this->generateLongLiveCookie();
        
        //Sprawdzenie długowiecznego znacznika użytkownika
        $this->checkLongLiveCookie();
        
        //Pobieranie danych dlka CMSa
        $this->getCmsUserId();
        
        //Startujemy sesję sociala
        $this->startSE4Sess();
        
        return true;
    }
    
    /**
     *  Zakończenie sesji użytkownika
     */
    public function stop_session()
    {
        $MAC = new main_auth_code($this->user_code);
        $res = $MAC->deleteItem();
        $this->user_code = $this->username = $this->data = null;
        $this->secured = false;
        $this->user_id = null;
        unset($_SESSION['auth']);
        if(isset($_SESSION['Zend_Auth']))
            unset($_SESSION['Zend_Auth']);
        
        //Stopowanie sesji sociala
        $this->stopSE4Sess();
        
        return true;
    }
    
    /**
     * Odswiezanie sesji
     * @todo soap http version
     */
    public function refresh_session()
    {
        //Sprawdzenie długowiecznego znacznika użytkownika
        $this->checkLongLiveCookie();
        
        //sprawdze sesji
        if($this->access['auth_secure']){
            if(!isset($_SESSION['auth']['secured_time'])){
                $this->errors['not_secured'] = true;
                return false;
            }elseif($_SESSION['auth']['secured_time']<time()){
                $this->errors['secured_expire'] = true;
                return false;
            }else{
                $_SESSION['auth']['secured_time'] = time() + SECURED_EXPIRE;
                $this->secured = true;
            }
        }elseif(isset($_SESSION['auth']['secured_time']) && $_SESSION['auth']['secured_time']>time()){
            $_SESSION['auth']['secured_time'] = time() + SECURED_EXPIRE;
            $this->secured = true;
        }
        if($this->access['auth_require']){
            if(!isset($_SESSION['auth']['session_time'])){
                $this->errors['not_logged'] = true;
                return false;
            }elseif($_SESSION['auth']['session_time']<time()){
                $this->errors['session_expire'] = true;
                $this->errors['not_logged'] = true;
                $this->stop_session();
                return false;
            }
        }
        
        //Przedluzanie czasu sesji
        if(isset($_SESSION['auth']['username'])){
            $this->data = $_SESSION['auth'];
            $this->user_code = $_SESSION['auth']['user_code'];
            $this->username = $_SESSION['auth']['username'];
            $this->user_id = isset($_SESSION['auth']['user_id'])?$_SESSION['auth']['user_id']:null;
            $_SESSION['auth']['session_time'] = time() + SESSION_EXPIRE;
        }
        
        //Odswiezenie sesji + statystyki odslony
        if($this->user_code){
            $MAC = new main_auth_code($this->user_code);
            $max_time = intval(SESSION_EXPIRE*1.2/60);
            $MAC->setField('terminate', "SQL:NOW() + INTERVAL '{$max_time} MINUTES'");
            $MAC->saveItem();
        }
        
        $MAL = new main_users_log();
        if($this->username){
            $MAL->setField('user_login', $this->username);
            $MAL->setField('is_logged', true);
        }elseif($this->tuser){
            $MAL->setField('user_login', $this->tuser);
            $MAL->setField('is_logged', false);
        }
        if(isset($_SERVER['HTTP_HOST']))
            $MAL->setField('host', $_SERVER['HTTP_HOST']);
        if(isset($_SERVER['REQUEST_URI']))
            $MAL->setField('uri', $_SERVER['REQUEST_URI']);
        if(isset($_SERVER['HTTP_REFERER']))
            $MAL->setField('referer', $_SERVER['HTTP_REFERER']);
        if($MAL->getField('user_login'))
            $MAL->saveItem();
        
        //Dla CMS'a
        $this->getCmsUserId();
        
        return true;
    }
    
    /**
     * Sprawdzenie czy użytkownik ma dostęp do podanej akcji
     * @param string $cont Nazwa wywoływanego kontrolera
     * @param string $action Nazwa wywoływanej akcji
     */
    public function checkCmsAccess($contr, $action)
    {
        if(!defined('CMS_USAGE') || !constant('CMS_USAGE'))
            return null;
        
        if(!$this->access['check_action'])
            return true;
        if($this->username==null)
            return false;
        $sql = <<< ESKUEL
SELECT COUNT(1) 
FROM 
cms.main_users a 
INNER JOIN 
cms.rel_group_user b ON a.user_id=b.user_id 
INNER JOIN 
cms.rel_group_action c ON b.group_id=c.group_id 
INNER JOIN 
cms.main_actions d ON c.action_id=d.action_id 
WHERE a.user_login='{$this->username}' 
AND d.controller='$contr' 
AND d.action='$action' 
ESKUEL;
        if(@PDOS::_()->getOne($sql))
            return true;
        $this->errors['action_access_deny'] = true;
        return false;
    }
    
    /**
     * Sprawdzenie czy to uzytkownik CMSa tylko w CMSie dziala
     */
    public function getCmsUserId()
    {
        if(!defined('CMS_USAGE') || !constant('CMS_USAGE'))
            return;
        
        if(isset($this->data['user_id']))
            return $this->user_id = $this->data['user_id'];
        
        $sql = "SELECT user_id FROM cms.main_users WHERE user_login='{$this->username}'";
        $user_id = @PDOS::_()->getOne($sql);
        if($user_id)
            $this->data['user_id'] = $_SESSION['auth']['user_id'] = $this->user_id = $user_id;
        return $user_id;
    }
    
    /**
     * Lista akcji zalogowanego użytkownika
     */
    public function cmsAccessList()
    {
        if(!defined('CMS_USAGE') || !constant('CMS_USAGE') || $this->username==null)
            return null;
        $sql = <<< ESKUEL
SELECT DISTINCT d.controller, d.action 
FROM 
cms.main_users a 
INNER JOIN 
cms.rel_group_user b ON a.user_id=b.user_id 
INNER JOIN 
cms.rel_group_action c ON b.group_id=c.group_id 
INNER JOIN 
cms.main_actions d ON c.action_id=d.action_id 
WHERE a.user_login='{$this->username}' 
ESKUEL;
        $access = null;
        foreach($this->db->getAll($sql) as $row){
            $access[$row['controller']][$row['action']] = true;
        }
        return $access;
    }
    
    /**
     *  Zwraca nazwę użytkownika
     */
    public function getUsername()
    {
        return $this->username;
    }
    
    /**
     *  Zwraca tablicę błędów
     */
    public function getErrors()
    {
        return $this->errors;
    }
    
    /**
     *  Sprawdzenie czy strone wywołuje bot wyszukiwarki
     */
    private function is_allowed_bot()
    {
        if(strpos($this->user_agent, "Googlebot") !== false)
            return true;
        if(strpos($this->user_agent, "msnbot") !== false)
            return true;
        if(strpos($this->user_agent, "Yahoo") !== false)
            return true;
        if(strpos($this->user_agent, "W3C_Validator") !== false)
            return true;
        return false;
    }
    
    /**
     *  Uruchamia sesję socialEngin4 tylko jeśli użytkownik ma konto w SE
     **/
    public function startSE4Sess()
    {
        if(!$this->username)
            return;
        $s4db = PDOS::getInstance('social4', true);
        $sql = "SELECT COUNT(1) FROM engine4_users WHERE username LIKE '{$this->username}'";
        if(!$s4db->getOne($sql))
            return;
        $sql = "SELECT user_id FROM engine4_users WHERE username LIKE '{$this->username}'";
        $email = $s4db->getOne($sql);
        
        //Wpisy autoryzacji SE4 do sesji
        $_SESSION['Zend_Auth']['storage'] = $email;
        $_SESSION['auth']['SE4'] = true;
        
        //Statystyki logowań
        $czas = date('Y-m-d');
        $sql = "SELECT COUNT(1) FROM engine4_core_statistics WHERE type='user.logins' AND date>'{$czas} 00:00:00' AND date<'{$czas} 23:59:59'";
        $res = $s4db->getOne($sql);
        if($res)
            $sql = "UPDATE engine4_core_statistics SET value=value+1 WHERE type='user.logins' AND date>'{$czas} 00:00:00' AND date<'{$czas} 23:59:59'";
        else
            $sql = "INSERT INTO engine4_core_statistics (type, date, value) VALUES ('user.logins', '{$czas} 22:00:00', 1)";
        $s4db->exec($sql);
    }
    
    /**
     *  Wyłączanie sesji socialEngine4
     **/
    public function stopSE4Sess()
    {
        if(isset($_SESSION['Zend_Auth']))
            unset($_SESSION['Zend_Auth']);
        if(isset($_SESSION['auth']['SE4']))
            unset($_SESSION['auth']['SE4']);
    }
    
    /**
     *  Sprawdzanie czy sesja socialEngine4 istnieje
     **/
    public function SE4SessExists()
    {
        if(!$this->username)
            return false;
        $s4db = PDOS::getInstance('social4', true);
        $sql = "SELECT COUNT(1) FROM engine4_users WHERE username LIKE '{$this->username}'";
        if($s4db->getOne($sql) && isset($_SESSION['Zend_Auth']))
            return true;
        $this->stopSE4Sess();
        return false;
    }
}



?>
