<?php

if(!defined('SESSION_EXPIRE'))
    define('SESSION_EXPIRE', 3600);
if(!defined('SECURED_EXPIRE'))
    define('SECURED_EXPIRE', 900);

/**
 * 
 * Autoryzacja bazująca na SOAPie i działająca tak jak stara za razem
 * gdy SOAP przy autoryzacji zabroniony robi redirect HTTP
 * @author sroszczyk
 * @uses SoapUserAuthorize
 *
 */
class AuthorizeSOAP
{
    public $username = null;
    private $tuser = null;
    public $cookie_user = null;
    public $funct = array();
    public $team = array();
    public $user_id = null;
    public $data = null;
    public $client_id = null;
    
    /**
     * Czy sesja ma byc bezpieczna
     * @var boolean
     */
    public $secured = false;
    
    /**
     *      Ograniczenia dostępu
     *      'auth_require'  - gdy true akcja wymaga autoryzacji do systemu
     *      'auth_secure'   - gdy true akcja wymaga zabezpieczonej autoryzacji
     *      'keep_secure'   - utrzymuje sesję bezpieczną gdy nie jest wymagana
     *      'ticket_ignore' - gdy true akcja moze ziostac wykonana pomimo aktywnych ticketów
     *      'allow_groups'  - lista grup które mają dostęp, null oznacza że wszyscy mają dostęp
     *      'check_action'  - gdy true dokonywane jest głębsze sprawdzenie dostępu (cms)
     *      'allow_bots'    - gdy true wpuszczane będą boty wyszukiwarek
     *      'save_log'      - gdy true zapisuje wywołanie do logu
     *      'save_bot_log'  - gdy true zapisuje wywołanie bota do logu
     **/
    private $access = array(
                        'auth_require'  => false,
                        'auth_secure'   => false,
                        'keep_secure'   => false,
                        'ticket_ignore' => false,
                        'allow_groups'  => null,
                        'allow_uts'		=> null,
                        'check_action'  => false,
                        'allow_bots'    => true,
                        'save_log'      => true,
                        'save_bot_log'  => false,
                        );
    
    /**
     *  Dane konfiguracyjne autoryzacji
     */
    private $config = array(
                        'users_table' => 'main_users',
                        'group_table' => 'main_admin_groups',
                        'rel_user_group' => 'rel_user_admin_group',
                        'access_table' => 'cms.access_action',
                        'function_table' => 'cms.rel_user_function',
                        'team_table' => 'cms.rel_user_team'
                        );
    
    /**
     *  Tablica ostrzeżeń dotyczących autoryzacji, autoryzacja jest dokonywana
     */
    public $warnings = null;
    
    /**
     *  Tablica błędów autoryzacji, autoryzacja nie jest dokonywana
     */
    public $errors = null;
    
    /**
     *  Przeglądarka usera
     *  @var string
     **/
    private $user_system = null;
    
    /**
     * @vat string
     **/
    private $user_ip = null;
    
    /**
     * @var string
     */
    private $user_proxy = null;
    
    /**
     * @var object SoapUserAuthorize
     */
    private $user = null;
    
    /**
     *  KONSTRUKTOR
     **/
    public function __construct($access = array(), $site_code = null , $wsdl = null )
    {
		
        //Nadpisywanie ograniczeń dostępu
        if(isset($access['allow_groups']) && $access['allow_groups']!=null){
            if(!is_array($access['allow_groups'])){
                if(strpos($access['allow_groups'], ','))
                    $access['allow_groups'] = explode(',', $access['allow_groups']);
                elseif(strpos($access['allow_groups'], ';'))
                    $access['allow_groups'] = explode(';', $access['allow_groups']);
                else
                    $access['allow_groups'] = explode(' ', $access['allow_groups']);
            }
            for($i=0;$i<count($access['allow_groups']);$i++){
                $access['allow_groups'][$i] = trim($access['allow_groups'][$i]);
            }
        }
        foreach($access as $key => $value){
            if(key_exists($key,$this->access))
                $this->access[$key] = $access[$key];
        }
        if($this->access['auth_secure']){
            $this->access['auth_require'] = $this->access['auth_secure'];
        }
        
        $this->user_system = isset($_SERVER['HTTP_USER_AGENT'])?$_SERVER['HTTP_USER_AGENT']:null;
        $this->user_proxy = isset($_SERVER['HTTP_VIA'])?$_SERVER['HTTP_VIA']:null;
        $this->user_ip = isset($_SERVER['REMOTE_ADDR'])?$_SERVER['REMOTE_ADDR']:null;
        require_once 'SoapUserAuthorize.php';
		
		
		try{
			
			
			$this->user = new SoapUserAuthorize($site_code , null , $wsdl );
			
		} catch( Exception $e ) { }
    }
    
    /**
     *  Tworzy długowieczne ciastko
     **/
    private function generateLongLiveCookie()
    {
        if($this->username && !isset($_COOKIE['ESCULAP_USER']))
            setcookie('ESCULAP_USER', md5('Long'.$this->username.'lifE').'+'.$this->username, time()+31000000, '/');
        return;
    }
    
    /**
     *  Sprawdza długowieczne ciastko
     **/
    private function checkLongLiveCookie()
    {
        if(isset($_COOKIE['ESCULAP_USER'])){
            $temp = explode('+', $_COOKIE['ESCULAP_USER']);
            $hash = $temp[0];
            $user = $temp[1];
            if(md5('Long'.$user.'lifE')==$hash)
                $this->tuser = $user;
        }
        return;
    }
    
    /**
     * Czy soap mozna autoryzowac czy tez nie
     */
    public function isSoapAuthenticate(){
        try{
            return $this->user->isSoapAuthenticate();
        }catch(Exception $e){
            return false;
        }
    }
    
    /**
     * Uruchamianie sesji przekierowaniem HTTP
     */
    public function redirectSession(){
        $this->user->httpLogin();
    }
    
    /**
     * Zwraca kod sesji uzytkownika
     * @return string
     */
    public function getUserCode(){
        return $this->user->getUserCode();
    }
    
    /**
     * Pobieranie sesji za pomocą kodu sesji
     * @param string $user_code
     */
    public function obtainSession($user_code){
        $this->errors = null;
        $this->warnings = null;
        
        try{
            $this->user->setUserCode($user_code);
            $this->data = $this->user->getUserData();
            $this->username = $this->data['user_login'];
        }catch(SoapFault $e){
            $this->errors['soap_fault'] = true;
            return false;
        }catch(SoapUserAuthorizeException $e){
            list($f, $msg) = explode('->', $e->getMessage());
            $this->errors[$msg] = true;
            return false;
        }catch(Exception $e){
            return false;
        }

        //uzupelnienie tablicy sesji
        $this->data['username'] = $this->username;
        $this->data['session_time'] = time() + SESSION_EXPIRE;
        $this->data['client_ip'] = $this->user_ip;
        $this->data['user_code'] = $this->user->getUserCode();
        //$this->data['secured_time'] = time() + SECURED_EXPIRE;
        $_SESSION['auth'] = $this->data;
        
        //Ustawianie długowiecznego ciastka
        $this->generateLongLiveCookie();
        
        //Sprawdzenie długowiecznego znacznika użytkownika
        $this->checkLongLiveCookie();
        
        //Startujemy sesję sociala
        $this->startSE4Sess();
        
        return true;
    }
    
    /**
     * Uruchamianie sesji SOAPem
     * @param string $login
     * @param string $pass
     * @param boolean $secured
     */
    public function start_session($login, $pass, $secured = false){
        $this->errors = null;
        $this->warnings = null;
		
        try{
            $this->user->soapLogin($login, $pass);
            $this->data = $this->user->getUserData();
			
            $this->username = $this->data['user_login'];
        }catch(SoapFault $e){
            $this->errors['soap_fault'] = true;
			
            return false;
        }catch(SoapUserAuthorizeException $e){
			
			//trigger_error( $e->getMessage() );
			
            list($f, $msg) = explode('->', $e->getMessage());
            $this->errors[$msg] = true;
            return false;
        }catch(Exception $e){
			
            return false;
        }

        //uzupelnienie tablicy sesji
        $this->data['username'] = $this->username;
        $this->data['session_time'] = time() + SESSION_EXPIRE;
        $this->data['client_ip'] = $this->user_ip;
        $this->data['user_code'] = $this->user->getUserCode();
        if($secured) $this->data['secured_time'] = time() + SECURED_EXPIRE;
        $_SESSION['auth'] = $this->data;
        
        //Ustawianie długowiecznego ciastka
        $this->generateLongLiveCookie();
        
        //Sprawdzenie długowiecznego znacznika użytkownika
        $this->checkLongLiveCookie();
        
        //Pobieranie danych dlka CMSa
        //$this->getCmsUserId();
        
        //Startujemy sesję sociala
        $this->startSE4Sess();
        
        return true;
    }
    
    /**
     *  Zakończenie sesji użytkownika
     */
    public function stop_session()
    {
        $resp = false;
        try{
            $resp = $this->user->logout();
        }catch(Exception $e){
            $resp = false;
        }
        $this->username = $this->data = null;
        $this->secured = false;
        $this->user_id = null;
        unset($_SESSION['auth']);
//	$this->user->setUserCode(null); //DAREK
        
        //Stopowanie sesji sociala
        $this->stopSE4Sess();
        
        return $resp;
    }
    
    /**
     * Odswiezanie sesji
     * @todo soap http version
     */
    public function refresh_session()
    {
        //Sprawdzenie długowiecznego znacznika użytkownika
        $this->checkLongLiveCookie();
        
        //sprawdze sesji
        if($this->access['auth_secure']){
            if(!isset($_SESSION['auth']['secured_time'])){
                $this->errors['not_secured'] = true;
                return false;
            }elseif($_SESSION['auth']['secured_time']<time()){
                $this->errors['secured_expire'] = true;
                return false;
            }else{
                $_SESSION['auth']['secured_time'] = time() + SECURED_EXPIRE;
                $this->secured = true;
            }
        }elseif(isset($_SESSION['auth']['secured_time']) && $_SESSION['auth']['secured_time']>time()){
            $_SESSION['auth']['secured_time'] = time() + SECURED_EXPIRE;
            $this->secured = true;
        }
        if($this->access['auth_require']){
            if(!isset($_SESSION['auth']['session_time'])){
                $this->errors['not_logged'] = true;
                return false;
            }elseif($_SESSION['auth']['session_time']<time()){
                $this->errors['session_expire'] = true;
                $this->errors['not_logged'] = true;
                $this->stop_session();
                return false;
            }
        }
		
		
        
        //Przedluzanie czasu sesji
		
		
		
        if(isset($_SESSION['auth']['username'])){
            $this->data = $_SESSION['auth'];
            $this->user->setUserCode($this->data['user_code']);
            $this->username = $this->data['user_login'];
            $this->user_id = isset($this->data['user_id'])?$this->data['user_id']:null;
			
			
			
            $_SESSION['auth']['session_time'] = time() + SESSION_EXPIRE;
            
            //Sprawdzam kod soapem, powinien działać
            if(!$this->user->isLogged()){
                $this->stop_session();
                throw new Exception('reload');
                return false;
            }
        }
        
        //Odswiezenie sesji + statystyki odslony
        if($this->tuser){
            try{
                $this->user->refresh($this->tuser);
            }catch(Exception $e){
                $this->errors['soap_fault'] = true;
                return false;
            }
        }
        
        return true;
    }
    
    /**
     *  Zwraca nazwę użytkownika
     */
    public function getUsername()
    {
        return $this->username;
    }
    
    /**
     * 
     */
    public function getSecured()
    {
        return $this->secured;
    }
    
    /**
     *  Zwraca tablicę błędów
     */
    public function getErrors()
    {
        return $this->errors;
    }
    
    /**
     *  Sprawdzenie czy strone wywołuje bot wyszukiwarki
     */
    private function is_allowed_bot()
    {
        if(strpos($this->user_agent, "Googlebot") !== false)
            return true;
        if(strpos($this->user_agent, "msnbot") !== false)
            return true;
        if(strpos($this->user_agent, "Yahoo") !== false)
            return true;
        if(strpos($this->user_agent, "W3C_Validator") !== false)
            return true;
        return false;
    }
    
	/**
     *  Uruchamia sesję socialEngin4 tylko jeśli użytkownik ma konto w SE
     **/
    public function startSE4Sess()
    {
        if(!$this->username)
            return;
        $s4db = PDOS::getInstance('social4', true);
        $sql = "SELECT COUNT(1) FROM engine4_users WHERE username LIKE '{$this->username}'";
        if(!$s4db->getOne($sql))
            return;
        $sql = "SELECT user_id FROM engine4_users WHERE username LIKE '{$this->username}'";
        $email = $s4db->getOne($sql);
        
        //Wpisy autoryzacji SE4 do sesji
        $_SESSION['Zend_Auth']['storage'] = $email;
        $_SESSION['auth']['SE4'] = true;
        
        //Statystyki logowań
        $czas = date('Y-m-d');
        $sql = "SELECT COUNT(1) FROM engine4_core_statistics WHERE type='user.logins' AND date>'{$czas} 00:00:00' AND date<'{$czas} 23:59:59'";
        $res = $s4db->getOne($sql);
        if($res)
            $sql = "UPDATE engine4_core_statistics SET value=value+1 WHERE type='user.logins' AND date>'{$czas} 00:00:00' AND date<'{$czas} 23:59:59'";
        else
            $sql = "INSERT INTO engine4_core_statistics (type, date, value) VALUES ('user.logins', '{$czas} 22:00:00', 1)";
        $s4db->exec($sql);
    }
    
    /**
     *  Wyłączanie sesji socialEngine4
     **/
    public function stopSE4Sess()
    {
        if(isset($_SESSION['Zend_Auth']))
            unset($_SESSION['Zend_Auth']);
        if(isset($_SESSION['auth']['SE4']))
            unset($_SESSION['auth']['SE4']);
    }
    
    /**
     *  Sprawdzanie czy sesja socialEngine4 istnieje
     **/
    public function SE4SessExists()
    {
        if(!$this->username)
            return false;
        $s4db = PDOS::getInstance('social4', true);
        $sql = "SELECT COUNT(1) FROM engine4_users WHERE username LIKE '{$this->username}'";
        if($s4db->getOne($sql) && isset($_SESSION['Zend_Auth']))
            return true;
        $this->stopSE4Sess();
        return false;
    }

public function getData() {
    return $this->data;
}

}