<?php

/******************************************************************************
 *                                                                            *
 *      Klasa obsługująca wyświetlanie banerów                                *
 *      @uses dimage.class.php                                                *
 *      @category graphic                                                     *
 *      @author Sławek                                                        *
 *                                                                            *
 *                                                                            *
 ******************************************************************************/

require_once('dimage.class.php');

class banners
{
    /** Uchwyt do bazy danych **/
    private $db;
    
    /** Identufykator miejsca na banery, strony na której się znajduje i użytkownika oglądającego stronę **/
    private $holder_id = null;
    private $site_id = null;
    private $user_login = null;
    private $controller = null;
    private $action = null;
    private $lang = null;
    
    /** Maksymalna szerokość, wysokość i ilość slotów **/
    private $max_width = null;
    private $max_height = null;
    private $max_slots = null;
    
    /** Status holdera **/
    private $debug = false;
    private $disabled = true;
    private $readonly = false;
    
    /** Instancja testowa **/
    private $dev_mode = false;
    private $test = 0;
    
    /**
     *  Konstruktor,
     *  @param string $holder_id - identyfikator miejsca wpięcia banerów
     *  @param integer $max_width - maksymalna szerokość obszaru dla banerów
     *  @param integer $max_height - maksymalna wysokość obszaru dla banerów
     *  @param integer $max_slots - maksymalna ilość banerów w obszarze
     *  @param boolean $dev - tryb developerski (wymusza bazę testową) UWAGA: ten parametr tylko dla Sławka ;-)
     *  @global object $db - uchwyt do bazy danych
     **/
    public function __construct($holder_id = null, $max_width = null, $max_height = null, $max_slots = null, $dev = false)
    {
        global $db;
        $this->db = $db;
        
        if($this->db->isDevel() || $dev)
            $this->dev_mode = true;
        if($this->dev_mode){
            if(!$this->db->isDevel())
                $this->db = DB_connect('burak');
        }
        
        if($holder_id==null)
            return null;
        
        $this->holder_id = $holder_id;
        $this->user_login = CONTR_USER;
        $this->controller = CONTR_NAME;
        $this->action = CONTR_ACTION;
        $this->lang = CONTR_LANG;
        
        if(defined('SITE_ID')){
            $this->site_id = SITE_ID;
        }elseif(defined('SYSTEM_NAME') && SYSTEM_NAME=='CMS'){
            $this->site_id = null;
            $this->readonly = true;
        }else{
            trigger_error('BANNER_CLASS: wymaga zdefiniowania stałej SITE_ID powiązanej z kontrolerem', E_USER_WARNING);
            return null;
        }
        
        $sql = "SELECT COUNT(1) FROM main_banners_holders WHERE holder_id='{$this->holder_id}'";
        $holder_exists = ($this->db->getOne($sql)>0)?true:false;
        
        if(!$holder_exists)
            $dev = true;
        
        if($dev && ($max_width==null || $max_height==null || $max_slots==null)){
            trigger_error('BANNER_CLASS: nie podano wysokości, szerokości i ilości miejsc', E_USER_WARNING);
            return null;
        }
        
        $this->max_width = $max_width;
        $this->max_height = $max_height;
        $this->max_slots = $max_slots;
        
        if($this->readonly)
            $this->loadHolder();
        elseif($holder_exists)
            if($dev)
                $this->updateHolder();
            else
                $this->loadHolder();
        else
            $this->saveHolder();
    }
    
    /**
     *  Odczyt banerów z bazy
     **/
    private function loadHolder()
    {
        $sql = <<< ESQUEL
SELECT * FROM main_banners_holders WHERE holder_id = '{$this->holder_id}' 
ESQUEL;
        $data = $this->db->getRow($sql);
        $this->disabled = true;
        if(!$this->readonly){
            if($data['site_id']!=$this->site_id){
                trigger_error('BANNER_CLASS: site_id nie zgadza się z tym w bazie danych, holder może być umieszczony tylko na jednej stronie ['.$data['site_id'].'!='.$this->site_id.'0]', E_USER_WARNING);
                return null;
            }
            $sql = "UPDATE main_banners_holders SET last_use=NOW() WHERE holder_id = '{$this->holder_id}'";
            $this->db->exec($sql);
        }
        $this->max_width = $data['max_width'];
        $this->max_height = $data['max_height'];
        $this->max_slots = $data['max_slots'];
        $this->disabled = false;
    }
    
    /**
     *  Aktualizacja miejsca z banerami
     **/
    private function updateHolder()
    {
        $sql = <<< ESQUEL
UPDATE main_banners_holders SET 
max_width = '{$this->max_width}', 
max_height = '{$this->max_height}', 
max_slots = '{$this->max_slots}', 
site_id = '{$this->site_id}' 
WHERE holder_id = '{$this->holder_id}'
ESQUEL;
        $res = $this->db->exec($sql);
        if($res)
            $this->disabled = false;
    }
    
    /**
     *  Zapis do bazy miejsca banerów
     **/
    private function saveHolder()
    {
        $sql = <<< ESQUEL
INSERT INTO main_banners_holders 
(holder_id, max_width, max_height, max_slots, site_id) 
VALUES 
('{$this->holder_id}', '{$this->max_width}', '{$this->max_height}', '{$this->max_slots}', '{$this->site_id}') 
ESQUEL;
        $res = $this->db->exec($sql);
        if($res)
            $this->disabled = false;
    }
    
    /**
     *  pobiera banery powiązane z holderem
     *  @return array - tablica banerów
     **/
    private function getBanners()
    {
        $user_login = $this->user_login;
        
        $warunek_zalogowania = ($user_login)?'rbhs.for_logged=true':'rbhs.for_nlogged=true';
        
        $sql = <<< ESQUEL
SELECT rbhs.*, mb.* 
FROM rel_banners_holders_campains rbhs 
INNER JOIN main_banners2 mb ON rbhs.banner_id=mb.banner_id 
INNER JOIN main_banners_campains mc ON rbhs.campain_id=mc.campain_id 
WHERE rbhs.holder_id='{$this->holder_id}' AND {$warunek_zalogowania} 
AND mc.ready=true AND (mc.start_date<NOW() AND mc.start_date IS NOT NULL) 
AND (mc.end_date>NOW() OR mc.end_date IS NULL) 
ORDER BY rbhs.h_position, rbhs.r_position 
ESQUEL;
        $data = $this->db->getAll($sql);
        $data2 = null;
        $rotations = null;
        $cookie_rot = null;
        
        //Szukanie rotacji w banerze
        foreach($data as $banner){
            $rotations[$banner['h_position']][] = $banner['r_position'];
        }
        
        //Odczytywanie już zapisanych rotacji z ciastka
        if(!$this->readonly && isset($_SESSION['banners_rotation'])){
            $cookie_rot = $_SESSION['banners_rotation'];
        }
        
        //Przepisywanie banerów
        $hpos = 0;
        foreach($data as $banner){
            if($hpos == $banner['h_position'])
                continue;
            $rot_amount = count($rotations[$banner['h_position']]);
            if($rot_amount==1){
                $data2[] = $banner;
            }else{
                if(!isset($cookie_rot[$this->holder_id.'_'.$banner['h_position']])){
                    $cookie_rot[$this->holder_id.'_'.$banner['h_position']] = $banner['r_position'];
                    $data2[] = $banner;
                    $hpos = $banner['h_position'];
                }elseif($cookie_rot[$this->holder_id.'_'.$banner['h_position']]==$rotations[$banner['h_position']][$rot_amount-1]){
                    $cookie_rot[$this->holder_id.'_'.$banner['h_position']] = $banner['r_position'];
                    $data2[] = $banner;
                    $hpos = $banner['h_position'];
                }elseif($banner['r_position']>$cookie_rot[$this->holder_id.'_'.$banner['h_position']]){
                    $cookie_rot[$this->holder_id.'_'.$banner['h_position']] = $banner['r_position'];
                    $data2[] = $banner;
                    $hpos = $banner['h_position'];
                }
            }
        }
        //Zapis rotacji do ciastka
        if(!$this->readonly){
            $_SESSION['banners_rotation'] = $cookie_rot;
        }
        return $data2;
    }
    
    /**
     *  Tworzy kod HTML wyświetlający dany holder
     *  @param string $style - atrybut style dla kontenera zawierającego banery
     *  @param string $class - atrybut class dla kontenera zawierającego banery
     **/
    public function generateHolderCode($style = '', $class = '')
    {
        $style = '';
        if($this->debug)
            $style .= "border:1px solid red; height:{$this->max_height}px;";
        if(strlen($class))
            $class = " class=\"{$class}\"";
        
        $banners = $this->getBanners();
        
        $code = "<div style=\"width:{$this->max_width}px;{$style}\"{$class}>";
        if($this->disabled)
            $code .= "Holder zablokowany";
        elseif($this->debug)
            $code .= "{$this->max_width}X{$this->max_height}";
        else{
            if(is_array($banners))
                foreach($banners as $banner){
                    $code .= $this->generateBannerCode(array('banner_id' => $banner['banner_id'], 'holder_id' => $banner['holder_id'], 'campain_id' => $banner['campain_id']));
                    //Podbicie statystyk
                    $this->increaseReveals($banner['banner_id'], $banner['holder_id'], $banner['campain_id']);
                }
        }
        $code .= "</div>";
        return $code;
    }
    
    /**
     *  Pobiera pełen rekord danych o banerze
     *  @param integer $banner_id - identyfikator banera
     *  @param string $holder_id - identyfikator holdera
     *  @param integer $campain_id - identyfikator kampanii
     *  @return array - pełne info o banerze
     **/
    private function getBanner($banner_id, $holder_id = null, $campain_id = null)
    {
        if($holder_id == null || $campain_id == null)
            $sql = <<< ESQUEL
SELECT mb.banner_id, mb.width, mb.height, mb.border, mb.target, mb.alt, mb.title, 
mb.link, mb.type, mb.lang, mb.text, mb.style, mb.flash_vars 
FROM main_banners2 mb 
WHERE mb.banner_id='{$banner_id}' 
ESQUEL;
        else
            $sql = <<< ESQUEL
SELECT mb.banner_id, mb.width, mb.height, mb.border, mb.target, mb.alt, mb.title, 
mb.link, mb.type, mb.lang, mb.text, mb.style, mb.flash_vars, r.* 
FROM main_banners2 mb 
LEFT OUTER JOIN rel_banners_holders_campains r ON mb.banner_id=r.banner_id AND r.campain_id='{$campain_id}' AND r.holder_id='{$holder_id}' 
WHERE mb.banner_id='{$banner_id}' 
ESQUEL;
        $data = $this->db->getRow($sql);
        $dimage = new dimage();
        $image = $dimage->get_related_images('banner', $banner_id);
        if($image)
            $data['image_name'] = $image[0]['image_name'];
        return $data;
    }
    
    /**
     *  Tworzy kod HTML odpowiadający danemu banerowi
     *  @param array $data - tablica z dantymi banera
     *  @return string - kod HTML, otoczka dla banera
     **/
    public function generateBannerCode($data)
    {
        if(is_numeric($data) && $data>0)
            $data = $this->getBanner($data, null, null);
        elseif(isset($data['banner_id']) && isset($data['holder_id']) && isset($data['campain_id']))
            $data = $this->getBanner($data['banner_id'], $data['holder_id'], $data['campain_id']);
        
        if($data['type']!='text' && (!isset($data['image_name']) || empty($data['image_name'])) && isset($data['banner_id'])){
            $dimage = new dimage();
            $image = $dimage->get_related_images('banner', $data['banner_id']);
            if($image)
                $data['image_name'] = $image[0]['image_name'];
        }
        
        if(isset($data['banner_id']) && isset($data['holder_id']) && isset($data['campain_id']))
            $link = "http://www.esculap.pl/banner/{$data['banner_id']},{$data['holder_id']},{$data['campain_id']}/";
        else
            $link = $data['link'];
        
        $code = null;
        switch($data['type']){
            case 'v2':
                $code = str_replace($data['link'], $link, $data['text']);
                break;
            case 'flash':
                $flash_vars = unserialize($data['flash_vars']);
                if(is_array($flash_vars) && count($flash_vars)>0){
                    $temp = null;
                    if(isset($flash_vars['target'])){
                        $temp[] = $flash_vars['target'].'='.$data['target'];
                    }
                    if(isset($flash_vars['link'])){
                        $temp[] = $flash_vars['link'].'='.$link;
                    }
                    if(is_array($temp)){
                        $temp = implode('&', $temp);
                        $flash_vars = "\n<param name=\"flashVars\" value=\"{$temp}\" />";
                    }else
                        $flash_vars = '';
                }else
                    $flash_vars = '';
                $code = <<< CODE
<!--[if !IE]> -->
<object type="application/x-shockwave-flash"
  data="http://www.esculap.pl/image/{$data['image_name']}/" width="{$data['width']}" height="{$data['height']}">
<!-- <![endif]-->
<!--[if IE]>
<object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"
  codebase="http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=6,0,0,0"
  width="{$data['width']}" height="{$data['height']}">
  <param name="movie" value="http://www.esculap.pl/image/{$data['image_name']}/" />
<!--><!--dgx-->{$flash_vars}
  <param name="wmode" value="transparent">
  <param name="loop" value="true" />
  <param name="menu" value="false" />
  <param name="allowFullScreen" value="false" />
  <param name="allowScriptAccess" value="always" />
  <param name="quality" value="high" />
  <p>{$data['alt']}</p>
</object>
<!-- <![endif]-->
CODE;
                break;
            case 'image':
                $border = isset($data['border'])?"border:{$data['border']}px solid black;":'';
                $width = isset($data['width'])?"width:{$data['width']}px;":'';
                $height = isset($data['height'])?"height:{$data['height']}px;":'';
                $code = <<< CODE
<a href="{$link}" target="{$data['target']}" style="text-decoration:none;">
    <img src="http://www.esculap.pl/image/{$data['image_name']},{$data['width']},{$data['height']}/" style="{$width}{$height}{$border}{$data['style']}" title="{$data['title']}" alt="{$data['alt']}"/>
</a>
CODE;
                break;
            case 'text':
                $border = isset($data['border'])?"border:{$data['border']}px solid black;":'';
                $width = isset($data['width'])?"width:{$data['width']}px;":'';
                $height = isset($data['height'])?"height:{$data['height']}px;":'';
                $code = <<< CODE
<a href="{$link}" target="{$data['target']}" style="text-decoration:none;display:block;{$width}{$height}{$border}{$data['style']}" title="{$data['title']}">
    {$data['text']}
</a>
CODE;
                break;
        }
        return $code;
    }
    
    /**
     *  Nalicza odsłonę banera
     *  @param integer $banner_id - identyfikator banera
     *  @param string $holder_id - identyfikator holdera
     *  @param integer $campain_id - identyfikator kampanii
     **/
    private function increaseReveals($banner_id, $holder_id, $campain_id)
    {
        if(!$this->readonly){
            $sql = "UPDATE rel_banners_holders_campains SET reveals=reveals+1 ".
            "WHERE banner_id='{$banner_id}' AND holder_id='{$holder_id}' AND campain_id='{$campain_id}'";
            $this->db->exec($sql);
        }
    }
    
    /**
     *  Nalicza kliknięcie w baner
     *  @param integer $banner_id - identyfikator banera
     *  @param string $holder_id - identyfikator holdera
     *  @param integer $campain_id - identyfikator kampanii
     **/
    private function increaseClicks($banner_id, $holder_id, $campain_id)
    {
        if(!$this->readonly){
            $sql = "UPDATE rel_banners_holders_campains SET clicks=clicks+1 ".
            "WHERE banner_id='{$banner_id}' AND holder_id='{$holder_id}' AND campain_id='{$campain_id}'";
            $this->db->exec($sql);
        }
    }
    
    /**
     *  Pobiera link na jaki ma kierować kliknięty baner
     *  uwaga: używać tylko w skrypcie przekierowania do prawdziwego linku!!!
     *  @param integer $banner_id - identyfikator banera
     *  @param string $holder_id - identyfikator holdera
     *  @param integer $campain_id - identyfikator kampanii
     *  @return string - link na który kieruje baner
     **/
    public function getBannerLink($banner_id, $holder_id, $campain_id)
    {
        $sql = "SELECT COUNT(1) FROM rel_banners_holders_campains WHERE banner_id='{$banner_id}' AND campain_id='{$campain_id}' AND holder_id='{$holder_id}'";
        if($this->db->getOne($sql)==0)
            return false;
        $data = $this->getBanner($banner_id, $holder_id, $campain_id);
        $this->increaseClicks($banner_id, $holder_id, $campain_id);
        return $data['link'];
    }
}

?>