<?php

require_once 'PDOS.class.php';

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Klasa abstrakcji obiektów prosto z bazy dla CMS 
 * (tylko zarządzanie, staty są zbyt liczne dla tego [agregacja])
 * @author sroszczyk
 * @abstract
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
abstract class dbObj
{
    protected $table = null;//Nazwa tabeli
    protected $pkey = null;//Klucz glowny
    protected $get = array();//Co mozna pobierac
    protected $set = array();//Co mozna ustawiac
    protected $addit = array();//Wartości obliczane poza SQL
    protected $boolean = array();//Pola bollowskie true/false, bez możliwości null (null -> false)
    protected $data = null;//Rekord aktualizowany na biezaco
    protected $base = null;//Rekord synchroniczny z baza danych
    protected $exist = false;//Istnieje czy tez nie
    protected $full = false;//Pobierac wszystkie informacje (powiazane podobiekty $rel)
    protected $rel = null;//Obiekty powiazane (listy obiektów) @deprecaded
    protected $parent = null;//Obiekty nadżędne (tablica obiektów $this->parents['nazwa'])
    protected $childs = null;//Obiekty podżędne (tablica tablic obiektów $this->childs['nazwa']['iteracja'])
    protected $debug = false;//Zapisywanie histori operacji
    protected $hist = null;//Historia operacji
    protected $errors = null;//Tablica błędów
    
    /**
     * Konstruktor z inicjacja danych z bazy, jesli istnieja
     */
    public function __construct($id = null, $full = false)
    {
        $this->full = $full;
        $this->data[$this->pkey] = null;
        if($id){
            $this->data[$this->pkey] = $id;
            $this->load();
        }
    }
    
    /**
     * 
     */
    public function exist()
    {
        return $this->exist;
    }
    
    /**
     * Wczytanie z bazy danych
     **/
    public function load()
    {
        if(!$this->data[$this->pkey])
            return false;
        
        $sql = "SELECT ".'"'.implode('", "', $this->get).'"'." FROM {$this->table} WHERE {$this->pkey}='{$this->data[$this->pkey]}'";
        $data = PDOS::getInstance()->getRow($sql);
        if(!$data)
            return false;
        $this->saveHist($sql);
        foreach($data as $k=>$v)
            if(in_array($k, $this->boolean))
                $data[$k] = $v?true:false;
            else
                $data[$k] = stripslashes($v);
        $this->errors = null;
        $this->exist = true;
        $this->data = $data;
        $this->additional();
        $this->base = $this->data;
        if($this->full)
            $this->fullLoad();
        return true;
    }
    
    /**
     * Obliczanie wartości dodatkowych pol
     * @abstract
     */
    protected function additional(){}
    
    /**
     * Zapis do bazy
     **/
    public function save($novalidate = false)
    {
        if(!$novalidate && !$this->validate()) return false;
        $data = null;
        foreach($this->data as $k=>$v)
            if(in_array($k, $this->set))
                $data[$k] = is_string($v)?addslashes($v):$v;
        $result = false;
        if(isset($this->data[$this->pkey]) && $this->data[$this->pkey]){
            $save = array();
            foreach($data as $k => $v){
                if($v===true)
                    $save[] = "\"{$k}\"=TRUE";
                elseif($v===false)
                    $save[] = "\"{$k}\"=FALSE";
                elseif($v)
                    $save[] = "\"{$k}\"='{$v}'";
                else
                    $save[] = "\"{$k}\"=NULL";
            }
            $sql = "UPDATE {$this->table} SET ".
                implode(', ', $save).' '.
                "WHERE {$this->pkey}='{$this->data[$this->pkey]}'";
            $result = PDOS::getInstance()->exec($sql);
            $this->saveHist($sql);
        }else{
            $sql = "INSERT INTO {$this->table} (".
                '"'.implode('", "', array_keys($data)).'"'.
                ") VALUES (".
                str_replace("''", 'NULL', "'".implode("', '", array_values($data))."'").
                ") RETURNING {$this->pkey}";
            $result = $this->data[$this->pkey] = PDOS::getInstance()->getOne($sql);
            $this->saveHist($sql);
        }
        if($result){
            $this->base = $this->data;
            $this->exist = true;
            return true;
        }
        return false;
    }
    
    /**
     * Usuwanie z bazy
     **/
    public function delete()
    {
        if($this->exist && $this->data[$this->pkey]){
            $sql = "DELETE FROM {$this->table} WHERE {$this->pkey}={$this->data[$this->pkey]}";
            PDOS::getInstance()->exec($sql);
            $this->saveHist($sql);
            $this->base = null;
            $this->exist = false;
            return true;
        }
        return false;
    }
    
    /**
     * Zwracanie pol
     **/
    public function get($key = null, $from_base = false)
    {
        $this->saveHist("GET: k=$key, fb=".($from_base?'tak':'nie'));
        if($from_base)
            $data = $this->base;
        else
            $data = $this->data;
        
        if(!$key)
            return $data;
        
        if($key=='id')
            return $this->data[$this->pkey];
        
        if(!in_array($key, $this->get) && !in_array($key, $this->addit))
            return false;
        
        return $data[$key];
    }
    
    /**
     * Ustawianie pol
     **/
    public function set($key, $value)
    {
        $this->saveHist("SET: k=$key, v=".substr($value, 0, 100));
        if($key==$this->pkey || $key=='id'){
            $this->data = null;
            $this->data[$this->pkey] = $value;
            $this->load();
            return true;
        }
        
        if(!in_array($key, $this->set))
            return false;
        
        if(in_array($key, $this->boolean))
            $value = $value?true:false;
        
        $this->data[$key] = $value;
        return true;
    }
    
    /**
     * Pozwala na ustawianie nie ustawialnych pól ;)
     */
    public function forceSet($key, $value)
    {
        if(in_array($key, $this->get))
            $this->set[] = $key; 
        return $this->set($key, $value);
    }
    
    /**
     * Ladowanie obiektow powiazanych
     * @abstract
     **/
    public function fullLoad(){}
    
    /**
     * Walidowanie
     * @abstract
     * @return boolean true = dobrze, false = zle
     */
    public function validate()
    {
        return true;
    }
    
    /**
     * Pobieranie listy bledow
     */
    public function getErrors()
    {
        return $this->errors;
    }
    
    /**
     * Pobieranie obiektów powiązanych
     * @deprecated
     **/
    public function getRel($key = null)
    {
        if($key){
            if(isset($this->rel[$key])) return $this->rel[$key];
            return null; 
        }
        return $this->rel;
    }
    
    /**
     * Pobieranie obiektów nadzednych (obiekty lub tablice)
     * @param string $key 
     * @param boolean $as_object
     * @return array/object
     */
    public function getParent($key = null, $as_object = false)
    {
        if($as_object){
            if($key){
                if(isset($this->parent[$key])) 
                    return $this->parent[$key];
            }else{
                return $this->parent;
            }
        }else{
            if($key){
                if(isset($this->parent[$key])) 
                    return $this->parent[$key]->get();
            }else{
                $tab = null;
                foreach($this->parent as $k=>$v)
                    $tab[$k] = $v->get();
                return $tab;
            }
        }
        return null;
    }
    
    /**
     * Pobieranie obiektów podrzędnych
     * @param string $key 
     * @param boolean $as_object
     * @return array
     */
    public function getChilds($key = null, $as_object = false)
    {
        if($as_object){
            if($key){
                if(isset($this->childs[$key])) 
                    return $this->childs[$key];
            }else{
                return $this->childs;
            }
        }else{
            if($key){
                if(isset($this->childs[$key])){
                    $tab = null;
                    foreach($this->childs[$key] as $k=>$v)
                        $tab[$k] = $v->get();
                    return $tab;
                }
            }else{
                $tab = null;
                foreach($this->childs as $k=>$v)
                    foreach($v as $k2=>$v2)
                        $tab[$k][$k2] = $v2->get();
                return $tab;
            }
        }
        return null;
    }
    
    /**
     * 
     **/
    public function setFull($full = false)
    {
        $this->full = $full;
    }
    
    /**
     * Przestawianie trybu debugera
     **/
    public function debug($debug)
    {
        $this->debug = $debug;
    }
    
    /**
     * Zapisywanie historii operacji
     **/
    protected function saveHist($event)
    {
        if($this->debug)
            $this->hist[] = $event;
    }
    
    /**
     * Wyswietlenie historii
     **/
    public function show($show = true)
    {
        if($show){
            echo '<div style="text_align:left;padding:20px;width:100%"><h2>Historia operacji:</h2><pre>';
            echo 'Debug:';var_dump($this->debug);echo "\n";
            print_r($this->hist);
            echo '<h3>Dane aktualne:</h3>';
            var_dump($this->data);
            echo '<h3>Dane syncDB:</h3>';
            var_dump($this->base);
            echo '</pre></div>';
            return;
        }else{
            return $this->hist();
        }
    }
}

/**
 * Przerabianie listy obiektów w listę tablic ;)
 * @deprecated
 */
function ol2al($ol)
{
    $al = null;
    if(is_array($ol)){
        foreach($ol as $obj){
            if(is_object($obj) && get_parent_class($obj)=='dbObj')
                $al[] = $obj->get();
        }
    }
    return $al;
}

?>