<?php

require_once 'dbObj.class.php';

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Klasa obsługuje set emailingu
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_set extends dbObj
{
    protected $table = 'emailing.em_sets';
    protected $pkey = 'set_id';
    protected $get = array('set_id', 'add_date', 'name', 'title', 'desc', 'user_id', 'from_name', 'from_email', 'client_id', 'type');
    protected $set = array('name', 'title', 'desc', 'user_id', 'from_name', 'from_email', 'client_id', 'type');
    
	/**
     * Walidowanie
     */
    public function validate()
    {
        $errors = $data = null;
        foreach($this->set as $s)
            $data[$s] = isset($this->data[$s])?$this->data[$s]:null;
        
        if(strlen($data['name'])<5) $errors[] = 'Wpisz dłuższą nazwę emailingu!';
        if(strlen($data['title'])<5) $errors[] = 'Wpisz dłuższy tytuł emailingu!';
        
        if(strlen($data['from_name'])<5) $errors[] = 'Wpisz dłuższy tytuł nadawcy emailingu!';
        if(strlen($data['from_email'])<5) $errors[] = 'Wpisz dłuższy email nadawcy emailingu!';
        
        if($errors){
            $this->errors = $errors;
            return false;
        }
        return true;
    }
    
    public function fullLoad()
    {
        //body
        $this->childs['body'] = null;
        $sql = "SELECT body_id as value FROM emailing.em_body WHERE set_id='{$this->data[$this->pkey]}'";
        $list = PDOS::getInstance()->getCol($sql);
        if(is_array($list))
        foreach($list as $body_id){
            $obj = new em_body($body_id);
            $this->childs['body'][] = $obj;
        }
        
        //targetlisty
        $this->childs['targetlist'] = null;
        $sql = "SELECT list_id as value FROM emailing.em_targetlist WHERE set_id='{$this->data[$this->pkey]}'";
        $list = PDOS::getInstance()->getCol($sql);
        if(is_array($list))
        foreach($list as $list_id){
            $obj = new em_targetlist($list_id);
            $this->childs['targetlist'][] = $obj;
        }
        
        //wysylki
        $this->childs['schedule'] = null;
        $sql = "SELECT a.schedule_id as value FROM emailing.em_schedule a INNER JOIN emailing.em_body b ON a.body_id=b.body_id WHERE b.set_id='{$this->data[$this->pkey]}'";
        $list = PDOS::getInstance()->getCol($sql);
        if(is_array($list))
        foreach($list as $schedule_id){
            $obj = new em_schedule($schedule_id);
            $this->childs['schedule'][] = $obj;
        }
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Klasa obsługuje treść i layout graficzny emailingu
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_body extends dbObj
{
    protected $table = 'emailing.em_body';
    protected $pkey = 'body_id';
    protected $get = array('body_id', 'set_id', 'add_date', 'user_id', 'real_title', 'layer_html', 'layer_plain', 'desc', 'name', 'check_links', 'status');
    protected $set = array('set_id', 'user_id', 'real_title', 'layer_html', 'layer_plain', 'desc', 'name', 'check_links', 'status');
    protected $boolean = array('check_links');
    
    /**
     * Ładowanie obiektów powiązanych
     **/
    public function fullLoad()
    {
        //set
        $obj = new em_set($this->data['set_id']);
        $this->parent['set'] = $obj;
        
        //linki
        $this->childs['link'] = null;
        $sql = "SELECT link_id as value FROM emailing.em_link WHERE body_id='{$this->data[$this->pkey]}'";
        $list = PDOS::getInstance()->getCol($sql);
        if(is_array($list))
        foreach($list as $link_id){
            $obj = new em_link($link_id);
            $this->childs['link'][] = $obj;
        }
        
        //pliki (file)
        $this->childs['file'] = null;
        $sql = "SELECT file_id as value FROM emailing.em_file WHERE body_id='{$this->data[$this->pkey]}'";
        $list = PDOS::getInstance()->getCol($sql);
        if(is_array($list))
        foreach($list as $file_id){
            $obj = new em_file($file_id);
            $this->childs['file'][] = $obj;
        }
        
        //harmonogram wysylki (schedule)
        $this->childs['schedule'] = null;
        $sql = "SELECT schedule_id as value FROM emailing.em_schedule WHERE body_id='{$this->data[$this->pkey]}'";
        $list = PDOS::getInstance()->getCol($sql);
        if(is_array($list))
        foreach($list as $schedule_id){
            $obj = new em_schedule($schedule_id);
            $this->childs['schedule'][] = $obj;
        }
        
    }
    
	/**
     * Walidowanie
     */
    public function validate()
    {
        $errors = $data = null;
        foreach($this->set as $s)
            $data[$s] = isset($this->data[$s])?$this->data[$s]:null;
        
        if(strlen($data['name'])<5) $errors[] = 'Wpisz dłuższą nazwę emailingu!';
        if(strlen($data['real_title'])<5) $errors[] = 'Wpisz dłuższy temat wiadomości!';
        if(intval($data['set_id'])<1) $errors[] = 'Brakuje identyfikatora emailingu!';
        
        if($errors){
            $this->errors = $errors;
            return false;
        }
        return true;
    }
    
    /**
     * Aktualizacja linkow
     */
    public function updateLinks()
    {
        //Gdy obiekt nie istnieje lub został zmieniany walimy false
        if(!$this->exist || serialize($this->data)!=serialize($this->base)) return false;
        
        //Gdy zaladowalismy nie pelny obiekt, ladujemy do pelna :D
        if(!$this->full) $this->fullLoad();
        
        $body_id = $this->data['body_id'];
        $layer_html = $this->data['layer_html'];
        $layer_plain = $this->data['layer_plain'];
        
        if($this->data['check_links']){
            $links = $link = null;
            //Sprawdzamy czy istniejące linki są w kodzie
            if(is_array($this->childs['link']))
            foreach($this->childs['link'] as &$EMLINK){
                $links[$EMLINK->get('link_id')] = $link = $EMLINK->get('value');
                if(strpos($layer_html, $link)===false && strpos($layer_plain, $link)===false){
                    $EMLINK->set('in_code', false);
                    $EMLINK->save();
                }
            }
            //Szukamy linków w kodzie html
            $match = null;
            if(preg_match_all("(http://[^ <>\n\t\r'\"]+)", $layer_html, $match)){
                foreach($match[0] as $link){
                    if($link_id = array_search($link, (array)$links)){
                        $EMLINK = new em_link($link_id);
                        $EMLINK->set('in_code', true);
                        $EMLINK->save();
                    }else{
                        $EMLINK = new em_link();
                        $EMLINK->set('body_id', $body_id);
                        $EMLINK->set('value', $link);
                        $EMLINK->set('active', true);
                        $EMLINK->set('in_code', true);
                        $EMLINK->set('google_analytics', true);
                        $EMLINK->save();
                        $links[$EMLINK->get('link_id')] = $link;
                        $this->childs['link'][] = $EMLINK;
                    }
                }
            }
            //Szukanie linkiów w tekscie
            $match = null;
            if(preg_match_all("(http://[^ <>\n\t\r'\"]+)", $layer_plain, $match)){
                foreach($match[0] as $link){
                    if($link_id = array_search($link, (array)$links)){
                        $EMLINK = new em_link($link_id);
                        $EMLINK->set('in_code', true);
                        $EMLINK->save();
                    }else{
                        $EMLINK = new em_link();
                        $EMLINK->set('body_id', $body_id);
                        $EMLINK->set('value', $link);
                        $EMLINK->set('active', true);
                        $EMLINK->set('in_code', true);
                        $EMLINK->set('google_analytics', true);
                        $EMLINK->save();
                        $links[$EMLINK->get('link_id')] = $link;
                        $this->childs['link'][] = $EMLINK;
                    }
                }
            }
            
            //Aktualizacja flagi czekera
            $this->data['check_links'] = false;
            $this->save();
            $this->fullLoad();
            return true;
        }else{
            return true;
        }
    }
    
    /**
     * Enter description here ...
     */
    public function createHash()
    {
        $hash = md5('Varant'.$this->get('body_id').'CapeDun');
        return $hash;
    }
    
    /**
     * Enter description here ...
     * @param unknown_type $hash
     */
    public function validHash($hash)
    {
        if(md5('Varant'.$this->get('body_id').'CapeDun')==$hash)
            return true;
        return false;
    }
    
    /**
     * Enter description here ...
     * @param unknown_type $target_id
     */
    public function createHashLink($target_id = null)
    {
        $link = '<!--gate-->open,'.$this->get('body_id').','.$this->createHash();
        if($target_id){
            $EMTGT = new em_target($target_id);
            if($EMTGT->exist())
                $link .= ','.$EMTGT->get('target_id').','.$EMTGT->createHash();
        }
        $link .= '/';
        return $link;
    }
    
    /**
     * 
     */
    public function raiseStats($target_id)
    {
        $sql = "INSERT INTO emailing.em_open (body_id, target_id) VALUES ('{$this->data['body_id']}', '$target_id')";
        PDOS::getInstance()->exec($sql);
    }
    
    /**
     * 
     */
    public function sendAsEmail($target_id = null, $email = null, $first_name = null, $last_name = null, $gender = null, $title_prefix = null)
    {
        //Gdy obiekt nie istnieje lub został zmieniany walimy false
        if(!$this->exist || serialize($this->data)!=serialize($this->base)) return false;
        
        //Gdy zaladowalismy nie pelny obiekt, ladujemy do pelna :D
        if(!$this->full) $this->fullLoad();
        
        $EMTGT = null;
        if($target_id){
            $EMTGT = new em_target($target_id);
            $email = $EMTGT->get('email');
            $first_name = $EMTGT->get('first_name');
            $last_name = $EMTGT->get('last_name');
            $gender = $EMTGT->get('gender');
        }
        
        $EMSET = new em_set($this->get('set_id'), true);
        
        $mail = new htmlMimeMail5();
        if($EMSET->get('from_name'))
            $mail->setFrom($EMSET->get('from_name')." <".$EMSET->get('from_email').">");
        else
            $EMSET->get('from_email');
        $mail->setSubject($title_prefix.$this->get('real_title'));
        $mail->setHTMLCharset('UTF-8');
        $mail->setTextCharset('UTF-8');
        $mail->setHeadCharset('UTF-8');
        $mail->setPriority('normal');
        $mail->setSMTPParams('smtp2.esculap.pl');
        
        $layer_plain = $this->get('layer_plain');
        $layer_html = $this->get('layer_html');
        
        //Podmiana aktywnych linków
        if($tmp = $this->getChilds('link', true)){
            $links = array();
            foreach($tmp as $EMLINK){
                if($EMLINK->get('active') && $EMLINK->get('in_code')){
                    $link = $EMLINK->get('value');
                    $nlink = $EMLINK->createHashLink($target_id);
                    $links[$link] = $nlink;
                }
            }
            krsort($links);
            $layer_plain = str_replace(array_keys($links), array_values($links), $layer_plain);
            $layer_html = str_replace(array_keys($links), array_values($links), $layer_html);
        }
        
        //Doklejamy licznik otwarc do warstwy HTML
        $imgHtml = '<img src="'.$this->createHashLink($target_id).'" width="1" height="1">';
        $layer_html = str_ireplace('</body>', $imgHtml.'</body>', $layer_html);
        
        //Wykrywamy plec jesli nie jest wykryta
        if($gender!='w' && $gender!='m'){
            if(empty($first_name))
                $gender = 'u';
            elseif(strtolower($first_name{strlen($first_name)-1})=='a' && !in_array(strtolower($first_name), array('barnaba', 'kuba', 'jarema', 'kosma')))
                $gender = 'w';
            else
                $gender = 'm';
        }
        
        //Podmiana zmiennych 
        //<!--woman>tresc dla kobiet<woman-->
        //<!--man>tresc dla facetow<man-->
        //<!--unknown>tresc dla niezidentyfikowanych<unknown-->
        //<!--delete-->tresc widoczna ale do wywalenia<!--delete-->
        //<!--first_name--> <!--last_name--> <!--year--> <!--month--> <!--day--> <!--hour--> <!--minute--> <!--second--> <!--gate-->
        if($gender=='w'){
            $layer_plain = str_ireplace(array('<!--woman>', '<woman-->'), '', $layer_plain);
            $layer_html = str_ireplace(array('<!--woman>', '<woman-->'), '', $layer_html);
        }else{
            $layer_plain = preg_replace('/<!--woman>.+<woman-->/misU', '', $layer_plain);
            $layer_html = preg_replace('/<!--woman>.+<woman-->/misU', '', $layer_html);
        }
        if($gender=='m'){
            $layer_plain = str_ireplace(array('<!--man>', '<man-->'), '', $layer_plain);
            $layer_html = str_ireplace(array('<!--man>', '<man-->'), '', $layer_html);
        }else{
            $layer_plain = preg_replace('/<!--man>.+<man-->/misU', '', $layer_plain);
            $layer_html = preg_replace('/<!--man>.+<man-->/misU', '', $layer_html);
        }
        if($gender=='u'){
            $layer_plain = str_ireplace(array('<!--unknown>', '<unknown-->'), '', $layer_plain);
            $layer_html = str_ireplace(array('<!--unknown>', '<unknown-->'), '', $layer_html);
        }else{
            $layer_plain = preg_replace('/<!--unknown>.+<unknown-->/misU', '', $layer_plain);
            $layer_html = preg_replace('/<!--unknown>.+<unknown-->/misU', '', $layer_html);
        }
        $layer_plain = preg_replace('/<!--delete-->.+<!--delete-->/misU', '', $layer_plain);
        $layer_html = preg_replace('/<!--delete-->.+<!--delete-->/misU', '', $layer_html);
        $find = array('<!--first_name-->', '<!--last_name-->', '<!--year-->', '<!--month-->', '<!--day-->', '<!--hour-->', 
        			  '<!--minute-->', '<!--second-->', '<!--gate-->');
        $repl = array($first_name, $last_name, date('Y'), date('m'), date('d'), date('G'), 
                      date('i'), date('s'), 'http://www.esculap.pl/emgate/');
        $layer_plain = str_ireplace($find, $repl, $layer_plain);
        $layer_html = str_ireplace($find, $repl, $layer_html);
        //Koniec podmianek
        
        $mail->setText($layer_plain);
        $mail->setHTML($layer_html);
        
        //Załączanie plików
        if($tmp = $this->getChilds('file', true)){
            foreach($tmp as $EMFILE){
                if($EMFILE->get('type')=='i'){
                    $mail->addEmbeddedImage(new fileEmbeddedImage($EMFILE->get('path')));
                }else{
                    $mail->addAttachment(new fileAttachment($EMFILE->get('path')));
                }
            }
        }
        
        if($first_name || $last_name)
            $target = "$first_name $last_name <$email>";
        else
            $target = "$email";
        
        $result = $mail->send(array($target), 'smtp');
        if($result && $EMTGT){
            $EMTGT->send($this->data['set_id']);
        }
        return $result;
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Link w emailingu
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_link extends dbObj
{
    protected $table = 'emailing.em_link';
    protected $pkey = 'link_id';
    protected $get = array('link_id', 'body_id', 'value', 'active', 'in_code', 'google_analytics');
    protected $set = array('body_id', 'value', 'active', 'in_code', 'google_analytics');
    protected $boolean = array('active', 'in_code', 'google_analytics');
    
    /**
     * Enter description here ...
     * @param unknown_type $target_id
     */
    public function createHashLink($target_id = null)
    {
        $link = '<!--gate-->link,'.$this->get('link_id').','.$this->createHash();
        if($target_id){
            $EMTGT = new em_target($target_id);
            if($EMTGT->exist())
                $link .= ','.$EMTGT->get('target_id').','.$EMTGT->createHash();
        }
        $link .= '/';
        return $link;
    }
    
    /**
     * Enter description here ...
     */
    public function createHash()
    {
        $hash = md5('Braga'.$this->get('link_id').'Ishtar');
        return $hash;
    }
    
    /**
     * Enter description here ...
     * @param unknown_type $hash
     */
    public function validHash($hash)
    {
        if(md5('Braga'.$this->get('link_id').'Ishtar')==$hash)
            return true;
        return false;
    }
    
    /**
     * 
     */
    public function raiseStats($target_id)
    {
        $sql = "INSERT INTO emailing.em_click (link_id, target_id) VALUES ('{$this->data['link_id']}', '$target_id')";
        PDOS::getInstance()->exec($sql);
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Plik w emailingu
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_file extends dbObj
{
    protected $table = 'emailing.em_file';
    protected $pkey = 'file_id';
    protected $get = array('file_id', 'body_id', 'name', 'path', 'type');
    protected $set = array('body_id', 'name', 'path', 'type');
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Lista odbiorcow emailingu
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_targetlist extends dbObj
{
    protected $table = 'emailing.em_targetlist';
    protected $pkey = 'list_id';
    protected $get = array('list_id', 'set_id', 'user_id', 'schedule_id', 'desc', 'name', 'add_date', 'status');
    protected $set = array('set_id', 'user_id', 'schedule_id', 'desc', 'name', 'status');
    protected $addit = array('target_amount', 'sent_amount', 'wait_amount');
    
    /**
     * Ładowanie obiektów powiązanych
     **/
    public function fullLoad()
    {
        //set
        $obj = new em_set($this->data['set_id']);
        $this->parent['set'] = $obj;
        
        //schedule
        if($this->data['schedule_id']){
            $obj = new em_schedule($this->data['schedule_id']);
            $this->parent['schedule'] = $obj;
        }
        
        //targety ?? 
        
        
    }

	/**
     * Walidowanie
     */
    public function validate()
    {
        $errors = $data = null;
        foreach($this->set as $s)
            $data[$s] = isset($this->data[$s])?$this->data[$s]:null;
        
        if(strlen($data['name'])<5) $errors[] = 'Wpisz dłuższą nazwę emailingu!';
        if(intval($data['set_id'])<1) $errors[] = 'Brakuje identyfikatora emailingu!';
        
        if($errors){
            $this->errors = $errors;
            return false;
        }
        return true;
    }
    
	/**
     * Obliczanie wartości dodatkowych pol
     */
    protected function additional()
    {
        $sql = "SELECT COUNT(DISTINCT target_id) FROM emailing.em_tt WHERE list_id='{$this->data['list_id']}'";
        $this->data['target_amount'] = PDOS::getInstance()->getOne($sql); 
        $sql = "SELECT COUNT(DISTINCT target_id) FROM emailing.em_tt WHERE list_id='{$this->data['list_id']}' AND send_date IS NOT NULL";
        $this->data['sent_amount'] = PDOS::getInstance()->getOne($sql); 
        $this->data['wait_amount'] = $this->data['target_amount'] - $this->data['sent_amount'];
    }
    
    /**
     * Dopisanie nowego targetu
     **/
    public function assignTarget($email, $first_name = null, $last_name = null, $gender='a', $check = true)
    {
        $email = trim($email);
        if(!$this->exist || !$this->full) return false;
        $this->errors = null;
        
        $sql = "SELECT target_id FROM emailing.em_target WHERE email='$email'";
        $target_id = PDOS::getInstance()->getOne($sql);
        
        if(intval($target_id)==0){//Dodawanie targetu
            $user_login = null;
            $sql = "SELECT mu.login, plu.first_name, plu.last_name
            FROM main_users AS mu
            JOIN pl_users AS plu ON (plu.user_id = mu.id)
            WHERE mu.email='$email' OR mu.email_esc='$email'";
            PDOS::getInstance('db1', true)->exec("SET search_path TO esculap_int");
            if($data = PDOS::getInstance('db1', true)->getRow($sql)){
                $user_login = $data['login'];
                $first_name = $data['first_name'];
                $last_name = $data['last_name'];
            }
            
            if($gender!='w' && $gender!='m'){
                if(empty($first_name))
                    $gender = 'u';
                elseif(strtolower($first_name{strlen($first_name)-1})=='a' && !in_array(strtolower($first_name), array('barnaba', 'kuba', 'jarema', 'kosma')))
                    $gender = 'w';
                else
                    $gender = 'm';
            }
            
            $target = new em_target();
            $target->set('email', $email);
            $target->set('first_name', $first_name);
            $target->set('last_name', $last_name);
            $target->set('gender', $gender);
            $target->set('user_login', $user_login);
            if($target->validate() && !$check){
                $target->save();
                $target_id = $target->get('target_id');
            }else{
                $this->errors = $target->getErrors();
            }
        }else{//Target juz istnieje, mozna tutaj dokonac aktualizacji
            $target = new em_target($target_id);
        }
        
        //Sprawdzenie czy jest zgoda na emailing
        if($user_login = $target->get('user_login')){
            $type = $this->parent['set']->get('type');
            $sql = "SELECT COUNT(1) FROM rel_users_services WHERE user_login='$user_login' AND service_code='$type'";
            if(PDOS::getInstance()->getOne($sql)<1){
                $this->errors['SERVICE_ERR'] = 'Odbiorca nie wyraził zgody na ten typ emailingu.';
                //return false;
            }
        }
        
        //Sprawdzenie czy target nie jest powiązany z żadną listą dla tego emailingu
        if(intval($target_id)>0){
            $sql = "SELECT COUNT(1) FROM emailing.em_targetlist a INNER JOIN emailing.em_tt b ON a.list_id=b.list_id ".
            	   "WHERE b.target_id=$target_id AND a.set_id={$this->data['set_id']}";
            if(PDOS::getInstance()->getOne($sql)>0){
                $this->errors['EXISTS_ERR'] = 'Odbiorca już dodany do list docelowych tego emailingu.'." [{$target_id}]";
                //return false;
            }
        }
        
        //
        if($this->errors) return false;
        if(!$check){
            $sql = "INSERT INTO emailing.em_tt (list_id, target_id) VALUES ({$this->data['list_id']}, $target_id)";
            PDOS::getInstance()->exec($sql);
        }
        return true;
    }
    
    /**
     * Wypisanie targetu ;)
     **/
    public function unassignTarget($email = null, $target_id = null, $check = true)
    {
        if(!$this->exist) return false;
        if($email == null && $target_id == null) return false;
        if(!$target_id)
            $target_id = PDOS::getInstance()->getOne("SELECT target_id FROM emailing.em_target WHERE email='$email'");
        
        if(intval($target_id)<1){
            $this->errors['EXISTS_ERR'] = 'Nie ma takiego odbiorcy.';
            return false;
        }
        
        //Sprawdzenie czy jest hjna liscie
        $sql = "SELECT COUNT(1) FROM emailing.em_tt WHERE target_id=$target_id AND list_id={$this->data['list_id']}";
        if(PDOS::getInstance()->getOne($sql)<1){
            $this->errors['EXISTS_ERR'] = 'Odbiorca nie powiązany z emailingiem.';
            return false;
        }
        
        if(!$check){
            $sql = "DELETE FROM emailing.em_tt WHERE list_id={$this->data['list_id']} AND target_id=$target_id";
            PDOS::getInstance()->exec($sql);
        }
        return true;
    }
    
    /**
     * 
     */
    public function isAssignTarget($target_id)
    {
        $sql = "SELECT COUNT(1) FROM emailing.em_tt WHERE list_id='{$this->data['list_id']}' AND target_id='$target_id'";
        if(PDOS::getInstance()->getOne($sql))
            return true;
        return false;
    }
    
    /**
     * 
     */
    public function getPartToSend($limit = 10)
    {
        $sql = "SELECT target_id as value FROM emailing.em_tt WHERE list_id='{$this->data['list_id']}' AND send_date IS NULL LIMIT $limit";
        $tids = PDOS::getInstance()->getCol($sql);
        $data = null;
        foreach($tids as $target_id)
            $data[] = new em_target($target_id);
        return $data;
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Odbiorca emailingów
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_target extends dbObj
{
    protected $table = 'emailing.em_target';
    protected $pkey = 'target_id';
    protected $get = array('target_id', 'email', 'first_name', 'last_name', 'gender', 'user_login', 'add_date');
    protected $set = array('email', 'first_name', 'last_name', 'gender', 'user_login');
    
    public function validate()
    {
        $this->errors = $errors = null;
        
        if(!preg_match('(^[-a-zA-Z0-9_.]+@[-a-zA-Z0-9_.]+[.][-a-zA-Z0-9_.]+$)', $this->data['email']))
            $errors['EMAIL_ERR'] = 'Email o nieprawidłowej składni.';
        
        if($errors){
            $this->errors = $errors;
            return false;
        }
        return true;
    }
    
    public function createHash()
    {
        $hash = md5('Ishtar'.$this->get('target_id').'Braga');
        return $hash;
    }
    
    public function validHash($hash)
    {
        if(md5('Ishtar'.$this->get('target_id').'Braga')==$hash)
            return true;
        return false;
    }
    
    public function send($set_id)
    {
        $EMSET = new em_set($set_id, true);
        $tls = $EMSET->getChilds('targetlist', true);
        $list_id = null;
        foreach($tls as $EMTL){
            if($EMTL->isAssignTarget($this->data['target_id'])){
                $list_id = $EMTL->get('list_id');
                break;
            }
        }
        if($list_id){
            $sql = "UPDATE emailing.em_tt SET send_date=NOW() WHERE list_id='$list_id' AND target_id='{$this->data['target_id']}'";
            PDOS::getInstance()->exec($sql);
        }
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Klasa obsługuje wysyłki emailingu - tak, może być klika rozłożonych w czasie i z różnym layoutem
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_schedule extends dbObj
{
    protected $table = 'emailing.em_schedule';
    protected $pkey = 'schedule_id';
    protected $get = array('schedule_id', 'body_id', 'add_date', 'start_date', 'user_id', 'desc', 'name', 'status');
    protected $set = array('body_id', 'start_date', 'user_id', 'desc', 'name', 'status');
    
	/**
     * Ładowanie obiektów powiązanych
     **/
    public function fullLoad()
    {
        $this->full = true;
        
        //body
        $obj = new em_body($this->data['body_id']);
        $this->parent['body'] = $obj;
        
        //targetlisty 
        $this->childs['targetlist'] = null;
        $sql = "SELECT list_id as value FROM emailing.em_targetlist WHERE schedule_id='{$this->data[$this->pkey]}'";
        $list = PDOS::getInstance()->getCol($sql);
        if(is_array($list))
        foreach($list as $list_id){
            $obj = new em_targetlist($list_id);
            $this->childs['targetlist'][] = $obj;
        }
        
    }
    
	/**
     * Walidowanie
     */
    public function validate()
    {
        $errors = $data = null;
        foreach($this->set as $s)
            $data[$s] = isset($this->data[$s])?$this->data[$s]:null;
        
        if(strlen($data['name'])<5) 
            $errors[] = 'Wpisz dłuższą nazwę wysyłki!';
        if(intval($data['body_id'])<1) 
            $errors[] = 'Wybierz wygląd wysyłki!';
        if($this->base['start_date']!=$data['start_date'] && strtotime($data['start_date'])<time()) 
            $errors[] = 'Data wysyłki powinna być określona w przyszłości.';
        
        
        if($errors){
            $this->errors = $errors;
            return false;
        }
        return true;
    }
    
    /**
     * 
     */
    public function delete()
    {
        if($this->exist && $this->data[$this->pkey]){
            if(!$this->full) $this->fullLoad();
            foreach($this->childs['targetlist'] as $EMTL){
                $EMTL->set('schedule_id', null);
                $EMTL->set('status', 'r');
                $EMTL->save();
            }
            return parent::delete();
        }
        return false;
    }
}

?>