<?php

/******************************************************************************
 *                                                                            *
 *      Klasa obsugujca kolekcje danych                                     *
 *      @uses comments.class.php                                              *
 *      @category data                                                        *
 *      @author Sawek                                                        *
 *                                                                            *
 *                                                                            *
 ******************************************************************************/

require_once('comments.class.php');

class collection
{
    /**
     *  Uchwyt do bazy danych poprzez PDO
     **/
    public $db;
    
    /**
     *  Tablica konfiguracji domylnej
     **/
    private $config = array(
		'global_limit' => 0,		//Maksymalna ilo elementw kolekcji, gdy 0 brak limitu
		'pagging_limit' => 0,		//Maksymalna ilo elementw na jednej stronie, gdy 0 brak limitu
		'pagging_start' => 0,		//Aktualna strona stronicowania, gdy 0 pierwsza strona
		'ignore_first' => 0,		//Pomi x pierwszych elementw kolekcji
		'pagging_range' => 5,		//Odlego od aktualnej strony z jak pokazywane bd odnoniki stronicowania
		'show_all' => false,		//Zdejmuje niektre warunki wywietlania
		'site_code' => '',			//Kod strony,uywany do tworzenia HASH_ID
		'old_site_code' => null,
		'element_id' => null,		//Gdy nie jest null prbuje pobra pasujcy element list, wpp. pobiera list
		'sql_filter' => null,		//Filtruje dane na poziemie sql
		'disable_timeout' => false,	//Wycza timeout
		'order_by' => null,			//Podek sortowania, domylnie array('id_elementu' => DESC)
		'debug' => false,			//debugowa
		'most_rated' => false,		//pobiera najbardziej komtowane
		'most_rated_days' => 30,	//ilo dni zliczania dla najbardziej komtowanych
		'most_read' => false,		//pobiera najczciej czytane
		'most_read_days' => 30,		//ilo dni zliczania dla najczciej czytanych
		'last_comments' => false,	//pobiera ostatnie komentarze wraz z caym elementem
		'search' => false,			//wczy wyszukiwanie
		'search_string'	=> '',		//acuch znakw ktry ma zosta poddany wyszukiwanie
		'search_type' => null,		//rodzaj pobieranego kontentu
		'search_lang' => 'PL',		//jzyk wyszukiwania
		'advance_search' => false,	//wcz wyszukiwanie zaawansowane
		'search_nstring' => '',		//sowa ktre nie powinny by odnalezione ;-)
		'search_advfields' => null,	//dodatkowe pola wyszukiwania zaawansowanego, musi by tablic (opis poniej)
		/*	'publ' => array('publ_title', 'publ_author', 'publ_type', 'publ_journal, 'publ_spec')
		 *	'news' => array('news_title', 'news_type', 'news_spec')
		 *	'event' => array('event_name', 'event_type', 'event_country', 'event_start', 'event_end', 'event_spec')
		 *	*/
		'expect_type' => null,	//spodziewany rodzaj kolekcji 'news', 'link', 'publ', 'event'
		'save_collect_use' => true, //Czy zapisywa uycie kolekcji
    );
    
    /**
     *  Identyfikator kolekcji
     **/
    private $collectionID;
    
    /**
     *  Nazwa kolekcji
     **/
    private $collectionName;
    
    /**
     *	Peen rekord z main_collection
     **/
    public $data = null;
    
    /**
     *  Tablica elementw kolekcji
     **/
    public $elements = null;
    
    /**
     *	Tablica z peym rekorderm elementu
     **/
    public $element = null;
    
    /**
     *	Ilo elementw w rekordzie
     **/
    public $amount = null;
    
    /**
     *	Tablica ze stronami paginacji
     **/
    public $pages = null;
    
    /**
     *  Tablica typw zawartoci elementw kolekcji
     **/
    private $content_types = array('publ', 'news', 'link', 'event');
    
    /**
     *  Tabele typw zawartoci
     **/
    private $tables = array(
		'publ'  => "main_publications",
		'news'  => "main_news",
		'link'  => "main_links",
		'event' => "main_events",
    );
    
    /**
     *	relacje do specjalizacji
     **/
    private $rel_specs = array(
		'publ'  => "rel_spec_publ",
		'news'  => "rel_spec_news",
		'link'  => "rel_spec_link",
		'event' => "rel_spec_event",
    );
    
    /**
     *	Tabele sownikowe, [ 1 <-> 1 ]
     **/
    private $join_tables = array(
		'publ'  => array(
			array('alias' => 'journal', 'table' => 'main_journals', 'type' => 'LEFT OUTER JOIN', 'on_cond' => 'publ.id_journal=journal.id_journal'),
			array('alias' => 'r_publ_ed', 'table' => 'rel_publ_edition', 'type' => 'LEFT OUTER JOIN', 'on_cond' => 'publ.id_publ=r_publ_ed.id_publ'),
			array('alias' => 'edition', 'table' => 'main_editions', 'type' => 'LEFT OUTER JOIN', 'on_cond' => 'r_publ_ed.id_edition=edition.id_edition'),
		),
		'news'  => array(),
		'link'  => array(),
		'event' => array(),
    );
    
    /**
     *  Pola tabel typw zawartoci, wycigane na licie
     **/
    private $fields = array(
        'publ' => array(
			'id'	=> "publ.id_publ",
			'content_type'	=> "'publ'",
			'title'	=> "publ.title",
			'engtitle'	=> "publ.eng_title",
			'author'	=> "publ.author",
			'head'	=> "publ.head",
			'info'	=> "publ.info",
			'date'	=> "publ.date::date",//"(CASE WHEN publ.date=NULL THEN NULL ELSE publ.date::date END)",
			'time'	=> "publ.date::time",//"(CASE WHEN publ.date = NULL THEN NULL ELSE publ.date::time END)",
			'id_journal'=> "publ.id_journal",
			'journal'	=> "journal.title",
			'journal_url'	=> "journal.url",
			'page_nr'	=> "publ.page_nr",
			'type_code'	=> "publ.publ_type_code2",
			'content'	=> "publ.content",
			'timeout'	=> "FALSE",
			'status'	=> "publ.status",
			'avatar'	=> "publ.avatar", //Nowe pole z avatarem
			'_expander'	=> "LOWER(publ.title)",
			'edition_nr'	=> "edition.edition_nr",
			'edition_book'	=> "edition.book_nr",
			'edition_month'	=> "edition.book_month",
			'edition_book_nr' => "edition_book_nr",
			'edition_year'	=> "edition.book_year"
		),
		'news' => array(
			'id'	=> "news.id_news",
			'content_type'	=> "'news'",
			'title'	=> "news.news_title",
			'head'	=> "news.news_head",
			'content'	=> "news.news_body",
			'date'	=> "news.news_date::date",//"(CASE WHEN news.news_date=NULL THEN NULL ELSE news.news_date::date END)",
			'time'	=> "news.news_date AT time zone 'CEST'",//"(CASE WHEN news.news_date=NULL THEN NULL ELSE news.news_date AT time zone 'CEST' END)",
			'author'	=> "news.news_author",
			'timeout'	=> "(NOT(news.news_life_time IS NULL OR (news.news_date + CAST((news.news_life_time ||' days') AS interval))>=CURRENT_TIMESTAMP))",
			'type_code'	=> "news.news_type_code",
			'image'	=> "news.picture",
			'avatar'	=> "news.avatar", //Nowy avatar, ma zastapic [image -> picture]
			'status'	=> "news.status",
			'_expander'	=> "LOWER(news.news_title)"
		),
		'link' => array(
			'id'	=> "link.id_link",
			'content_type'	=> "'link'",
			'title'	=> "link.name",
			'url'	=> "link.url",
			'type_code'	=> "link.link_type_code",
			'timeout'	=> "FALSE",
			'status'	=> "link.status",
			'_expander'	=> "LOWER(link.name)"
		),
		'event' => array(
			'id'	=> "event.id_event",
			'content_type'	=> "'event'",
			'type_code'	=> "event.event_type_code",
			'title'	=> "event.event_title",
			'head'	=> "event.info",
			//'content'	=> "(e.org_address || ('<br /><b>Termin:<br />' || e.date_start || '&nbsp;&nbsp;-&nbsp;&nbsp;' || e.date_end || '</b>')) AS content",
			'date_start'=> "event.date_start",
			'date_end'	=> "event.date_end",
			'timeout'	=> "(NOT (event.date_end + CAST(('2 days') AS interval)>=CURRENT_TIMESTAMP))",
			'org_address'   => "event.org_address",
			'telefon'	=> "event.org_tel",
			'fax'	=> "event.org_fax",
			'address'   => "event.address",
			'city_name'	=> "event.city_name",
			'info'	=> "event.info",
			//'data'	=> "(e.date_start || '-' || e.date_end) AS data",
			'url'	=> "event.url",
			'email'	=> "event.org_email",
			'status'	=> "event.status",
			'_expander'	=> "LOWER(event.event_title)"
		)
    );
    
    /**
     *	Konstruktor, pobiera pene dane kolekcji i decyduje czy pobra ca list czy tylko
     *	jeden peny element na podstawie $config['element_id']
     *	@param str/int/array $collectionID - Identyfikator(y) kolekcji, numeryczne lub tekstowe
     *	@param array $config - Tablica konfiguracyjna dla kolekcji
     *	@param array $order - tablica sortowania o strukturze array('alias'='ASC/DESC'),
     *	                      np. array('type'='DESC', 'id'='ASC')
     *  		      zalene od parametru $content_type, klucze z $this->fields
     *  @global object $db - obiekt bazy danych przekazywany do parametru obiektu
     **/
    public function __construct($collectionID = null, $config = null)
    {
		if(defined('SITE_ID'))
			$this->config['site_code'] = SITE_ID;
        foreach($config as $key => $value)
            $this->config[$key] = $value;
        global $db;
        $this->db = $db;
		if(!is_array($collectionID))
			$collectionID = array($collectionID);
		$this->collectionID = $collectionID;
		$whr = null;
		foreach($collectionID as $colid){
			if(is_numeric($colid))
				$whr[] = "id_kolekcji='{$colid}'";
			elseif(strlen($colid)>3)
				$whr[] = "collection_id LIKE '{$colid}%'";
		}
		$where = null;
		$where[] = 'active=TRUE';
		if(count($whr)>0)
			$where[] = '('.implode(' OR ', $whr).')';
		if(in_array($this->config['expect_type'], $this->content_types))
			$where[] = "type='{$this->config['expect_type']}'";
		if(count($where))
			$where = 'WHERE '.implode(' AND ', $where);
		
		$sql = "SELECT * FROM main_collection {$where} ORDER BY id_kolekcji DESC";
		$this->data = $this->db->getAll($sql);
		if($this->config['most_rated']){
			//Moe zawiera elementy rnego typy
			$this->getMostRated();
			$this->getPages();
		}elseif($this->config['most_read']){
			//Moe zawiera elementy rnego typy
			$this->getMostRead();
			$this->getPages();
		}elseif($this->config['last_comments']){
			$this->getLastComments();
			$this->getPages();
		}elseif($this->config['search']){
			$this->getSearch();
			$this->getPages();
		}elseif($this->config['advance_search']){
			$this->getAdvanceSearch();
			$this->getPages();
		}elseif($this->config['element_id']!=null){
			//Pojedyczy element
			$this->getElement();
			$this->raiseCounter();
			$this->saveCollectionUse();
		}else{
			//Wszystkie elemety musz by jednego typu
			$this->getElements();
			$this->getPages();
		}
    }
    
    /**
     *	Pobiera peny rekord elementu
     *	@param str $hash_id - Identyfikator elementu
     *	@param str $site_code - Kod strony
     *	@param str $content_type - Rodzaj elementu
     *	@return array - tablica z rekorderm elementu
     **/
    public function getElement($hash_id = null, $site_code = null, $content_type = null)
    {
		$hash_id = $hash_id==null?$this->config['element_id']:$hash_id;
		$site_code = $site_code==null?$this->config['site_code']:$site_code;
		$content_type = $content_type==null?$this->data[0]['type']:$content_type;
		$q_table = null;
		$q_fields = null;
		$q_join = null;
		$q_where = null;
		$data = null;
		
		//Ustawianie tabeli oraz pobieranych pl
		$q_table = $this->tables[$content_type];
		foreach($this->fields[$content_type] as $alias => $field){
			$q_fields[] = "{$field} AS {$alias}";
		}
		$q_fields = implode(', ', $q_fields);
		
		//Doczanie tabel sownikowych
		if(count($this->join_tables[$content_type])>0){
			foreach($this->join_tables[$content_type] as $join){
			$q_join = $q_join . " {$join['type']} {$join['table']} {$join['alias']} ON {$join['on_cond']}";
			}
		}
		
		//newsy i eventy musz by aktualne
		if(!$this->config['show_all']){
			$q_where[] = "{$content_type}.status='VIS'";
			if(($content_type=='news' || $content_type=='event') && $this->config['disable_timeout']==false){
			//$q_where[] = 'timeout>NOW()';
			}
		}
		
		//Sprawdzenie przynalenoci do kolekcji
		$collections_id = null;
		foreach($this->data as $collection)
			$collections_id[] = $collection['id_kolekcji'];
		$q_where[] = 'rc.id_kolekcji IN ('.implode(', ', $collections_id).')';
		
		//Sprawdzenie hasha z identyfikatorem
		if(is_numeric($hash_id)){
		    $q_where[] = "{$this->fields[$content_type]['id']}={$hash_id}";
		}elseif(isset($this->config['old_site_code']) && $this->config['old_site_code']!=null)
			$q_where[] = "(md5('0123456789' || {$this->fields[$content_type]['id']} || '*_*' || '{$site_code}' || '9876543210')='{$hash_id}' OR ".
			"md5('0123456789' || {$this->fields[$content_type]['id']} || '*_*' || '{$this->config['old_site_code']}' || '9876543210')='{$hash_id}')";
		else
			$q_where[] = "md5('0123456789' || {$this->fields[$content_type]['id']} || '*_*' || '{$site_code}' || '9876543210')='{$hash_id}'";
		
		//Tworzenie warunkw
		if($q_where!=null)
			$q_where = 'AND '.implode(' AND ', $q_where);
		
		$sql = "SELECT {$content_type}.*, {$q_fields} FROM rel_collection rc INNER JOIN {$q_table} {$content_type} ".
			   "ON rc.id_elementu={$this->fields[$content_type]['id']} {$q_join} ".
				   "WHERE rc.id_elementu={$this->fields[$content_type]['id']} ".
			   " {$q_where} LIMIT 1";
		$data = $this->db->getRow($sql);
		
		if($this->config['debug']){
			echo "<hr />SQL QUERRY:<br />{$sql}<br />DATA:<br /><pre>".print_r($data, true)."</pre><hr />";
		}
		
		if(!is_array($data))
			return null;
		
		$data = array(0 => $data);
		$data = $this->extendElements($data, $content_type);
		$data = $data[0];
		
		$this->element = $data;
		return $data;
    }
    
    /**
     *  Pobiera list elementw kolekcji
     *  @param str $content_type - rodzaj zawartoci, warto z $this->$content_types
     *  @param array $order - tablica sortowania o strukturze array('alias'='ASC/DESC'), np. array('type'='DESC', 'id'='ASC')
     *  		      zalene od parametru $content_type, klucze z $this->fields
     **/
    public function getElements($content_type = null)
    {
		if(!$content_type){
			if(isset($this->data[0]['type']))
				$content_type = $this->data[0]['type'];
			elseif(isset($this->config['expect_type']))
				$content_type = $this->config['expect_type'];
			else
				return null;
		}
		$q_table = null;
		$q_fields = null;
		$q_fields_dist = null;
		$q_join = null;
		$q_where = null;
		$q_order = null;
		$q_limit = null;
		$data = null;
		$amount = 0;
		
		//Ustawianie tabeli oraz pobieranych pl
		$q_table = $this->tables[$content_type];
		foreach($this->fields[$content_type] as $alias => $field){
			$q_fields[] = "{$field} AS {$alias}";
			if($q_fields_dist==null)
			$q_fields_dist = $field;
		}
		$q_fields_dist = 'COUNT(DISTINCT('.$q_fields_dist.'))';
		$q_fields = 'DISTINCT '.implode(', ', $q_fields);
		
		//Doczanie tabel sownikowych
		if(count($this->join_tables[$content_type])>0){
			foreach($this->join_tables[$content_type] as $join){
			$q_join = $q_join . " {$join['type']} {$join['table']} {$join['alias']} ON {$join['on_cond']}";
			}
		}
		
		//newsy i eventy musz by aktualne
		if(!$this->config['show_all']){
			$q_where[] = "{$content_type}.status='VIS'";
			if(/*$content_type=='news' || $content_type=='event'*/$this->config['disable_timeout']==false){
				//$q_where[] = "timeout='0'";
				$q_where[] = $this->fields[$content_type]['timeout']."=FALSE";
			}
		}		
		//Selekcja elementw nalecych do kolekcji
		$collections_id = null;
		foreach($this->data as $collection)
			$collections_id[] = $collection['id_kolekcji'];
		$q_where[] = 'rc.id_kolekcji IN ('.implode(', ', $collections_id).')';
		
		//Filtrowanie poprzez sql
		if($this->config['sql_filter']!=null)
			$q_where[] = $this->config['sql_filter'];
		
		//Tworzenie warunkw
		if($q_where!=null)
			$q_where = implode(' AND ', $q_where);
		
		//Grupowanie, usuwanie duplikatw
		//$q_group = 'GROUP BY id_elementu, '.implode(', ', array_values($this->fields[$content_type]));
		
		//Sortowanie
		if(is_array($this->config['order_by'])){
			foreach($this->config['order_by'] as $alias => $order_direct)
			$q_order[] = "{$alias} {$order_direct}";
			$q_order = "ORDER BY ".implode(', ', $q_order);
		}elseif($content_type=='event'){
			$q_order = "ORDER BY date_start ASC";
		}else{
			$q_order = "ORDER BY id DESC";
		}
		
		//Przycinanie wynikw
		if($this->config['global_limit']>0){
			$amount = $this->config['global_limit'];
			$q_limit = "LIMIT {$this->config['global_limit']}";
			if($this->config['ignore_first']>0)
			$q_limit .= "OFFSET {$this->config['ignore_first']}";
		}
		if($this->config['pagging_limit']>0){
			$start = $this->config['pagging_start']*$this->config['pagging_limit'];
			$q_limit = "LIMIT {$this->config['pagging_limit']} OFFSET ".($start+$this->config['ignore_first']);
		}
		
		if($q_where!=null)
			$q_where = 'WHERE '.$q_where;
		
		$sql = "SELECT {$q_fields} FROM rel_collection rc INNER JOIN {$q_table} {$content_type} ".
			   "ON rc.id_elementu={$this->fields[$content_type]['id']} {$q_join} ".
			   " {$q_where} {$q_order} {$q_limit}";
		$data = $this->db->getAll($sql);
		
		$sql2 = "SELECT {$q_fields_dist} FROM rel_collection rc INNER JOIN {$q_table} {$content_type} ".
			   "ON rc.id_elementu={$this->fields[$content_type]['id']} {$q_join} ".
			   "{$q_where}";
		$amount2 = $this->db->getOne($sql2);
		
		if($this->config['debug']){
			echo "[-----<pre>\nSQL QUERRY:\n{$sql}\n{$sql2}\nAMOUNT:{$amount2}\nCOUNT:".count($data)."\n</pre>-----]";
		}
		
		$data = $this->extendElements($data, $content_type);
		
		$this->elements = $data;
		$this->amount = ($amount<$amount2 && $amount>0)?$amount:$amount2;
		return $data;
    }
	
	/**
     *  Pobiera typy kontentu najbardziej komtowane przez ostatnie dni
     *  @param integer $last_days - ilo ostatnich dni brana pod uwag
     *  @param integer $glimit - ilo zwracanycvh rekordw
     *  @param integer $plimit - ilo rekordw na stron
     *  @param integer $slimit - aktualna strona
     **/
    public function getMostRated($last_days = null, $glimit = null, $plimit = null, $slimit = null)
    {
		$last_days = $last_days==null?$this->config['most_rated_days']:$last_days;
		$glimit = $glimit==null?$this->config['global_limit']:$glimit;
		$plimit = $plimit==null?$this->config['pagging_limit']:$plimit;
		$slimit = ($slimit==null?$this->config['pagging_start']:$slimit) * $plimit;
		$q_where = null;
		$data = null;
		$amount = 0;
		
		$q_where[] = "commentary_date>NOW() - INTERVAL '$last_days days'";
		$collections_id = null;
		foreach($this->data as $collection)
			$collections_id[] = $collection['id_kolekcji'];
		$q_where[] = 'rc.id_kolekcji IN ('.implode(', ', $collections_id).')';
		
		//Filtrowanie poprzez sql
		if($this->config['sql_filter']!=null)
			$q_where[] = $this->config['sql_filter'];
		
		$q_where = 'WHERE '.implode(' AND ', $q_where);
		
		$limit = max($glimit, $plimit) * 5;
		
        $sql = <<< ESQUEL
SELECT mc.content_type, mc.content_id as id, COUNT(DISTINCT(mc.*)) as amount 
FROM main_commentaries2 mc 
INNER JOIN rel_collection rc ON rc.id_elementu=mc.content_id AND rc.content_type=mc.content_type
{$q_where} 
GROUP BY mc.content_type, mc.content_id 
ORDER BY amount DESC, MAX(mc.commentary_date) DESC, MAX(mc.commentary_time) DESC 
LIMIT $limit 
ESQUEL;
        $data = $this->db->getAll($sql);
		$data = $this->bacicElements($data);
		$data2 = null;
		foreach($data as $dd)
			if($dd['status']=='VIS' && !$dd['timeout'])
				$data2[] = $dd;
		$data = $data2;
		
		$amount = count($data);
		if($glimit!=0 && $amount>$glimit)
			$amount = $glimit;
		
		$data2 = null; $z = 0;
		if(is_array($data))
			foreach($data as $dd){
				$z++;
				if($slimit!=0 && $z<$slimit)
					continue;
				$data2[] = $dd;
				if($plimit!=0 && $z>=$plimit+$slimit)
					break;
				if($glimit!=0 && $z>=$glimit)
					break;
			}
		$data = $data2;
		$data = $this->extendElements($data);
		$data = $this->addDirectLink($data);
		
		$this->elements = $data;
		$this->amount = $amount;
        return $data;
    }
	
	
    /**
     *	Pobiera typy kontentu najczciej czytane w ostatnich dniach (podobne do getMostRated)
     *  @param integer $last_days - ilo ostatnich dni brana pod uwag
     *  @param integer $glimit - ilo zwracanycvh rekordw
     *  @param integer $plimit - ilo rekordw na stron
     *  @param integer $slimit - aktualna strona
	**/
    public function getMostRead($last_days = null, $glimit = null, $plimit = null, $slimit = null)
    {
		$last_days = $last_days==null?$this->config['most_rated_days']:$last_days;
		$glimit = $glimit==null?$this->config['global_limit']:$glimit;
		$plimit = $plimit==null?$this->config['pagging_limit']:$plimit;
		$slimit = ($slimit==null?$this->config['pagging_start']:$slimit) * $plimit;
		$q_where = null;
		$data = null;
		$amount = 0;
		
		$q_where[] = "mvc.count_date>CURRENT_DATE - INTERVAL '$last_days days'";
		
		//Filtrowanie poprzez sql
		if($this->config['sql_filter']!=null)
			$q_where[] = $this->config['sql_filter'];
		
		$collections_id = null;
		foreach($this->data as $collection)
			$collections_id[] = $collection['id_kolekcji'];
		$q_where[] = 'rc.id_kolekcji IN ('.implode(', ', $collections_id).')';
		
		$q_where = 'WHERE '.implode(' AND ', $q_where);
		
		$limit = max($glimit, $plimit) * 5;
		
		//SUM(DISTINCT(mvc.counter)) <--> SUM(mvc.counter)/COUNT(1)
        $sql = <<< ESQUEL
SELECT mvc.content_type, mvc.content_id as id, SUM(mvc.counter)/COUNT(1) as amount 
FROM main_view_content mvc 
INNER JOIN rel_collection rc ON rc.id_elementu=mvc.content_id AND rc.content_type=mvc.content_type 
{$q_where} 
GROUP BY mvc.content_type, mvc.content_id 
ORDER BY amount DESC, MAX(mvc.count_date) DESC 
LIMIT $limit 
ESQUEL;
		//echo $sql."<br><br>";
		$data = $this->db->getAll($sql);
		$data = $this->bacicElements($data);
		$data2 = null;
		if(is_array($data))
			foreach($data as $dd)
				if($dd['status']=='VIS' && !$dd['timeout'])
					$data2[] = $dd;
		$data = $data2;
		
		$amount = count($data);
		if($glimit!=0 && $amount>$glimit)
			$amount = $glimit;
		
		$data2 = null; $z = 0;
		if(is_array($data))
			foreach($data as $dd){
				$z++;
				if($slimit!=0 && $z<$slimit)
					continue;
				$data2[] = $dd;
				if($plimit!=0 && $z>=$plimit+$slimit)
					break;
				if($glimit!=0 && $z>=$glimit)
					break;
			}
		$data = $data2;
		$data = $this->extendElements($data);
		$data = $this->addDirectLink($data);
		
		$this->elements = $data;
		$this->amount = $amount;
		return $data;
    }
    
    /**
     *  Pobiera ostatnio dodane komtarze wraz z elementem ktrego dotyczy
     *  @param integer $glimit - ilo zwracanycvh rekordw
     *  @param integer $plimit - ilo rekordw na stron
     *  @param integer $slimit - aktualna strona
     **/
    public function getLastComments($glimit = null, $plimit = null, $slimit = null)
    {
		$glimit = $glimit==null?$this->config['global_limit']:$glimit;
		$plimit = $plimit==null?$this->config['pagging_limit']:$plimit;
		$slimit = ($slimit==null?$this->config['pagging_start']:$slimit) * $plimit;
		$q_where = null;
		$data = null;
		$amount = 0;
		
		$collections_id = null;
		foreach($this->data as $collection)
			$collections_id[] = $collection['id_kolekcji'];
		$q_where[] = 'rc.id_kolekcji IN ('.implode(', ', $collections_id).')';
		
		//Filtrowanie poprzez sql
		if($this->config['sql_filter']!=null)
			$q_where[] = $this->config['sql_filter'];
		
		$q_where[] = 'deleted IS NULL';
		$q_where[] = 'moderator IS NULL';
		$q_where[] = 'commentary_date > NOW() - INTERVAL \'200 DAYS\'';
		
		$q_where = 'WHERE '.implode(' AND ', $q_where);
		
		$limit = max($glimit, $plimit) * 2;
		
        $sql = <<< ESQUEL
SELECT DISTINCT 
mc.content_type, mc.content_id as id, mc.commentary as comment_body, mc.nickname as comment_nick, mc.title as comment_title, mc.commentary_date, mc.commentary_time 
FROM main_commentaries2 mc 
INNER JOIN rel_collection rc ON rc.id_elementu=mc.content_id AND rc.content_type=mc.content_type 
{$q_where} 
ORDER BY mc.commentary_date DESC, mc.commentary_time DESC 
LIMIT $limit 
ESQUEL;
        
		$data = $this->db->getAll($sql);
		$data = $this->bacicElements($data);
		$data2 = null;
		if(is_array($data))
			foreach($data as $dd)
				if($dd['status']=='VIS' && !$dd['timeout'])
					$data2[] = $dd;
		$data = $data2;
		
		$amount = count($data);
		if($glimit!=0 && $amount>$glimit)
			$amount = $glimit;
		
		$data2 = null; $z = 0;
		if(is_array($data))
			foreach($data as $dd){
				$z++;
				if($slimit!=0 && $z<$slimit)
					continue;
				$data2[] = $dd;
				if($plimit!=0 && $z>=$plimit+$slimit)
					break;
				if($glimit!=0 && $z>=$glimit)
					break;
			}
		$data = $data2;
		$data = $this->extendElements($data);
		$data = $this->addDirectLink($data);
		
		$this->elements = $data;
		$this->amount = $amount;
		
		return $data;
    }
	
	/**
	 *	Wyszukiwanie globalne (proste)
	 **/
	public function getSearch($search_string = null, $search_type = null, $search_lang = 'PL', $glimit = null, $plimit = null, $slimit = null)
	{
		$search_string = $search_string==null?$this->config['search_string']:$search_string;
		$search_type = $search_type==null?$this->config['search_type']:$search_type;
		$search_lang = $search_lang==null?$this->config['search_lang']:$search_lang;
		$glimit = $glimit==null?$this->config['global_limit']:$glimit;
		$plimit = $plimit==null?$this->config['pagging_limit']:$plimit;
		$slimit = ($slimit==null?$this->config['pagging_start']:$slimit) * $plimit;
		
		$config = $this->db->getOne("SELECT config_name FROM dict_lang WHERE lang_id='$search_lang'");
		
		$q_fields = null;
		$q_where = null;
		
		if(in_array($search_type, array('publ', 'news'))){
			if(!$search_string || !$config)
				$q_fields[] = '1 AS search_rank';
			else{
				$q_fields[] = "ts_rank_cd(ms.tsearch, plainto_tsquery('$config', '$search_string'), 32) AS search_rank";
				$q_where[] = "ms.tsearch @@ plainto_tsquery('$config', '$search_string') ";
			}
		}else{
			if(!$search_string || !$config)
				return null;
			$q_fields[] = "ts_rank_cd(ms.tsearch, plainto_tsquery('$config', '$search_string'), 32) AS search_rank";
			$q_where[] = "ms.tsearch @@ plainto_tsquery('$config', '$search_string') ";
		}
		$q_fields[] = 'ms.id_elementu as id';
		$q_fields[] = 'ms.content_type';
		
		$collections_id = null;
		foreach($this->data as $collection)
			$collections_id[] = $collection['id_kolekcji'];
		$q_where[] = 'rc.id_kolekcji IN ('.implode(', ', $collections_id).')';
		$q_where[] = "ms.active=TRUE";
		
		//Filtrowanie poprzez sql
		if($this->config['sql_filter']!=null)
			$q_where[] = $this->config['sql_filter'];
		
		if(in_array($search_type, array('news', 'publ')))
			$q_where[] = "ms.content_type='{$search_type}'";
		
		$q_fields = implode(', ', $q_fields);
		$q_where = 'WHERE '.implode(' AND ', $q_where);
		
		$limit = min($glimit!=0?$glimit:99999, $plimit!=0?$plimit:99999);
		
		$sql = <<< ESQUEL
SELECT DISTINCT 
{$q_fields}
FROM main_search ms 
INNER JOIN rel_collection rc ON rc.id_elementu=ms.id_elementu AND rc.content_type=ms.content_type 
{$q_where}
ORDER BY search_rank DESC, id DESC 
LIMIT $limit OFFSET $slimit 
ESQUEL;
		
		$sql2 = <<< ESQUEL
SELECT COUNT(DISTINCT(ms.*)) 
FROM main_search ms 
INNER JOIN rel_collection rc ON rc.id_elementu=ms.id_elementu AND rc.content_type=ms.content_type 
{$q_where}
ESQUEL;
		
		$data = $this->db->getAll($sql);
		$amount = min($glimit!=0?$glimit:99999, $this->db->getOne($sql2));
		
		$data = $this->bacicElements($data);
		$data = $this->extendElements($data);
		$data = $this->addDirectLink($data);
		$data = $this->headlinesForSearch($data, $search_string, $config);
		$data = $this->addDirectLink($data);
		
		$this->elements = $data;
		$this->amount = $amount;
		
		return $data;
	}
	
	/**
	 *	Wyszukiwanie globalne (zaawansowane)
	 **/
	public function getAdvanceSearch($search_string = null, $search_nstring=null, $search_type = null, $search_advfields=null, $search_lang = 'PL', $glimit = null, $plimit = null, $slimit = null)
	{
		$search_string = $search_string==null?$this->config['search_string']:$search_string;
		$search_nstring = $search_nstring==null?$this->config['search_nstring']:$search_nstring;
		$search_advfields = $search_advfields==null?$this->config['search_advfields']:$search_advfields;
		$search_type = $search_type==null?$this->config['search_type']:$search_type;
		$search_lang = $search_lang==null?$this->config['search_lang']:$search_lang;
		$glimit = $glimit==null?$this->config['global_limit']:$glimit;
		$plimit = $plimit==null?$this->config['pagging_limit']:$plimit;
		$slimit = ($slimit==null?$this->config['pagging_start']:$slimit) * $plimit;
		
		$config = $this->db->getOne("SELECT config_name FROM dict_lang WHERE lang_id='$search_lang'");
		
		if(!in_array($search_type, array('publ', 'news', 'event')) || !$config)
			return null;
		
		$q_table = null;
		$q_fields = null;
		$q_join = null;
		$q_where = null;
		$q_order = null;
		$q_limit = null;
		$data = null;
		$amount = 0;
		
		$q_fields[] = "ms.id_elementu as id";
		$q_fields[] = "ms.content_type";
		
		$q_where[] = "ms.active=TRUE";
		
		$collections_id = null;
		foreach($this->data as $collection)
			$collections_id[] = $collection['id_kolekcji'];
		$q_where[] = 'rc.id_kolekcji IN ('.implode(', ', $collections_id).')';
		
		//Filtrowanie poprzez sql
		if($this->config['sql_filter']!=null)
			$q_where[] = $this->config['sql_filter'];
		
		if(strlen($search_string)){
			$q_fields[] = "ts_rank_cd(ms.tsearch, plainto_tsquery('$config', '$search_string'), 32) AS search_rank";
			$q_where[] = "ms.tsearch @@ plainto_tsquery('$config', '$search_string')";
		}else{
			$q_fields[] = "1 AS search_rank";
		}
		
		if(strlen($search_nstring))
			$q_where[] = "NOT (ms.tsearch @@ plainto_tsquery('$config', '$search_nstring'))";
		
		$q_join[] = "INNER JOIN rel_collection rc ON rc.id_elementu=ms.id_elementu AND rc.content_type=ms.content_type";
		
		switch($search_type){
			case 'publ':
				$q_join[] = "INNER JOIN main_publications mp ON ms.id_elementu=mp.id_publ AND ms.content_type='publ'";
				if(isset($search_advfields['title']) && !empty($search_advfields['title']))
					$q_where[] = "LOWER(mp.title) LIKE LOWER('%{$search_advfields['title']}%')";
				if(isset($search_advfields['publ_type']) && !empty($search_advfields['publ_type']))
					$q_where[] = "mp.publ_type_code2='{$search_advfields['publ_type']}'";
				if(isset($search_advfields['publ_author']) && !empty($search_advfields['publ_author']))
					$q_where[] = "LOWER(mp.author) LIKE LOWER('%{$search_advfields['publ_author']}%')";
				if(isset($search_advfields['publ_journal']) && !empty($search_advfields['publ_journal']))
					$q_where[] = "mp.id_journal='{$search_advfields['publ_journal']}'";
				if(isset($search_advfields['spec']) && !empty($search_advfields['spec'])){
					$q_join[] = "INNER JOIN rel_spec_publ rsp ON rsp.id_publ=ms.id_elementu";
					if(is_array($search_advfields['spec'])){
						$tmp = null;
						foreach($search_advfields['spec'] as $spec)
							$tmp[] = "rsp.spec_code='{$spec}'";
						$q_where[] = '('.implode(' OR ', $tmp).')';
					}else
						$q_where[] = "rsp.spec_code='{$search_advfields['spec']}'";
				}
				break;
			case 'news':
				$q_join[] = "INNER JOIN main_news mn ON ms.id_elementu=mn.id_news AND ms.content_type='news'";
				if(isset($search_advfields['title']) && !empty($search_advfields['title']))
					$q_where[] = "LOWER(mn.news_title) LIKE LOWER('%{$search_advfields['title']}%')";
				if(isset($search_advfields['news_type']) && !empty($search_advfields['news_type']))
					$q_where[] = "mn.news_type_code='{$search_advfields['news_type']}'";
				if(isset($search_advfields['spec']) && !empty($search_advfields['spec'])){
					$q_join[] = "INNER JOIN rel_spec_news rsn ON rsn.id_news=ms.id_elementu";
					if(is_array($search_advfields['spec'])){
						$tmp = null;
						foreach($search_advfields['spec'] as $spec)
							$tmp[] = "rsn.spec_code='{$spec}'";
						$q_where[] = '('.implode(' OR ', $tmp).')';
					}else
						$q_where[] = "rsn.spec_code='{$search_advfields['spec']}'";
				}
				break;
			case 'event':
				$q_join[] = "INNER JOIN main_events me ON ms.id_elementu=me.id_event AND ms.content_type='event'";
				if(isset($search_advfields['title']) && !empty($search_advfields['title']))
					$q_where[] = "LOWER(me.event_title) LIKE LOWER('%{$search_advfields['title']}%')";
				if(isset($search_advfields['event_type']) && !empty($search_advfields['event_type']))
					$q_where[] = "me.event_type_code='{$search_advfields['event_type']}'";
				if(isset($search_advfields['event_country']) && !empty($search_advfields['event_country']))
					$q_where[] = "me.country_code='{$search_advfields['event_country']}'";
				if(isset($search_advfields['event_start']) && !empty($search_advfields['event_start']))
					$q_where[] = "me.date_end>='{$search_advfields['event_start']}'";
				if(isset($search_advfields['event_end']) && !empty($search_advfields['event_end']))
					$q_where[] = "me.date_start<='{$search_advfields['event_end']}'";
				if(isset($search_advfields['spec']) && !empty($search_advfields['spec'])){
					$q_join[] = "INNER JOIN rel_spec_event rse ON rse.id_event=ms.id_elementu";
					if(is_array($search_advfields['spec'])){
						$tmp = null;
						foreach($search_advfields['spec'] as $spec)
							$tmp[] = "rse.spec_code='{$spec}'";
						$q_where[] = '('.implode(' OR ', $tmp).')';
					}else
						$q_where[] = "rse.spec_code='{$search_advfields['spec']}'";
				}
				break;
		}
		
		//Sortowanie
		if(is_array($this->config['order_by'])){
			foreach($this->config['order_by'] as $alias => $order_direct)
			$q_order[] = "{$alias} {$order_direct}";
			$q_order = "ORDER BY ".implode(', ', $q_order);
		}elseif(strlen($search_string)){
			$q_order = "ORDER BY search_rank DESC, id DESC";
		}elseif($search_type=='event'){
			$q_fields[] = "me.date_start";
			$q_order = "ORDER BY me.date_start DESC";
		}else{
			$q_order = "ORDER BY id DESC";
		}
		
		$q_fields = implode(',', $q_fields);
		$q_where = 'WHERE '.implode(' AND ', $q_where);
		$q_join = implode(' ', $q_join);
		$limit = min($glimit!=0?$glimit:99999, $plimit!=0?$plimit:99999);
		
		$sql = <<< ESQUEL
SELECT DISTINCT 
{$q_fields} 
FROM main_search ms 
{$q_join}
{$q_where}
{$q_order} 
LIMIT $limit OFFSET $slimit 
ESQUEL;
		
		$sql2 = <<< ESQUEL
SELECT COUNT(DISTINCT(ms.*)) 
FROM main_search ms 
{$q_join} 
{$q_where}
ESQUEL;
		
		$data = $this->db->getAll($sql);
		$amount = min($glimit!=0?$glimit:99999, $this->db->getOne($sql2));
		
		$data = $this->bacicElements($data);
		$data = $this->extendElements($data);
		$data = $this->addDirectLink($data);
		$data = $this->headlinesForSearch($data, $search_string, $config);
		$data = $this->addDirectLink($data);
		
		$this->elements = $data;
		$this->amount = $amount;
		
		return $data;
	}
    
    /**
     *	Tworzy tablic stronicowania
     *	@param int $range - wywietla ilo stron od aktualnej, dalsze zastpuje separatorem
     **/
    public function getPages($range = null)
    {
		$range = $range==null?$this->config['pagging_range']:$range;
		if($this->config['pagging_limit']==0)
			return null;//Stronicowanie wyczone
		
		$pages = null;
		$pages_amount = ceil($this->amount / $this->config['pagging_limit']);
		$page = $this->config['pagging_start'];
		
		if($pages_amount<=1)
			return null;//Jedna lub brak stron
		
		//pierwsza strona
		$pages['first'] = array('view' => 'first', 'goto' => 0);
		
		//popszednia strona
		if($page>0)
			$pages['prev'] = array('view' => 'prev', 'goto' => $page-1);
		
		//pierwszy separator
		if($page>$range+1+1)
			$pages['sep1'] = array('view' => 'sep', 'goto' => floor(($page-$range)/2));
		
		//miso
		$start = $page-$range;
		if($start<0) $start = 0;
		$stop = $page+$range;
		if($stop>$pages_amount) $stop = $pages_amount;
		for($i=$start; $i<$stop; $i++){
			$key = 'step'.($page+$i);
			$pages[$key] = array('view' => $i+1, 'goto' => $i);
		}
		
		//drugi separator
		if($pages_amount-$page>$range+1+1)
			$pages['sep2'] = array('view' => 'sep', 'goto' => ceil(($pages_amount-$page+$range)/2));
		
		//nastpna strona
		if($page<$pages_amount-1)
			$pages['next'] = array('view' => 'next', 'goto' => $page+1);
		
		//ostatnia strona
		$pages['last'] = array('view' => 'last', 'goto' => $pages_amount-1);
		
		$this->pages = $pages;
		return $pages;
    }
    
    /**
     *	Wyciga pola odpowiednie dla danego typu kontentu, potrzebne w przypadku kolekcji mieszanych (!!mostRated!!)
     **/
    private function bacicElements($data)
    {
		for($i=0; $i<count($data); $i++){
			$content_id = $data[$i]['id'];
			$content_type = $data[$i]['content_type'];
			$q_fields = null;
			$q_table = null;
			$q_join = null;
			$q_where = null;
			//Ustawianie tabeli oraz pobieranych pl
			$q_table = "{$this->tables[$content_type]} {$content_type}";
			foreach($this->fields[$content_type] as $alias => $field)
			$q_fields[] = "{$field} AS {$alias}";
			$q_fields = implode(', ', $q_fields);
			//Doczanie tabel sownikowych
			if(count($this->join_tables[$content_type])>0){
			foreach($this->join_tables[$content_type] as $join){
				$q_join = $q_join . " {$join['type']} {$join['table']} {$join['alias']} ON {$join['on_cond']}";
			}
			}
			$q_where = "WHERE {$this->fields[$content_type]['id']}={$content_id}";
			$sql = "SELECT $q_fields FROM $q_table $q_join $q_where";
			$data2 = $this->db->getRow($sql);
			foreach($data2 as $k => $v)
			$data[$i][$k] = $v;
		}
		return $data;
    }
    
    /**
     *	Dodaje dodatkowe pola, generowane w php oraz pobierazne z tabel zewntrznych
     **/
    private function extendElements($data, $content_type = null)
    {
		$comments = new comments();
		for($i=0; $i<count($data); $i++){
			$content_type = $data[$i]['content_type'];
			$data[$i]['type'] = $this->getElementTypeDict($content_type, $data[$i]['type_code']);//$dict_publ_type[$data[$i]['type_code']];
			$data[$i]['comments_amount'] = $comments->countComments($data[$i]['id'], $content_type);
			$data[$i]['hash'] = md5('0123456789'.$data[$i]['id'].'*_*'.$this->config['site_code'].'9876543210');
			$data[$i]['_expander'] = $this->prepare_expander($data[$i]['_expander']);
			if($content_type=='publ'){
				$data[$i]['keyword'] = $this->getpublKeyword($data[$i]['id']);
				$data[$i]['terapia_nr'] = $this->getpublEdit($data[$i]['id']);
			}
			$data[$i]['specs'] = $this->getSpecs($data[$i]['id'], $content_type);
		}
		return $data;
    }
	
	/**
	 *
	 **/
	public function headlinesForSearch($data, $string, $lang)
	{
		for($i=0; $i<count($data); $i++){
			$content_id = $data[$i]['id'];
			$content_type = $data[$i]['content_type'];
			$sql = <<< ESQUEL
SELECT 
ts_headline('polish', search, plainto_tsquery('$lang', '$string'), 'HighlightAll=TRUE, 
  ShortWord=3, MaxWords=10, MinWords=6, MaxFragments=3, FragmentDelimiter=" ... "') AS search_head 
FROM 
main_search 
WHERE 
id_elementu = '$content_id' 
AND 
content_type = '$content_type' 
ESQUEL;
			$data[$i]['search_headline'] = $this->db->getOne($sql);
			$data[$i]['search_percent'] = ceil($data[$i]['search_rank']*100);
		}
		return $data;
	}
	
	/**
	 *	Generuje link do danego kontentu (lub losuje jedn z moliwoci)
	 *	naley wykona po $this->extendElements aby mie ju _expander ;-)
	 **/
	private function addDirectLink($data)
	{
		for($i=0; $i<count($data); $i++){
			if(!isset($data[$i]['_expander']))
				continue;
			$content_id = $data[$i]['id'];
			$content_type = $data[$i]['content_type'];
			$expander = $data[$i]['_expander'];
			$sql = <<< ESQUEL
SELECT 
rc.id_kolekcji, ms.host, rcs.site_id, rcs.controller, rcs.action, rcs.parametr  
FROM rel_collection rc 
INNER JOIN rel_collection_site rcs ON rcs.id_kolekcji = rc.id_kolekcji 
INNER JOIN main_site ms ON rcs.site_id = ms.site_id 
WHERE 
rc.id_elementu = '$content_id' AND 
rc.content_type = '$content_type' AND 
ms.host IS NOT NULL AND 
rcs.last_use>NOW() - INTERVAL '3 days' 
ORDER BY rc.id_kolekcji DESC 
ESQUEL;
			$opt = $this->db->getAll($sql);
			if(!$opt)
				continue;
			$opt = $opt[rand(0, count($opt)-1)];
			$hash = md5('0123456789'.$content_id.'*_*'.$opt['site_id'].'9876543210');
			if($opt['controller']=='home')
				$link = $opt['host'].$opt['action'].'/'.$expander.'/'.$opt['parametr'].','.$hash.'/';
			else
				$link = $opt['host'].$opt['controller'].'/'.$opt['action'].'/'.$expander.'/'.$opt['parametr'].','.$hash.'/';
			$data[$i]['direct_link'] = $link;
		}
		return $data;
	}
    
    /**
     *	Pobiera typ elementu danej kolekcji
     *	@param string $content_class - rodzaj elementu
     *	@param string $code - kod typu elementu
     *	@return string - typ elementu
     **/
    private function getElementTypeDict($content_type, $code)
    {
		switch($content_type) {
			case "news":
			$sql = "SELECT news_type_name AS name FROM dict_news_type WHERE news_type_code = '{$code}' ORDER BY name";
			break;
			case "event":
			$sql = "SELECT event_type_name AS name FROM dict_event_type WHERE event_type_code = '{$code}' ORDER BY name";
			break;
			case "link":
			$sql = "SELECT link_type_name AS name FROM dict_link_type WHERE link_type_code = '{$code}' ORDER BY name";
			break;
			case "publ_old":
			$sql = "SELECT name FROM dict_publ_type WHERE publ_type_code = '{$code}' ORDER BY name";
			break;
			case "publ":
			$sql = "SELECT name FROM dict_publ_type2 WHERE publ_type_code = '{$code}' ORDER BY name";
			break;
			default:
			break;
		}
		$type = $this->db->getOne($sql);	
		return $type;
    }
    
    /**
     *	Pobiera sowa kluczowe publikacji
     *	@param int $publ_id - identyfikator publikacji
     *	@return string - sowa kluczowe
     **/
    private function getpublKeyword($publ_id)
    {
		$sql = "SELECT keyword FROM rel_publ_keyword WHERE id_publ='{$publ_id}'";
		$keywords = $this->db->getAll($sql);
		$tmp = null;
		foreach($keywords as $keyword){
			$tmp[] = $keyword['keyword'];
		}
		if($tmp == null)
			return null;
		$keywords = implode(', ', $tmp);
		return $keywords;
    }
	
	/**
	 *
	 **/
	public function getpublEdit($publ_id)
	{
		$sql = <<< ESQUEL
SELECT 
a.book_nr 
FROM 
main_editions a 
INNER JOIN 
rel_publ_edition b 
ON 
a.id_edition=b.id_edition 
WHERE 
id_publ = '{$publ_id}'
ESQUEL;
		$nr = $this->db->getOne($sql);
		if($nr)
			return $nr;
		return null;
	}
    
    /**
     *
     **/
    private function getSpecs($content_id, $content_type)
    {
		$rel_table = $this->rel_specs[$content_type];
		$id = str_replace($content_type.'.' ,'', $this->fields[$content_type]['id']);
		
		$sql = <<< ESQUEL
SELECT b.spec_code as key, b.mdm_name as value 
FROM 
{$rel_table} a 
INNER JOIN 
dict_spec b 
ON a.spec_code=b.spec_code
WHERE 
a.{$id} = '{$content_id}' 
ORDER BY b.mdm_name
ESQUEL;
		$specs = $this->db->getCol($sql);
		return $specs;
    }
    
    /**
     *	Przygotowuje tekst wstawiany do url'a jako rozpychacz
     *	@param string $expander - tekst do przerobienia na url'owy rozpychacz
     *	@return string - url'owy rozpychacz
     **/
    private function prepare_expander($expander)
    {
		$from = array('', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', ' ');
		$to = array('e', 'e', 'o', 'o', 'a', 'a', 's', 's', 'l', 'l', 'z', 'z', 'z', 'z', 'c', 'c', 'n', 'n', '_');
		$expander = str_replace($from, $to, $expander);
		$from = array('!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '-', '+', '=', '`', '~', '[', ']', '{', '}', ':', ';', '"', "'", '?', '/', '\\', ',', '.', '<', '>');
		$to = '_';
		$expander = str_replace($from, $to, $expander);
		$from = array('______', '_____', '____', '___', '__');
		$to = '_';
		$expander = str_replace($from, $to, $expander);
		$expander = trim($expander, '_');
		return $expander;
    }
	
    /**
     *	Zwiksza licznik ogldnie danego kontentu
     **/
    private function raiseCounter($content_id = null, $content_type = null)
    {
		$content_id = $content_id==null?$this->element['id']:$content_id;
		$content_type = $content_type==null?$this->element['content_type']:$content_type;
		$sql = "INSERT INTO main_view_content (content_id, content_type) VALUES ($content_id, '$content_type')";
		$this->db->exec($sql);
    }
	
    /**
     *	Zapisuje miejsca w ktrych wywietlane s kolekcje
     **/
    private function saveCollectionUse()
    {
		if(!$this->config['save_collect_use'])
			return false;
		
		if(strpos($_SERVER['HTTP_HOST'], 'tst.') || strpos($_SERVER['HTTP_HOST'], 'dev3.'))
			return false;
		
		$site_id = null;
		$controller = null;
		$action = null;
		$parametr = 'id';
		
		if(defined('SITE_ID')) $site_id = SITE_ID;
		if(defined('CONTR_NAME')) $controller = CONTR_NAME;
		if(defined('CONTR_ACTION')) $action = CONTR_ACTION;
		
		if(!$site_id || !$controller || !$action || !$parametr)
			return false;
		
		foreach($this->data as $collection){
			$id_kolekcji = $collection['id_kolekcji'];
			
			$sql = "SELECT COUNT(1) FROM rel_collection_site WHERE site_id='{$site_id}' AND ".
			"controller='{$controller}' AND action='{$action}' AND id_kolekcji='{$id_kolekcji}'";
			if($this->db->getOne($sql)){
			$sql = "UPDATE rel_collection_site SET last_use=NOW() WHERE id_kolekcji='{$id_kolekcji}' AND ".
				   "site_id='{$site_id}' AND controller='{$controller}' AND action='{$action}'";
			$this->db->exec($sql);
			}else{
			$sql = "INSERT INTO rel_collection_site (id_kolekcji, site_id, controller, action, parametr) ".
				   "VALUES ('$id_kolekcji', '$site_id', '$controller', '$action', '$parametr')";
			$this->db->exec($sql);
			}
		}
		
		return true;
    }
	 /**
     *	Pobiera id elementw przypisane do danej kolekcji
     **/
	public function getPublications($collections, $limit, $offset = 0)
	{
			
			if($offset)
		    $offset=$limit*$offset - $limit;
			
			$sql ="SELECT 
			a.content_id, 
			a.author, 
			a.title,
			a.keywords, 
			a.head,
			c.avatar_id,
			left(a.body, 250) as short,
			a.body,
			LOWER(a.title) as _expander
			FROM 
			pl_content.main_content_publ a 
			INNER JOIN 
			pl_content.rel_content_collection b 
			ON 
			a.content_id = b.main_content_id
			LEFT JOIN 
            pl_content.main_content c
            ON
            a.content_id = c.content_id  
			WHERE 
			b.main_collection_id = '{$collections}' 
			ORDER BY 
			a.content_id desc
			LIMIT
			{$limit}
			OFFSET
			{$offset}
			";
			
			$elements = $this->db->getAll($sql);
			return $elements;
	}
	
	public function getPublication($id)
	{
					
				$sql ="SELECT
				*,
				a.title as titlee
				FROM pl_content.main_content_publ a
				LEFT JOIN
				pl_content.main_journals b
				on
				a.journal_id=b.id_journal
				WHERE content_id='{$id}'
				";
				
				$publication = $this->db->getAll($sql);				
				return $publication;
							
	}
	public function getEvents($collections, $limit, $offset)
	{
			
			if($offset != 0)
		    $offset=$limit*$offset - $limit;
			$date=date('Y-m-d');
			$sql ="SELECT 
			*			
			FROM 
			pl_content.main_content_event a 
			INNER JOIN 
			pl_content.rel_content_collection b 
			ON 
			a.content_id = b.main_content_id
			WHERE 
			b.main_collection_id = '{$collections}' and a.date_start > '{$date}'
			
			ORDER BY 
			a.date_start asc
			LIMIT
			{$limit}
			OFFSET
			{$offset}
			";
			
			$elements = $this->db->getAll($sql);
			return $elements;
	}
	
	public function getEvent($id)
	{
	
				
				$sql ="SELECT
				*				
				FROM pl_content.main_content_event	
				WHERE content_id='{$id}'
				";
				
				$event = $this->db->getAll($sql);				
				return $event;
				
			
	}
	
	
	public function getNews($collections, $collections_2, $collections_3, $limit, $offset = 0)
	{
			
			$sql ="SELECT 
			a.content_id, 
			a.author, 
			a.title,
			a.date,
			a.head,
			c.avatar_id as avatar
			FROM 
			pl_content.main_content_news a 
			INNER JOIN 
			pl_content.rel_content_collection b 
			ON 
			a.content_id = b.main_content_id
			LEFT JOIN 
            pl_content.main_content c
            ON
            a.content_id = c.content_id  
			WHERE 
			b.main_collection_id = '{$collections}' OR b.main_collection_id = '{$collections_2}' OR  b.main_collection_id = '{$collections_3}'
			ORDER BY 
			a.content_id desc
            LIMIT 
			{$limit}
			OFFSET
			{$offset}
			";
			
			$elements = $this->db->getAll($sql);
			return $elements;
	}
	
	public function getNews2($whr, $limit, $offset = 0)
	{
	
			if($offset != 0)
		    $offset=$limit*$offset - $limit;
			
			$sql ="SELECT 
			
			distinct(a.content_id), 
			a.author, 
			a.title,
			a.date,
			a.head,
			a.news_type,
			c.avatar_id as avatar
			FROM 
			pl_content.main_content_news a 
			INNER JOIN 
			pl_content.rel_content_collection b 
			ON 
			a.content_id = b.main_content_id
			LEFT JOIN 
            pl_content.main_content c
            ON
            a.content_id = c.content_id  
			WHERE 
			{$whr} 
			ORDER BY 
			a.content_id desc
            LIMIT 
			{$limit}
			OFFSET
			{$offset}
			";
			
			$elements = $this->db->getAll($sql);
			return $elements;
	}
	 
	public function getOneNews($id)
	{
	
			$sql="SELECT 
			*
			FROM 
			pl_content.main_content_news a
			JOIN
			pl_content.main_content b
			ON
			a.content_id = b.content_id
			WHERE
			a.content_id  = '{$id}'
			";
				
			$news = $this->db->getAll($sql);				
			return $news;		
	}
	public function getAmountCollection($whr)
	{
	
			$sql="SELECT
			COUNT(*)
			FROM 
			pl_content.main_content a
			JOIN
			pl_content.rel_content_collection b
			ON
			a.content_id = b.main_content_id
			WHERE
			{$whr}
			";
			
			$amount = $this->db->getOne($sql);				
			return $amount;
	}
    public function getAmountEventCollection($whr)
    {

        $sql="SELECT
			COUNT(*)
			FROM
			pl_content.main_content a
			JOIN
			pl_content.rel_content_collection b
			ON
			a.content_id = b.main_content_id
			JOIN
			pl_content.main_content_event c
			ON
			c.content_id = a.content_id
			WHERE
			{$whr}
			";

        $amount = $this->db->getOne($sql);
        return $amount;
    }
	public function getComments($id_publ) {

		
		//commenty przypiete do publ
		$sql = "SELECT * FROM pl_content.main_content_comment WHERE content_id={$id_publ} and lang_id='PL' ORDER BY add_date DESC ";
        $comments = $this->db->getAll($sql);
		//iloc
		
		 
		return $comments;
	}
	
	public function getCountComments($id_publ){ 
	 
	
	
		$sql= "SELECT count(*) FROM pl_content.main_content_comment WHERE content_id={$id_publ} and lang_id='PL'";		
		$count = $this->db->getOne($sql);
		
		
		return $count; 
		
	}
	 
	public function SaveComments($content_id, $lang_id,$title, $body, $nick, $user_login, $source_domain, $source_site){
	
							
		$sql = "INSERT INTO pl_content.main_content_comment (content_id,lang_id,title,body,nick,user_login, source_domain, source_site) VALUES (:content_id,:lang_id,:title,:body,:nick,:user_login,:source_domain,:source_site)";
		$q = $this->db->prepare($sql);
		$q->execute(array(':content_id'=>$content_id,':lang_id'=>$lang_id,':title'=>$title,':body'=>$body,':nick'=>$nick,':user_login'=>$user_login,':source_domain'=>$source_domain,':source_site'=>$source_site));
		
		
		return true;
		
	}
		
}

?>