<?php

/******************************************************************************
 *                                                                            *
 *      Klasa obsługująca komentarzy                                          *
 *      @category data                                                        *
 *      @author Sławek                                                        *
 *                                                                            *
 *                                                                            *
 ******************************************************************************/

class comments
{
    /**
     *  Uchwyt do bazy danych poprzez PDO
     **/
    public $db = null;
    
    /**
     *  Tablica konfiguracji domyślnej
     **/
    private $config = array(
        'element_type' => 'news',   //typ elementu
        'reply_levels' => 5,        //limit zagnieżdżeń odpowiedzi
        'source_filter' => null,    //ograniczenie źródeł z których mogą pochodzić komentarze, null = wszystkie, str = tylko jedno, array = kilka
        'sql_filter' => null,       //warun(ek|ki) sql dla pobieranej listy komentarzy, np. source='cipronet.pl' @deprecated
        'source' => 'esculap.pl',   //źródło pochodzenia komentarza, @deprecated
        'source_domain' => 'self',  //domena źródeł komętarzy 'self' = ta w której klasa jest powołana, może być array !!!
        'source_site' => null,      //site źródła komętarza 'self' = ta w której klasa jest powołana, może być array !!!
        'username' => '',           //użytkownik aktualnie oglądajęcy/komentujący
    );
    
    /**
     *  Loginy które mogą zarządzać komentarzami
     **/
    private $moderators = array('gerej', 'jkosiaty', 'dstynski', 'plawecki');
    
    /**
     *  Id elemtu komentowanego
     **/
    private $elementID;
    
    /**
     *  Tablica z komentarzami
     **/
    public $data = null;
    
    /**
     *  Ilość wszystkich komentarzy
     **/
    public $amount_all = null;
    
    /**
     *  Ilość komentarzy pierwszego poziomu
     **/
    public $amount_first = null;
    
    /**
     *  Tablica błędów
     **/
    public $errors = null;
    
    /** Źródło komętarza **/
    private $source_domain;
    private $source_site;
    
    /**
     *  Konstruktor, dodaje i pobiera komentarze
     *  @param int $elementID - id elementu dla którego pobierane są komentarze
     *  @param array $config - tablica konfiguracyjna
     *  @param array $data - tablica z danymi do zapisu array('addComment'=>???, 'subject'=>temat,
     *                       'content'=>treść, 'sign'=>podpis, 'parent_id'=>id nadżędnego lub nic)
     *  @global object $db - obiekt bazy danych przekazywany do parametru obiektu
     **/
    public function __construct($elementID = 0, $config = null, $data = null)
    {
        $this->elementID = $elementID;
        if(isset($_SESSION['auth']['username']))
            $this->config['username'] = $_SESSION['auth']['username'];
        if(is_array($config))
            foreach($config as $key => $value)
                $this->config[$key] = $value;
        global $db;
        $this->db = $db;
        
        $this->source_domain = str_replace(array('dev3.', 'tst.', 'www.'), '', $_SERVER['HTTP_HOST']);
        $this->source_site = defined('SITE_ID')?SITE_ID:null;
        
        //Próba dodania komentarza
        if(is_array($data) && isset($data['addComment']) && !empty($data['subject']) && !empty($data['content']))
            $this->addComment($data['subject'], $data['content'], $data['sign'], $data['parent_id']);
        elseif(is_array($data) && isset($data['comment_id']) && $data['comment_id']>0)
            $this->deleteComment($data['comment_id']);
        
        //Pobieranie listy komentarzy
        if($this->elementID>0)
            $this->getComments();
    }
    
    /**
     *  Pobiera rekurencyjnie komentarze wraz z ich odpowiedziami
     *  @param integer $parent_id - id komentarza rodzica
     *  @param integer $reply_level - poziom zagnieżdżenia
     *  @param array $comments - referencja do tablicy z komentarzami
     *  @return array - tablica z komentarzami
     **/
    public function getComments($parent_id = 0, $reply_level = 0, &$comments = null)
    {
        if($parent_id == 0){
            $comments = null;
            $whr_f = null;
            $whr_s = null;
            if($this->config['sql_filter']!=null)
                $whr_f = "AND ({$this->config['sql_filter']})";
            if($this->config['source_filter']!=null){
                if(!is_array($this->config['source_filter']))
                    $this->config['source_filter'] = array($this->config['source_filter']);
                if(count($this->config['source_filter'])>0)
                    $whr_s = "AND source_id IN ('".implode("', '", $this->config['source_filter'])."')";
            }
            if($this->config['source_domain']){
                if(!is_array($this->config['source_domain']))
                    $this->config['source_domain'] = array($this->config['source_domain']);
                if(count($this->config['source_domain'])>0)
                    foreach($this->config['source_domain'] as $source_domain)
                        if($source_domain=='self')
                            $whr_s .= " AND source_domain LIKE '%{$this->source_domain}%'";
                        else
                            $whr_s .= " AND source_domain LIKE '%$source_domain%'";
            }
            if($this->config['source_site']){
                if(!is_array($this->config['source_site']))
                    $this->config['source_site'] = array($this->config['source_site']);
                if(count($this->config['source_site'])>0)
                    foreach($this->config['source_site'] as $source_site)
                        if($source_site=='self')
                            $whr_s .= " AND source_site LIKE '%{$this->source_site}%'";
                        else
                            $whr_s .= " AND source_site LIKE '%$source_site%'";
            }
            $sql = "SELECT * FROM main_commentaries2 WHERE content_id='{$this->elementID}' ".
                   "AND content_type='{$this->config['element_type']}' AND parent='0' AND deleted IS NULL {$whr_f} ".
                   "{$whr_s} ORDER BY commentary_date DESC, commentary_time DESC";
        }else{
            $sql = "SELECT * FROM main_commentaries2 WHERE parent='{$parent_id}' AND deleted IS NULL ".
                   "ORDER BY id_commentary ASC";
        }
        $z1 = ($reply_level<=$this->config['reply_levels'])?1:0;
        $z2 = in_array($this->config['username'], $this->moderators)?1:0;
        $data = $this->db->getAll($sql);
        
        for($i=0; $i<count($data); $i++){
            $data[$i]['repamo'] = $this->countCommentReplys($data[$i]['id_commentary']);
            $data[$i]['replays'] = ($data[$i]['repamo']>0)?1:0;
            $data[$i]['replay'] = $z1;
            $data[$i]['change'] = $z2;
            $data[$i]['level'] = $reply_level;
            $comments[] = $data[$i];
            if($data[$i]['repamo'])
                $this->getComments($data[$i]['id_commentary'], $reply_level+1, $comments);
        }
        
        if($parent_id == 0){
            $this->amount_all = count($comments);
            $this->amount_first = count($data);
            $this->data = $comments;
        }
        return $comments;
    }
    
    /**
     *  Zlicza ilość odpowiedzi
     *  @param integer $comment_id - id komentarza
     *  return integer - ilość odpowiedzi
     **/
    public function countCommentReplys($comment_id)
    {
        $amount = 0;
        $sql = "SELECT id_commentary FROM main_commentaries2 WHERE parent='{$comment_id}' AND deleted IS NULL";
        
        $data = $this->db->getAll($sql);
        $amount += count($data);
        for($i=0;$i<count($data);$i++)
        {
            $amount += $this->countCommentReplys($data[$i]['id_commentary']);
        }
        return $amount;
    }
    
    /**
     *  Sprawdza poziom zagnieżdżenia komentarza
     *  @param int $commentary_id - identyfikator komentarza
     *  @param int $step - parametr dla rekurencji, nie należy go podawać
     *  @return int - Poziom zagnieżdżenia
     **/
    public function checkReplyLevel($commentary_id, $step = 0)
    {
        $sql = "SELECT parent FROM main_commentaries2 WHERE id_commentary='{$commentary_id}'";
        $parent_id = $this->db->getOne($sql);
        if($parent_id>0)
            $step = $this->checkReplyLevel($parent_id, $step+1);
        
        return $step;
    }
    
    /**
     *  Dodaje nowy komentarz do elementu
     *  @param str $subject - Temat komentarza
     *  @param str $content - Treść komentarza
     *  @param str $sign - Podpis pod komentarzem
     *  @param int $parent_id - id komentarza, tylko w przypadku odpowiedzi na inny komentarz
     *  @return bool - Czy się udało
     **/
    public function addComment($subject, $content, $sign = '', $parent_id = 0)
    {
        $subject = strip_tags($subject);
        $content = strip_tags($content);
        $sign = strip_tags($sign);
        if($this->checkReplyLevel($parent_id)>=$this->config['reply_levels']){
            $this->errors['err_reply_level_acquired'] = true;
            return false;
        }
        if($this->config['username']==''){
            $this->errors['err_need_user_login'] = true;
            return false;
        }
        if(!is_numeric($parent_id)) $parent_id = 0;
        $sql = "SELECT COUNT(*) FROM main_commentaries2 WHERE content_type='{$this->config['element_type']}' AND ".
               "content_id='{$this->elementID}' AND title LIKE '{$subject}' AND ".
               "nickname='{$sign}' AND parent='{$parent_id}' AND commentary LIKE '{$content}'".
               "AND deleted IS NULL";
        if($this->db->getOne($sql)>0)//Zabezpieczenie przed dodaniem dwóch identycznych komentarzy
            return true;
        if($parent_id>0){
            $sql = "SELECT COUNT(*) FROM main_commentaries2 WHERE content_type='{$this->config['element_type']}' AND ".
                   "content_id='{$this->elementID}' AND id_commentary='{$parent_id}'";
            if($this->db->getOne($sql)==0){
                $this->errors['err_wrong_thread'] = true;
                return false;
            }
        }
        
        $site = $this->source_site?"'{$this->source_site}'":'NULL';
        $sql = "INSERT INTO main_commentaries2 (content_type, content_id , user_login, commentary, nickname, ".
               "commentary_date, source_id, commentary_time, parent, title, source_domain, source_site) ".
               "VALUES ('{$this->config['element_type']}', '{$this->elementID}', '{$this->config['username']}', ".
               "'{$content}', '{$sign}', NOW(), '{$this->config['source']}', NOW(), '{$parent_id}', '{$subject}',".
               "'{$this->source_domain}', {$site})";
        $this->db->exec($sql);
        return true;
    }
    
    /**
     *  Usuwa komentarz
     *  @param int $commentary_id - identyfikator komentarza
     *  @return bool - Czy się udało
     **/
    public function deleteComment($commentary_id)
    {
        if(in_array($this->config['username'], $this->moderators)){
            $comments = $this->getComments($commentary_id, 1);
            if(is_array($comments) && count($comments)>0)
                foreach($comments as $comment){
                    $sql = "UPDATE main_commentaries2 SET deleted=NOW() WHERE id_commentary = '{$comment['id_commentary']}'";
                    $this->db->exec($sql);
                }
            $sql = "UPDATE main_commentaries2 SET deleted=NOW() WHERE id_commentary = '{$commentary_id}'";
            $this->db->exec($sql);
            return true;
        }else{
            $this->errors['err_need_mod_priv'] = true;
            return false;
        }
    }
    
    /**
     *  Tworzy skrypt JavaScript obsługujący formularz
     *  @param array $config2 - tablica konfiguracyjna
     *  @return str - kod Javascript
     **/
    public function renderJS($config2 = null)
    {
        $config = array(
            'id_comment_form_element' => 'comment_form',        //Element zawierający formularze
            'id_add_form' => 'add_comment',                     //Formularz dodawania komentarza
            'id_delete_form' => 'delete_comment',               //Formularz usuwania komentarza
            'name_comment_space' => 'comment_expand_space',     //Nazwa elementów zawierających odpowiedzi na komentarze
            'name_comment_expand_images' => 'comment_expand_image', //Nazwa grafik reprezentujących ukrycie/pokazanie odpowiedzi na komentarze
            'image_expand' => 'images/icon_zwiniete.gif',       //Grafika reprezentująca możliwość pokazania odpowiedzi do komentarzy
            'image_collapse' => 'images/icon_rozwiniete.gif',   //Grafika reprezentująca możliwość ukrycia odpowiedzi do komentarzy
            'name_expand' => 'rozwiń komentarze',               //Napis rozwijania komentarzy
            'name_collapse' => 'zwiń komentarze',               //Napis zwijający komentarze
        );
        
        foreach($config as $key => $value){
            if(isset($this->config[$key]) && !empty($this->config[$key]))
                $config[$key] = $this->config[$key];
            if(isset($config2[$key]) && !empty($config2[$key]))
                $config[$key] = $config2[$key];
        }
        
        //echo '<pre>';print_r($this->config);echo '</pre>';
        
        /*if(is_array($config2))
            foreach($config2 as $option => $value)
                $config[$option] = $value;*/
        
        $script = <<< JS
<script type="text/javascript">
/* <![CDATA[ */
function addNewComment(){
    var div_id = '{$config['id_comment_form_element']}';
    var add_form = '{$config['id_add_form']}';
    var username = '{$this->config['username']}';
    if(username==''){
        alert('Aby dodawać komentarze musisz być zalogowany!');
        return;
    }
    document.forms[add_form].subject.value = '';
    document.forms[add_form].content.value = '';
    document.forms[add_form].sign.value = '';
    document.forms[add_form].parent_id.value = 0;
    document.getElementById(div_id).style.display = '';
    return;
}

function addChildComment(parent_id){
    var div_id = '{$config['id_comment_form_element']}';
    var add_form = '{$config['id_add_form']}';
    var username = '{$this->config['username']}';
    if(username==''){
        alert('Aby dodawać komentarze musisz być zalogowany!');
        return;
    }
    document.forms[add_form].subject.value = '';
    document.forms[add_form].content.value = '';
    document.forms[add_form].sign.value = '';
    document.forms[add_form].parent_id.value = parent_id;
    document.getElementById(div_id).style.display = '';
    return;
}

function addFormSubmit(){
    var add_form = '{$config['id_add_form']}';
    if(document.forms[add_form].subject.value.length<1){
        alert('Wpisz temat swojej wypowiedzi!');
        return false;
    }
    if(document.forms[add_form].content.value.length<5){
        alert('Wpisz dłuższą wypowiedź!');
        return false;
    }
    if(document.forms[add_form].sign.value.length<1){
        alert('Uzupełnij podpis swojej wypowiedzi!');
        return false;
    }
    return true;
}

function cancelComment(){
    var div_id = '{$config['id_comment_form_element']}';
    var add_form = '{$config['id_add_form']}';
    document.getElementById(div_id).style.display = 'none';
    document.forms[add_form].subject.value = '';
    document.forms[add_form].content.value = '';
    document.forms[add_form].sign.value = '';
    document.forms[add_form].parent_id.value = 0;
    return;
}

function collapseExpandComments(section_id, image_id){
    var section_id = document.getElementById(section_id);
    var image_id = document.getElementById(image_id);
    var image_collapse = '{$config['image_collapse']}';
    var image_expand = '{$config['image_expand']}';
    
    if(section_id.style.display=='none'){
        section_id.style.display='';
        image_id.src = image_collapse;
    }else{
        section_id.style.display='none';
        image_id.src = image_expand;
    }
    return;
}

function collapseAllComments(trigger){
    var list = document.getElementsByName('{$config['name_comment_space']}');
    var list2 = document.getElementsByName('{$config['name_comment_expand_images']}');
    var image_collapse = '{$config['image_collapse']}';
    var image_expand = '{$config['image_expand']}';
    var name_collapse = '{$config['name_collapse']}';
    var name_expand = '{$config['name_expand']}';
    var image = '';
    var name = '';
    var display = '';
    if(list.length==0)
        return;
    if(trigger.innerHTML == name_collapse){
        image = image_expand;
        name = name_expand;
        display = 'none';
    }else{
        image = image_collapse;
        name = name_collapse;
        display = '';
    }
    
    trigger.innerHTML = name;
    
    for(i=0; i<list.length; i++){
        document.getElementById(list.item(i).id).style.display = display;
    }
    
    for(i=0; i<list2.length; i++){
        document.getElementById(list2.item(i).id).src = image;
    }
    return;
}

function deleteComment(id_comment){
    var delete_form = '{$config['id_delete_form']}';
    if(window.confirm('Czy na pewno chcesz usunąć komentarz nr. ' + id_comment + '.')){
        document.forms[delete_form].comment_id.value = id_comment;
        document.forms[delete_form].submit();
    }
    return;
}
/* ]]> */
</script>
JS;
        return $script;
    }
    
    /**
     *  Zlicza ilość komętarzy danego contentu
     *  @param integet $elementID - identyfikatior contentu
     *  @param string $content_type - rodzaj contentu
     **/
    public function countComments($elementID, $content_type = null)
    {
        if($content_type==null)
            $content_type = $this->config['element_type'];
        
        $sql = <<< ESQUEL
SELECT COUNT(1) FROM main_commentaries2 
WHERE content_type='{$content_type}' 
AND content_id={$elementID} 
AND deleted IS NULL 
ESQUEL;
        $amount = $this->db->getOne($sql);
        return $amount;
    }
}

?>