<?php
/**
 *
 * Klasa klient SOAP dla autoryzacji e-login
 * @author sroszczyk
 * @throws SoapFault SoapUserAuthorizeException
 * @version 0.1
 * @example warto rozszerzac ta klase dodajac przechowywanie pol ($usercode, $userdata, $soapAllow) w $_SESSION & $_COOKIE
 *
 */
class SoapUserAuthorize{
    /**
     * Kod strony, definiowany specjalnie dla kazdej aplikacji webowej
     */
    protected $sitecode = null;

    /**
     * Kod uzytkownika => identyfikator sesji
     */
    protected $usercode = null;

    /**
     * Dane uzytkownika, tablica dostepnych dla danej instancji danych uzytkownika
     */
    protected $userdata = null;

    /**
     * Obiekt klienta soap
     */
    protected $soapClient = null;

    /**
     * Sprawdza czy dostepny jest soap czy robic redirecta
     */
    protected $soapAllow = null;
    
    /**
     * Czy dany user jest zalogowany
     */
    protected $isLogged = null;

    /**
     * SOAP server
     */

   ///public static $SOAP_LOCATION = 'http://sauth.esculap.pl/soap';
    public static $SOAP_LOCATION = 'http://sauth.esculap.pl/soap_com';

    /**
     * Konstruktor moze ustawis kod strony i kod uzytkownika
     * @param string $sitecode
     * @param string $usercode
     */
    public function __construct($sitecode = null, $usercode = null){
        $this->sitecode = $sitecode;
        $this->usercode = $usercode;

       $options = array(
            'location' => self::$SOAP_LOCATION,
        	'uri' => self::$SOAP_LOCATION,
            //'location' => $SOAP_LOC,
        	//'uri' => $SOAP_LOC,
            'exceptions' => true,
			'wsdl_cache' => 0,
			'soap_version' => 2
			);

        $this->soapClient = new SoapClient(null, $options);
		
		 /*
		$options = array(
            'soap_version' => 2,
			'wsdl_cache' => 0
		);
        $this->soapClient = new SoapClient('http://dev6.esculap.pl/authorize.wsdl', $options);*/
    }

    /**
     * Ustawia kod strony
     * @param string $sitecode
     */
    public function setSiteCode($sitecode){
        $this->sitecode = $sitecode;
    }

    /**
     * Ustawia kod usera
     * @param string $usercode
     */
    public function setUserCode($usercode){
        $this->isLogged = null;
        $this->usercode = $usercode;
    }

    /**
     * Zwraca kod strony
     * @return string
     */
    public function getSiteCode(){
        return $this->sitecode;
    }

    /**
     * Zwraca kod usera
     * @return unknown_type
     */
    public function getUserCode(){
        return $this->usercode;
    }

    /**
     * Czy uzytkownik zalogowany
     * @throws SoapFault SoapUserAuthorizeException
     * @return boolean
     */
    public function isLogged(){
        if($this->isLogged===null){
            $result = $this->soapClient->isLogged($this->sitecode, $this->usercode);
            list($status, $msg) = explode('->', $result);
            if($status == 'success'){
                $this->isLogged = true;
            }else{
                if($msg == 'bad_usercode'){
                    $this->isLogged = false;
                }else{
                    throw new SoapUserAuthorizeException("isSoapAuthenticate->{$msg}");
                }
            }
        }
        return $this->isLogged;
    }

    /**
     * Czy mozliwa jest autoryzacja przez soap
     * @throws SoapFault SoapUserAuthorizeException
     * @return boolean
     */
    public function isSoapAuthenticate(){
        if($this->soapAllow===null){
            $result = $this->soapClient->isSoapAllowed($this->sitecode);
            list($status, $msg) = explode('->', $result);
            if($status == 'success'){
                if($msg == 'on') $this->soapAllow = true;
                else $this->soapAllow = false;
            }else{
                throw new SoapUserAuthorizeException("isSoapAuthenticate->{$msg}");
            }
        }
        return $this->soapAllow;
    }

    /**
     * Przekierowanie HTTP na strone esculaplogin.com
     * @param string $url
     * @param boolean $return
     * @return string
     */
    public function httpLogin($url = null, $return = false){
        if($url){
            $url = base64_encode($url);
        }elseif(isset($_SERVER['HTTP_HOST'])){
            list($protocol, $ver) = explode('/', $_SERVER['SERVER_PROTOCOL']);
            $protocol = $protocol?strtolower($protocol):'http';
            $host = $_SERVER['HTTP_HOST'];
            $uri = isset($_SERVER['REQUEST_URI'])?$_SERVER['REQUEST_URI']:'';
            $url = base64_encode($protocol.'://'.$host.$uri);
        }else{
            $url = null;
        }
        $uri = null;
        if($this->sitecode) $uri[] = 'sc/'.$this->sitecode;
        if($url) $uri[] = 'url/'.$url;
        if($uri) $uri = implode('/', $uri);
        $surl = 'http://esculaplogin.com/user/login/'.$uri;
        if($return) return $surl;
        header("Location: $surl");
        exit;
    }

    /**
     * Logowanie przez soap
     * @param unknown_type $login
     * @param unknown_type $pass
     * @throws SoapFault SoapUserAuthorizeException
     * @return boolean
     */
    public function soapLogin($login, $pass){
        $us = isset($_SERVER['HTTP_USER_AGENT'])?$_SERVER['HTTP_USER_AGENT']:null;
        $up = isset($_SERVER['HTTP_VIA'])?$_SERVER['HTTP_VIA']:null;
        $ui = isset($_SERVER['REMOTE_ADDR'])?$_SERVER['REMOTE_ADDR']:null;
        $result = $this->soapClient->beginSession($this->sitecode, $login, $pass, $us, $up, $ui);
        list($status, $msg) = explode('->', $result);
        if($status == 'success'){
            $this->usercode = $msg;
            $this->isLogged = true;
            return true;
        }else{
            throw new SoapUserAuthorizeException("soapLogin->{$msg}");
        }
		//trigger_error("logout".var_dump($status));
        return false;
    }

    /**
     * Wylogowanie
     * @throws SoapFault SoapUserAuthorizeException
     * @return boolean
     */
    public function logout(){
        if($this->usercode){
            $result = $this->soapClient->endSession($this->sitecode, $this->usercode);
            list($status, $msg) = explode('->', $result);
            if($status == 'success'){
                $this->usercode = null;
                $this->isLogged = false;
                return true;
            }else{
                throw new SoapUserAuthorizeException("logout->{$msg}");
            }
        }
		//trigger_error("logout".var_dump($status));
        return false;
    }

    /**
     * Odnowienie istniejace sesji oraz zapis statystyk
     * @param string $last_logged
     * @throws SoapFault SoapUserAuthorizeException
     * @return boolean
     */
    public function refresh($last_logged_user = null){
        if($this->usercode){
            $host = isset($_SERVER['HTTP_HOST'])?$_SERVER['HTTP_HOST']:null;
            $uri = isset($_SERVER['REQUEST_URI'])?$_SERVER['REQUEST_URI']:null;
            $referer = isset($_SERVER['HTTP_REFERER'])?$_SERVER['HTTP_REFERER']:null;
            if($this->isLogged()){
                $result = $this->soapClient->refresh($this->sitecode, $this->usercode, $host, $uri, $referer);
            }elseif($last_logged_user){
                $result = $this->soapClient->refreshUnlogged($this->sitecode, $last_logged_user, $host, $uri, $referer);
				
            }else{
                return false;
            }
            list($status, $msg) = explode('->', $result);
            if($status == 'success'){
                return true;
            }else{
                throw new SoapUserAuthorizeException("refresh->{$msg}");
            }
        }
		//trigger_error("logout".var_dump($last_logged_user));
        return false;
    }

    /**
     * Zwraca tablice danych uzytkownika lub null gdy jej nie ma
     * @throws SoapFault SoapUserAuthorizeException
     * @return array
     */
    public function getUserData(){
        if($this->userdata==null && $this->isLogged()){
            $result = $this->soapClient->userData($this->sitecode, $this->usercode);
            list($status, $msg) = explode('->', $result);
            if($status == 'success'){
				
                $this->userdata = unserialize($msg);
            }else{
                throw new SoapUserAuthorizeException("getUserData->{$msg}");
            }
        }
        return $this->userdata;
    }
}

/**
 * Wyjatki dla klasy SoapUserAuthorize
 * @author sroszczyk
 */
class SoapUserAuthorizeException extends Exception{}
