<?php

/******************************************************************************
 *                                                                            *
 *      Klasa obsługująca dynamiczne grafiki powiązane z różnymi typami       *
 *      kontentu oraz innymi obiektami                                        *
 *                                                                            *
 *                                                                            *
 *                                                                            *
 ******************************************************************************/

class dimage
{
    /**
     *  Uchwyt bazy danych
     **/
    private $db;
    
    /**
     *  Tryb pracy deweloperski lub produkcyjny zależny od dostępnej bazy danych
     **/
    private $dev_mode = false;
    
    /**
     *  Folder zawierający grafiki
     **/
    private $img_root;
    
    /**
     *  Limit wersji przeskalowanych obrazka
     **/
    private $scale_limit = 5;
    
    /**
     *  Tablica obsługiwanych formatów
     **/
    private $img_types = array(
        'image/jpeg' => 'jpeg',
        'image/jpg' => 'jpg',
        'image/png' => 'png',
        'image/gif' => 'gif',
        'application/x-shockwave-flash' => 'swf'
    );
    
    /**
     *  Tablica powiązanych kontentów
     **/
    private $related_content = array(
        //Typy specjalne
        'offer' => array('main_offers3', 'offer_id'),//Ogłoszenia
        'product' => array('main_products', 'product_id'),//Produkty w sklepie
        'banner' => array('main_banners', 'banner_id'),//Banery
        'edition' => array('main_editions', 'id_edition'),//Edycje terapii
        'avatar' => array('main_image_avatar', 'avatar_id'),//Słownik grafik (avatary)
        'calendar' => array('stuff_calendar', 'id'),
        //Typy kontentu posiadajace grafiki
        'news' => array('main_news', 'id_news'),//Nowości
        'publ' => array('main_publications', 'id_publ'),//Publikacje
        'sect' => array('main_sections', 'section_id'),//Sekcje
        //'event' => array('main_events', 'id_event'),//Wydarzenia
        'content' => array('main_content', 'content_id', 'lang_id'),//Nowe kontenty
    );
    
    /**
     *  Tablica błędów uploadu
     **/
    public $errors = null;
    
    /**
     *  konstruktor klasy dimage
     *  @param boolean $dev - wymusza tryb deweloperski
     **/
    public function __construct($dev = false)
    {
        global $db;
        if($db){
            $this->db = $db;
            if($this->db->isDevel() || $dev)
                $this->dev_mode = true;
            $this->img_root = '/nfs/files/images';
            if($this->dev_mode){
                $this->img_root = '/nfs/files/images_dev';
                if(!$this->db->isDevel())
                    $this->db = DB_connect('burak');
            }
        }else{
            if($dev)
                $this->db = PDOS::getInstance('burak');
            else
                $this->db = PDOS::getInstance();
            $this->img_root = '/nfs/files/images';
            if($this->db->isDevel())
            {
                $this->dev_mode = true;
                $this->img_root = '/nfs/files/images_dev';
            }
        }
        
        /*global $db;
        $this->db = $db;
        if($this->db->isDevel() || $dev)
            $this->dev_mode = true;
        $this->img_root = '/nfs/files/images';
        if($this->dev_mode){
            $this->img_root = '/nfs/files/images_dev';
            if(!$this->db->isDevel())
                $this->db = DB_connect('burak');
        }*/
    }
    
    /**
     *  Losowanie nowego identyfikatora dla grafiki innego niż dostępne w bazie danych
     **/
    private function draw_new_id()
    {
        //Przyjęte stałe
        $id_lenght = 12;
        $id_chars = array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F',
                          'G', 'H', 'I', 'K', 'L', 'N', 'O', 'P', 'R', 'S', 'T', 'U', 'W', 'X', 'Y', 'Z');
        $chars_amount = 32;//count($id_chars);
        
        $new_id = '';
        
        for($i=0;$i<$id_lenght;$i++){
            $rand_key = rand(0, $chars_amount-1);
            $new_id .= $id_chars[$rand_key];
        }
        
        $sql = "SELECT COUNT(1) FROM main_dimages WHERE image_name='{$new_id}'";
        if($this->db->getOne($sql))
            $new_id = $this->draw_new_id();
        
        return $new_id;
    }
    
    /**
     *  Pobiera informacje o grafice
     **/
    public function getImageData($image_id)
    {
        $sql = "SELECT * FROM main_dimages WHERE image_name='{$image_id}'";
        $data = $this->db->getRow($sql);
        return $data;
    }
    public function get_image_data($image_id)
    {
        return $this->getImageData($image_id);
    }
    
    /**
     *  Zwraca wewnętrzną ścieżkę do grafiki ścieżkę 
     *  @param string $image_id - identyfikator pliku
     **/
    public function outputImage($image_id, $new_width = null, $new_height = null)
    {
        $data = $this->get_image_data($image_id);
        if(!isset($data['image_name']))
            return false;
        $folder = $data['image_name']{0}.$data['image_name']{1};
        $path = $this->img_root.'/'.$folder.'/'.$data['image_name'].'.'.$data['ext'];
        if(!file_exists($path))
            return false;
        
        if(($new_width && is_numeric($new_width) && $data['width']!=$new_width) || ($new_height && is_numeric($new_height) && $data['height']!=$new_height)){
            $ratio = $data['width']/$data['height'];
            
            //Gdy nowa szerokość lub nowa wysokość nieustalone, obliczamy na podstawie ratio
            if(!$new_width || !is_numeric($new_width))
                $new_width = round($new_height*$ratio);
            if(!$new_height || !is_numeric($new_height))
                $new_height = round($new_width/$ratio);
            
            $new_path = $this->img_root.'/'.$folder.'/'.$data['image_name'].'_'.$new_width.'_'.$new_height.'.'.$data['ext'];
            $new_ratio = $new_width/$new_height;
            $scale_w = $data['width']/$new_width;
            $scale_h = $data['height']/$new_height;
            if($ratio>$new_ratio){
                $tmp_width = ($data['height']*($new_ratio));
                $tmp_height = $data['height'];
                $width_crop = round(($data['width'] - $tmp_width)/2);
                $height_crop = 0;
            }else{
                $tmp_width = $data['width'];
                $tmp_height = ($data['width']/($new_ratio));
                $width_crop = 0;
                $height_crop = round(($data['height'] - $tmp_height)/2);
            }
            
            //
            $data = $this->get_image_data($image_id);
            $folder = $image_id{0}.$image_id{1};
            $path = $this->img_root.'/'.$folder.'/'.$image_id.'.'.$data['ext'];
            $path_all = $this->img_root.'/'.$folder.'/'.$image_id.'_*.'.$data['ext'];
            $sum = count(glob($path_all));
            
            if($sum>$this->scale_limit /*|| 1==1*/){//Wywalic 1==1 gdy wszystkie przeskalowane grafiki beda juz dobre
                foreach(glob($path_all) as $dpath){
                    if($dpath!=$path)
                        unlink($dpath);
                }
            }
            
            if(!file_exists($new_path)){
                $image_p = imagecreatetruecolor($new_width, $new_height);
                $image_tmp = imagecreatetruecolor($tmp_width, $tmp_height);
                switch($data['data_type']){
                    case 'image/jpg':
                    case 'image/jpeg':
                        $image = imagecreatefromjpeg($path);
                        imagecopyresampled($image_tmp, $image, 0, 0, $width_crop, $height_crop, $tmp_width, $tmp_height, $tmp_width, $tmp_height);
                        imagecopyresampled($image_p, $image_tmp, 0, 0, 0, 0, $new_width, $new_height, $tmp_width, $tmp_height);
                        imagejpeg($image_p, $new_path, 80);
                        break;
                    case 'image/png':
                        $image = imagecreatefrompng($path);
                        imagecopyresampled($image_tmp, $image, 0, 0, $width_crop, $height_crop, $tmp_width, $tmp_height, $tmp_width, $tmp_height);
                        imagecopyresampled($image_p, $image_tmp, 0, 0, 0, 0, $new_width, $new_height, $tmp_width, $tmp_height);
                        imagepng($image_p, $new_path, 5);
                        break;
                    case 'image/gif':
                        $image = imagecreatefromgif($path);
                        imagecopyresampled($image_tmp, $image, 0, 0, $width_crop, $height_crop, $tmp_width, $tmp_height, $tmp_width, $tmp_height);
                        imagecopyresampled($image_p, $image_tmp, 0, 0, 0, 0, $new_width, $new_height, $tmp_width, $tmp_height);
                        imagegif($image_p, $new_path);
                        break;
                    default:
                        break;
                }
                imagedestroy($image);
                imagedestroy($image_p);
                imagedestroy($image_tmp);
            }
            $path = $new_path;
            $data['width'] = $new_width;
            $data['height'] = $new_height;
        }
        $data['path'] = $path;
        $data['size'] = filesize($path);
        return $data;
    }
    public function output_image($image_id, $new_width = null, $new_height = null)
    {
        return $this->outputImage($image_id, $new_width, $new_height);
    }
    
    /**
     *  Dodawanie nowej grafiki, należy wykonać tuż po uploadzie grafiki
     *  @param array/string $file - tablica uploadowenego pliku np. $_FILES['plik'] lub string do pliku na dysku
     *  @param string $max_size - limit rozmiaru grafiki w KB
     *  @return string - identyfikator grafiki
     *  @tutorial - jako parametr $file należy podać element tablicy $_FILES tylko wtedy gdy ona istnieje!!!
     **/
    public function uploadImage($file, $max_size = 1000)
    {
        $upload_mode = false;
        if(is_array($file))
            $upload_mode = true;
        $idata = getimagesize($upload_mode?$file['tmp_name']:$file);
        $checksum = md5(file_get_contents($upload_mode?$file['tmp_name']:$file));
        $width = $idata[0];
        $height = $idata[1];
        $type = $idata['mime'];
        $ext = isset($this->img_types[$type])?$this->img_types[$type]:null;
        if($ext==null){
            $this->errors['ERROR_UNSERVED_FORMAT'] = 'Niepoprawne format pliku.';
            return false;
        }
        if($file['size']>($max_size*1000)){
            $this->errors['ERROR_FILE_TOO_BIG'] = 'Plik przekracza maksymalny rozmiar.';
            return false;
        }
        
        $sql = "SELECT image_name FROM main_dimages WHERE checksum='$checksum'";
        $new_id = $this->db->getOne($sql);
        if(strlen($new_id)==12)
            return $new_id;
        $new_id = $this->draw_new_id();
        
        $folder = $new_id{0}.$new_id{1};
        if(!is_dir($this->img_root.'/'.$folder))
            mkdir($this->img_root.'/'.$folder);
        $path = $this->img_root.'/'.$folder.'/'.$new_id.'.'.$ext;
        if($upload_mode){
            if(!is_uploaded_file($file['tmp_name']) || !move_uploaded_file($file['tmp_name'], $path)){
                $this->errors['ERROR_CANNOT_UPLOAD'] = 'Błąd podczas wgrywania.';
                return false;
            }
        }else{
            if(file_put_contents($path, file_get_contents($file))===false){
                $this->errors['ERROR_CANNOT_UPLOAD'] = 'Drugi błąd podczas wgrywania.';
                return false;
            }
        }
        
        $sql = "INSERT INTO main_dimages (image_name, ext, width, height, data_type, checksum) ".
               "VALUES ('{$new_id}', '{$ext}', '{$width}', '{$height}', '{$type}', '{$checksum}')";
        $this->db->exec($sql);
        
        return $new_id;
    }
    public function upload_image($file, $max_size = 1000)
    {
        return $this->uploadImage($file, $max_size);
    }
    
    /**
     *  Usuwanie grafiki
     **/
    public function deleteImage($image_id)
    {
        $data = $this->getImageData($image_id);
        
        $folder = $image_id{0}.$image_id{1};
        $path = $this->img_root.'/'.$folder.'/'.$image_id.'*.'.$data['ext'];
        foreach(glob($path) as $ppp){
            unlink($ppp);
        }
        $sql = "DELETE FROM main_dimages WHERE image_name='{$image_id}'";
        $this->db->exec($sql);
        $sql = "DELETE FROM rel_dimages WHERE image_name='{$image_id}'";
        $this->db->exec($sql);
        return true;
    }
    public function delete_image($image_id)
    {
        return $this->deleteImage($image_id);
    }
    
    /**
     *  Sprawdza czy istnieje dany content_type i content_id w odpowiedniej tabeli
     *  @param string $content_type - rodzaj kontentu
     *  @param integer $content_id - id kontentu
     **/
    private function check_content($content_type, $content_id, $content_id2 = null)
    {
        if(!in_array($content_type, array_keys($this->related_content)))
            return false;
        $sql = "SELECT COUNT(1) FROM {$this->related_content[$content_type][0]} ".
               "WHERE {$this->related_content[$content_type][1]}='{$content_id}'";
        if($content_id2){
            if(!isset($this->related_content[$content_type][2]))
                return false;
            $sql .= " AND {$this->related_content[$content_type][2]}='{$content_id2}'";
        }
        if(!$this->db->getOne($sql))
            return false;
        return true;
    }
    
    /**
     *  Tworzenie relacji
     *  @param string $content_type - rodzaj kontentu
     *  @param integer $content_id - id kontentu
     **/
    public function relateImage($image_id, $content_type, $content_id, $content_id2 = null)
    {
        if(!$this->check_content($content_type, $content_id))
            return false;
        
        $sql = "SELECT COUNT(1) FROM rel_dimages WHERE image_name='{$image_id}' AND content_type='{$content_type}' AND content_id='{$content_id}'";
        if($content_id2) $sql .= " AND content_id2='{$content_id2}'";
        if($this->db->getOne($sql))
            return true;
            
        $ak = $av = null;
        if($content_id2){
            $ak = ", content_id2";
            $av = ", '{$content_id2}'";
        }
        $sql = "INSERT INTO rel_dimages (image_name, content_type, content_id{$ak}) ".
               "VALUES ('{$image_id}', '{$content_type}', '{$content_id}'{$av}) ";
        $res = $this->db->exec($sql);
        return $res?true:false;
    }
    public function relate_image($image_id, $content_type, $content_id, $content_id2 = null)
    {
        return $this->relateImage($image_id, $content_type, $content_id, $content_id2);
    }
    
    /**
     *  Usuwanie relacji
     *  @param string $content_type - rodzaj kontentu
     *  @param integer $content_id - id kontentu
     **/
    public function unrelateImage($image_id, $content_type, $content_id, $content_id2 = null)
    {
        $sql = "SELECT COUNT(*) FROM rel_dimages WHERE image_name='{$image_id}' AND ".
               "content_type='{$content_type}' AND content_id='{$content_id}'";
        if($content_id2) $sql .= " AND content_id2='{$content_id2}'";
        if(!$this->db->getOne($sql))
            return false;
        
        $sql = "DELETE FROM rel_dimages WHERE image_name='{$image_id}' AND ".
               "content_type='{$content_type}' AND content_id='{$content_id}'";
        if($content_id2) $sql .= " AND content_id2='{$content_id2}'";
        $this->db->exec($sql);
        return true;
    }
    public function unrelate_image($image_id, $content_type, $content_id, $content_id2 = null)
    {
        return $this->unrelateImage($image_id, $content_type, $content_id, $content_id2);
    }
    
    /**
     *
     **/
    public function unrelateImages($content_type, $content_id, $content_id2 = null)
    {
        $data = $this->getRelatedImages($content_type, $content_id, $content_id2);
        $i = 0;
        foreach($data as $dd){
            if($this->unrelateImage($dd['image_name'], $content_type, $content_id, $content_id2))
                $i++;
        }
        return $i;
    }
    
    /**
     *  Pobiera wszystkie grafiki powiązane z danym elementem
     *  @param string $content_type - rodzaj kontentu
     *  @param integer $content_id - id kontentu
     **/
    public function getRelatedImages($content_type, $content_id, $content_id2 = null)
    {
        if(!$this->check_content($content_type, $content_id, $content_id2))
            return false;
        
        $sql = "SELECT md.* FROM main_dimages md INNER JOIN rel_dimages rd ON ".
               "md.image_name=rd.image_name WHERE content_type='{$content_type}' ".
               "AND content_id='{$content_id}'";
        if($content_id2) $sql .= " AND content_id2='{$content_id2}'";
        $data = $this->db->getAll($sql);
        return $data;
    }
    public function get_related_images($content_type, $content_id, $content_id2 = null)
    {
        return $this->getRelatedImages($content_type, $content_id, $content_id2);
    }
    
    /**
     *  Usuwa grafiki nie powiązane
     *  CRON
     **/
    public function clearUnrelatedImages()
    {
        // --- dopisac kiedy bedzie potrzebne ---
    }
}

?>