<?php

/******************************************************************************
 *                                                                            *
 *      Klasa obsługująca dynamiczne okresowe hasła                           *
 *                                                                            *
 ******************************************************************************/

require_once('Mail/htmlMimeMail5.php');
require_once('Functions/sms.php');

class dpass
{
    /** Uchwyt do bazy danych **/
    private $db;
    
    /** Parametry **/
    private $id = null;
    private $name = null;
    private $value = null;
    
    private $start_date = null;
    private $stop_date = null;
    
    /**
     *  Konstruktor,
     *  @param string $name - nazwa hasła
     *  @param string $email - email lub telefon komurkowy (wysyła smsa) do powiadamiania o nowych hasłach
     *  @param string $expire - po jakim czasie hasło ma wygasać
     **/
    public function __construct($name, $email, $expire = '1 month', $db = null)
    {
        if($db == null)
            global $db;
        $this->db = $db;
        
        $this->name = $name;
        $sql = "SELECT id FROM main_dpass WHERE name='$name' AND NOW()>start_date AND NOW()<stop_date";
        $id = $this->db->getOne($sql);
        if(is_numeric($id) && $id>0)
            $this->id = $id;
        
        if($this->id==null){
            $this->value = $this->generatePass();
            $tarr = explode(' ', $expire);
            $num = $tarr[0];
            $val = $tarr[1];
            if(!is_numeric($num) || $num<1 || $num>50 || !in_array($val, array('day', 'week', 'month', 'year'))){
                trigger_error('Nieprawidłowy czas ekspiracji hasła', E_USER_WARNING);
                return false;
            }
            $start_date = date('Y-m-d 00:00:00');
            switch($val){
                case 'day':
                    $stop_date = date('Y-m-d 23:59:59', mktime(0,0,0, date('m'), date('d')+$num, date('Y')));
                    break;
                case 'week':
                    $stop_date = date('Y-m-d 23:59:59', mktime(0,0,0, date('m'), date('d')+$num*7, date('Y')));
                    break;
                case 'month':
                    $stop_date = date('Y-m-d 23:59:59', mktime(0,0,0, date('m')+$num, date('d'), date('Y')));
                    break;
                case 'year':
                    $stop_date = date('Y-m-d 23:59:59', mktime(0,0,0, date('m'), date('d'), date('Y')+$num));
                    break;
            }
            $sql = "INSERT INTO main_dpass (name, value, start_date, stop_date, email) VALUES ".
                   "('{$this->name}', '{$this->value}', '{$start_date}', '{$stop_date}', '{$email}')";
            $this->db->exec($sql);
            
            if(strpos($email, '@')!==false){
                $mail = new htmlMimeMail5Ext();
                //$mail->setSMTPParams('217.153.222.73');
                $mail->setHTMLCharset('UTF-8');
                $mail->setTextCharset('UTF-8');
                $mail->setHeadCharset('UTF-8');
                $mail->setFrom('info@esculap.pl');
                $mail->setSubject('Nowe haslo');
                $mail->setPriority('high');
                $mail->setText("Hasło: {$this->value} \nWażne od $start_date do $stop_date.");
                $result = $mail->send(explode(',', $email), 'sendmail');
            }else{
                send_sms($email, "Hasło: {$this->value} \nWażne od $start_date \ndo $stop_date.");
            }
        }else{
            //$sql = "SELECT value FROM main_dpass WHERE id='{$this->id}'";
            //$this->value = $this->db->getOne($sql);
            $arr = $this->db->getRow("SELECT value, start_date, stop_date FROM main_dpass WHERE id='$id'");
            $start_date = $arr['start_date'];
            $stop_date = $arr['stop_date'];
            $this->value = $arr['value'];
        }
        $this->start_date = $start_date;
        $this->stop_date = $stop_date;
    }
    
    /**
     *
     **/
    private function generatePass()
    {
        $id_lenght = 8;
        $id_chars = array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F',
                          'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'Q', 'P', 'R', 'S', 'T', 'U', 'W',
                          'V', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l',
                          'm', 'n', 'o', 'q', 'p', 'r', 's', 't', 'u', 'w', 'v', 'x', 'y', 'z', '_', '@');
        $chars_amount = 64;//count($id_chars);
        
        $pass = '';
        
        for($i=0;$i<$id_lenght;$i++){
            $rand_key = rand(0, $chars_amount-1);
            $pass .= $id_chars[$rand_key];
        }
        
        return $pass;
    }
    
    /**
     *
     **/
    public function getPass()
    {
        return $this->value;
    }
    
    /**
     *
     **/
    public function getStart()
    {
        return $this->start_date;
    }
    
    /**
     *
     **/
    public function getStop()
    {
        return $this->stop_date;
    }
}

?>