<?php

/******************************************************************************
 *                                                                            *
 *      Klasa obsługująca formularze                                          *
 *      @category data                                                        *
 *      @author Sławek                                                        *
 *      UWAGA: używać w klasach poytomnych Controller.class.php               *
 *                                                                            *
 *                                                                            *
 ******************************************************************************/

require_once('Functions/validation.php');

class form
{
    /** Uchwyt do bazy danych poprzez PDO **/
    protected $db;
    
    /** Identyfikator formularza **/
    protected $form_id;
    
    /** Identyfikator użytkownika **/
    protected $user_login;
    
    /** Wersja formularza **/
    protected $form_ver;
    
    /** URI żądania **/
    protected $request_uri;
    
    /** IP żądania **/
    protected $request_ip;
    
    /** URI referera, skąd żądanie zostało wysłane **/
    protected $form_uri;
    
    /** Identyfikator odpowiedzi na formularz **/
    protected $submit_id;
    
    /** Krok odpowiedzi na formularz **/
    protected $step;
    
    /** Kroki formularza, pomaga używać numerycznego indentyfikatora (1, 2, 3, ...) **/
    protected $steps = null;
    
    /** Struktura formularza, kroki i pola **/
    protected $fields = null;
    
    /** Odpowiedz z formularza zagregowane **/
    protected $values = null;
    
    /** Odpowiedzi z formularza dopiero wysłane **/
    protected $new_values = null;
    
    /** Tablica błędów użytkownika **/
    public $errors = null;
    
    /** Blokada zmiany struktury formularza **/
    protected $lock = false;
    
    /** Blokada formularza **/
    protected $disabled = false;
    
    /** Zakończony formularz **/
    protected $close = false;
    
    /** Formularz jednokrotnego użycia **/
    protected $one_use = false;
    
    /** Tylko dla zalogowanych **/
    protected $only_auth = true;
    
    /** Czy formularz był już używany przez użytkownika **/
    protected $used = false;
    
    /** Czy formularz został wypełniony kompletnie **/
    protected $finish = false;
    
    /** Czy można już zapisywać nowe wyniki **/
    protected $save_values = false;
    
    /** Czy formularz jest aktywny **/
    protected $active = false;
    
    /** Sposób wykorzystania formularza **/
    protected $form_use = 'standard';//standardowy
    
    /**
     *  Konstruktor,
     *  @param string $form_id - identyfikator formularza
     *  @param string $user_login - identyfikator użytkownika
     *  @param boolean $one_use - czy to formularz który można wypełnić tylko raz
     *  @param boolean $disabled - czy formularz ma być zablokowany
     **/
    public function __construct($form_id, $user_login = null, $one_use = true, $disabled = false, $only_auth = true)
    {
        global $db;
        $this->db = $db;
        $this->form_id = $form_id;
        $this->only_auth = $only_auth;
        if($user_login===false)
            $this->user_login = null;
        else
            $this->user_login = $user_login?$user_login:(defined('CONTR_USER')?CONTR_USER:null);
        $this->one_use = $one_use;
        $this->disabled = $disabled;
        if($this->db->getOne("SELECT COUNT(1) FROM main_form_fields WHERE form_id='{$this->form_id}'"))
            $this->form_ver = $this->db->getOne("SELECT MAX(form_ver) FROM main_form_fields WHERE form_id='{$this->form_id}'");
        else
            $this->form_ver = 0;
        if(isset($_SERVER['HTTP_HOST']) && isset($_SERVER['REQUEST_URI']))
            $this->request_uri = 'http://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];
        else
            $this->request_uri = '';
        $this->request_ip = isset($_SERVER['REMOTE_ADDR'])?$_SERVER['REMOTE_ADDR']:'0.0.0.0';
        $this->form_uri = isset($_SERVER['HTTP_REFERER'])?$_SERVER['HTTP_REFERER']:'';

        //$this->form_ver = 1;
        //$this->form_id = 'quiz_szczepienia5';

        $sql = "SELECT COUNT(1) FROM main_form_fields WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}' AND is_active=TRUE";
        if($this->db->getOne($sql))
            $this->active = true;
        elseif($this->form_ver>0)
            $this->disabled = true;
        
        $submit_id = isset($_SESSION[$this->form_id]['submit_id'])?$_SESSION[$this->form_id]['submit_id']:0;
        $sql = "SELECT submit_id FROM main_form_submits WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}' AND finish=false AND user_login='{$this->user_login}'";
        $this->submit_id = $this->db->getOne($sql);
        if(!$this->submit_id && $submit_id!=0)
            $this->submit_id = $this->db->getOne("SELECT submit_id FROM main_form_submits WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}' AND finish=false AND submit_id='{$submit_id}'");
        if($this->submit_id!=null)
            $this->step = $this->db->getOne("SELECT step FROM main_form_submits WHERE submit_id='{$this->submit_id}'");
        else
            $this->step = 0;
        
        $sql = "SELECT COUNT(1) FROM main_form_submits WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}' AND (user_login='{$this->user_login}' OR submit_id='{$submit_id}') AND finish=true";
        if($this->db->getOne($sql)){
            $this->used = true;
            if($this->user_login!=null && $submit_id!=0){
                $sql = "UPDATE main_form_submits SET user_login='{$this->user_login}' WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}' AND (user_login='{$this->user_login}' OR submit_id='{$submit_id}') AND finish=true";
                $this->db->exec($sql);
                unset($_SESSION[$this->form_id]['submit_id']);
            }
        }
        
        $sql = "SELECT COUNT(1) FROM main_form_fields WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}' AND ((end_date<NOW() AND end_date IS NOT NULL) OR is_close=TRUE)";
        if($this->db->getOne($sql)){
            $this->disabled = true;
            $this->close = true;
        }
    }
    
    /**
     *  Dodaje krok formularza, każdy formularz musi posiadać przynajmniej jeden krok
     *  @param string $name - nazwa kroku formularza
     **/
    public function addStep($name)
    {
        if($this->lock)
            return false;
        if(isset($this->fields[$name])){
            $this->errors['FORM_STEP_DOUBLED'] = true;
            return false;
        }
        $this->fields[$name] = array();
        $steps_amount = count($this->steps)+1;
        $this->steps[$steps_amount] = $name;
        return true;
    }
    
    /**
     *  Dodaje pole do kroku formularza
     *  @param string $step - nazwa kroku
     *  @param string $name - nazwa pola
     *  @param string $type - rodzaj pola ('checkbox', 'radio', 'text', 'select', 'textarea')
     *  @param string $label - etykieta pola
     *  @param string $validation - rodzaj validacji ('not_empty', 'in_options', 'number', 'email')
     **/
    public function addField($step, $name, $type = null, $label = null, $validation = null)
    {
        if($this->lock)
            return false;
        if(!isset($this->fields[$step])){
            $this->errors['FORM_STEP_NOTEXISTS'] = true;
            return false;
        }
        if(isset($this->fields[$step][$name])){
            $this->errors['FORM_FIELD_DOUBLED'] = true;
            return false;
        }
        
        $this->fields[$step][$name]['name'] = $name;
        if($type)
            $this->fields[$step][$name]['type'] = $type;
        if($label)
            $this->fields[$step][$name]['label'] = $label;
        if($validation)
            $this->fields[$step][$name]['validation'] = $validation;
        
        return true;
    }
    
    /**
     *  Dodanie możliwej wartości dla opcji danego pola
     *  @param string $step - nazwa kroku
     *  @param string $name - nazwa pola
     *  @param string $option_value - wartość wyświetlana przy opcji
     *  @param string $option_key - wartość wysłana do skryptu jako odpowiedz
     **/
    public function addOption($step, $name, $option_value, $option_key = null)
    {
        if($option_key)
            $this->fields[$step][$name]['options'][$option_key] = $option_value;
        else
            $this->fields[$step][$name]['options'][$option_value] = $option_value;
    }
    
    /**
     *  Ustawienie odpowiedzi z formularza, wymusz zapis
     *  @param array $values - tablica odpowiedzi z formularza ($_POST lub inna tablica)
     *                      powinna zawierać wszystkie pola z danego kroku
     **/
    public function setValues($values)
    {
        $this->new_values = $values;
        $this->save_values = true;
    }
    
    /**
     *  Przetwarzanie formularza
     **/
    public function process()
    {
        $this->lock = true;
        if($this->disabled)
            return false;
        //echo 'dsadsadsa'; die();
        $this->saveFields();
        $this->loadFields();
        $this->loadValues();
        $this->saveValues();
        return true;
    }
    
    /**
     *  Odczytuje mapę pól formularza
     **/
    protected function loadFields()
    {
        if($this->fields!=null)
            return null;//Gdy podano strukturę pomiń odczytywanie jej z bazy
        
        $sql = "SELECT fields FROM main_form_fields WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
        $data = unserialize($this->db->getOne($sql));
        $this->steps = $data['steps'];
        $this->fields = $data['fields'];
        return true;
    }
    
    /**
     *  Zapisuje mapę pół formularza
     **/
    protected function saveFields()
    {
        if($this->fields==null)
            return null;//Gdy nie podano struktury pomiń próbę zapisu
        $data = array('steps' => $this->steps, 'fields' => $this->fields);
        $serial_data = serialize($data);
        $checksum = md5($serial_data);
        if($this->db->getOne("SELECT COUNT(1) FROM main_form_fields WHERE form_id='{$this->form_id}' AND form_ver<'{$this->form_ver}' AND checksum='{$checksum}'")){
            trigger_error('FORM_CLASS: juz uzywales takiej struktury i ja zmieniles, sprawdz wpisy w bazie oraz jesli uzywasz tego '.
                          'formularza na kilku stronach czy nie ma konfliktu struktury (musi być zawsze identyczna!)', E_USER_ERROR);
            return false;//Trzeba zwrócić false ponieważ mogło by dojść do ciągłego dodawania wpisów w przypadku użycia tego samego id
            //w kilku miejscach i z inną strukturą
        }
        $sql = "SELECT COUNT(1) FROM main_form_fields WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}' AND checksum='{$checksum}'";
        if($this->db->getOne($sql)){
            return null;//Struktura taka zapisana jest już w bazie :-)
        }
        
        $sql = "SELECT COUNT(1) FROM main_form_submits WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
        if($this->db->getOne($sql)){
            $sql = "UPDATE main_form_fields SET fields='{$serial_data}', checksum='$checksum' WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
            $result = $this->db->exec($sql);
        }else{
            $sql = "UPDATE main_form_fields SET end_date=NOW(), is_active=FALSE, is_close=TRUE WHERE form_id='{$this->form_id}' AND is_close=FALSE";
            $this->db->exec($sql);
            
            $this->form_ver += 1;
            
            $sql = "INSERT INTO main_form_fields ".
                   "(form_id, form_ver, fields, checksum, form_use) ".
                   "VALUES ".
                   "('{$this->form_id}', '{$this->form_ver}', '{$serial_data}', '{$checksum}', '{$this->form_use}')";
            $result = $this->db->exec($sql);
        }
        return $result?true:false;
    }
    
    /**
     *  Odczytuje wyniki wcześniejszych kroków
     **/
    protected function loadValues()
    {
        if($this->submit_id==null)
            return false;
        
        $sql = "SELECT values FROM main_form_submits WHERE submit_id='{$this->submit_id}'";
        $this->values = unserialize($this->db->getOne($sql));
        return true;
    }
    
    /**
     *  Zapisuje wartości odpowiedzi
     **/
    protected function saveValues()
    {
        //echo 'dsadsadsa'; die();

        if(!$this->save_values)
            return false;
        
        if($this->only_auth && !$this->user_login){
            $this->errors['FORM_MUST_BE_LOGGED'] = true;
            return false;
        }
        
        if($this->one_use){
            $sql = "SELECT COUNT(1) FROM main_form_submits WHERE form_id='{$this->form_id}' ".
                   "AND user_login='{$this->user_login}' AND finish=TRUE";
            if($this->db->getOne($sql))
                return false;
        }
        
        $steps = count($this->fields);
        $this->step += 1;
        $step_name = $this->steps[$this->step];
        $process_fields = $this->fields[$step_name];
        
        //Zapisujemy tylko te pola które występują w strukturze formularza
        //gdy ich nie ma w tablicy nowych wartości ustawiamy pusty wpis
        foreach($process_fields as $name => $opts){
            if(isset($this->new_values[$name]))
                $this->values[$step_name][$name] = $this->new_values[$name];
            else
                $this->values[$step_name][$name] = null;
        }
        
        if(!$this->validateValues()){
            $this->step -= 1;
            return false;
        }
        
        $finish = $this->step==$steps?1:0;
        $this->finish = $this->step==$steps?true:false;
        
        $save_values = serialize($this->values);
        
        if($this->submit_id==null){
            $sql = <<< SQL
INSERT INTO main_form_submits 
(form_id, form_ver, user_login, values, step, 
finish, request_date, request_uri, request_ip, form_uri) 
VALUES 
('{$this->form_id}', '{$this->form_ver}', '{$this->user_login}', '{$save_values}', '{$this->step}', 
{$finish}::boolean, NOW(), '{$this->request_uri}', '{$this->request_ip}', '{$this->form_uri}') 
RETURNING submit_id
SQL;
            $this->submit_id = $this->db->getOne($sql);
            if($this->user_login==null)
                $_SESSION[$this->form_id]['submit_id'] = $this->submit_id;
        }else{
            $sql = <<< SQL
UPDATE main_form_submits SET 
values = '{$save_values}', 
step = '{$this->step}', 
finish = {$finish}::boolean, 
request_date = NOW() 
WHERE submit_id = '{$this->submit_id}'
SQL;
            $this->db->exec($sql);
        }
        return true;
    }
    
    /**
     *  Walidacja wyników danego kroku
     **/
    protected function validateValues()
    {
        $step_name = $this->steps[$this->step];
        $process_fields = $this->fields[$step_name];
        
        foreach($process_fields as $name => $opts)
            if(isset($opts['validation']))
                switch($opts['validation']){
                    case 'not_empty':
                        if(!isset($this->values[$step_name][$name]) || empty($this->values[$step_name][$name])){
                            $this->errors['FORM_RESULT_NOT_VALID'] = true;
                            return false;
                        }
                        break;
                    case 'in_options':
                        if(!isset($this->values[$step_name][$name]) || !isset($opts['options']) || !is_array($opts['options']) || !in_array($this->values[$step_name][$name], array_keys($opts['options']))){
                            $this->errors['FORM_RESULT_NOT_VALID'] = true;
                            return false;
                        }
                        break;
                    case 'number':
                        if(!isset($this->values[$step_name][$name]) || !is_numeric($this->values[$step_name][$name])){
                            $this->errors['FORM_RESULT_NOT_VALID'] = true;
                            return false;
                        }
                        break;
                    case 'email':
                        if(!isset($this->values[$step_name][$name]) || !valid_email($this->values[$step_name][$name])){
                            $this->errors['FORM_RESULT_NOT_VALID'] = true;
                            return false;
                        }
                        break;
                    default: break;
                }
        return true;
    }
    
    /**
     *  Zwraca kolejny krok na który powinien zareagować użytkownik 
     *  @param boolean $name - gdy true zwraca nazwę kroku gdy false zwraca numer kroku
     **/
    public function getNextStep($name = true)
    {
        if($this->step >= count($this->fields))
            return null;
        $step = $this->step + 1;
        if($name)
            return $this->steps[$step];
        else
            return $step;
    }
    
    /**
     *  Czy użytkownik już wypełnił ten formularz
     *  @deprecated
     **/
    public function isSubmited()
    {
        return $this->used;
    }
    
    /**
     *  Czy formularz jest zablokowany
     *  @deprecated
     **/
    public function isDisabled()
    {
        return $this->disabled;
    }
    
    /**
     *  Czy formularz został właśnie wypełniony
     *  @deprecated
     **/
    public function isFinish()
    {
        return $this->finish;
    }
    
    /**
     *  Czy formularz jest aktywny
     *  @deprecated
     **/
    public function isActive()
    {
        return $this->active;
    }
    
    /**
     *  Zwraca aktualny status formularza dla danego użytkownika
     *  @return string - status ('disabled', 'active', 'used', 'completed', 'auth_req')
     **/
    public function getStatus()
    {
        if($this->finish)
            return 'completed';//właśnie został wypełnione
        if($this->used && $this->one_use)
            return 'used';//już był wypełniany i nie może być wypełniony poraz kolejny
        if($this->close)
            return 'close';//formularz został już zakończony
        if($this->disabled)
            return 'disabled';//zablokowany
        if($this->only_auth && $this->user_login==null)
            return 'auth_req';
        return 'active';//można wysyłać
    }
    
    /**
     *  Zwraca pola formularza
     *  @param mixed $step - zwracaj pola podanego kroku, gdy true aktualnego kroku
     *  @param string $field - zwracaj parametry podanego pola
     **/
    public function getFieldsDefinition($step = null, $field = null)
    {
        if(!$this->isActive())
            return null;
        
        if(is_string($step) && !isset($this->fields[$step])){
            $this->errors['FORM_STEP_NOTEXISTS'] = true;
            return false;
        }
        if(is_string($step) && is_string($field) && !isset($this->fields[$step][$field])){
            $this->errors['FORM_FIELD_NOTEXISTS'] = true;
            return false;
        }
        
        if($step==null){
            return $this->fields;
        }
        if($step===true){
            if(isset($this->fields[$this->getNextStep()]))
                return $this->fields[$this->getNextStep()];
            else
                return null;
        }
        if($field==null){
            return $this->fields[$step];
        }
        return $this->fields[$step][$field];
    }
    public function getFields($step = null, $field = null){
        $this->getFieldsDefinition($step, $field);
    }
    
    /**
     *  Zwraca pola aktualnego kroku
     **/
    public function getActualFields()
    {
        return $this->getValues($this->step);
    }
    
    /**
     *
     **/
    public function getFormData()
    {
        if(!$this->form_id)
            return null;
        
        $sql = "SELECT * FROM main_form_fields WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
        $data = $this->db->getRow($sql);
        $data['hash'] = md5('HARCONEN_10285'.$data['numeric_id']);
        return $data;
    }
    
    /**
     *
     **/
    protected function decodeHash($hash)
    {
        $sql = <<< ESQUEL
SELECT form_id 
FROM main_form_fields 
WHERE 
md5('HARCONEN_10285' || numeric_id) = '$hash'
ESQUEL;
        $form_id = $this->db->getOne($sql);
        if($form_id)
            return $form_id;
        return null;
    }
}

?>