<?php

/******************************************************************************
 *                                                                            *
 *      Klasa obsługująca formularze                                          *
 *      + rozszerzenie klasy form                                             *
 *                                                                            *
 ******************************************************************************/

require_once('form.class.php');

class formQuiz extends form
{
    /** Ankieta nie przypisana do holdera **/
    private $no_assign = true;
    
    /** Sposób wykorzystania formularza **/
    protected $form_use = 'quiz';//quiz
    
    /**
     *  Konstruktor,
     *  @param string $query_holder_id - identyfikator miejsca wyświetlania ankiety
     *  @param string $user_login - identyfikator użytkownika
     *  @param boolean $disabled - czy formularz ma być zablokowany
     *  @param string $form_id - wejście dla zarządzania ankietami
     **/
    public function __construct($holder_id = null, $user_login = null, $disabled = false, $form_id = null)
    {
        //Uchwyt do bazy danych należy przechwycić jak najszybciej
        global $db;
        $this->db = $db;
        if($holder_id){
            $this->saveHolder($holder_id);
            $form_id = $this->getHolderQuiz($holder_id);
        }
        parent::__construct($form_id, $user_login, true, $disabled);
    }
    
    /**
     *  Dodaje pole do kroku formularza
     *  @param string $step - nazwa kroku
     *  @param string $name - nazwa pola
     *  @ignore string $type - rodzaj pola ('checkbox', 'radio', 'text', 'select', 'textarea')
     *  @param string $label - etykieta pola
     *  @ignore string $validation - rodzaj validacji ('not_empty', 'in_options', 'number', 'email')
     **/
    public function addField($step, $name, $label = null)
    {
        $type = 'radio';
        $validation = 'in_options';
        return parent::addField($step, $name, $type, $label, $validation);
    }
    
    /**
     *  Dodanie możliwej wartości dla opcji danego pola
     *  @param string $step - nazwa kroku
     *  @param string $name - nazwa pola
     *  @param string $option_value - wartość wyświetlana przy opcji
     *  @param string $option_key - wartość wysłana do skryptu jako odpowiedz
     *  @param boolean $is_valid - oznacza czy dane pole jest poprawną odpowiedzią
     **/
    public function addOption($step, $name, $option_value, $option_key = null, $is_valid = false)
    {
        if($option_key){
            $this->fields[$step][$name]['options'][$option_key] = $option_value;
            if($is_valid)
                $this->fields[$step][$name]['valid'] = $option_key;
        }else{
            $this->fields[$step][$name]['options'][$option_value] = $option_value;
            if($is_valid)
                $this->fields[$step][$name]['valid'] = $option_value;
        }
    }
    
    /**
     *  Przetwarzanie formularza
     **/
    public function process()
    {
        $this->lock = true;
        if($this->disabled || $this->no_assign)
            return false;
        $this->loadFields();
        $this->loadValues();
        $this->saveValues();
        return true;
    }
    
    /**
     *
     **/
    protected function saveHolder($holder_id)
    {
        $sql = "SELECT COUNT(1) FROM main_form_holders WHERE holder_id='{$holder_id}' AND form_use='quiz'";
        if(!$this->db->getOne($sql)){
            $site_id = defined('SITE_ID')?SITE_ID:null;
            $sql = "INSERT INTO main_form_holders (holder_id, site_id, active, form_use) ".
                   "VALUES ('{$holder_id}', ".($site_id!=null?"'{$site_id}'":"NULL").", TRUE, 'quiz')";
            $this->db->exec($sql);
        }
    }
    
    /**
     *
     **/
    protected function getHolderQuiz($holder_id)
    {
        //Szuka pierwszego dodanego wpisu, aby wyświetlić kolejne trzeba deaktywować poprzednie
        $sql = "SELECT a.form_id FROM rel_form_holders a INNER JOIN main_form_fields b ".
               "ON a.form_id=b.form_id WHERE b.is_active=TRUE AND b.form_use = 'quiz' ".
               "AND a.holder_id='{$holder_id}' ORDER BY start_date ASC LIMIT 1";
        $form_id = $this->db->getOne($sql);
        if($form_id){
            $this->no_assign = false;
            return $form_id;
        }
        return null;
    }
    
    /**
     *  Zapisuje mapę pól formularza
     *  Używać tylko w CMSie
     **/
    public function saveFields()
    {
        $sfres = parent::saveFields();
        if($sfres){
            $counters = null;
            foreach($this->fields as $step => $value)
                foreach($value as $field => $value2)
                    if(is_array($value2['options']))
                        foreach($value2['options'] as $option => $value3)
                            $counters[$step][$field][$option] = 0;
            $counters = serialize($counters);
            $sql = "INSERT INTO main_form_agregations (form_id, form_ver, counters) ".
                   "VALUES ('{$this->form_id}', '{$this->form_ver}', '{$counters}')";
            $res = $this->db->exec($sql);
            $sfres = $res?true:false;
        }
        return $sfres;
    }
    
    /**
     *  
     **/
    public function saveValues()
    {
        if(!$this->form_id)
            return null;
        
        if($this->fields==null)
            $this->loadFields();
        
        $res = parent::saveValues();
        if($res){
            $sql = "SELECT counters FROM main_form_agregations WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
            $counters = unserialize($this->db->getOne($sql));
            foreach($this->values as $step => $value)
                foreach($value as $field => $value2)
                    $counters[$step][$field][$value2]++;
            $counters = serialize($counters);
            $sql = "UPDATE main_form_agregations SET counters='{$counters}' WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
            $res = $this->db->exec($sql)?true:false;
        }
        return $res;
    }
    
    /**
     *  
     **/
    public function getAgregationData()
    {
        /*if(!$this->isActive())
            return null;*/
        
        if(!$this->fields)
            $this->loadFields();
        
        $sql = "SELECT counters FROM main_form_agregations WHERE form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
        $counters = unserialize($this->db->getOne($sql));
        $data = null;
        foreach($counters as $step => $fields){
            foreach($fields as $field => $replys){
                $sum = 0;
                foreach($replys as $key => $counter)
                    $sum += $counter;
                $tmp = null;
                $tmp['label'] = $this->fields[$step][$field]['label'];
                $suma = 0;$i = 0;
                foreach($replys as $key => $counter){
                    $i++;
                    $tmp2 = null;
                    $tmp2['label'] = $this->fields[$step][$field]['options'][$key];
                    $tmp2['counter'] = $counter;
                    if($i==count($replys)){
                        $tmp2['percent'] = 100 - $suma;
                    }else{
                        $tmp2['percent'] = $sum>0?round(($counter/$sum)*100):0;
                        $suma += $tmp2['percent'];
                    }
                    $tmp2['valid'] = 0;
                    if(isset($this->fields[$step][$field]['valid']) && $key == $this->fields[$step][$field]['valid'])
                        $tmp2['valid'] = 1;
                    $tmp['replys'][] = $tmp2;
                }
                $data[] = $tmp;
            }
        }
        return $data;
    }
    
    /**
     *  Zwraca Pytania, prawidłowe odpowiedzi i odpowiedzi użytkownika
     *  @return array - tekstowee wartości powyższych
     **/
    public function getQuizResult()
    {
        if(!$this->fields)
            $this->loadFields();
        
        $data = null;
        $sql = "SELECT COUNT(1) FROM main_form_submits WHERE user_login='{$this->user_login}' AND form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
        if($this->db->getOne($sql)){
            $sql = "SELECT * FROM main_form_submits WHERE user_login='{$this->user_login}' AND form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
            $mfs = $this->db->getRow($sql);
            $answers = unserialize($mfs['values']);
            $answer_date = $mfs['request_date'];
            $temp = null;
            foreach($this->fields as $step => $fields){
                foreach($fields as $field => $opts){
                    $tmp = null;
                    $tmp['question'] = $opts['label'];
                    $tmp['your_answers'] = $opts['options'][$answers[$step][$field]];
                    $tmp['correct_answers'] = $opts['options'][$opts['valid']];
                    $temp[] = $tmp;
                }
            }
            $data['quest'] = $temp;
            $data['answer_date'] = $mfs['request_date'];
        }
        return $data;
    }
    
    /**
     *  Pobieranie listy ankiet archiwalnych dla holdera
     **/
    public function getList($page = 0, $rows = 10, $range = 5)
    {
        if(!$this->holder_id)
            return false;
        
        $start = $page * $rows;
        $sql = <<< ESQUEL
SELECT a.* 
FROM 
main_form_fields a 
INNER JOIN 
rel_form_holders b 
ON 
a.form_id = b.form_id 
WHERE 
(a.is_close = TRUE OR a.is_active = TRUE) 
AND 
a.form_use = 'quiz' 
AND 
b.holder_id = '{$this->holder_id}' 
ORDER BY a.end_date DESC 
LIMIT $rows OFFSET $start 
ESQUEL;
        $sql2 = <<< ESQUEL
SELECT COUNT(1) 
FROM 
main_form_fields a 
INNER JOIN 
rel_form_holders b 
ON 
a.form_id = b.form_id 
WHERE 
(a.is_close = TRUE OR a.is_active = TRUE) 
AND 
a.form_use = 'quiz' 
AND 
b.holder_id = '{$this->holder_id}' 
ESQUEL;
        $data = $this->db->getAll($sql);
        $amount = $this->db->getOne($sql2);
        
        for($i=0;$i<count($data);$i++){
            $fields = unserialize($data[$i]['fields']);
            $data[$i]['first_quest'] = $fields['fields']['query']['quest_1']['label'];
            $data[$i]['hash'] = md5('HARCONEN_10285'.$data[$i]['numeric_id']);
        }
        
        $pages = null;
        $pages_amount = ceil($amount / $rows);
        
        if($pages_amount<=1)
            return null;//Jedna lub brak stron
        
        //pierwsza strona
        $pages['first'] = array('view' => 'first', 'goto' => 0);
        
        //popszednia strona
        if($page>0)
            $pages['prev'] = array('view' => 'prev', 'goto' => $page-1);
        
        //pierwszy separator
        if($page>$range+1+1)
            $pages['sep1'] = array('view' => 'sep', 'goto' => floor(($page-$range)/2));
        
        //mięso
        $start = $page-$range;
        if($start<0) $start = 0;
        $stop = $page+$range;
        if($stop>$pages_amount) $stop = $pages_amount;
        for($i=$start; $i<$stop; $i++){
            $key = 'step'.($page+$i);
            $pages[$key] = array('view' => $i+1, 'goto' => $i);
        }
        
        //drugi separator
        if($pages_amount-$page>$range+1+1)
            $pages['sep2'] = array('view' => 'sep', 'goto' => ceil(($pages_amount-$page+$range)/2));
        
        //następna strona
        if($page<$pages_amount-1)
            $pages['next'] = array('view' => 'next', 'goto' => $page+1);
        
        //ostatnia strona
        $pages['last'] = array('view' => 'last', 'goto' => $pages_amount-1);
        
        $this->data = $data;
        $this->pages = $pages;
        return array('data' => $data, 'pages' => $pages);
    }
    
    /**
     *
     **/
    public function getValidationPercent()
    {
        if(!$this->fields)
            $this->loadFields();
        
        $sql = "SELECT COUNT(1) FROM main_form_submits WHERE user_login='{$this->user_login}' AND form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
        if($this->db->getOne($sql)){
            $sql = "SELECT values FROM main_form_submits WHERE user_login='{$this->user_login}' AND form_id='{$this->form_id}' AND form_ver='{$this->form_ver}'";
            $answers = unserialize($this->db->getOne($sql));
            
            $good = 0;
            $bad = 0;
            
            $data = null;
            foreach($this->fields as $step => $fields){
                foreach($fields as $field => $opts){
                    if($answers[$step][$field]==$opts['valid'])
                        $good++;
                    else
                        $bad++;
                }
            }
            if($good==0)
                return 0;
            return round(($good/($good+$bad))*100);
        }
        return null;
    }
    
    /**
     *  Zwraca aktualny status formularza dla danego użytkownika
     *  @return string - status ('disabled', 'active', 'used', 'completed', 'auth_req', 'no_assign')
     **/
    public function getStatus()
    {
        if($this->no_assign)
            return 'no_assign';
        
        return parent::getStatus();
    }
}

?>