<?php

/******************************************************************************
 *                                                                            *
 *      Klasa obsługująca wyświetlanie i zapisywanie wyników gier             *
 *      @category graphic                                                     *
 *      @author Sławek                                                        *
 *      @internal wymaga zainicjowanego uchwytu do bazy danych (zmienna $db)  *
 *      @internal wymaga uruchomionej sesji php                               *
 *                                                                            *
 ******************************************************************************/

class games
{
    /** Uchwyt bazy danych **/
    private $db;
    
    /** Dane gry **/
    //INTEGER
    private $game_id = null;
    private $instance_id = null;
    private $delay = 10000;
    //STRING
    private $username = null;
    private $userkey = null;
    private $gid = null;
    //BOOLEAN
    private $game_ready = false;
    //ARRAY
    private $errors = null;
    
    /**
     *  Konstruktor,
     *  @param mixed $game - nazwa lub identyfikator gry
     *  @param mixed $instance - nazwa lub instancja gry
     *  @param string $username - login użytkownika grającego
     **/
    public function __construct($game, $instance, $username = null)
    {
        global $db;
        $this->db = $db;
        
        $this->game_ready = true;
        $this->checkGame($game);
        $this->checkInstance($instance);
        $this->gid = "game_{$game}_{$instance}";
        
        if($username==null)
            $username = isset($_SESSION['auth']['username'])?$_SESSION['auth']['username']:null;
        if($username){
            $this->username = $username;
            $this->userkey = md5("01001001{$this->game_id}{$this->instance_id}{$this->username}10100100");
        }else{
            $this->game_ready = false;
            $this->errors['NOT_LOGGED'] = 'Musisz być zalogowany.';
        }
    }
    
    /**
     *  Sprawdzanie gry
     **/
    private function checkGame($game)
    {
        $sql = "SELECT game_id FROM gry.games WHERE game_id='{$game}' OR name LIKE '{$game}'";
        $data = $this->db->getAll($sql);
        
        if(count($data)!=1){
            $this->errors['GAME_NOT_FOUND'] = 'Nie odnaleziono takiej gry.';
            $this->game_ready = false;
            return false;
        }
        
        $this->game_id = $data[0]['game_id'];
        return true;
    }
    
    /**
     *  Sprawdzanie instancji gry
     **/
    private function checkInstance($instance)
    {
        $sql = "SELECT instance_id FROM gry.games_instances WHERE instance_id='{$instance}' OR name LIKE '{$instance}'";
        $data = $this->db->getAll($sql);
        
        if(count($data)!=1){
            $this->errors['GAME_INSTANCE_NOT_FOUND'] = 'Nie odnaleziono takiej gry.';
            $this->game_ready = false;
            return false;
        }
        
        $this->instance_id = $data[0]['instance_id'];
        return true;
    }
    
    /**
     *
     **/
    public function getTopResults($limit = 10)
    {
        $sql = <<< SQL
SELECT 
nick_name, MIN(result) as result 
FROM 
gry.games_results 
WHERE 
game_id='{$this->game_id}' 
AND 
instance_id='{$this->instance_id}' 
GROUP BY 
nick_name 
ORDER BY 
MIN(result) asc LIMIT 10 
SQL;
        $data = $this->db->getAll($sql);
       /* for($i=0;$i<count($data);$i++){
            $sql = <<< SQL
SELECT 
nick_name 
FROM 
gry.games_results 
WHERE 
user_login='{$data[$i]['user_login']}' 
AND 
result='{$data[$i]['result']}' 
AND 
game_id='{$this->game_id}' 
AND 
instance_id='{$this->instance_id}' 
LIMIT 1 
SQL;
            $data[$i]['nick_name'] = stripcslashes($this->db->getOne($sql));
        }*/
        return $data;
    }
    
    /**
     *  Zwraca kod HTML osadzania gry
     *  @param string $game_file - nazwa pliku z grą
     *  @param integer $width - szerokość gry
     *  @param integer $height - wysokość
     *  @param string $script_url - skrypt obsługujący wywołania z gry
     *  @param string $config_url - plik konfiguracyjny w postaci XML
     **/
    public function getHTML($game_file, $width, $height, $script_url, $config_url)
    {
        if(!$this->game_ready)
            return false;
        
        //Startowanie liczników gry
        $_SESSION[$this->gid]['php_time'] = time();
        $_SESSION[$this->gid]['js_time'] = 0;
        
        $flashWarsArr = array(
            'id_g' => $this->game_id,
            'id_i' => $this->instance_id,
            'key' => $this->userkey,
            'cfg' => $script_url,
            'xml_url' => $config_url,
        );
        
        $flashWarsStr = null;
        foreach($flashWarsArr as $k => $v)
            $flashWarsStr[] = "{$k}={$v}";
        $flashWarsStr = implode('&amp;', $flashWarsStr);
        
        $html = <<< HTML
<!--[if !IE]> -->
<object type="application/x-shockwave-flash" data="{$game_file}" height="{$height}" width="{$width}">	 
<!-- <![endif]-->
<!--[if IE]>
<object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"
  codebase="http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=6,0,0,0"
  width="{$width}" height="{$height}">
  <param name="movie" value="{$game_file}" />
<!--><!--dgx-->
  <param name="wmode" value="transparent">
  <param name="loop" value="true">
  <param name="menu" value="false">
  <param name="allowFullScreen" value="false">
  <param name="allowScriptAccess" value="always">
  <param name="quality" value="high">
  <param name="FlashVars" value="{$flashWarsStr}">
<!-- <![endif]-->
</object>

HTML;
        return $html;
    }
    
    /**
     *  Zwraca skrypty JS obsługujące grę
     **/
    public function getJS($refresh_url)
    {
        if(!$this->game_ready)
            return false;
        
        $js = <<< JS
function refresh(){
  var delay = {$this->delay};
  var req = new XMLHttpRequest();
  req.open('GET', '{$refresh_url}');
  req.send(null);
  setTimeout("refresh();", delay);
  //if(req.status == 200)
    //alert(req.responseText);
}
refresh();
JS;
        return $js;
    }
    
    /**
     *  Zwraca wychwycone błędy
     **/
    public function getErrors()
    {
        return $this->errors;
    }
    
    /**
     *  Podbija licznik czasu
     **/
    public function refreshTime()
    {
        $_SESSION[$this->gid]['js_time'] += $this->delay/1000;
    }
    
    /**
     *  Zapisywanie uzyskanego wyniku
     **/
    public function saveResult($nickname, $result, $userkey)
    {
        if(!$this->game_ready)
            return false;
        
        $nickname = addslashes(trim(strip_tags($nickname)));
        trigger_error($userKey);
       // $php_time = time() - $_SESSION[$this->gid]['php_time'];
       // $js_time = $_SESSION[$this->gid]['js_time'];
        
        /*if($userkey!=$this->userkey){
            $this->errors['INVALID_USER'] = 'Nieprawidłowa sesja użytkownika.';
            return false;
        }*/
        
        $sql = <<< ESQUEL
INSERT INTO 
gry.games_results 
(game_id, instance_id, user_login, nick_name, result, save_date) 
VALUES 
('{$this->game_id}', '{$this->instance_id}', '$this->username', '$nickname', '$result', NOW())
ESQUEL;
        $this->db->exec($sql);
        
        
        //Po zapisie zerowanie liczników czasu!
      //  $_SESSION[$this->gid]['php_time'] = time();
     //   $_SESSION[$this->gid]['js_time'] = 0;
        
        return true;
    }
}

?>