<?php

require_once 'abstractTable.class.php';
require_once 'Mail/html2text.class.php';

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Klasa obsługuje set emailingu
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_set extends abstractTable
{
    protected $table = 'emailing.em_sets';
    protected $related = array(
        'em_body' => array('type' => 'child'),
        'em_targetlist' => array('type' => 'child')
    );
    //protected $pkey = 'set_id';
    //protected $get = array('set_id', 'add_date', 'name', 'title', 'desc', 'user_id', 'from_name', 'from_email', 'client_id', 'type', 'archive', 'project');
    //protected $set = array('name', 'title', 'desc', 'user_id', 'from_name', 'from_email', 'client_id', 'type', 'archive', 'project');
    //protected $boolean = array('archive');

	/**
     * Walidowanie
     */
    public function validateItem()
    {
        $errors = $data = null;
        foreach($this->fields as $s)
            $data[$s] = isset($this->data[$s])?$this->data[$s]:null;

        if(strlen($data['name'])<5) $errors[] = 'Wpisz dłuższą nazwę emailingu!';
        if(strlen($data['title'])<5) $errors[] = 'Wpisz dłuższy tytuł emailingu!';

        if(strlen($data['from_name'])<5) $errors[] = 'Wpisz dłuższy tytuł nadawcy emailingu!';
        if(strlen($data['from_email'])<5) $errors[] = 'Wpisz dłuższy email nadawcy emailingu!';

        if($errors){
            $this->errors = $errors;
            return false;
        }
        return true;
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Klasa obsługuje treść i layout graficzny emailingu
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_body extends abstractTable
{
    protected $table = 'emailing.em_body';
    protected $related = array(
        'em_schedule' => array('type'=>'child'),
        'em_link' => array('type'=>'child'),
    	'em_file' => array('type'=>'child'),
        'em_set' => array('type'=>'parent', 'natural'=>'set_id')
    );
    protected $mailObj = null;

    protected $emset = null;
    protected $activeTargetlist = null;

	/**
     * Walidowanie
     */
    public function validateItem()
    {
        $errors = $data = null;
        foreach($this->fields as $s)
            $data[$s] = isset($this->data[$s])?$this->data[$s]:null;

        if(strlen($data['name'])<5) $errors[] = 'Wpisz dłuższą nazwę emailingu!';
        if(strlen($data['real_title'])<5) $errors[] = 'Wpisz dłuższy temat wiadomości!';
        if(intval($data['set_id'])<1) $errors[] = 'Brakuje identyfikatora emailingu!';

        if($errors){
            $this->errors = $errors;
            return false;
        }
        return true;
    }

    /**
     * Aktualizacja linkow
     * @todo poprawic na nowa klase abstractTable
     */
    public function updateLinks()
    {
        //Gdy obiekt nie istnieje lub został zmieniany walimy false
        if(!$this->exists) return false;

        $body_id = $this->getField('body_id');
        $layer_html = $this->getField('layer_html');
        $layer_plain = $this->getField('layer_plain');

        if($this->data['check_links']){
            $links = array();
            $link = null;
            //Sprawdzamy czy istniejące linki są w kodzie
            if($tmp = $this->getChilds('em_link')->getListObject())
            foreach($tmp as &$EMLINK){
                $links[$EMLINK->getField('link_id')] = $link = $EMLINK->getField('value');
                if(strpos($layer_html, $link)===false && strpos($layer_plain, $link)===false){
                    $EMLINK->setField('in_code', false);
                    $EMLINK->saveItem();
                }
            }

            $document = new \DOMDocument();
            $document->strictErrorChecking = false;
            $document->loadHTML($layer_html);

            $xpath = new DOMXPath($document);
            $col1 = $xpath->query("//*[@id='goniec-col1']")->item(0);
            $col2 = $xpath->query("//*[@id='goniec-col2']")->item(0);

            if ($col1) {
                $col1a = $col1->getElementsByTagName('a');
            }
            if ($col2) {
                $col2a = $col2->getElementsByTagName('a');
            }

            if (isset($col1a) && $col1a) {
                $i = 1;
                foreach ($col1a as $a) {
                    if ($i < 10) {
                        $number = '0' . $i;
                    } else {
                        $number = $i;
                    }

                    $link = $a->getAttribute('href');
                    if (strpos($link, '<!--') !== false) {
                        continue;
                    } elseif ($link_id = array_search($link, $links)) {
                        $EMLINK = new em_link($link_id);
                        $EMLINK->setField('in_code', true);
                        $EMLINK->setField('google_analytics', true);
                        $EMLINK->setField('html_position', 'L' . $number);
                        $EMLINK->saveItem();
                    } else {
                        $EMLINK = new em_link();
                        $EMLINK->setField('body_id', $body_id);
                        $EMLINK->setField('html_position', 'L' . $number);
                        $EMLINK->setField('value', $link);
                        $EMLINK->setField('active', true);
                        $EMLINK->setField('in_code', true);
                        $EMLINK->setField('google_analytics', true);
                        $EMLINK->saveItem();
                        $links[$EMLINK->getField('link_id')] = $link;
                        $this->childs['link'][] = $EMLINK;
                    }

                    $i++;
                }
            }

            if (isset($col2a) && $col2a) {
                $i = 1;
                foreach ($col2a as $a) {
                    if ($i < 10) {
                        $number = '0' . $i;
                    } else {
                        $number = $i;
                    }

                    $link = $a->getAttribute('href');
                    if (strpos($link, '<!--') !== false) {
                        continue;
                    } elseif ($link_id = array_search($link, $links)) {
                        $EMLINK = new em_link($link_id);
                        $EMLINK->setField('in_code', true);
                        $EMLINK->setField('google_analytics', true);
                        $EMLINK->setField('html_position', 'P' . $number);
                        $EMLINK->saveItem();
                    } else {
                        $EMLINK = new em_link();
                        $EMLINK->setField('body_id', $body_id);
                        $EMLINK->setField('html_position', 'P' . $number);
                        $EMLINK->setField('value', $link);
                        $EMLINK->setField('active', true);
                        $EMLINK->setField('in_code', true);
                        $EMLINK->setField('google_analytics', true);
                        $EMLINK->saveItem();
                        $links[$EMLINK->getField('link_id')] = $link;
                        $this->childs['link'][] = $EMLINK;
                    }

                    $i++;
                }
            }

            //Szukamy linkow w kodzie html
            $match = null;
            if(preg_match_all("(https?://[^ >\n\t\r'\"]+)", $layer_html, $match)){
                foreach($match[0] as $link){
                    if(strpos($link, '<!--')!==false){
                        continue;
                    }elseif($link_id = array_search($link, (array)$links)){
                        $EMLINK = new em_link($link_id);
                        $EMLINK->setField('in_code', true);
                        $EMLINK->saveItem();
                    }else{
                        $EMLINK = new em_link();
                        $EMLINK->setField('body_id', $body_id);
                        $EMLINK->setField('value', $link);
                        $EMLINK->setField('active', true);
                        $EMLINK->setField('in_code', true);
                        $EMLINK->saveItem();
                        $links[$EMLINK->getField('link_id')] = $link;
                        $this->childs['link'][] = $EMLINK;
                    }
                }
            }

            //Szukanie linkiow w tekscie
            $match = null;
            if(preg_match_all("(https?://[^ >\n\t\r'\"]+)", $layer_plain, $match)){
                foreach($match[0] as $link){
                    if(strpos($link, '<!--')!==false){
                        continue;
                    }elseif($link_id = array_search($link, (array)$links)){
                        $EMLINK = new em_link($link_id);
                        $EMLINK->setField('in_code', true);
                        $EMLINK->saveItem();
                    }else{
                        $EMLINK = new em_link();
                        $EMLINK->setField('body_id', $body_id);
                        $EMLINK->setField('value', $link);
                        $EMLINK->setField('active', true);
                        $EMLINK->setField('in_code', true);
                        $EMLINK->saveItem();
                        $links[$EMLINK->getField('link_id')] = $link;
                        $this->childs['link'][] = $EMLINK;
                    }
                }
            }

            //Aktualizacja flagi czekera
            $this->data['check_links'] = false;
            $this->prepareLayers();
            return true;
        }else{
            return true;
        }
    }

    /**
     * Sprawdzenie grafik w kodzie html
     */
    public function updateImages()
    {
        $layer_html = $this->getField('layer_html');
        if($zxc = $this->getChilds('em_file')->getListObject())
        foreach($zxc as $EMFILE){
            if(stripos($layer_html, $EMFILE->getField('name'))!==false){
                $EMFILE->setField('in_code', true);
            }else{
                $EMFILE->setField('in_code', false);
            }
            $EMFILE->saveItem();
        }
    }

    public function getEmSet() {
        if (!$this->emset) {
            $this->emset = new em_set($this->data['set_id']);
        }

        return $this->emset;
    }

    public function setActiveTargetlist($activeTargetlist) {
        $this->activeTargetlist = $activeTargetlist;
    }

    public function getActiveTargetlist() {
        return $this->activeTargetlist;
    }

    public function getCampaignNameForGA() {
        if (isset($this->data['campaign_name_ga']) && $this->data['campaign_name_ga']) {
            return $this->data['campaign_name_ga'];
        } else {
            $EMSET = $this->getEmSet();
            $campaignType = $EMSET->getField('type');

            if ($campaignType === 'NLG') {
                $number = PDOS::getInstance()->getOne("SELECT number FROM emailing.goniec_project WHERE set_id='{$this->data['set_id']}'");
                $campaignName = 'goniec' . $number;
            } else {
                $campaignName = $EMSET->getField('campaign_name');
                $campaignName = str_replace(
                    ['Ą', 'Ć', 'Ę', 'Ł', 'Ń', 'Ó', 'Ś', 'Ź', 'Ż', 'ą', 'ć', 'ę', 'ł', 'ń', 'ó', 'ś', 'ź', 'ż', ' '],
                    ['A', 'C', 'E', 'L', 'N', 'O', 'S', 'Z', 'Z', 'a', 'c', 'e', 'l', 'n', 'o', 's', 'z', 'z', '_'],
                    $campaignName
                );
            }

            $this->data['campaign_name_ga'] = strtolower($campaignName);
            return $this->data['campaign_name_ga'];
        }
    }

    public function getTargetListNameForGA()
    {
        $list = $this->getActiveTargetlist();
        if ($list) {
            $targetlistName = $list->getField('name');
            $targetlistName = str_replace(
                ['Ą', 'Ć', 'Ę', 'Ł', 'Ń', 'Ó', 'Ś', 'Ź', 'Ż', 'ą', 'ć', 'ę', 'ł', 'ń', 'ó', 'ś', 'ź', 'ż', ' '],
                ['A', 'C', 'E', 'L', 'N', 'O', 'S', 'Z', 'Z', 'a', 'c', 'e', 'l', 'n', 'o', 's', 'z', 'z', '_'],
                $targetlistName
            );
            if (empty($targetlistName)) {
                $targetlistName = 'email_testowy';
            }
        } else {
            $targetlistName = 'email_testowy';
        }

        return strtolower($targetlistName);
    }

    /**
     * Wparsowuje linki w html i plain wiadomosci
     */
    public function prepareLayers()
    {
        $layer_html = $this->getField('layer_html');
        $layer_plain = $this->getField('layer_plain');

        $EMSET = $this->getEmSet();
        $campaignType = $EMSET->getField('type');

        if($tmp = $this->getChilds('em_link')->getListObject()){
            $links = array();
            foreach($tmp as $EMLINK){
                if ($EMLINK->getField('active') && $EMLINK->getField('in_code')) {
                    $link = $EMLINK->getField('value');
                    $nlink = trim($EMLINK->createHashLink(), '/').'<!--target_id-->/';
                    if ($EMLINK->getField('google_analytics') || $campaignType == 'NLG') {
                        $nlink .= '?utm_campaign=<!--utm_campaign-->';
                        $nlink .= '&utm_source=<!--utm_source-->';
                        $nlink .= '&utm_medium=<!--utm_medium-->';
                        if ($htmlPosition = urlencode($EMLINK->getField('html_position'))) {
                            $nlink .= '&utm_content=' . $htmlPosition;
                        }
                    }
                    $links[$link] = $nlink;
                }
            }
            krsort($links);
            $layer_plain = str_replace(array_keys($links), array_values($links), $layer_plain);
            $layer_html = str_replace(array_keys($links), array_values($links), $layer_html);
        }

        $this->setField('pre_html', $layer_html);
        $this->setField('pre_plain', $layer_plain);
        $this->saveItem();
    }

    /**
     * Enter description here ...
     */
    public function createHash()
    {
        $hash = md5('Varant'.$this->getField('body_id').'CapeDun');
        return $hash;
    }

    /**
     * Enter description here ...
     * @param unknown_type $hash
     */
    public function validHash($hash)
    {
        if(md5('Varant'.$this->getField('body_id').'CapeDun')==$hash)
            return true;
        return false;
    }

    /**
     * Enter description here ...
     * @param unknown_type $target_id
     */
    public function createHashLink($target_id = null)
    {
        $link = '<!--gate-->open,'.$this->getField('body_id').','.$this->createHash();
        if($target_id){
            $EMTGT = new em_target($target_id);
            if($EMTGT->exists())
                $link .= ','.$EMTGT->getField('target_id').','.$EMTGT->createHash();
        }
        $link .= '/';
        return $link;
    }

    /**
     *
     */
    public function raiseStats($target_id, $client_ip = null)
    {
        $f = $v = null;
        if($client_ip){
            $f = ', client_ip';
            $v = ", '$client_ip'";
        }
        $sql = "INSERT INTO emailing.em_open (body_id, target_id{$f}) VALUES ('{$this->data['body_id']}', '$target_id'{$v})";
        PDOS::getInstance()->exec($sql);
    }

    /**
     *
     */
    protected function createEmailingObject()
    {
        if($this->mailObj==null){
            $EMSET = $this->getParent('em_set');
            $this->mailObj = new htmlMimeMail5();
            if($EMSET->getField('from_name'))
                $this->mailObj->setFrom($EMSET->getField('from_name')." <".$EMSET->getField('from_email').">");
            else
                $this->mailObj->setFrom($EMSET->getField('from_email'));
            $this->mailObj->setHTMLCharset('UTF-8');
            $this->mailObj->setTextCharset('UTF-8');
            $this->mailObj->setHeadCharset('UTF-8');
            $this->mailObj->setPriority('normal');
            $this->mailObj->setSMTPParams('smtp2.esculap.pl');
            //Załączanie plików
            if($tmp = $this->getChilds('em_file')->getListObject()){
                foreach($tmp as $EMFILE){
                    if($EMFILE->getField('type')=='i'){
                        $itype = 'application/octet-stream';
                        if(stripos($EMFILE->getField('name'), '.jpg')!==false || stripos($EMFILE->getField('name'), '.jpeg')!==false)
                            $itype = image_type_to_mime_type(IMAGETYPE_JPEG);
                        elseif(stripos($EMFILE->getField('name'), '.gif')!==false)
                            $itype = image_type_to_mime_type(IMAGETYPE_GIF);
                        elseif(stripos($EMFILE->getField('name'), '.png')!==false)
                            $itype = image_type_to_mime_type(IMAGETYPE_PNG);
                        elseif(stripos($EMFILE->getField('name'), '.bmp')!==false)
                            $itype = image_type_to_mime_type(IMAGETYPE_BMP);

                        $this->mailObj->addEmbeddedImage(new fileEmbeddedImage($EMFILE->getField('path'), $itype));
                    }else{
                        $this->mailObj->addAttachment(new fileAttachment($EMFILE->getField('path')));
                    }
                }
            }
            return $this->mailObj;
        }
        return $this->mailObj;
    }

    /**
     * Wysyla email
     * @todo poprawic na nowa klase abstractTable
     */
    public function sendAsEmail($target_id = null, $email = null, $first_name = null, $last_name = null, $gender = null, $title_prefix = null, $external_key = null)
    {
        //$zz = microtime(true);
        //Gdy obiekt nie istnieje
        if(!$this->exists) return false;

        $EMTGT = null;
        if(is_numeric($target_id)){
            $EMTGT = new em_target($target_id);
            $email = $EMTGT->getField('email');
            $first_name = $EMTGT->getField('first_name');
            $last_name = $EMTGT->getField('last_name');
            $gender = $EMTGT->getField('gender');
        }elseif(is_object($target_id)){
            $EMTGT = $target_id;
            $target_id = $EMTGT->getField('target_id');
            $email = $EMTGT->getField('email');
            $first_name = $EMTGT->getField('first_name');
            $last_name = $EMTGT->getField('last_name');
            $gender = $EMTGT->getField('gender');
            if(!$external_key)
                $external_key = $EMTGT->getField('external_key');
        }else{
            $EMTGT = new em_target();
        }
        //echo('Ladowanie targetu='.(microtime(true)-$zz)."\n");

        //$EMSET = new em_set($this->getField('set_id'), true);

        $mail = clone $this->createEmailingObject();
        if(!$this->getField('pre_html') && !$this->getField('pre_plain')) {
            $this->prepareLayers();
        }
        $layer_plain = $this->getField('pre_plain');
        $layer_html = $this->getField('pre_html');

        $target = $EMTGT->getField('target_id');
        $kampania = $this->getCampaignNameForGA();
        $lista = $this->getTargetListNameForGA();

        $pixel = '<img src="http://www.google-analytics.com/collect?v=1&tid=UA-10506232-20&cid='.$target.'&t=event&ec=email&ea=open&cs='.$lista.'&cm=email&cn='.$kampania.'&cm2=1?" width="1" height="1">';
        $imgHtml = '<img src="'.$this->createHashLink($target_id).'" width="1" height="1">';
        $layer_html = str_ireplace('</body>', $pixel. $imgHtml.'</body>', $layer_html);

        //Wykrywamy plec jesli nie jest wykryta
        if($gender!='w' && $gender!='m'){
            if(empty($first_name))
                $gender = 'u';
            elseif(strtolower($first_name{strlen($first_name)-1})=='a' && !in_array(strtolower($first_name), array('barnaba', 'kuba', 'jarema', 'kosma')))
                $gender = 'w';
            else
                $gender = 'm';
        }

        //Podmiana zmiennych 
        //<!--woman>tresc dla kobiet<woman-->
        //<!--man>tresc dla facetow<man-->
        //<!--unknown>tresc dla niezidentyfikowanych<unknown-->
        //<!--delete-->tresc widoczna ale do wywalenia<!--delete-->
        //<!--first_name--> <!--last_name--> <!--year--> <!--month--> <!--day--> <!--hour--> <!--minute--> <!--second--> <!--gate-->
        //<!--key-->
        if($gender=='w'){
            $layer_plain = str_ireplace(array('<!--woman>', '<woman-->'), '', $layer_plain);
            $layer_html = str_ireplace(array('<!--woman>', '<woman-->'), '', $layer_html);
        }else{
            $layer_plain = preg_replace('/<!--woman>.+<woman-->/misU', '', $layer_plain);
            $layer_html = preg_replace('/<!--woman>.+<woman-->/misU', '', $layer_html);
        }
        if($gender=='m'){
            $layer_plain = str_ireplace(array('<!--man>', '<man-->'), '', $layer_plain);
            $layer_html = str_ireplace(array('<!--man>', '<man-->'), '', $layer_html);
        }else{
            $layer_plain = preg_replace('/<!--man>.+<man-->/misU', '', $layer_plain);
            $layer_html = preg_replace('/<!--man>.+<man-->/misU', '', $layer_html);
        }
        if($gender=='u'){
            $layer_plain = str_ireplace(array('<!--unknown>', '<unknown-->'), '', $layer_plain);
            $layer_html = str_ireplace(array('<!--unknown>', '<unknown-->'), '', $layer_html);
        }else{
            $layer_plain = preg_replace('/<!--unknown>.+<unknown-->/misU', '', $layer_plain);
            $layer_html = preg_replace('/<!--unknown>.+<unknown-->/misU', '', $layer_html);
        }
        $layer_plain = preg_replace('/<!--delete-->.+<!--delete-->/misU', '', $layer_plain);
        $layer_html = preg_replace('/<!--delete-->.+<!--delete-->/misU', '', $layer_html);

        $find = array('<!--first_name-->', '<!--last_name-->', '<!--year-->', '<!--month-->', '<!--day-->', '<!--hour-->',
        			  '<!--minute-->', '<!--second-->', '<!--gate-->', '<!--hashmail-->');

        $repl = array($first_name, $last_name, date('Y'), date('m'), date('d'), date('G'),
                      date('i'), date('s'), 'http://emgate.esculap.com/emgate/', base64_encode($email));

        $find[] = '<!--target_id-->';
        $repl[] = $EMTGT->exists()?(','.$EMTGT->getField('target_id').','.$EMTGT->createHash()):'';
        $find[] = '<!--key-->';
        $repl[] = $external_key?$external_key:'';

        $EMSET = $this->getEmSet();
        $campaignType = $EMSET->getField('type');
        $medium = 'email';
        if ($campaignType == 'NLE') {
            $medium = 'express';
        } elseif ($campaignType == 'NLG') {
            $medium = 'goniec';
        }

        $find[] = '<!--utm_campaign-->';
        $repl[] = urlencode($kampania);
        $find[] = '<!--utm_source-->';
        $repl[] = urlencode($lista);
        $find[] = '<!--utm_medium-->';
        $repl[] = urlencode($medium);

        $layer_plain = str_ireplace($find, $repl, $layer_plain);
        $layer_html = str_ireplace($find, $repl, $layer_html);
        //Koniec podmianek

        //echo('Zmienne='.(microtime(true)-$zz)."\n");

        if($layer_plain)
            $mail->setText($layer_plain);
        if($layer_html)
            $mail->setHTML($layer_html);

        //echo('Ladowanie HTML i PLAIN ='.(microtime(true)-$zz)."\n");

        if($first_name || $last_name)
            $target = "$first_name $last_name <$email>";
        else
            $target = $email;
        //$target = 'slawek@esculap.pl';
        $mail->setSubject($title_prefix.$this->getField('real_title'));
        $mail->setHeader('message-id', ($EMTGT->exists()?$EMTGT->getField('target_id'):'0').'.'.$this->getField('set_id').'#EMAILING');
        $result = $mail->send(array($target), 'smtp');
        if($result && $EMTGT->exists()){
            $EMTGT->send($this->data['set_id']);
        }
        //echo('Czas calkowity='.(microtime(true)-$zz)."\n");
        return $result;
    }

    /**
     * Generuje wersje tekstowa wiadomosci z wersji HTML
     */
    public function generatePlainFromHtml()
    {
        $html = $this->getField('layer_html');
        $norm = array('<!--woman>', '<woman-->', '<!--man>', '<man-->', '<!--unknown>', '<unknown-->',
        	'<!--delete-->', '<!--first_name-->', '<!--last_name-->', '<!--year-->', '<!--target_id-->',
            '<!--month-->', '<!--day-->', '<!--hour-->', '<!--minute-->', '<!--second-->', '<!--gate-->', '<!--key-->');
        $hash = array('OSTARTWOMANC', 'OSTOPWOMANC', 'OSTARTMANC', 'OSTOPMANC', 'OSTARTUNKC', 'OSTOPUNKC',
        	'ODELETEC', 'OFNAMEC', 'OLNAMEC', 'OYEARC', 'OTARGETIDC',
            'OMONTHC', 'ODAYC', 'OHOURC', 'OMINUTEC', 'OSECONDC', 'OGATEC', 'OKEYC');
        $html = str_replace($norm, $hash, $html);
        $h2t = new html2text($html);
        $plain = $h2t->get_text();
        $plain = str_replace($hash, $norm, $plain);
        $this->setField('layer_plain', $plain);
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Link w emailingu
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_link extends abstractTable
{
    protected $table = 'emailing.em_link';
    protected $related = array(
        'em_body' => array('type' => 'parent', 'natural' => 'body_id')
    );

    /**
     * Enter description here ...
     * @param unknown_type $target_id
     */
    public function createHashLink($target_id = null)
    {
        $link = '<!--gate-->link,'.$this->getField('link_id').','.$this->createHash();
        if($target_id){
            $EMTGT = new em_target($target_id);
            if($EMTGT->exists())
                $link .= ','.$EMTGT->getField('target_id').','.$EMTGT->createHash();
        }
        $link .= '/';
        return $link;
    }

    /**
     * Enter description here ...
     */
    public function createHash()
    {
        $hash = md5('Braga'.$this->getField('link_id').'Ishtar');
        return $hash;
    }

    /**
     * Enter description here ...
     * @param unknown_type $hash
     */
    public function validHash($hash)
    {
        if(md5('Braga'.$this->getField('link_id').'Ishtar')==$hash)
            return true;
        return false;
    }

    /**
     *
     */
    public function raiseStats($target_id, $client_ip = null)
    {
        $f = $v = null;
        if($client_ip){
            $f = ', client_ip';
            $v = ", '$client_ip'";
        }
        $sql = "INSERT INTO emailing.em_click (link_id, target_id{$f}) VALUES ('{$this->data['link_id']}', '$target_id'{$v})";
        PDOS::getInstance()->exec($sql);
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Plik w emailingu
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_file extends abstractTable
{
    protected $table = 'emailing.em_file';
    protected $related = array(
        'em_body' => array('type' => 'parent', 'natural' => 'body_id')
    );

    public function deleteItem()
    {
        unlink($this->data['path']);
        parent::deleteItem();
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Lista odbiorcow emailingu
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_targetlist extends abstractTable
{
    protected $table = 'emailing.em_targetlist';
    protected $fields_extend = array('target_amount', 'sent_amount', 'wait_amount');
    protected $related = array(
        'em_set' => array('type' => 'parent', 'natural' => 'set_id'),
    	'em_schedule' => array('type' => 'parent', 'natural' => 'schedule_id')
    );
    protected $serials = array('tmp_info');

	/**
     * Walidowanie
     */
    public function validateItem()
    {
        $errors = $data = null;
        foreach($this->fields as $s)
            $data[$s] = isset($this->data[$s])?$this->data[$s]:null;

        if(strlen($data['name'])<5) $errors[] = 'Wpisz dłuższą nazwę listy!';
        if(intval($data['set_id'])<1) $errors[] = 'Brakuje identyfikatora emailingu!';

        if($errors){
            $this->errors = $errors;
            return false;
        }
        return true;
    }

	/**
     * Obliczanie wartości dodatkowych pol
     */
    protected function extendItem()
    {
        $sql = "SELECT COUNT(DISTINCT target_id) FROM emailing.em_tt WHERE list_id='{$this->data['list_id']}'";
        $this->data['target_amount'] = PDOS::getInstance()->getOne($sql);
        $sql = "SELECT COUNT(DISTINCT target_id) FROM emailing.em_tt WHERE list_id='{$this->data['list_id']}' AND send_date IS NOT NULL";
        $this->data['sent_amount'] = PDOS::getInstance()->getOne($sql);
        $this->data['wait_amount'] = $this->data['target_amount'] - $this->data['sent_amount'];
    }

    /**
     * Dopisanie nowego targetu
     **/
    public function assignTarget($email, $first_name = null, $last_name = null, $gender='a', $check = true, $noval = false, $external_key = null)
    {
        $email = trim($email);
        if(!$this->exists){
            $this->errors[] = 'Lista nie istnieje, lub nie jest w pelni zalasdowana!';
            return false;
        }
        $this->errors = null;

        $sql = "SELECT target_id FROM emailing.em_target WHERE email='" . mb_strtolower($email, 'UTF-8') . "'";
        $target_id = PDOS::getInstance()->getOne($sql);

        $user_login = null;
        $sql = "SELECT mu.login, plu.first_name, plu.last_name
                FROM main_users AS mu
                JOIN pl_users AS plu ON (plu.user_id = mu.id)
                WHERE mu.email='$email' OR mu.email_esc='$email'";
        PDOS::getInstance('db1', true)->exec("SET search_path TO esculap_int");
        if($data = PDOS::getInstance('db1', true)->getRow($sql)){
            $user_login = $data['login'];
            $first_name = $data['first_name'];
            $last_name = $data['last_name'];
        }

        if($gender!='w' && $gender!='m'){
            if(empty($first_name))
                $gender = 'u';
            elseif(strtolower($first_name{strlen($first_name)-1})=='a' && !in_array(strtolower($first_name), ['barnaba', 'kuba', 'jarema', 'kosma']))
                $gender = 'w';
            else
                $gender = 'm';
        }

        if(intval($target_id)==0){//Dodawanie targetu
            $target = new em_target();
        }else{//Target juz istnieje, mozna tutaj dokonac aktualizacji
            $target = new em_target($target_id);
        }

        $target->setField('email', mb_strtolower($email, 'UTF-8'));
        $target->setField('first_name', $first_name);
        $target->setField('last_name', $last_name);
        $target->setField('gender', $gender);
        $target->setField('user_login', $user_login);
        if($target->validateItem()){
            if($check)
                $z = null;
            elseif($target->saveItem())
                $target_id = $target->getField('target_id');
            else
                $this->errors['CANT_SAVE'] = 'Nie można zapisać rekordu.';
        }else{
            $this->errors = $target->getErrors();
        }

        //Sprawdzenie czy jest zgoda na emailing
        PDOS::getInstance('db1', true)->exec("SET search_path TO esculap_int");
        $sql = "SELECT login FROM main_users WHERE email='$email' OR email_esc='$email'";
        if($data = PDOS::getInstance('db1', true)->getRow($sql)){
            $user_login_test = $data['login'];
        } else {
            $user_login_test = null;
        }
        if (!$noval && $user_login_test){
            $type = $this->getParent('em_set')->getField('type');

            PDOS::getInstance('db1', true)->exec("SET search_path TO esculap_int");

            $sql = "SELECT COUNT(1) FROM rel_users_services rus
                    JOIN main_users mu ON (mu.id = rus.user_id)
                    WHERE (mu.email = '{$email}' OR mu.email_esc = '{$email}')
                    AND rus.service_code = '{$type}'
                   ";

            //$sql = "SELECT COUNT(1) FROM rel_users_services WHERE user_login='$user_login' AND service_code='$type'";
            if(PDOS::getInstance('db1', true)->getOne($sql)<1){
                $this->errors['SERVICE_ERR'] = 'Odbiorca nie wyraził zgody na ten typ emailingu.';
                //return false;
            }
        }

        //Sprawdzenie czy target nie jest powiązany z żadną listą dla tego emailingu
        if(intval($target_id)>0){
            $sql = "SELECT COUNT(1) FROM emailing.em_targetlist a INNER JOIN emailing.em_tt b ON a.list_id=b.list_id ".
            	   "WHERE b.target_id=$target_id AND a.set_id={$this->data['set_id']}";
            if(PDOS::getInstance()->getOne($sql)>0){
                $this->errors['EXISTS_ERR'] = 'Odbiorca już dodany do list docelowych tego emailingu.'." [{$target_id}]";
                //return false;
            }
        }

        //
        if($this->errors) return false;
        if(!$check){
            //$sql = "INSERT INTO emailing.em_tt (list_id, target_id) VALUES ({$this->data['list_id']}, $target_id)";
            //PDOS::getInstance()->exec($sql);
            $ETT = new em_tt();
            $ETT->setField('list_id', $this->data['list_id']);
            $ETT->setField('target_id', $target_id);
            if($external_key)
                $ETT->setField('external_key', $external_key);
            if(!$ETT->saveItem()){
                $this->errors[] = 'Błąd podczas wpisywania do relacji.';
                return false;
            }
        }
        return $target_id?$target_id:true;
    }

    /**
     * Wypisanie targetu ;)
     **/
    public function unassignTarget($email = null, $target_id = null, $check = true)
    {
        if(!$this->exists) return false;
        if($email == null && $target_id == null) return false;
        if(!$target_id)
            $target_id = PDOS::getInstance()->getOne("SELECT target_id FROM emailing.em_target WHERE email='$email'");

        if(intval($target_id)<1){
            $target_id = PDOS::getInstance()->getOne("SELECT target_id FROM emailing.em_target WHERE email='" . mb_strtolower($email, 'UTF-8') . "'");

            if(intval($target_id)<1) {
                $this->errors['EXISTS_ERR'] = 'Nie ma takiego odbiorcy.';

                return false;
            }
        }

        //Sprawdzenie czy jest hjna liscie
        $sql = "SELECT COUNT(1) FROM emailing.em_tt WHERE target_id=$target_id AND list_id={$this->data['list_id']}";
        if(PDOS::getInstance()->getOne($sql)<1){
            $this->errors['EXISTS_ERR'] = 'Odbiorca nie powiązany z emailingiem.';
            return false;
        }

        if(!$check){
            $sql = "DELETE FROM emailing.em_tt WHERE list_id={$this->data['list_id']} AND target_id=$target_id";
            PDOS::getInstance()->exec($sql);
        }
        return true;
    }

    /**
     *
     */
    public function isAssignTarget($target_id)
    {
        $sql = "SELECT COUNT(1) FROM emailing.em_tt WHERE list_id='{$this->data['list_id']}' AND target_id='$target_id'";
        if(PDOS::getInstance()->getOne($sql))
            return true;
        return false;
    }

    /**
     * Pobiera czesc targetu na ktora wysylana bedzie wiadomosc
     */
    public function getPartToSend($limit = 10)
    {
        $sql = "SELECT * FROM emailing.em_tt WHERE list_id='{$this->data['list_id']}' AND send_date IS NULL LIMIT $limit";
        $tids = PDOS::getInstance()->getAll($sql);
        $data = null;
        foreach($tids as $dd){
            $target_id = $dd['target_id'];
            $external_key = $dd['external_key'];
            $EMTGT = new em_target($target_id);
            $EMTGT->setField('external_key', $external_key);
            $data[] = $EMTGT;
        }
        return $data;
    }

    /**
     * Dodanie odbiorców bezpośrednio z pól input
     */
    public function targetFromImput($data, $remove = false, $check = true, $toschedule = false)
    {
        if(!$this->exists){
            $this->errors[] = 'Lista jeszcze nie istnieje!';
            return false;
        }
        $this->errors = $errors = null;
        $amount = array('valid'=>0, 'wrong'=>0);
        $need_keys = $this->getParent('em_set')->getField('need_keys');

        /*if($need_keys){
            $this->errors[] = "Wymagane są klucze zewnętrzne użytkowników, dodaj liste użytkowników przez plik CSV zawierający kolumne external_key!";
            return array('amount'=>$amount, 'errors'=>$this->errors);
        }*/

        if($toschedule){
            return false;
        }
        for($i=0;$i<count($data['email']);$i++){
            $key = isset($data['key'][$i])?$data['key'][$i]:null;
            if(!strlen($data['email'][$i])) continue;
            if($remove)
                $res = $this->unassignTarget($data['email'][$i], null, $check);
            else
                $res = $this->assignTarget($data['email'][$i],
                $data['first_name'][$i],
                $data['last_name'][$i],
                $data['gender'][$i], $check, false, $key);

            if($res){
                $amount['valid']++;
            }else{
                $amount['wrong']++;
                $errors[] = implode(' ', (array)$this->getErrors()).' -> '.$data['email'][$i];
            }
        }

        return array('amount'=>$amount, 'errors'=>$errors);
    }

    /**
     * Dodanie odbiorców z bazy danych z pomocą filtra użytkowników
     */
    public function targetFromDb($data, $remove = false, $check = true, $toschedule = false)
    {
        if(!$this->exists){
            $this->errors[] = 'Lista jeszcze nie istnieje!';
            return false;
        }
        $this->errors = $errors = null;

        $amount = array('valid'=>0, 'wrong'=>0);
        $type = $this->getParent('em_set')->getField('type');
        $need_keys = $this->getParent('em_set')->getField('need_keys');

        if($need_keys){
            $this->errors[] = "Wymagane są klucze zewnętrzne użytkowników, dodaj liste użytkowników przez plik CSV zawierający kolumne external_key!";
            return array('amount'=>$amount, 'errors'=>$this->errors);
        }

        $join = $where = null;
        if(isset($data['act_ut'])){
            $where .= " AND mu.user_type IN ('".str_replace(",", "','", $data['user_types'])."')";
        }

        if(isset($data['act_int'])){
            $join .= "JOIN esculap_int.rel_users_spec spec ON mu.id = spec.user_id ";
            $where .= " AND spec.spec_code IN ('".str_replace(",", "','", $data['specs'])."')";
        }

        if($data['genderf']=='w') {
            $join .= " JOIN esculap_int.pl_users mui ON (mu.id = mui.user_id) ";
            $where .= " AND LOWER(mui.first_name) LIKE '%a' AND LOWER(mui.first_name) NOT IN ('barnaba', 'kuba', 'jarema', 'kosma')";
        }

        if($data['genderf']=='m') {
            $join .= " JOIN esculap_int.pl_users mui ON (mu.id = mui.user_id) ";
            $where .= " AND (LOWER(mui.first_name) NOT LIKE '%a' OR LOWER(mui.first_name) IN ('barnaba', 'kuba', 'jarema', 'kosma'))";
        }

        $sql = <<<ESKUEL
SELECT DISTINCT mu.email AS email FROM esculap_int.main_users mu 
JOIN esculap_int.rel_users_services serv1 ON mu.id = serv1.user_id AND serv1.service_code = '{$type}'
JOIN esculap_int.rel_users_services serv2 ON mu.id = serv2.user_id AND serv2.service_code = 'MEX'
{$join}
WHERE mu.email != ''
AND mu.retired = false
{$where}
UNION
SELECT DISTINCT mu.email_esc AS email FROM esculap_int.main_users mu
JOIN esculap_int.rel_users_services serv1 ON mu.id = serv1.user_id AND serv1.service_code = '{$type}'
--JOIN esculap_int.rel_users_services serv2 ON mu.id = serv2.user_id AND serv2.service_code = 'MIN'
{$join}
WHERE mu.email_esc != ''
AND mu.retired = false
{$where}
ESKUEL;

        $sth = PDOS::getInstance('db1', true)->prepare($sql);
        $sth->execute();

        $i = 0;
        while($result = $sth->fetch(PDO::FETCH_ASSOC)){
            if($remove)
                $res = $this->unassignTarget($result['email'], null, $check);
            else
                $res = $this->assignTarget($result['email'], null, null, null, $check);

            if($res){
                $amount['valid']++;
            }else{
                $amount['wrong']++;
                $errors[] = implode(' ', (array)$this->getErrors()).' -> '.$result['email'];
            }
            $i++;
        }
        return array('amount'=>$amount, 'errors'=>$errors);
    }

    /**
     * Dodanie odbiorców z pliku csv
     */
    public function targetFromCsv($file, $remove = false, $check = true, $toschedule = false)
    {
        if(!$this->exists){
            $this->errors[] = 'Lista jeszcze nie istnieje!';
            return false;
        }
        $errors = null;
        $amount = array('valid'=>0, 'wrong'=>0);
        if($toschedule){
            return false;
        }
        $type = $this->getParent('em_set')->getField('type');
        $need_keys = $this->getParent('em_set')->getField('need_keys');

        $load = array('esculap_id'=>null, 'email'=>null, 'first_name'=>null, 'last_name'=>null, 'gender'=>null, 'external_key'=>null);
        $mode = null;
        $delimiter = ';';$dd = array(';', "\t", ',');
        $enclosure = '"';$ee = array('"', "'");

        if(file_exists($file) && $f = fopen($file, 'r')){
            $fline = file_get_contents($file, false, null, -1, 1024);
            $charset = mb_detect_encoding($fline, 'UTF-8, ISO-8859-2, WINDOWS-1252', true);

            //Pobieranie nagłówka (pierwsza linia)
            $fline = iconv($charset, 'UTF-8', fgets($f));
            foreach($ee as $e) if($e==$fline{0}) $enclosure = $e;
            foreach($dd as $d) if(strpos($fline, $d)!==false) $delimiter = $d;
            fclose($f);$f = fopen($file, 'r');
            $header = fgetcsv($f, 1024, $delimiter, $enclosure);
            foreach($header as $key => $field)
                foreach($load as $fname=>$v) if($fname==$field) $load[$fname] = trim($key);

            if($need_keys && $load['external_key']===null){
                $this->errors[] = "Wymagane są klucze zewnętrzne użytkowników, dodaj liste użytkowników przez plik CSV zawierający kolumne external_key!";
                return array('amount'=>$amount, 'errors'=>$this->errors);
            }

            if($load['esculap_id']!==null){
                $errors[] = "Info: wczytywanie po identyfikatorach esculap.";
                $mode = 'esculap_id';
            }elseif($load['email']!==null){
                if($load['last_name']!==null || $load['first_name']!==null){
                    $errors[] = "Info: wczytywanie danych odbiorców (email, imie, nazwisko).";
                    $mode = 'data';
                }else{
                    $errors[] = "Info: wczytywanie po emailach odbiorców.";
                    $mode = 'email';
                }
            }else{
                $errors[] = "Błąd parsowania pliku csv, w nagłówku oczekiwano na pole esculap_id lub email.";
            }

            if($mode){

                while($row = fgetcsv($f, 1024, $delimiter, $enclosure)){
                    $emails = $first_name = $last_name = $gender = $external_key = null;
                    if($mode=='esculap_id'){
                        $esculap_id = $load['esculap_id']!==null?$row[$load['esculap_id']]:null;
                        $sql = "SELECT COUNT(1) FROM main_users_sync WHERE esc_id='$esculap_id'";
                        if(!PDOS::getInstance()->getOne($sql)){
                            $errors[] = "Brak esculap_id=$esculap_id w bazie uzytkownikow.";
                            continue;
                        }
                        $sql = "SELECT a.user_login, a.email_ext, a.email_esc FROM main_users a ".
                        	   "INNER JOIN main_users_sync b ON a.user_login=b.user_login WHERE esc_id='$esculap_id'";
                        $row = PDOS::getInstance()->getRow($sql);
                        $user_login = $row['user_login'];
                        $sql = "SELECT service_code as value FROM rel_users_services WHERE user_login='$user_login'";
                        $services = PDOS::getInstance()->getCol($sql);
                        if(!in_array($type, $services)){
                            $errors[] = "Brak zgody na emailing tego typu [esculap_id=$esculap_id].";
                            continue;
                        }else{
                            if(in_array('MEX', $services) && $row['email_ext']) $emails[] = $row['email_ext'];
                            if($row['email_esc']) $emails[] = $row['email_esc']; //in_array('MIN', $services) &&
                        }

                    }else{
                        $emails[] = $load['email']!==null?$row[$load['email']]:null;
                        $first_name = $load['first_name']!==null?$row[$load['first_name']]:null;
                        $last_name = $load['last_name']!==null?$row[$load['last_name']]:null;
                        $gender = $load['gender']!==null?$row[$load['gender']]:null;
                    }
                    $external_key = $load['external_key']!==null?$row[$load['external_key']]:null;

                    foreach($emails as $email){
                        if($remove)
                            $res = $this->unassignTarget($email, null, $check);
                        else
                            $res = $this->assignTarget($email, $first_name, $last_name, $gender, $check, false, $external_key);

                        if($res){
                            $amount['valid']++;
                        }else{
                            $amount['wrong']++;
                            if($this->errors)
                                $errors[] = implode(' ', $this->errors).' -> '.$email;
                            else
                                $errors[] = 'Nieznany błąd. -> '.$email;
                        }
                    }
                }
            }
        }else{
            $errors[] = "Błąd podczas otwierania pliku.";
        }
        //echo "[$delimiter][$enclosure][$charset] ".print_r($load, true).$fline;
        return array('amount'=>$amount, 'errors'=>$errors);
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Odbiorca emailingów
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_target extends abstractTable
{
    protected $table = 'emailing.em_target';
    protected $fields_extend = array('external_key');

    /**
     * Walidacja
     * @see abstractTable::validateItem()
     */
    public function validateItem()
    {
        $this->errors = $errors = null;

        if(!preg_match('(^[-a-zA-Z0-9_.]+@[-a-zA-Z0-9_.]+[.][-a-zA-Z0-9_.]+$)', $this->data['email']))
            $errors['EMAIL_ERR'] = 'Email o nieprawidłowej składni.';

        if($errors){
            $this->errors = $errors;
            return false;
        }
        return true;
    }

    /**
     * Tworzy hasz targetu
     */
    public function createHash()
    {
        $hash = md5('Ishtar'.$this->getField('target_id').'Braga');
        return $hash;
    }

    /**
     * Sprawdza hasz targetu
     * @param string $hash
     */
    public function validHash($hash)
    {
        if(md5('Ishtar'.$this->getField('target_id').'Braga')==$hash)
            return true;
        return false;
    }

    /**
     * Zapisuje jaki emailing zostal wyslany na ten target
     * @param unknown_type $set_id
     */
    public function send($set_id)
    {
        $EMSET = new em_set($set_id);
        $tls = $EMSET->getChilds('em_targetlist')->getListObject();
        $list_id = null;
        foreach($tls as $EMTL){
            if($EMTL->isAssignTarget($this->data['target_id'])){
                $list_id = $EMTL->getField('list_id');
                break;
            }
        }
        if($list_id){
            $sql = "UPDATE emailing.em_tt SET send_date=NOW() WHERE list_id='$list_id' AND target_id='{$this->data['target_id']}'";
            PDOS::getInstance()->exec($sql);
        }
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Klasa obsługuje wysyłki emailingu - tak, może być klika rozłożonych w czasie i z różnym layoutem
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_schedule extends abstractTable
{
    protected $table = 'emailing.em_schedule';
    protected $related = array(
    	'em_body' => array('type'=>'parent', 'natural'=>'body_id'),
        'em_targetlist' => array('type'=>'child')
    );

	/**
     * Walidowanie
     */
    public function validateItem()
    {
        $errors = $data = null;
        foreach($this->fields as $s)
            $data[$s] = isset($this->data[$s])?$this->data[$s]:null;

        if(strlen($data['name'])<5)
            $errors[] = 'Wpisz dłuższą nazwę wysyłki!';
        if(intval($data['body_id'])<1)
            $errors[] = 'Wybierz wygląd wysyłki!';
        if($this->sync_data['start_date']!=$data['start_date'] && strtotime($data['start_date'])<time())
            $errors[] = 'Data wysyłki powinna być określona w przyszłości.';

        if($errors){
            $this->errors = $errors;
            return false;
        }
        return true;
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Testowy odbiorca emailingów
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_test extends abstractTable
{
    protected $table = 'emailing.em_test';
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Statystyka klikniec
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_click extends abstractTable
{
    protected $table = 'emailing.em_click';
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Statystyka otwarc
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_open extends abstractTable
{
    protected $table = 'emailing.em_open';
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Lacznik targetu z listami
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class em_tt extends abstractTable
{
    protected $table = 'emailing.em_tt';
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Tymczasowe miejsca dla statystyk dostarczalnosci maili
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class zxc_maillog extends abstractTable
{
    protected $table = 'emailing.zxc_maillog';
    protected $related = array(
    	'em_target' => array('type'=>'parent', 'natural'=>'target_id'),
        'em_set' => array('type'=>'parent', 'natural'=>'set_id')
    );

    /**
     * Przeniesienie kodu do statystyk
     */
    public function toStats()
    {
        if(!$this->exists) return false;
        $EMSET = $this->getParent('em_set');
        $EMTGT = $this->getParent('em_target');
        $EMB = $EMSET->getChilds('em_targetlist');
        $i = 0;
        while($EMB->list2item($i)){
            if($EMB->isAssignTarget($EMTGT->getField('target_id'))){
                $sql = "UPDATE emailing.em_tt SET status='".$this->getField('status')."' ".
                       "WHERE list_id=".$EMB->getField('list_id')." AND target_id=".$EMTGT->getField('target_id');
                PDOS::_()->exec($sql);
            }
            $i++;
        }
    }
}

?>