<?php

require_once 'abstractTable.class.php';

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Grupa zewnetrznych kluczy
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class extkeygroup extends abstractTable
{
    protected $table = 'esculap.main_extkeygroup';
    protected $related = array(
        'extkey' => array('type' => 'child')
    );
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Pojedynczy klucz zewnetrzny uzytkownika
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class extkey extends abstractTable
{
    protected $table = 'esculap.main_extkey';
    protected $related = array(
        'extkeygroup' => array('type'=>'parent', 'natural'=>'extkeygroup_id')
    );
    
    /**
     * Generowanie kodu ktory bedzie unikalny na ten czas
     */
    public function generateCode($length = 5)
    {
        $charTab = array('1','2','3','4','5','6','7','8','9','0','a','b','c','d','e','f','g','h','i','j','k',
        				 'l','m','n','o','p','r','s','t','u','w','y','z');
        $code = null;
        for($i=0;$i<$length;$i++){
            $code .= $charTab[rand(0,count($charTab)-1)];
        }
        //Sprawdzenie czy jest aktywny kod
        $CHCK = new extkey();
        $CHCK->addCondition('code', 'LIKE', $code);
        $CHCK->addCondition('valid_date', '>=', 'NOW()');
        $CHCK->search();
        if($CHCK->getAmount())
            return $this->generateCode($length);
        $this->setField('code', $code);
        return $code;
    }
    
    /**
     * Uzycie kodu, sprawdza istnienie i dokonuje zmian
     * jesli kod jest jednorazowy za drugim razem zwroci false
     * @param string $code Kod ktory ma byc uzyty
     * @param array $groups Grupy kodow (extkeygroup_id) brane pod uwage, null=wszystkie
     */
    public function useCode($code, $groups = null)
    {
        if(!strlen($code))
            return false;
        $this->clear();
        $this->addCondition('code', 'LIKE', $code);
        $this->addCondition('valid_date', '>=', 'NOW()');
        if(is_array($groups))
            $this->addCondition('extkeygroup_id', 'IN', '('.implode(',', $groups).')');
        $this->search();
        if(!$this->getAmount())
            return false;
        $this->list2item();
        $EXTKEYG = $this->getParent('extkeygroup');
        if($EXTKEYG->getField('one_use') && $this->getField('first_use_date'))
            return false;
        if(!$this->getField('first_use_date'))
            $this->setField('first_use_date', 'SQL:NOW()');
        $this->setField('last_use_date', 'SQL:NOW()');
        $this->saveItem();
        return true;
    }
}

?>