<?php

require_once 'abstractTable.class.php';

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Główna tabela użytkowników
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class main_users extends abstractTable
{
    protected $table = 'esculap.main_users';
    protected $related = array(
        'main_users_sync' => array('type'=>'child'),
    	'rel_users_spec' => array('type'=>'child'),
    	'rel_users_services' => array('type'=>'child'),
        'mailbox' => array('type'=>'child', 'from'=>'user_login', 'to'=>'userlogin'),
    	'main_users_changes' => array('type'=>'child'),
    	'main_users_login' => array('type'=>'child')
    );
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Tabela synchronizacji użytkowników
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class main_users_sync extends abstractTable
{
    protected $table = 'esculap.main_users_sync';
    protected $related = array(
        'main_users' => array('type'=>'parent', 'natural'=>'user_login')
    );
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Relacja użytkownik - usługa
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class mailbox extends abstractTable
{
    protected  $table = 'postfix.mailbox';
    protected $related = array(
        'main_users' => array('type'=>'parent', 'from'=>'userlogin', 'to'=>'user_login')
    );
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Tabela historii zmian konta uzytkownika
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class main_users_changes extends abstractTable
{
    protected $table = 'esculap.main_users_changes';
    protected $related = array(
        'main_users' => array('type'=>'parent', 'natural'=>'user_login')
    );
}

/**
 * 
 * Specjalizacje
 * @author sroszczyk
 *
 */
require_once 'dict_spec.tab.php';

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Relacja użytkownik - specjalizacja
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class rel_users_spec extends abstractTable
{
    protected $table = 'esculap.rel_users_spec';
    protected $related = array(
        'main_users' => array('type'=>'parent', 'natural'=>'user_login'),
        'dict_spec' => array('type'=>'parent', 'natural'=>'spec_code')
    );
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Tabela usług
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class dict_services extends abstractTable
{
    protected $table = 'esculap.dict_services';
    protected $related = array(
        'rel_users_services' => array('type'=>'child')
    );
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Relacja użytkownik - usługa
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class rel_users_services extends abstractTable
{
    protected $table = 'esculap.rel_users_services';
    protected $related = array(
        'main_users' => array('type'=>'parent', 'natural'=>'user_login'),
        'dict_services' => array('type'=>'parent', 'natural'=>'service_code')
    );
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Widok na logowania uzytkownikow
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class main_users_login extends abstractTable
{
    protected $table = 'esculap.main_users_login';
    protected $primary = array(
		'user_login', 'start_date'
	);
	protected $fields = array(
		'user_login', 'start_date', 'stop_date', 'code', 'user_system', 'user_proxy', 'user_ip'
	);
	protected $f_type = array(
		'user_login' => 'character varying',
		'start_date' => 'timestamp with time zone',
		'stop_date' => 'timestamp with time zone',
	    'code' => 'character varying',
    	'user_system' => 'character varying',
    	'user_proxy' => 'character varying',
    	'user_ip' => 'character varying'
	);
    protected $related = array(
        'main_users' => array('type'=>'parent', 'natural'=>'user_login')
    );
    protected $use_returning = false;
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Widok na log uzytkownikow
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class main_users_log extends abstractTable
{
    protected $table = 'esculap.main_users_log';
    protected $primary = array(
		'user_login', 'action_time'
	);
	protected $fields = array(
		'user_login', 'action_time', 'is_logged', 'host', 'uri', 'referer'
	);
	protected $f_type = array(
		'user_login' => 'character varying',
		'action_time' => 'timestamp with time zone',
		'is_logged' => 'boolean',
	    'host' => 'character varying',
    	'uri' => 'character varying',
    	'referer' => 'character varying'
	);
    protected $related = array(
        'main_users' => array('type'=>'parent', 'natural'=>'user_login')
    );
    protected $use_returning = false;
    
    public function saveItem($s = false)
    {
        if($this->exists()) return false;
        
        if(($host = $this->getField('host')) && ($referer = $this->getField('referer')) && preg_match("({$host})", $referer))
            $this->setField('referer', null);
        
        return parent::saveItem($s);
    }
    
    public function deleteItem()
    {
        return false;
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Lista stron majacych dostep do zdalnego logowania
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class main_auth_site extends abstractTable
{
    protected $table = 'esculap.main_auth_site';
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Tymczasowa tabela uzytkownika sesji
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class main_auth_code extends abstractTable
{
    protected $table = 'esculap.main_auth_code';
    
    public function saveItem($s = false)
    {
        //Generowanie identyfikatora
        /*$test_code = null;
        while(strlen($test_code)<12) $test_code .= chr(($z=rand(55,90))>64?$z:$z-7);
        $MAC = new main_auth_code();
        $MAC->addCondition('auth_code', '=', $test_code);
        $MAC->search();
        if($MAC->getAmount())
            $this->saveItem($s);
        $this->setField('auth_code', $test_code);*/
        
        $oe = $this->exists();
        if(!$oe && !$this->getField('auth_code')) $this->generateCode();
        $res = parent::saveItem($s);
        
        //Zapis do main_users_login
        if($res && !$oe){
            $MUL = new main_users_login();
            $MUL->setField('user_login', $this->getField('user_login'));
            $MUL->setField('code', $this->getField('auth_code'));
            $MUL->setField('start_date', 'SQL:NOW()');
            if($tmp = $this->getField('user_system')) $MUL->setField('user_system', $tmp);
            if($tmp = $this->getField('user_proxy')) $MUL->setField('user_proxy', $tmp);
            if($tmp = $this->getField('user_ip')) $MUL->setField('user_ip', $tmp);
            $MUL->saveItem();
        }
        
        //Kasowanie smieci, przy tworzeniu jednego szansa 25% na usuniecie 20
       // if(rand(0,99)<25) $this->GC();
         $this->GC(); //DAREK
        
        return $res;
    }
    
    public function deleteItem($is_timeout = false)
    {
        //Zmiana main_users_login
        if($this->exists){
            $MUL = new main_users_login();
            $MUL->addCondition('code', '=', $this->getField('auth_code'));
            $MUL->addCondition('stop_date', 'IS', null);
            $MUL->search(); $i = 0;
            while($MUL->list2item($i++)){
                $MUL->setField('code', null);
                if($is_timeout && ($tmp = $this->getField('terminate')))
                    $MUL->setField('stop_date', $tmp);
                else
                    $MUL->setField('stop_date', 'SQL:NOW()');
                //if($tmp = $this->getField('user_system')) $MUL->setField('user_system', $tmp);
                //if($tmp = $this->getField('user_proxy')) $MUL->setField('user_proxy', $tmp);
                //if($tmp = $this->getField('user_ip')) $MUL->setField('user_ip', $tmp);
                $MUL->saveItem();
            }
        }
        
        return parent::deleteItem();
    }
    
    /**
     * Generuje unikalne kody
     * @param integer $l dlugosc kodu <5,20>
     */
    public function generateCode($l = 12)
    {
        if($l<5) $l=5;
        if($l>20) $l=20;
        
        $code = null;
	    while(strlen($code)<$l) $code .= chr(($z=rand(55,90))>64?$z:$z-7);
        
	    $MAC = new main_auth_code();
	    $MAC->addCondition('auth_code', '=', $code);
	    $MAC->search();
	    if($MAC->getAmount()) return $this->generateCode($l);
	    $this->setField('auth_code', $code);
	    return $code;
    }
    
    /**
     * Zbieracz smieci
     */
    public function GC()
    {
        $MAC = new main_auth_code();
        $MAC->addCondition('terminate', '<', 'NOW()');
        $MAC->setLimit(20);
        $MAC->addOrder('terminate', true);
        $MAC->search();$i = 0;
        while($MAC->list2item($i++)){
            $MAC->deleteItem(true);
        }
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Tabela zadan do/od uzytkownika
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class main_users_request extends abstractTable
{
    protected $table = 'esculap.main_users_request';
    protected $serials = array('data');
    
    public function saveItem($p=false)
    {
        $this->GC();
        if(!$this->exists){
            $sql = "SELECT MAX(request_repeat) FROM {$this->table} WHERE user_login='{$this->data['user_login']}' AND request_type='{$this->data['request_type']}'";
            $this->data['request_repeat'] = ($z=PDOS::_()->getOne($sql))?$z+1:1;
        }
        return parent::saveItem($p);
    }
    
    /**
     * Generuje unikalne kody
     * @param integer $l dlugosc kodu <5,20>
     */
    public function generateCode($l = 10)
    {
        if($l<5) $l=5;
        if($l>20) $l=20;
        
        $code = null;
	    while(strlen($code)<$l) $code .= chr(($z=rand(55,90))>64?$z:$z-7);
        
	    $MUR = new main_users_request();
	    $MUR->addCondition('code', '=', $code);
	    $MUR->search();
	    if($MUR->getAmount()) return $this->generateCode($l);
	    $this->setField('code', $code);
	    return $code;
    }
    
    /**
     * 
     */
    public function GC()
    {
        $MUR = new main_users_request();
        $MUR->addCondition('timeout', '<', 'NOW()');
        $MUR->addOrder('timeout');
        $MUR->setLimit(25);
        $MUR->search(); $i = 0;
        while($MUR->list2item($i++)){
            $MUR->deleteItem(true);
        }
    }
    
    /**
     * Usuwanie requestow
     */
    public function deleteItem($timeout = false){
        if($timeout && $this->getField('request_type')=='activation'){
            $MU = new main_users($this->getField('user_login'));
            $res = parent::deleteItem();
            $MU->deleteItem();
            return $res;
        }
        return parent::deleteItem();
    }
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Rodzaje zainteresowan strona
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class dict_informed_source extends abstractTable
{
    protected $table = 'esculap.dict_informed_source';
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Rodzaje zainteresowan strona
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class main_users_informed_source extends abstractTable
{
    protected $table = 'esculap.main_users_informed_source';
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Rodzaje zainteresowan strona
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class dict_user_type extends abstractTable
{
    protected $table = 'esculap.dict_user_type';
}

/** ------------------------------------------------------------------------------------------------------ **
 *                                                                                                         **
 * Zablokowane loginy
 * @author sroszczyk
 *                                                                                                         **
---------------------------------------------------------------------------------------------------------- */
class main_users_locked_names extends abstractTable
{
    protected $table = 'esculap.main_users_locked_names';
}

?>
