<?php

/******************************************************************************
 *                                                                            *
 *      Klasa obsługująca ogłoszenia                                          *
 *      @category data                                                        *
 *      @author Sławek                                                        *
 *      UWAGA: używać w klasach poytomnych Controller.class.php               *
 *                                                                            *
 ******************************************************************************/

require_once('dimage.class.php');
require_once('payment.class.php');

class offer
{
    /** Uchwyt do bazy danych **/
    private $db;
    
    /** Zalogowany użytkownik **/
    private $user_login;
    private $allow_tags = '<b><p><span><ul><li><ol><u><i><strong><br>';
    
    /** Używane w panelu ogłoszeń **/
    private $manage = false;
    private $cms = false;
    
    /** **/
    private $amount = 0;
    private $rows = 5;
    private $range = 5;
    
    /**
     *  Konstruktor, 
     **/
    public function __construct($manage = null)
    {
        global $db;
        $this->db = $db;
        
        if($manage=='user')
            $this->manage = true;
        elseif($manage=='cms')
            $this->cms = true;
        $this->user_login = defined('CONTR_USER')?CONTR_USER:null;
    }
    
    /**
     *  Pobiera listę ogłoszeń
     *  @param integer $offer_type - identyfikator rodzaju ogłoszenia
     *  @param string $offer_group - grupa ogłoszeń
     *  @param boolean $is_premium - czy wyciągać ogłoszenia płatne
     *  @param boolean $is_prefer - czy wyciągać ogłoszenia polecane
     *  return array - tablica ogłoszeń
     **/
    public function getItems($offer_type = null, $offer_group = null, $is_premium = null, $is_prefer = null, $page=0)
    {
        if($this->manage || $this->cms)
            $payment = new payment();
        
        $rows = $this->rows;
        $whr = '';
        if($this->cms){
            
        }
        if($this->manage){
            $whr[] = "user_login='{$this->user_login}'";
            if($offer_type)
                switch($offer_type){
                    case 1:
                        $whr[] = 'is_active=FALSE';
                        $whr[] = 'is_finish=FALSE';
                        break;
                    case 2:
                        $whr[] = 'is_active=TRUE';
                        break;
                    case 3:
                        $whr[] = 'is_finish=TRUE';
                        break;
                    default:
                    break;
                }
        }else{
            $whr[] = 'is_active=TRUE';
            if($offer_type)
                $whr[] = "b.offer_type = '$offer_type'";
            if($offer_group)
                $whr[] = "b.category = '$offer_group'";
        }
        if($whr)
            $whr = 'WHERE '.implode(' AND ', $whr);
        
        $page = $page*$rows;
        
        $sql = <<< ESQUEL
SELECT a.*, a.date_start + (a.life_days || ' DAYS')::INTERVAL AS date_end 
FROM main_offers3 a 
INNER JOIN dict_offer3_type b 
ON a.type=b.offer_type 
{$whr} 
ORDER BY 
date_start DESC 
LIMIT {$rows} OFFSET {$page} 
ESQUEL;
        $sql2 = <<< ESQUEL
SELECT COUNT(1) 
FROM main_offers3 a 
INNER JOIN dict_offer3_type b 
ON a.type=b.offer_type 
{$whr} 
ESQUEL;
        $data = $this->db->getAll($sql);
        $this->amount = $this->db->getOne($sql2);
        for($i=0;$i<count($data);$i++){
            if(is_numeric($data[$i]['premium_payment']))
                $data[$i]['is_premium'] = true;
            else
                $data[$i]['is_premium'] = false;
            if($this->manage || $this->cms){
                if(is_numeric($data[$i]['premium_payment']))
                    if($payment->isPaid($data[$i]['premium_payment']))
                        $data[$i]['payment_status'] = 'paid';
                    else
                        $data[$i]['payment_status'] = 'need';
                else
                    $data[$i]['payment_status'] = 'free';
            }
        }
        
        //Usuwanie przeterminowanych ogłoszeń
        if(rand(0,100)<5)
            $this->expireOld();
        
        return $data;
    }
    
    /**
     *  Zwraca listę ogłoszeń polecanych
     *  @param string $offer_group
     **/
    public function getPreferItems($offer_group = null)
    {
        $whr = '';
        $whr[] = 'a.is_active=TRUE';
        $whr[] = 'c.is_paid=TRUE';
        if($offer_group)
            $whr[] = "b.category = '$offer_group'";
        if($whr)
            $whr = 'WHERE '.implode(' AND ', $whr);
        $sql = <<< ESQUEL
SELECT a.* 
FROM main_offers3 a 
INNER JOIN dict_offer3_type b 
ON a.type=b.offer_type 
INNER JOIN main_payments c 
ON a.prefer_payment=c.payment_id 
{$whr} 
ORDER BY 
date_start DESC 
ESQUEL;
        $data = $this->db->getAll($sql);
        return $data;
    }
    
    /**
     *  Paginacja wszystkiego z tej klasy ;-)
     **/
    public function getPages($page = 0)
    {
        $rows = $this->rows;
        $range = $this->range;
        $range = $range==null?$this->config['pagging_range']:$range;
        
        $pages = null;
        $pages_amount = ceil($this->amount / $rows);
        
        if($pages_amount<=1)
            return null;//Jedna lub brak stron
        
        //pierwsza strona
        $pages['first'] = array('view' => 'first', 'goto' => 0);
        
        //popszednia strona
        if($page>0)
            $pages['prev'] = array('view' => 'prev', 'goto' => $page-1);
        
        //pierwszy separator
        if($page>$range+1+1)
            $pages['sep1'] = array('view' => 'sep', 'goto' => floor(($page-$range)/2));
        
        //mięso
        $start = $page-$range;
        if($start<0) $start = 0;
        $stop = $page+$range;
        if($stop>$pages_amount) $stop = $pages_amount;
        for($i=$start; $i<$stop; $i++){
            $key = 'step'.($page+$i);
            $pages[$key] = array('view' => $i+1, 'goto' => $i);
        }
        
        //drugi separator
        if($pages_amount-$page>$range+1+1)
            $pages['sep2'] = array('view' => 'sep', 'goto' => ceil(($pages_amount-$page+$range)/2));
        
        //następna strona
        if($page<$pages_amount-1)
            $pages['next'] = array('view' => 'next', 'goto' => $page+1);
        
        //ostatnia strona
        $pages['last'] = array('view' => 'last', 'goto' => $pages_amount-1);
        
        return $pages;
    }
    
    /**
     *  Pobiera pełne dane ogłoszenia
     **/
    public function getItem($offer_id)
    {
        $whr = null;
        $whr[] = "offer_id='{$offer_id}'";
        if(!$this->manage && !$this->cms)
            $whr[] = "is_active=TRUE";
        
        if($whr!=null)
            $whr = 'WHERE '.implode(' AND ', $whr);
        
        $sql = "SELECT * FROM main_offers3 {$whr} LIMIT 1";
        $data = $this->db->getRow($sql);
        $data['price'] = str_replace('Zł', '', $data['price']);
        $dimage = new dimage();
        $data['images'] = $dimage->getRelatedImages('offer', $data['offer_id']);
        return $data;
    }
    
    /**
     *  Pobiera typy ogłoszeń
     *  @param string $group - z jakiej grupy pobrać
     **/
    public function getTypes($group = null)
    {
        $whr = '';
        if($group)
            $whr = "WHERE category='{$group}'";
        $sql = "SELECT offer_type, name, LOWER(category) AS category FROM dict_offer3_type {$whr} ORDER BY sort_order ASC";
        $data = $this->db->getAll($sql);
        for($i=0;$i<count($data);$i++){
            $data[$i]['amount'] = $this->db->getOne("SELECT COUNT(1) FROM main_offers3 WHERE type='{$data[$i]['offer_type']}' AND is_active=TRUE AND date_start>NOW() - INTERVAL '7 DAYS'");
        }
        return $data;
    }
    
    /**
     *  Pobiera grupy kategorii ogłoszeń
     **/
    public function getGroups()
    {
        $sql = "SELECT category as value FROM dict_offer3_type GROUP BY category ORDER BY MIN(sort_order) ASC";
        $data = $this->db->getCol($sql);
        return $data;
    }
    
    /***************************************************************************
     *  Zarządzanie ogłoszeniami
     **************************************************************************/
    
    /**
     *  Dodawanie ogłoszenia
     *  @param string $title
     *  @param integer $offer_type
     *  @param string $description
     *  @param string $phone
     *  @param string $email
     *  @param integer $life_time
     *  @param integer $price
     *  @param array $images
     *  @param string $avatar
     *  @return boolean
     **/
    public function addItem($title, $offer_type, $description, $phone = null, $email = null, $life_days, $price, $images = null, $avatar, $premium = false, $prefer = false)
    {
        if(!$this->manage)
            return false;
        
        $title = "'".addslashes(strip_tags($title))."'";
        $offer_type = "'$offer_type'";
        if(!$premium)
            $description = strip_tags($description);
        else
            $description = strip_tags($description, $this->allow_tags);
        $description = "'".addslashes($description)."'";
        if(strlen($phone))
            $phone = "'$phone'";
        else
            $phone = 'NULL';
        if(strlen($email))
            $email = "'$email'";
        else
            $email = 'NULL';
        $life_days = "'$life_days'";
        if(strlen($price))
            $price = "'$price'";
        else
            $price = 'NULL';
        if(strlen($avatar)==12 && in_array($avatar, $images))
            $avatar = "'$avatar'";
        elseif(isset($images[0]) && strlen($images[0]))
            $avatar = "'{$images[0]}'";
        else
            $avatar = 'NULL';
        
        $sql = <<< ESQUEL
INSERT INTO main_offers3  
(title, type, description, phone, email, life_days, price, avatar, user_login) 
VALUES 
($title, $offer_type, $description, $phone, $email, $life_days, $price, $avatar, '{$this->user_login}') 
RETURNING offer_id
ESQUEL;
        $offer_id = $this->db->getOne($sql);
        if(!$offer_id)
            return false;
        
        if($premium || $prefer){
            $payment = new payment();
            $payment->open();
            $payment_id = $payment->returnId();
            if($premium){
                $payment->addProduct('OFFER_PREMIUM', 1);
                $sql = "UPDATE main_offers3 SET premium_payment='{$payment_id}' WHERE offer_id='{$offer_id}'";
                $this->db->exec($sql);
            }
            if($prefer){
                $payment->addProduct('OFFER_PREFER', 1);
                $sql = "UPDATE main_offers3 SET prefer_payment='{$payment_id}' WHERE offer_id='{$offer_id}'";
                $this->db->exec($sql);
            }
            $payment->setReady();
        }
        
        if(is_array($images)){
            $dimage = new dimage();
            foreach($images as $image)
                $dimage->relateImage($image, 'offer', $offer_id);
        }
        
        return true;
    }
    
    /**
     *
     **/
    public function changeItem($offer_id, $title, $offer_type, $description, $phone = null, $email = null, $life_time, $price, $images, $avatar)
    {
        if(!$this->manage)
            return false;
        
        $premium = false;
        $old = $this->getItem($offer_id);
        if(is_numeric($old['premium_payment']))
            $premium = true;
        
        $old_images = null;
        foreach($old['images'] as $img)
            $old_images[] = $img;
        
        if(!$premium)
            $description = strip_tags($description);
        else
            $description = strip_tags($description, $this->allow_tags);
        $title = "'".addslashes(strip_tags($title))."'";
        $offer_type = "'$offer_type'";
        if(in_array($avatar, $images))
            $avatar = "'$avatar'";
        else
            $avatar = 'NULL';
        $description = "'".addslashes($description)."'";
        if(strlen($phone))
            $phone = "'$phone'";
        else
            $phone = 'NULL';
        if(strlen($email))
            $email = "'$email'";
        else
            $email = 'NULL';
        $life_time = "'$life_time'";
        if(strlen($price))
            $price = "'$price'";
        else
            $price = 'NULL';
        
        $dimage = new dimage();
        
        $sql = <<< ESQUEL
UPDATE main_offers3 SET 
title={$title}, 
type={$offer_type}, 
description={$description}, 
phone={$phone}, 
email={$email}, 
life_days={$life_time}, 
price={$price},
avatar={$avatar}
WHERE 
offer_id = {$offer_id} 
ESQUEL;
        $res = $this->db->exec($sql);
        
        $dimage = new dimage();
        
        if(is_array($images))
            foreach($images as $img)
                if(!in_array($img, $old_images))
                    $dimage->relateImage($img, 'offer', $offer_id);
        
        if(is_array($old_images))
            foreach($old_images as $img)
                if(!in_array($img, $images))
                    $dimage->unrelateImage($img, 'offer', $offer_id);
        
        if($res) return true;
        return false;
    }
    
    /**
     *
     **/
    public function setAvatar($offer_id, $image_name = null)
    {
        if(!$this->manage)
            return false;
        
        $data = $this->getItem($offer_id);
        if($data['user_login']!=$this->user_login){
            return false;
        }
        
        if($image_name)
            $image_name = "'{$image_name}'";
        else
            $image_name = 'NULL';
        $sql = "UPDATE main_offers3 SET avatar=$image_name WHERE offer_id='$offer_id'";
        $res = $this->db->exec($sql);
        if($res) return true;
        return false;
    }
    
    /**
     *  Aktywacja ogłoszenia
     **/
    public function setActive($offer_id)
    {
        if(!$this->manage)
            return false;
        
        $data = $this->getItem($offer_id);
        if($data['user_login']!=$this->user_login){
            return false;
        }
        if(is_numeric($data['premium_payment'])){
            $payment = new payment();
            if(!$payment->isPaid($data['premium_payment']))
                return false;
        }
        
        $sql = "UPDATE main_offers3 SET is_active=TRUE WHERE offer_id='$offer_id'";
        $res = $this->db->exec($sql);
        $sql = "UPDATE main_offers3 SET date_start=NOW() WHERE offer_id='$offer_id' AND date_start IS NULL";
        $this->db->exec($sql);
        if($res) return true;
        return false;
    }
    
    /**
     *  De-aktywacja ogłoszenia
     **/
    public function setNotActive($offer_id)
    {
        if(!$this->manage)
            return false;
        
        $data = $this->getItem($offer_id);
        if($data['user_login']!=$this->user_login){
            return false;
        }
        
        $sql = "UPDATE main_offers3 SET is_active=FALSE WHERE offer_id='$offer_id'";
        $res = $this->db->exec($sql);
        if($res) return true;
        return false;
    }
    
    /**
     * 
     */
    public function delete($offer_id)
    {
        if(!$this->manage)
            return false;
            
        $data = $this->getItem($offer_id);
        if($data['user_login']!=$this->user_login){
            return false;
        }
        
        $sql = "DELETE FROM main_offers3 WHERE offer_id='$offer_id'";
        $res = $this->db->exec($sql);
        if($res) return true;
        return false;
    }
    
    /**
     *  Zamknięcie ogłoszenia
     **/
    public function setFinish($offer_id)
    {
        if(!$this->manage)
            return false;
        
        $data = $this->getItem($offer_id);
        if($data['user_login']!=$this->user_login){
            return false;
        }
        
        $sql = "UPDATE main_offers3 SET is_active=FALSE, is_finish=TRUE WHERE offer_id='$offer_id'";
        $res = $this->db->exec($sql);
        if($res) return true;
        return false;
    }
    
    /**
     *
     **/
    public function expireOld()
    {
        $sql = <<< ESQUEL
SELECT 
offer_id AS value 
FROM 
main_offers3 
WHERE 
date_start IS NOT NULL 
AND 
is_active = TRUE 
AND 
date_start + (life_days || ' DAYS')::INTERVAL < NOW()
ESQUEL;
        $data = $this->db->getCol($sql);
        if(is_array($data))
            foreach($data as $offer_id){
                $sql = "UPDATE main_offers3 SET is_active=FALSE, is_finish=TRUE WHERE offer_id='$offer_id'";
                $this->db->exec($sql);
            }
        return true;
    }
    
    /**
     *  Zwraca ilość znalezionych ogłoszeń
     **/
    public function returnAmount()
    {
        return $this->amount;
    }
}

?>
