<?php

/******************************************************************************
 *                                                                            *
 *      Klasa obsługująca dodatkowe funkcje systemu autoryzacji               *
 *                                                                            *
 ******************************************************************************/

require_once('Mail/htmlMimeMail5Ext.php');
require_once('PDOS.class.php');

/*if(!defined('PORTAL_DB')){
    define('PORTAL_DB', 'portal');//Kod połączenia z zwykłym użytkownikiem
}
if(!defined('EDITOR_DB')){
    define('EDITOR_DB', 'editor');//Kod połączenia z editorem
}*/

if(!defined('PDOS_DBNAME')) define('PDOS_DBNAME', 'portal');

class system
{
    private $system_email = 'esculap@esculap.pl';
    private $system_name = 'Portal esculap';
    private $db;
    private $user_login = null;
    
    /**
     *  Konstruktor ustawia połączenie na poziomie edytora
     **/
    public function __construct($user_login = null)
    {
        $this->db = PDOS::getInstance();
        
        if($user_login!=null)
            $this->user_login = $user_login;
        elseif(isset($_SESSION['auth']['username']) && $user_login!==false)//Gdy nie ma loginu, próbuje go zdobyć z sesji
            $this->user_login = $_SESSION['auth']['username'];
    }
    
    /***************************************************************************
     *          Funkcje obsługi notyfikacji systemowych (email)
     **************************************************************************/
    
    /**
     *  Funkcja wysyłająca maile systemowe
     *  @param str $subject - Temat wiadomości
     *  @param str $text - Wersja tekstowa wiadomości
     *  @param str $html - Wersja html wiadomości
     *  @return bool - Czy wysłano wiadomość
     **/
    public function system_email($user_login, $subject, $header, $text_body, $html_body, $email = null)
    {
        if($user_login==null)
            $user_login = $this->user_login;
        if($email == null)
            $email = $this->db->getOne("SELECT email_ext FROM main_users WHERE user_login='{$user_login}'");
        
        $html_content = <<< MAIL
<body align="center"><div style="width:600px; text-align:left;"><img src="header.png" style="margin-bottom: 30px;">
<h1 style="font: bold 16px Tahoma, Helvetica, sans-serif; color:#2C8F89; margin: 0;">{$header}</h1>
<div style="font: normal 12px Arial, Helvetica, sans-serif; color:#333333; margin-top: 20px;">{$html_body}</div>
<div style="background-color:#F9F8F4; font: normal 12px Arial, Helvetica, sans-serif; color:#333333; border-bottom:dotted 1px #CFCEC4; border-top:dotted 1px #CFCEC4; margin-top: 27px; padding: 17px;">
Będziemy wdzięczni za wszelkie uwagi, komentarze i sugestie<br>
na temat funkcjonowania portalu Esculap.pl<br>
oraz udostępnianych za jego pośrednictwem materiałów i usług. 
</div></div></body>
MAIL;
        $text_content = <<< MAIL
{$header}\n\n\n{$text_body}\n\n\n-----Będziemy wdzięczni za wszelkie uwagi, komentarze i sugestie \nna temat funkcjonowania portalu Esculap.pl \noraz udostępnianych za jego pośrednictwem materiałów i usług. \n
MAIL;
        
        $mail = new htmlMimeMail5Ext();
        $mail->setSMTPParams('home.esculap.pl');
        $mail->setHTMLCharset('UTF-8');
        $mail->setTextCharset('UTF-8');
        $mail->setHeadCharset('UTF-8');
        $mail->setHTMLEncoding(new eightBitEncoding);
        $mail->setFrom("{$this->system_name} <{$this->system_email}>");
        $mail->setSubject($subject);
        $mail->setPriority('high');
        $mail->setText($text_content);
        $mail->setHTML($html_content);
        $mail->addEmbeddedImage(new fileEmbeddedImage('/data/web/global_files/system_email/header.png', $mail->checkContentType('header.png')));
        $result = $mail->send(array($email), 'sendmail');
        return true;
    }
    
    /**
     *  Wysłanie emaila systemowego z bazy templatów
     *  @param str $template_name - Nazwa templatu
     *  @return bool - Czy wysłano wiadomość
     **/
    public function system_email_tpl($user_login, $template_name)
    {
        if($user_login==null)
            $user_login = $this->user_login;
        $sql = "SELECT * FROM main_system_templates WHERE name='{$template_name}'";
        $template = $this->db->getRow($sql);
        return $this->system_email($user_login, $template['subject'], $template['header'], $template['text'], $template['html']);
    }
    
    /***************************************************************************
     *          Funkcje obsługi notyfikacji systemowych (na stronie)
     **************************************************************************/
    
    /**
     *  Sprawdzenie czy konto nie posiada wymuszonych akcji
     *  @param str $user_login - Nazwa użytkownika
     *  @return bool - true = zablokowane, false = nie zablokowane
     **/
    public function check_lock($user_login = null)
    {
        if($user_login==null)
            $user_login = $this->user_login;
        
        $sql = <<< ESKUEL
SELECT COUNT(1) FROM main_system_tickets t INNER JOIN main_system_operations_actions oa 
ON oa.operation_name = t.operation_name AND oa.event_name='LOGIN' AND oa.step=t.step 
WHERE t.user_login='{$user_login}' AND oa.action_name='PAGE_RESTRICTION'
ESKUEL;
        $count = $this->db->getOne($sql);
        return $count>0?true:false;
    }
    
    /**
     *  Sprawdzenie czy użytkownik powinien być przekierowany do strony z notyfikacjami
     **/
    public function check_attention($user_login = null)
    {
        if($user_login==null)
            $user_login = $this->user_login;
        
        $sql = "SELECT COUNT(1) FROM main_users WHERE ".
               "user_login='{$user_login}' AND need_attention=true";
        $need_attention = $this->db->getOne($sql)>0?true:false;
        return $need_attention;
    }
    
    /**
     *  Zdejmuje need_attention
     **/
    public function clear_attention($user_login = null)
    {
        if($user_login==null)
            $user_login = $this->user_login;
        
        //$sql = "UPDATE main_users SET need_attention=false WHERE user_login LIKE '{$user_login}'";
        $this->db->exec($sql);
    }
    
    /**
     *  Funkcja pobiera z ticketów wszystkie eventy 'LOGIN' i szuka wyświetleń
     *  notyfikacji na stronie
     *  @param str $user_login - Nazwa użytkownika
     *  $return array - Tablica notyfikacji do wyświetlenia na stronie
     **/
    public function get_page_notification($user_login = null)
    {
        if($user_login==null)
            $user_login = $this->user_login;
        
        $tickets = $this->get_user_tickets($user_login);
        
        $data = null;
        
        if(is_array($tickets)){
            foreach($tickets as $ticket){
                $sql = <<< ESKUEL
SELECT oa.operation_name, oa.action_name, t.* FROM main_system_operations_actions oa 
LEFT OUTER JOIN main_system_templates t ON oa.template_name = t.name 
WHERE oa.operation_name='{$ticket['operation_name']}' 
AND oa.event_name='LOGIN' AND oa.step='{$ticket['step']}'
ESKUEL;
                $tmp = $this->db->getRow($sql);
                if(isset($tmp['action_name'])){
                    if(strlen($ticket['data'])){
                        $tmp['header'] = str_replace('[TICKET_DATA]', $ticket['data'], $tmp['header']);
                        $tmp['html'] = str_replace('[TICKET_DATA]', $ticket['data'], $tmp['html']);
                    }
                    if(strlen($ticket['data2'])){
                        $tmp['header'] = str_replace('[TICKET_DATA2]', $ticket['data'], $tmp['header']);
                        $tmp['html'] = str_replace('[TICKET_DATA2]', $ticket['data'], $tmp['html']);
                    }
                    $data[] = $tmp;
                }
            }
        }
        
        return $data;
    }
    
    /***************************************************************************
     *          Funkcje obsługi operacji systemowych
     **************************************************************************/
    
    /**
     *  UŻYCIE PROCESU NA ZDAŻENIE -------------------------------------------->
     *  NALEŻY UŻYWAĆ TYLKO TEJ FUNKCJI DO URUCHAMIANIA, ZMIANY STANU ORAZ ---->
     *  KOŃCZENIA DZIAŁANIA PROCEDUR OBSŁUGI UŻYTKOWNIKA ---------------------->
     *  @param str $operation_name - Nazwa operacji
     *  @param str $event_name - Nazwa zdażenia
     *  @param mixed $ext_data - dodatkowe informacje np. Klucz aktywacyjny,
     *                           nowe hasło, lub nowy adres email
     *  @return bool - Czy się udało
     **/
    public function use_process($operation_name, $event_name, $ext_data = null, $user_login = null)
    {
        //Zdażenia które nie mogą być wykonywane kilka razy z rzędu
        $no_repeat_event = array('BEGIN', 'END');
        if($user_login==null)
            $user_login = $this->user_login;
        
        //Pobieranie danych systemowych
        $op_ac = $this->get_operation_actions($operation_name);
        $ticket = $this->get_user_ticket($operation_name, $user_login, $ext_data);
        if($user_login==null)
            $user_login = $ticket['user_login'];
        $step = $ticket!=null?$ticket['step']:1;
        
        if($ticket==null && $event_name!='BEGIN'){
            return false;//Procedura musi zacząć się od BEGIN
        }
        if(!isset($op_ac[$step][$event_name])){
            return false;//Brak definicji w bazie do obsługi zdarzenia
        }
        if($op_ac[$step][$event_name]==$ticket['event_name']){
            return false;//Operacja dopiero co wykonana
        }
        $action = $op_ac[$step][$event_name];
        
        //Zmiany w tickecie
        if($ticket==null){
            $this->create_ticket($operation_name, $user_login, $event_name, $action['timeout_hours']);
            $this->set_ticket_data($operation_name, $user_login, $ext_data, 1);
            $ticket = $this->get_user_ticket($operation_name, $user_login);
        }else{
            $close = ($action['ticket_close'])?true:false;
            $this->update_ticket($operation_name, $user_login, $event_name, $action['timeout_hours'], $close);
            $ticket = $this->get_user_ticket($operation_name, $user_login);
        }
        
        //Uruchomienie funkcji systemowej
        if(!empty($action['process_name'])){
            $process = $action['process_name'];
            if($ext_data!=null){
                $result = $this->$process($ticket, $ext_data);
            }else{
                $result = $this->$process($ticket);
            }
            //var_dump($result);exit;
            if(!$result)
                return false;
        }
        
        //Uruchomienie akcji, action_name czyli wysłanie emaila.
        //Notyfikacje na stronie będą wyświetlane za pomocą funkcji get_page_notification
        switch($action['action_name']){
            case 'MAIL_NOTIFICATION':
                $this->system_email_tpl($user_login, $action['template_name']);
                break;
            default:
            //Nic nie rób
            break;
        }
        
        //Zamykanie ticketa
        if($action['ticket_close']){
            if($event_name=='TIMEOUT')
                $this->destroy_ticket($operation_name, $user_login, false);
            else
                $this->destroy_ticket($operation_name, $user_login);
        }
        
        //Przejście do kolejnego kroku ticketa
        if($action['ticket_next']){
            $this->ticket_step($operation_name, $user_login);
        }
        
        //Skok do konkretnego kroku ticketa
        if(is_numeric($action['jump_to']) && $action['jump_to']>0){
            $this->ticket_step($operation_name, $user_login, $action['jump_to']);
        }
        
        return true;
    }
    
    /**
     *  POBIERA LISTĘ AKCJI WYBRANEJ OPERACJI --------------------------------->
     *  @param str $operation_name - Nazwa operacji
     *  @return array - tablica akcji danej operacji, zagnieżdzona po krok (step), zdażenie (event_name)
     **/
    public function get_operation_actions($operation_name)
    {
        $sql = "SELECT * FROM main_system_operations_actions WHERE operation_name LIKE '{$operation_name}'";
        $operations_actions = $this->db->getAll($sql);
        $data = null;
        if(is_array($operations_actions) && count($operations_actions)>0){
            foreach($operations_actions as $action){
                $action2 = $action;
                unset($action2['step']);
                unset($action2['event_name']);
                $data[$action['step']][$action['event_name']] = $action2;
            }
        }
        return $data;
    }
    
    /**
     *  POBIERANIE JEDNEGO TICKET'A ------------------------------------------->
     *  @param str $operation_name - Nazwa operacji
     *  @param str $user_login - Nazwa użytkownika
     *  @param str | int $ext_data - Gdy nie ma loginu jedyna szansa by go pozyskać z ticketa
     *  @return array - Dane ticketa
     **/
    public function get_user_ticket($operation_name, $user_login, $ext_data = null)
    {
        //Gdy nie ma loginu, próbuje odnaleźć ticket po danych zewnętrznych. Np. kod aktywacyjny, nowy email. Używwać tylko dla danych unikalnych.
        if(strlen($user_login)<1)
            if(is_array($ext_data))
                if(isset($ext_data['code']))
                    $whr = "data LIKE '{$ext_data['code']}'";
                elseif($ext_data['email'])
                    $whr = "data LIKE '{$ext_data['email']}'";
                else
                    return null;
            else
                $whr = "data LIKE '{$ext_data}'";
        else
            $whr = "user_login LIKE '{$user_login}'";
        
        $sql = "SELECT * FROM main_system_tickets WHERE {$whr} AND operation_name LIKE '{$operation_name}'";
        $ticket = $this->db->getAll($sql);
        $data = null;
        if(is_array($ticket) && count($ticket)==1){
            $this->user_login = $ticket[0]['user_login'];
            $data = $ticket[0];
        }
        return $data;
    }
    
    /**
     *  POBIERANIE WSZYSTKICH TICKET'ÓW USERA --------------------------------->
     *  @param str $user_login - Nazwa użytkownika
     *  @return array - tablica ticketów użytkownika
     **/
    public function get_user_tickets($user_login)
    {
        $sql = "SELECT * FROM main_system_tickets WHERE user_login LIKE '{$user_login}'";
        $tickets = $this->db->getAll($sql);
        $data = null;
        if(is_array($tickets) && count($tickets)>0){
            $data = $tickets;
        }
        return $data;
    }
    
    /**
     *  TWORZENIE NOWEGO TICKET'A --------------------------------------------->
     *  @param str $operation_name - Nazwa operacji
     *  @param str $user_login - Nazwa użytkownika
     *  @param str $event_name - Nazwa zdażenia
     *  @return bool - Czy się udało
     **/
    private function create_ticket($operation_name, $user_login, $event_name, $timeout = 0)
    {
        $timeout = intval($timeout)>0?'NOW() + INTERVAL \''.intval($timeout).' HOURS\'':'NULL';
        $sql = "INSERT INTO main_system_tickets (user_login, operation_name, event_name, timeout) ".
               "VALUES ('{$user_login}', '{$operation_name}', '{$event_name}', $timeout)";
        $this->db->exec($sql);
        $this->user_attention($user_login);
        return true;
    }
    
    /**
     *  AKTUALIZACJA ISTNIEJĄCEGO TICKET'A ------------------------------------>
     *  @param str $operation_name - Nazwa operacji
     *  @param str $user_login - Nazwa użytkownika
     *  @param str $event_name - Nazwa zdażenia
     *  @return bool - Czy się udało
     **/
    private function update_ticket($operation_name, $user_login, $event_name, $timeout = 0, $closed = false)
    {
        //history na początku
        $this->history_ticket($operation_name, $user_login);
        
        $timeout = intval($timeout)>0?'timeout=NOW() + INTERVAL \''.intval($timeout).' HOURS\',':'';
        $closed = $closed?'true':'false';
        
        $sql = "UPDATE main_system_tickets SET event_name='{$event_name}', {$timeout} ".
               "closed={$closed} WHERE user_login='{$user_login}' AND operation_name='{$operation_name}'";
        $this->db->exec($sql);
        $this->user_attention($user_login);
        return true;
    }
    
    /**
     *  PRZESKOK TICKET'A NA KOLEJNY KROK ------------------------------------->
     *  @param str $operation_name - Nazwa operacji
     *  @param str $user_login - Nazwa użytkownika
     *  @param int $jump_to - Skok do podanego kroku, jeśli NULL skacze do następnego kroku
     *  @return bool - Czy się udało
     **/
    public function ticket_step($operation_name, $user_login, $jump_to = null)
    {
        $ticket = $this->get_user_ticket($operation_name, $user_login=null);
        $this_step = $ticket['step'];
        $next_step = $jump_to!=null?$jump_to:$this_step + 1;
        $op_ac = $this->get_operation_actions($operation_name);
        if(!isset($op_ac[$next_step]))
            return false;
        
        if(isset($op_ac[$this_step]['END']))
            $this->use_process($operation_name, 'END');
        
        $sql = "UPDATE main_system_tickets SET step='{$next_step}' WHERE user_login='{$user_login}' ".
               "AND operation_name='{$operation_name}'";
        $this->db->exec($sql);
        
        if(isset($op_ac[$next_step]['BEGIN']))
            $this->use_process($operation_name, 'BEGIN');
        
        $this->user_attention($user_login);
        return true;
    }
    
    /**
     *  USTAWIENIE DANYCH DODATKOWYCH TICKET'A -------------------------------->
     *  @param str $operation_name - Nazwa operacji
     *  @param str $user_login - Nazwa użytkownika
     *  @param str | int $data - Dane do zapisu
     *  @return bool - Czy się udało
     **/
    private function set_ticket_data($operation_name, $user_login, $data, $slot = 1)
    {
        $field = 'data';
        if($slot == 1)
            $field = 'data';
        if($slot == 2)
            $field = 'data2';
        $sql = "UPDATE main_system_tickets SET {$field}='{$data}' WHERE user_login='{$user_login}' ".
               "AND operation_name='{$operation_name}'";
        $this->db->exec($sql);
        return true;
    }
    
    /**
     *  NISZCZENIE NIEPOTRZEBNEGO JUŻ TICKET'A -------------------------------->
     *  @param str $operation_name - Nazwa operacji
     *  @return bool - Czy się udało
     **/
    private function destroy_ticket($operation_name, $user_login, $end_event = true)
    {
        //Wywołanie dict_system_event -> END
        if($end_event)
            $this->use_process($operation_name, 'END');
        
        //history po zakończeniu event->END przed DELETE
        $this->history_ticket($operation_name, $user_login);
        
        $sql = "DELETE FROM main_system_tickets WHERE user_login='{$user_login}' AND operation_name='{$operation_name}'";
        $this->db->exec($sql);
        
        $this->user_attention($user_login);
        
        return true;
    }
    
    /**
     *  ARCHIWIZACJA STANU TICKET'A ------------------------------------------->
     *  @param str $operation_name - Nazwa operacji
     *  @return bool - Czy się udało
     **/
    private function history_ticket($operation_name, $user_login)
    {
        $ticket = $this->get_user_ticket($operation_name, $user_login);
        if($ticket==null)
            return false;
        
        $closed = ($ticket['closed']===true)?'true':'false';
        $timeout = (strlen($ticket['timeout'])<1)?'null':"'{$ticket['timeout']}'";
        
        $sql = "INSERT INTO main_system_tickets_history (user_login, operation_name, ".
               "event_name, start_date, finish_date, step, timeout, closed, data, data2) ".
               "VALUES ('{$ticket['user_login']}', '{$ticket['operation_name']}', ".
               "'{$ticket['event_name']}', '{$ticket['start_date']}', NOW(), ".
               "'{$ticket['step']}', {$timeout}, {$closed}, ".
               "'{$ticket['data']}', '{$ticket['data2']}')";
        $this->db->exec($sql);
        
        $sql = "UPDATE main_system_tickets SET start_date=NOW() WHERE user_login='{$user_login}' ".
               "AND operation_name='{$operation_name}'";
        $this->db->exec($sql);
        
        return true;
    }
    
    /**
     *  Wywołanie eventu TIMEOUT dla wszystkich przedawnionych ticketów
     *  Odpalac z CRONa
     **/
    public function handle_timeout_tickets()
    {
        $sql = "SELECT * FROM main_system_tickets WHERE timeout IS NOT NULL AND timeout<NOW() ORDER BY timeout LIMIT 5000";
        $data = $this->db->getAll($sql);
        
        $user_login = $this->user_login;
        foreach($data as $ticket){
            echo "{$ticket['user_login']} {$ticket['operation_name']}\n";
            $this->use_process($ticket['operation_name'], 'TIMEOUT', null, $ticket['user_login']);
        }
        $this->user_login = $user_login;
    }
    
    /**
     *  Zmienia status main_users.need_attention
     *  @param str $user_login - Nazwa użytkownika
     *  @return bool - Czy się udało
     **/
    public function user_attention($user_login)
    {
        $tickets = $this->get_user_tickets($user_login);
        $need_attention = 'false';
        foreach($tickets as $tt){
            $sql = "SELECT COUNT(1) FROM main_system_operations_actions WHERE operation_name='{$tt['operation_name']}' ".
                   "AND event_name='LOGIN' AND step='{$tt['step']}'";
            if($this->db->getOne($sql)>0){
                $need_attention = 'true';
                break;
            }
        }
        //$need_attention = count($tickets)>0?'true':'false';
        $sql = "UPDATE main_users SET need_attention={$need_attention} WHERE user_login='{$user_login}'";
        $this->db->exec($sql);
        return true;
    }
    
    /***************************************************************************
     *          Procesy systemowe, wywoływane przez procedury
     **************************************************************************/
    
    /**
     *  Wysyła email z nowym kodem aktywacyjnym
     *  @return bool - Czy się udało
     **/
    public function send_act_mail($ticket)
    {
        $user_login = $ticket['user_login'];
        $operation_name = 'ACOUNT_ACTIVATION';
        $act_code = rand(100000000, 999999999);
        
        //Zapis kodu aktywacyjnego w tickecie
        $this->set_ticket_data($operation_name, $user_login, $act_code);
        $act_link = DOMAIN.'user/activate/code,'.$act_code.'/';
        $sql = "SELECT * FROM main_users WHERE user_login='{$user_login}'";
        $user_data = $this->db->getRow($sql);
        $subject = 'Potwierdzenie rejestracji w portalu esculap.pl';
        $header = "Gratulujemy udanego pierwszego etapu rejestracji i witamy w portalu medycznym Esculap.pl.";
        $html = <<< MAIL
Każdy Użytkownik naszego serwisu jest naszym miłym Gościem,<br>
dlatego pragniemy uczynić wszystko, aby jak najszybciej poczuł się Domownikiem.<br><br><br>
W celu zakończenia procesu rejestracji i pełnej aktywacji dostępu <br>
do wszystkich materiałów i usług w portalu Esculap.pl<br>
uprzejmie prosimy o kliknięcie w ciągu najbliższych 48 godzin na poniższy link:<br>
<a href="{$act_link}">{$act_link}</a><br><br><br>
Przypominamy, że prawo do korzystania z portalu Esculap.pl,<br>
zgodnie z nowym prawem farmaceutycznym, mają wyłącznie osoby <br>
zawodowo związane z medycyną i farmacją.<br>
Jeśli nie spełniasz tego warunku, prosimy o kontakt z Działem Użytkownika<br>
tel. (22) 389-10-52 ; mailto: <a href="mailto:info@esculap.pl">info@esculap.pl</a><br><br><br>
<strong>Twoje dane:</strong><br><br>
Login: {$user_data['user_login']}<br>
Imię: {$user_data['first_name']}<br>
Nazwisko: {$user_data['last_name']}<br>
E-mail: {$user_data['email_ext']}<br>
<br><br><br>
UWAGA: w każdej chwili istnieje możliwość samodzielnej zmiany powyższych danych<br>
(z wyjątkiem loginu, który jest na stałe przypisany do danego użytkownika)<br>
- służy do tego formularz dostępny na stronie:<br>
<a href="https://www.esculap.pl/user2/">https://www.esculap.pl/user2/</a><br><br><br>
Pełna rejestracja  w portalu Esculap.pl oznacza wyrażenie zgody na otrzymywanie <br>
dwa razy w tygodniu biuletynu informacyjno-edukacyjnego "Goniec Medyczny",<br>
zawierającego skrót najważniejszych informacji z życia polskich placówek medycznych,<br>
porady prawne, dane o konferencjach i szkoleniach, recenzje nowości wydawniczych<br>
oraz najciekawsze komentarze użytkowników portalu na temat bieżących wydarzeń.<br><br><br>
W przypadku wystąpienia problemów z konfiguracją/obsługą konta poczty elektronicznej <br>
uprzejmie zapraszamy do zapoznania się ze wskazówkami podanymi na stronie:<br>
<a href="http://www.esculap.pl/help/poczta.html">http://www.esculap.pl/help/poczta.html</a><br><br><br>
W przypadku innych problemów związanych z dostępem i korzystaniem<br>
z materiałów i usług portalu Esculap.pl prosimy o przeczytanie informacji na stronie:<br>
<a href="http://www.esculap.pl/help/">http://www.esculap.pl/help/</a><br><br><br>
Zachęcamy także do bezpośredniego kontaktu z naszym Działem Użytkownika,<br>
który jest czynny od poniedziałku do piątku w godz. 8.00-16.00<br>
pod nr tel. (22) 389 10 52 oraz adresem e-mail: <a href="mailto:info@esculap.pl">info@esculap.pl</a><br><br><br>
Korespondencję papierową prosimy kierować na adres:<br>
Redakcja portalu medycznego Esculap.pl<br>
Cegedim Group Poland Sp. z o.o.<br>
02-305 Warszawa, Al. Jerozolimskie 136<br><br><br>
Będziemy wdzięczni za wszelkie uwagi, komentarze i sugestie <br>
na temat funkcjonowania portalu Esculap.pl oraz udostępnianych za jego pośrednictwem<br>
materiałów i usług.<br><br><br>
Życzymy samych błyskawicznych połączeń nie tylko z naszym serwisem w Internecie.<br><br><br>
Z wyrazami szacunku<br><br><br>
Redakcja portalu Esculap.pl
MAIL;
        $text = strip_tags(str_replace('<br>', "\n", str_replace("\n", '', $html)));
        $result = $this->system_email($user_login, $subject, $header, $text, $html);
        return $result;
    }
    
    /**
     *  Obsługa kodu aktywacyjnego
     *  @param str $code - Kod aktywacyjny
     *  @return bool - Czy się udało
     **/
    public function activate_acount($ticket, $code)
    {
        $sql = "SELECT COUNT(*) FROM main_system_tickets WHERE data='{$code}'";
        if($this->db->getOne($sql)==0)
            return false;
        
        $sql = "SELECT user_login FROM main_system_tickets WHERE data='{$code}'";
        $user_login = $this->db->getOne($sql);
        $sql = "UPDATE main_users SET status='PARTIAL', real_status='ACTIVE', ".
               "activated=true WHERE user_login='{$user_login}'";
        $this->db->exec($sql);
        
        return true;
    }
    
    /**
     *  Usunięcie konta użytkownika
     *  @return bool - Czy się udało
     **/
    public function delete_acount($ticket)
    {
        /*$user_login = $ticket['user_login'];
        $sql = "UPDATE main_users SET status='NO_ACCESS', real_status='DELETED' ".
               "WHERE user_login='{$user_login}'";
        $this->db->exec($sql);*/
        $res = $this->use_process('ACOUNT_DELETE', 'BEGIN', 'ACT_EXP', $ticket['user_login']);
        //$this->DeleteAccount($ticket, 'ACTIVATION_TIMEOUT');
        return true;
    }
    
    /**
     *  Wysyła email z kodem zmiany hasła
     **/
    public function send_pass_ch_mail($ticket)
    {
        $user_login = $ticket['user_login'];
        $operation_name = 'PASS_CHANGE';
        $act_code = rand(100000000, 999999999);
        
        //Zapis kodu aktywacyjnego w tickecie
        $this->set_ticket_data($operation_name, $user_login, $act_code);
        
        $link = DOMAIN.'user/password/code,'.$act_code.'/';
        
        $subject = 'Zmiana hasła w portalu esculap.pl';
        $header = 'Aby zmienić hasło, kliknij w poniższy link:';
        $text = "{$link}\n\n";
        
        $html = "<a href=\"{$link}\" style=\"color:#2C8F89; ".
                "text-decoration:underline;\">{$link}</a>";
        
        $result = $this->system_email($user_login, $subject, $header, $text, $html);
        return $result;
    }
    
    /**
     *  Sprawdzenie czy kod jest poprawny
     *  @param str $code - kod potwierdzający dostęp
     **/
    public function check_code($ticket, $code = null)
    {
        //$sql = "SELECT COUNT(*) FROM main_system_tickets WHERE data='{$code}'";
        //if($this->db->getOne($sql)==0)
        if($code!=$ticket['data'])
            return false;
        return true;
    }
    
    /**
     *  Zmiana hasła
     *  @param array $arr - tablica zawierająca kod potwierdzający oraz nowe hasło
     **/
    public function change_pass($ticket, $arr)
    {
        $code = $arr['code'];
        $new_password = $arr['password'];
        $sql = "SELECT COUNT(*) FROM main_system_tickets WHERE data='{$code}'";
        if($this->db->getOne($sql)==0)
            return false;
        
        $sql = "SELECT user_login FROM main_system_tickets WHERE data='{$code}'";
        $user_login = $this->db->getOne($sql);
        $sql = "UPDATE main_users SET password='{$new_password}' WHERE user_login='{$user_login}'";
        $this->db->exec($sql);
        $sql = "UPDATE postfix.mailbox SET password='{$new_password}' WHERE userlogin='{$user_login}'";
        $this->db->exec($sql);
        
        //Zmiana adresu w części socialnej
        $sdb = DB_connect('social');
        $sql = "SELECT user_code FROM se_users WHERE user_username='{$user_login}'";
        $user_code = $sdb->getOne($sql);
        $new_password = substr($user_code, 0, strlen($user_code)/2).$new_password.substr($user_code, (strlen($user_code)/2)*(-1));
        $sql = "UPDATE se_users SET user_password=MD5('{$new_password}') WHERE user_username='{$user_login}'";
        $sdb->exec($sql);
        
        return true;
    }
    
    /**
     *  Funkcja wysyła email potwierdzający na nowy adres email
     *  @param str $new_mail - nowy email
     **/
    public function send_email_ch_mail($ticket, $new_mail)
    {
        $user_login = $ticket['user_login'];
        $operation_name = $ticket['operation_name'];
        $code = rand(100000000, 999999999);
        
        //Zapis kodu aktywacyjnego w tickecie
        $this->set_ticket_data($operation_name, $user_login, $code);
        //Zapis emaila do ukrytego pola (slot 2)
        $this->set_ticket_data($operation_name, $user_login, $new_mail, 2);
        
        $link = DOMAIN.'user/email/code,'.$code.'/';
        
        $subject = 'Potwierdzenie zmiany adresu email w portalu esculap.pl';
        $header = 'Wyraziłeś cheć zmiany adresu email w portalu esculap.pl, aby potwierdzić zmianę adresu email kliknij w poniższy link:';
        $text = "{$link}";
        $html = "<a href=\"{$link}\" style=\"color:#2C8F89; text-decoration:underline;\">{$link}</a>";
        
        $result = $this->system_email($user_login, $subject, $header, $text, $html, $new_mail);
        return $result;
    }
    
    /**
     *  Funkcja zmienia adres email użytkownika
     **/
    public function change_email($ticket, $code)
    {
        if($code!=$ticket['data'])
            return false;
        $user_login = $ticket['user_login'];
        
        $sql = "SELECT email_ext FROM main_users WHERE user_login='{$user_login}'";
        $old_email = $this->db->getOne($sql);
        
        $sql = "UPDATE main_users SET email_ext='{$ticket['data2']}' WHERE user_login='{$user_login}'";
        $this->db->exec($sql);
        
        //Zmiana adresu w części socialnej
        $sdb = DB_connect('social');
        $sql = "UPDATE se_users SET user_email='{$ticket['data2']}', user_newemail='{$ticket['data2']}' WHERE user_email='{$old_email}'";
        $sdb->exec($sql);
        
        $sql = "SELECT real_status FROM main_users WHERE user_login='{$user_login}'";
        if('NEW' == $this->db->getOne($sql))
            $this->use_process('ACOUNT_ACTIVATION', 'BEGIN');
        
        return true;
    }
    
    /**
     *  Zablokowanie konta użytkownika
     **/
    public function LockAccount($ticket, $comment = null)
    {
        $user_login = $ticket['user_login'];
        $sql = "SELECT count(1) FROM main_users WHERE user_login='{$user_login}' AND real_status='ACTIVE'";
        if($this->db->getOne($sql)!=1)
            return false;
        $this->set_ticket_data($ticket['operation_name'], $user_login, $comment);
        $sql = "UPDATE main_users SET real_status='LOCK' WHERE user_login='{$user_login}'";
        $this->db->exec($sql);
        $sql = "UPDATE postfix.mailbox SET active=FALSE WHERE userlogin='{$user_login}'";
        $this->db->exec($sql);
        
        $subject = 'Twoje konto w portalu esculap.pl zostało zablokowane.';
        $header = 'Twoje konto zostało zablokowane.';
        $text = "Konto zablokowane.";
        $html = "Konto zablokowane.";
        $result = $this->system_email($user_login, $subject, $header, $text, $html);
        
        return true;
    }
    
    /**
     *  Odblokowanie konta użytkownika
     **/
    public function UnlockAccount($ticket, $comment = null)
    {
        $user_login = $ticket['user_login'];
        $sql = "SELECT count(1) FROM main_users WHERE user_login='{$user_login}' AND real_status='LOCK'";
        if($this->db->getOne($sql)!=1)
            return false;
        $this->set_ticket_data($ticket['operation_name'], $user_login, $comment);
        $sql = "UPDATE main_users SET real_status='ACTIVE' WHERE user_login='{$user_login}'";
        $this->db->exec($sql);
        
        $subject = 'Przywrócono ci dostęp do portalu esculap.pl.';
        $header = 'Twoje konto jest znów aktywne.';
        $text = "Konto odblokowane.";
        $html = "Konto odblokowane.";
        $result = $this->system_email($user_login, $subject, $header, $text, $html);
        
        return true;
    }
    
    /**
     *  Usuwanie konta użytkownika
     **/
    public function DeleteAccount($ticket, $comment = null)
    {
        $user_login = $ticket['user_login'];
        $sql = "SELECT count(1) FROM main_users WHERE user_login='{$user_login}' AND (real_status='NEW' OR real_status='ACTIVE' OR real_status='LOCK')";
        if($this->db->getOne($sql)!=1)
            return false;
        $this->set_ticket_data($ticket['operation_name'], $user_login, $comment);
        $sql = "UPDATE main_users SET real_status='DELETE' WHERE user_login='{$user_login}'";
        $this->db->exec($sql);
        if($comment=='SYNC_DEL'){
            $subject = 'Informacja o wygaśnięiu konta dostępowego w portalu Esculap.pl';
            $header = 'Informacja o wygaśnięiu konta dostępowego w portalu Esculap.pl';
            $text = <<< TXT
Szanowny Użytkowniku,

szanujemy Twoją prywatność i nie chcemy się narzucać,
dlatego przyjęliśmy, że błędna, niekompletna,
lub uniemożliwiająca kontakt (celem potwierdzenia
założenia konta dostępowego) rejestracja w Esculapie,
oznacza rezygnację z korzystania z naszego portalu medycznego.

W związku z powyższym, uprzejmie informujemy,
że Twoje konto w portalu Esculap.pl zostało usunięte.

Jeśli chciałbyś jednak korzystać z Esculapa,
serdecznie zapraszamy do ponownej, poprawnej rejestracji! :)

Jednocześnie pragniemy przypomnieć,
iż zgodnie z obowiązującymi przepisami polskiego prawa
(a dokładnie nowego prawa farmaceutycznego)
prawo do korzystania z informacji i materiałów 
prezentowanych za pośrednictwem portalu Esculap.pl
mają wyłącznie osoby zawodowo związane z ochroną zdrowia
(tj. lekarze, farmaceuci, dentyści, pielęgniarki, ratownicy,
pracownicy administracji placówek medycznych, 
studenci uczelni medycznych, itd.).

Uwaga: Państwa dane nie są i nie będą udostępniane
żadnym innym podmiotom zewnętrznym, a ich przechowywanie
jest monitorowane przez Generalnego Inspektora Ochrony Danych Osobowych.

W razie dodatkowych pytań bądź wątpliwości 
prosimy o kontakt z naszym Działem Użytkownika
tel. 22 389 10 52 (od pon. do piąt. w godz. 8.00-16.00),
e-mail: info@esculap.pl
TXT;
            $html = '<pre>'.$text.'</pre>';
        }elseif($comment=='ACT_EXP'){
            $subject = 'Brak aktywacji konta w przeciągu 48 godzin';
            $header = 'Brak aktywacji konta w przeciągu 48 godzin';
            $text = <<< TXT
Szanowny Użytkowniku,

uprzejmie przypominamy, iż do szczęśliwego zakończenia 
procesu rejestracji w portalu medycznym Esculap.pl 
(wymuszonego przepisami polskiego prawa farmaceutycznego)
niezbędne jest kliknięcie w ciągu 48 godzin 
na przesłanym pocztą elektroniczną linku aktywacyjnym.

Ponieważ w wymaganym wyżej czasie nie otrzymaliśmy
takiego potwierdzenia, konto dostępowe do portalu 
nie może być dłużej aktywne (zostało automatycznie zablokowane przez serwer).

Aby uzyskać możliwość bezpłatnego korzystania 
ze wszystkich materiałów i usług w portalu 
niezbędne jest ponowne wypełnienie 
formularza rejestracyjnego i kliknięcie w ciągu 48 godzin
na przesłanym linku aktywacyjnym.

Pragniemy podkreślić, iż Państwa dane nie są i nie będą udostępniane
żadnym innym podmiotom zewnętrznym, a ich przechowywanie
jest monitorowane przez Generalnego Inspektora Ochrony Danych Osobowych.

Serdecznie zapraszamy do ponownej rejestracji. Będzie nam niezwykle miło, 
jeśli dołączysz do licznego grona naszych Użytkowników.
TXT;
            $html = '<pre>'.$text.'</pre>';
        }elseif($comment=='ACC_DOUBLED'){
            $subject = 'Informacja o wygaśnięiu konta dostępowego w portalu Esculap.pl';
            $header = 'Informacja o wygaśnięiu konta dostępowego w portalu Esculap.pl';
            $text = <<< TXT
Szanowny Użytkowniku,
serdecznie dziękujemy za zainteresowanie portalem Esculap.pl

Przypominamy, że aby chronić
bezawaryjną i bezpieczną pracę naszego serwera,
każda osoba ma prawo do założenia
wyłącznie jednego konta dostępowego w portalu.

W związku z powyższym, uprzejmie informujemy,
że jedno z założonych przez Ciebie, zdublowanych kont dostępowych
(które jest nieaktywne, tj. nie były notowane z niego wejścia do portalu) 
zostało automatycznie usunięte z naszego serwera.

Zapraszamy serdecznie do korzystania z aktywnego konta dostępowego.
W razie dodatkowych pytań bądź wątpliwości 
prosimy o kontakt z naszym Działem Użytkownika
tel. 22 389 10 52 (od pon. do piąt. w godz. 8.00-16.00),
e-mail: info@esculap.pl
TXT;
            $html = '<pre>'.$text.'</pre>';
        }else{
            $subject = 'Twoje konto w portalu esculap.pl zostało usunięte.';
            $header = 'Twoje konto zostało usunięte.';
            $text = "Konto usunięte.";
            $html = "Konto usunięte.";
        }
        $result = $this->system_email($user_login, $subject, $header, $text, $html);
        
        return true;
    }
    
    /**
     *  Usuwanie konta użytkownika
     **/
    public function CompleteDeleteAccount($ticket, $comment = null)
    {
        $user_login = $ticket['user_login'];
        $sql = "SELECT count(1) FROM main_users WHERE user_login='{$user_login}' AND real_status='DELETE'";
        if($this->db->getOne($sql)!=1)
            return false;
        $this->set_ticket_data($ticket['operation_name'], $user_login, $comment);
        $sql = "DELETE FROM main_users WHERE user_login='{$user_login}'";
        $this->db->exec($sql);
        $sql = "DELETE FROM mailbox.postfix WHERE userlogin='{$user_login}'";
        $this->db->exec($sql);
        $sql = "DELETE FROM postfix.alias WHERE address LIKE '{$user_login}@'";
        $this->db->exec($sql);
        return true;
    }
    
    /**
     *  Dodawanie konta pocztowego esculap
     **/
    public function AddMailbox($ticket, $comment = null)
    {
        $user_login = $ticket['user_login'];
        $this->set_ticket_data($ticket['operation_name'], $user_login, $comment);
        $sql = "SELECT * FROM main_users WHERE user_login='{$user_login}'";
        $data = $this->db->getRow($sql);
        $email_esc = "{$data['user_login']}@esculap.pl";
        $sql = "INSERT INTO postfix.mailbox (username, userlogin, password, name, maildir, quota, domain) ".
               "VALUES ('{$email_esc}', '{$data['user_login']}', '{$data['password']}', ".
               "'{$data['first_name']} {$data['last_name']}', 'esculap.pl/{$data['user_login'][0]}/{$data['user_login']}/Maildir/', ".
               "'512000000', 'esculap.pl')";
        $this->db->exec($sql);
        $sql = "UPDATE main_users SET email_esc='{$email_esc}' WHERE user_login = '{$data['user_login']}'";
        $this->db->exec($sql);
        
        $subject = "Założono konto {$email_esc} w domenie Esculap.pl - gratulujemy.";
        $header = "Założono konto {$email_esc} w domenie Esculap.pl - gratulujemy.";
        $text = <<<MAIL
Dane konta:
Adres: {$email_esc}
Serwer POP3: poczta.esculap.pl 
Serwer SMTP: poczta.esculap.pl (Wymagane jest włączenie autoryzacji SMTP) 
Identyfikator: {$user_login}
MAIL;
        $html = <<<MAIL
Dane konta:<br>
Adres: {$email_esc}<br>
Serwer POP3: poczta.esculap.pl<br>
Serwer SMTP: poczta.esculap.pl (Wymagane jest włączenie autoryzacji SMTP)<br>
Identyfikator: {$user_login}
MAIL;
        $result = $this->system_email($user_login, $subject, $header, $text, $html);
        $result = $this->system_email($user_login, $subject, $header, $text, $html, $email_esc);
        
        return true;
    }
    
    /**
     *  Blokowanie konta pocztowego esculap
     **/
    public function LockMailbox($ticket, $comment = null)
    {
        $user_login = $ticket['user_login'];
        $this->set_ticket_data($ticket['operation_name'], $user_login, $comment);
        $sql = "UPDATE postfix.mailbox SET active=FALSE WHERE userlogin='{$user_login}'";
        $this->db->exec($sql);
        
        $subject = 'Twoje konto pocztowe w portalu esculap.pl zostało zablokowane.';
        $header = 'Twoje konto pocztowe zostało zablokowane.';
        $text = "Konto pocztowe zablokowane.";
        $html = "Konto pocztowe zablokowane.";
        $result = $this->system_email($user_login, $subject, $header, $text, $html);
        
        return true;
    }
    
    /**
     *  Odblokowywanie konta pocztowego esculap
     **/
    public function UnlockMailbox($ticket, $comment = null)
    {
        $user_login = $ticket['user_login'];
        $this->set_ticket_data($ticket['operation_name'], $user_login, $comment);
        $sql = "UPDATE postfix.mailbox SET active=TRUE WHERE userlogin='{$user_login}'";
        $this->db->exec($sql);
        
        $subject = 'Przywrócono ci dostęp do konta pocztowego w portalu esculap.pl.';
        $header = 'Twoje konto pocztowe zostało znowu aktywne.';
        $text = "Konto odblokowane.";
        $html = "Konto odblokowane.";
        $result = $this->system_email($user_login, $subject, $header, $text, $html);
        
        return true;
    }
    
    /**
     *  Usuwanie konta mailowego esculap
     **/
    public function DeleteMailbox($ticket, $comment = null)
    {
        $user_login = $ticket['user_login'];
        $this->set_ticket_data($ticket['operation_name'], $user_login, $comment);
        $sql = "UPDATE postfix.mailbox SET active=FALSE WHERE userlogin='{$user_login}'";
        $this->db->exec($sql);
        
        $subject = 'Twoje konto pocztowe w portalu esculap.pl zostało usunięte.';
        $header = 'Twoje konto pocztowe zostało usunięte.';
        $text = "Konto usunięte.";
        $html = "Konto usunięte.";
        $result = $this->system_email($user_login, $subject, $header, $text, $html);
        
        return true;
    }
}

?>