<?php

/******************************************************************************
 *                                                                            *
 *      Klasa ułatwiająca zapis i odczyt z plików xml                         *
 *      XMLWriter                                                             *
 *      XMLReader                                                             *
 *      DOMDocument                                                           *
 *                                                                            *
 ******************************************************************************/

class xml
{
    /**
     *  Wersja standardu dokumentu xml
     **/
    private $version = '1.0';
    
    /**
     *  Kodowanie dokumentu xml
     **/
    private $charset = 'UTF-8';
    
    /**
     *  Domyślny tag startowy, gdy tablica nie ma jednego punktu zaczepienia
     **/
    private $start_tag = 'data';
    
    /**
     *  Dane z/do dokumentu xml (array)
     **/
    private $data = null;
    
    /**
     *  Utworzony dokument xml (string)
     **/
    private $document = null;
    
    /**
     *  Konstruktor, nadpisuje wartości domyślne
     *  @param array $config - Tablica konfiguracyjna
     **/
    public function __construct($config = null)
    {
        if(isset($config['version']))
            $this->version = $config['version'];
        if(isset($config['charset']))
            $this->charset = $config['charset'];
        if(isset($config['start_tag']))
            $this->start_tag = $config['start_tag'];
        
        
    }
    
    /**
     *  Składa dokument xml za pomocą XMLWriter'a
     *  @param array $data - Tablica do przetworzenia
     **/
    public function render($data = null)
    {
        $this->document = null;
        
        if($data != null)
            $this->data = $data;
        
        $xml = new XmlWriter();
        $xml->openMemory();
        $xml->startDocument($this->version, $this->charset);
        
        if(count($this->data)!=1 || isset($this->data[0]))
            $xml->startElement($this->start_tag);
        
        //Konwersja tablicy z danymi na xml'a
        $this->basic_render($xml, $this->data);
        
        if(count($this->data)!=1 || isset($this->data[0]))
            $xml->endElement();
        
        $this->document = $xml->outputMemory();
    }
    
    /**
     *  Konwertuje rekurencyjnie tablicę do obiektu XMLWriter, nie uwzględnia atrybutów
     *  @param object &$xml - Referencja do obiektu XMLWriter
     *  @param array $data_array - Tablica do przerobienia na xml
     *  @param string $last_field - Ostatnia nazwa tagu dla xml, tylko dla rekurencji
     **/
    private function basic_render(&$xml, $data_array, $last_field = null)
    {
        foreach($data_array as $field => $data){
            if(is_numeric($field) && ($last_field == null || is_numeric($last_field))){
                //trigger_error("XML: Nieprawidłowy format tablicy wejściowej", E_USER_ERROR);
            }else{
                if(is_array($data)){
                    $numeric = false;
                    foreach($data as $key => $val)
                    {
                        if(is_numeric($key))
                            $numeric = true;
                    }
                    if(!$numeric)
                        $xml->startElement(is_numeric($field)?$last_field:$field);
                    $this->basic_render($xml, $data, $field);
                    if(!$numeric)
                        $xml->endElement();
                }elseif(empty($data)){
                    $xml->writeElement(is_numeric($field)?$last_field:$field);
                }elseif(is_numeric($data)){
                    $xml->writeElement(is_numeric($field)?$last_field:$field, $data);
                }else{
                    $xml->startElement(is_numeric($field)?$last_field:$field);
                    $xml->writeCData("{$data}");
                    $xml->endElement();
                }
            }
        }
    }
    
    /**
     *  Sprawdza zgodność struktury pliku xml z schematem
     *  @param string $xsd_doc - Lokalizacja pliku z schematem xsd
     *  @param string $xml_doc - Lokalizacja pliku xml, gdy jest pusta sprawdzany jest $this->document
     *  @param array &$errors - Przekazuje błędy
     *  @return boolean - Czy struktura xml jest zgodny z schematem xsd
     **/
    public function validate($xsd_doc, $xml_doc = null, &$errors = false)
    {
        $DOM = new DOMDocument();
        if($xml_doc != null){
            $DOM->load($xml_doc);
        //    if($errors!==false)
        //        $this->load($xml_doc);
        }else
            $DOM->loadXML($this->document);
        
        $xml = explode("\n", $this->document);
        //libxml_use_internal_errors(true);
        $result = $DOM->schemaValidate($xsd_doc);
        unset($DOM);
        //if($errors!==false)
        //    foreach(libxml_get_errors() as $error)
        //        $errors .= $this->display_xml_error($error, $xml);
        //libxml_clear_errors();
        return $result;
    }
    
    /**
     *  Parsuje dokument xml do tablicy
     *  @param string $xml_doc - Lokalizacja pliku xml
     *  @return object SimpleXML
     **/
    public function parse($xml_doc = null)
    {
        if($xml_doc!=null)
            $this->document = file_get_contents($xml_doc);
        
        $xml = new SimpleXMLElement($this->document, LIBXML_NOBLANKS);
        
        return $xml;
    }
    
    /**
     *  Ustawia wewnętrzną zmienną z zawartością xml
     **/
    public function set_document($document)
    {
        $this->document = $document;
    }
    
    /**
     *  Zwraca przechowywaną zawartość xml
     **/
    public function get_document()
    {
        return $this->document;
    }
    
    /**
     *
     **/
    public function get_data()
    {
        return $this->data;
    }
    
    /**
     *
     **/
    public function set_data($data)
    {
        $this->data = $data;
    }
    
    /**
     *
     **/
    public function load($filepath)
    {
        $this->document = file_get_contents($filepath);
    }
    
    /**
     *  Zapisuje zawartość zgodną z XML do pliku
     *  @param string $filename - Nazwa dla pliku
     *  @param string $path - Ścieżka dla pliku
     *  @return boolean - Czy się udało
     **/
    public function save_as($filename, $path)
    {
        $result = file_put_contents($path.$filename, $this->document);
        if($result === false)
            return false;
        else
            return true;
    }
    
    /**
     *
     **/
    function display_xml_error($error, $xml)
    {
        $return  = trim($xml[$error->line - 1]) . "\n";
        //$return .= str_repeat('-', $error->column) . "^\n";
        
        switch ($error->level) {
            case LIBXML_ERR_WARNING:
                $return .= "Warning $error->code: ";
                break;
             case LIBXML_ERR_ERROR:
                $return .= "Error $error->code: ";
                break;
            case LIBXML_ERR_FATAL:
                $return .= "Fatal Error $error->code: ";
                break;
        }
        
        $return .= trim($error->message) .
                   "\n  Line: $error->line" .
                   "\n  Column: $error->column";
        
        if ($error->file) {
            $return .= "\n  File: $error->file";
        }
        
        return "$return\n\n--------------------------------------------\n\n";
    }
}


?>